﻿/*-----------------------------------------------------------------------------
 |  Copyright (c) 2013 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


/******************************************************************************
HOW TO USE:
        This script was made for Photoshop CS6.  It may or may not work 
        on CS5 or lower.  In Photoshop, go to File->Scripts->Browse to 
        find the script to load.  It will then prompt for the directory of 
        textures that you want to resize.  In this directory and/or its
        subdirectories can exist a file called _SkipResize.txt, which is a list
        of files and/or folders to be ignored anywhere in the tree structure.
        Only PNG files are resized.  A new folder is created alongside the 
        original root folder to store the newly resized images.
 *****************************************************************************/


#target photoshop
preferences.rulerUnits = Units.PIXELS;

  
var RootInputFolder = Folder.selectDialog("Select folder to process"); 


if ( RootInputFolder != null )
{
    var RootInputFolderName = RootInputFolder.fullName;
    //$.writeln( "RootInputFolderName: ", RootInputFolderName );

    var InputFileList = [];
    var SkipFoldersList = [];
    var SkipFilesList = [];
    
    
    // Start reading items in the folder.   // See REM at function.
    GetSubFolders( RootInputFolder );


    for ( var i = 0; i < InputFileList.length; i ++ )
    {
        var FileInputFolderName = InputFileList[ i ].path;
        var PathRelativeToRoot = "_Half";
        
        // Subtract the root of the string so we can create the matching folding in the Output 'root' folder.
        for ( var j = RootInputFolderName.length; j < FileInputFolderName.length; j ++ )
        {
             PathRelativeToRoot += FileInputFolderName.charAt( j );
        }
     
        var OutputFolder = new Folder( RootInputFolderName + PathRelativeToRoot );
        OutputFolder.create();
        //$.writeln( "File input folder temp: ", FileInputFolderName );
        //$.writeln( "File input folder final: ", RootInputFolderName + PathRelativeToRoot );
        //$.writeln( "Output folder URI: ", OutputFolder.relativeURI );

         // Generate the new image name
        var SplitName = InputFileList[ i ].name.split( '.' );               
        var sNewFile = OutputFolder.relativeURI + "/" + SplitName[ 0 ] + "_Half.png";
        //$.writeln( "Output File: ", sNewFile );
        
        // Check if the new name already exists.  If not, then do the resize.  
        //  If it already exists, then skip it.
        var FileAlreadyExists = File ( sNewFile );
        var bExists = FileAlreadyExists.open( 'r' );
        
        if ( !bExists )
        {
            var SourceFile = open( InputFileList[ i ] );
            //$.writeln( "Source File ", InputFileList[ i ].fsName );
            //$.writeln( "Source File ", SourceFile.name );

            var iOldWidth =SourceFile.width;
            var iOldHeight = SourceFile.height;

            var iNewWidth = iOldWidth / 2;
            var iNewHeight = iOldHeight / 2;
                     

            // Do the resize
            SourceFile.resizeImage( iNewWidth, iNewHeight, null, ResampleMethod.BICUBICSHARPER );


            // Save it
            var pngOptions = new ExportOptionsSaveForWeb;
            pngOptions.format = SaveDocumentType.PNG;
            pngOptions.PNG8 = false;
            pngOptions.transparency = true;
            pngOptions.interlaced = false;
            pngOptions.quality = 100;
            activeDocument.exportDocument( new File( sNewFile ), ExportType.SAVEFORWEB, pngOptions );
            
            // Close the source image
            activeDocument.close( SaveOptions.DONOTSAVECHANGES );
        }
    }
}



/******************************************************************************
 FUNCTION:  GetSubFolders
 Notes: This function will first load a SkipResize CSV if one exists, and add
              the items to the lists.  It will then an add any PNG files to the list
              and recursively call itself on any subfolders in this folder, provided
              that the folder is not on a skip list.  Any individual files on the skip
              list are also ignored.  
******************************************************************************/
function GetSubFolders( ParentFolder )
{
    var ItemList = ParentFolder.getFiles();
    LoadSkipResizeCSV( ParentFolder.fsName + "\\_SkipResize.txt" );
    
    for ( var i = 0; i < ItemList.length; i ++ )
    {
        var CurrentItem = ItemList[ i ];
        if ( CurrentItem instanceof Folder )
        {
            if ( !IsSkippedFolder ( CurrentItem.name ) )
                 GetSubFolders( CurrentItem );
        }
        else if ( CurrentItem instanceof File && CurrentItem.name.match( /\.png$/i) )
        {
            if ( !IsSkippedFile ( CurrentItem.name ) )
                InputFileList.push( CurrentItem );
        }
    }
}



/******************************************************************************
 FUNCTION:  LoadSkipResizeCSV
******************************************************************************/ 
function LoadSkipResizeCSV( sFileName )
{
    var SkipResizeCSV = File ( sFileName );
    SkipResizeCSV.open( 'r' );
                  
    while ( !SkipResizeCSV.eof )
    {
        var sCurrentLine = SkipResizeCSV.readln();
        
        if ( sCurrentLine.match( /\.png$/i ) )
            SkipFilesList.push( sCurrentLine );
        else
            SkipFoldersList.push( sCurrentLine );
    }

    SkipResizeCSV.close();    
}



/******************************************************************************
 FUNCTION:  IsSkippedFile
******************************************************************************/ 
function IsSkippedFile( sFileName )
{
    for ( var j = 0; j < SkipFilesList.length; j ++ )
    {
        if ( sFileName == SkipFilesList[ j ] )
            return true;
    }

        return false;
}



/******************************************************************************
 FUNCTION:  IsSkippedFolder
******************************************************************************/ 
function IsSkippedFolder( sFolderName )
{
    for ( var j = 0; j < SkipFoldersList.length; j ++ )
    {
        if ( sFolderName == SkipFoldersList[ j ]  )
            return true;
    }

        return false;
}




























