/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpAsciiScreen.h"
#include "JpUtils.h"

#include "mmgrChooser.h"



USING_NS_CC;



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
JpAsciiScreen::JpAsciiScreen() :
_pPrintLineDelegate( NULL ),
_pacScreenBuffer( NULL ),
_iWidth( 0 ),
_iHeight( 0 )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void JpAsciiScreen::Init(  int iWidth, int iHeight )
{
	_iWidth = iWidth;
	_iHeight = iHeight;

	ASSERTF( !_pacScreenBuffer, ( "\nInitializing a second time without a deinit between!" ) );
	if ( !_pacScreenBuffer )
		_pacScreenBuffer = new char[ _iHeight * _iWidth ];

	Clear();
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void JpAsciiScreen::Deinit()
{
	if ( _pacScreenBuffer )
	{
		delete[] _pacScreenBuffer;
		_pacScreenBuffer = NULL;
	}

	_iWidth = 0;
	_iHeight = 0;
}



/******************************************************************************
 FUNCTION:  Clear
******************************************************************************/
void JpAsciiScreen::Clear()
{
	int iNumPixels = _iWidth * _iHeight;
	memset( _pacScreenBuffer, '.', sizeof(char) * iNumPixels ); 
}



/******************************************************************************
 FUNCTION:  Plot
******************************************************************************/
void JpAsciiScreen::Plot( int iX, int iY, char cToPlot )
{
	iX = Clamp( iX, 0, _iWidth - 1 );
	iY = Clamp( iY, 0, _iHeight - 1 );

	_pacScreenBuffer[ ( iY * _iWidth ) + iX ] = cToPlot;
}



/******************************************************************************
 FUNCTION:  PlotLine
******************************************************************************/
void JpAsciiScreen::PlotLine( int iX0, int iY0, int iX1, int iY1, char cToPlot )
{
	JpRasterLineIterator LineIter( iX0, iY0, iX1, iY1 );
	while ( LineIter.Next() )
	{
		Plot( LineIter.GetX(), LineIter.GetY(), cToPlot );
	}
}



/******************************************************************************
 FUNCTION:  Render
******************************************************************************/
void JpAsciiScreen::Render()
{
	char* pacLine = new char[ _iWidth + 1 ];

	for ( int i = 0; i < _iHeight; i ++ )
	{
		memcpy( pacLine, &_pacScreenBuffer[ _iWidth * i ], _iWidth );

		pacLine[ _iWidth ] = 0;

		PrintLine( pacLine );
	}

	delete[] pacLine;
}



/******************************************************************************
 FUNCTION:  Resize
******************************************************************************/
void JpAsciiScreen::Resize( int iWidth, int iHeight )
{
	Deinit();
	Init( iWidth, iHeight );
}



/******************************************************************************
 FUNCTION:  PrintLine
******************************************************************************/
void JpAsciiScreen::PrintLine( char* pacLine )
{
	if ( _pPrintLineDelegate )
		_pPrintLineDelegate->PrintLine( pacLine );
	else
		CCLog( pacLine );	
}
