/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"


USING_NS_CC;

/*
	This code simulates a screen buffer, but one which only prints ascii characters
	at each "pixel".  This screen is useful in situations where you need to debug
	something visually, but only have text/tty (ie, printf style) debugging available
	to you.  

	My use for this originally was in rendering audio data as a waveform in an
	android build setup that only had printf debugging.

	Here is an example of how to use this code:


		class MyClass
		{
			public:
				void Init();
				void Deinit();
				void Update();
				JpAsciiScreen AsciiScreen;
		};

		void MyClass::Init() { AsciiScreen.Init( 160, 20 ); }

		void MyClass::Deinit() { AsciiScreen.Deinit(); }

		void MyClass::Update()
		{
			AsciiScreen.Clear();

			int iHalfHeight = AsciiScreen.GetHeight() / 2;
			int iHalfWidth = AsciiScreen.GetWidth() / 2;

			// Draw vertical and horizontal axes
			AsciiScreen.PlotLine( 0, iHalfHeight, AsciiScreen.GetWidth(), iHalfHeight, '-' );
			AsciiScreen.PlotLine( iHalfWidth, 0, iHalfWidth, AsciiScreen.GetHeight(), '|' );

			// plot a single dot, using the '+' character
			AsciiScreen.Plot( 10, 15, '+' );

			// "render" the output to the tty
			AsciiScreen.Render();
		}

	mdomowicz 2013_01_18


	
	I've added a PrintLineDelegate class so that you can replce the text printing function
	with your own.

		class MyClass
		{
			public:
				class AsciiScreenPrintDelegate : public JpAsciiScreen::PrintLineDelegate
				{
					public:
						virtual void PrintLine( char* pacText ) { _pOwner->MyCustomPrint( pacText ); }
						SoConsole* _pOwner;
				};

				void Init();
				void Deinit();
				void Update();
				void MyCustomPrint( const std::string& s ) { !!!Your custom print code goes here!!! };

				JpAsciiScreen AsciiScreen;
				AsciiScreenPrintDelegate _Delegate;
		};

		void MyClass::Init() 
		{ 
			_Delegate.SetOwner( this );
			AsciiScreen.Init( 160, 20 ); 
			AsciiScreen.SetPrintLineDelegate( &_Delegate );
		}

	mdomowicz 2012_01_27
*/



class JpAsciiScreen
{
	public:

		//-----------------------
		// public classes
		class PrintLineDelegate
		{
			public:
				virtual void PrintLine( char* pacText ) = 0;
		};


		//-----------------------
		// public functions
		JpAsciiScreen();
		virtual ~JpAsciiScreen() {}

		void Init( int iWidth, int iHeight );
		void Deinit();

		int GetWidth() { return _iWidth; }
		int GetHeight() { return _iHeight; }

		void SetPrintLineDelegate( PrintLineDelegate* p ) { _pPrintLineDelegate = p; }

		void Plot( int iX, int iY, char cToPlot );

		void PlotLine( int iX0, int iY0, int iX1, int iY1, char cToPlot );

		void Clear();

		void Render();

		void Resize( int iWidth, int iHeight );

	private:

		//-----------------------
		// private variables
		int _iWidth;
		int _iHeight;

		char* _pacScreenBuffer;

		PrintLineDelegate* _pPrintLineDelegate;


		//-----------------------
		// private functions
		void PrintLine( char* pacLine );
};
