/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpChecksum.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  JpChecksumFletcher32
******************************************************************************/
JpChecksumFletcher32::JpChecksumFletcher32()
{
	Init();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void JpChecksumFletcher32::Init()
{
	_uiSum1 = 0;
	_uiSum2 = 0;
	_uiChecksum = 0;
}



/******************************************************************************
 FUNCTION:  AddUint32
 NOTE:      The wikipedia article on Fletcher's Checksum has a much faster
            implementation of this checksum, but it processes a whole stream
            of data at a time.  I needed a version that takes a single uin32
            at a time, and so I had to implement the algorithm by hand.  In
            doing this, I opted for the clearer, but slower, implementation.
            If you need a fast checksum, you will want to reimplement this.
            mdomowicz 2013_09_06
 ******************************************************************************/
void JpChecksumFletcher32::AddUint32( ChecksumUint32 ui )
{
	ChecksumUint32 uiHighBits = ( ui & 0xffff0000 ) >> 16;
	ChecksumUint32 uiLowBits =  ( ui & 0x0000ffff );
	

	// Since most values will be fully represented in the low bits,
	// this check is an optimization.
	// mdomowicz 2013_09_06
	if ( uiHighBits != 0 )
	{
		_uiSum1 = ( _uiSum1 + uiHighBits ) % 65535;
		_uiSum2 = ( _uiSum2 + _uiSum1 ) % 65535;
	}

	_uiSum1 = ( _uiSum1 + uiLowBits ) % 65535;
	_uiSum2 = ( _uiSum2 + _uiSum1 ) % 65535;


	_uiChecksum = ( ( _uiSum2 << 16 ) | _uiSum1 );
}



