/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#ifndef __JpDebug_h


	#define __JpDebug_h

	#include "stdafx.h"

	#include <stdio.h>
	#include "assert.h"


	#ifdef _DEBUG

		#define MAX_DEBUG_TEXT_LENGTH 1024

		void AssertPrintf( const char* pFormat, ... );
		int debug_printf( const char* format, ... );


		#define ASSERT assert

		#define ASSERTF( assertion, message )		\
		{											\
			if( !( assertion ) )					\
			{										\
				AssertPrintf( "%s(%d): - ASSERTION FAILED", __FILE__, __LINE__ ); \
				AssertPrintf message; \
				debug_printf(( "\n" ));				\
				assert( assertion );				\
				_asm { int 3 }						\
			}										\
		}

	#else// _DEBUG

		inline int debug_printf( const char* format, ... ) { return 0; }
		#define ASSERT assert
		#define ASSERTF( assertion, message )  {}

	#endif



/*
	I got the following code from:
	http://www.morearty.com/code/breakpoint/

	This code is a way to set hardware breakpoints from within code.
	This is useful because Visual Studio 6 (and maybe later versions) has really bad
	support for hardware breakpoints.

	The following usage notes are from the website:


	-------------------------------------
	The class is called CBreakpoint. If, for example, you have a DWORD called "x" 
	and you want to break the next time it's written to, do this:

		DWORD x = 1;

		CBreakpoint bp;
		bp.Set(&x, sizeof(x), CBreakpoint::Write);

	As your code continues to execute, if Visual C++ stops at a location where you 
	didn't have a VC++ breakpoint set, it's possible that the CBreakpoint triggered.  
	If it did, the assembly instruction immediately before the instruction pointer 
	is the one that caused it to trigger.

	The breakpoint will be cleared automatically when the CBreakpoint object falls 
	out of scope.  Or, to clear it explicitly, do this:

		bp.Clear();

	You can also break the moment any code even tries to read the value of the 
	variable x! (Actually, this will break when someone tries to read or write it.)

		bp.Set(&x, sizeof(x), CBreakpoint::Read);



	This code is Intel-specific.  It will run on Win95/98/ME and on WinNT/2000/XP, 
	as long as the machine is running an Intel or compatible chip (e.g. not Alpha).
	-------------------------------------
*/

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32

	class CBreakpoint
	{
		public:
			CBreakpoint() { m_index = -1; }
			~CBreakpoint() { Clear(); }

			// The enum values correspond to the values used by the Intel Pentium,
			// so don't change them!
			enum Condition { Write = 1, Read /* or write! */ = 3 };

			void Set(void* address, int len /* 1, 2, or 4 */, Condition when);
			void Clear();

		protected:

			inline void SetBits(unsigned long& dw, int lowBit, int bits, int newValue)
			{
				int mask = (1 << bits) - 1; // e.g. 1 becomes 0001, 2 becomes 0011, 3 becomes 0111

				dw = (dw & ~(mask << lowBit)) | (newValue << lowBit);
			}

			int m_index; // -1 means not set; 0-3 means we've set that hardware bp
	};
#endif

#endif  // __JpDebug_h
