/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


//-----------------------
/*
 This is a perlin noise implementation I got from Ken Miller.
 Used with permission via Email.
 mdomowicz 2008_11_23
*/


#pragma once

#include "stdafx.h"


extern float JpNoise1D(float x);
extern float JpNoise2D(float x, float y);
extern float JpNoise3D(float x, float y, float z);
extern float JpNoise4D(float x, float y, float z, float w);





//-----------------------
/*
	Everything below this comment was taken from Ken Miller's
	Videoventure (videoventure.googlecode.com) so that this
	perlin noise implementation would compile properly as a
	stand-alone module.

	Update:

	The code below is for doing extra fast rounding.  It comes from this article:
	Know your FPU: Fixing Floating Fast
	Sree Kotay, 2006

	http://stereopsis.com/sree/fpu2006.html
*/

// linear interpolation
template<typename T> inline const T Lerp(T v0, T v1, float s)
{
	return (1 - s) * v0 + s * v1;
}

#if _xs_BigEndian_
	#define _xs_iexp_				0
	#define _xs_iman_				1
#else
	#define _xs_iexp_				1       //intel is little endian
	#define _xs_iman_				0
#endif //BigEndian_

#if DCG_IS_TARGET_WIN32
	#define finline                 __forceinline
#else
	#define finline                 inline
#endif
	
const double _xs_doublemagic			= double (6755399441055744.0); 	    //2^52 * 1.5,  uses limited precisicion to floor
const double _xs_doublemagicdelta      	= (1.5e-8);                         //almost .5f = .5f + 1e^(number of exp bit)
const double _xs_doublemagicroundeps	= (.5f-_xs_doublemagicdelta);       //almost .5f = .5f - 1e^(number of exp bit)

int xs_CRoundToInt      (double val, double dmr =  _xs_doublemagic);
int xs_CRoundToInt      (int val);
int xs_FloorToInt       (double val, double dme =  _xs_doublemagicroundeps);


