/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#ifndef __JpString_h
#define __JpString_h

#include "stdafx.h"

#include <stdio.h>
#include <stdlib.h>


#include "JpHashTable.h"
#include "JpList.h"


#define JpStringList JpList<JpString>


class JpString
{
	public:

		//-----------------------
		// public functions
		JpString() : _pCString( NULL ) {}

		JpString( JpString* p ) : _pCString( NULL )
		{ 
			SetCString( p->GetCString() ); 
		}

		JpString( char* pacString ) : _pCString( NULL )
		{ 
			SetCString( pacString ); 
		}

		// copy constructor
		JpString( const JpString& Src ) : _pCString( NULL )
		{ 
			SetCString( Src.GetCString() ); 
		}

		virtual ~JpString() { ClearString(); }


		JpString& operator=( const JpString &Src ) 
		{
			if ( this != &Src ) 
				SetCString( Src.GetCString() ); 		

			return *this;
		}

		void SetByPrintf( char* pacFormat, ... );

		void ClearString();

		char* GetCString() const { return _pCString; }
		void SetCString( const char* pacString );

		bool MatchesString( JpString* psString );

		bool MatchesCString( char* pacString );

		bool EndsWith( char* pacEnd );

		static int ParseInt( const char* pacString );
		static hash ParseHash( const char* pacString );
		static float ParseFloat( const char* pacString );
		static bool ParseBool( const char* pacString );

		static std::string BoolToString( bool b );
		static std::string UnsignedCharToString( unsigned char uc );
		static std::string CharToString( char c );
		static std::string UnsignedIntToString( unsigned int ui );
		static std::string IntToString( int i );
		static std::string FloatToString( float f );
		static std::string HashToString( unsigned int ui );

	private:

		//-----------------------
		// private variables

		char* _pCString;
		int _iLength;
};


#endif //__JpString_h



