/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpUtils.h"
#include "mmgrChooser.h"


float JpUtils::_fPi = 3.141592653589f;
float JpUtils::_fHalfPi = JpUtils::_fPi * 0.5f;
float JpUtils::_fTwoPi = JpUtils::_fPi * 2.0f;


ccColor3B JpUtils::_Black = { 0, 0, 0 };
ccColor3B JpUtils::_White = { 255, 255, 255 };
ccColor3B JpUtils::_Red = { 255, 0, 0 };
ccColor3B JpUtils::_Green = { 0, 255, 0 };
ccColor3B JpUtils::_Blue = { 0, 0, 255 };
ccColor3B JpUtils::_Yellow = { 255, 255, 0 };
ccColor3B JpUtils::_Orange = { 255, 128, 0 };
ccColor3B JpUtils::_Magenta = { 255, 0, 255 };
ccColor3B JpUtils::_Cyan = { 0, 255, 255 };
ccColor3B JpUtils::_Purple = { 128, 0, 128 };
ccColor3B JpUtils::_Grey16 = { 16, 16, 16 };
ccColor3B JpUtils::_Grey32 = { 32, 32, 32 };
ccColor3B JpUtils::_Grey64 = { 64, 64, 64 };
ccColor3B JpUtils::_Grey96 = { 96, 96, 96 };
ccColor3B JpUtils::_Grey128 = { 128, 128, 128 };
ccColor3B JpUtils::_Grey160 = { 160, 160, 160 };
ccColor3B JpUtils::_Grey192 = { 192, 192, 192 };
ccColor3B JpUtils::_Grey224 = { 224, 224, 224 };
ccColor3B JpUtils::_Grey240 = { 240, 240, 240 };



/******************************************************************************
 FUNCTION:  JpRasterLineIterator::Next
 NOTES:     This is a modified version of the bresenham line drawing 
            algorithm implementation found here:

roguebasin.roguelikedevelopment.org/index.php?title=Bresenham's_Line_Algorithm
******************************************************************************/
bool JpRasterLineIterator::Next()
{
	if ( !_bRunOnce )
	{
		return First();
	}

	int x1 = _iCurX;
	int y1 = _iCurY;
	int x2 = _iX1;
	int y2 = _iY1;

	signed char ix;
	signed char iy;

	// if x1 == x2 or y1 == y2, then it does not matter what we set here
	int delta_x = (x2 > x1?(ix = 1, x2 - x1):(ix = -1, x1 - x2)) << 1;
	int delta_y = (y2 > y1?(iy = 1, y2 - y1):(iy = -1, y1 - y2)) << 1;

	//plot(x1, y1);
 
	if (delta_x >= delta_y)
	{
		// error may go below zero
		if ( !_bFirstErrorRecoreded )
		{
			_bFirstErrorRecoreded = true;
			_iAccumulatedError = delta_y - (delta_x >> 1);
		}
 
		while (x1 != x2)
		{
			if (_iAccumulatedError >= 0)
			{
				if (_iAccumulatedError || (ix > 0))
				{
					y1 += iy;
					_iAccumulatedError -= delta_x;
				}
				// else do nothing
			}
			// else do nothing

			x1 += ix;
			_iAccumulatedError += delta_y;

			//plot(x1, y1);
			_iCurX = x1;
			_iCurY = y1;
			_bHasMore = true;
			return true;
		}
	}
	else
	{
		// error may go below zero
		if ( !_bFirstErrorRecoreded )
		{
			_bFirstErrorRecoreded = true;
			_iAccumulatedError = delta_x - (delta_y >> 1);
		}
 
		while (y1 != y2)
		{
			if (_iAccumulatedError >= 0)
			{
				if (_iAccumulatedError || (iy > 0))
				{
					x1 += ix;
					_iAccumulatedError -= delta_y;
				}
				// else do nothing
			}
			// else do nothing

			y1 += iy;
			_iAccumulatedError += delta_x;

			//plot(x1, y1);
			_iCurX = x1;
			_iCurY = y1;
			_bHasMore = true;
			return true;
		}
	}

	_bHasMore = false;
	return false;
}
