/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#pragma once

#include "stdafx.h"

#include <math.h>
#include "JpDebug.h"


#define JpVector2iList JpList< JpVector2i >
#define JpVector2fList JpList< JpVector2f >
#define JpVector4fList JpList< JpVector4f >



/******************************************************************************
*******************************************************************************
*******************************************************************************
*******************************************************************************
*******************************************************************************

                                JpVector2i

*******************************************************************************
*******************************************************************************
*******************************************************************************
*******************************************************************************
******************************************************************************/


class JpVector2i
{
	public:

		//-----------------------
		// variable declarations
		int X;
		int Y;


		//-----------------------
		// function declarations
		JpVector2i()
		{ 
			// Components are not initialized!
			// This is for speed reasons.
		};

		JpVector2i( const JpVector2i& vSource )
		{
			X = vSource.X;
			Y = vSource.Y;
		}

		JpVector2i( int iX, int iY )
		{ 
			X = iX;
			Y = iY;
		};

		void Init();
		void Init( int iX, int iY );
		float Length() const;
		float LengthSquared() const;

		void Rotate( float fRadians );
		void Zero() { X = Y = 0; }
		bool IsZero() { return ( X == 0 && Y == 0 ); }

	private:
};

// GLOBAL VECTOR FUNCTIONS
int DotProduct( JpVector2i vVec1, JpVector2i vVec2 );
float Length( const JpVector2i& vVec1 );
float LengthSquared( const JpVector2i& vVec1 );
void Swap( JpVector2i* vVec1, JpVector2i* vVec2 );

// GLOBAL VECTOR OVERLOADED OPERATORS
JpVector2i operator * ( JpVector2i vLeftHandArg, int iRightHandArg );
JpVector2i operator * ( int iLeftHandArg, JpVector2i vRightHandArg );
JpVector2i& operator *= ( JpVector2i& vLeftHandArg, int iRightHandArg );
JpVector2i operator + ( JpVector2i vLeftHandArg, JpVector2i vRightHandArg );
JpVector2i& operator += ( JpVector2i& vLeftHandArg, JpVector2i vRightHandArg );
JpVector2i operator - ( JpVector2i vLeftHandArg, JpVector2i vRightHandArg );
JpVector2i& operator -= ( JpVector2i& vLeftHandArg, JpVector2i vRightHandArg );
bool operator == ( JpVector2i vLeftHandArg, JpVector2i vRightHandArg );
bool operator != ( JpVector2i vLeftHandArg, JpVector2i vRightHandArg );
const JpVector2i operator-( JpVector2i vArg );




/******************************************************************************
 FUNCTION:  *
 PURPOSE:   used to multiply a vector by a scaler value
 NOTES:     There are two versions of the overloaded * so that the operation
            can be both float * JpVector2 and JpVector2 * float
******************************************************************************/
inline JpVector2i operator * ( JpVector2i vLeftHandArg, int iRightHandArg )
{
	vLeftHandArg.X *= iRightHandArg;
	vLeftHandArg.Y *= iRightHandArg;

	return vLeftHandArg;
}

inline JpVector2i operator * ( int iLeftHandArg, JpVector2i vRightHandArg )
{
	vRightHandArg.X *= iLeftHandArg;
	vRightHandArg.Y *= iLeftHandArg;

	return vRightHandArg;
}




/******************************************************************************
 FUNCTION:  *=
 PURPOSE:   used to multiply a vector by a scaler value
******************************************************************************/
inline JpVector2i& operator *= ( JpVector2i& vLeftHandArg, int iRightHandArg )
{
	vLeftHandArg.X *= iRightHandArg;
	vLeftHandArg.Y *= iRightHandArg;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  +
 PURPOSE:   used to add to vectors together
******************************************************************************/
inline JpVector2i operator + ( JpVector2i vLeftHandArg, JpVector2i vRightHandArg )
{
	vLeftHandArg.X += vRightHandArg.X;
	vLeftHandArg.Y += vRightHandArg.Y;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  +=
 PURPOSE:   used to add to vectors together
******************************************************************************/
inline JpVector2i& operator += ( JpVector2i& vLeftHandArg, JpVector2i vRightHandArg )
{
	vLeftHandArg.X += vRightHandArg.X;
	vLeftHandArg.Y += vRightHandArg.Y;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  -
 PURPOSE:   used to subtract two vectors
 ARGS:      vRightHandArg gets subtracted from vLeftHandArg
******************************************************************************/
inline JpVector2i operator - ( JpVector2i vLeftHandArg, JpVector2i vRightHandArg )
{
	vLeftHandArg.X -= vRightHandArg.X;
	vLeftHandArg.Y -= vRightHandArg.Y;

	return vLeftHandArg;
}





/******************************************************************************
 FUNCTION:  -=
 PURPOSE:   used to subtract two vectors
 ARGS:      vRightHandArg gets subtracted from vLeftHandArg
******************************************************************************/
inline JpVector2i& operator -= ( JpVector2i& vLeftHandArg, JpVector2i vRightHandArg )
{
	vLeftHandArg.X -= vRightHandArg.X;
	vLeftHandArg.Y -= vRightHandArg.Y;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  ==
 PURPOSE:   used to compare two vectors for equality
******************************************************************************/
inline bool operator == ( JpVector2i vLeftHandArg, JpVector2i vRightHandArg )
{
	return
	(
		vLeftHandArg.X == vRightHandArg.X &&
		vLeftHandArg.Y == vRightHandArg.Y
	);
}




/******************************************************************************
 FUNCTION:  !=
 PURPOSE:   used to compare two vectors for inequality
******************************************************************************/
inline bool operator != ( JpVector2i vLeftHandArg, JpVector2i vRightHandArg )
{
	return
	(
		vLeftHandArg.X != vRightHandArg.X ||
		vLeftHandArg.Y != vRightHandArg.Y
	);
}





/******************************************************************************
 FUNCTION:  -
 PURPOSE:   unary negation
******************************************************************************/
inline const JpVector2i operator-( JpVector2i vArg )
{
	return vArg * -1;
}




/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
inline void JpVector2i::Init()
{
	X = Y = 0;
}




/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
inline void JpVector2i::Init( int iX, int iY )
{
	X = iX;
	Y = iY;
}




/******************************************************************************
 FUNCTION:  Length
******************************************************************************/
inline float JpVector2i::Length() const
{
	return sqrtf( (float)( X*X + Y*Y ) );
}




/******************************************************************************
 FUNCTION:  LengthSquared
******************************************************************************/
inline float JpVector2i::LengthSquared() const
{
	return (float)( X*X + Y*Y );
}




/******************************************************************************
 FUNCTION:  Rotate
******************************************************************************/
inline void JpVector2i::Rotate( float fRadians )
{
	float fCos = cosf( fRadians );
	float fSin = sinf( fRadians );

	float X2 = ( X * fCos ) - ( Y * fSin );
	float Y2 = ( Y * fCos ) + ( X * fSin );

	X = (int)X2;
	Y = (int)Y2;
}




/******************************************************************************
 FUNCTION:  DotProduct
 PURPOSE:   calculate the dot product between two vectors
 NOTES:     dot product is commutative.  ie DotProduct(a, b) == DotProduct(b, a)
******************************************************************************/
inline int DotProduct( JpVector2i vVec1, JpVector2i vVec2 )
{
	return
	(
		vVec1.X * vVec2.X +
		vVec1.Y * vVec2.Y
	);
}




/******************************************************************************
 FUNCTION:  Length
 PURPOSE:   return the length of the given vector.
******************************************************************************/
inline float Length( const JpVector2i& vVec )
{
	return vVec.Length();
}




/******************************************************************************
 FUNCTION:  LengthSquared
 PURPOSE:   return the squared length of the given vector.
******************************************************************************/
inline float LengthSquared( const JpVector2i& vVec )
{
	return vVec.LengthSquared();
}




/******************************************************************************
 FUNCTION:  Swap
******************************************************************************/
inline void Swap( JpVector2i* vVec1, JpVector2i* vVec2 )
{
	JpVector2i vTemp = *vVec2;
	*vVec2 = *vVec1;
	*vVec1 = vTemp;
}








/******************************************************************************
*******************************************************************************
*******************************************************************************
*******************************************************************************
*******************************************************************************

                                JpVector2f

*******************************************************************************
*******************************************************************************
*******************************************************************************
*******************************************************************************
******************************************************************************/


class JpVector2f
{
	public:

		//-----------------------
		// variable declarations
		float     X;
		float     Y;


		//-----------------------
		// function declarations
		JpVector2f()
		{ 
			// Components are not initialized!
			// This is for speed reasons.
		};

		JpVector2f( const JpVector2f& vSource )
		{
			X = vSource.X;
			Y = vSource.Y;
		}

		JpVector2f( float fX, float fY )
		{ 
			X = fX;
			Y = fY;
		};

		void Init();
		void Init( float fX, float fY );
		void Normalize();
		float Length() const;
		float LengthSquared() const;

		void Rotate( float fRadians );
		void Zero() { X = Y = 0.0f; }
		bool IsZero() { return ( X == 0.0f && Y == 0.0f ); }

	private:
};




// GLOBAL VECTOR FUNCTIONS
float DotProduct( JpVector2f vVec1, JpVector2f vVec2 );
float Length( const JpVector2f& vVec1 );
float LengthSquared( const JpVector2f& vVec1 );
JpVector2f Normalize( JpVector2f vVec );
void Swap( JpVector2f* vVec1, JpVector2f* vVec2 );

// GLOBAL VECTOR OVERLOADED OPERATORS
JpVector2f operator * ( JpVector2f vLeftHandArg, float fRightHandArg );
JpVector2f operator * ( float fLeftHandArg, JpVector2f vRightHandArg );
JpVector2f& operator *= ( JpVector2f& vLeftHandArg, float fRightHandArg );
JpVector2f operator + ( JpVector2f vLeftHandArg, JpVector2f vRightHandArg );
JpVector2f& operator += ( JpVector2f& vLeftHandArg, JpVector2f vRightHandArg );
JpVector2f operator - ( JpVector2f vLeftHandArg, JpVector2f vRightHandArg );
JpVector2f& operator -= ( JpVector2f& vLeftHandArg, JpVector2f vRightHandArg );
bool operator == ( JpVector2f vLeftHandArg, JpVector2f vRightHandArg );
bool operator != ( JpVector2f vLeftHandArg, JpVector2f vRightHandArg );
const JpVector2f operator-( JpVector2f vArg );




/******************************************************************************
 FUNCTION:  *
 PURPOSE:   used to multiply a vector by a scaler value
 NOTES:     There are two versions of the overloaded * so that the operation
            can be both float * JpVector2 and JpVector2 * float
******************************************************************************/
inline JpVector2f operator * ( JpVector2f vLeftHandArg, float fRightHandArg )
{
	vLeftHandArg.X *= fRightHandArg;
	vLeftHandArg.Y *= fRightHandArg;

	return vLeftHandArg;
}

inline JpVector2f operator * ( float fLeftHandArg, JpVector2f vRightHandArg )
{
	vRightHandArg.X *= fLeftHandArg;
	vRightHandArg.Y *= fLeftHandArg;

	return vRightHandArg;
}




/******************************************************************************
 FUNCTION:  *=
 PURPOSE:   used to multiply a vector by a scaler value
******************************************************************************/
inline JpVector2f& operator *= ( JpVector2f& vLeftHandArg, float fRightHandArg )
{
	vLeftHandArg.X *= fRightHandArg;
	vLeftHandArg.Y *= fRightHandArg;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  +
 PURPOSE:   used to add to vectors together
******************************************************************************/
inline JpVector2f operator + ( JpVector2f vLeftHandArg, JpVector2f vRightHandArg )
{
	vLeftHandArg.X += vRightHandArg.X;
	vLeftHandArg.Y += vRightHandArg.Y;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  +=
 PURPOSE:   used to add to vectors together
******************************************************************************/
inline JpVector2f& operator += ( JpVector2f& vLeftHandArg, JpVector2f vRightHandArg )
{
	vLeftHandArg.X += vRightHandArg.X;
	vLeftHandArg.Y += vRightHandArg.Y;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  -
 PURPOSE:   used to subtract two vectors
 ARGS:      vRightHandArg gets subtracted from vLeftHandArg
******************************************************************************/
inline JpVector2f operator - ( JpVector2f vLeftHandArg, JpVector2f vRightHandArg )
{
	vLeftHandArg.X -= vRightHandArg.X;
	vLeftHandArg.Y -= vRightHandArg.Y;

	return vLeftHandArg;
}





/******************************************************************************
 FUNCTION:  -=
 PURPOSE:   used to subtract two vectors
 ARGS:      vRightHandArg gets subtracted from vLeftHandArg
******************************************************************************/
inline JpVector2f& operator -= ( JpVector2f& vLeftHandArg, JpVector2f vRightHandArg )
{
	vLeftHandArg.X -= vRightHandArg.X;
	vLeftHandArg.Y -= vRightHandArg.Y;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  ==
 PURPOSE:   used to compare two vectors for equality
******************************************************************************/
inline bool operator == ( JpVector2f vLeftHandArg, JpVector2f vRightHandArg )
{
	return
	(
		vLeftHandArg.X == vRightHandArg.X &&
		vLeftHandArg.Y == vRightHandArg.Y
	);
}




/******************************************************************************
 FUNCTION:  !=
 PURPOSE:   used to compare two vectors for inequality
******************************************************************************/
inline bool operator != ( JpVector2f vLeftHandArg, JpVector2f vRightHandArg )
{
	return
	(
		vLeftHandArg.X != vRightHandArg.X ||
		vLeftHandArg.Y != vRightHandArg.Y
	);
}





/******************************************************************************
 FUNCTION:  -
 PURPOSE:   unary negation
******************************************************************************/
inline const JpVector2f operator-( JpVector2f vArg )
{
	return vArg * -1.0;
}




/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
inline void JpVector2f::Init()
{
	X = Y = 0;
}




/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
inline void JpVector2f::Init( float fX, float fY )
{
	X = fX;
	Y = fY;
}




/******************************************************************************
 FUNCTION:  Length
******************************************************************************/
inline float JpVector2f::Length() const
{
	return sqrtf( X*X + Y*Y );
}




/******************************************************************************
 FUNCTION:  LengthSquared
******************************************************************************/
inline float JpVector2f::LengthSquared() const
{
	return ( X*X + Y*Y );
}




/******************************************************************************
 FUNCTION:  Rotate
******************************************************************************/
inline void JpVector2f::Rotate( float fRadians )
{
	float fCos = cosf( fRadians );
	float fSin = sinf( fRadians );

	float X2 = ( X * fCos ) - ( Y * fSin );
	float Y2 = ( Y * fCos ) + ( X * fSin );

	X = X2;
	Y = Y2;
}




/******************************************************************************
 FUNCTION:  Normalize
******************************************************************************/
inline void JpVector2f::Normalize()
{
	float fLength = Length();
	float fOneOverLength = 0.0f;

	if ( fLength > 0.0f )
		fOneOverLength = ( 1.0f/fLength );

	X *= fOneOverLength;
	Y *= fOneOverLength;
}




/******************************************************************************
 FUNCTION:  DotProduct
 PURPOSE:   calculate the dot product between two vectors
 NOTES:     dot product is commutative.  ie DotProduct(a, b) == DotProduct(b, a)
******************************************************************************/
inline float DotProduct( JpVector2f vVec1, JpVector2f vVec2 )
{
	return
	(
		vVec1.X * vVec2.X +
		vVec1.Y * vVec2.Y
	);
}




/******************************************************************************
 FUNCTION:  Length
 PURPOSE:   return the length of the given vector.
******************************************************************************/
inline float Length( const JpVector2f& vVec )
{
	return vVec.Length();
}




/******************************************************************************
 FUNCTION:  LengthSquared
 PURPOSE:   return the squared length of the given vector.
******************************************************************************/
inline float LengthSquared( const JpVector2f& vVec )
{
	return vVec.LengthSquared();
}




/******************************************************************************
 FUNCTION:  Normalize
 ******************************************************************************/
inline JpVector2f Normalize( JpVector2f vVec )
{
	vVec.Normalize();
	return vVec;
}




/******************************************************************************
 FUNCTION:  Swap
******************************************************************************/
inline void Swap( JpVector2f* vVec1, JpVector2f* vVec2 )
{
	JpVector2f vTemp = *vVec2;
	*vVec2 = *vVec1;
	*vVec1 = vTemp;
}







/******************************************************************************
*******************************************************************************
*******************************************************************************
*******************************************************************************
*******************************************************************************

                                JpVector4f

*******************************************************************************
*******************************************************************************
*******************************************************************************
*******************************************************************************
******************************************************************************/


class JpVector4f
{
	public:

		//-----------------------
		// variable declarations
		float     X;
		float     Y;
		float     Z;
		float     W;


		//-----------------------
		// function declarations
		JpVector4f()
		{ 
			// Components are not initialized!
			// This is for speed reasons.
		};

		JpVector4f( const JpVector4f& vSource )
		{
			X = vSource.X;
			Y = vSource.Y;
			Z = vSource.Z;
			W = vSource.W;
		}

		JpVector4f( float fX, float fY, float fZ )
		{ 
			//Init( fX, fY, fZ, 1.0 );
			X = fX;
			Y = fY;
			Z = fZ;
			W = 0.0f;
		};

		JpVector4f( float fX, float fY, float fZ, float fW )
		{ 
			//Init( fX, fY, fZ, fW );
			X = fX;
			Y = fY;
			Z = fZ;
			W = fW;
		};

		void Init();
		void Init( float fX, float fY, float fZ, float fW );
		void Init( float fX, float fY, float fZ ) { Init( fX, fY, fZ, 1.0f ); }
		void Normalize();
		float Length() const;
		float LengthSquared() const;

		void NormalizeXY();
		float LengthXY() const;
		float LengthSquaredXY() const;

		void NormalizeXZ();
		float LengthXZ() const;
		float LengthSquaredXZ() const;

		void Zero() { X = Y = Z = W = 0.0f; }
		bool IsZero() { return ( X == 0.0f && Y == 0.0f && Z == 0.0f && W == 0.0f ); }


		JpVector2f GetVector2() { return JpVector2f( X, Y ); }

	private:
};




// GLOBAL VECTOR FUNCTIONS
JpVector4f CrossProduct( JpVector4f vVec1, JpVector4f vVec2 );
float DotProduct( JpVector4f vVec1, JpVector4f vVec2 );
float Length( const JpVector4f& vVec1 );
float LengthSquared( const JpVector4f& vVec1 );
float LengthXY( const JpVector4f& vVec1 );
float LengthSquaredXY( const JpVector4f& vVec1 );
float LengthXZ( const JpVector4f& vVec1 );
float LengthSquaredXZ( const JpVector4f& vVec1 );
JpVector4f Normalize( JpVector4f vVec );
JpVector4f NormalizeXY( JpVector4f vVec );
JpVector4f NormalizeXZ( JpVector4f vVec );
void Swap( JpVector4f* vVec1, JpVector4f* vVec2 );

JpVector2f GetVector2( JpVector4f& vVec );
JpVector4f GetVector4( JpVector2f& vVec );


// GLOBAL VECTOR OVERLOADED OPERATORS
JpVector4f operator * ( JpVector4f vLeftHandArg, float fRightHandArg );
JpVector4f operator * ( float fLeftHandArg, JpVector4f vRightHandArg );
JpVector4f& operator *= ( JpVector4f& vLeftHandArg, float fRightHandArg );
JpVector4f operator + ( JpVector4f vLeftHandArg, JpVector4f vRightHandArg );
JpVector4f& operator += ( JpVector4f& vLeftHandArg, JpVector4f vRightHandArg );
JpVector4f operator - ( JpVector4f vLeftHandArg, JpVector4f vRightHandArg );
JpVector4f& operator -= ( JpVector4f& vLeftHandArg, JpVector4f vRightHandArg );
bool operator == ( JpVector4f vLeftHandArg, JpVector4f vRightHandArg );
bool operator != ( JpVector4f vLeftHandArg, JpVector4f vRightHandArg );
const JpVector4f operator-( JpVector4f vArg );




/******************************************************************************
 FUNCTION:  *
 PURPOSE:   used to multiply a vector by a scaler value
 NOTES:     There are two versions of the overloaded * so that the operation
            can be both float * JpVector4 and JpVector4 * float
******************************************************************************/
inline JpVector4f operator * ( JpVector4f vLeftHandArg, float fRightHandArg )
{
	vLeftHandArg.X *= fRightHandArg;
	vLeftHandArg.Y *= fRightHandArg;
	vLeftHandArg.Z *= fRightHandArg;

	return vLeftHandArg;
}

inline JpVector4f operator * ( float fLeftHandArg, JpVector4f vRightHandArg )
{
	vRightHandArg.X *= fLeftHandArg;
	vRightHandArg.Y *= fLeftHandArg;
	vRightHandArg.Z *= fLeftHandArg;

	return vRightHandArg;
}




/******************************************************************************
 FUNCTION:  *=
 PURPOSE:   used to multiply a vector by a scaler value
******************************************************************************/
inline JpVector4f& operator *= ( JpVector4f& vLeftHandArg, float fRightHandArg )
{
	vLeftHandArg.X *= fRightHandArg;
	vLeftHandArg.Y *= fRightHandArg;
	vLeftHandArg.Z *= fRightHandArg;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  +
 PURPOSE:   used to add to vectors together
******************************************************************************/
inline JpVector4f operator + ( JpVector4f vLeftHandArg, JpVector4f vRightHandArg )
{
	vLeftHandArg.X += vRightHandArg.X;
	vLeftHandArg.Y += vRightHandArg.Y;
	vLeftHandArg.Z += vRightHandArg.Z;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  +=
 PURPOSE:   used to add to vectors together
******************************************************************************/
inline JpVector4f& operator += ( JpVector4f& vLeftHandArg, JpVector4f vRightHandArg )
{
	vLeftHandArg.X += vRightHandArg.X;
	vLeftHandArg.Y += vRightHandArg.Y;
	vLeftHandArg.Z += vRightHandArg.Z;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  -
 PURPOSE:   used to subtract two vectors
 ARGS:      vRightHandArg gets subtracted from vLeftHandArg
******************************************************************************/
inline JpVector4f operator - ( JpVector4f vLeftHandArg, JpVector4f vRightHandArg )
{
	vLeftHandArg.X -= vRightHandArg.X;
	vLeftHandArg.Y -= vRightHandArg.Y;
	vLeftHandArg.Z -= vRightHandArg.Z;

	return vLeftHandArg;
}





/******************************************************************************
 FUNCTION:  -=
 PURPOSE:   used to subtract two vectors
 ARGS:      vRightHandArg gets subtracted from vLeftHandArg
******************************************************************************/
inline JpVector4f& operator -= ( JpVector4f& vLeftHandArg, JpVector4f vRightHandArg )
{
	vLeftHandArg.X -= vRightHandArg.X;
	vLeftHandArg.Y -= vRightHandArg.Y;
	vLeftHandArg.Z -= vRightHandArg.Z;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  ==
 PURPOSE:   used to compare two vectors for equality
******************************************************************************/
inline bool operator == ( JpVector4f vLeftHandArg, JpVector4f vRightHandArg )
{
	return
	(
		vLeftHandArg.X == vRightHandArg.X &&
		vLeftHandArg.Y == vRightHandArg.Y &&
		vLeftHandArg.Z == vRightHandArg.Z 
	);
}




/******************************************************************************
 FUNCTION:  !=
 PURPOSE:   used to compare two vectors for inequality
******************************************************************************/
inline bool operator != ( JpVector4f vLeftHandArg, JpVector4f vRightHandArg )
{
	return
	(
		vLeftHandArg.X != vRightHandArg.X ||
		vLeftHandArg.Y != vRightHandArg.Y ||
		vLeftHandArg.Z != vRightHandArg.Z 
	);
}





/******************************************************************************
 FUNCTION:  -
 PURPOSE:   unary negation
******************************************************************************/
inline const JpVector4f operator-( JpVector4f vArg )
{
	return vArg * -1.0;
}




/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
inline void JpVector4f::Init()
{
	X = Y = Z = W = 0;
}




/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
inline void JpVector4f::Init( float fX, float fY, float fZ, float fW )
{
	X = fX;
	Y = fY;
	Z = fZ;
	W = fW;
}




/******************************************************************************
 FUNCTION:  Length
******************************************************************************/
inline float JpVector4f::Length() const
{
	return sqrtf( X*X + Y*Y + Z*Z );
}




/******************************************************************************
 FUNCTION:  LengthSquared
******************************************************************************/
inline float JpVector4f::LengthSquared() const
{
	return ( X*X + Y*Y + Z*Z );
}




/******************************************************************************
 FUNCTION:  LengthXY
******************************************************************************/
inline float JpVector4f::LengthXY() const
{
	return sqrtf( X*X + Y*Y );
}




/******************************************************************************
 FUNCTION:  LengthSquaredXY
******************************************************************************/
inline float JpVector4f::LengthSquaredXY() const
{
	return ( X*X + Y*Y );
}




/******************************************************************************
 FUNCTION:  LengthXZ
******************************************************************************/
inline float JpVector4f::LengthXZ() const
{
	return sqrtf( X*X + Z*Z );
}




/******************************************************************************
 FUNCTION:  LengthSquaredXZ
******************************************************************************/
inline float JpVector4f::LengthSquaredXZ() const
{
	return ( X*X + Z*Z );
}




/******************************************************************************
 FUNCTION:  Normalize
******************************************************************************/
inline void JpVector4f::Normalize()
{
	float fLength = Length();
	float fOneOverLength = 0.0f;

	if ( fLength > 0.0f )
		fOneOverLength = ( 1.0f/fLength );

	X *= fOneOverLength;
	Y *= fOneOverLength;
	Z *= fOneOverLength;
	W = 0.0f;
}




/******************************************************************************
 FUNCTION:  NormalizeXY
******************************************************************************/
inline void JpVector4f::NormalizeXY()
{
	float fLength = LengthXY();
	float fOneOverLength = 0.0f;

	if ( fLength > 0.0f )
		fOneOverLength = ( 1.0f/fLength );

	X *= fOneOverLength;
	Y *= fOneOverLength;
	Z = 0.0f;
	W = 0.0f;
}




/******************************************************************************
 FUNCTION:  NormalizeXZ
******************************************************************************/
inline void JpVector4f::NormalizeXZ()
{
	float fLength = LengthXZ();
	float fOneOverLength = 0.0f;

	if ( fLength > 0.0f )
		fOneOverLength = ( 1.0f/fLength );

	X *= fOneOverLength;
	Y = 0.0f;
	Z *= fOneOverLength;
	W = 0.0f;
}




/******************************************************************************
 FUNCTION:  CrossProduct
 PURPOSE:   calculate the cross product between two vectors
 ARGS:      vVec1 is crossed with vVec2
 NOTES:     cross product is anti-commutative.  
            ie CrossProduct( a, b ) == CrossProduct( -b, a )
******************************************************************************/
inline JpVector4f CrossProduct( JpVector4f vVec1, JpVector4f vVec2 )
{
	// need to call constructor to get rid of a release compile warning.  :/
	// mdomowicz 2009_02_16
	JpVector4f vResult( 0.0f, 0.0f, 0.0f ); 

	vResult.X = (vVec1.Y * vVec2.Z) - (vVec1.Z * vVec2.Y);
	vResult.Y = (vVec1.Z * vVec2.X) - (vVec1.X * vVec2.Z);
	vResult.Z = (vVec1.X * vVec2.Y) - (vVec1.Y * vVec2.X);

	return vResult;
}




/******************************************************************************
 FUNCTION:  DotProduct
 PURPOSE:   calculate the dot product between two vectors
 NOTES:     dot product is commutative.  ie DotProduct(a, b) == DotProduct(b, a)
******************************************************************************/
inline float DotProduct( JpVector4f vVec1, JpVector4f vVec2 )
{
	return
	(
		vVec1.X * vVec2.X +
		vVec1.Y * vVec2.Y +
		vVec1.Z * vVec2.Z
	);
}




/******************************************************************************
 FUNCTION:  Length
 PURPOSE:   return the length of the given vector.
******************************************************************************/
inline float Length( const JpVector4f& vVec )
{
	return vVec.Length();
}




/******************************************************************************
 FUNCTION:  LengthSquared
 PURPOSE:   return the squared length of the given vector.
******************************************************************************/
inline float LengthSquared( const JpVector4f& vVec )
{
	return vVec.LengthSquared();
}




/******************************************************************************
 FUNCTION:  LengthXY
******************************************************************************/
inline float LengthXY( const JpVector4f& vVec )
{
	return vVec.LengthXY();
}




/******************************************************************************
 FUNCTION:  LengthSquaredXY
******************************************************************************/
inline float LengthSquaredXY( const JpVector4f& vVec )
{
	return vVec.LengthSquaredXY();
}




/******************************************************************************
 FUNCTION:  LengthSquaredXZ
******************************************************************************/
inline float LengthSquaredXZ( const JpVector4f& vVec )
{
	return vVec.LengthSquaredXZ();
}




/******************************************************************************
 FUNCTION:  Normalize
 ******************************************************************************/
inline JpVector4f Normalize( JpVector4f vVec )
{
	vVec.Normalize();
	return vVec;
}




/******************************************************************************
 FUNCTION:  NormalizeXY
 ******************************************************************************/
inline JpVector4f NormalizeXY( JpVector4f vVec )
{
	vVec.NormalizeXY();
	return vVec;
}




/******************************************************************************
 FUNCTION:  NormalizeXZ
 ******************************************************************************/
inline JpVector4f NormalizeXZ( JpVector4f vVec )
{
	vVec.NormalizeXZ();
	return vVec;
}




/******************************************************************************
 FUNCTION:  Swap
******************************************************************************/
inline void Swap( JpVector4f* vVec1, JpVector4f* vVec2 )
{
	JpVector4f vTemp = *vVec2;
	*vVec2 = *vVec1;
	*vVec1 = vTemp;
}




/******************************************************************************
 FUNCTION:  GetVector2
******************************************************************************/
inline JpVector2f GetVector2( JpVector4f& vVec )
{
	JpVector2f vResult( vVec.X, vVec.Y );
	return vResult;
}




/******************************************************************************
 FUNCTION:  GetVector4
******************************************************************************/
inline JpVector4f GetVector4( JpVector2f& vVec )
{
	JpVector4f vResult( vVec.X, vVec.Y, 0.0f, 0.0f );
	return vResult;
}

