/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"

#include "JpHashTable.h"
#include "JpString.h"
#include "NeConsole.h"



// !! This Ne level class shouldn't make use of an So level class, but not
// sure how best to fix this now.  Will have to revisit later.
// mdomowicz 2011_11_29
#include "SoInputKeyboard.h"
#include "SoInputOisManager.h"

#include "mmgrChooser.h"




NeConsole* NeConsole::_pInstance = NULL;



/******************************************************************************
 FUNCTION: Init
******************************************************************************/
void NeConsole::Init()
{
	static std::string sName = "Dcg";
	static int sHp = 0;
	AddItem("name", &sName, NECONSOLE_STRING);
	AddItem("hp", (void*)&sHp, NECONSOLE_INT);
	AddItem(".remove", (void*)&RemoveCommand, NECONSOLE_FUNCTION );
	AddItem(".list", (void*)&PrintList, NECONSOLE_FUNCTION );
	AddItem(".clear", (void*)&Clear, NECONSOLE_FUNCTION );
	SetDefaultCommand( &ErrorFunction );


	_sOutput = "";
	_sSuggestionPartial = "";

	_iCommandListOffset = 0;
	_iCommandListSize = 15;
	_iSuggestionListOffset = 0;

	_bBackspaceIsDown = false;
	_bSkipAutoSuggest = false;

	_fBackspaceCooldown = 0.0f;

	_bNextStringIsAUserAnswerNotACommand = false;
}



/******************************************************************************
 FUNCTION: Deinit
******************************************************************************/
void NeConsole::Deinit()
{
	_ItemTable.DeleteAll();
	_CommandList.DeleteAll();
	_CommandQueue.DeleteAll();
	_SuggestionList.DeleteAll();
}



/*****************************************************************************
 FUNCTION: Update
******************************************************************************/
void NeConsole::Update( float dt )
{
	if ( _bNextStringIsAUserAnswerNotACommand )
		SetCurrentSuggestion( "" );
	else
		SetSuggestion();

	UpdateRepeatBackspace( dt );
}



/******************************************************************************
 FUNCTION: UpdateInputPressed
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void NeConsole::UpdateInputPressed( OIS::KeyCode eKeyCode )
	{
		if ( _bNextStringIsAUserAnswerNotACommand )
		{
			UpdateInputPressedForAnswerString( eKeyCode );
		}
		else
		{
			UpdateInputPressedForCommand( eKeyCode );
		}
	}
#endif



/******************************************************************************
 FUNCTION: UpdateInputPressedForCommand
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void NeConsole::UpdateInputPressedForCommand( OIS::KeyCode eKeyCode )
	{
		_ePrevInputPressed = _eLastInputPressed;
		_eLastInputPressed = eKeyCode;

		_bSkipAutoSuggest = false;

		switch ( _eLastInputPressed )
		{
			case  OIS::KC_NUMPADENTER:
			case  OIS::KC_TAB:
			case  OIS::KC_RETURN:
			{
				if ( _sSuggestionPartial.length() == 0 )
				{
					ParseCommandLine( _sCommand );
					_sCommand.clear();
					_sOutput.clear();
					SetCurrentCommandLine( _sOutput );
					SetCurrentSuggestion( "" );
				}
				else
				{
					int iCommandSize = _sCommand.length();
					_sOutput.erase( _sOutput.length() - iCommandSize, iCommandSize );
					_sOutput += _sSuggestionFull;
					SetCurrentCommandLine( _sOutput );
					_sCommand = _sSuggestionFull;
					_sSuggestionPartial.clear();
					_sSuggestionFull.clear();
					SetCurrentSuggestion( "" );
				}
				break;
			}
			case  OIS::KC_DELETE:
			case  OIS::KC_BACK:
			case  OIS::KC_LEFT:
			{
				if ( _sCommand.length() != 0 )
				{
					_sCommand.erase( _sCommand.length() - 1 );
					_sOutput.erase( _sOutput.length() - 1 );
					SetCurrentCommandLine( _sOutput );
				}
				_bBackspaceIsDown = true;
				_fBackspaceCooldown = 0.2f;
				break;
			}
			case  OIS::KC_UP:
			{
				if ( _sSuggestionPartial.length() == 0 )
				{
					// Up is 'command history'
					if ( _CommandList.GetNumEntries() > 0 )
					{
						_iCommandListOffset = Clamp( _iCommandListOffset + 1, 0, _CommandList.GetNumEntries() );

						std::string* psCommand = _CommandList.GetAtIndex( _CommandList.GetNumEntries() - _iCommandListOffset )->GetData();

						// We dont want to hit up and have the next command in history be identical
						// to the command we just ran.
						while 
							( 
							psCommand && 
							strcmp( psCommand->data(), _sCommand.data() ) == 0 &&
							(_iCommandListOffset != (_CommandList.GetNumEntries()))
							)
						{
							_iCommandListOffset = Clamp( _iCommandListOffset + 1, 0, _CommandList.GetNumEntries() );

							psCommand = _CommandList.GetAtIndex( _CommandList.GetNumEntries() - _iCommandListOffset )->GetData();
						}

						// If we found a valid new command to display, then do it
						if ( psCommand )
						{
							int iLength = _sCommand.length();

							_sOutput.erase( _sOutput.length() - iLength, iLength );
							_sCommand.clear();
							_sCommand += *psCommand;		
							_sOutput  += *psCommand;

							SetCurrentCommandLine( _sOutput );
						}
					}
				}
				else
				{
					if ( _SuggestionList.GetNumEntries() > 0 )
					{
						_iSuggestionListOffset ++;

						if ( _iSuggestionListOffset >= _SuggestionList.GetNumEntries() )
							_iSuggestionListOffset = 0;

						std::string* psSuggestion = _SuggestionList.GetAtIndex( _iSuggestionListOffset )->GetData();

						// If we found a valid new suggestion to display, then do it
						if ( psSuggestion )
						{
							int iLength = _sCommand.length();

							_sSuggestionFull    = *psSuggestion;
							_sSuggestionPartial = *psSuggestion;
							_sSuggestionPartial = _sSuggestionPartial.erase( 0, iLength );
							_bSkipAutoSuggest = true;
							SetCurrentSuggestion( _sSuggestionFull );
						}
					}
				}
				break;
			}
			case  OIS::KC_DOWN:
			{
				if ( _sSuggestionPartial.length() == 0 )
				{
					// Down is also 'command history', but down
					if ( _CommandList.GetNumEntries() > 0 )
					{
						_iCommandListOffset = Clamp( _iCommandListOffset - 1, 0, _CommandList.GetNumEntries() );

						std::string* psCommand = _CommandList.GetAtIndex( _CommandList.GetNumEntries() - 1 - _iCommandListOffset )->GetData();

						// We dont want to hit up and have the next command in history be identical
						// to the command we just ran.
						while 
							( 
							psCommand && 
							strcmp( psCommand->data(), _sCommand.data() ) == 0 &&
							( _iCommandListOffset != 0 )
							)
						{
							_iCommandListOffset = Clamp( _iCommandListOffset - 1, 0, _CommandList.GetNumEntries() );

							psCommand = _CommandList.GetAtIndex( _CommandList.GetNumEntries() - 1 - _iCommandListOffset )->GetData();
						}

						// If we found a valid new command to display, then do it
						if ( psCommand )
						{
							int iLength = _sCommand.length();

							_sOutput.erase( _sOutput.length() - iLength, iLength );
							_sCommand.clear();
							_sCommand += *psCommand;		
							_sOutput  += *psCommand;

							SetCurrentCommandLine( _sOutput );
						}
					}
				}
				else
				{
					if ( _SuggestionList.GetNumEntries() > 0 )
					{
						_iSuggestionListOffset --;

						if ( _iSuggestionListOffset < 0 )
							_iSuggestionListOffset = _SuggestionList.GetNumEntries() - 1;

						std::string* psSuggestion = _SuggestionList.GetAtIndex( _iSuggestionListOffset )->GetData();

						// If we found a valid new suggestion to display, then do it
						if ( psSuggestion )
						{
							int iLength = _sCommand.length();

							_sSuggestionFull    = *psSuggestion;
							_sSuggestionPartial = *psSuggestion;
							_sSuggestionPartial = _sSuggestionPartial.erase( 0, iLength );
							_bSkipAutoSuggest = true;
							SetCurrentSuggestion( _sSuggestionFull );
						}
					}
				}

				break;

			}
			case  OIS::KC_GRAVE:
			{
				break;
			}

			case  OIS::KC_ESCAPE:
			{
				_sCommand.clear();
				_sOutput.clear();
				SetCurrentCommandLine( _sOutput );
				SetCurrentSuggestion( "" );
				break;
			}

			case  OIS::KC_LSHIFT:
			case  OIS::KC_RSHIFT:
			case  OIS::KC_LCONTROL:
			case  OIS::KC_RCONTROL:
			case  OIS::KC_LMENU:  // L Alt
			case  OIS::KC_RMENU:  // R Alt
				break;

			default:
			{
				// Most keys go to the console, here...
				if ( _eLastInputPressed != OIS::KC_UNASSIGNED )
				{
					char cChar = SoInputOisManager::GetCharacterFromKeyCode( eKeyCode );
					if 
					( 
						SoInputOisManager::GetInstance()->GetOisKeyboard()->isKeyDown( OIS::KC_LSHIFT ) ||
						SoInputOisManager::GetInstance()->GetOisKeyboard()->isKeyDown( OIS::KC_RSHIFT )
					)
					{
						cChar = toupper( cChar );
					}

					if( _sCommand.length() < 255 ) 
					{
						_sCommand += cChar;	
						_sOutput += cChar;	
						SetCurrentCommandLine( _sOutput );
					}
				}
				break;
			}
		};


		// Reset our history starting point if we stop hitting up/down
		if ( _eLastInputPressed != OIS::KC_DOWN && _eLastInputPressed != OIS::KC_UP )
			_iCommandListOffset = 0;
	}
#endif // DCG_IS_TARGET_WIN32




/******************************************************************************
 FUNCTION: UpdateInputPressedForAnswerString
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void NeConsole::UpdateInputPressedForAnswerString( OIS::KeyCode eKeyCode )
	{
		_ePrevInputPressed = _eLastInputPressed;
		_eLastInputPressed = eKeyCode;

		_bSkipAutoSuggest = false;

		switch ( _eLastInputPressed )
		{
			case  OIS::KC_NUMPADENTER:
			case  OIS::KC_TAB:
			case  OIS::KC_RETURN:
			{
				if ( _sOutput != "" )
				{
					_sUserAnswerString = _sCommand;
					AddOutputString( _sUserAnswerString );
					_sCommand.clear();
					_sOutput.clear();
					SetCurrentCommandLine( _sOutput );
					SetCurrentSuggestion( "" );
					_bNextStringIsAUserAnswerNotACommand = false;
				}

				break;
			}
			case  OIS::KC_DELETE:
			case  OIS::KC_BACK:
			case  OIS::KC_LEFT:
			{
				if ( _sCommand.length() != 0 )
				{
					_sCommand.erase( _sCommand.length() - 1 );
					_sOutput.erase( _sOutput.length() - 1 );
					SetCurrentCommandLine( _sOutput );
				}
				_bBackspaceIsDown = true;
				_fBackspaceCooldown = 0.2f;
				break;
			}
			case  OIS::KC_GRAVE:
			case  OIS::KC_ESCAPE:
			case  OIS::KC_LSHIFT:
			case  OIS::KC_RSHIFT:
			case  OIS::KC_LCONTROL:
			case  OIS::KC_RCONTROL:
			case  OIS::KC_LMENU:  // L Alt
			case  OIS::KC_RMENU:  // R Alt
				break;

			default:
			{
				// Most keys go to the console, here...
				if ( _eLastInputPressed != OIS::KC_UNASSIGNED )
				{
					char cChar = SoInputOisManager::GetCharacterFromKeyCode( eKeyCode );
					if 
					( 
						SoInputOisManager::GetInstance()->GetOisKeyboard()->isKeyDown( OIS::KC_LSHIFT ) ||
						SoInputOisManager::GetInstance()->GetOisKeyboard()->isKeyDown( OIS::KC_RSHIFT )
					)
					{
						cChar = toupper( cChar );
					}

					if( _sCommand.length() < 255 ) 
					{
						_sCommand += cChar;	
						_sOutput += cChar;	
						SetCurrentCommandLine( _sOutput );
					}
				}
				break;
			}
		};
	}
#endif



/******************************************************************************
 FUNCTION: UpdateVirtualKeyboardInputPressed
******************************************************************************/
void NeConsole::UpdateVirtualKeyboardInputPressed( char cKey )
{
	_bSkipAutoSuggest = false;

	switch ( cKey )
	{
		case '\n':
		{
			if ( _sSuggestionPartial.length() == 0 )
			{
				ParseCommandLine( _sCommand );
				_sCommand.clear();
				_sOutput.clear();
				SetCurrentCommandLine( _sOutput );
				SetCurrentSuggestion( "" );
			}
			else
			{
				int iCommandSize = _sCommand.length();
				_sOutput.erase( _sOutput.length() - iCommandSize, iCommandSize );
				_sOutput += _sSuggestionFull;
				SetCurrentCommandLine( _sOutput );
				_sCommand = _sSuggestionFull;
				_sSuggestionPartial.clear();
				_sSuggestionFull.clear();
				SetCurrentSuggestion( "" );
			}
			break;
		}

		default:
		{
			if( _sCommand.length() < 255 ) 
			{
				_sCommand += cKey;	
				_sOutput += cKey;	
				SetCurrentCommandLine( _sOutput );
			}
			break;
		}
	};
}



/******************************************************************************
 FUNCTION: UpdateVirtualKeyboardDeleteBackward
******************************************************************************/
void NeConsole::UpdateVirtualKeyboardDeleteBackward()
{
	if ( _sCommand.length() != 0 )
	{
		_sCommand.erase( _sCommand.length() - 1 );
		_sOutput.erase( _sOutput.length() - 1 );
		SetCurrentCommandLine( _sOutput );
	}
}



/******************************************************************************
 FUNCTION: UpdateInputReleased
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void NeConsole::UpdateInputReleased( OIS::KeyCode eKeyCode )
	{
		_eLastKeyReleased = eKeyCode;

		if  ( 
			_eLastKeyReleased == OIS::KC_BACK ||
			_eLastKeyReleased == OIS::KC_DELETE ||
			_eLastKeyReleased == OIS::KC_LEFT
			)
		{
			_bBackspaceIsDown = false;
			_fBackspaceCooldown = 0.2f;
		}
	}
#endif


/******************************************************************************
 FUNCTION: UpdateRepeatBackspace
******************************************************************************/
void NeConsole::UpdateRepeatBackspace( float dt )
{
	if ( _bBackspaceIsDown )
	{
		_fBackspaceCooldown -= dt;

		if ( _fBackspaceCooldown <= 0.0f )
		{
			if ( _sCommand.length() != 0 )
			{
				_sCommand.erase( _sCommand.length() - 1 );
				_sOutput.erase( _sOutput.length() - 1 );
				SetCurrentCommandLine( _sOutput );
			}

			_fBackspaceCooldown = 0.09f;
		}
	}
}



/******************************************************************************
 FUNCTION: PadString
******************************************************************************/
std::string NeConsole::PadString( std::string sString, int iWidth )
{
	// If the font has consistent character widths, this will make nice columns
	int sLength = sString.length();

	for ( int i = sLength; i <= iWidth; i ++ )
	{
		sString += "_";
	}

	return sString;
}



/******************************************************************************
 FUNCTION: Print
******************************************************************************/
void NeConsole::Print( std::string sString )
{
	AddOutputString( sString );
}



/******************************************************************************
 FUNCTION: ParseCommandLine
******************************************************************************/
bool NeConsole::ParseCommandLine( std::string sCommandLine )
{
	std::ostringstream Out;
	std::string::size_type Index = 0;
	std::vector<std::string> Arguments;

	// add to text buffer
	Print( sCommandLine );

	// add to command line buffer
	std::string* sNewString = new std::string;
	*sNewString = sCommandLine;

	if ( _CommandList.GetNumEntries() > 0 )
	{
		// If this command is the same as the previous command, don't add it to our list
		if ( strcmp( sNewString->data(), _CommandList.GetLast()->GetData()->data() ) != 0 )
			_CommandList.AddLast( sNewString );
		else
			delete sNewString;
	}
	else
		_CommandList.AddLast( sNewString );

	if ( (int)_CommandList.GetNumEntries() > _iCommandListSize ) 
		_CommandList.Delete( _CommandList.GetFirst()->GetData() );


	TokenizeString( &Arguments, sCommandLine );

	if ( Arguments.size() > 0 )
	{
		NeConsoleItem* pConsoleItem = _ItemTable.Find( JpHashWithCase( Arguments[0].data() ) );

		if ( pConsoleItem )
		{
			switch( pConsoleItem->_Type )
			{
				case NECONSOLE_BOOL:
				{
					if( Arguments.size() > 2 )
					{
						return true;
					}
					else if( Arguments.size() == 1 )
					{
						Out.str("");
						std::string sBool;
						if ( *( (bool*)pConsoleItem->pVariable ) )
							sBool = "true";
						else
							sBool = "false";
						Out << pConsoleItem->_sName << " = " << sBool.c_str();
						Print(Out.str());
						return true;
					}
					else if( Arguments.size() == 2 )
					{
						*( (bool*)pConsoleItem->pVariable ) = JpString::ParseBool( Arguments[1].c_str() );
						return true;
					}
					break;
				}
				case NECONSOLE_UCHAR:
				{
					if( Arguments.size() > 2 )
					{
						return true;
					}
					else if( Arguments.size() == 1 )
					{
						Out.str("");
						Out << pConsoleItem->_sName << " = " << *( (unsigned char *)pConsoleItem->pVariable );
						Print(Out.str());
						return true;
					}
					else if( Arguments.size() == 2 )
					{
						*( (unsigned char *)pConsoleItem->pVariable ) = (unsigned char)atoi( Arguments[1].c_str() );
						return true;
					}
					break;
				}
				case NECONSOLE_CHAR:
					if(Arguments.size() > 2)
					{
						return true;
					}
					else if(Arguments.size() == 1)
					{
						Out.str("");
						Out << pConsoleItem->_sName << " = " << *( (char *)pConsoleItem->pVariable );
						Print(Out.str());
						return true;
					}
					else if(Arguments.size() == 2)
					{
						*( (char *)pConsoleItem->pVariable ) = (char)atoi( Arguments[1].c_str() );
						return true;
					}
					break;

				case NECONSOLE_UINT:
					if( Arguments.size() > 2 )
					{
						return true;
					}
					else if( Arguments.size() == 1 )
					{
						Out.str("");
						Out << pConsoleItem->_sName << " = " << *( (unsigned int *)pConsoleItem->pVariable );
						Print(Out.str());
						return true;
					}
					if(Arguments.size() == 2)
					{
						*( (unsigned int *)pConsoleItem->pVariable ) = (unsigned int)atoi( Arguments[1].c_str() );
						return true;
					}
					break;

				case NECONSOLE_INT:
					if( Arguments.size() > 2 )
					{
						return true;
					}
					else if( Arguments.size() == 1 )
					{
						Out.str("");
						Out << pConsoleItem->_sName << " = " << *((int *)pConsoleItem->pVariable);
						Print(Out.str());
						return true;
					}
					if( Arguments.size() == 2 )
					{
						int iNew = JpString::ParseInt( Arguments[1].data() );
						*( (int *)pConsoleItem->pVariable ) = iNew;
						return true;
					}
					break;

				case NECONSOLE_FLOAT:
					if( Arguments.size() > 2 )
					{
						return true;
					}
					else if( Arguments.size() == 1 )
					{
						Out.str( "" );
						Out << pConsoleItem->_sName << " = " << *( (float *)pConsoleItem->pVariable );
						Print(Out.str());
						return true;
					}
					if(Arguments.size() == 2)
					{
						*( (float *)pConsoleItem->pVariable ) = (float)atof( Arguments[1].c_str() );
						return true;
					}
					break;

				case NECONSOLE_STRING:
					if( Arguments.size() == 1 )
					{
						Out.str("");
						Out << pConsoleItem->_sName << " = " << *(std::string *)pConsoleItem->pVariable;
						Print(Out.str());
						return false;
					}
					else if( Arguments.size() > 1 )
					{
						// reset variable
						*( (std::string *)pConsoleItem->pVariable ) = "";

						// add new string
						for( int i = 1; i < (int)Arguments.size(); ++i )
						{
							*( (std::string *)pConsoleItem->pVariable ) += Arguments[i];
						}
						return true;
					}
					break;

				case NECONSOLE_FUNCTION:
					pConsoleItem->pFunction( this, Arguments );
					return true;
					break;

				default:
					DefaultCommand( this, Arguments );
					return false;
					break;
			}
		}


		// Couldn't match the command
		DefaultCommand( this, Arguments );
	}


	return false;
}


/******************************************************************************
 FUNCTION: TokenizeString
******************************************************************************/
void NeConsole::TokenizeString( std::vector<std::string>* psResults, std::string sToBeTokenized )
{
	struct Local
	{
		static bool IsDelimeter( char c ) { return  ( c == '=' || c == ' ' || c == '\n' || c == '\t' ); }
	};


	const int STATE_READ_WORD = 0;
	const int STATE_EAT_DELIMETER = 1;
	const int STATE_DONE = -1;

	int iCurrentState = STATE_READ_WORD;
	int iStart = 0;
	int iEnd = 0;
	int iMax = sToBeTokenized.length();

	// This is a small state machine for tokenizing the string.
	// It turns "Foo Bar", "Foo=Bar", "Foo = Bar", "Foo =  === = Bar", etc
	// into "Foo" and "Bar"
	while ( iCurrentState != STATE_DONE )
	{
		switch ( iCurrentState )
		{
			case STATE_READ_WORD:
			{
				char c = sToBeTokenized[ iStart ];
				while ( !Local::IsDelimeter( c ) && iEnd < iMax )
				{
					iEnd ++;
					c = sToBeTokenized[ iEnd ];
				}

				if ( iStart != iEnd )
				{
					psResults->push_back( sToBeTokenized.substr( iStart, iEnd ) );
					iStart = iEnd;
					iCurrentState = STATE_EAT_DELIMETER;
				}
				else
				{
					iCurrentState = STATE_DONE;
				}
				
				break;
			}
			case STATE_EAT_DELIMETER:
			{
				char c = sToBeTokenized[ iStart ];
				while ( Local::IsDelimeter( c ) && iEnd < iMax)
				{
					iEnd ++;
					c = sToBeTokenized[ iEnd ];
				}

				if ( iStart != iEnd )
				{
					iStart = iEnd;
					iCurrentState = STATE_READ_WORD;
				}
				else
				{
					iCurrentState = STATE_DONE;
				}

				break;
			}
			default:
				ASSERTF( 0, ( "\nUnknown state" ) );
		}
	}
}



/******************************************************************************
 FUNCTION: SendMessage
******************************************************************************/
void NeConsole::SendMessage( std::string sCommand )
{
	// If you are using multiple threads this is where you 
	// should grab the semaphore to prevent data corruption.
	_CommandQueue.AddLast( &sCommand );
	// Release the semaphore here.
}



/******************************************************************************
 FUNCTION: AddItem
******************************************************************************/
void NeConsole::AddItem( const char* pacName, void *pPointer, NeConsoleItemType Type )
{
	hash hKey = JpHashWithCase( pacName );

	if ( !_ItemTable.Exists( hKey ) )
	{
		NeConsoleItem* pNewConsoleItem = new NeConsoleItem;

		// fill item properties
		pNewConsoleItem->_sName = pacName;
		pNewConsoleItem->_Type  = Type;

		// address
		if( Type == NECONSOLE_FUNCTION )
			pNewConsoleItem->pFunction = (NeConsoleFunction)pPointer;
		else
			pNewConsoleItem->pVariable = pPointer;

		_ItemTable.Store( hKey, pNewConsoleItem );
	}
}



/******************************************************************************
 FUNCTION: DeleteItem
******************************************************************************/
void NeConsole::DeleteItem( const char* pacName )
{
	NeConsoleItem* pItem = _ItemTable.Find( JpHashWithCase( pacName ) );
	_ItemTable.Remove( JpHashWithCase( pacName ) );
	
	delete pItem;
}



/******************************************************************************
 FUNCTION: ParseCommandQueue
******************************************************************************/
void NeConsole::ParseCommandQueue()
{
	JpList< std::string >::Iterator Iter( &_CommandQueue );

	while ( Iter.Next() )
	{
		ParseCommandLine( Iter.GetNodeData()->data() );
	}

	_CommandQueue.DeleteAll();
}



/******************************************************************************
 FUNCTION: StringToUpper
******************************************************************************/
inline void NeConsole::StringToUpper( char* cpString )
{    
	while ( *cpString )    
	{        
		*cpString = toupper(*cpString);
		cpString ++;
	}
}



/******************************************************************************
 FUNCTION: SetSuggestion
******************************************************************************/
void NeConsole::SetSuggestion()
{
	if ( !_bSkipAutoSuggest )
	{
		_sSuggestionFull.clear();
		_sSuggestionPartial.clear();
		_SuggestionList.DeleteAll();

		if ( _sCommand.length() > 0 )
		{
			int iClosestMatch = 0;
			NeConsoleItemTable::Iterator Iter( GetItemTable() );

			while ( Iter.Next() )
			{
				NeConsoleItem* pConsoleItem = Iter.GetData();

				std::string sName( pConsoleItem->_sName.c_str() );
				std::string sNameUpper( pConsoleItem->_sName.c_str() );
				std::string sCommandUpper( _sCommand.c_str() );

				// Change all chars to uppercase for apple/apple comparison
				StringToUpper( (char*)sNameUpper.data() );
				StringToUpper( (char*)sCommandUpper.data() );

				//int iNumMatched = ( strspn( sNameUpper.c_str(), sCommandUpper.c_str() ) );
				int iNumMatched = ( CustomStrspn( sNameUpper, sCommandUpper ) );

				if ( iNumMatched > 0 )
				{
					if ( iNumMatched == _sCommand.length() )
					{
						std::string* pNewString = new std::string;
						*pNewString = sName;
						_SuggestionList.AddLast( pNewString );
					}
				}

				if ( iNumMatched > iClosestMatch )
				{
					if ( iNumMatched == (int)_sCommand.length() )
					{
						iClosestMatch = iNumMatched;
						_sSuggestionFull    = sName;
						_sSuggestionPartial = sName.erase( 0, _sCommand.length() );
						SetCurrentSuggestion( _sSuggestionFull );
					}
					else
					{
						_sSuggestionFull.clear();
						_sSuggestionPartial.clear();
						SetCurrentSuggestion( "" );
					}
				}
			}
		}
		else
		{
			_sSuggestionFull.clear();
			_sSuggestionPartial.clear();
			SetCurrentSuggestion( "" );
		}
	}
}



/******************************************************************************
 FUNCTION: CustomStrspn
 NOTES:    The standard C strspn() seemed to error on _fEffectsVolume ( always 
           returning '6' wtf ), so I wrote this to do the same thing.
******************************************************************************/
int NeConsole::CustomStrspn( std::string sString1, std::string sPattern )
{
	int iLength = 0;

	const char* pString  = sString1.c_str();
	const char* pPattern = sPattern.c_str(); 

	while ( pString[ iLength ] == pPattern[ iLength ] && iLength < (int)sPattern.length() )
	{
		iLength++;
	}

	return iLength;
}



/******************************************************************************
 FUNCTION: WaitForUserAnswerString
 NOTE:     This is for when the programmer wants to query the user for
           a response.  The next string entered to the console is not
           considered a command, but rather is provided as data for the
           programmer to retrieve.
******************************************************************************/
void NeConsole::WaitForUserAnswerString()
{
	_bNextStringIsAUserAnswerNotACommand = true; 
	ClearUserAnswerString();
}



/******************************************************************************
 FUNCTION: ClearUserAnswerString
 NOTE:     Not sure if it's safe to clear strings in headers when using
           mmgr, so just to be safe, this is implemented here.
******************************************************************************/
void NeConsole::ClearUserAnswerString() 
{
	_sUserAnswerString = ""; 
}



/******************************************************************************
 FUNCTION: SetInstance
******************************************************************************/
void NeConsole::SetInstance( NeConsole* p )
{
	if ( p )
	{
		ASSERTF( !_pInstance, ( "\nInstance already set!" ) );
		if ( !_pInstance )
			_pInstance = p;
	}
	else
	{
		_pInstance = p;
	}
}







/******************************************************************************
 FUNCTION: ErrorFunction
******************************************************************************/
void NeConsole::ErrorFunction( NeConsole* pInstance, const std::vector<std::string> & args)
{
	pInstance->Print( "Unknown command " + args[0] );
}



/******************************************************************************
 FUNCTION: RemoveCommand
******************************************************************************/
void NeConsole::RemoveCommand( NeConsole* pInstance, const std::vector<std::string> & args)
{
	pInstance->DeleteItem(args[1].c_str());
}



/******************************************************************************
 FUNCTION: PrintList
******************************************************************************/
void NeConsole::PrintList( NeConsole* pInstance, const std::vector<std::string> & args )
{
	NeConsoleItemTable::Iterator Iter( pInstance->GetItemTable() );

	while ( Iter.Next() )
	{
		NeConsoleItem* pConsoleItem = Iter.GetData();

		std::string sString = pInstance->PadString( pConsoleItem->_sName, 20 );

		switch ( pConsoleItem->_Type )
		{
			case NECONSOLE_BOOL:
			{
				bool b = *(bool*)pConsoleItem->pVariable;
				sString += pInstance->PadString( JpString::BoolToString( b ), 20 ) + "CTYPE_BOOL";

				break;
			}
			case NECONSOLE_UCHAR:
			{
				unsigned char uc = *(unsigned char*)pConsoleItem->pVariable;
				sString += pInstance->PadString( JpString::UnsignedCharToString( uc ), 20 ) + "CTYPE_UCHAR";

				break;
			}
			case NECONSOLE_CHAR:
			{
				char uc = *(char*)pConsoleItem->pVariable;
				sString += pInstance->PadString( JpString::CharToString( uc ), 20 ) + "CTYPE_CHAR";

				break;
			}
			case NECONSOLE_UINT:
			{
				unsigned int ui = *(unsigned int*)pConsoleItem->pVariable;
				sString += pInstance->PadString( JpString::UnsignedIntToString( ui ), 20 ) + "CTYPE_UINT";

				break;
			}
			case NECONSOLE_INT:
			{
				int i = *(int*)pConsoleItem->pVariable;
				sString += pInstance->PadString( JpString::IntToString( i ), 20 ) + "CTYPE_INT";

				break;
			}
			case NECONSOLE_FLOAT:
			{
				float f = *(float*)pConsoleItem->pVariable;
				sString += pInstance->PadString( JpString::FloatToString( f ), 20 ) + "CTYPE_FLOAT";

				break;
			}
			case NECONSOLE_STRING:
			{
				std::string s = *(std::string*)pConsoleItem->pVariable;
				sString += pInstance->PadString( s, 20 );
				sString += "CTYPE_STRING";

				break;
			}
			case NECONSOLE_FUNCTION:
			{
				sString += "CTYPE_FUNCTION";

				break;
			}
		default:
			break;
		};

		pInstance->Print( sString );
	}
}



/******************************************************************************
 FUNCTION: Clear
******************************************************************************/
void NeConsole::Clear( NeConsole* pInstance, const std::vector<std::string> & args )
{
	pInstance->ClearConsole();
}




