/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "NeCsvOut.h"

#include "mmgrChooser.h"



/******************************************************************************
 FUNCTION:  NeCsvWriter
******************************************************************************/
NeCsvOut::NeCsvOut()
{
}



/******************************************************************************
 FUNCTION:  ~NeCsvWriter
******************************************************************************/
NeCsvOut::~NeCsvOut()
{
	Clear();
}



/******************************************************************************
 FUNCTION:  Clear
******************************************************************************/
void NeCsvOut::Clear()
{
	CellListList::Iterator RowIterator( &_Cells );
	while ( RowIterator.Next() )
	{
		CellList* pCellList = RowIterator.GetNodeData();
		pCellList->DeleteAll();
	}

	_Cells.DeleteAll();
}



/******************************************************************************
 FUNCTION:  AddNewRow
******************************************************************************/
void NeCsvOut::AddNewRow()
{
	CellList* pNewList = new CellList;
	_Cells.AddLast( pNewList );
}



/******************************************************************************
 FUNCTION:  AppendIntToLastRow
******************************************************************************/
void NeCsvOut::AppendIntToLastRow( int i )
{
	Cell* pNewCell = new Cell;
	pNewCell->SetInt( i );

	CellList* pLastRow = _Cells.GetLast()->GetData();
	pLastRow->AddLast( pNewCell );
}



/******************************************************************************
 FUNCTION:  AppendUnsignedIntToLastRow
******************************************************************************/
void NeCsvOut::AppendUnsignedIntToLastRow( unsigned int ui )
{
	Cell* pNewCell = new Cell;
	pNewCell->SetUnsignedInt( ui );

	CellList* pLastRow = _Cells.GetLast()->GetData();
	pLastRow->AddLast( pNewCell );
}



/******************************************************************************
 FUNCTION:  AppendLongIntToLastRow
******************************************************************************/
void NeCsvOut::AppendLongIntToLastRow( long int li )
{
	Cell* pNewCell = new Cell;
	pNewCell->SetLongInt( li );

	CellList* pLastRow = _Cells.GetLast()->GetData();
	pLastRow->AddLast( pNewCell );
}



/******************************************************************************
 FUNCTION:  AppendFloatToLastRow
******************************************************************************/
void NeCsvOut::AppendFloatToLastRow( float f )
{
	Cell* pNewCell = new Cell;
	pNewCell->SetFloat( f );

	CellList* pLastRow = _Cells.GetLast()->GetData();
	pLastRow->AddLast( pNewCell );
}



/******************************************************************************
 FUNCTION:  AppendBoolToLastRow
******************************************************************************/
void NeCsvOut::AppendBoolToLastRow( bool b )
{
	Cell* pNewCell = new Cell;
	pNewCell->SetBool( b );

	CellList* pLastRow = _Cells.GetLast()->GetData();
	pLastRow->AddLast( pNewCell );
}



/******************************************************************************
 FUNCTION:  AppendStringToLastRow
******************************************************************************/
void NeCsvOut::AppendStringToLastRow( const char* pacText )
{
	Cell* pNewCell = new Cell;
	pNewCell->SetString( pacText );

	CellList* pLastRow = _Cells.GetLast()->GetData();
	pLastRow->AddLast( pNewCell );
}



/******************************************************************************
 FUNCTION:  WriteToFile
******************************************************************************/
void NeCsvOut::WriteToFile( const char* pacPathAndFilename )
{
	FILE* pFile;
	pFile = fopen( pacPathAndFilename, "w" );
	ASSERTF( pFile, ( "\nUnable to open file to write." ) );

	if ( pFile )
	{
		CellListList::Iterator RowIterator( &_Cells );
		while ( RowIterator.Next() )
		{
			CellList* pCellList = RowIterator.GetNodeData();
			CellList::Iterator CellIter( pCellList );
			while ( CellIter.Next() )
			{
				Cell* pCell = CellIter.GetNodeData();
				pCell->WriteToFile( pFile );

				// If it's not the last item in the row, add a comma
				if ( CellIter.GetNode() != pCellList->GetLast() )
					fputs( ",", pFile );
			}

	
			// if this isn't the last row, add a carriage return
			if ( RowIterator.GetNode() != _Cells.GetLast() )
				fputs( "\n", pFile );
		}

		fclose( pFile );
	}
}








/******************************************************************************
 FUNCTION:  Cell
******************************************************************************/
NeCsvOut::Cell::Cell() :
_eType( TYPE_LAST )
{
	_u._liValue = 0;
}



/******************************************************************************
 FUNCTION:  ~Cell
******************************************************************************/
NeCsvOut::Cell::~Cell()
{
	if ( _eType == TYPE_STRING )
	{
		if ( _u._pacString ) 
			delete[] _u._pacString;
	}
}



/******************************************************************************
 FUNCTION:  SetString
 NOTE:      This is in the cpp so that it's handled by mmgr
            mdomowicz 2011_09_18
******************************************************************************/
void NeCsvOut::Cell::SetString( const char* pacText )
{
	_eType = TYPE_STRING;
	int iSize = strlen( pacText );
	_u._pacString = new char[ iSize + 1 ];
	strncpy( _u._pacString, pacText, iSize );
	_u._pacString[ iSize ] = 0;
}



/******************************************************************************
 FUNCTION:  WriteToFile
******************************************************************************/
void  NeCsvOut::Cell::WriteToFile( FILE* pFile )
{
	if ( _eType == TYPE_STRING )
	{
		fputs( _u._pacString, pFile );
	}
	else
	{
		char acText[ 64 ] = "";
		switch ( _eType )
		{
			case TYPE_INT:
				sprintf( acText, "%i", _u._iValue );
				break;
			case TYPE_UNSIGNED_INT:
				sprintf( acText, "%u", _u._uiValue );
				break;
			case TYPE_LONG_INT:
				sprintf( acText, "%il", _u._liValue );
				break;
			case TYPE_FLOAT:
				sprintf( acText, "%f", _u._fValue );
				break;
			case TYPE_BOOL:
				if ( _u._iValue )
					sprintf( acText, "true" );
				else
					sprintf( acText, "false" );
				break;
			default:
				ASSERTF( 0, ( "\nUnknown data type!" ) );
		}

		fputs( acText, pFile );
	}
}
