/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#include "JpHashTable.h"
#include "JpList.h"


/*
	This is a utility class for writing values to a CSV (Comma Separated Value) text file.

	Simple usage example:

		NeCsvOut CsvOut;
		CsvOut.AddNewRow();
		CsvOut.AppendStringToLastRow( "Hello" );
		CsvOut.AppendStringToLastRow( "Csv File" );

		CsvOut.AddNewRow();
		CsvOut.AppendStringToLastRow( "This" );
		CsvOut.AppendFloatToLastRow( 1.5f );
		CsvOut.AppendIntToLastRow( 4 );
		CsvOut.AppendBoolToLastRow( false );

		CsvOut.WriteToFile( "Save/Test.csv" );

	Outputs:

		Hello,Csv File
		This,1.500000,4,false
*/


#define CellList  JpList< NeCsvOut::Cell > 
#define CellListList JpList< CellList >


class NeCsvOut
{
	public:

		//-----------------------
		// public classes and enums
		class Cell
		{
			public:
				enum TYPE
				{
					TYPE_BOOL,
					TYPE_FLOAT,
					TYPE_INT,
					TYPE_LONG_INT,
					TYPE_STRING,
					TYPE_UNSIGNED_INT,
					TYPE_LAST
				};

				Cell();

				~Cell();

				char* GetString() { return _u._pacString; }
				void SetString( const char* pacText );

				float GetFloat() { return _u._fValue; }
				void SetFloat( float f ) 
				{ 
					_eType = TYPE_FLOAT;
					_u._fValue = f; 
				}

				int GetInt() { return _u._iValue; }
				void SetInt( int i ) 
				{ 
					_eType = TYPE_INT;
					_u._iValue = i; 
				}

				int GetUnsignedInt() { return _u._uiValue; }
				void SetUnsignedInt( int ui ) 
				{ 
					_eType = TYPE_UNSIGNED_INT;
					_u._uiValue = ui; 
				}

				int GetLongInt() { return _u._liValue; }
				void SetLongInt( int li ) 
				{ 
					_eType = TYPE_LONG_INT;
					_u._liValue = li; 
				}

				bool GetBool() { return ( _u._iValue != 0 ); }
				void SetBool( bool b ) 
				{
					_eType = TYPE_BOOL;
					_u._iValue = b ? 1 : 0; 
				}

				void WriteToFile( FILE* pFile );

			private:

				union
				{
					char* _pacString;
					int _iValue;
					unsigned int _uiValue;
					long int _liValue;
					float _fValue;
				} _u;

				TYPE _eType;
		};



		//-----------------------
		// public functions

		NeCsvOut();

		virtual ~NeCsvOut();

		void Clear();

		bool IsClear() { return ( GetFirstRow() == NULL ); }
		CellList* GetFirstRow() { return _Cells.GetFirst()->GetData(); }

		void AddNewRow();

		void AppendIntToLastRow( int i );
		void AppendLongIntToLastRow( long int li );
		void AppendUnsignedIntToLastRow( unsigned int ui );
		void AppendFloatToLastRow( float f );
		void AppendBoolToLastRow( bool b );
		void AppendStringToLastRow( const char* pacText );

		void WriteToFile( const char* pacPathAndFilename );

	private:

		//-----------------------
		// private variables

		CellListList _Cells;
};




