/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpDebug.h"
#include "NeFile.h"
#include "NeUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;

#define NE_FILE_IS_TESTING_ANDROID 0


/******************************************************************************
 FUNCTION:  NeFile
******************************************************************************/
NeFile::NeFile() :
_pFile( NULL ),
_pacFileData( NULL ),
_ulFileDataIndex( 0 ),
_ulFileSize( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~NeFile
******************************************************************************/
NeFile::~NeFile()
{
	#if DCG_IS_TARGET_ANDROID || NE_FILE_IS_TESTING_ANDROID
		if ( _pacFileData )
		{
			ASSERTF( 0, ( "\nNeFile was not closed after use!" ) );
			CloseFile();
		}
	#else
		if ( _pFile )
		{
			ASSERTF( 0, ( "\nNeFile was not closed after use!" ) );
			CloseFile();
		}
	#endif	
}



/******************************************************************************
 FUNCTION:  OpenFile
******************************************************************************/
bool NeFile::OpenFile( const char* pacPathAndFilename, char* pacModeFlags )
{
	#if DCG_IS_TARGET_ANDROID || NE_FILE_IS_TESTING_ANDROID
		_pacFileData = NeUtils::GetFileData( pacPathAndFilename, pacModeFlags, &_ulFileSize );
		return ( _pacFileData != NULL );
	#else
		_pFile = fopen( pacPathAndFilename, pacModeFlags );	
		return ( _pFile != NULL );
	#endif
}



/******************************************************************************
 FUNCTION:  CloseFile
******************************************************************************/
void NeFile::CloseFile()
{
	#if DCG_IS_TARGET_ANDROID || NE_FILE_IS_TESTING_ANDROID
		delete[] _pacFileData;
		_pacFileData = NULL;
		_ulFileSize = 0;
		_ulFileDataIndex = 0;
	#else
		fclose( _pFile );
		_pFile = NULL;
	#endif
}



/******************************************************************************
 FUNCTION:  Seek
 NOTES:     This function is a wrapper for fseek.  The iOriginFlag can have
            the following values:
                SEEK_SET    Beginning of file
                SEEK_CUR    Current position of the file pointer/index
                SEEK_END    End of file
******************************************************************************/
void NeFile::Seek( long int lOffset, int iOriginFlag )
{
	#if DCG_IS_TARGET_ANDROID || NE_FILE_IS_TESTING_ANDROID
		if ( iOriginFlag == SEEK_SET )      // seek from beginning
			_ulFileDataIndex = lOffset;
		else if ( iOriginFlag == SEEK_CUR ) // seek from current position
			_ulFileDataIndex += lOffset;
		else if ( iOriginFlag == SEEK_END ) // seek from the end
			_ulFileDataIndex = ( _ulFileSize - 1 ) + lOffset;

		if ( _ulFileDataIndex < 0 )
			_ulFileDataIndex = 0;
		else if ( _ulFileDataIndex > ( _ulFileSize - 1 ) )
			_ulFileDataIndex = ( _ulFileSize - 1 );
	#else
		fseek( _pFile , lOffset, iOriginFlag );
	#endif
}



/******************************************************************************
 FUNCTION:  GetChar
******************************************************************************/
char NeFile::GetChar()
{
	#if DCG_IS_TARGET_ANDROID || NE_FILE_IS_TESTING_ANDROID
		if ( _ulFileDataIndex == _ulFileSize )
			return (char)EOF;
		else
		{
			char c = (char)_pacFileData[ _ulFileDataIndex ];
			_ulFileDataIndex ++;
			return c;
		}
	#else
		return fgetc( _pFile );
	#endif
}
