/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "fmod.hpp"

#include "JpUtils.h"

#include "NeSoundEffect.h"
#include "NeSoundManager.h"

#include "mmgrChooser.h"



FMOD::System* NeSoundEffect::_pFmodSystem = NULL;



/******************************************************************************
 FUNCTION:  NeSoundEffect
******************************************************************************/
NeSoundEffect::NeSoundEffect() :
_pSound( NULL ),
_pChannel( NULL ),
_iLoopCount( 0 )
{

}



/******************************************************************************
 FUNCTION:  ChannelCallback
******************************************************************************/
FMOD_RESULT F_CALLBACK NeSoundEffect::ChannelCallback
(
	FMOD_CHANNEL *channel, 
	FMOD_CHANNEL_CALLBACKTYPE type, 
	void *commanddata1, 
	void *commanddata2
)
{ 
	return FMOD_OK;
}




/******************************************************************************
 FUNCTION:  Play
******************************************************************************/
void NeSoundEffect::Play()
{
	FMOD_RESULT result;
	result = _pFmodSystem->playSound(FMOD_CHANNEL_FREE, _pSound, false, &_pChannel);
	NeSoundManager::ERRCHECK( result );
	
	result = _pChannel->setLoopCount( _iLoopCount );
	NeSoundManager::ERRCHECK( result );
	
    result = _pChannel->setUserData( (void*)this );
	NeSoundManager::ERRCHECK( result );
	
	result = _pChannel->setCallback( ChannelCallback );
	NeSoundManager::ERRCHECK( result );
}



/******************************************************************************
 FUNCTION:  Pause
******************************************************************************/
void NeSoundEffect::Stop()
{
	if ( _pChannel )
	{
		FMOD_RESULT result;
		result = _pChannel->stop();
		NeSoundManager::ERRCHECK( result );
	}
	
	_pChannel = NULL;
}



/******************************************************************************
 FUNCTION:  Pause
******************************************************************************/
void NeSoundEffect::Pause( bool bPause )
{
	if ( _pChannel )
		_pChannel->setPaused( bPause );
}



/******************************************************************************
 FUNCTION:  IsPaused
******************************************************************************/
bool NeSoundEffect::IsPaused()
{
	bool bIsPaused = false;

	if ( _pChannel )
		_pChannel->getPaused( &bIsPaused );

	return bIsPaused;
}



/******************************************************************************
 FUNCTION:  SetVolume
******************************************************************************/
void NeSoundEffect::SetVolume( float fNormalizedVolume )
{
	fNormalizedVolume = Clamp( fNormalizedVolume, 0.0f, 1.0f );

	if ( _pChannel )
	{	
		FMOD_RESULT result;
		result = _pChannel->setVolume( fNormalizedVolume );
		NeSoundManager::ERRCHECK( result );
	}
}
