/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#include "fmod.hpp"
#include "cocos2d.h"

#include "JpHashTable.h"
#include "NeSoundEffect.h"


USING_NS_CC;



#define SOUND_MANAGER_MAX_SOUND_EFFECTS    128
#define NeSoundEffectTable	JpHashTable< NeSoundEffect, SOUND_MANAGER_MAX_SOUND_EFFECTS * 2 >


class SoHudLog;


class NeSoundManager
{
	public:

		//-----------------------
		// public classes/structs
		struct SoftwareFormatParams
		{
			int _iSampleRate;
			FMOD_SOUND_FORMAT _eSoundFormat;
			int _iNumOutputChannels;
			int _iMaxChannels;
			FMOD_DSP_RESAMPLER _eResampleMethod;
			int _iBits;
		};


		//-----------------------
		// public functions

		NeSoundManager();

		virtual ~NeSoundManager() {};


		virtual void Init();
		virtual void Deinit();
		virtual void Update( float dt );

		virtual void LoadMusicTrack( const char* pacPathAndFileName );
		virtual void UnloadMusicTrack();
		virtual void PlayMusic();
		virtual void StopMusic();
		virtual void PauseMusic( bool bPause );

		virtual NeSoundEffect* LoadSoundEffect( const char* pacPathAndFileName );
		virtual void UnloadSoundEffect( hash hSound );
		virtual void UnloadAllSoundEffects();
		virtual NeSoundEffect* GetSoundEffect( hash hSound ) { return _SoundEffectTable.Find( hSound ); }

		FMOD::Sound* GetMusicTrack() { return _pMusicTrack; }
		FMOD::Channel* GetMusicTrackChannel() { return _pMusicTrackChannel; }
		FMOD::System* GetFmodSystem() { return _pFmodSystem; }

		void GetSoftwareFormat( SoftwareFormatParams* pResults );
		NeSoundEffectTable* GetSoundEffectTable() { return &_SoundEffectTable; }

		static void ERRCHECK( FMOD_RESULT result );


	private:

		//-----------------------
		// private variables

		FMOD::System* _pFmodSystem; 

		FMOD::Sound* _pMusicTrack;
		FMOD::Channel* _pMusicTrackChannel;

		unsigned char* _pucMusicTrackDataForAndroid;

		NeSoundEffectTable _SoundEffectTable;

		//-----------------------
		// private functions
};




