/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#pragma once


#include "stdafx.h"

#include "JpHashTable.h"

USING_NS_CC;


class NeStamp
{
	public:

		//-----------------------
		// public functions
		NeStamp()
		{
			Init();
		}

		~NeStamp()
		{
			Deinit();
		}

		void Init();

		void Deinit();

		void LoadFromImageFile( char acPathAndFileName[] );

		int GetWidth() { return _iWidth; }
		int GetHeight() { return _iHeight; }

		unsigned char* GetData() { return _pacData; }

		bool IsWhiteAtCoordinates( int iX, int iY );
		bool SameColorAtCoordinates( int iX, int iY, const ccColor4F& CompareColor );
		ccColor4F GetColorAtCoordinates( int iX, int iY );

	private:

		//-----------------------
		// private variables

		int _iWidth;
		int _iHeight;
		int _iNumChannels;
		unsigned char* _pacData;
};





#define NeStampTable JpHashTable< NeStamp, 256 >

class NeStampManager
{
	public:

		//-----------------------
		// public functions
		NeStampManager() {}
		~NeStampManager() { RemoveAndDeleteAll(); }

		void LoadStampsFromIniFile( char acPathAndFileName[] );

		void LoadStampsFromCsv( char acPathAndFileName[] );

		NeStamp* LoadStampFromFile
		( 
			char acPathAndFileName[], 
			unsigned int iLookupKey
		);

		NeStamp* FindStamp( unsigned int uiKey ) { return _StampTable.Find( uiKey ); }

		void RemoveStamp( unsigned int uiKey ) { _StampTable.Remove( uiKey ); }

		void RemoveAndDeleteStamp( unsigned int uiKey );

		void RemoveAndDeleteAll();


		static NeStampManager* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();

	private:


		//-----------------------
		// private variables
		NeStampTable _StampTable;

		static NeStampManager* _pInstance;


		//-----------------------
		// private functions

		// Only the manager can store a Stamp!  This is so that
		// Stamps that are not allocated using "new" are not
		// stored here.  Consistent mem allocation method is needed
		// for the "RemoveAndDeleteAll" function to work properly.
		// mdomowicz 2005_01_29
		bool StoreStamp( unsigned int uiKey, NeStamp* pStamp )
		{
			if ( _StampTable.Find( uiKey ) )
				return false;

			return _StampTable.Store( uiKey, pStamp );	
		}

};




