/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#include "stdafx.h"

#include "JpDebug.h"
#include "NeTgaIo.h"

#include "mmgrChooser.h"




/******************************************************************************
 FUNCTION:  TGA_DEVELOPER_DATA
******************************************************************************/
TGA_DEVELOPER_DATA::TGA_DEVELOPER_DATA()
{
	umTag = 0;
	umOffset = 0;
	umSize = 0;
	umData = NULL;
}



/******************************************************************************
 FUNCTION:  ~TGA_DEVELOPER_DATA
******************************************************************************/
TGA_DEVELOPER_DATA::~TGA_DEVELOPER_DATA()
{
	if ( umData != NULL )
	{
		delete[] umData;
		umData = NULL;
	}
};



/******************************************************************************
 FUNCTION:  ~NesTgaIo
******************************************************************************/
NeTgaIo::~NeTgaIo()
{
	/*
	This class was ported over from my codebank.
	Thus it had to be organized a bit to fit this
	app.  One change is that this class is now
	a singleton.  It it left to the user to
	manage the free-ing of the image data.
	Also the ColorMapData and the DeveloperDirData
	are not read in at all anymore, so no need
	to free them either.
	mdomowicz 2005_07_21			

	if ( pabmColorMapData != NULL )
	{
		free( pabmColorMapData );
		pabmColorMapData = NULL;
	}

	if ( pabmImageData != NULL )
	{
		free( pabmImageData );
		pabmImageData = NULL;
	}

	if ( pamDeveloperDirData != NULL )
	{
		free( pamDeveloperDirData );
		pamDeveloperDirData;
	}
	*/
};



/******************************************************************************
 FUNCTION:  ReadLong
 PURPOSE:   reads 4 bytes worth of data
 ARGS:      pFile is the file pointer
 RETURN:    returns a 4 byte integer
 NOTES:     the file pointer is changed ( incremented ) by this function.
			This function does not attempt to restore the file pointer.
******************************************************************************/
_UINT32_ NeTgaIo::ReadLong( FILE* pFile )
{
	_UINT32_	uData;
	fread( &uData, 4, 1, pFile );
	return( uData );
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void NeTgaIo::Init()
{
	bmIsNewTGA = false;
	umExtensionAreaOffset = 0;
	umDeveloperDirOffset = 0;
	umIDLength = 0;

	umColorMapSpecFirstEntryIndex = 0;
	umColorMapSpecMapLength = 0;
	bmColorMapSpecMapEntrySize = 0;

	umImageSpecXOrigin = 0;
	umImageSpecYOrigin = 0;
	umImageSpecImageWidth = 0;
	umImageSpecImageHeight = 0;
	bmImageSpecPixelDepth = 0;
	bmImageSpecImageDescriptor = 0;

	memset( acmImageID, 0, 256 );
	pabmColorMapData = NULL;
	pabmImageData = NULL;

	umDeveloperDirNumTags = 0;
	pamDeveloperDirData = NULL;

	umExtensionSize = 0;
	memset( acmExtensionAuthorName, 0, 41 );
	memset( acmExtensionAuthorCommentsLine1, 0, 81 );
	memset( acmExtensionAuthorCommentsLine2, 0, 81 );
	memset( acmExtensionAuthorCommentsLine3, 0, 81 );
	memset( acmExtensionAuthorCommentsLine4, 0, 81 );
	umExtensionTimeStampMonth = 0;
	umExtensionTimeStampDay = 0;
	umExtensionTimeStampYear = 0;
	umExtensionTimeStampHour = 0;
	umExtensionTimeStampMinute = 0;
	umExtensionTimeStampSecond = 0;
	memset( acmExtensionJobName, 0, 41 );
	umExtensionJobTimeHours = 0;
	umExtensionJobTimeMinutes = 0;
	umExtensionJobTimeSeconds = 0;
	memset( acmExtensionSoftwareID, 0, 41 );
	umExtensionSoftwareVerNumber = 0;
	cmExtensionSoftwareVerLetter = ' ';
	umExtensionTransparentColor = 0;
	umExtensionAspectRatioNumerator = 0;
	umExtensionAspectRatioDenominator = 0;
	umExtensionGammaRatioNumerator = 0;
	umExtensionGammaRatioDenominator = 0;
	umExtensionColorCorrectionOffset = 0;
	umExtensionPostageStampOffset = 0;
	umExtensionScanLineOffset = 0;
	bmExtensionAttributesType = 0;
}





/******************************************************************************
 FUNCTION:  ReadShort
 PURPOSE:   reads 2 bytes worth of data
 ARGS:      pFile is the file pointer
 RETURN:    returns a 2 byte integer
 NOTES:     the file pointer is changed ( incremented ) by this function.
			This function does not attempt to restore the file pointer.
******************************************************************************/
_UINT16_ NeTgaIo::ReadShort( FILE* pFile )
{
	_UINT16_	uData;
	fread( &uData, 2, 1, pFile );
	return( uData );
}



/******************************************************************************
 FUNCTION:  ReadByte
 PURPOSE:   reads 1 byte of data
 ARGS:      pFile is the file pointer
 RETURN:    returns the byte that was read in
 NOTES:     the file pointer is changed ( incremented ) by this function.
			This function does not attempt to restore the file pointer.
******************************************************************************/
_BYTE_ NeTgaIo::ReadByte( FILE* pFile )
{
	_BYTE_	bData;
	fread( &bData, 1, 1, pFile );
	return( bData );
}



/******************************************************************************
 FUNCTION:  Read
 PURPOSE:   does the actual loading of the TGA
 ARGS:      cFileName is the fully qualified name and path of the file to be read
 RETURN:    return TRUE if successful.  FALSE otherwise.
******************************************************************************/
_BOOL_ NeTgaIo::Read( _CHAR_ cFileName[] )
{

	//-----------------------
	// open the file
	//-----------------------
	FILE* pFile;
	pFile = fopen( cFileName, "rb" );
	if ( pFile == NULL )
	{
		if ( IS_TGA_CONSOLE_APP )
			std::cout << "\n\nERROR: Unable to Open file " << cFileName << "\n\n";
		return false;
	}


	//-----------------------
	// read in the file
	//-----------------------
	ReadFileFooter( pFile );
	ReadFileHeader( pFile );
	ReadImageID( pFile );

	// removed this when this file
	// was ported into this app from
	// my codebank.
	// mdomowicz 2005_07_21
	//ReadColorMapData( pFile ); 

	ReadImageData( pFile );

	// Same as the ColorMapData above.
	// This was removed in the port
	// process.
	// mdomowicz 2005_07_21
	//ReadDeveloperArea( pFile );

	ReadExtensionArea( pFile );


	//-----------------------
	// close the file
	//-----------------------
	fclose( pFile );


	//-----------------------
	// some extra formatting
	//-----------------------
	if ( TGA_VERBOSE )
		std::cout << "\n\n\n";


	return true;
}



/******************************************************************************
 FUNCTION:  Read
******************************************************************************/
_BOOL_ NeTgaIo::Read
( 
	_BYTE_** pabImageData, 
	int* piWidth, 
	int* piHeight, 
	int* piBitsPerPixel, 
	_CHAR_ acFileName[] 
)
{
	if ( Read( acFileName ) )
	{
		*pabImageData = pabmImageData;
		pabmImageData = NULL;

		*piWidth = (int)umImageSpecImageWidth;
		*piHeight = (int)umImageSpecImageHeight;

		//-----------------------
		// the crossbar format is always 32 
		// bits, so this is safe.
		//-----------------------
		*piBitsPerPixel = 32; 

		return true;
	}

	ASSERTF( 0, ("\nFailed to read TGA file.") );
	return false;
}



/******************************************************************************
 FUNCTION:  ReadFileFooter
 PURPOSE:   reads the TGA footer
 ARGS:      pFile is the pointer to the file.  note that pFile must already
			have been opened ( via fopen ) sometime before being passed to 
			this function.
 RETURN:    return TRUE if successful.  FALSE otherwise.
******************************************************************************/
_BOOL_ NeTgaIo::ReadFileFooter( FILE* pFile )
{
	//-----------------------
	// since we dont want to alter the file pointer
	// that was passed in, we need to make a copy
	//-----------------------
	FILE* pFileTemp = pFile;


	//-----------------------
	// set the pointer 26 bytes in from the end of the file
	//-----------------------
	_INT_ iSeekResult = fseek( pFileTemp, -26L, SEEK_END);

	if ( iSeekResult > 0 )
	{
		if ( IS_TGA_CONSOLE_APP )
			std::cout << "\n\nERROR: Unable to seek from end of file\n\n";
		return false;
	}

	assert( iSeekResult == 0 );


	//-----------------------
	// assume this is indeed a NEW TGa.  Get some data
	//-----------------------
	umExtensionAreaOffset = ReadLong( pFileTemp );
	umDeveloperDirOffset = ReadLong( pFileTemp );
	fgets( acmSignature, 18, pFileTemp );


	//-----------------------
	// now check for the signature, if it does not
	// exist, this must be an OLD tga, else it's new TGA
	//-----------------------
	if ( strcmp( acmSignature, "TRUEVISION-XFILE." )  != 0 )
	{
		umExtensionAreaOffset = 0;
		umDeveloperDirOffset = 0;
		bmIsNewTGA = false;
	}
	else
	{
		bmIsNewTGA = true;
	}


	//-----------------------
	// output some info
	//-----------------------
	if ( TGA_VERBOSE )
	{
		std::cout << "  FOOTER INFO:";
		std::cout << "\nbmIsNewTGA:  " << bmIsNewTGA;
		std::cout << "\numExtensionAreaOffset:  " << umExtensionAreaOffset;
		std::cout << "\numDeveloperDirOffset:  " << umDeveloperDirOffset;
	}


	//-----------------------
	// return the result
	//-----------------------
	return bmIsNewTGA;
}



/******************************************************************************
 FUNCTION:  ReadFileHeader
 PURPOSE:   reads the TGA header
 ARGS:      pFile is the pointer to the file.  note that pFile must already
            have been opened ( via fopen ) sometime before being passed to 
            this function.
 RETURN:    return TRUE if successful.  FALSE otherwise.
******************************************************************************/
_BOOL_ NeTgaIo::ReadFileHeader( FILE* pFile )
{
	//-----------------------
	// set the file pointer to the beginning of the file
	//-----------------------
	_INT_ iSeekResult = fseek( pFile, 0L, SEEK_SET);

	if ( iSeekResult > 0 )
	{
		std::cout << "\n\nERROR: Unable to seek from beginning of file\n\n";
		return false;
	}

	assert( iSeekResult == 0 );


	//-----------------------
	// get the first three fields of the Header
	//-----------------------
	umIDLength = ReadByte( pFile );
	//#pragma REMIND("remember that when outputting true-color TGA files, this values must be 0")
	bmColorMapType = ReadByte( pFile );
	bmImageType = ReadByte( pFile );


	//-----------------------
	// read the Color Map Specification
	// sub-fields
	//-----------------------
	//#pragma REMIND("These 5 Bytes must always exist in a TGA file, so dont forget to write it")
	umColorMapSpecFirstEntryIndex = ReadShort( pFile );
	umColorMapSpecMapLength = ReadShort( pFile );
	bmColorMapSpecMapEntrySize = ReadByte( pFile );


	//-----------------------
	// read in the Image Specification sub-fields
	//-----------------------
	umImageSpecXOrigin = ReadShort( pFile );
	umImageSpecYOrigin = ReadShort( pFile );
	umImageSpecImageWidth = ReadShort( pFile );
	umImageSpecImageHeight = ReadShort( pFile );
	bmImageSpecPixelDepth = ReadByte( pFile );
	bmImageSpecImageDescriptor = ReadByte( pFile );

	//-----------------------
	// output some info
	//-----------------------
	if ( TGA_VERBOSE )
	{
		std::cout << "\n\n  MISC HEADER INFO:";
		if ( umIDLength > 0 )
			std::cout << "\numIDLength:  " << (int)umIDLength;
		else
			std::cout << "\numIDLength:  No Image ID";

		std::cout << "\nbmColorMapType:  " << (int)bmColorMapType;


		std::cout << "\nbmImageType:  " << (int)bmImageType;
		switch ( bmImageType )
		{
			case 0:
			{
				std::cout << " No Image Data Included";
				break;
			}
			case 1:
			{
				std::cout << " Uncompressed, Color-mapped Image";
				break;
			}
			case 2:
			{
				std::cout << " Uncompressed, True-color Image";
				break;
			}
			case 3:
			{
				std::cout << " Uncompressed, Black-and-white image";
				break;
			}
			case 9:
			{
				std::cout << " Run-length encoded, Color-mapped Image";
				break;
			}
			case 10:
			{
				std::cout << " Run-length encoded, True-color Image";
				break;
			}
			case 11:
			{
				std::cout << " Run-length encoded, Black-and-white image";
				break;
			}
			default:
			{
				std::cout << " unknown image type";
				break;
			}
		}

		std::cout << "\n\n  COLOR MAP SPECIFICATION:";
		std::cout << "\numColorMapSpecFirstEntryIndex:  " << (int)umColorMapSpecFirstEntryIndex;
		std::cout << "\numColorMapSpecMapLength:  " << (int)umColorMapSpecMapLength;
		std::cout << "\nbmColorMapSpecMapEntrySize:  " << (int)bmColorMapSpecMapEntrySize;

		std::cout << "\n\n  IMAGE SPECIFICATION:";
		std::cout << "\numImageSpecXOrigin:  " << (int)umImageSpecXOrigin;
		std::cout << "\numImageSpecYOrigin:  " << (int)umImageSpecYOrigin;
		std::cout << "\numImageSpecImageWidth:  " << (int)umImageSpecImageWidth;
		std::cout << "\numImageSpecImageHeight:  " << (int)umImageSpecImageHeight;
		std::cout << "\nbmImageSpecPixelDepth:  " << (int)bmImageSpecPixelDepth;
		std::cout << "\nbmImageSpecImageDescriptor:  " << (int)bmImageSpecImageDescriptor;
	}

	return true;
}



/******************************************************************************
 FUNCTION:  ReadImageID
 PURPOSE:   reads the image ID, which is an optional field containing
			identifying information about the image
 ARGS:      pFile is the pointer to the file.  note that pFile must already
			have been opened ( via fopen ) sometime before being passed to 
			this function.
 RETURN:    return TRUE if successful.  FALSE otherwise.
******************************************************************************/
_BOOL_ NeTgaIo::ReadImageID( FILE* pFile )
{
	//-----------------------
	// set the file pointer to the Image ID
	//-----------------------
	_INT_ iSeekResult = fseek( pFile, 18L, SEEK_SET);

	if ( iSeekResult > 0 )
	{
		if ( IS_TGA_CONSOLE_APP )
			std::cout << "\n\nERROR: Unable to seek to Image ID\n\n";
		return false;
	}

	assert( iSeekResult == 0 );


	//-----------------------
	// read in the Image ID
	//-----------------------
	memset( acmImageID, 0, 256 );
	if ( umIDLength > 0 )
	{
		fread( acmImageID, 1, umIDLength, pFile );
	}


	//-----------------------
	// output some info
	//-----------------------
	if ( TGA_VERBOSE )
		std::cout << "\nacmImageID:  " << acmImageID;



	return true;
}



/******************************************************************************
 FUNCTION:  ReadColorMapData
 PURPOSE:   reads in the color map
 ARGS:      pFile is the pointer to the file.  note that pFile must already
			have been opened ( via fopen ) sometime before being passed to 
			this function.
 RETURN:    return TRUE if successful.  FALSE otherwise.
******************************************************************************/
_BOOL_ NeTgaIo::ReadColorMapData( FILE* pFile )
{
	ASSERTF( 0, ( "\nNo longer supported.  This code only kept for reference." ) );
	return false;

	#if 0
		//#pragma REMIND("This code is untested, because most files dont have a ColorMap")

		//-----------------------
		// check the easy out case, there is no color map data
		//-----------------------
		if ( bmColorMapType == 0 )
			return true;


		//-----------------------
		// calcaulate where the color map data is
		//-----------------------
		_UINT32_ uColorMapOffset = 18 + umIDLength;


		//-----------------------
		// set the file pointer to the color map data
		//-----------------------
		_INT_ iSeekResult = fseek( pFile, uColorMapOffset, SEEK_SET);

		if ( iSeekResult > 0 )
		{
			if ( IS_TGA_CONSOLE_APP )
				std::cout << "\n\nERROR: Unable to seek to Color Map Data\n\n";
			return false;
		}

		assert( iSeekResult == 0 );


		//-----------------------
		// calculate the size of the color map data
		//-----------------------
		if ( !( bmColorMapSpecMapEntrySize == 8  ||
				bmColorMapSpecMapEntrySize == 16 ||
				bmColorMapSpecMapEntrySize == 24 ||
				bmColorMapSpecMapEntrySize == 32 ) )
		{
			if ( IS_TGA_CONSOLE_APP )
			{
				std::cout << "\n\nERROR: Invalid Number of bits per Color Map Entry (MapEntrySize)";
				std::cout << "\nThe bmColorMapSpecMapEntrySize must be 8, 16, 24, or 32.";
				std::cout << "\nThis restriction is imposed by this TGA I/O implementation,";
				std::cout << "\nand not necessarily the TGA specification";
			}
		}

		assert( bmColorMapSpecMapEntrySize == 8  ||
				bmColorMapSpecMapEntrySize == 16 ||
				bmColorMapSpecMapEntrySize == 24 ||
				bmColorMapSpecMapEntrySize == 32 );


		_UINT32_  uNumColorMapDataBytes = (bmColorMapSpecMapEntrySize / 8) * umColorMapSpecMapLength;


		//-----------------------
		// read in the color map data
		//-----------------------
		if ( pabmColorMapData != NULL )
			delete[] pabmColorMapData;
		pabmColorMapData = new _BYTE_[ uNumColorMapDataBytes ];
		fread( pabmColorMapData, 1, uNumColorMapDataBytes, pFile );


		return true;
	#endif
}



/******************************************************************************
 FUNCTION:  ReadImageData
 PURPOSE:   This reads in the actual image information
 ARGS:      pFile is the pointer to the file.  note that pFile must already
            have been opened ( via fopen ) sometime before being passed to 
            this function.
 RETURN:    return TRUE if successful.  FALSE otherwise.
******************************************************************************/
_BOOL_ NeTgaIo::ReadImageData( FILE* pFile )
{
	//-----------------------
	// calcaulate where the image data is
	//-----------------------
	_UINT32_  uNumColorMapDataBytes = (bmColorMapSpecMapEntrySize / 8) * umColorMapSpecMapLength;
	_UINT32_ uImageDataOffset = 18 + umIDLength + uNumColorMapDataBytes  -0;


	//-----------------------
	// set the file pointer to the image data
	//-----------------------
	_INT_ iSeekResult = fseek( pFile, uImageDataOffset, SEEK_SET);

	if ( iSeekResult > 0 )
	{
		if ( IS_TGA_CONSOLE_APP )
			std::cout << "\n\nERROR: Unable to seek to Image Data\n\n";
		return false;
	}

	assert( iSeekResult == 0 );


	//-----------------------
	// make sure we are reading in an acceptable file
	//-----------------------
	if ( !( bmImageSpecPixelDepth == 8  ||
			bmImageSpecPixelDepth == 16 ||
			bmImageSpecPixelDepth == 24 ||
			bmImageSpecPixelDepth == 32 ) )
	{
		if ( IS_TGA_CONSOLE_APP )
		{
			std::cout << "\n\nERROR: Invalid Number of bits per pixel (ImageSpecPixelDepth)";
			std::cout << "\nThe bmImageSpecPixelDepth must be 8, 16, 24, or 32.";
			std::cout << "\nThis restriction is imposed by this TGA I/O implementation,";
			std::cout << "\nand not necessarily the TGA specification";
		}
	}

	assert( bmImageSpecPixelDepth == 8  ||
			bmImageSpecPixelDepth == 16 ||
			bmImageSpecPixelDepth == 24 ||
			bmImageSpecPixelDepth == 32 );


	//-----------------------
	// allocate space for our "cross-bar" format
	//-----------------------
	assert( pabmImageData == NULL );
	if ( pabmImageData != NULL )
		delete[] pabmImageData;

	_UINT32_ uNumIFormatDataBytes = umImageSpecImageWidth * umImageSpecImageHeight * 4;
	pabmImageData = new _BYTE_[ uNumIFormatDataBytes ];


	//-----------------------
	// error if pixel depth is unsupported
	//-----------------------
	if ( !(bmImageSpecPixelDepth == 32 || bmImageSpecPixelDepth == 24) )
	{
		if ( IS_TGA_CONSOLE_APP )
			std::cout << "\n\nERROR: currently only reads 24 and 32 bit images\n\n";
		return false;
	}

	assert( bmImageSpecPixelDepth == 32 || bmImageSpecPixelDepth == 24 );


	//-----------------------
	// error if image type unsupported
	//-----------------------
	ASSERTF( bmImageType == 2, ( "\nWrong image type.  Could this image be RLE compressed?  Make sure it isn't" ) );
	if ( !(bmImageType == 2 && bmColorMapType == 0) )
	{
		if ( IS_TGA_CONSOLE_APP )
			std::cout << "\n\nERROR: bmImageType must be 2, bmColorMapType must be 0 \n\n";
		return false;
	}

	assert( bmImageSpecPixelDepth == 32 || bmImageSpecPixelDepth == 24 );


	//-----------------------
	// find the orientation of the file   
	//-----------------------
	int iOrientation = (bmImageSpecImageDescriptor & 48);
	iOrientation = iOrientation >> 4;


	//-----------------------
	// read in the image data
	//-----------------------
	if ( bmImageType == 2 && bmColorMapType == 0 )
	{
		switch ( iOrientation )
		{
			case 0:     // Bottom left
			{
				for( int y = umImageSpecImageHeight-1; y >= 0; y -- )
				{
					for ( int x = 0; x < (umImageSpecImageWidth)*4; x += 4 )
					{
						pabmImageData[(y*umImageSpecImageWidth*4) + x + 2] = ReadByte( pFile ); // Read in the Blue Value
						pabmImageData[(y*umImageSpecImageWidth*4) + x + 1] = ReadByte( pFile ); // Read in the Green Value
						pabmImageData[(y*umImageSpecImageWidth*4) + x    ] = ReadByte( pFile ); // Read in the Red Value

						if ( bmImageSpecPixelDepth == 32 )
							pabmImageData[(y*umImageSpecImageWidth*4) + x + 3] = ReadByte( pFile ); // Read in the Alpha Value
						else
							pabmImageData[(y*umImageSpecImageWidth*4) + x + 3] = 255;

					}
				}

				break;
			}
			case 1:     // Bottom right
			{
				for( int y = umImageSpecImageHeight-1; y >= 0; y -- )
				{
					for ( int x = (umImageSpecImageWidth)*4 - 4; x >= 0; x -= 4 )
					{
						pabmImageData[(y*umImageSpecImageWidth*4) + x + 2] = ReadByte( pFile ); // Read in the Blue Value
						pabmImageData[(y*umImageSpecImageWidth*4) + x + 1] = ReadByte( pFile ); // Read in the Green Value
						pabmImageData[(y*umImageSpecImageWidth*4) + x    ] = ReadByte( pFile ); // Read in the Red Value

						if ( bmImageSpecPixelDepth == 32 )
							pabmImageData[(y*umImageSpecImageWidth*4) + x + 3] = ReadByte( pFile ); // Read in the Alpha Value
						else
							pabmImageData[(y*umImageSpecImageWidth*4) + x + 3] = 255;

					}
				}

				break;
			}
			case 2:     // Top left
			{
				for( int y = 0; y < umImageSpecImageHeight; y ++ )
				{
					for ( int x = 0; x < (umImageSpecImageWidth)*4; x += 4 )
					{
						pabmImageData[(y*umImageSpecImageWidth*4) + x + 2] = ReadByte( pFile ); // Read in the Blue Value
						pabmImageData[(y*umImageSpecImageWidth*4) + x + 1] = ReadByte( pFile ); // Read in the Green Value
						pabmImageData[(y*umImageSpecImageWidth*4) + x    ] = ReadByte( pFile ); // Read in the Red Value

						if ( bmImageSpecPixelDepth == 32 )
							pabmImageData[(y*umImageSpecImageWidth*4) + x + 3] = ReadByte( pFile ); // Read in the Alpha Value
						else
							pabmImageData[(y*umImageSpecImageWidth*4) + x + 3] = 255;

					}
				}

				break;
			}
			case 3:     // Top Right
			{
				for( int y = 0; y < umImageSpecImageHeight; y ++ )
				{
					for ( int x = (umImageSpecImageWidth)*4 - 4; x >= 0; x -= 4 )
					{
						pabmImageData[(y*umImageSpecImageWidth*4) + x + 2] = ReadByte( pFile ); // Read in the Blue Value
						pabmImageData[(y*umImageSpecImageWidth*4) + x + 1] = ReadByte( pFile ); // Read in the Green Value
						pabmImageData[(y*umImageSpecImageWidth*4) + x    ] = ReadByte( pFile ); // Read in the Red Value

						if ( bmImageSpecPixelDepth == 32 )
							pabmImageData[(y*umImageSpecImageWidth*4) + x + 3] = ReadByte( pFile ); // Read in the Alpha Value
						else
							pabmImageData[(y*umImageSpecImageWidth*4) + x + 3] = 255;

					}
				}

				break;
			}
		}
	}

	return true;
}



/******************************************************************************
 FUNCTION:  ReadDeveloperArea
 PURPOSE:   read in developer specific information.  see TGA spec for details.
 ARGS:      pFile is the pointer to the file.  note that pFile must already
			have been opened ( via fopen ) sometime before being passed to 
			this function.
 RETURN:    return TRUE if successful.  FALSE otherwise.
******************************************************************************/
_BOOL_ NeTgaIo::ReadDeveloperArea( FILE* pFile )
{
	ASSERTF( 0, ( "\nNo longer supported.  This code only kept for reference." ) );
	return false;

	#if 0
		//#pragma REMIND("This code is untested, because most files dont have Developer Data")


		//-----------------------
		// check the easy out case, there is no color Developer Area
		//-----------------------
		if ( umDeveloperDirOffset == 0 )
			return true;


		//-----------------------
		// set the pointer to the start of the developer directory
		//-----------------------
		_INT_ iSeekResult = fseek( pFile, umDeveloperDirOffset, SEEK_SET);

		if ( iSeekResult > 0 )
		{
			if ( IS_TGA_CONSOLE_APP )
				std::cout << "\n\nERROR: Unable to seek from end of file\n\n";
			return false;
		}

		assert( iSeekResult == 0 );


		//-----------------------
		// allocate space for all the developer 
		// directory dir fields
		//-----------------------
		umDeveloperDirNumTags = ReadShort( pFile );

		if ( pamDeveloperDirData != NULL )
			delete[] pamDeveloperDirData;
		pamDeveloperDirData = new TGA_DEVELOPER_DATA[ umDeveloperDirNumTags ];


		//-----------------------
		// read in the Tag, Offset, and Size
		// of each Developer Directory Entry
		//-----------------------
		for ( int i = 0; i < umDeveloperDirNumTags; i++ )
		{
			pamDeveloperDirData[i].umTag = ReadShort( pFile );
			pamDeveloperDirData[i].umOffset = ReadLong( pFile );
			pamDeveloperDirData[i].umSize = ReadLong( pFile );
		}


		//-----------------------
		// now read in all the data
		//-----------------------
		//#pragma REMIND("This needs some error checking/handling")
		for ( int j = 0; j < umDeveloperDirNumTags; j++ )
		{
			// make the file pointer point to the beginning of the data
			_INT_ iSeekResult = fseek( pFile, pamDeveloperDirData[j].umOffset, SEEK_SET);

			// allocate some space for the data
			if ( pamDeveloperDirData[j].umData != NULL )
				delete[] pamDeveloperDirData[j].umData;

			pamDeveloperDirData[j].umData = new _BYTE_[ pamDeveloperDirData[j].umSize * sizeof( _BYTE_ ) ];

			// read in the data
			fread( pamDeveloperDirData[j].umData, 1, pamDeveloperDirData[j].umSize, pFile );
		}


		return true;
	#endif
}



/******************************************************************************
 FUNCTION:  ReadExtensionArea
 PURPOSE:   reads the extension area which contains even more image stats
 ARGS:      pFile is the pointer to the file.  note that pFile must already
			have been opened ( via fopen ) sometime before being passed to 
			this function.
 RETURN:    return TRUE if successful.  FALSE otherwise.
******************************************************************************/
_BOOL_ NeTgaIo::ReadExtensionArea( FILE* pFile )
{
	//-----------------------
	// check the easy out case, there is no Extension Area
	//-----------------------
	if ( umExtensionAreaOffset == 0 )
		return true;


	//-----------------------
	// set the pointer to the start of the Extension Area
	//-----------------------
	_INT_ iSeekResult = fseek( pFile, umExtensionAreaOffset, SEEK_SET);

	if ( iSeekResult > 0 )
	{
		if ( IS_TGA_CONSOLE_APP )
			std::cout << "\n\nERROR: Unable to seek from end of file\n\n";
		return false;
	}

	assert( iSeekResult == 0 );


	//-----------------------
	// read in each individual field
	//-----------------------
	umExtensionSize = ReadShort( pFile );
	fread( acmExtensionAuthorName, 1, 41, pFile );
	fread( acmExtensionAuthorCommentsLine1, 1, 81, pFile );
	fread( acmExtensionAuthorCommentsLine2, 1, 81, pFile );
	fread( acmExtensionAuthorCommentsLine3, 1, 81, pFile );
	fread( acmExtensionAuthorCommentsLine4, 1, 81, pFile );
	umExtensionTimeStampMonth = ReadShort( pFile );
	umExtensionTimeStampDay = ReadShort( pFile );
	umExtensionTimeStampYear = ReadShort( pFile );
	umExtensionTimeStampHour = ReadShort( pFile );
	umExtensionTimeStampMinute = ReadShort( pFile );
	umExtensionTimeStampSecond = ReadShort( pFile );
	fread( acmExtensionJobName, 1, 41, pFile );
	umExtensionJobTimeHours = ReadShort( pFile );
	umExtensionJobTimeMinutes = ReadShort( pFile );
	umExtensionJobTimeSeconds = ReadShort( pFile );
	fread( acmExtensionSoftwareID, 1, 41, pFile );
	umExtensionSoftwareVerNumber = ReadShort( pFile );
	cmExtensionSoftwareVerLetter = ReadByte( pFile );
	umExtensionTransparentColor = ReadLong( pFile );
	umExtensionAspectRatioNumerator = ReadShort( pFile );
	umExtensionAspectRatioDenominator = ReadShort( pFile );
	umExtensionGammaRatioNumerator = ReadShort( pFile );
	umExtensionGammaRatioDenominator = ReadShort( pFile );

	umExtensionColorCorrectionOffset = ReadLong( pFile );
	umExtensionPostageStampOffset = ReadLong( pFile );
	umExtensionScanLineOffset = ReadLong( pFile );
	bmExtensionAttributesType = ReadByte( pFile );


	//-----------------------
	// read in Scan line table, postage stamp image, 
	// and Color Correction Table
	//-----------------------

	// Note: the reading of the scan line table, postage
	// stamp image and Color Correction table has
	// not yet been implemented.


	//-----------------------
	// output some info
	//-----------------------
	if ( TGA_VERBOSE )
	{
		std::cout << "\n\n  EXTENSION AREA:";
		std::cout << "\numExtensionSize:  " << (int)umExtensionSize;
		std::cout << "\nacmExtensionAuthorName:  " << acmExtensionAuthorName;
		std::cout << "\nacmExtensionAuthorCommentsLine1:  " << acmExtensionAuthorCommentsLine1;
		std::cout << "\nacmExtensionAuthorCommentsLine2:  " << acmExtensionAuthorCommentsLine2;
		std::cout << "\nacmExtensionAuthorCommentsLine3:  " << acmExtensionAuthorCommentsLine3;
		std::cout << "\nacmExtensionAuthorCommentsLine4:  " << acmExtensionAuthorCommentsLine4;
		std::cout << "\numExtensionTimeStampMonth:  " << (int)umExtensionTimeStampMonth;
		std::cout << "\numExtensionTimeStampDay:  " << (int)umExtensionTimeStampDay;
		std::cout << "\numExtensionTimeStampYear:  " << (int)umExtensionTimeStampYear;
		std::cout << "\numExtensionTimeStampHour:  " << (int)umExtensionTimeStampHour;
		std::cout << "\numExtensionTimeStampMinute:  " << (int)umExtensionTimeStampMinute;
		std::cout << "\numExtensionTimeStampSecond:  " << (int)umExtensionTimeStampSecond;
		std::cout << "\nacmExtensionJobName:  " << acmExtensionJobName;
		std::cout << "\numExtensionJobTimeHours:  " << (int)umExtensionJobTimeHours;
		std::cout << "\numExtensionJobTimeMinutes:  " << (int)umExtensionJobTimeMinutes;
		std::cout << "\numExtensionJobTimeSeconds:  " << (int)umExtensionJobTimeSeconds;
		std::cout << "\nacmExtensionSoftwareID:  " << acmExtensionSoftwareID;
		std::cout << "\numExtensionSoftwareVerNumber:  " << (int)umExtensionSoftwareVerNumber;
		std::cout << "\ncmExtensionSoftwareVerLetter:  " << cmExtensionSoftwareVerLetter;
		std::cout << "\numExtensionTransparentColor:  " << (int)umExtensionTransparentColor;
		std::cout << "\numExtensionAspectRatioNumerator:  " << (int)umExtensionAspectRatioNumerator;
		std::cout << "\numExtensionAspectRatioDenominator:  " << (int)umExtensionAspectRatioDenominator;
		std::cout << "\numExtensionGammaRatioNumerator:  " << (int)umExtensionGammaRatioNumerator;
		std::cout << "\numExtensionGammaRatioDenominator:  " << (int)umExtensionGammaRatioDenominator;
		std::cout << "\numExtensionColorCorrectionOffset:  " << (int)umExtensionColorCorrectionOffset;
		std::cout << "\numExtensionPostageStampOffset:  " << (int)umExtensionPostageStampOffset;
		std::cout << "\numExtensionScanLineOffset:  " << (int)umExtensionScanLineOffset;
		std::cout << "\nbmExtensionAttributesType:  " << (int)bmExtensionAttributesType;

	}


	return true;
}

