/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#if DCG_IS_TARGET_ANDROID
	#include <android/log.h>
#endif

#include "JpDebug.h"
#include "NeUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  NeUtils
******************************************************************************/
NeUtils::NeUtils()
{
}



/******************************************************************************
 FUNCTION:  ~NeUtils
******************************************************************************/
NeUtils::~NeUtils()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void NeUtils::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void NeUtils::Deinit()
{
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void NeUtils::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  GetFileData
 NOTES:     This function is mostly a copy of the getFileData in 
            \platform\win32\CCFileUtils.
            It is ported here so the memory allocations can be tracked by mmgr.h
            You should always use this version of GetFileData, and never use
            the one in CCFileUtils.
            mdomowicz 2013_10_31
******************************************************************************/
unsigned char* NeUtils::GetFileData(const char* pszFileName, const char* pszMode, unsigned long* pSize)
{
	#if DCG_IS_TARGET_WIN32

		unsigned char* pBuffer = NULL;
		CCAssert(pszFileName != NULL && pSize != NULL && pszMode != NULL, "Invaild parameters.");
		*pSize = 0;
		do 
		{
			// read the file from hardware
			FILE *fp = fopen(pszFileName, pszMode);
			CC_BREAK_IF(!fp);

			fseek(fp,0,SEEK_END);
			*pSize = ftell(fp);
			fseek(fp,0,SEEK_SET);
			pBuffer = new unsigned char[*pSize];
			*pSize = fread(pBuffer,sizeof(unsigned char), *pSize,fp);
			fclose(fp);
		} while (0);

		CCFileUtils* pFileUtils = CCFileUtils::sharedFileUtils();
		if (! pBuffer && pFileUtils->isPopupNotify())
		{
			std::string title = "Notification";
			std::string msg = "Get data from file(";
			msg.append(pszFileName).append(") failed!");

			CCMessageBox(msg.c_str(), title.c_str());
		}
		return pBuffer;

	#else

		// Since mmgr is only used on Win32 builds, we can just route
		// to the standard getFileData for all other platforms.
		// mdomowicz 2013_10_31

		CCFileUtils* pFileUtils = CCFileUtils::sharedFileUtils();
		return pFileUtils->getFileData( pszFileName, pszMode, pSize );

	#endif
}



/******************************************************************************
 FUNCTION:  DoesFileExist
******************************************************************************/
bool NeUtils::DoesFileExist( const char* pszFileName )
{
	#if DCG_IS_TARGET_ANDROID

		std::string fullPath(pszFileName);
		bool bFileExists = false;

		FILE *fp = fopen(pszFileName, "r");

		if ( fp )
		{
			bFileExists = true;
			fclose(fp);
		}

		return bFileExists;

	#else
		ASSERTF( 0, ( "\nThis function not yet implemented for the current platform." ) );
		return false;
	#endif
}



/******************************************************************************
 FUNCTION:  DcgLog
 NOTES:     CCLOG seems to only work on android builds if you set:
                 LOCAL_CPPFLAGS := -DCOCOS2D_DEBUG=1
            in the Android.mk.  This is a pain to set/unset since it
            triggers a full project recompile.  This function, however
            will work at all times.
******************************************************************************/
void NeUtils::DcgLog( const char* pFormat, ... )
{
	if ( strlen( pFormat ) < 2048 )
	{
		char buffer[2048];
		va_list args;
		va_start (args, pFormat);
		vsprintf (buffer,pFormat, args);
		va_end (args);


		#if DCG_IS_TARGET_ANDROID
			__android_log_write( ANDROID_LOG_INFO, "DcgApp", buffer );
		#else
			CCLOG( buffer );
		#endif
	}
}












