/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include <float.h>

#include "JpHashTable.h"
#include "JpList.h"
#include "NeCompass.h"
#include "SoActorTypes.h"
#include "SoSprite.h"
#include "SoSpriteAnimSet.h"
#include "SoUtils.h"


USING_NS_CC;


#define SoActorList JpList< SoActor >


class SoActorVisible;
class SoActorVisibleBoardSpot;
class SoActorVisibleHotel;
class SoActorVisiblePlayer;
class SoActorVisibleTile;
class SoActorVisibleWorker;
class SoLabelBmFont;
class SoSaveSystem;


class SoActor
{
	public:

		//-----------------------
		// public functions
		SoActor();
		virtual ~SoActor() {}

		virtual void Init( CCNode* pParentNode ); 
		virtual void Deinit();

		virtual void Update( float dt );

		virtual void OnDeath();
		virtual SoActorTypes::TYPE GetType() { return SoActorTypes::TYPE_ACTOR; }

		virtual hash GetCsvRowIndex() { return 0; }
		virtual const char* GetDisplayName() { return NULL; }

		virtual SoActorVisible* GetAsVisible() { return NULL; }
		virtual SoActorVisibleBoardSpot* GetAsVisibleBoardSpot() { return NULL; }
		virtual SoActorVisibleHotel* GetAsVisibleHotel() { return NULL; }
		virtual SoActorVisiblePlayer* GetAsVisiblePlayer() { return NULL; }
		virtual SoActorVisibleTile* GetAsVisibleTile() { return NULL; }


		// actor management
		void AddToActorManagerByPriority();
		unsigned int GetGuid() { return _gGuid; }
		void GenerateAndSetGuid();
		guid GenerateGuid();


		// world saving and loading
		virtual void SaveToFile( SoSaveSystem* pSaveSystem );
		virtual void LoadFromFile( SoSaveSystem* pSaveSystem );
		virtual void DefineCsvColumns( SoSaveSystem* pSaveSystem );


		// CCNode overrides
		virtual void draw(void);
		virtual bool init() { return true; }


		// utility / misc
		virtual bool UseResampledTexture() { return false; }
		SoSprite* CreateSprite( CCNode* pParent, const char* pacPathAndFilename );
		SoLabelBmFont* CreateBmFontLabel( CCNode* pParent, const std::string& sText, const std::string& sFontName, cocos2d::CCTextAlignment Alignment );
		static void RemoveAndDestroyNode( CCNode* pParent, CCNode* pNodeToDestroy );

		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }



		virtual void ResetAnimSet() {}
		virtual void SetAnimSet( SoSpriteAnimSet* p ) {}
		virtual SoSpriteAnimSet* GetAnimSet() { return NULL; }


	protected:

		//-----------------------
		// protected variables
		
		CCNode* _pParentNode;


	private:

		//-----------------------
		// private variables
		guid _gGuid;
};


