/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#pragma once


#include "stdafx.h"

#include "JpHashTable.h"
#include "JpList.h"
#include "SoActor.h"


#define MAX_ACTORS    4096

class SoSaveSystem;

class SoActorManager
{
	public:

		//-----------------------
		// function declarations
		SoActorManager();

		virtual ~SoActorManager() {}


		void Init() {}

		void Deinit() {};

		void Update( float dt );

		JpList< SoActor >* GetActorList() { return &_ActorList; }

		void AddActor( SoActor* pActor, int iUpdatePriority )
		{
			_ActorList.AddUniquely( pActor, iUpdatePriority );
			_ActorGuidTable.Store( pActor->GetGuid(), pActor );
		}

		bool Exists( unsigned int uiGuid ) { return ( GetActorByGuid( uiGuid ) != NULL ); }

		void RemoveAndDeleteActor( SoActor* pActor );
		
		void RemoveAndDeleteAllActors();

		void RemoveAndDeleteAllActorsNow();

		void DeleteAllActorsInPendingDeletionList();

		int AppendFilterdActors
		(
			JpList< SoActor >* pActorList, 
			SoActorTypes::TYPE eActorType 
		);

		int GetCountByType( SoActorTypes::TYPE eActorType );

		SoActor* GetActorByGuid( unsigned int uiGuid );

		guid GenerateGuid();


		// Cocos2d specific 
		// This parent node should be used as the parent of all actors.
		// Not sure if this is the best way to organize parent/child nodes for actors, but this should be fine for now.
		// mdomowicz 2012_18_12
		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }


		// In some situations, you want to clear out most actors from the
		// actor manager, but not all of them.  These functions make that
		// possible.
		void RemoveAndDeleteUnpreservedActorsNow();
		void SetActorPreservedStatus( SoActor* pActor, bool bPreserve );
		bool IsActorPreserved( SoActor* pActor ) { return ( _PreservedActorList.Find( pActor ) != NULL ); }


		void SaveToFile( SoSaveSystem* pSaveSystem );
		void LoadFromFile( SoSaveSystem* pSaveSystem );
		void LoadHeroFromFile( SoSaveSystem* pSaveSystem );


		static SoActorManager* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();


	private:

		//-----------------------
		// private variables
		JpList< SoActor > _ActorList;
		JpList< SoActor > _PendingDeletionList;

		unsigned int _uiCachedPlayerActorNameHash;
		SoActor* _pCachedPlayerActor;


		JpHashTable< SoActor, MAX_ACTORS * 2 > _ActorGuidTable;


		JpList< SoActor > _PreservedActorList;

		CCNode* _pParentNode;

		static SoActorManager* _pInstance;
};

