/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"
#include <stdio.h>


#include "JpUtils.h"

#include "SoActorSorter.h"
#include "SoActorVisible.h"
#include "SoGameBit.h"
#include "SoGameBitPlayerHuman.h"

#include "mmgrChooser.h"


SoActorSorter* SoActorSorter::_pInstance = NULL;


int SoActorSorter::_aiExistingActorTallies[ SoActorTypes::TYPE_LAST ];

SoActorList SoActorSorter::_HudMapActorList;
SoActorList SoActorSorter::_MovableActorsList;



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoActorSorter::Init()
{
}




/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoActorSorter::Deinit()
{
	ClearAllLists();
	ClearActorCounts();
}




/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoActorSorter::Update( float dt )
{
	ClearAllLists();
	ClearActorCounts();

	JpList< SoActor >::Iterator Iter( SoActorManager::GetInstance()->GetActorList() );
	while ( Iter.Next() )
	{
		SoActor* pActor = Iter.GetNodeData();
		SoActorTypes::TYPE eType = pActor->GetType();

		switch ( eType )
		{
			case SoActorTypes::TYPE_LAST:
			default:
				break;
		}


		SoActorVisible* pActorVisble = pActor->GetAsVisible();
		if ( pActorVisble && pActorVisble->IsUserMovable() )
		{
			_MovableActorsList.AddLast( pActorVisble );
		}


		IncrementExistingActorsByType( eType );
	}
}



/******************************************************************************
 FUNCTION:  ClearAllLists
******************************************************************************/
void SoActorSorter::ClearAllLists()
{
	_HudMapActorList.RemoveAll();
	_MovableActorsList.RemoveAll();
}




/******************************************************************************
 FUNCTION:  GetNumExistingActorsByType
******************************************************************************/
int SoActorSorter::GetNumExistingActorsByType( SoActorTypes::TYPE eType )
{
	return _aiExistingActorTallies[ eType ];
}




/******************************************************************************
 FUNCTION:  IncrementExistingActorsByType
******************************************************************************/
void SoActorSorter::IncrementExistingActorsByType( SoActorTypes::TYPE eType )
{
	_aiExistingActorTallies[ eType ] ++;
}




/******************************************************************************
 FUNCTION:  ClearActorCounts
******************************************************************************/
void SoActorSorter::ClearActorCounts()
{
	for ( int i = 0; i < (int)SoActorTypes::TYPE_LAST; i ++ )
	{
		_aiExistingActorTallies[ i ] = 0;
	}
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoActorSorter::CreateInstance()
{
	ASSERTF( _pInstance == NULL, ( "\nCan't create more than one instance.  Not allowed." ) );

	if ( !_pInstance )
		_pInstance = new SoActorSorter;
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoActorSorter::DeleteInstance()
{
	if ( _pInstance )
	{
		delete _pInstance;
		_pInstance = NULL;
	}
}
