/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"

#include "SoActorManager.h"
#include "SoActorVisible.h"

#include "mmgrChooser.h"


int SoActorVisible::_iVisibleActorCounter = 0;



/******************************************************************************
 FUNCTION:  SoActorVisible
******************************************************************************/
SoActorVisible::SoActorVisible() :
_pGame( NULL ),
_bIsUserMovable( false ),
_vUserMoveOffset( 0.0f, 0.0f ),
_fUserMoveMaxDist( 0.0f ),
_bUserIsTouching( false ),
_pSprite( NULL ),
_vWorldPosition( 0.0f, 0.0f ),
_vOrganizeOffset( 0.0f, 0.0f ),
_iRenderOrder( -1 ),
_iVisibleActorNumber( -1 )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoActorVisible::Init( CCNode* pParentNode )
{
	SoActor::Init( pParentNode );

	_iVisibleActorNumber = _iVisibleActorCounter;
	_iVisibleActorCounter ++;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoActorVisible::Deinit()
{	
	SoActor::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoActorVisible::Update( float dt )
{
	SoActor::Update( dt );
	SetOpacity( 1.0f );	
}



/******************************************************************************
 FUNCTION:  OnInput
******************************************************************************/
void SoActorVisible::OnInput( SoInputDispatcher::InputPacket* pInput )
{
	if ( pInput->_pSoCcTouchesDownList )
	{
		_bUserIsTouching = true;
	}
	else if ( pInput->_pSoCcTouchesMoveList )
	{
		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pInput->_pSoCcTouchesMoveList;
		
		if ( pNonConstTouchList->GetNumEntries() > 0 )
		{
			CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
			JpVector2f vP0 = SoUtils::ToJpVector2f( pTouch0->locationInView() );
			vP0.Y *= -1.0f;

			CCTouch* pTouch1 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
			JpVector2f vP1 = SoUtils::ToJpVector2f( pTouch1->previousLocationInView() );
			vP1.Y *= -1.0f;

			JpVector2f vDelta = ( vP0 - vP1 ) * ( 1.0f / SoUtils::GetSpriteZoom() );
			vDelta.X *= ( 1.0f / SoUtils::GetiPadRatioX() );
			vDelta.Y *= ( 1.0f / SoUtils::GetiPadRatioY() );


			_vUserMoveOffset += vDelta;
		}
	}
	else if ( pInput->_pSoCcTouchesUpList )
	{
		_bUserIsTouching = false;

		if ( _vUserMoveOffset.Length() > _fUserMoveMaxDist )
		{
			// Set the current position to where the user dropped the actor.
			// Thus when the visible actor updates, it interpolates from this
			// position back to the target position.
			_vWorldPosition += _vUserMoveOffset;

			// Clear the offset, so that it no longer can affect the final positioning.
			_vUserMoveOffset.Zero();
		}
	}
}



/******************************************************************************
 FUNCTION:  IsPositionInside
******************************************************************************/
bool SoActorVisible::IsPositionInside( const JpVector2f& vWorldPos )
{
	return IsPositionInsideSprite( vWorldPos, _pSprite );
}



/******************************************************************************
 FUNCTION:  IsPositionInsideSprite
******************************************************************************/
bool SoActorVisible::IsPositionInsideSprite( const JpVector2f& vWorldPos, SoSprite* pSprite )
{
	if ( pSprite )
	{
		float fHalfWidth = ( pSprite->getContentSize().width / 2 );
		float fHalfHeight = ( pSprite->getContentSize().height / 2 );

		JpVector2f vCenter = _vWorldPosition + _vUserMoveOffset + _vOrganizeOffset;

		return 
		( 
			vWorldPos.X > ( vCenter.X - fHalfWidth ) &&
			vWorldPos.X < ( vCenter.X + fHalfWidth ) &&
			vWorldPos.Y > ( vCenter.Y - fHalfHeight ) &&
			vWorldPos.Y < ( vCenter.Y + fHalfHeight )
		);
	}

	return false;
}



/******************************************************************************
 FUNCTION:  RemoveSprite
******************************************************************************/
void SoActorVisible::RemoveSprite()
{
	if ( _pSprite )
	{
		RemoveAndDestroyNode( _pParentNode, _pSprite );
		_pSprite = NULL;
	}
}



/******************************************************************************
 FUNCTION: GetWorldPosition 
******************************************************************************/
JpVector2f SoActorVisible::GetWorldPosition()
{ 
	return _vWorldPosition;
}



/******************************************************************************
 FUNCTION:  SetWorldPosition
******************************************************************************/
void SoActorVisible::SetWorldPosition( const JpVector2f& v )
{ 
	_vWorldPosition = v;
}



/******************************************************************************
 FUNCTION:  SetScale
******************************************************************************/
void SoActorVisible::SetScale( const JpVector2f& vScale )
{
	if ( _pSprite )
	{
		_pSprite->setScaleX( vScale.X * SoUtils::GetiPadRatioX() );
		_pSprite->setScaleY( vScale.Y * SoUtils::GetiPadRatioY() );
	}
}



/******************************************************************************
 FUNCTION:  GetScale
******************************************************************************/
JpVector2f SoActorVisible::GetScale()
{
	ASSERTF( 0, ( "\nSoActor::GetScale not yet implemented" ) );
	return JpVector2f( 0.0f, 0.0f );
}



/******************************************************************************
 FUNCTION:  GetOpacity
******************************************************************************/
float SoActorVisible::GetOpacity()
{
	if ( _pSprite )
		return ( _pSprite->getOpacity() / 255.0f );

	return 0.0f;
}



/******************************************************************************
 FUNCTION:  SetOpacity
******************************************************************************/
void SoActorVisible::SetOpacity( float fPercent )
{
	if ( _pSprite )
		_pSprite->setOpacity( fPercent * 255.0f );
}



/******************************************************************************
 FUNCTION:  GetColor
******************************************************************************/
ccColor3B SoActorVisible::GetColor()
{
	if ( _pSprite )
		return _pSprite->getColor();

	return JpUtils::_Black;
}



/******************************************************************************
 FUNCTION:  SetColor
******************************************************************************/
void SoActorVisible::SetColor( const ccColor3B& Color )
{
	if ( _pSprite )
		_pSprite->setColor( Color );
}



/******************************************************************************
 FUNCTION:  GetZorder
******************************************************************************/
int SoActorVisible::GetZorder()
{
	SoSprite* pSprite = GetSprite();

	if ( pSprite )
		return pSprite->getZOrder();

	return 0;
}



