/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoActor.h"
#include "SoGameState.h"
#include "SoInputDispatcher.h"
#include "SoUtils.h"


class SoGameBitPlayer;


class SoActorVisible : public SoActor
{
	public:

		//-----------------------
		// public functions
		SoActorVisible();
		virtual ~SoActorVisible() {}

		virtual void Init( CCNode* pParentNode ); 
		virtual void Deinit();

		virtual void SynchronizeAfterLoad() {};

		virtual void Update( float dt );

		virtual SoActorTypes::TYPE GetType() { return SoActorTypes::TYPE_ACTOR_VISIBLE; }
		virtual SoActorVisible* GetAsVisible() { return this; }

		virtual void OnInput( SoInputDispatcher::InputPacket* pInput );

		virtual bool IsPositionInside( const JpVector2f& vWorldPos );

		// position/size
		virtual JpVector2f GetWorldPosition(); 
		virtual void SetWorldPosition( const JpVector2f& v ); 
		virtual void SetScale( const JpVector2f& vScale );
		virtual JpVector2f GetScale();


		// opacity
		virtual float GetOpacity();
		virtual void SetOpacity( float fPercent );


		// color
		virtual ccColor3B GetColor();
		virtual void SetColor( const ccColor3B& Color );


		virtual int GetZorder();
		virtual void RemoveSprite();
		SoSprite* GetSprite() { return _pSprite; }

		bool IsUserMovable() { return _bIsUserMovable; }

		JpVector2f GetUserMoveOffset() { return _vUserMoveOffset; }
		JpVector2f GetOrganizeOffset() { return _vOrganizeOffset; }
		void SetOrganizeOffset( JpVector2f v ) { _vOrganizeOffset = v; }

		SoGameState* GetGameState() { return _pGame; }
		void SetGameState( SoGameState* p ) { _pGame = p; }


		int GetRenderOrder() { return _iRenderOrder; }

		static void ResetVisibleActorCounter() { _iVisibleActorCounter = 0; }

	protected:

		//-----------------------
		// protected variables
		SoGameState* _pGame;

		JpVector2f _vWorldPosition;

		SoSprite* _pSprite;

		bool _bIsUserMovable;

		JpVector2f _vUserMoveOffset;
		JpVector2f _vOrganizeOffset;

		float _fUserMoveMaxDist;
		bool _bUserIsTouching;

		int _iRenderOrder;
		int _iVisibleActorNumber;

		//-----------------------
		// protected functions
		bool IsPositionInsideSprite( const JpVector2f& vWorldPos, SoSprite* pSprite );


	private:

		//-----------------------
		// private variables
		static int _iVisibleActorCounter;
};


