/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"

#include "JpString.h"
#include "SoActorManager.h"
#include "SoActorVisibleCoordinate.h"
#include "SoActorVisibleTile.h"
#include "SoCamera.h"
#include "SoFontDefs.h"
#include "SoGameBitPlayer.h"
#include "SoGameBitPlayerHuman.h"
#include "SoGameClient.h"
#include "SoGameGlobals.h"
#include "SoLabelBmFont.h"
#include "SoTextureUtils.h"

#include "mmgrChooser.h"




/******************************************************************************
 FUNCTION:  SoActorVisibleCoordinate
******************************************************************************/
SoActorVisibleCoordinate::SoActorVisibleCoordinate() :
_pLabel( NULL )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoActorVisibleCoordinate::Init( CCNode* pParentNode )
{
	SoActorVisible::Init( pParentNode );

	if ( !_pLabel )
			CreateLabel();
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoActorVisibleCoordinate::Deinit()
{	
	RemoveAndDestroyNode( _pParentNode, _pSprite );

	if ( _pLabel )	
		RemoveAndDestroyNode( _pParentNode, _pLabel );

	SoActorVisible::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoActorVisibleCoordinate::Update( float dt )
{
	SoActorVisible::Update( dt );

	
	//-----------------------

	JpVector2f vTargetPos = GetTargetPosition();

	float fDt = ClampMax( dt, 0.1f );
	_vWorldPosition = InterpolateClamped( _vWorldPosition, vTargetPos, 5.0f * fDt );


	JpVector2f vScreenSpacePos = SoCameraManager::GetInstance()->GetCurrentCamera()->TransformWorldToScreenSpace( _vWorldPosition );
	
	if ( _pLabel )
	{
		_pLabel->setVisible( !SoGameGlobals::IsLoadingGame() );
		_pLabel->SetPosition( vScreenSpacePos );
	}
}



/******************************************************************************
 FUNCTION:  SetScale
******************************************************************************/
void SoActorVisibleCoordinate::SetScale( const JpVector2f& vScale )
{
	if ( _pLabel )
	{
		_pLabel->setScaleX( vScale.X * SoUtils::GetiPadRatioX() );
		_pLabel->setScaleY( vScale.Y * SoUtils::GetiPadRatioY() );
	}
}



/******************************************************************************
 FUNCTION:  SetLabelText
******************************************************************************/
void SoActorVisibleCoordinate::SetLabelText( const std::string& s )
{
	if ( _pLabel )
		_pLabel->setString( s.c_str() );
}



/******************************************************************************
 FUNCTION:  CreateLabel
******************************************************************************/
void SoActorVisibleCoordinate::CreateLabel()
{
	if ( !_pLabel  )
	{
		std::string sLabelText = "A";
		_pLabel = CreateBmFontLabel( _pParentNode, sLabelText, SoFontDefs::GetBoardSpot(), kCCTextAlignmentCenter );
		_pLabel->setVisible( false );
		_pLabel->setColor( JpUtils::_Grey128 );
		SoUtils::SetRenderZOrder( _pParentNode, _pLabel, BASE_RENDER_ORDER_COORDINATE );
		SetScale( (1.0f / SoTextureUtils::GetTextureScaler()) * JpVector2f( SoUtils::GetSpriteZoom(), SoUtils::GetSpriteZoom() ) );
	}
}



/******************************************************************************
 FUNCTION:  GetTargetPosition
******************************************************************************/
JpVector2f SoActorVisibleCoordinate::GetTargetPosition()
{
	return GetWorldPosition();
}


