/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"


#include "SoActorManager.h"
#include "SoActorVisiblePlayer.h"
#include "SoCamera.h"
#include "SoConsole.h"
#include "SoFsmStateMain.h"
#include "SoGameBitPlayer.h"
#include "SoHudPlayerGuide.h"
#include "SoHudSwooshNotice.h"
#include "SoKicker.h"
#include "SoTextureUtils.h"

#include "mmgrChooser.h"




/******************************************************************************
 FUNCTION:  SoActorVisiblePlayer
******************************************************************************/
SoActorVisiblePlayer::SoActorVisiblePlayer() :
_pGameBitPlayer( NULL ),
_eState( STATE_LAST ),
_iPreviousMoney( 0 ),
_iPreviousScore( 0 ),
_iLastPlayer( PLAYER_UNDEFINED_PLAYER_NUMBER )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoActorVisiblePlayer::Init( CCNode* pParentNode )
{
	SoActorVisible::Init( pParentNode );

	// This is a bit of a hack.  Since all players start the game with $60, 
	// setting this value prevents the +$60 kicker from showing at games start.
	// mdomowicz 2013_10_22
	_iPreviousMoney = GAMERULES_STARTING_MONEY;

	_eState = STATE_WAIT;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoActorVisiblePlayer::Deinit()
{	
	RemoveAndDestroyNode( _pParentNode, _pSprite );

	SoActorVisible::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoActorVisiblePlayer::Update( float dt )
{
	SoActorVisible::Update( dt );

	switch ( _eState )
	{
		case STATE_WAIT:
		{
			if ( _iPreviousMoney != _pGameBitPlayer->GetMoney() )
			{
				char acText[ 16 ] = "";

				int iDeltaMoney = ( _pGameBitPlayer->GetMoney() - _iPreviousMoney );

				if ( iDeltaMoney > 0 )
					sprintf( acText, "+%i", iDeltaMoney );
				else
					sprintf( acText, "%i", iDeltaMoney );


				JpVector2f vKickerPos = GetBaseKickerPosition();
				vKickerPos.X -= 25.0f * SoUtils::GetiPadRatioX();
				

				SoKickerManager::GetInstance()->AddKicker( this, vKickerPos, acText );

				_iPreviousMoney = _pGameBitPlayer->GetMoney();
			}

			if ( _iLastPlayer != _pGame->GetCurrentPlayerNumber() )
			{
				_iLastPlayer = _pGame->GetCurrentPlayerNumber();
				if ( _iLastPlayer == _pGameBitPlayer->GetPlayerNumber() )
				{
					// The random/easy player moves too quickly, making the notices
					// appear and disappear too fast in a pretty ugly looking way.
					// So for now, I'm just disabling the notices for them.
					// mdomowicz 2013_10_22
					if ( !_pGameBitPlayer->GetAsPlayerMctsRandom() )
						ShowPlayerTurnNotice();
				}
			}

			break;
		}
		case STATE_WON:
		{
			break;
		}
		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknown visual player state." ) );
	}

}



/******************************************************************************
 FUNCTION:  ShowPlayerWinNotice
******************************************************************************/
void SoActorVisiblePlayer::ShowPlayerWinNotice()
{
	std::string sFilenameLeft;
	std::string sFilenameRight;

	SoGameBitPlayer::PLAYER_COLOR eColor = _pGameBitPlayer->GetPlayerColor();

	switch ( eColor )
	{
		case SoGameBitPlayer::PLAYER_COLOR_BLUE:
			sFilenameLeft = "HudWins.png";
			sFilenameRight = "ActorVisibleWorkerLaborerBlue.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_GREEN:
			sFilenameLeft = "HudWins.png";
			sFilenameRight = "ActorVisibleWorkerLaborerGreen.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_PURPLE:
			sFilenameLeft = "HudWins.png";
			sFilenameRight = "ActorVisibleWorkerLaborerPurple.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_RED:
			sFilenameLeft = "HudWins.png";
			sFilenameRight = "ActorVisibleWorkerLaborerRed.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_WHITE:
			sFilenameLeft = "HudWins.png";
			sFilenameRight = "ActorVisibleWorkerLaborerWhite.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_YELLOW:
			sFilenameLeft = "HudWins.png";
			sFilenameRight = "ActorVisibleWorkerLaborerYellow.png";
			break;
	}
	

	SoHudSwooshNoticeManager* pNoticeManager = SoHudSwooshNoticeManager::GetInstance();
	pNoticeManager->AddNotices
	( 
		SoUtils::AddRelativePathToTexture( sFilenameLeft.c_str() ), 
		SoUtils::AddRelativePathToTexture( sFilenameRight.c_str() ),
		20.0f // fShowDuration
	);
}



/******************************************************************************
 FUNCTION:  ShowPlayerScoreNotice
******************************************************************************/
void SoActorVisiblePlayer::ShowPlayerScoreNotice()
{
	std::string sFilenameLeft;
	std::string sFilenameRight;

	SoGameBitPlayer::PLAYER_COLOR eColor = _pGameBitPlayer->GetPlayerColor();

	switch ( eColor )
	{
		case SoGameBitPlayer::PLAYER_COLOR_BLUE:
			sFilenameLeft = "HudScores.png";
			sFilenameRight = "ActorVisibleWorkerLaborerBlue.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_GREEN:
			sFilenameLeft = "HudScores.png";
			sFilenameRight = "ActorVisibleWorkerLaborerGreen.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_PURPLE:
			sFilenameLeft = "HudScores.png";
			sFilenameRight = "ActorVisibleWorkerLaborerPurple.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_RED:
			sFilenameLeft = "HudScores.png";
			sFilenameRight = "ActorVisibleWorkerLaborerRed.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_WHITE:
			sFilenameLeft = "HudScores.png";
			sFilenameRight = "ActorVisibleWorkerLaborerWhite.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_YELLOW:
			sFilenameLeft = "HudScores.png";
			sFilenameRight = "ActorVisibleWorkerLaborerYellow.png";
			break;
	}
	

	SoHudSwooshNoticeManager* pNoticeManager = SoHudSwooshNoticeManager::GetInstance();
	pNoticeManager->AddNotices
	( 
		SoUtils::AddRelativePathToTexture( sFilenameLeft.c_str() ), 
		SoUtils::AddRelativePathToTexture( sFilenameRight.c_str() ),
		0.0f // fShowDuration
	);
}



/******************************************************************************
 FUNCTION:  ShowPlayerTurnNotice
******************************************************************************/
void SoActorVisiblePlayer::ShowPlayerTurnNotice()
{
	std::string sFilenameLeft;
	std::string sFilenameRight;

	SoGameBitPlayer::PLAYER_COLOR eColor = _pGameBitPlayer->GetPlayerColor();

	switch ( eColor )
	{
		case SoGameBitPlayer::PLAYER_COLOR_BLUE:
			sFilenameLeft = "BoardSpot.png";
			sFilenameRight = "ActorVisibleWorkerLaborerBlue.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_GREEN:
			sFilenameLeft = "BoardSpot.png";
			sFilenameRight = "ActorVisibleWorkerLaborerGreen.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_PURPLE:
			sFilenameLeft = "BoardSpot.png";
			sFilenameRight = "ActorVisibleWorkerLaborerPurple.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_RED:
			sFilenameLeft = "BoardSpot.png";
			sFilenameRight = "ActorVisibleWorkerLaborerRed.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_WHITE:
			sFilenameLeft = "BoardSpot.png";
			sFilenameRight = "ActorVisibleWorkerLaborerWhite.png";
			break;
		case SoGameBitPlayer::PLAYER_COLOR_YELLOW:
			sFilenameLeft = "BoardSpot.png";
			sFilenameRight = "ActorVisibleWorkerLaborerYellow.png";
			break;
	}
	

	SoHudSwooshNoticeManager* pNoticeManager = SoHudSwooshNoticeManager::GetInstance();
	pNoticeManager->AddNotices
	( 
		SoUtils::AddRelativePathToTexture( sFilenameLeft.c_str() ), 
		SoUtils::AddRelativePathToTexture( sFilenameRight.c_str() ),
		0.0f // fShowDuration
	);
}



/******************************************************************************
 FUNCTION:  GetBaseKickerPosition
******************************************************************************/
JpVector2f SoActorVisiblePlayer::GetBaseKickerPosition()
{
	JpVector2f vKickerPos;
	vKickerPos = SoHudPlayerGuide::GetHudPositionFromPlayerNumber( _pGameBitPlayer->GetPlayerNumber() );
	vKickerPos.Y += 35.0f * SoUtils::GetiPadRatioY();

	if ( _pGameBitPlayer == _pGame->GetCurrentPlayer() )
		vKickerPos.Y += 15.0f * SoUtils::GetiPadRatioY();

	return vKickerPos;
}



/******************************************************************************
 FUNCTION:  GetGameBitPlayer
******************************************************************************/
SoGameBitPlayer* SoActorVisiblePlayer::GetGameBitPlayer()
{
	return _pGameBitPlayer;
}



/******************************************************************************
 FUNCTION:  SetGameBitPlayer
******************************************************************************/
void SoActorVisiblePlayer::SetGameBitPlayer( SoGameBitPlayer* p )
{
	_pGameBitPlayer = p;
}



/******************************************************************************
 FUNCTION:  GetBoardWorldPositionForPlayer
******************************************************************************/
JpVector2f SoActorVisiblePlayer::GetBoardWorldPositionForPlayer( int iPlayerNumber )
{
	switch ( iPlayerNumber )
	{
		case 0: return JpVector2f( -1100.0f, -800.0f );
		case 1: return JpVector2f( -1285.0f, 0.0f );
		case 2: return JpVector2f( -1100.0f, 800.0f );
		case 3: return JpVector2f( 1100.0f, 800.0f );
		case 4: return JpVector2f( 1285, 0.0f );
		case 5: return JpVector2f( 1100.0f, -800.0f );
		default:
			ASSERTF( 0, ( "\nUnhandled player number." ) );
	};

	return JpVector2f( 0.0f, 0.0f );
}



/******************************************************************************
 FUNCTION:  GetPlayerNameByPlayerNumber
******************************************************************************/
std::string SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( int iPlayerNumber )
{
	switch ( iPlayerNumber )
	{
		case 0: return "P1";
		case 1: return "P2";
		case 2: return "P3";
		case 3: return "P4";
		case 4: return "P5";
		case 5: return "P6";

		default: 
			ASSERTF( 0, ( "\nUnhandled player number" ) );
	}

	return "ERROR";
}



/******************************************************************************
 FUNCTION:  GetColorByPlayerNumber
******************************************************************************/
ccColor3B  SoActorVisiblePlayer::GetColorByPlayerNumber( int iPlayerNumber )
{
	SoGameBitPlayer::PLAYER_COLOR eColor = SoGameBitPlayer::GetPlayerColorByNumber( iPlayerNumber );
	switch ( eColor )
	{
		case SoGameBitPlayer::PLAYER_COLOR_BLUE: return ccc3( 128, 150, 200 );
		case SoGameBitPlayer::PLAYER_COLOR_GREEN: return JpUtils::_Green;
		case SoGameBitPlayer::PLAYER_COLOR_PURPLE: return ccc3( 195, 95, 195 );
		case SoGameBitPlayer::PLAYER_COLOR_RED: return JpUtils::_Red;
		case SoGameBitPlayer::PLAYER_COLOR_WHITE: return JpUtils::_Grey192;
		case SoGameBitPlayer::PLAYER_COLOR_YELLOW: return JpUtils::_Yellow;
		default: 
			ASSERTF( 0, ( "\nUnhandled player color" ) );
	}

	return JpUtils::_Grey64;
}



/******************************************************************************
 FUNCTION:  PrintToHudLog
******************************************************************************/
void SoActorVisiblePlayer::PrintToHudLog( int iPlayerNumber, const std::string& s )
{
	SoHudLog* pLog = SoFsmMainLayer::GetSharedHudLog();

	if ( iPlayerNumber >= 0 )
	{
		char buffer[2048];
		sprintf( buffer, "P%i: %s", iPlayerNumber, s.c_str() );

		pLog->AddMessageF( JpUtils::_White, buffer );
	}
	else
	{
		pLog->AddMessageF( JpUtils::_White, s.c_str() );
	}
}



/******************************************************************************
 FUNCTION:  PrintToHudLogF
******************************************************************************/
void SoActorVisiblePlayer::PrintToHudLogF( int iPlayerNumber, const char* pFormat, ... )
{
	if ( strlen( pFormat ) < 2048 )
	{
		char buffer[2048];
		va_list args;
		va_start (args, pFormat);
		vsprintf (buffer,pFormat, args);
		va_end (args);

		PrintToHudLog( iPlayerNumber, buffer );
	}
}
