/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoActorVisible.h"
#include "SoUtils.h"


#define VISIBLE_TILE_SHOW_COORDINATE_LABELS false


class SoGameBitPlayer;


class SoActorVisibleTile : public SoActorVisible
{
	public:

		//-----------------------
		// public classes
		class OrganizePos
		{
			public:
				OrganizePos() :
				_iOwningTile( TILE_UNDEFINED_ID ),
				_vOffset( 0.0f, 0.0f )
				{}

				~OrganizePos() {}

				int GetOwningTile() { return _iOwningTile; }
				void SetOwningTile( int i ) { _iOwningTile = i; }

				JpVector2f GetOffset() { return _vOffset; }
				void SetOffset( JpVector2f v ) { _vOffset = v; }

			private:
				JpVector2f _vOffset;
				int _iOwningTile;
		};


		//-----------------------
		// public functions
		SoActorVisibleTile();
		virtual ~SoActorVisibleTile() {}

		virtual void Init( CCNode* pParentNode ); 
		virtual void Deinit();

		virtual void SynchronizeAfterLoad();

		virtual void Update( float dt );

		virtual SoActorTypes::TYPE GetType() { return SoActorTypes::TYPE_ACTOR_VISIBLE_TILE; }

		virtual SoActorVisibleTile* GetAsVisibleTile() { return this; }

		virtual void SetScale( const JpVector2f& vScale );


		SoGameBitTile* GetGameBitTile() { return _pGameBitTile; }
		void SetGameBitTile( SoGameBitTile* p ) { _pGameBitTile = p; }

		SoActorVisibleBoardSpot* GetVisibleBoardSpot() { return _pVisibleBoardSpot; }
		void SetVisibleBoardSpot( SoActorVisibleBoardSpot* p ) { _pVisibleBoardSpot = p; }

		static void InitializeOrganizePosArrays();

		static std::string GenerateTileLabelTextFromXy( int iX, int iY );

	private:

		//-----------------------
		// private variables
		SoGameBitTile* _pGameBitTile;

		SoActorVisibleBoardSpot* _pVisibleBoardSpot;

		SoLabelBmFont* _pLabel;
		SoSprite* _pTileBackSprite;


		JpVector2f _vLastCameraPos;


		int _iOrganizeState;
		int _iOrganizeIndex;
		static OrganizePos _OrganizePosArrays[ GAMERULES_MAX_PLAYERS ][ GAMERULES_NUM_TILES_PER_HAND ];

		int _iRenderOrderState;
		int _iPreviousHotelId;

		//-----------------------
		// private functions
		void UpdateTilePosition( float dt );
		void UpdateTileVisibility( float dt );
		void UpdateOrganizePos( float dt );
		void UpdateRenderOrder( float dt );
		void UpdateTileSprite( float dt );

		void SetRenderZOrder( int iRenderZOrder );

		void CreateTileSprites();
		void RecreateMainSprite();
		void CreateLabel();

		JpVector2f GetTargetPosition();

		bool IsFrontVisible();

		static int AssignOrganizePos( SoGameBitTile* pGameBitTile );
		static void UnassignOrganizePos( SoGameBitTile* pGameBitTile );
		static JpVector2f GetOrganizePosFromIndex( SoGameBitTile* pGameBitTile, int iIndex );
};


