/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "JpList.h"
#include "JpVector.h"


USING_NS_CC;



#define MIST_DISTANCE_RANGE   2000.0f

#define SoMistCloudList    JpList< SoMistCloud >


class SoSprite;


class SoMistCloud
{
	public:
		//-----------------------
		// public enums and classes
		enum eSubState
		{
			eSubState_FadeIn,
			eSubState_FadeOut,
			eSubState_FadeOutQuick,
			eSubState_Show,
			eSubState_Done,
			eSubState_Last,
		};


		//-----------------------
		// public functions
		SoMistCloud();

		virtual ~SoMistCloud() {}

		void Init( CCNode* pParentNode, const char* pacMistTexturePathAndFilename );

		void Deinit();

		void Update( float dt );

		JpVector2f GetPosition() { return _vPosition; }
		void SetPosition( JpVector2f& v );

		bool IsDone() { return _eCurrentSubState == eSubState_Done; }

	private:

		//-----------------------
		// private variables
		SoSprite* _pSprite;
		CCNode* _pParentNode;

		JpVector2f _vPosition;
		float _fRotationRate;
		float _fRotation;
		float _fTimer;
		float _fLifetime;
		float _fAlpha;
		float _fScale;
		float _fScaleRate;

		eSubState _eCurrentSubState;


		//-----------------------
		// private functions
		void UpdateBillboard();
		SoSprite* CreateSprite( CCNode* pParent, const char* pacPathAndFilename );
};



class SoBillboardMist
{
	public:

		//-----------------------
		// public functions

		SoBillboardMist();

		virtual ~SoBillboardMist();


		void Init( CCNode* pParentNode, const char* pacMistTexturePathAndFilename, int iMinBillboardCount );

		void Deinit();

		void Update( float dt );

		void CreateAndAddCloud();

		void UpdatePositionOnPlayerJump( float fX, float fY, float fZ );

	private:

		//-----------------------
		// private variables
		CCNode* _pParentNode;

		SoMistCloudList _CloudList;
		int _iMinCloudCount;
		int _iMaxCloudCount;

		std::string _sMistTexturePathAndFilename;
};




