/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpDebug.h"
#include "JpHashTable.h"
#include "SoCamera.h"
#include "SoCameraGameplay.h"

#include "mmgrChooser.h"


USING_NS_CC;


SoCameraManager* SoCameraManager::_pInstance = NULL;


/******************************************************************************
 FUNCTION:  SoCamera
******************************************************************************/
SoCamera::SoCamera() :
_vPosition( 0.0f, 0.0f ),
_hName( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoCamera
******************************************************************************/
SoCamera::~SoCamera()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoCamera::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoCamera::Deinit()
{
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoCamera::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  SetName
******************************************************************************/
void SoCamera::SetName( char* pacName )
{
	_sName = pacName;
	_hName = JpHash( pacName );
}



/******************************************************************************
 FUNCTION:  TransformWorldToScreenSpace
******************************************************************************/
JpVector2f SoCamera::TransformWorldToScreenSpace( const JpVector2f& vWorldSpacePos )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	JpVector2f vCameraSpacePos = vWorldSpacePos - _vPosition;
	vCameraSpacePos.X *= SoUtils::GetSpriteZoom() * SoUtils::GetiPadRatioX();
	vCameraSpacePos.Y *= SoUtils::GetSpriteZoom() * SoUtils::GetiPadRatioY();

	JpVector2f vScreenOffset( ( SizeInPixels.width / 2.0f ), ( SizeInPixels.height / 2.0f ) );

	JpVector2f vFinal = ( vCameraSpacePos + vScreenOffset );
	return vFinal;
}



/******************************************************************************
 FUNCTION:  TransformScreenToWorldSpace
******************************************************************************/
JpVector2f SoCamera::TransformScreenToWorldSpace( const JpVector2f& vScreenSpacePos )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	JpVector2f vScreenOffset( ( SizeInPixels.width / 2.0f ), ( SizeInPixels.height / 2.0f ) );
	
	JpVector2f vCameraSpacePos = vScreenSpacePos - vScreenOffset;

	vCameraSpacePos.X /= SoUtils::GetSpriteZoom();
	vCameraSpacePos.Y /= SoUtils::GetSpriteZoom();
	vCameraSpacePos.X /= SoUtils::GetiPadRatioX();
	vCameraSpacePos.Y /= SoUtils::GetiPadRatioY();

	JpVector2f vWorldSpacePos = vCameraSpacePos + _vPosition;

	return vWorldSpacePos;
}






/******************************************************************************
 FUNCTION:  SoCameraManager
******************************************************************************/
SoCameraManager::SoCameraManager() :
_pCurrentCamera( NULL )
{
	Init();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoCameraManager::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoCameraManager::Deinit()
{
	_CameraList.DeleteAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoCameraManager::Update( float dt )
{
	if ( _pCurrentCamera )
		_pCurrentCamera->Update( dt );

}



/******************************************************************************
 FUNCTION:  CreateFreeCamera
******************************************************************************/
SoCamera* SoCameraManager::CreateFreeCamera( char* pacName )
{
/*
	SoCameraFree* pNewCam = new SoCameraFree();
	pNewCam->SetOgreCamera( _pOgreCamera );
	pNewCam->SetName( pacName );
	pNewCam->Init( _pSceneMgr );
	_CameraList.AddLast( pNewCam );

	return pNewCam;
*/
	return NULL;
}



/******************************************************************************
 FUNCTION:  CreateGameplayCamera
******************************************************************************/
SoCamera* SoCameraManager::CreateGameplayCamera( char* pacName )
{
	SoCameraGameplay* pNewCam = new SoCameraGameplay();
	pNewCam->SetName( pacName );
	pNewCam->Init();
	_CameraList.AddLast( pNewCam );

	return pNewCam;
}



/******************************************************************************
 FUNCTION:  FindCamera
******************************************************************************/
SoCamera* SoCameraManager::FindCamera( char* pacName )
{
	hash hName = JpHash( pacName );

	SoCameraList::Iterator Iter( &_CameraList );
	while ( Iter.Next() )
	{
		SoCamera* pCamera = Iter.GetNodeData();
		if( pCamera->GetNameHash() == hName )
		{
			return pCamera;
		}
	}

	ASSERTF( 0, ( "\nAn SoCamera with the given name was not found." ) );
	return NULL;
}



/******************************************************************************
 FUNCTION:  SwitchToCamera
******************************************************************************/
void SoCameraManager::SwitchToCamera( SoCamera* pCamera )
{
	if ( pCamera )
	{
		if ( _pCurrentCamera )
			_pCurrentCamera->Deactivate();

		pCamera->Activate();

		_pCurrentCamera = pCamera;
	}
}



/******************************************************************************
 FUNCTION:  SwitchToCameraByName
******************************************************************************/
void SoCameraManager::SwitchToCameraByName( char* pacName )
{
	SoCamera* pCamera = FindCamera( pacName );
	SwitchToCamera( pCamera );
}



/******************************************************************************
 FUNCTION:  SwitchToNextCameraInList
******************************************************************************/
void SoCameraManager::SwitchToNextCameraInList()
{
	SoCameraList::Iterator Iter( &_CameraList );
	while ( Iter.Next() )
	{
		SoCamera* pCamera = Iter.GetNodeData();
		if( pCamera == _pCurrentCamera)
		{
			SoCameraList::Node* pCurrent = Iter.GetNode();
			pCurrent = pCurrent->GetNext();

			if ( pCurrent == _CameraList.GetHead() )
				pCurrent = pCurrent->GetNext();

			SoCamera* pNextCamera = pCurrent->GetData();

			SwitchToCamera( pNextCamera );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoCameraManager::CreateInstance()
{
	ASSERTF( _pInstance == NULL, ( "\nCan't create more than one instance.  Not allowed." ) );

	if ( !_pInstance )
		_pInstance = new SoCameraManager;
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoCameraManager::DeleteInstance()
{
	if ( _pInstance )
	{
		delete _pInstance;
		_pInstance = NULL;
	}
}



/******************************************************************************
 FUNCTION:  InitCameraShake
******************************************************************************/
void SoCameraManager::InitCameraShake( float fMagnitude, float fDuration )
{
	SoCamera* pCamera = FindCamera( "Gameplay" );

	if ( pCamera )
	{
		SoCameraGameplay* pGameplayCam = (SoCameraGameplay*)pCamera;

		pGameplayCam->InitCameraShake( fMagnitude, fDuration );
	}
}




