/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "JpHashTable.h"
#include "JpList.h"

USING_NS_CC;


#define SoCameraList JpList< SoCamera >


class SoCameraGameplay;


class SoCamera
{
	public:

		//-----------------------
		// public functions

		SoCamera();

		virtual ~SoCamera();

		virtual void Init();
		virtual void Deinit();
		virtual void Update( float dt );

		virtual void Activate() {};
		virtual void Deactivate() {};

		virtual SoCameraGameplay* GetAsGameplay() { return NULL; }

		virtual JpVector2f TransformWorldToScreenSpace( const JpVector2f& vWorldSpacePos );
		virtual JpVector2f TransformScreenToWorldSpace( const JpVector2f& vScreenSpacePos );

		JpVector2f GetPosition() { return _vPosition; }
		void SetPosition( const JpVector2f& v ) { _vPosition = v; }

		float GetPosX() { return _vPosition.X; }
		void SetPosX( float f ) { _vPosition.X = f; }

		float GetPosY() { return _vPosition.Y; }
		void SetPosY( float f ) { _vPosition.Y = f; }

		void SetName( char* pacName );
		
		std::string* GetNameString() { return &_sName; }

		hash GetNameHash() { return _hName; }


	protected:

		//-----------------------
		// protected variables
		JpVector2f _vPosition;

		std::string _sName;
		hash _hName;
};




class SoCameraManager
{
	public:
		//-----------------------
		// public functions
		SoCameraManager();
		virtual ~SoCameraManager() { Deinit(); }

		void Init();

		void Deinit();

		void Update( float dt );

		SoCamera* CreateFreeCamera( char* pacName );
		SoCamera* CreateGameplayCamera( char* pacName );

		SoCamera* GetCurrentCamera() { return _pCurrentCamera; }
		SoCamera* FindCamera( char* pacName );

		void SwitchToCamera( SoCamera* pCamera );

		void SwitchToCameraByName( char* pacName );

		void SwitchToNextCameraInList();

		void InitCameraShake( float fMagnitude, float fDuration );

		static SoCameraManager* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();


	private:

		//-----------------------
		// private variables
		static SoCameraManager* _pInstance;

		SoCamera* _pCurrentCamera;

		SoCameraList _CameraList;
};


