/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#pragma once

#include "stdafx.h"


#include "JpHashTable.h"
#include "JpList.h"
#include "JpString.h"
#include "JpUtils.h"

#include "SoActor.h"
#include "SoCamera.h"
#include "SoInputDispatcher.h"


#define CAMERA_GAMEPLAYER_MAX_WORLD_X 2000.0f
#define CAMERA_GAMEPLAYER_MAX_WORLD_Y 2000.0f

#define CAMERA_GAMEPLAY_STANDARD_POS_1 JpVector2f( 425.0f, -45.0f )
#define CAMERA_GAMEPLAY_STANDARD_ZOOM_1 0.4f

#define CAMERA_GAMEPLAY_STANDARD_POS_2 JpVector2f( 0.0f, -70.0f )
#define CAMERA_GAMEPLAY_STANDARD_ZOOM_2 0.5f


class SoCameraGameplay : public SoCamera
{
	public:

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_STANDARD,
			STATE_EDGE_PAN,
			STATE_MOVE_TO_TARGET,
			STATE_MOVE_TO_TARGET_AND_ZOOM,
			STATE_LAST
		};


		//-----------------------
		// public functions
		SoCameraGameplay();
		virtual ~SoCameraGameplay() { Deinit(); }

		virtual void Init();
		virtual void Deinit();
		virtual void Update( float dt );

		virtual void Activate();
		virtual void Deactivate();

		virtual SoCameraGameplay* GetAsGameplay() { return this; }

		void OnInput( SoInputDispatcher::InputPacket* pInput );

		SoActor* GetFocusActor();
		void SetFocusActor( SoActor* pActor );
		void ResetFocus();

		void SetZoom( float f );
		void SetTargetZoom( float f ) { _fTargetZoom = f; }

		void SetState( STATE e ) { _eState = e; }

		void InitCameraShake( float fMagnitude, float fDuration );

		void SetVelocity( const JpVector2f& v ) { _vVelocity = v; }

		void MoveCameraTargetPosition( JpVector2f vTargetPos );

		void SetStartPosition( JpVector2f v ) { _vStartPos = v; }
		void JumpaToStartPosition() { SetPosition( _vStartPos ); }

		void MoveCameraTargetPositionAndZoom( JpVector2f vTargetPos, float fZoom );
		void SetCameraTargetPositionAndZoom( JpVector2f vPos, float fZoom );

		void MoveToOpenWindowPositionAndZoom();
		void MoveToStandardPositionAndZoom();
		void JumpToStandardPositionAndZoom();

		void SetStandardPosition( const JpVector2f& v ) { _vStandardPos = v; }
		void SetStandardZoom( float f ) { _fStandardZoom = f; }

	private:

		//-----------------------
		// private variables
		guid _gFocusActor;
		float _fTimeElapsed;

		float _fCurrentZoom;
		float _fTargetZoom;

		float _fShakeTimer;
		float _fShakeMagnitude;
		float _fShakeDuration;

		JpVector2f _vVelocity;
		float _fSpeedMax;
		float _fDrag;

		JpVector2f _vInputTouchVelocity;

		bool _bUserIsTouching;

		STATE _eState;
		JpVector2f _vMoveToTargetPos;

		JpVector2f _vStartPos;

		JpVector2f _vEdgePanDir;

		JpVector2f _vStandardPos;
		float _fStandardZoom;


		//-----------------------
		// private functions
		void SnapToPosition( JpVector2f* pvResult, JpVector2f vTarget, float dt );
};


