/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCardDeck.h"
#include "SoGameBitPlayer.h"
#include "SoConsole.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoCardDeck
******************************************************************************/
SoCardDeck::SoCardDeck()
{
}



/******************************************************************************
 FUNCTION:  ~SoCardDeck
******************************************************************************/
SoCardDeck::~SoCardDeck()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoCardDeck::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoCardDeck::Deinit()
{
	_AllCards.RemoveAll();
	_DrawPile.RemoveAll();
	_DiscardPile.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoCardDeck::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  AddCard
******************************************************************************/
void SoCardDeck::AddCard( SoGameBitCard* pCard )
{
	_AllCards.AddLast( pCard );
}



/******************************************************************************
 FUNCTION:  NewGameShuffle
******************************************************************************/
void SoCardDeck::NewGameShuffle( hash hRngSeed )
{
	JpRandom LocalRng( hRngSeed );
	ShuffleCardList( &_AllCards, &LocalRng );
}



/******************************************************************************
 FUNCTION:  ShuffleCardList
******************************************************************************/
void SoCardDeck::ShuffleCardList( SoGameBitCardList* pList, JpRandom* pRng )
{
	//-----------------------
	// add every card to a temp list, and make sure each
	// new list node is given a random sort key
	SoGameBitCardList TempList;

	SoGameBitCardList::Iterator Iter( pList );
	while ( Iter.Next() )
	{
		SoGameBitCard* pCard = Iter.GetNodeData();
		TempList.AddLast( pCard );
	
		SoGameBitCardList::Node* pLastNode = TempList.GetLast();
		pLastNode->SetSortKey( pRng->GetInt() );
	}


	//-----------------------
	// sort the temp list - this "shuffles" the list
	TempList.QuickSort();


	//-----------------------
	// rebuild the given list, with the values from
	// the temp list.
	pList->RemoveAll();
	SoGameBitCardList::Iterator Iter2( &TempList );
	while ( Iter2.Next() )
	{
		SoGameBitCard* pCard = Iter2.GetNodeData();
		pList->AddLast( pCard );
	}
}



/******************************************************************************
 FUNCTION:  PrintToConsole
******************************************************************************/
void SoCardDeck::PrintToConsole( const std::string& s )
{
	SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();
	pConsole->AddOutputString( s );
	pConsole->SetCurrentTextColor( JpUtils::_White );
	CCLOG( s.c_str() );
}



/******************************************************************************
 FUNCTION:  CPrintF
******************************************************************************/
void SoCardDeck::CPrintF( const char* pFormat, ... )
{
	if ( strlen( pFormat ) < 2048 )
	{
		char buffer[2048];
		va_list args;
		va_start (args, pFormat);
		vsprintf (buffer,pFormat, args);
		va_end (args);

		PrintToConsole( buffer );
	}
}






