/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"



/*
	This class can be used to enable OpenGL Scissor Clipping.

	To use it, make an instance of this class, and make it a parent
	of any CCNode that you want to be clipped.  

	Your code might look like:


		_pClipNode = new SoClipNode;                     // Create clip node instance
		_pParentNode->addChild( _pClipNode );            // Add the new clip node to some parent node.
		_pClipNode->addChild( pNodeToBeClipped );        // Add the node to be clipped to the clip node.
		_pClipNode->EnableClipping( true );              // Enable clipping
		_pClipNode->SetClippingRegion( 0, 0, 100, 100 ); // Set the clipping rect


	This class is based on examples found here:

		http://www.cocos2d-x.org/forums/6/topics/15432
		http://www.learn-cocos2d.com/2011/01/cocos2d-gem-clippingnode/


	mdomowicz 2013_10_03	
*/


USING_NS_CC;


class SoClipNode : public CCNode
{

	public:

		//-----------------------
		// public functions

		SoClipNode();
		virtual ~SoClipNode();

		virtual void visit(void);

		void SetClippingRegion( float fLeftX, float fBottomY, float fWidth, float fHeight );

		void EnableClipping( bool b ) { _bEnableClipping = b; }

	private:

		//-----------------------
		// private variables

		float _fLeftX;
		float _fBottomY;
		float _fWidth;
		float _fHeight;

		bool _bEnableClipping;
};




