/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCodec.h"
#include "SoGameMessage.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SimulateUntilNextPlayerMoveCommand
 NOTES:     For debugging
******************************************************************************/
int SoCodec::ExtractPlayerNumberFromMessage( SoGameMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case Msg::TYPE_CLI_REQUESTS_ADD_TO_SVR:
		case Msg::TYPE_SVR_COMMAND_ADD_CLI:
		case Msg::TYPE_CLI_REQUESTS_SEAT:
		case Msg::TYPE_SVR_COMMAND_START_GAME:
		case Msg::TYPE_SVR_COMMAND_INIT_NEW_GAME:
			return PLAYER_UNDEFINED_PLAYER_NUMBER;

		case Msg::TYPE_SVR_COMMAND_SEAT_ASSIGNMNET:
		{
			SoCodecSvrCommandSeatAssignment Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_TAKE_TURN:		
		{
			SoCodecSvrCommandPlyrTakeTurn Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}


		case Msg::TYPE_SVR_COMMAND_PLYR_PURCHASE_STOCK:
		{
			SoCodecSvrCommandPlyrPurchaseStock Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_PICK_NEW_HOTEL:
		{
			SoCodecSvrCommandPlyrPickNewHotel Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_PICK_MERGE_HOTELS:
		{
			SoCodecSvrCommandPlyrPickMergeHotels Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_MANAGE_OLD_STOCK:
		{
			SoCodecSvrCommandPlyrManageOldStock Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}

		case Msg::TYPE_PLYR_MOVE_PLACE_TILE:
		{
			SoCodecPlyrMovePlaceTile Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}

		case Msg::TYPE_PLYR_MOVE_STOCK_PURCHASES:
		{
			SoCodecPlyrMoveStockPurchases Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}
		case Msg::TYPE_PLYR_MOVE_NEW_HOTEL_PICK:
		{
			SoCodecPlyrMoveNewHotelPick Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}
		case Msg::TYPE_PLYR_MOVE_MERGE_HOTELS:
		{
			SoCodecPlyrMoveMergeHotels Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}
		case Msg::TYPE_PLYR_MOVE_MANAGE_OLD_STOCK:
		{
			SoCodecPlyrManageOldStock Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}
		case Msg::TYPE_PLYR_MOVE_DECLARE_LAST_TURN:
		{
			SoCodecPlyrMoveDeclareLastTurn Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}

		case Msg::TYPE_PLYR_MOVE_UNDO:
		{
			SoCodecPlyrMoveUndo Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}
		case Msg::TYPE_PLYR_MOVE_RESTART_TURN:
		{
			SoCodecPlyrMoveRestartTurn Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}
		case Msg::TYPE_PLYR_MOVE_END_TURN:
		{
			SoCodecPlyrMoveEndTurn Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}

		case Msg::TYPE_DEBUG_COMMAND:
		{
			SoCodecDebugCommand Decoder;
			Decoder.Decode( pMessage );
			return Decoder._iPlayerNumber;
		}

		case Msg::TYPE_LAST:
			ASSERTF( 0, ( "\nUnknown message type." ) );

	}

	return PLAYER_UNDEFINED_PLAYER_NUMBER;
}
