/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

USING_NS_CC;


#include "JpHashTable.h"
#include "JpList.h"

#include "SoActorTypes.h"
#include "SoDebug.h"
#include "SoGameBitHotel.h"
#include "SoGameBitPlayer.h"
#include "SoGameBitTile.h"
#include "SoGameMessage.h"





class SoCodec
{
	public:
		SoCodec() {}
		virtual ~SoCodec() {}

		// For debugging
		static int ExtractPlayerNumberFromMessage( SoGameMessage* pMessage );

	private:
};



class SoCodecSvrCommandAddCli : public SoCodec
{
	public:
		SoCodecSvrCommandAddCli()
		{
			_hRngSeed = 0;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_SVR_COMMAND_ADD_CLI );
			pM->AddHash( _hRngSeed );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_hRngSeed = pM->NextHash();
		}

		hash _hRngSeed;
};



class SoCodecCliRequestsSeat : public SoCodec
{
	public:
		SoCodecCliRequestsSeat()
		{
			_eGameBitTypeOfNewPlayer = SoGameBit::TYPE_GAMEBIT_LAST;
			_eAiPlayerStrength = SoGameBitPlayer::AI_PLAYER_STRENGTH_LAST;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_CLI_REQUESTS_SEAT );
			pM->AddInt( (int)_eGameBitTypeOfNewPlayer );
			pM->AddInt( (int)_eAiPlayerStrength );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_eGameBitTypeOfNewPlayer = (SoGameBit::TYPE)pM->NextInt();
			_eAiPlayerStrength = (SoGameBitPlayer::AI_PLAYER_STRENGTH)pM->NextInt();
		}

		SoGameBit::TYPE _eGameBitTypeOfNewPlayer;	
		SoGameBitPlayer::AI_PLAYER_STRENGTH _eAiPlayerStrength;
};



class SoCodecSvrCommandSeatAssignment : public SoCodec
{
	public:
		SoCodecSvrCommandSeatAssignment()
		{
			_hClientToReceiveSeat = 0;
			_eGameBitTypeOfNewPlayer = SoGameBit::TYPE_GAMEBIT_LAST; 
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
			_eAiPlayerStrength = SoGameBitPlayer::AI_PLAYER_STRENGTH_LAST;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_SVR_COMMAND_SEAT_ASSIGNMNET );
			pM->AddHash( _hClientToReceiveSeat );
			pM->AddInt( (int)_eGameBitTypeOfNewPlayer );
			pM->AddInt( _iPlayerNumber );
			pM->AddInt( (int)_eAiPlayerStrength );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_hClientToReceiveSeat = pM->NextHash();
			_eGameBitTypeOfNewPlayer = (SoGameBit::TYPE)pM->NextInt();
			_iPlayerNumber = pM->NextInt();
			_eAiPlayerStrength = (SoGameBitPlayer::AI_PLAYER_STRENGTH)pM->NextInt();
		}

		hash _hClientToReceiveSeat;
		SoGameBit::TYPE _eGameBitTypeOfNewPlayer;
		int _iPlayerNumber;
		SoGameBitPlayer::AI_PLAYER_STRENGTH _eAiPlayerStrength;
};



class SoCodecSvrCommandInitNewGame : public SoCodec
{
	public:
		SoCodecSvrCommandInitNewGame()
		{
			_bIsVisibleStocks = false;
			_bIsVisibleMoney = false;
			_iStartingPlayer = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_SVR_COMMAND_INIT_NEW_GAME );
			pM->AddBool( _bIsVisibleStocks );
			pM->AddBool( _bIsVisibleMoney );
			pM->AddInt( _iStartingPlayer );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_bIsVisibleStocks = pM->NextBool();
			_bIsVisibleMoney = pM->NextBool();
			_iStartingPlayer = pM->NextInt();
		}

		bool _bIsVisibleStocks;
		bool _bIsVisibleMoney;
		int _iStartingPlayer;
};



class SoCodecSvrCommandPlyrTakeTurn : public SoCodec
{
	public:
		SoCodecSvrCommandPlyrTakeTurn()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_SVR_COMMAND_PLYR_TAKE_TURN );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
		}

		int _iPlayerNumber;
};



class SoCodecSvrCommandPlyrPurchaseStock : public SoCodec
{
	public:
		SoCodecSvrCommandPlyrPurchaseStock()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_SVR_COMMAND_PLYR_PURCHASE_STOCK );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
		}

		int _iPlayerNumber;
};



class SoCodecSvrCommandPlyrPickNewHotel : public SoCodec
{
	public:
		SoCodecSvrCommandPlyrPickNewHotel()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_SVR_COMMAND_PLYR_PICK_NEW_HOTEL );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
		}

		int _iPlayerNumber;
};



class SoCodecSvrCommandPlyrPickMergeHotels : public SoCodec
{
	public:
		SoCodecSvrCommandPlyrPickMergeHotels()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_SVR_COMMAND_PLYR_PICK_MERGE_HOTELS );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
		}

		int _iPlayerNumber;
};



class SoCodecSvrCommandPlyrManageOldStock : public SoCodec
{
	public:
		SoCodecSvrCommandPlyrManageOldStock()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_SVR_COMMAND_PLYR_MANAGE_OLD_STOCK );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
		}

		int _iPlayerNumber;
};



class SoCodecPlyrMovePlaceTile : public SoCodec
{
	public:
		SoCodecPlyrMovePlaceTile() { Reset(); }

		void Reset()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
			_iPlacedTileId = TILE_UNDEFINED_ID;
			_bSkippingTilePlacement = false;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_PLYR_MOVE_PLACE_TILE );
			pM->AddInt( _iPlayerNumber );
			pM->AddInt( _iPlacedTileId );
			pM->AddBool( _bSkippingTilePlacement );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
			_iPlacedTileId = pM->NextInt();
			_bSkippingTilePlacement = pM->NextBool();
		}

		int _iPlayerNumber;
		int _iPlacedTileId;
		bool _bSkippingTilePlacement;
};



class SoCodecPlyrMoveStockPurchases : public SoCodec
{
	public:
		SoCodecPlyrMoveStockPurchases() { Reset(); }

		void Reset()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
			_iAmountOfHotelWithId0 = 0;
			_iAmountOfHotelWithId1 = 0;
			_iAmountOfHotelWithId2 = 0;
			_iAmountOfHotelWithId3 = 0;
			_iAmountOfHotelWithId4 = 0;
			_iAmountOfHotelWithId5 = 0;
			_iAmountOfHotelWithId6 = 0;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_PLYR_MOVE_STOCK_PURCHASES );
			pM->AddInt( _iPlayerNumber );
			pM->AddInt( _iAmountOfHotelWithId0 );
			pM->AddInt( _iAmountOfHotelWithId1 );
			pM->AddInt( _iAmountOfHotelWithId2 );
			pM->AddInt( _iAmountOfHotelWithId3 );
			pM->AddInt( _iAmountOfHotelWithId4 );
			pM->AddInt( _iAmountOfHotelWithId5 );
			pM->AddInt( _iAmountOfHotelWithId6 );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
			_iAmountOfHotelWithId0 = pM->NextInt();
			_iAmountOfHotelWithId1 = pM->NextInt();
			_iAmountOfHotelWithId2 = pM->NextInt();
			_iAmountOfHotelWithId3 = pM->NextInt();
			_iAmountOfHotelWithId4 = pM->NextInt();
			_iAmountOfHotelWithId5 = pM->NextInt();
			_iAmountOfHotelWithId6 = pM->NextInt();
		}

		int _iPlayerNumber;
		int _iAmountOfHotelWithId0;
		int _iAmountOfHotelWithId1;
		int _iAmountOfHotelWithId2;
		int _iAmountOfHotelWithId3;
		int _iAmountOfHotelWithId4;
		int _iAmountOfHotelWithId5;
		int _iAmountOfHotelWithId6;
};



class SoCodecPlyrMoveNewHotelPick : public SoCodec
{
	public:
		SoCodecPlyrMoveNewHotelPick() { Reset(); }

		void Reset()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
			_iHotelId = HOTEL_UNDEFINED_ID;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_PLYR_MOVE_NEW_HOTEL_PICK );
			pM->AddInt( _iPlayerNumber );
			pM->AddInt( _iHotelId );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
			_iHotelId = pM->NextInt();
		}

		int _iPlayerNumber;
		int _iHotelId;
};



class SoCodecPlyrMoveMergeHotels : public SoCodec
{
	public:
		SoCodecPlyrMoveMergeHotels() { Reset(); }

		void Reset()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
			_iSurvivorHotelId = HOTEL_UNDEFINED_ID;
			_iVictimHotelId = HOTEL_UNDEFINED_ID;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_PLYR_MOVE_MERGE_HOTELS );
			pM->AddInt( _iPlayerNumber );
			pM->AddInt( _iSurvivorHotelId );
			pM->AddInt( _iVictimHotelId );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
			_iSurvivorHotelId = pM->NextInt();
			_iVictimHotelId = pM->NextInt();
		}

		int _iPlayerNumber;
		int _iSurvivorHotelId;
		int _iVictimHotelId;
};



class SoCodecPlyrManageOldStock : public SoCodec
{
	public:
		SoCodecPlyrManageOldStock() { Reset(); }

		void Reset()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
			_iNumStocksToKeep = 0;
			_iNumStocksToSell = 0;
			_iNumStocksToTrade = 0;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_PLYR_MOVE_MANAGE_OLD_STOCK );
			pM->AddInt( _iPlayerNumber );
			pM->AddInt( _iNumStocksToKeep );
			pM->AddInt( _iNumStocksToSell );
			pM->AddInt( _iNumStocksToTrade );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
			_iNumStocksToKeep = pM->NextInt();
			_iNumStocksToSell = pM->NextInt();
			_iNumStocksToTrade = pM->NextInt();
		}

		int _iPlayerNumber;
		int _iNumStocksToKeep;
		int _iNumStocksToSell;
		int _iNumStocksToTrade;
};



class SoCodecPlyrMoveDeclareLastTurn : public SoCodec
{
	public:
		SoCodecPlyrMoveDeclareLastTurn() { Reset(); }

		void Reset()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_PLYR_MOVE_DECLARE_LAST_TURN );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
		}

		int _iPlayerNumber;
};



class SoCodecPlyrMoveUndo : public SoCodec
{
	public:
		SoCodecPlyrMoveUndo()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_PLYR_MOVE_UNDO );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
		}

		int _iPlayerNumber;
};



class SoCodecPlyrMoveRestartTurn : public SoCodec
{
	public:
		SoCodecPlyrMoveRestartTurn()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_PLYR_MOVE_RESTART_TURN );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
		}

		int _iPlayerNumber;
};



class SoCodecPlyrMoveEndTurn : public SoCodec
{
	public:
		SoCodecPlyrMoveEndTurn()
		{
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_PLYR_MOVE_END_TURN );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iPlayerNumber = pM->NextInt();
		}

		int _iPlayerNumber;
};



class SoCodecDebugCommand : public SoCodec
{
	public:
		SoCodecDebugCommand()
		{
			_iDebugMessageId = SoDebug::DEBUG_MESSAGE_ID_LAST;
			_iPlayerNumber = PLAYER_UNDEFINED_PLAYER_NUMBER;
		}

		void Encode( SoGameMessage* pM, hash hServerId, hash hClientId )
		{
			pM->ClearDataList();
			pM->Set( hServerId, hClientId, Msg::TYPE_DEBUG_COMMAND );
			pM->AddInt( _iDebugMessageId );
			pM->AddInt( _iPlayerNumber );
		}

		void Decode( SoGameMessage* pM )
		{
			pM->StartDataRead();
			_iDebugMessageId = pM->NextInt();
			_iPlayerNumber = pM->NextInt();
		}

		int _iDebugMessageId;
		int _iPlayerNumber;
};
