/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "NeUtils.h"
#include "SoConfigManager.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


SoConfigManager* SoConfigManager::_pInstance = NULL;



/******************************************************************************
 FUNCTION:  SoConfigManager
******************************************************************************/
SoConfigManager::SoConfigManager()
{
}



/******************************************************************************
 FUNCTION:  ~SoConfigManager
******************************************************************************/
SoConfigManager::~SoConfigManager()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoConfigManager::Init()
{
	#if DCG_IS_TARGET_WIN32
		// Read the _Local.ini
		// Note that we only do this on Windows because on android, 
		// if the _Local.ini is missing (which is true for a clean build)
		// Android will very visibly complain to the user.
		// mdomowicz 2012_12_28
		_Local.ReadFile
		( 
			CCFileUtils::sharedFileUtils()->fullPathFromRelativePath( SoUtils::AddRelativePathToIni("_Local.ini") ) 
		);

		// Read the end user Settings.ini
		_EndUserSettings.ReadFile
		( 
			CCFileUtils::sharedFileUtils()->fullPathFromRelativePath( SoUtils::AddRelativePathToIni("Settings.ini") ) 
		);

	#endif


	#if DCG_IS_TARGET_IOS
		// Just in case the save directory doesn't exist, we create it - thus accessing it won't cause problems.
		SoUtils::CreateSaveDirectory();

		// Read the end user Settings.ini
		// Notice that on iOS, the Settings.ini is in the saves folder.
		_EndUserSettings.ReadFile
		( 
			CCFileUtils::sharedFileUtils()->fullPathFromRelativePath( SoUtils::AddRelativePathToSaveData("Settings.ini") ) 
		);
	#endif 

	#if DCG_IS_TARGET_ANDROID
		// Just in case the save directory doesn't exist, we create it - thus accessing it won't cause problems.
		SoUtils::CreateSaveDirectory();

		CCFileUtils* pFileUtils = CCFileUtils::sharedFileUtils();
		char acFullPathAndFilename[ 256 ] = "";
		sprintf( acFullPathAndFilename, "%s%s", pFileUtils->getWriteablePath().c_str(), SoUtils::AddRelativePathToSaveData("Settings.ini") );

		if ( NeUtils::DoesFileExist( acFullPathAndFilename ) )
			_EndUserSettings.ReadFile( acFullPathAndFilename );
	#endif 

	_First.ReadFile
	( 
		CCFileUtils::sharedFileUtils()->fullPathFromRelativePath( SoUtils::AddRelativePathToIni("_First.ini") ) 
	);
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoConfigManager::Deinit()
{
}



/******************************************************************************
 FUNCTION:  FindLine
******************************************************************************/
NeConfig::Line* SoConfigManager::FindLine( const char* pacFirstItemOnLine )
{
	NeConfig::Line* pLine = _EndUserSettings.FindLine( pacFirstItemOnLine );

	if ( !pLine )
	{
		pLine = _Local.FindLine( pacFirstItemOnLine );

		if ( !pLine )
		{
			pLine = _First.FindLine( pacFirstItemOnLine );
		}
	}

	return pLine;
}



/******************************************************************************
 FUNCTION:  WriteEndUserSettingsToDisk
******************************************************************************/
void SoConfigManager::WriteEndUserSettingsToDisk()
{
	#if DCG_IS_TARGET_WIN32
		_EndUserSettings.WriteFile
		( 
			CCFileUtils::sharedFileUtils()->fullPathFromRelativePath( SoUtils::AddRelativePathToIni("Settings.ini") ) 
		);
	#endif

	#if DCG_IS_TARGET_IOS
		// Write the end user Settings.ini
		// It's saved in the save folder, because on iOS, the Save Folder is a known valid place to write data.
		// Most other locations are not valid for writing.  mdomowicz 2013_11_04
		_EndUserSettings.WriteFile
		( 
			CCFileUtils::sharedFileUtils()->fullPathFromRelativePath( SoUtils::AddRelativePathToSaveData("Settings.ini") ) 
		);
	#endif

	#if DCG_IS_TARGET_ANDROID
		CCFileUtils* pFileUtils = CCFileUtils::sharedFileUtils();

		char acFullPathAndFilename[ 256 ] = "";
		sprintf( acFullPathAndFilename, "%s%s", pFileUtils->getWriteablePath().c_str(), SoUtils::AddRelativePathToSaveData("Settings.ini") );

		_EndUserSettings.WriteFile( acFullPathAndFilename );
	#endif
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoConfigManager::CreateInstance()
{
	ASSERTF( !_pInstance, ( "\nCan't create a second static instance of this class." ) )
	if ( !_pInstance )
	{
		_pInstance = new SoConfigManager;
	}
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoConfigManager::DeleteInstance()
{
	if ( _pInstance )
	{
		_pInstance->Deinit();

		delete _pInstance;
		_pInstance = NULL;
	}
}













