/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"

#include "curl/curl.h"

#include "JpHashTable.h"

#include "SoActorManager.h"
#include "SoActorSorter.h"
#include "SoActorTypes.h"
#include "SoCamera.h"
#include "SoCameraGameplay.h"
#include "SoCodec.h"
#include "SoConsole.h"
#include "SoCurl.h"
#include "SoFontDefs.h"
#include "SoGameBitPlayer.h"
#include "SoGameGlobals.h"
#include "SoHud.h"
#include "SoHudSwooshNotice.h"
#include "SoKicker.h"
#include "SoLabelBmFont.h"
#include "SoLightOverlay.h"
#include "SoNetworkComms.h"
#include "SoSaveSystem.h"
#include "SoTextureUtils.h"

#include "mmgrChooser.h"



bool SoConsoleFunctions::bDisplayStats = false;
float SoConsoleFunctions::fVolumeMusic   = 1.0f;
float SoConsoleFunctions::fVolumeEffects = 1.0f;
float SoConsoleFunctions::fA = 0.0f;
float SoConsoleFunctions::fB = 0.0f;
float SoConsoleFunctions::fC = 0.0f;
float SoConsoleFunctions::fX = 0.0f;
float SoConsoleFunctions::fY = 0.0f;



/******************************************************************************
 FUNCTION:  SoConsole
******************************************************************************/
SoConsole::SoConsole() :
_fTimer( 0.0f ),
_eCurrentState( STATE_LAST ),
_bIsVisible( false ),
_vCurrentPosition( 0.0f, 0.0f ),
_vTargetShowingPosition( 0.0f, 0.0f ),
_vTargetHidingPosition( 0.0f, 0.0f ),
_vBackgroundCurrentPosition( 0.0f, 0.0f ),
_vBackgroundTargetShowingPosition( 0.0f, 0.0f ),
_vBackgroundTargetHidingPosition( 0.0f, 0.0f ),
_pParentNode( NULL ),
_pCursor( NULL ),
_pCurrentCommandLine( NULL ),
_pCurrentSuggestion( NULL ),
_bCursorBlinkFlag( false ),
_fCursorBlinkTimer( 0.0f ),
_iDebugSkipToLevelNumber( -1 )
{
	_CurrentTextColor = JpUtils::_White;
}



/******************************************************************************
 FUNCTION:  ~SoConsole
******************************************************************************/
SoConsole::~SoConsole()
{
}



/******************************************************************************
 FUNCTION: Init
******************************************************************************/
void SoConsole::Init( CCNode* pParentNode )
{
	NeConsole::Init();

	SetInstance( this );

	SetVisible( false );

	_fTimer = 0.0f;
	_eCurrentState = STATE_HIDING;

	_pParentNode = pParentNode;

	#if DCG_IS_TARGET_WIN32
		// Setup the keyboard listener
		_InputKeyboardListener.SetOwner( this );
		SoInputKeyboard::GetInstance()->AddListener( &_InputKeyboardListener );

		_InputKeyboardListener.EnableRepeatForKey( OIS::KC_BACK );
		_InputKeyboardListener.EnableRepeatForKey( OIS::KC_UP );
		_InputKeyboardListener.EnableRepeatForKey( OIS::KC_DOWN );
	#endif

	_InputGestureListener.SetOwner( this );
	SoInputGesture::GetInstance()->AddListener( &_InputGestureListener );

	_InputVirtualKeyboardListener.SetOwner( this );
	SoInputVirtualKeyboard::GetInstance()->AddListener( &_InputVirtualKeyboardListener );


	// Add console callable functions.
	AddItem(".quit", (void*)&SoConsoleFunctions::Quit, NECONSOLE_FUNCTION);
	AddItem(".ToggleDisplayStats", (void*)&SoConsoleFunctions::ToggleDisplayStats, NECONSOLE_FUNCTION );
	AddItem(".SetSpriteZoom", (void*)&SoConsoleFunctions::SetSpriteZoom, NECONSOLE_FUNCTION );
	AddItem(".DebugSave", (void*)&SoConsoleFunctions::DebugSave, NECONSOLE_FUNCTION );
	AddItem(".DebugLoad", (void*)&SoConsoleFunctions::DebugLoad, NECONSOLE_FUNCTION );
	AddItem(".InvalidateSaveData", (void*)&SoConsoleFunctions::InvalidateSaveData, NECONSOLE_FUNCTION );
	AddItem(".DebugPrintActorCounts", (void*)&SoConsoleFunctions::DebugPrintActorCounts, NECONSOLE_FUNCTION );
	AddItem(".SetTextureScaler", (void*)&SoConsoleFunctions::SetTextureScaler, NECONSOLE_FUNCTION );
	AddItem(".GetTextureScaler", (void*)&SoConsoleFunctions::GetTextureScaler, NECONSOLE_FUNCTION );
	AddItem(".SetTextureFilter", (void*)&SoConsoleFunctions::SetTextureFilter, NECONSOLE_FUNCTION );
	AddItem(".GetTextureFilter", (void*)&SoConsoleFunctions::GetTextureFilter, NECONSOLE_FUNCTION );
	AddItem(".GetTextureDump", (void*)&SoConsoleFunctions::GetTextureDump, NECONSOLE_FUNCTION );
	AddItem(".SetLightOverlayBlendMode", (void*)&SoConsoleFunctions::SetLightOverlayBlendMode, NECONSOLE_FUNCTION );
	AddItem(".ListLightOverlayBlendModes", (void*)&SoConsoleFunctions::ListLightOverlayBlendModes, NECONSOLE_FUNCTION );

	AddItem( "fVolumeMusic", &SoConsoleFunctions::fVolumeMusic, NECONSOLE_FLOAT );
	AddItem( "fVolumeEffects", &SoConsoleFunctions::fVolumeEffects, NECONSOLE_FLOAT );

	AddItem( "fA", &SoConsoleFunctions::fA, NECONSOLE_FLOAT );
	AddItem( "fB", &SoConsoleFunctions::fB, NECONSOLE_FLOAT );
	AddItem( "fC", &SoConsoleFunctions::fC, NECONSOLE_FLOAT );
	AddItem( "fX", &SoConsoleFunctions::fX, NECONSOLE_FLOAT );
	AddItem( "fY", &SoConsoleFunctions::fY, NECONSOLE_FLOAT );

	AddItem(".CreateServer", (void*)&SoConsoleFunctions::CreateServer, NECONSOLE_FUNCTION );
	AddItem(".AddClients", (void*)&SoConsoleFunctions::AddClients, NECONSOLE_FUNCTION );
	AddItem(".StartGame", (void*)&SoConsoleFunctions::StartGame, NECONSOLE_FUNCTION );
	AddItem(".EasyStartGameMacro", (void*)&SoConsoleFunctions::EasyStartGameMacro, NECONSOLE_FUNCTION );

	AddItem(".ShowTilesForPlayer", (void*)&SoConsoleFunctions::ShowTilesForPlayer, NECONSOLE_FUNCTION );
	AddItem(".GiveMoneyToPlayer", (void*)&SoConsoleFunctions::GiveMoneyToPlayer, NECONSOLE_FUNCTION );

	AddItem(".GiveTileToPlayer", (void*)&SoConsoleFunctions::GiveTileToPlayer, NECONSOLE_FUNCTION );
	AddItem(".GiveAllTilesToPlayer", (void*)&SoConsoleFunctions::GiveAllTilesToPlayer, NECONSOLE_FUNCTION );

	AddItem(".NetCurlTest", (void*)&SoConsoleFunctions::NetCurlTest, NECONSOLE_FUNCTION );
	AddItem(".NetCurlTestGet", (void*)&SoConsoleFunctions::NetCurlTestGet, NECONSOLE_FUNCTION );
	AddItem(".NetCurlTestPost", (void*)&SoConsoleFunctions::NetCurlTestPost, NECONSOLE_FUNCTION );

	AddItem(".AddNotice", (void*)&SoConsoleFunctions::AddNotice, NECONSOLE_FUNCTION );

	CCSize size = CCDirector::sharedDirector()->getWinSize();

	_HudLog.SetParentNode( pParentNode );
	_HudLog.SetFontName( SoFontDefs::GetTestSmall() );
	_HudLog.SetPositionAndSize( 0, 0, size.width, size.height );
	_HudLog.SetMessageShowDuration( -1.0f );
	SetMaxOutputLines( 25 );


	#if DCG_IS_TARGET_WIN32
		_vTargetShowingPosition.Init( 0.0f, 200.0f * SoUtils::GetiPadRatioY() );
	#else
		_vTargetShowingPosition.Init( 0.0f, 390.0f * SoUtils::GetiPadRatioY() );
	#endif
	
	
	_vTargetHidingPosition.Init( 0.0f, size.height );

	_pBackground = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "imgBlack.png" ) );


	// Place and scale the image so that it's stretched to fill the entire screen
	_pBackground->setPosition( ccp(size.width/2, size.height/2) );
	_pBackground->setScaleX( size.width / _pBackground->getContentSize().width );
	_pBackground->setScaleY( size.height / _pBackground->getContentSize().height );
	_pBackground->setVisible( false );

	_pBackground->setColor( SoUtils::ccc3BFromFloats( 1.0f, 1.0f, 1.0f ) );
	_pBackground->setOpacity( 220.0f );

	#if DCG_IS_TARGET_WIN32
		_vBackgroundTargetShowingPosition.Init( size.width / 2.0f, 600.0f * SoUtils::GetiPadRatioY() );
	#else
		_vBackgroundTargetShowingPosition.Init( size.width / 2.0f, 800.0f * SoUtils::GetiPadRatioY() );
	#endif

	_vBackgroundTargetHidingPosition.Init( size.width / 2.0f, size.height + ( size.height / 2 ) ); 

	_pParentNode->addChild( _pBackground, BASE_RENDER_ORDER_SHELL );

	int iRenderOrder = 0;
	// 		_pScreenFader = SoScreenFader::create();
	// 		_pScreenFader->retain();
	// 		_pScreenFader->Setup( this, iRenderOrder );
	// 		_pScreenFader->SetFadeSpeed( 1.0f );
	// 		_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_OPAQUE_TO_TRANSPARENT );
	// 		_pScreenFader->BeginFade();
}



/******************************************************************************
 FUNCTION: Deinit
******************************************************************************/
void SoConsole::Deinit()
{
	NeConsole::Deinit();

	SetInstance( NULL );

	_HudLog.Deinit();

	#if DCG_IS_TARGET_WIN32
		SoInputKeyboard::GetInstance()->RemoveListener( &_InputKeyboardListener );
	#endif

	SoInputGesture::GetInstance()->RemoveListener( &_InputGestureListener );
	SoInputVirtualKeyboard::GetInstance()->RemoveListener( &_InputVirtualKeyboardListener );


	if ( _pCurrentCommandLine )
	{
		_pParentNode->removeChild( _pCurrentCommandLine, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pCurrentCommandLine->release();
		_pCurrentCommandLine = NULL;
	}

	if ( _pCurrentSuggestion )
	{
		_pParentNode->removeChild( _pCurrentSuggestion, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pCurrentSuggestion->release();
		_pCurrentSuggestion = NULL;
	}

	if ( _pCursor )
	{
		_pParentNode->removeChild( _pCursor, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pCursor->release();
		_pCursor = NULL;
	}


	_pParentNode = NULL;
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoConsole::Update( float dt )
{
	_pBackground->update( dt );
	NeConsole::Update( dt );
	_HudLog.Update( dt );

	UpdateCursor( dt );


	_fTimer += dt;

	switch ( _eCurrentState )
	{
		case STATE_HIDING:
		{
			_vCurrentPosition = _vTargetHidingPosition;
			_vBackgroundCurrentPosition = _vBackgroundTargetHidingPosition;
			break;
		}
		case STATE_TRANSITION_IN_INIT:
		{
			_fTimer = 0.0f;
			SetVisible( true );
			_pBackground->setVisible( true );
			_eCurrentState = STATE_TRANSITION_IN;
		}
		case STATE_TRANSITION_IN:
		{
			float fTween = _fTimer * 2.0f;
			_vCurrentPosition = Interpolate( _vCurrentPosition, _vTargetShowingPosition, fTween );
	
			_vBackgroundCurrentPosition = Interpolate( _vBackgroundCurrentPosition, _vBackgroundTargetShowingPosition, fTween );
			_pBackground->setPosition( ccp( _vBackgroundCurrentPosition.X, _vBackgroundCurrentPosition.Y ) );

			if ( fTween > 1.0f )
			{
				_fTimer = 0.0f;
				SetVisible( true );
				_eCurrentState = STATE_SHOWING;
			}
			break;
		}
		case STATE_SHOWING:
		{
			_vCurrentPosition = _vTargetShowingPosition;
			break;
		}
		case STATE_TRANSITION_OUT:
		{
			float fTween = _fTimer * 2.0f;
			_vCurrentPosition = Interpolate( _vCurrentPosition, _vTargetHidingPosition, fTween );
		
			_vBackgroundCurrentPosition = Interpolate( _vBackgroundCurrentPosition, _vBackgroundTargetHidingPosition, fTween );
			_pBackground->setPosition( ccp( _vBackgroundCurrentPosition.X, _vBackgroundCurrentPosition.Y ) );

			if ( fTween > 1.0f )
			{
				_fTimer = 0.0f;
				SetVisible( false );
				_pBackground->setVisible( false );
				_eCurrentState = STATE_HIDING;
			}
			break;
		}
	}

	if ( _pCurrentCommandLine )
		_pCurrentCommandLine->setPosition( _vCurrentPosition.X, _vCurrentPosition.Y );

	if ( _pCurrentSuggestion )
		_pCurrentSuggestion->setPosition( _vCurrentPosition.X, _vCurrentPosition.Y );

	float fY = _vCurrentPosition.Y;
	if ( _pCurrentCommandLine )
		fY += ( _pCurrentCommandLine->GetLineHeight() * SoUtils::GetiPadRatioY() );

	_HudLog.SetPosition( _vCurrentPosition.X, fY );

}



/******************************************************************************
 FUNCTION:  UpdateCursor
******************************************************************************/
void SoConsole::UpdateCursor( float dt )
{
	if ( IsVisible() )
		_fCursorBlinkTimer += dt;

	if ( _fCursorBlinkTimer > 0.5f )
	{
		_fCursorBlinkTimer -= 0.5f;
		_bCursorBlinkFlag = !_bCursorBlinkFlag;
	}

	std::string sCursor = " ";
	if ( _bCursorBlinkFlag )
		sCursor = "_";


	if ( _pCursor )
		_pCursor->setString( sCursor.c_str() );
	else
	{
		_pCursor = CreateBmFontLabel( sCursor, _sFontName );
		_pCursor->setScaleX( SoUtils::GetiPadRatioX() );
		_pCursor->setScaleY( SoUtils::GetiPadRatioY() );
	}


	float fX = _vCurrentPosition.X;
	if ( _pCurrentCommandLine )
	{
		if ( strlen( _pCurrentCommandLine->getString() ) != 0 )
			fX += _pCurrentCommandLine->GetCalculatedWidth() * SoUtils::GetiPadRatioX();
	}

	_pCursor->setPosition( fX, _vCurrentPosition.Y );
}



/******************************************************************************
 FUNCTION:  SetVisible
******************************************************************************/
void SoConsole::SetVisible( bool b )
{
	_bIsVisible = b;
	_HudLog.SetVisible( b );

	if ( _pCursor )
		_pCursor->setVisible( b );

	if ( _pCurrentCommandLine )
		_pCurrentCommandLine->setVisible( b );

	if ( _pCurrentSuggestion )
		_pCurrentSuggestion->setVisible( b );
}



/******************************************************************************
 FUNCTION:  SetPosition
******************************************************************************/
void SoConsole::SetPosition
( 
	float fBottomLeftX, 
	float fBottomLeftY
)
{
	_vCurrentPosition.Init( fBottomLeftX, fBottomLeftY );
}



/******************************************************************************
 FUNCTION:  SetFontName
******************************************************************************/
void SoConsole::SetFontName()
{
	_sFontName = SoFontDefs::GetTestSmall();
	_HudLog.SetFontName( SoFontDefs::GetTestSmall() );
}



/******************************************************************************
 FUNCTION:  AddOutputString
******************************************************************************/
void SoConsole::AddOutputString( const std::string& s )
{
	_HudLog.AddMessage( s, _CurrentTextColor );
}



/******************************************************************************
 FUNCTION:  SetCurrentCommandLine
******************************************************************************/
void SoConsole::SetCurrentCommandLine( const std::string& s )
{
	if ( !_pCurrentCommandLine )
	{
		if ( _sFontName == "" )
			SetFontName();

		_pCurrentCommandLine = CreateBmFontLabel( s, _sFontName );
		_pCurrentCommandLine->setScaleX( SoUtils::GetiPadRatioX() );
		_pCurrentCommandLine->setScaleY( SoUtils::GetiPadRatioY() );
	}
	else
	{
		_pCurrentCommandLine->setColor( _CurrentTextColor );
		_pCurrentCommandLine->setString( s.c_str() );
	}
}



/******************************************************************************
 FUNCTION:  SetCurrentSuggestion
******************************************************************************/
void SoConsole::SetCurrentSuggestion( const std::string& s )
{
	if ( !_pCurrentSuggestion )
	{
		if ( _sFontName == "" )
			SetFontName();

		_pCurrentSuggestion = CreateBmFontLabel( s, _sFontName );
		_pCurrentSuggestion->setScaleX( SoUtils::GetiPadRatioX() );
		_pCurrentSuggestion->setScaleY( SoUtils::GetiPadRatioY() );
		_pCurrentSuggestion->setOpacity( 128 );
	}
	else
	{
		_pCurrentSuggestion->setColor( _CurrentTextColor );
		_pCurrentSuggestion->setString( s.c_str() );
	}
}



/******************************************************************************
 FUNCTION:  ClearConsole
******************************************************************************/
void SoConsole::ClearConsole()
{
	_HudLog.Clear();
}



/******************************************************************************
 FUNCTION:  OnKeyPressed
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void SoConsole::OnKeyPressed( OIS::KeyCode Key )
	{
		// Disable debug features for the final build!
		#ifdef FINAL_BUILD
			return;
		#endif


		if ( Key == OIS::KC_GRAVE )
		{
			if ( _eCurrentState == STATE_HIDING )
			{
				_fTimer = 0.0f;
				_eCurrentState = STATE_TRANSITION_IN_INIT;
			}
			else if ( _eCurrentState == STATE_SHOWING )
			{
				_fTimer = 0.0f;
				_eCurrentState = STATE_TRANSITION_OUT;
			}
		}
		else
		{
			if ( _eCurrentState == STATE_SHOWING || _eCurrentState == STATE_TRANSITION_IN )
				UpdateInputPressed( Key );
		}
	}
#endif



/******************************************************************************
 FUNCTION:  OnKeyReleased
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void SoConsole::OnKeyReleased( OIS::KeyCode Key )
	{
		UpdateInputReleased( Key );
	}
#endif



/******************************************************************************
 FUNCTION:  OnVirtualKeyPressed
******************************************************************************/
void SoConsole::OnVirtualKeyPressed( char cKey )
{
	// Disable debug features for the final build!
	#ifdef _FINAL
		return;
	#endif

	#ifdef FINAL_BUILD
		return;
	#endif


	if ( _eCurrentState == STATE_SHOWING || _eCurrentState == STATE_TRANSITION_IN )
		UpdateVirtualKeyboardInputPressed( cKey );
}



/******************************************************************************
 FUNCTION:  OnVirtualKeyReleased
******************************************************************************/
void SoConsole::OnVirtualKeyReleased( char cKey )
{
}



/******************************************************************************
 FUNCTION:  OnVirtualKeyDeleteBackward
******************************************************************************/
void SoConsole::OnVirtualKeyDeleteBackward()
{
	if ( _eCurrentState == STATE_SHOWING || _eCurrentState == STATE_TRANSITION_IN )
		UpdateVirtualKeyboardDeleteBackward();
}



/******************************************************************************
 FUNCTION:  CreateBmFontLabel
******************************************************************************/
SoLabelBmFont* SoConsole::CreateBmFontLabel( const std::string& sText, const std::string& sFontName )
{
	CCSize size = CCDirector::sharedDirector()->getWinSize();

	SoLabelBmFont* pLabel = SoLabelBmFont::create
	(
		sText.c_str(), 
		SoUtils::AddRelativePathToFont( sFontName.c_str() ), 
		size.width, 
		kCCTextAlignmentLeft
	);
	pLabel->setColor( _CurrentTextColor );
	pLabel->setOpacity( 255.0f );
	pLabel->retain();
	pLabel->setAnchorPoint( CCPoint( 0, 0 ) );

	_pParentNode->addChild(pLabel, BASE_RENDER_ORDER_SHELL + 1);


	return pLabel;
}



/******************************************************************************
 FUNCTION:  OnGestureDown
 *****************************************************************************/
bool SoConsole::OnGestureDown( SoGesture* pGesture )
{
	return false;
}



/******************************************************************************
 FUNCTION:  OnGestureUp
 *****************************************************************************/
bool SoConsole::OnGestureUp( SoGesture* pGesture )
{
	#ifdef FINAL_BUILD
		return false;
	#endif

	if ( pGesture->_iNumTouches == 3 )
	{
		if ( _eCurrentState == STATE_HIDING )
		{
			_fTimer = 0.0f;
			_eCurrentState = STATE_TRANSITION_IN_INIT;

			SoInputVirtualKeyboard* pVirtualKeyboard = SoInputVirtualKeyboard::GetInstance();
			if ( pVirtualKeyboard )
				pVirtualKeyboard->BeginTextEntryShowVirtualKeyboardIfNeeded();
		}
		else if ( _eCurrentState == STATE_SHOWING )
		{
			_fTimer = 0.0f;
			_eCurrentState = STATE_TRANSITION_OUT;
		}

		return true;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnGestureMove
 *****************************************************************************/
bool SoConsole::OnGestureMove( SoGesture* pGesture )
{
	switch ( pGesture->_eGestureType )
	{
		case SoGesture::GESTURE_TYPE_ZOOM:
		case SoGesture::GESTURE_TYPE_PINCH:
		{
			break;
		}
		default:
			break;
	}

	return false;
}

















/******************************************************************************
 FUNCTION: Quit
******************************************************************************/
void SoConsoleFunctions::Quit( NeConsole* pInstance, const std::vector<std::string> & args )
{
	CCDirector::sharedDirector()->end();
}



/******************************************************************************
 FUNCTION: ToggleDisplayStats
******************************************************************************/
void SoConsoleFunctions::ToggleDisplayStats( NeConsole* pInstance, const std::vector<std::string> & args )
{
	bDisplayStats = !bDisplayStats;
	CCDirector::sharedDirector()->setDisplayStats( bDisplayStats );
}



/******************************************************************************
 FUNCTION: SetSpriteZoom
******************************************************************************/
void SoConsoleFunctions::SetSpriteZoom( NeConsole* pInstance, const std::vector<std::string> & args )
{
	std::string sArg1 = args[ 1 ];
	float fArg = JpString::ParseFloat( sArg1.c_str() );

	//-----------------------
	// reset the camera
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	if ( pCameraManager )
	{
		SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
		SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();
		pCameraGameplay->SetTargetZoom( fArg );
	}
}



/******************************************************************************
 FUNCTION: DebugSave
******************************************************************************/
void SoConsoleFunctions::DebugSave( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoGameServer* pServer = SoGameGlobals::GetServer();
	pServer->WriteSaveGameCsvToDisk();	
}



/******************************************************************************
 FUNCTION: DebugLoad
******************************************************************************/
void SoConsoleFunctions::DebugLoad( NeConsole* pInstance, const std::vector<std::string> & args )
{
}



/******************************************************************************
 FUNCTION: InvalidateSaveData
 NOTES:    
******************************************************************************/
void SoConsoleFunctions::InvalidateSaveData( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoGameServer* pServer = SoGameGlobals::GetServer();
	pServer->InvalidateSaveData();	
}


/******************************************************************************
 FUNCTION: DebugPrintActorCounts
******************************************************************************/
void SoConsoleFunctions::DebugPrintActorCounts( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoHudLog* pHudLog = ((SoConsole*)pInstance)->GetHudLog();
	SoActorSorter* pSorter = SoActorSorter::GetInstance();

	for ( int i = 0; i < (int)SoActorTypes::TYPE_LAST; i ++ )
	{
		SoActorTypes::TYPE eType = (SoActorTypes::TYPE)i;
		int iCount = pSorter->GetNumExistingActorsByType( eType );

		pHudLog->AddMessageF( JpUtils::_Magenta, "%s: %i", SoActorTypes::GetStringFromType( eType ), iCount );
	}
}



/******************************************************************************
 FUNCTION: GetTextureScaler
******************************************************************************/
void SoConsoleFunctions::GetTextureScaler( NeConsole* pInstance )
{
	SoHudLog* pHudLog = ((SoConsole*)pInstance)->GetHudLog();
	ccColor3B Color = JpUtils::_Grey224;

	pHudLog->AddMessageF( Color, "Texture scaler set to: %f", SoTextureUtils::GetTextureScaler() );
	pHudLog->AddMessageF( Color, "------------------------------------------------------" );
}



/******************************************************************************
 FUNCTION: SetTextureScaler
******************************************************************************/
void SoConsoleFunctions::SetTextureScaler( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoTextureUtils::ResetSharedTextures();

	// Set the new texture scaler.
	std::string sArg1 = args[ 1 ];
	float fArg = JpString::ParseFloat( sArg1.c_str() );

	SoTextureUtils::SetTextureScaler( fArg );

	SoHudLog* pHudLog = ((SoConsole*)pInstance)->GetHudLog();
	ccColor3B Color = JpUtils::_Grey224;

	pHudLog->AddMessageF( Color, "Texture scaler set to: %f", SoTextureUtils::GetTextureScaler() );
}



/******************************************************************************
 FUNCTION: GetTextureFilter
******************************************************************************/
void SoConsoleFunctions::GetTextureFilter( NeConsole* pInstance )
{
	SoHudLog* pHudLog = ((SoConsole*)pInstance)->GetHudLog();
	ccColor3B Color = JpUtils::_Grey224;

	pHudLog->AddMessageF( Color, "Texture Filter set to: %i", SoTextureUtils::GetTextureFilter() );
	pHudLog->AddMessageF( Color, "------------------------------------------------------" );
}



/******************************************************************************
 FUNCTION: SetTextureFilter
******************************************************************************/
void SoConsoleFunctions::SetTextureFilter( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoTextureUtils::ResetSharedTextures();

	// Set the new texture filter.
	std::string sArg1 = args[ 1 ];
	int iArg = JpString::ParseInt( sArg1.c_str() );

	SoTextureUtils::SetTextureFilter( iArg );

	SoHudLog* pHudLog = ((SoConsole*)pInstance)->GetHudLog();
	ccColor3B Color = JpUtils::_Grey224;

	pHudLog->AddMessageF( Color, "Texture filter set to: %i", SoTextureUtils::GetTextureFilter() );
}



/******************************************************************************
 FUNCTION: GetTextureDump
******************************************************************************/
void SoConsoleFunctions::GetTextureDump( NeConsole* pInstance )
{
	CCTextureCache::sharedTextureCache()->dumpCachedTextureInfo();

	SoHudLog* pHudLog = ((SoConsole*)pInstance)->GetHudLog();
	ccColor3B Color = JpUtils::_Grey224;

	pHudLog->AddMessageF( Color, "Texture memory info dumped to debug screen." );
}



/******************************************************************************
 FUNCTION: SetLightOverlayBlendMode
******************************************************************************/
void SoConsoleFunctions::SetLightOverlayBlendMode( NeConsole* pInstance, const std::vector<std::string> & args )
{
	std::string sArg1 = args[ 1 ];
	std::string sArg2 = args[ 2 ];
	int iArg1 = JpString::ParseInt( sArg1.c_str() );
	int iArg2 = JpString::ParseInt( sArg2.c_str() );

	/*
	SoGameLevel* pGameLevel = SoGameDungeon::GetInstance()->GetCurrentGameLevel();

	if ( pGameLevel )
	{
		SoLightOverlay* pOverlay = pGameLevel->GetLightOverlay();
		ccBlendFunc BlendFunc;
		BlendFunc.src = iArg1;
		BlendFunc.dst = iArg2;
		if ( pOverlay )
			pOverlay->GetSprite()->setBlendFunc( BlendFunc );
	}
	*/
}



/******************************************************************************
 FUNCTION: ListLightOverlayBlendModes
******************************************************************************/
void SoConsoleFunctions::ListLightOverlayBlendModes( NeConsole* pInstance )
{
	SoHudLog* pHudLog = ((SoConsole*)pInstance)->GetHudLog();
	ccColor3B Color = JpUtils::_Magenta;

	pHudLog->AddMessageF( Color, "0     GL_ZERO" );
	pHudLog->AddMessageF( Color, "1     GL_ONE" );
	pHudLog->AddMessageF( Color, "0x300  GL_SRC_COLOR" );
	pHudLog->AddMessageF( Color, "0x301  GL_ONE_MINUS_SRC_COLOR" );
	pHudLog->AddMessageF( Color, "0x306  GL_DST_COLOR" );
	pHudLog->AddMessageF( Color, "0x307  GL_ONE_MINUS_DST_COLOR" );
	pHudLog->AddMessageF( Color, "0x302  GL_SRC_ALPHA" );
	pHudLog->AddMessageF( Color, "0x303  GL_ONE_MINUS_SRC_ALPHA" );
	pHudLog->AddMessageF( Color, "0x308  GL_SRC_ALPHA_SATURATE" );
	pHudLog->AddMessageF( Color, "0x8001 GL_CONSTANT_COLOR" );
	pHudLog->AddMessageF( Color, "0x8002 GL_ONE_MINUS_CONSTANT_COLOR" );
	pHudLog->AddMessageF( Color, "0x8003 GL_CONSTANT_ALPHA" );
	pHudLog->AddMessageF( Color, "0x8004 GL_ONE_MINUS_CONSTANT_ALPHA" );
	pHudLog->AddMessageF( Color, "Usage: '\\SetLightOverlayBlendMode SourceInt DestInt'" );
	pHudLog->AddMessageF( Color, "'\\SetLightOverlayBlendMode 0x301 0x308'" );
}



/******************************************************************************
 FUNCTION: CreateServer
******************************************************************************/
void SoConsoleFunctions::CreateServer( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoGameGlobals::CreateServer( "" );
}



/******************************************************************************
 FUNCTION: AddClients
******************************************************************************/
void SoConsoleFunctions::AddClients( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoGameClient* pClient = SoGameGlobals::CreateClient();

	char acTemp[ 128 ];
	sprintf( acTemp, "Client created.  Client Id: %x", pClient->GetClientId() );
	pInstance->AddOutputString( acTemp );
	pClient->ConnectToServer( "" );

	/*
	SoGameClient* pClient2 = SoGameGlobals::CreateClient();
	sprintf( acTemp, "Client created.  Client Id: %x", pClient2->GetClientId() );
	pInstance->AddOutputString( acTemp );
	pClient2->ConnectToServer();
	*/


	pClient->CreatePlayerHuman();
	pClient->CreatePlayerHuman();
	pClient->CreatePlayerHuman();
	pClient->CreatePlayerHuman();
	pClient->CreatePlayerHuman();
}



/******************************************************************************
 FUNCTION: StartGame
******************************************************************************/
void SoConsoleFunctions::StartGame( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();

	// hack! - this won't work when we need to support multiple local servers.
	hash hServerId = pNetwork->FindAvailableServer(); 

	SoGameMessage Message( hServerId, -1, Msg::TYPE_SVR_COMMAND_START_GAME );
	pNetwork->SendMessageToServer( Message );
}



/******************************************************************************
 FUNCTION: EasyStartGameMacro
******************************************************************************/
void SoConsoleFunctions::EasyStartGameMacro( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoGameGlobals::CreateServer( "" );

	SoGameClient* pClient = SoGameGlobals::CreateClient();
	pClient->ConnectToServer( "" );


	//SoGameMessage Message( hServerId, -1, Msg::TYPE_TOSERVER_START_GAME );
	//pNetwork->SendMessageToServer( Message );
}



/******************************************************************************
 FUNCTION: ShowTilesForPlayer
******************************************************************************/
void SoConsoleFunctions::ShowTilesForPlayer( NeConsole* pInstance, const std::vector<std::string> & args )
{
	if ( args.size() > 0 )
	{
		std::string sArg1 = args[ 1 ];
		int iPlayerNumber = JpString::ParseInt( sArg1.c_str() );

		SoDebug::SetPlayerWithDebugHighlightEnabled( iPlayerNumber );
	}
}


/******************************************************************************
 FUNCTION: GiveMoneyToPlayer
******************************************************************************/
void SoConsoleFunctions::GiveMoneyToPlayer( NeConsole* pInstance, const std::vector<std::string> & args )
{
	// get the player number parameter
	std::string sArg1 = args[ 1 ];
	int iPlayerNumber = JpString::ParseInt( sArg1.c_str() );

	// setup the encoder
	SoCodecDebugCommand Encoder;
	Encoder._iDebugMessageId = SoDebug::DEBUG_MESSAGE_ID_GIVE_PLAYER_MONEY;
	Encoder._iPlayerNumber = iPlayerNumber;

	// encode the message
	SoGameMessage Message;
	Encoder.Encode( &Message, GetServerId(), 0 );

	// send the message
	SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();
	pNetwork->SendMessageToServer( Message );
}



/******************************************************************************
 FUNCTION: GiveTileToPlayer
******************************************************************************/
void SoConsoleFunctions::GiveTileToPlayer( NeConsole* pInstance, const std::vector<std::string> & args )
{
	// get the player number parameter
	std::string sArg1 = args[ 1 ];
	int iPlayerNumber = JpString::ParseInt( sArg1.c_str() );

	// setup the encoder
	SoCodecDebugCommand Encoder;
	Encoder._iDebugMessageId = SoDebug::DEBUG_MESSAGE_ID_GIVE_PLAYER_TILE;
	Encoder._iPlayerNumber = iPlayerNumber;

	// encode the message
	SoGameMessage Message;
	Encoder.Encode( &Message, GetServerId(), 0 );

	// send the message
	SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();
	pNetwork->SendMessageToServer( Message );
}



/******************************************************************************
 FUNCTION: GiveAllTilesToPlayer
******************************************************************************/
void SoConsoleFunctions::GiveAllTilesToPlayer( NeConsole* pInstance, const std::vector<std::string> & args )
{
	// get the player number parameter
	std::string sArg1 = args[ 1 ];
	int iPlayerNumber = JpString::ParseInt( sArg1.c_str() );

	// setup the encoder
	SoCodecDebugCommand Encoder;
	Encoder._iDebugMessageId = SoDebug::DEBUG_MESSAGE_ID_GIVE_PLAYER_ALL_TILES;
	Encoder._iPlayerNumber = iPlayerNumber;

	// encode the message
	SoGameMessage Message;
	Encoder.Encode( &Message, GetServerId(), 0 );

	// send the message
	SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();
	pNetwork->SendMessageToServer( Message );
}



/******************************************************************************
 FUNCTION: NetCurlTest
******************************************************************************/
void SoConsoleFunctions::NetCurlTest( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoCurl::GetInstance()->CurlTest();
}



/******************************************************************************
 FUNCTION: NetCurlTestGet
******************************************************************************/
void SoConsoleFunctions::NetCurlTestGet( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoCurl::GetInstance()->CurlTestGet();
}



/******************************************************************************
 FUNCTION: NetCurlTestPost
******************************************************************************/
void SoConsoleFunctions::NetCurlTestPost( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoCurl::GetInstance()->CurlTestPost();
}



/******************************************************************************
 FUNCTION: AddNotice
******************************************************************************/
void SoConsoleFunctions::AddNotice( NeConsole* pInstance, const std::vector<std::string> & args )
{
	SoHudSwooshNoticeManager* pNoticeManager = SoHudSwooshNoticeManager::GetInstance();
	//pNoticeManager->AddNotices( SoUtils::AddRelativePathToTexture( "ActorVisibleCardBomb07.png" ), SoUtils::AddRelativePathToTexture( "ActorVisibleWorkerEngineerGreen.png" ), 2.0f );
	pNoticeManager->AddNotices( SoUtils::AddRelativePathToTexture( "HudWins.png" ), SoUtils::AddRelativePathToTexture( "ActorVisibleWorkerEngineerGreen.png" ), 2.0f );
}



/******************************************************************************
 FUNCTION: GetServerId
******************************************************************************/
hash SoConsoleFunctions::GetServerId()
{
	SoGameClientList* pClientList = SoGameGlobals::GetClientList();
	SoGameClientList::Node* pFirstNode = pClientList->GetFirst();
	SoGameClient* pClient = pFirstNode->GetData();
	return pClient->GetServerId();
}
