/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

// Some of this was based off of: http://www.ogre3d.org/tikiwiki/ConsoleCode


#pragma once

#include "stdafx.h"

#include <iostream>

#include "NeConsole.h"

#include "SoHudLog.h"
#include "SoInputGesture.h"
#include "SoInputKeyboard.h"
#include "SoInputVirtualKeyboard.h"
#include "SoSprite.h"


class SoConsole : public NeConsole
{
	public:
		//-----------------------
		// public enums
		enum STATE
		{
			STATE_HIDING,
			STATE_TRANSITION_IN_INIT,
			STATE_TRANSITION_IN,
			STATE_SHOWING,
			STATE_TRANSITION_OUT,
			STATE_LAST
		};


		//-----------------------
		// public functions
		SoConsole();
		virtual ~SoConsole();
		void Init( CCNode* pParentNode );
		void Deinit();
		virtual void Update( float dt );

		virtual void AddOutputString( const std::string& s );
		virtual void SetCurrentCommandLine( const std::string& s );
		virtual void SetCurrentSuggestion( const std::string& s );
		virtual void ClearConsole();

		bool IsShowing() { return _eCurrentState == STATE_SHOWING; }

		// The console might be "showing", but yet be "invisible".
		bool IsVisible() { return _bIsVisible; }
		void SetVisible( bool b );

		void SetPosition( float fBottomLeftX, float fBottomLeftY );

		const std::string& GetFontName() { return _sFontName; }
		void SetFontName();

		void SetCurrentTextColor( const ccColor3B& Color ) { _CurrentTextColor = Color; }

		void SetMaxOutputLines( int i ) { _HudLog.SetMaxMessagesBeforeFifoRemoval( i ); }

		SoHudLog* GetHudLog() { return &_HudLog; }

		void WriteFullHistoryToDisk() { _HudLog.WriteFullHistoryToDisk(); }
		void ClearFullHistory() { _HudLog.ClearFullHistory(); }

		#if DCG_IS_TARGET_WIN32
			// SoInputKeyboardListener callbacks
			virtual void OnKeyPressed( OIS::KeyCode Key );
			virtual void OnKeyReleased( OIS::KeyCode Key );
		#endif

		virtual void OnVirtualKeyPressed( char cKey );
		virtual void OnVirtualKeyReleased( char cKey );
		virtual void OnVirtualKeyDeleteBackward();

		// SoInputGesture callbacks
		virtual bool OnGestureDown( SoGesture* pGesture );
		virtual bool OnGestureUp( SoGesture* pGesture );
		virtual bool OnGestureMove( SoGesture* pGesture );

		int GetDebugSkipToLevelNumber() { return _iDebugSkipToLevelNumber; }
		void SetDebugSkipToLevelNumber( int i ) { _iDebugSkipToLevelNumber = i; }


	private:

		//-----------------------
		// private variables
		float _fTimer;
		STATE _eCurrentState;

		#if DCG_IS_TARGET_WIN32
			SoInputKeyboardListener< SoConsole > _InputKeyboardListener;
		#endif

		SoInputGestureListener< SoConsole > _InputGestureListener;
		SoInputVirtualKeyboardListener< SoConsole > _InputVirtualKeyboardListener;

		CCNode* _pParentNode;

		SoSprite* _pBackground;

		SoHudLog _HudLog;

		SoLabelBmFont* _pCurrentCommandLine;
		SoLabelBmFont* _pCurrentSuggestion;
		SoLabelBmFont* _pCursor;

		std::string _sFontName;
		ccColor3B _CurrentTextColor;

		bool _bIsVisible;

		JpVector2f _vCurrentPosition;
		JpVector2f _vTargetShowingPosition;
		JpVector2f _vTargetHidingPosition;

		JpVector2f _vBackgroundCurrentPosition;
		JpVector2f _vBackgroundTargetShowingPosition;
		JpVector2f _vBackgroundTargetHidingPosition;

		bool _bCursorBlinkFlag;
		float _fCursorBlinkTimer;

		int _iDebugSkipToLevelNumber;


		//-----------------------
		// private functions
		SoLabelBmFont* CreateBmFontLabel( const std::string& sText, const std::string& sFontName );

		void UpdateCursor( float dt );
};






class SoConsoleFunctions
{
	public:
		static void ToggleDisplayStats( NeConsole* pInstance, const std::vector<std::string> & args );
		static void Quit( NeConsole* pInstance, const std::vector<std::string> & args );
		static void SetSpriteZoom( NeConsole* pInstance, const std::vector<std::string> & args );
		static void DebugSave( NeConsole* pInstance, const std::vector<std::string> & args );
		static void DebugLoad( NeConsole* pInstance, const std::vector<std::string> & args );
		static void InvalidateSaveData( NeConsole* pInstance, const std::vector<std::string> & args );
		static void DebugPrintActorCounts( NeConsole* pInstance, const std::vector<std::string> & args );
		static void SetTextureScaler( NeConsole* pInstance, const std::vector<std::string> & args );
		static void GetTextureScaler( NeConsole* pInstance );
		static void SetTextureFilter( NeConsole* pInstance, const std::vector<std::string> & args );
		static void GetTextureFilter( NeConsole* pInstance );
		static void GetTextureDump( NeConsole* pInstance );
		static void SetLightOverlayBlendMode( NeConsole* pInstance, const std::vector<std::string> & args );
		static void ListLightOverlayBlendModes( NeConsole* pInstance );

		static void CreateServer( NeConsole* pInstance, const std::vector<std::string> & args );
		static void AddClients( NeConsole* pInstance, const std::vector<std::string> & args );
		static void StartGame( NeConsole* pInstance, const std::vector<std::string> & args );
		static void EasyStartGameMacro( NeConsole* pInstance, const std::vector<std::string> & args );

		static void ShowTilesForPlayer( NeConsole* pInstance, const std::vector<std::string> & args );

		static void GiveMoneyToPlayer( NeConsole* pInstance, const std::vector<std::string> & args );

		static void GiveTileToPlayer( NeConsole* pInstance, const std::vector<std::string> & args );
		static void GiveAllTilesToPlayer( NeConsole* pInstance, const std::vector<std::string> & args );

		static void NetCurlTest( NeConsole* pInstance, const std::vector<std::string> & args );
		static void NetCurlTestGet( NeConsole* pInstance, const std::vector<std::string> & args );
		static void NetCurlTestPost( NeConsole* pInstance, const std::vector<std::string> & args );

		static void AddNotice( NeConsole* pInstance, const std::vector<std::string> & args );

		static bool bDisplayStats;

		static float fVolumeMusic;
		static float fVolumeEffects;

		static float fA;
		static float fB;
		static float fC;
		static float fX;
		static float fY;

	public:
		static hash GetServerId();
};
