/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "curl/curl.h"

#include "JpList.h"


#define CURL_URL_MAIN_PHP  "http://www.strangejacket.com/Achoir.php"
#define CURL_URL_CHAT_PHP  "http://www.strangejacket.com/AchoirChat.php"


USING_NS_CC;


class SoCurl
{
	public:

		//-----------------------
		// public enum and classes
		class Listener
		{
			public:
				Listener() {}
				virtual ~Listener() {};
				virtual void OnCurlDataReceived( const std::string& sData ) {}
		};



		//-----------------------
		// public functions
		virtual ~SoCurl();

		void Init();
		void Deinit();

		void Update( float dt );


		void DoPost( const char* pacPostData );
		void DoGet( const char* pacGetData );

		void CurlTest();
		void CurlTestGet();
		void CurlTestPost();


		void AddListener( Listener* pListener );
		void RemoveListener( Listener* pListener );
		void RemoveAllListeners();
		void BroadcastDataToListeners( const std::string& sData );

		bool IsReadyForNextQuery() { return _pCurlHandle == NULL; }

		void SetUrlToPhpFile( const std::string& s ) { _sUrlToPhpFile = s; }


		static SoCurl* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();


	private:

		//-----------------------
		// private variables

		static SoCurl* _pInstance;

		JpList< Listener > _ListenerList;
		bool _bListenerListLocked;

		int _iNumRunningOperations;

		CURL* _pCurlHandle;
		CURLM* _pMultiHandle; 

		std::string _sUrlToPhpFile;

		std::string _sPostData;
		std::string _sGetUrl;

		//-----------------------
		// private functions
		SoCurl();

		static size_t CallbackOnWrite( char* pacBuffer, size_t iSizeOfType, size_t iSizeOfBuffer, void* pUserData );

		void CheckResultCurl( CURLcode Result );
		void CheckResultCurlM( CURLMcode Result );
};



template< class TYPE >
class SoCurlListener : public SoCurl::Listener
{
	public:
		//-----------------------
		// public functions
		
		SoCurlListener() : _pOwner( NULL ) {}
		~SoCurlListener() {}
		
		void SetOwner( TYPE* p ) { _pOwner = p; }
		
		virtual void OnCurlDataReceived( const std::string& sData )
		{
			_pOwner->OnCurlDataReceived( sData );
		}
		
	
	private:
		
		//-----------------------
		// private variables.
		TYPE* _pOwner;

		
		//-----------------------
		// private functions.
		SoCurlListener( SoCurlListener& Copy )
		{
			ASSERTF( 0, ( "\nCannot copy instances of this class" ) );
		}
};












