/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpArray.h"
#include "JpString.h"
#include "SoCodec.h"
#include "SoDebug.h"
#include "SoGameState.h"

#include "mmgrChooser.h"


USING_NS_CC;


int SoDebug::_iPlayerWithDebugHightlightEnabled = PLAYER_UNDEFINED_PLAYER_NUMBER;


/******************************************************************************
 FUNCTION:  SoDebug
******************************************************************************/
SoDebug::SoDebug()
{
}



/******************************************************************************
 FUNCTION:  ~SoDebug
******************************************************************************/
SoDebug::~SoDebug()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoDebug::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoDebug::Deinit()
{
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoDebug::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  HandleDebugMessage
******************************************************************************/
void SoDebug::HandleDebugMessage( SoGameState* pGame, SoGameMessage* pMessage )
{
	SoCodecDebugCommand Decoder;
	Decoder.Decode( pMessage );
	int iMessageId = Decoder._iDebugMessageId;
	int iPlayerNumber = Decoder._iPlayerNumber;

	switch ( iMessageId )
	{
		case DEBUG_MESSAGE_ID_GIVE_PLAYER_TILE:
		{
			SoTileSet* pTileSet = pGame->GetTileSet();
			SoGameBitTile* pTile = pTileSet->DrawNextTile();
			pTile->SetOwnerPlayerNumber( iPlayerNumber );
			break;
		}

		case DEBUG_MESSAGE_ID_GIVE_PLAYER_ALL_TILES:
		{
			SoTileSet* pTileSet = pGame->GetTileSet();

			for ( int i = 0; i < pTileSet->GetNumTiles(); i ++ )
			{
				SoGameBitTile* pTile = pTileSet->GetTileById( i );

				if ( !pTile->IsPlaced() )
					pTile->SetOwnerPlayerNumber( iPlayerNumber );
			}

			break;
		}

		case DEBUG_MESSAGE_ID_GIVE_PLAYER_MONEY:
		{
			SoGameBitPlayer* pPlayer = pGame->GetPlayerByPlayerNumber( iPlayerNumber );
			pPlayer->AdjustMoney( 1 );
			break;
		}

		default:
			ASSERTF( 0, ( "\nUnknown debug message" ) );
	}
}



/******************************************************************************
 FUNCTION:  CreatePrettyGameMessageString
******************************************************************************/
void SoDebug::CreatePrettyGameMessageString( std::string* psResult, SoGameMessage* pMessage )
{
	Msg::TYPE eType = pMessage->GetType();
	const char* pacType = Msg::GetStringFromType( eType );

	*psResult += pacType;

	switch ( eType )
	{
		case Msg::TYPE_CLI_REQUESTS_ADD_TO_SVR:
			break;

		case Msg::TYPE_SVR_COMMAND_ADD_CLI:
		{
			SoCodecSvrCommandAddCli Codec;
			Codec.Decode( pMessage );
			*psResult += ", _hRngSeed: ";
			*psResult += JpString::HashToString( Codec._hRngSeed );
			break;
		}

		case Msg::TYPE_CLI_REQUESTS_SEAT:
		{
			SoCodecCliRequestsSeat Codec;
			Codec.Decode( pMessage );
			*psResult += ", _eGameBitTypeOfNewPlayer: ";
			*psResult += JpString::IntToString( (int)Codec._eGameBitTypeOfNewPlayer );
			break;
		}

		case Msg::TYPE_SVR_COMMAND_SEAT_ASSIGNMNET:
		case Msg::TYPE_SVR_COMMAND_START_GAME:
		case Msg::TYPE_SVR_COMMAND_INIT_NEW_GAME:
		case Msg::TYPE_SVR_COMMAND_PLYR_TAKE_TURN:
		{
			ASSERTF( 0, ( "\nPrettyPrint not yet supported for this message types." ) );
			break;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_PURCHASE_STOCK:
		case Msg::TYPE_SVR_COMMAND_PLYR_PICK_NEW_HOTEL:
		case Msg::TYPE_SVR_COMMAND_PLYR_PICK_MERGE_HOTELS:
		case Msg::TYPE_SVR_COMMAND_PLYR_MANAGE_OLD_STOCK:
		{
			break;
		}

		case Msg::TYPE_PLYR_MOVE_PLACE_TILE:
		{
			SoCodecPlyrMovePlaceTile Codec;
			Codec.Decode( pMessage );
			*psResult += ", _iPlayerNumber: ";
			*psResult += JpString::IntToString( Codec._iPlayerNumber );
			*psResult += ", _iPlacedTileId: ";
			*psResult += JpString::IntToString( Codec._iPlacedTileId );
			*psResult += ", _bSkippingTilePlacement: ";

			if ( Codec._bSkippingTilePlacement )
				*psResult += "T";
			else
				*psResult += "F";

			break;
		}

		case Msg::TYPE_PLYR_MOVE_STOCK_PURCHASES:
		{
			SoCodecPlyrMoveStockPurchases Codec;
			Codec.Decode( pMessage );
			*psResult += ", _iPlayerNumber: ";
			*psResult += JpString::IntToString( Codec._iPlayerNumber );
			*psResult += ", AmtOfHotel0: ";
			*psResult += JpString::IntToString( Codec._iAmountOfHotelWithId0 );
			*psResult += ", AmtOfHotel1: ";
			*psResult += JpString::IntToString( Codec._iAmountOfHotelWithId1 );
			*psResult += ", AmtOfHotel2: ";
			*psResult += JpString::IntToString( Codec._iAmountOfHotelWithId2 );
			*psResult += ", AmtOfHotel3: ";
			*psResult += JpString::IntToString( Codec._iAmountOfHotelWithId3 );
			*psResult += ", AmtOfHotel4: ";
			*psResult += JpString::IntToString( Codec._iAmountOfHotelWithId4 );
			*psResult += ", AmtOfHotel5: ";
			*psResult += JpString::IntToString( Codec._iAmountOfHotelWithId5 );
			*psResult += ", AmtOfHotel6: ";
			*psResult += JpString::IntToString( Codec._iAmountOfHotelWithId6 );

			break;
		}
		case Msg::TYPE_PLYR_MOVE_NEW_HOTEL_PICK:
		{
			SoCodecPlyrMoveNewHotelPick Codec;
			Codec.Decode( pMessage );
			*psResult += ", _iPlayerNumber: ";
			*psResult += JpString::IntToString( Codec._iPlayerNumber );
			*psResult += ", _iHotelId: ";
			*psResult += JpString::IntToString( Codec._iHotelId );
			break;
		}
		case Msg::TYPE_PLYR_MOVE_MERGE_HOTELS:
		{
			SoCodecPlyrMoveMergeHotels Codec;
			Codec.Decode( pMessage );
			*psResult += ", _iPlayerNumber: ";
			*psResult += JpString::IntToString( Codec._iPlayerNumber );
			*psResult += ", _iSurvivorHotelId: ";
			*psResult += JpString::IntToString( Codec._iSurvivorHotelId );
			*psResult += ", _iVictimHotelId: ";
			*psResult += JpString::IntToString( Codec._iVictimHotelId );
			break;
		}
		case Msg::TYPE_PLYR_MOVE_MANAGE_OLD_STOCK:
		{
			SoCodecPlyrManageOldStock Codec;
			Codec.Decode( pMessage );
			*psResult += ", _iPlayerNumber: ";
			*psResult += JpString::IntToString( Codec._iPlayerNumber );
			*psResult += ", _iNumStocksToKeep: ";
			*psResult += JpString::IntToString( Codec._iNumStocksToKeep );
			*psResult += ", _iNumStocksToSell: ";
			*psResult += JpString::IntToString( Codec._iNumStocksToSell );
			*psResult += ", _iNumStocksToTrade: ";
			*psResult += JpString::IntToString( Codec._iNumStocksToTrade );
			break;
		}
		case Msg::TYPE_PLYR_MOVE_DECLARE_LAST_TURN:
		{
			SoCodecPlyrMoveDeclareLastTurn Codec;
			Codec.Decode( pMessage );
			*psResult += ", _iPlayerNumber: ";
			*psResult += JpString::IntToString( Codec._iPlayerNumber );
			break;
		}

		case Msg::TYPE_PLYR_MOVE_UNDO:
		{
			SoCodecPlyrMoveUndo Codec;
			Codec.Decode( pMessage );

			*psResult += ", _iPlayerNumber: ";
			*psResult += JpString::IntToString( Codec._iPlayerNumber );

			break;
		}

		case Msg::TYPE_PLYR_MOVE_RESTART_TURN:
		{
			SoCodecPlyrMoveRestartTurn Codec;
			Codec.Decode( pMessage );

			*psResult += ", _iPlayerNumber: ";
			*psResult += JpString::IntToString( Codec._iPlayerNumber );

			break;
		}

		case Msg::TYPE_PLYR_MOVE_END_TURN:
		{
			SoCodecPlyrMoveEndTurn Codec;
			Codec.Decode( pMessage );

			*psResult += ", _iPlayerNumber: ";
			*psResult += JpString::IntToString( Codec._iPlayerNumber );

			break;
		}


		case Msg::TYPE_DEBUG_COMMAND:
		case Msg::TYPE_LAST:
		{
			ASSERTF( 0, ( "\nPrettyPrint not yet supported for this message types." ) );
			break;
		}

		default: ASSERTF( 0, ( "\nMessage Type not found." ) );
	}
}
