/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoDrawCustom.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoDrawCustom
******************************************************************************/
SoDrawCustom::SoDrawCustom() :
_pParentNode( NULL ),
_bIsActive( false ),
_fBlipLineWindowWidth( 100.0f ),
_fSampleLineWindowWidth( 100.0f )
{
	_BlipLineColor = JpUtils::_Blue;
	_SampleLineColor = JpUtils::_Blue;
}



/******************************************************************************
 FUNCTION:  ~SoDrawCustom
******************************************************************************/
SoDrawCustom::~SoDrawCustom()
{
}



/******************************************************************************
 FUNCTION:  Setup
******************************************************************************/
void SoDrawCustom::Setup( CCNode* pParentNode )
{
	SetParentNode( pParentNode );
	pParentNode->addChild( this, 0 );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoDrawCustom::Deinit()
{
	if ( _pParentNode )
	{
		_pParentNode->removeChild( this, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pParentNode = NULL;
	}

	ClearBlipLine();
	ClearSampleLine();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoDrawCustom::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  draw
******************************************************************************/
void SoDrawCustom::draw()
{
	if ( _bIsActive )
	{
		DrawBlipLine();
		DrawSampleLine();
	}
}



/******************************************************************************
 FUNCTION:  DrawBlipLine
******************************************************************************/
void SoDrawCustom::DrawBlipLine()
{
	SoPointList::Iterator Iter( &_BlipLine );

	CCPoint* pP1 = Iter.First();
	CCPoint* pP2 = Iter.Next();

	float fX = 0.0f;
	float fIncrement = -10.0f * SoUtils::GetiPadRatioX();

	while ( pP1 && pP2 && fabsf( fX ) < _fBlipLineWindowWidth )
	{
	    glLineWidth( 8.0f );
		ccDrawColor4B
		( 
			_BlipLineColor.r * 0.9f, 
			_BlipLineColor.g * 0.9f,	
			_BlipLineColor.b * 0.9f, 
			//_BlipLineColor.r, 
			//_BlipLineColor.g,	
			//_BlipLineColor.b, 
			128 
		);

		ccDrawLine( ccp( fX, pP1->y ), ccp( fX + fIncrement, pP2->y ) );

	    glLineWidth( 4.0f );
		ccDrawColor4B
		( 
			//_BlipLineColor.r, 
			//_BlipLineColor.g,	
			//_BlipLineColor.b, 
			JpUtils::_White.r,
			JpUtils::_White.g,
			JpUtils::_White.b,
			200 
		);
		ccDrawLine( ccp( fX, pP1->y ), ccp( fX + fIncrement, pP2->y ) );

		fX += fIncrement;
		pP1 = pP2;
		pP2 = Iter.Next();
	}


	// Remove any points off the end of the list
	// that weren't drawn above.
	while ( pP2 )
	{
		_BlipLine.Remove( pP2 );
		delete pP2;
		pP2 = Iter.Next();
	}
}



/******************************************************************************
 FUNCTION:  DrawSampleLine
******************************************************************************/
void SoDrawCustom::DrawSampleLine()
{
	SoPointList::Iterator Iter( &_SampleLine );

	CCPoint* pP1 = Iter.First();
	CCPoint* pP2 = Iter.Next();

	float fX = 0.0f;
	float fIncrement = -10.0f * SoUtils::GetiPadRatioX();

	while ( pP1 && pP2 && fabsf( fX ) < _fSampleLineWindowWidth )
	{
	    glLineWidth( 8.0f );
		ccDrawColor4B
		( 
			_SampleLineColor.r * 0.9f, 
			_SampleLineColor.g * 0.9f,	
			_SampleLineColor.b * 0.9f, 
			//_SampleLineColor.r, 
			//_SampleLineColor.g,	
			//_SampleLineColor.b, 
			128 
		);

		ccDrawLine( ccp( fX, pP1->y ), ccp( fX + fIncrement, pP2->y ) );

	    glLineWidth( 4.0f );
		ccDrawColor4B
		( 
			//_SampleLineColor.r, 
			//_SampleLineColor.g,	
			//_SampleLineColor.b, 
			JpUtils::_White.r,
			JpUtils::_White.g,
			JpUtils::_White.b,
			200 
		);
		ccDrawLine( ccp( fX, pP1->y ), ccp( fX + fIncrement, pP2->y ) );

		fX += fIncrement;
		pP1 = pP2;
		pP2 = Iter.Next();
	}


	// Remove any points off the end of the list
	// that weren't drawn above.
	while ( pP2 )
	{
		_SampleLine.Remove( pP2 );
		delete pP2;
		pP2 = Iter.Next();
	}
}



/******************************************************************************
 FUNCTION:  AddPointToBlipLine
******************************************************************************/
void SoDrawCustom::AddPointToBlipLine( float fY )
{
	CCPoint* pNewPoint = new CCPoint;
	pNewPoint->x = 0.0f;
	pNewPoint->y = fY;
	_BlipLine.AddFirst( pNewPoint );	
}



/******************************************************************************
 FUNCTION:  AddPointToSampleLine
******************************************************************************/
void SoDrawCustom::AddPointToSampleLine( float fY )
{
	CCPoint* pNewPoint = new CCPoint;
	pNewPoint->x = 0.0f;
	pNewPoint->y = fY;
	_SampleLine.AddFirst( pNewPoint );	
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoDrawCustom* SoDrawCustom::create()
{
	SoDrawCustom* pRet = new SoDrawCustom();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}
