/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoDrawDebug.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoDrawDebug
******************************************************************************/
SoDrawDebug::SoDrawDebug() :
_pParentNode( NULL ),
_bIsActive( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoDrawDebug
******************************************************************************/
SoDrawDebug::~SoDrawDebug()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Setup
******************************************************************************/
void SoDrawDebug::Setup( CCNode* pParentNode )
{
	SetParentNode( pParentNode );
	pParentNode->addChild( this, 0 );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoDrawDebug::Deinit()
{
	if ( _pParentNode )
	{
		_pParentNode->removeChild( this, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pParentNode = NULL;
	}

	_ShapeList.DeleteAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoDrawDebug::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  AddBox
******************************************************************************/
void SoDrawDebug::AddBox( float fX, float fY, float fWidth, float fHeight )
{
	ShapeBox* pNewBox = new ShapeBox;
	pNewBox->SetPosition( fX, fY );
	pNewBox->SetWidth( fWidth );
	pNewBox->SetHeight( fHeight );
	_ShapeList.AddLast( pNewBox );
}



/******************************************************************************
 FUNCTION:  draw
******************************************************************************/
void SoDrawDebug::draw()
{
	if ( _bIsActive )
	{
		SoDebugShapeList::Iterator Iter( &_ShapeList );
		while ( Iter.Next() )
		{
			Shape* pShape = Iter.GetNodeData();
			pShape->draw();
		}
	}
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoDrawDebug* SoDrawDebug::create()
{
	SoDrawDebug* pRet = new SoDrawDebug();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}











/******************************************************************************
 FUNCTION:  ShapeBox::ShapeBox
******************************************************************************/
SoDrawDebug::ShapeBox::ShapeBox() :
_fX( 0.0f ),
_fY( 0.0f ),
_fWidth( 0.0f ),
_fHeight( 0.0f )
{
}



/******************************************************************************
 FUNCTION:  ShapeBox::draw
******************************************************************************/
void SoDrawDebug::ShapeBox::draw()
{
    ccDrawColor4B(255, 255, 0, 255);
    glLineWidth(3);

	float fHalfWidth = _fWidth / 2.0f;
	float fHalfHeight = _fHeight / 2.0f;

    CCPoint vertices[] = 
	{ 
		ccp( -fHalfWidth, fHalfHeight ), 
		ccp( fHalfWidth, fHalfHeight ), 
		ccp( fHalfWidth, -fHalfHeight ), 
		ccp( -fHalfWidth, -fHalfHeight )
	};

    ccDrawPoly( vertices, 4, DCG_CLOSE_POLYGON);
}
