/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFontDefs.h"
#include "SoLabelBmFont.h"

#include "mmgrChooser.h"


USING_NS_CC;


JpList< SoLabelBmFont > SoFontDefs::_PreLoadedFontsList;




const char* SoFontDefs::GetTestLarge() { return "StringLiteral_Str437_32pt.fnt"; }
const char* SoFontDefs::GetTestMedium() { return "StringLiteral_Str437_20pt.fnt"; }
const char* SoFontDefs::GetTestSmall() { return "StringLiteral_Str437_12pt.fnt"; }
const char* SoFontDefs::GetNameTextEntry() { return "StringLiteral_Str437_12pt.fnt"; }
const char* SoFontDefs::GetHudLog() { return "old_sans_black_30.fnt"; }

#if DCG_IS_TARGET_ANDROID
	const char* SoFontDefs::GetWinLog() { return "old_sans_black_40.fnt"; }
#elif DCG_IS_TARGET_IOS
	const char* SoFontDefs::GetWinLog() { return "old_sans_black_40.fnt"; }
#else
	const char* SoFontDefs::GetWinLog() { return "old_sans_black_30.fnt"; }
#endif

const char* SoFontDefs::GetKicker() { return "old_sans_black_20.fnt"; }
const char* SoFontDefs::GetHudGold() { return "Rumpelstiltskin32_extended.fnt"; }
const char* SoFontDefs::GetHudPlayerGuide() { return "old_sans_black_20.fnt"; }
const char* SoFontDefs::GetHudPlayerGuideScore() { return "old_sans_black_30.fnt"; }

const char* SoFontDefs::GetHudTurns() { return "old_sans_black_30.fnt"; }

const char* SoFontDefs::GetGuiDefault() { return "old_sans_black_30.fnt"; }
const char* SoFontDefs::GetGuiDefaultMedium() { return "old_sans_black_35.fnt"; }
const char* SoFontDefs::GetGuiDefaultLarge() { return "old_sans_black_48.fnt"; }
const char* SoFontDefs::GetVisibleDamageMultiplier() { return "old_sans_black_48.fnt"; }

const char* SoFontDefs::GetCredits() { return "old_sans_black_20.fnt"; }
const char* SoFontDefs::GetCreditsMedium() { return "old_sans_black_30.fnt"; }
const char* SoFontDefs::GetCreditsLarge() { return "old_sans_black_40.fnt"; }

const char* SoFontDefs::GetBoardSpot() { return "old_sans_black_40.fnt"; }
const char* SoFontDefs::GetHotelStats() { return "old_sans_black_20.fnt"; }

const char* SoFontDefs::GetWatermark() { return "old_sans_black_20.fnt"; }





/******************************************************************************
 FUNCTION:  PreloadFont
******************************************************************************/
void SoFontDefs::PreloadFont( const char* pFontName )
{
	if ( !FontHasBeenPreloaded( pFontName ) )
		_PreLoadedFontsList.AddLast( PerformActualPreload( pFontName ) );
}




/******************************************************************************
 FUNCTION:  DestroyPreloadedFonts
******************************************************************************/
void SoFontDefs::DestroyPreloadedFonts()
{
	JpList< SoLabelBmFont >::Iterator Iter( &_PreLoadedFontsList );
	while ( Iter.Next() )
	{
		SoLabelBmFont* pLabel = Iter.GetNodeData();
		delete pLabel;
	}

	_PreLoadedFontsList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  FontHasBeenPreloaded
******************************************************************************/
bool SoFontDefs::FontHasBeenPreloaded( const char* pFontName )
{
	JpList<SoLabelBmFont>::Iterator Iter( &_PreLoadedFontsList );

	while ( Iter.Next() )
	{
		SoLabelBmFont* pFont = Iter.GetNodeData();

		if ( strcmp( pFont->getFntFile(), SoUtils::AddRelativePathToFont( pFontName ) ) == 0 )
			return true;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  PerformActualPreload
******************************************************************************/
SoLabelBmFont* SoFontDefs::PerformActualPreload( std::string sFontName )
{
	SoLabelBmFont* pLabel = SoLabelBmFont::create
	(
		".", 
		SoUtils::AddRelativePathToFont( sFontName.c_str() ), 
		100, 
		kCCTextAlignmentCenter
	);
	pLabel->retain();

	return pLabel;
}



/******************************************************************************
 FUNCTION:  HighScoreFontsAreLoaded
******************************************************************************/
bool SoFontDefs::HighScoreFontsAreLoaded()
{
	if  ( 
		FontHasBeenPreloaded( "Rumpelstiltskin32_extended.fnt" )  &&
		FontHasBeenPreloaded( "Rumpelstiltskin40_extended.fnt" )  &&
		FontHasBeenPreloaded( "Rumpelstiltskin50_extended.fnt" )
		)
		return true;
	else
		return false;
}



