/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFrame.h"

#include "mmgrChooser.h"




/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFrame::Init()
{
	_papElements = NULL;
	_iNumElements = 0;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFrame::Deinit()
{
	if ( _papElements )
	{
		for ( int i = 0; i < _iNumElements; i ++ )
		{
			Element* pElement = GetElementByIndex( i );
			delete pElement;
		}

		delete[] _papElements;
		_papElements = NULL;
	}

	_RoomDataList.DeleteAll();
}



/******************************************************************************
 FUNCTION:  Create
******************************************************************************/
void SoFrame::Create( int iWidth, int iHeight )
{
	_iWidth = iWidth;
	_iHeight = iHeight;
	_iNumElements = _iWidth * _iHeight;

	if ( _papElements == NULL )
	{
		_papElements = new Element*[ _iNumElements ];
	}

	for ( int i = 0; i < _iNumElements; i ++ )
	{
		Element* p = new Element;
		_papElements[ i ] = p;
	}
}



/******************************************************************************
 FUNCTION:  GetElementByXz
******************************************************************************/
SoFrame::Element* SoFrame::GetElementByXz( int iIndexX, int iIndexY )
{
	if 
	( 
		iIndexX >= 0 && iIndexX < _iWidth &&
		iIndexY >= 0 && iIndexY < _iHeight
	)
	{
		Element* pElement = _papElements[ ( iIndexY * _iWidth ) + iIndexX ];
		return pElement;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  GetElementByIndex
******************************************************************************/
SoFrame::Element* SoFrame::GetElementByIndex( int i )
{
	if ( i >= 0 && i < _iNumElements )
		return _papElements[ i ];

	return NULL;
}



/******************************************************************************
 FUNCTION:  IsXzAdjacentToFlag
 NOTE:      This function only tests adjacent tiles, and DOES NOT TEST the
            tile at iX, iZ (the tile at iX, iZ is not *adjacent* to iX,iZ...
            it *is* iX, iZ).
            Certain uses of this function depend on this behavior!
            mdomowicz 2011_09_09
******************************************************************************/
bool SoFrame::IsXzAdjacentToFlag( int iX, int iZ, Element::BIT_FLAG e )	
{
	int iDist = 1;

	for ( int z = iZ - iDist; z <= iZ + iDist; z ++ )
	{
		for ( int x = iX - iDist; x <= iX + iDist; x ++ )
		{
			if ( x != iX || z != iZ )
			{
				SoFrame::Element* pElement = GetElementByXz( x, z );
				if ( pElement && pElement->IsBitFlagSet( e ) )
					return true;			
			}
		}
	}

	return false;
}








/******************************************************************************
 FUNCTION:  RoomData::SetRect
******************************************************************************/
void SoFrame::RoomData::SetRect( int iX, int iZ, int iWidth, int iHeight )
{
	_iX = iX;
	_iZ = iZ;
	_iWidth = iWidth;
	_iHeight = iHeight;
}
