/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "JpList.h"


USING_NS_CC;


#define RoomDataList JpList< SoFrame::RoomData >

class SoFrame
{
	public:

		//-----------------------
		// public classes
		class Element
		{
			public:
				enum BIT_FLAG
				{
					BIT_FLAG_UNUSED      = 1 << 0,
					BIT_FLAG_DOOR        = 1 << 1,
					BIT_FLAG_WALL        = 1 << 2,
					BIT_FLAG_FLOOR       = 1 << 3,
					BIT_FLAG_ROOM        = 1 << 4,
					BIT_FLAG_HALL        = 1 << 5,
					BIT_FLAG_ROAD        = 1 << 6,

					BIT_FLAG_DOOR_FACING_NORTH     = 1 << 10,
					BIT_FLAG_DOOR_FACING_SOUTH     = 1 << 11,
					BIT_FLAG_DOOR_FACING_WEST      = 1 << 12,
					BIT_FLAG_DOOR_FACING_EAST      = 1 << 13,

					BIT_FLAG_LAST        = 0
				};

				Element() { Init(); }
				virtual ~Element() { Deinit(); }

				void Init() { ExclusiveSetBitFlag( BIT_FLAG_UNUSED ); }
				void Deinit() {};

				int GetBitFlags() { return _iBitFlags; }
				void SetBitFlags( int i ) { _iBitFlags = i; }

				void ClearBitFlags() { _iBitFlags = 0; }
				bool AreBitFlagsClear() { return _iBitFlags == 0; }

				void SetBitFlag( BIT_FLAG e ) { _iBitFlags |= e; }
				void ClearBitFlag( BIT_FLAG e ) { _iBitFlags &= ~e; }

				void ExclusiveSetBitFlag( BIT_FLAG e ) 
				{
					ClearBitFlags(); 
					SetBitFlag( e ); 
				}

				bool IsBitFlagSet( BIT_FLAG e ) { return ( _iBitFlags & e ) > 0; }
				bool AreAnyOfTheseBitFlagsSet( int iBitFlags ) { return ( _iBitFlags & iBitFlags ) > 0; }
				bool AreAllOfTheseBitFlagsSet( int iBitFlags ) { return ( _iBitFlags & iBitFlags ) == iBitFlags; }
				bool AreOnlyTheseBitFlagsSet( int iBitFlags ) { return ( _iBitFlags == iBitFlags ); }

			private:

				int _iBitFlags;
		};


		class RoomData
		{
			public:
				RoomData() {};
				~RoomData() {};

				void SetRect( int iX, int iZ, int iWidth, int iHeight );
				
				int GetX() { return _iX; }
				int GetZ() { return _iZ; }
				int GetWidth() { return _iWidth; }
				int GetHeight() { return _iHeight; }

			private:
				int _iX;
				int _iZ;
				int _iWidth;
				int _iHeight;
		};


		//-----------------------
		// public functions
		SoFrame() { Init(); }
		virtual ~SoFrame() { Deinit(); }

		void Init();
		void Deinit(); 

		void Create( int iWidth, int iHeight );

		int GetWidth() { return _iWidth; }
		int GetHeight() { return _iHeight; }

		Element* GetElementByXz( int iIndexX, int iIndexZ );
		Element* GetElementByIndex( int i );

		bool IsXzAdjacentToFlag( int iX, int iZ, Element::BIT_FLAG e );


		RoomDataList* GetRoomDataList() { return &_RoomDataList; }

		void AddRoomData( RoomData* p ) { _RoomDataList.AddLast( p ); }


	private:

		Element** _papElements;

		int _iWidth;
		int _iHeight;
		int _iNumElements;

		RoomDataList _RoomDataList;
};
