/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include <string.h>

#include "JpDebug.h"
#include "SoFsmState.h"
#include "SoUtils.h"

#include "mmgrChooser.h"



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFsmState::Init()
{
	#if DCG_IS_TARGET_WIN32
		_KeyboardListener.SetOwner( this );
		SoInputKeyboard::GetInstance()->AddListener( &_KeyboardListener );
	#endif
}



/******************************************************************************
 FUNCTION:  Deinit
 NOTES:     This reverses the order of deinits - just good practice!
******************************************************************************/
void SoFsmState::Deinit()
{
	// force the deinit of the current state.
	_FsmStateManager.SetState( 0 );

	#if DCG_IS_TARGET_WIN32
		SoInputKeyboard::GetInstance()->RemoveListener( &_KeyboardListener );
	#endif
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmState::Update( float dt )
{
	_FsmStateManager.Update( dt );
	return 0;
}



/******************************************************************************
 FUNCTION:  SoFsmStateManager
******************************************************************************/
SoFsmStateManager::SoFsmStateManager() :
_pCurrentState( NULL ),
_hStateToSwitchToOnNextUpdate( 0 )
{
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmStateManager::Update( float dt ) 
{
	if ( _hStateToSwitchToOnNextUpdate )
	{
		SetState( _hStateToSwitchToOnNextUpdate );
		_hStateToSwitchToOnNextUpdate = 0;
	}

	if ( _pCurrentState )
	{
		return _pCurrentState->Update( dt );
	}
	
	return 0;
};



/******************************************************************************
 FUNCTION:  SetState
******************************************************************************/
void SoFsmStateManager::SetState( hash hKey )
{
	SoFsmState* pState = _StateTable.Find( hKey );

	if ( _pCurrentState != NULL )
		_pCurrentState->Deinit();

	_pCurrentState = pState;

	if ( _pCurrentState != NULL )
		_pCurrentState->Init();
}



/******************************************************************************
 FUNCTION:  StoreState
******************************************************************************/
void SoFsmStateManager::StoreState( hash hKey, SoFsmState* pState )
{
	if ( pState != NULL )
	{
		pState->SetParentFsmStateManager( this );

		if ( _StateTable.Find( hKey ) == NULL )
			_StateTable.Store( hKey, pState );	
	}
}








