/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#if DCG_IS_TARGET_WIN32
    #include "OIS.h"
#endif

#include "JpHashTable.h"

#include "SoInputKeyboard.h"
#include "SoScreenFader.h"
#include "SoSprite.h"


class SoFsmState;


class SoFsmStateManager
{
	public:
		//-----------------------
		// public functions

		SoFsmStateManager();

		~SoFsmStateManager() {}

		void Init() {};

		void Deinit() {};

		hash Update( float dt );

		void SetState( hash hKey );

		// Usually SetState works fine, but sometimes a state cannot be set on the current
		// update.  For those situations we have this...
		void SetStateOnNextUpdate( hash hKey ) { _hStateToSwitchToOnNextUpdate = hKey; }

		SoFsmState* GetCurrentState() { return _pCurrentState; }

		SoFsmState* FindState( hash hKey ) { return _StateTable.Find( hKey ); }

		bool IsCurrentState( hash hKey ) { return ( _pCurrentState == _StateTable.Find( hKey ) ); }

		void StoreState( hash hKey, SoFsmState* pState );

		void RemoveState( hash hKey ) { _StateTable.Remove( hKey ); }


private:

		//-----------------------
		// private variables

		JpHashTable< SoFsmState, 64 > _StateTable;
		SoFsmState* _pCurrentState;
		hash _hStateToSwitchToOnNextUpdate;
};



class SoFsmState
{
	public:

		//-----------------------
		// public functions

		SoFsmState() : _pParentStateManager( NULL ) {};

		virtual ~SoFsmState() {};

		virtual void Init();
		virtual void Deinit();
		virtual hash Update( float dt );
		
		SoFsmStateManager* GetFsmStateManager() { return &_FsmStateManager; }

		void SetParentFsmStateManager( SoFsmStateManager* pStateManager ) {	_pParentStateManager = pStateManager; }
		SoFsmStateManager* GetParentFsmStateManager() {	return _pParentStateManager; }


        #if DCG_IS_TARGET_WIN32
        // SoInputKeyboardListener callbacks
            virtual void OnKeyPressed( OIS::KeyCode Key ) {};
            virtual void OnKeyReleased( OIS::KeyCode Key ) {};
        #endif

    
	protected:

		//-----------------------
		// protected variables
		SoFsmStateManager _FsmStateManager;

        #if DCG_IS_TARGET_WIN32
            SoInputKeyboardListener< SoFsmState > _KeyboardListener;
        #endif

	private:

		//-----------------------
		// private variables
		SoFsmStateManager* _pParentStateManager;
};





