/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFontDefs.h"
#include "SoFsmStateBoot.h"
#include "SoSoundManager.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoFsmStateBoot
******************************************************************************/
SoFsmStateBoot::SoFsmStateBoot() :
_pLayer( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmStateBoot
******************************************************************************/
SoFsmStateBoot::~SoFsmStateBoot()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFsmStateBoot::Init()
{
	SoSoundManager::GetInstance()->LoadMusicTrack( SoUtils::AddRelativePathToSfx( "MusicTrackTitle.ogg" ) );
	SoSoundManager::GetInstance()->PlayMusic();
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmStateBoot::Deinit()
{
	if ( _pLayer )
	{
		_pLayer->removeFromParentAndCleanup( true );
		_pLayer->release();
		_pLayer = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmStateBoot::Update( float dt )
{
	if ( !_pLayer )
	{
		_pLayer = SoFsmBootLayer::create();
		_pLayer->retain();
		CCDirector::sharedDirector()->getRunningScene()->addChild(_pLayer);
	}

	if ( _pLayer->IsDone() )
	{
		SoFsmStateManager* pStateMgr = GetParentFsmStateManager();
		pStateMgr->SetStateOnNextUpdate( JpHash( "Shell" ) );
	}

	return 0;
}








/******************************************************************************
 FUNCTION:  SoFsmBootLayer
******************************************************************************/
SoFsmBootLayer::SoFsmBootLayer() :
_bTouched( false ),
_fTimer( 0.0f ),
_eState( eSubState_Last )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmBootLayer
******************************************************************************/
SoFsmBootLayer::~SoFsmBootLayer()
{
	_WatermarkPreRelease.Deinit();
}



/******************************************************************************
 FUNCTION:  init
******************************************************************************/
bool SoFsmBootLayer::init()
{
	if ( CCLayerColor::initWithColor( ccc4(0,0,0,255) ) )
	{
		_bTouched = false;

		CCSize size = CCDirector::sharedDirector()->getWinSize();

		#if DCG_IS_TARGET_IOS
			//SoSprite* pSprite = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "ImgLaunchSplashWin32.png" ) );
			_pSplashSprite1 = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "ImgSplash2.png" ) );
		#else
			_pSplashSprite1 = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "ImgLaunchSplashWin32.png" ) );
		#endif
		
		// place the image in the center of the screen		
		_pSplashSprite1->setPosition( ccp(size.width/2, size.height/2) );

		#if DCG_IS_TARGET_IOS
			_pSplashSprite1->setVisible( false );
		#else
			_pSplashSprite1->setVisible( true );
		#endif

		addChild(_pSplashSprite1, 0);



		_pSplashSprite2 = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "ImgSplash2.png" ) );
	
		// place the image in the center of the screen		
		_pSplashSprite2->setPosition( ccp(size.width/2, size.height/2) );

		// scale the image so that it's stretched to fill the entire screen
		CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
		if ( SizeInPixels.width > SizeInPixels.height )
		{
			float fExactScreenWidth = SizeInPixels.width / _pSplashSprite2->getContentSize().width;
			_pSplashSprite2->setScaleX( fExactScreenWidth );
			_pSplashSprite2->setScaleY( fExactScreenWidth );
		}
		else
		{
			float fExactScreenHeight = SizeInPixels.height / _pSplashSprite2->getContentSize().height;
			_pSplashSprite2->setScaleX( fExactScreenHeight );
			_pSplashSprite2->setScaleY( fExactScreenHeight );
		}

		_pSplashSprite2->setVisible( false );
		addChild(_pSplashSprite2, 0);

		scheduleUpdate();
		setTouchEnabled( true );

		_fTimer = 0.0f;
		_eState = eSubState_Screen1;

		_WatermarkPreRelease.Init( this );

		return true;
	}


	return false;
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoFsmBootLayer::update( float dt )
{
	_fTimer += dt;

	#if DCG_IS_TARGET_IOS
		_eState = eSubState_Done;
	#else
		switch ( _eState )
		{
			case eSubState_Screen1:
			{
				if ( _fTimer > 3.0f || _bTouched )
				{
					_pSplashSprite1->setVisible( false );
					_eState = eSubState_Done;

					/*
					_pSplashSprite2->setVisible( true );
					_bTouched = false;
					_fTimer = 0.0f;
					_eState = eSubState_Screen2;
					*/

				}
				break;
			}
			case eSubState_Screen2:
			{
				if ( _fTimer > 3.0f || _bTouched )
				{
					_bTouched = false;
					_eState = eSubState_Done;
				}
				break;
			}
			case eSubState_Done:
			{
				break;
			}
		}
	
	#endif
}



/******************************************************************************
 FUNCTION:  IsDone
******************************************************************************/
bool SoFsmBootLayer::IsDone()
{
	return ( _eState == eSubState_Done );
}



/******************************************************************************
 FUNCTION:  ccTouchesEnded
******************************************************************************/
void SoFsmBootLayer::ccTouchesEnded(CCSet *pTouches, CCEvent *pEvent)
{
	_bTouched = true;
}



/******************************************************************************
 FUNCTION:  ccTouchesCancelled
******************************************************************************/
void SoFsmBootLayer::ccTouchesCancelled(CCSet *pTouches, CCEvent *pEvent)
{
	ccTouchesEnded(pTouches, pEvent);
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoFsmBootLayer* SoFsmBootLayer::create()
{
	SoFsmBootLayer* pRet = new SoFsmBootLayer();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}
