/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFontDefs.h"
#include "SoFsmStateCredits.h"
#include "SoGuiScrollText.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoFsmStateCredits
******************************************************************************/
SoFsmStateCredits::SoFsmStateCredits() :
_pLayer( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmStateCredits
******************************************************************************/
SoFsmStateCredits::~SoFsmStateCredits()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFsmStateCredits::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmStateCredits::Deinit()
{
	if ( _pLayer )
	{
		_pLayer->removeFromParentAndCleanup( true );
		_pLayer->release();
		_pLayer = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmStateCredits::Update( float dt )
{
	if ( !_pLayer )
	{
		_pLayer = SoFsmCreditsLayer::create();
		_pLayer->retain();
		CCDirector::sharedDirector()->getRunningScene()->addChild(_pLayer);
	}

	if ( _pLayer->IsDone() )
	{
		SoFsmStateManager* pStateMgr = GetParentFsmStateManager();
		pStateMgr->SetStateOnNextUpdate( JpHash( "Shell" ) );
	}

	return 0;
}








/******************************************************************************
 FUNCTION:  SoFsmCreditsLayer
******************************************************************************/
SoFsmCreditsLayer::SoFsmCreditsLayer() :
_bTouched( false ),
_fTimer( 0.0f ),
_eState( eSubState_Last ),
_gGuiScrollText( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmCreditsLayer
******************************************************************************/
SoFsmCreditsLayer::~SoFsmCreditsLayer()
{
}



/******************************************************************************
 FUNCTION:  init
******************************************************************************/
bool SoFsmCreditsLayer::init()
{
	if ( CCLayerColor::initWithColor( ccc4(255,255,255,255) ) )
	{
		_bTouched = false;

		CCSize size = CCDirector::sharedDirector()->getWinSize();

		//--------------------
		// Sprite 1
		_pSplashSprite1 = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "ImgCredits.png" ) );
		
		// place the image in the center of the screen		
		_pSplashSprite1->setPosition( ccp(size.width/2, size.height/2) );

		// scale the image so that it's stretched to fill the entire screen
		CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
		if ( SizeInPixels.width > SizeInPixels.height )
		{
			float fExactScreenWidth = SizeInPixels.width / _pSplashSprite1->getContentSize().width;
			_pSplashSprite1->setScaleX( fExactScreenWidth );
			_pSplashSprite1->setScaleY( fExactScreenWidth );
		}
		else
		{
			float fExactScreenHeight = SizeInPixels.height / _pSplashSprite1->getContentSize().height;
			_pSplashSprite1->setScaleX( fExactScreenHeight );
			_pSplashSprite1->setScaleY( fExactScreenHeight );
		}
		_pSplashSprite1->setVisible( true );



		addChild(_pSplashSprite1, 0);

		scheduleUpdate();
		setTouchEnabled( true );

		_fTimer = 0.0f;
		_eState = eSubState_Screen1;


		_GuiManager.Init( this, SoGuiManager::INPUT_SOURCE_INTERNAL );
		_GuiManager.SetHeadGuiAbsolutionPositionToScreenCenter();

		SoGuiScrollText* pScrollText = new SoGuiScrollText;
		_gGuiScrollText = _GuiManager.AddGui( pScrollText, NULL );

		pScrollText->SetRelativePos( JpVector2f( 0.0f, -250.0f * SoUtils::GetiPadRatioY() ) );
		pScrollText->EnableClipping( true );
		pScrollText->SetClippingRegion
		( 
			-512.0f * SoUtils::GetiPadRatioX(), 
			0.0f, 
			1024.0f * SoUtils::GetiPadRatioX(), 
			500.0f * SoUtils::GetiPadRatioY() 
		);

		pScrollText->SetAutoScrollSpeedInPixelPerSecond( 50.0f * SoUtils::GetiPadRatioY() );

		pScrollText->SetLineAnchorPoint( JpVector2f( 0.5f, 0.0f ) );

		pScrollText->SetDefaultFontTypeA( SoFontDefs::GetCredits() );
		pScrollText->SetDefaultFontTypeB( SoFontDefs::GetCreditsMedium() );
		pScrollText->SetDefaultFontTypeC( SoFontDefs::GetCreditsLarge() );

		pScrollText->AddLineTypeB( "Achoir is dedicated to Sid Sackson," );
		pScrollText->AddLineTypeB( "and inspired by his game design masterpiece, Acquire." );
		pScrollText->AddLineBlankCustomY( 130.0f * SoUtils::GetiPadRatioY() );
		pScrollText->AddLineTypeA( "Software Development" );
		pScrollText->AddLineTypeB( "Domowicz Creative Group, LLC" );
		pScrollText->AddLineBlankCustomY( 50.0f * SoUtils::GetiPadRatioY() );
		pScrollText->AddLineTypeA( "Project Lead" );
		pScrollText->AddLineTypeB( "Mark F. Domowicz" );
		pScrollText->AddLineBlankTypeA();
		pScrollText->AddLineTypeA( "Programmers" );
		pScrollText->AddLineTypeB( "Mark F. Domowicz" );
		pScrollText->AddLineTypeB( "Mark Wimmer" );
		pScrollText->AddLineBlankTypeA();
		pScrollText->AddLineTypeA( "Art" );
		pScrollText->AddLineTypeB( "Randis Albion" );
		pScrollText->AddLineBlankTypeA();
		pScrollText->AddLineTypeA( "Music" );
		pScrollText->AddLineTypeB( "Cameron 'gribbly' Brown" );
		pScrollText->AddLineBlankTypeA();
		pScrollText->AddLineTypeA( "Special Thanks" );
		pScrollText->AddLineTypeB( "Brian Chan" );
		pScrollText->AddLineTypeB( "Geni Bruhin" );
		pScrollText->AddLineTypeB( "Grace Yang" );
		pScrollText->AddLineTypeB( "James S. \"Jim\" Conover" );
		pScrollText->AddLineTypeB( "Jason Block" );
		pScrollText->AddLineTypeB( "Jessica Harry" );
		pScrollText->AddLineTypeB( "Lia Domowicz" );
		pScrollText->AddLineTypeB( "Timo Pihlajamaki" );
		pScrollText->AddLineTypeB( "www.reddit.com/r/boardgames" );
		pScrollText->AddLineTypeB( "www.reddit.com/r/digitaltabletop" );


		#if DCG_IS_TARGET_WIN32
			pScrollText->AddLineBlankCustomY( 130.0f * SoUtils::GetiPadRatioY() );
			pScrollText->AddLineTypeA( "Legal" );
			pScrollText->AddLineBlankCustomY( 30.0f * SoUtils::GetiPadRatioY() );
			pScrollText->AddLineTypeA( "Apple and the Apple logo are trademarks of Apple Inc., registered in the U.S. and other countries." );
			pScrollText->AddLineTypeA( "App Store is a service mark of Apple Inc." );
			pScrollText->AddLineBlankTypeA();
			pScrollText->AddLineTypeA( "Google Play is a trademark of Google Inc." );
		#endif

		pScrollText->AddLineBlankCustomY( 130.0f * SoUtils::GetiPadRatioY() );
		pScrollText->AddLineTypeA( "Technology and Licensing" );
		pScrollText->AddLineBlankCustomY( 30.0f * SoUtils::GetiPadRatioY() );

		pScrollText->AddLineTypeA( "Powered by Cocos2d-x (www.cocos2d-x.org)" );
		pScrollText->AddLineBlankTypeA();
		pScrollText->AddLineTypeA( "Strange Jacket Board Game Engine, by Domowicz Creative Group, LLC" );
		pScrollText->AddLineBlankTypeA();
		pScrollText->AddLineTypeA( "Object Input System (OIS), Copyright (c) 2005-2010 Phillip Castaneda (pjcast -- www.wreckedgames.com)" );
		pScrollText->AddLineBlankTypeA();
		pScrollText->AddLineTypeA( "Audio engine : FMOD by Firelight Technologies" );


		pScrollText->AddLineBlankCustomY( 270.0f * SoUtils::GetiPadRatioY() );
		pScrollText->AddLineTypeC( "Thanks for Playing!" );
		pScrollText->AddLineBlankCustomY( 225.0f * SoUtils::GetiPadRatioY() );

		return true;
	}


	return false;
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoFsmCreditsLayer::update( float dt )
{
	_GuiManager.Update( dt );

	_fTimer += dt;

	switch ( _eState )
	{
		case eSubState_Screen1:
		{
			if ( _bTouched )
			{
				//_pSplashSprite1->setVisible( false );
				_bTouched = false;
				_eState = eSubState_Done;
			}
			break;
		}
		case eSubState_Done:
		{
		}
	}
}



/******************************************************************************
 FUNCTION:  IsDone
******************************************************************************/
bool SoFsmCreditsLayer::IsDone()
{
	return ( _eState == eSubState_Done );
}



/******************************************************************************
 FUNCTION:  ccTouchesEnded
******************************************************************************/
void SoFsmCreditsLayer::ccTouchesEnded(CCSet *pTouches, CCEvent *pEvent)
{
	_bTouched = true;
}



/******************************************************************************
 FUNCTION:  ccTouchesCancelled
******************************************************************************/
void SoFsmCreditsLayer::ccTouchesCancelled(CCSet *pTouches, CCEvent *pEvent)
{
	ccTouchesEnded(pTouches, pEvent);
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoFsmCreditsLayer* SoFsmCreditsLayer::create()
{
	SoFsmCreditsLayer* pRet = new SoFsmCreditsLayer();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}
