/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActorManager.h"
#include "SoActorSorter.h"
#include "SoConfigManager.h"
#include "SoFontDefs.h"
#include "SoFsmStateMain.h"
#include "SoGameGlobals.h"
#include "SoGuiManager.h"
#include "SoHangdown.h"
#include "SoHud.h"
#include "SoHudSwooshNotice.h"
#include "SoInputDispatcher.h"
#include "SoKicker.h"
#include "SoLuaState.h"
#include "SoNetworkComms.h"
#include "SoParticleManager.h"
#include "SoSaveSystem.h"
#include "SoScript.h"
#include "SoSoundManager.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



SoFsmMainLayer* SoFsmStateMain::_pSharedMainLayer = NULL;

SoHudLog* SoFsmMainLayer::_pSharedHudLog = NULL;



/******************************************************************************
 FUNCTION:  SoFsmStateMain
******************************************************************************/
SoFsmStateMain::SoFsmStateMain() :
_pLayer( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmStateMain
******************************************************************************/
SoFsmStateMain::~SoFsmStateMain()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFsmStateMain::Init()
{
	SoFsmState::Init();
	_FsmStateManager.StoreState( JpHash( "MainCore" ), &_FsmStateMainCore );

	_eCurrentSubState = eSubState_NewGameInit;

	SoActorManager::CreateInstance();
	SoActorSorter::CreateInstance();
	SoLuaState::CreateInstance();
	SoKickerManager::CreateInstance();
	SoScriptManager::CreateInstance();
	SoHud::CreateInstance();
	SoHudSwooshNoticeManager::CreateInstance();
	SoParticleManager::CreateInstance();
	SoHangdownManager::CreateInstance();
	SoInputDispatcher::CreateInstance();
	SoGuiManager::CreateInstance();

	SoGameGlobals::Init();

	// This turns off the music that gets started from SoFsmStateBoot
	// mdomowicz 2013_03_16
	SoSoundManager::GetInstance()->StopMusic();
	SoSoundManager::GetInstance()->UnloadMusicTrack();

	if ( !SoSoundManager::GetInstance()->GetMusicTrack() )
	{
		SoSoundManager::GetInstance()->LoadMusicTrack( SoUtils::AddRelativePathToSfx( "MusicTrackInGame01.ogg" ) );
		SoSoundManager::GetInstance()->PlayMusic();
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmStateMain::Deinit()
{
	SoFsmState::Deinit();

	SoGameGlobals::Deinit();

	SoGuiManager::DeleteInstance();
	SoInputDispatcher::DeleteInstance();

	SoHudSwooshNoticeManager::GetInstance()->Deinit();
	SoHudSwooshNoticeManager::DeleteInstance();
	SoHud::DeleteInstance();

	SoScriptManager::DeleteInstance();
	SoKickerManager::DeleteInstance();
	SoLuaState::DeleteInstance();
	SoActorSorter::DeleteInstance();
	SoActorManager::DeleteInstance();
	SoHangdownManager::DeleteInstance();
	SoParticleManager::DeleteInstance();

	SoSoundManager::GetInstance()->UnloadMusicTrack();


	if (_pLayer)
	{
		_pLayer->removeFromParentAndCleanup( true );
		_pLayer->Deinit();
		_pLayer->release();
		_pLayer = NULL;
		SetSharedMainLayer( NULL );
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmStateMain::Update( float dt )
{
	if ( !_pLayer )
	{
		_pLayer = SoFsmMainLayer::create();
		_pLayer->retain();
		CCDirector::sharedDirector()->getRunningScene()->addChild(_pLayer);
		SoActorManager::GetInstance()->SetParentNode( _pLayer );
		SoKickerManager::GetInstance()->SetParentNode( _pLayer );
		SoHud::GetInstance()->Init( _pLayer );
		SoHudSwooshNoticeManager::GetInstance()->SetParentNode( _pLayer );
		SoParticleManager::GetInstance()->SetParentNode( _pLayer );
		SoInputDispatcher::GetInstance()->Init( _pLayer );
		SetSharedMainLayer( _pLayer );

		SoGuiManager::GetInstance()->Init( _pLayer, SoGuiManager::INPUT_SOURCE_DISPATCHER );
	}

	if ( _pLayer->IsDone() )
	{
		SoFsmStateManager* pStateMgr = GetParentFsmStateManager();
		pStateMgr->SetStateOnNextUpdate( JpHash( "Shell" ) );
	}


	unsigned int uiReturnHash = 0;


	switch ( _eCurrentSubState )
	{
		case eSubState_NewGameInit:
		{	
			bool bIsNewGame = true;
			bool bLoadLastSavedDungeon = PrepareNewDungeon( bIsNewGame );


			if ( bLoadLastSavedDungeon  )
			{
				SoFsmStateMainCore::LoadGame();
			}
			else
			{
				SoScriptManager* pScriptManager = SoScriptManager::GetInstance();
				//SoLuaState::GetInstance()->OpenScript( SoUtils::AddRelativePathToScript( "GmMain.lua" ) );
			}

			_eCurrentSubState = eSubState_EnterCore;

			break;
		}

		case eSubState_EnterCore:
		{
			_FsmStateManager.SetState( JpHash( "MainCore" ) );

			_eCurrentSubState = eSubState_UpdateCore;
			break;
		}

		case eSubState_UpdateCore:
		{
			uiReturnHash = _FsmStateManager.Update( dt );

			break;
		}
	
		case eSubState_ExitCore:
		{
			break;
		}

		case eSubState_HandleGameWonSequenceInit:
		{
			break;
		}

		case eSubState_HandleGameWonSequenceUpdate:
		{
			break;
		}

		case eSubState_Last:
		default:
			ASSERTF( 0, ( "\nUnknown substate." ) );
			break;
	}

	return uiReturnHash;
}







/******************************************************************************
 FUNCTION:  PrepareNewDungeon
 NOTES:     This function is a bit crazy, but it basically preps the game
            by creating the dungeon (and therefore the levelnode tree).  
            However, there are a number of ways this creation process can
            happen, and so this function uses the following logic:

    if DebugForceRandomWorld is specified and true, randomly generate a new MasterRngSeed
    else if DebugWorldName is specified, use it to set the MasterRngSeed.
    else if DebugWorldHash is specified, use it to set the MasterRngSeed.
    else if DebugLevelType is specified, use it to set the MasterRngSeed.
    else if LastDungeon.csv exists load the last used dungeon
    else randomly generate a new MasterRngSeed

******************************************************************************/
bool SoFsmStateMain::PrepareNewDungeon( bool bRequestNewGame )
{
	bool bLoadLastSavedDungeon = false;
	return bLoadLastSavedDungeon;
}








/******************************************************************************
 FUNCTION:  SoFsmMainLayer
******************************************************************************/
SoFsmMainLayer::SoFsmMainLayer() :
_pScreenFader( NULL ),
_eCurrentState( STATE_LAST ),
_fTimer( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmMainLayer
******************************************************************************/
SoFsmMainLayer::~SoFsmMainLayer()
{
}


/******************************************************************************
 FUNCTION:  init
******************************************************************************/
bool SoFsmMainLayer::init()
{
	if ( CCLayerColor::initWithColor( ccc4(0,0,0,255) ) )
	{
		CCDirector::sharedDirector()->setProjection(kCCDirectorProjection2D);
		CCSize size = CCDirector::sharedDirector()->getWinSize();

		float fPosX = SoUtils::iPadPixelToPixelX( 407.0f );
		float fPosY = SoUtils::iPadPixelToPixelY( 356.0f );
		ccColor3B InnerTextColor = { 218, 245, 255 };

		_HudLog.SetParentNode( this );
		_HudLog.SetFontName( SoFontDefs::GetHudLog() );
		_HudLog.SetPositionAndSize( 20 * SoUtils::GetiPadRatioX(), 664.0f * SoUtils::GetiPadRatioY(), size.width, size.height );
		_HudLog.SetMessageShowDuration( 10.0f );
		_HudLog.SetMaxMessagesBeforeFifoRemoval( 3 ); // was 5 - reduced for screen realestate reasons.  mdomowicz 2014_04_09
		_HudLog.SetFillHeight( 86.0f * SoUtils::GetiPadRatioY() );
		SetSharedHudLog( &_HudLog );


		int iRenderOrder = 0;
		_pScreenFader = SoScreenFader::create();
		_pScreenFader->retain();
		_pScreenFader->Setup( this, iRenderOrder );
		_pScreenFader->SetFadeSpeed( 0.5f );
		_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_TRANSPARENT_TO_OPAQUE );
		_pScreenFader->BeginFade();


		_WatermarkPreRelease.Init( this );

		scheduleUpdate();
		setTouchEnabled( true );

		_fTimer = 0.0f;
		_eCurrentState = STATE_LAST;

		return true;
	}


	return false;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmMainLayer::Deinit()
{
	_HudLog.Deinit();
	SetSharedHudLog( NULL );


	if ( _pScreenFader )
	{
		_pScreenFader->Deinit();
		_pScreenFader->release();
		_pScreenFader = NULL;
	}

	_WatermarkPreRelease.Deinit();

	removeAllChildrenWithCleanup( DCG_CLEANUP_ALL_RUNNING_ACTIONS );
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoFsmMainLayer::update( float dt )
{
	_HudLog.Update( dt );

	_fTimer += dt;

	switch ( _eCurrentState )
	{

		case STATE_DONE:
		case STATE_LAST:
		default:
			break;
	}
}



/******************************************************************************
 FUNCTION:  ccTouchesEnded
******************************************************************************/
void SoFsmMainLayer::ccTouchesEnded(CCSet *pTouches, CCEvent *pEvent)
{
}



/******************************************************************************
 FUNCTION:  ccTouchesCancelled
******************************************************************************/
void SoFsmMainLayer::ccTouchesCancelled(CCSet *pTouches, CCEvent *pEvent)
{
	ccTouchesEnded(pTouches, pEvent);
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoFsmMainLayer* SoFsmMainLayer::create()
{
	SoFsmMainLayer* pRet = new SoFsmMainLayer();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}



















