/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoFsmState.h"
#include "SoFsmStateMainCore.h"
#include "SoHudLog.h"
#include "SoScreenFader.h"
#include "SoUiButton.h"
#include "SoWatermark.h"


class SoFsmMainLayer;


class SoFsmStateMain : public SoFsmState
{
	public:

		//-----------------------
		// public enums
		enum eSubState
		{
			eSubState_NewGameInit,
			eSubState_EnterCore,
			eSubState_UpdateCore,
			eSubState_ExitCore,
			eSubState_HandleGameWonSequenceInit,
			eSubState_HandleGameWonSequenceUpdate,
			eSubState_Last
		};


		//-----------------------
		// public functions

		SoFsmStateMain();

		virtual ~SoFsmStateMain();


		virtual void Init();
		virtual void Deinit();
		virtual hash Update( float dt );

		static SoFsmMainLayer* GetSharedMainLayer() { return _pSharedMainLayer; }
		static void SetSharedMainLayer( SoFsmMainLayer* p ) { _pSharedMainLayer = p; }


	private:

		//-----------------------
		// private variables

		SoFsmMainLayer* _pLayer;
		static SoFsmMainLayer* _pSharedMainLayer;

		SoFsmStateMainCore   _FsmStateMainCore;

		eSubState _eCurrentSubState;

		#if DCG_IS_TARGET_WIN32
			OIS::KeyCode _eLastInputPressed;
		#endif

		//-----------------------
		// private functions
		bool PrepareNewDungeon( bool bRequestNewGame );
};






class SoFsmMainLayer : public cocos2d::CCLayerColor
{
	public:

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_DONE,
			STATE_LAST
		};

		//-----------------------
		// public functions

		SoFsmMainLayer();
		virtual ~SoFsmMainLayer();

		virtual bool init();
		void Deinit();

		virtual void update( float dt );

		LAYER_NODE_FUNC(SoFsmMainLayer);

		bool IsDone() { return _eCurrentState == STATE_DONE; }


		static SoHudLog* GetSharedHudLog() { return _pSharedHudLog; }
		static void SetSharedHudLog( SoHudLog* p ) { _pSharedHudLog = p; }


		virtual void ccTouchesBegan(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent) {};
		virtual void ccTouchesMoved(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent) {};
		virtual void ccTouchesEnded(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);
		virtual void ccTouchesCancelled(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);


		static SoFsmMainLayer* create();


	private:

		//-----------------------
		// private variables
		float _fTimer;

		SoHudLog _HudLog;
		static SoHudLog* _pSharedHudLog;

		SoScreenFader* _pScreenFader;

		STATE _eCurrentState;

		SoWatermarkPreRelease _WatermarkPreRelease;
};