/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"


#include "SoActorSorter.h"
#include "SoActorVisible.h"
#include "SoBillboardMist.h"
#include "SoCamera.h"
#include "SoCameraGameplay.h"
#include "SoConfigManager.h"
#include "SoFontDefs.h"
#include "SoFsmStateMain.h"
#include "SoFsmStateMainCore.h"
#include "SoGameBitPlayerHuman.h"
#include "SoGameGlobals.h"
#include "SoGuiAnimWinCoreMainLauncher.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoHud.h"
#include "SoHudSwooshNotice.h"
#include "SoInputDispatcher.h"
#include "SoInputOisManager.h"
#include "SoInputTouch.h"
#include "SoKicker.h"
#include "SoLuaState.h"
#include "SoLoadScreen.h"
#include "SoNetworkComms.h"
#include "SoParticleManager.h"
#include "SoSaveSystem.h"
#include "SoScript.h"
#include "SoSoundManager.h"
#include "SoSkyPlane.h"
#include "SoSprite.h"
#include "SoSpritePlus.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;

SoGuiManager* SoFsmStateMainCore::_pCustomGuiManager = NULL;

bool SoFsmStateMainCore::_bIsZoomEnabled = false;


SoFsmStateMainCore::eGameConnectionType SoFsmStateMainCore::_eGameConnectionType = SoFsmStateMainCore::eGameConnectionType_Last;
std::string SoFsmStateMainCore::_sOnlineServerName = "";
bool SoFsmStateMainCore::_bLoadGameIfPossible = false;


hash SoFsmStateMainCore::_hPlayerType1 = 0;
hash SoFsmStateMainCore::_hPlayerType2 = 0;
hash SoFsmStateMainCore::_hPlayerType3 = 0;
hash SoFsmStateMainCore::_hPlayerType4 = 0;
hash SoFsmStateMainCore::_hPlayerType5 = 0;
hash SoFsmStateMainCore::_hPlayerType6 = 0;

bool SoFsmStateMainCore::_bIsVisibleStock = false;
bool SoFsmStateMainCore::_bIsVisibleMoney = false;




/******************************************************************************
 FUNCTION:  SoFsmStateMainCore
******************************************************************************/
SoFsmStateMainCore::SoFsmStateMainCore() :
_pLayer( NULL ),
_iCameraZoomState( 3 ),
_pGameOverSplash( NULL ),
_fGameOverSplashTimer( 0.0f ),
_pGameWonSplash( NULL ),
_fGameWonSplashTimer( 0.0f ),
_pScreenFader( NULL ),
_fCameraZoom( 1.0f ),
_gGuiCoreMainLauncher( 0 ),
_bOnlineServerGamesLocalHumanPlayerAdded( false ),
_iPlayerAddingState( 0 ),
_pClient( NULL ),
_pBillboardMist( NULL ),
_pCustomGuiManagerNode( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmStateMainCore
******************************************************************************/
SoFsmStateMainCore::~SoFsmStateMainCore()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFsmStateMainCore::Init()
{
	SoFsmState::Init();

	//-----------------------
	// Setup the input listeners
	_InputTouchListener.SetOwner( this );
	SoInputTouch::GetInstance()->AddListener( &_InputTouchListener );

	_InputGestureListener.SetOwner( this );
	SoInputGesture::GetInstance()->AddListener( &_InputGestureListener );

	#if DCG_IS_TARGET_WIN32
		_InputMouseListener.SetOwner( this );
		SoInputMouse::GetInstance()->AddListener( &_InputMouseListener );
	#endif


	//-----------------------
	// Camera setup
	SoCameraManager::CreateInstance();
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();

	pCameraManager->CreateGameplayCamera( "Gameplay" );

	NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "AllowFreeEyeCam" );
	if ( pLine && pLine->GetBool( 1 ) )
		pCameraManager->CreateFreeCamera( "FreeCam" );

	pCameraManager->SwitchToCameraByName( "Gameplay" );

	SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
	if ( pCamera )
	{
		SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();
		if ( pCameraGameplay )
		{
			pCameraGameplay->ResetFocus();

			pCameraGameplay->SetStartPosition( JpVector2f( 0.0f, 0.0f ) );
			pCameraGameplay->JumpaToStartPosition();

			pCameraGameplay->SetZoom( 0.65f );
			pCameraGameplay->SetTargetZoom( 0.65f );
			_fCameraZoom = 0.65f;
		}
	}


	//-----------------------
	// Screen fader setup
	_pScreenFader = SoScreenFader::create();
	_pScreenFader->retain();
	_pScreenFader->Setup( SoFsmStateMain::GetSharedMainLayer(), BASE_RENDER_ORDER_FADER );
	_pScreenFader->SetFadeSpeed( 2.0f );

	_eCurrentSubState = eSubState_CreateNewGame;
	
	#if DCG_IS_TARGET_WIN32
		_eLastInputPressed = OIS::KC_UNASSIGNED;
	#endif


	//-----------------------
	_pGameOverSplash = new SoSpritePlus();
	_pGameOverSplash->Init( SoFsmStateMain::GetSharedMainLayer(), SoUtils::AddRelativePathToTexture( "HudGameOverSplash.png" ), true );
	_pGameOverSplash->SetType( SoSpritePlus::TYPE_FADE_IN_OUT );
	_pGameOverSplash->SetRenderZOrder( BASE_RENDER_ORDER_LOG );
	_fGameOverSplashTimer = 0.0f;

	_pGameWonSplash = new SoSpritePlus();
	_pGameWonSplash->Init( SoFsmStateMain::GetSharedMainLayer(), SoUtils::AddRelativePathToTexture( "HudGameWonSplash.png" ), true );
	_pGameWonSplash->SetType( SoSpritePlus::TYPE_FADE_IN_OUT );
	_pGameWonSplash->SetRenderZOrder( BASE_RENDER_ORDER_LOG );
	_fGameWonSplashTimer = 0.0f;


	//-----------------------
	_pSkyPlane = new SoSkyPlane;
	_pSkyPlane->Init( SoFsmStateMain::GetSharedMainLayer(), SoUtils::AddRelativePathToTexture( "SkyPlaneCave.png" ) );



	//-----------------------
	// Create a CCNode for the custom gui manager to use.
	// With this node, we can control the render order of the
	// Custom Gui manager.
	_pCustomGuiManagerNode = new CCNode();
	_pCustomGuiManagerNode->setVisible( true );
	SoFsmStateMain::GetSharedMainLayer()->addChild( _pCustomGuiManagerNode );
	SoUtils::SetRenderZOrder( SoFsmStateMain::GetSharedMainLayer(), _pCustomGuiManagerNode, BASE_RENDER_ORDER_GUI + 1 );


	//-----------------------
	// Create the custom Gui Manager.  This is needed so so that
	// the Core Menu and Window Log can be easily rendered in front of
	// all other Gui menus/elements (which belong to the global GuiManager) 
	_pCustomGuiManager = new SoGuiManager;
	_pCustomGuiManager->Init( _pCustomGuiManagerNode, SoGuiManager::INPUT_SOURCE_DISPATCHER );
	_pCustomGuiManager->SetHeadGuiAbsolutionPositionToScreenCenter();


	//-----------------------
	// create and setup the CoreMainLauncher
	SoGuiAnimWinCoreMainLauncher* pNewWindow = new SoGuiAnimWinCoreMainLauncher;
	_gGuiCoreMainLauncher = _pCustomGuiManager->AddGui( pNewWindow, NULL );
	_pCustomGuiManager->SendMessageToGui( _gGuiCoreMainLauncher, &SoGuiMessage::_BasicTransitionIn );
	_GuiListener.SetOwner( this );
	pNewWindow->AddListener( &_GuiListener );
	

	//-----------------------
	// Some setup for the global GuiManager - not sure if this is
	// the best place for this.
	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();
	pGuiManager->SetHeadGuiAbsolutionPositionToScreenCenter();


	//-----------------------
	_pBillboardMist = new SoBillboardMist;
	_pBillboardMist->Init( SoFsmStateMain::GetSharedMainLayer(), SoUtils::AddRelativePathToTexture( "MistCloud01.png" ), 3 );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmStateMainCore::Deinit()
{
	SoFsmState::Deinit();

	if ( _pSkyPlane )
	{
		delete _pSkyPlane;
		_pSkyPlane = NULL;
	}

	if ( _pGameOverSplash )
	{
		delete _pGameOverSplash;
		_pGameOverSplash = NULL;
	}

	if ( _pGameWonSplash )
	{
		delete _pGameWonSplash;
		_pGameWonSplash = NULL;
	}

	if ( _pScreenFader )
	{
		_pScreenFader->Deinit();
		_pScreenFader->release();
		_pScreenFader = NULL;
	}

	if (_pLayer)
	{
		_pLayer->removeFromParentAndCleanup( true );
		_pLayer->Deinit();
		_pLayer->release();
		_pLayer = NULL;
	}


	// If the game is over, clear out the save data, since now there is no more game
	// to continue.  If the game is NOT over, save the game to disk.
	SoGameServer* pServer = SoGameGlobals::GetServer();
	SoGameClientList* pClientList = SoGameGlobals::GetClientList();

	// This needs to be fixed if we ever want to support more than one local client.
	ASSERTF( pClientList->GetNumEntries() <= 1, ( "\nRight now this only works for one local client." ) );

	SoGameClientList::Node* pFirstNode = pClientList->GetFirst();
	SoGameClient* pClient = pFirstNode->GetData();

	if ( pServer && pClient )
	{
		SoGameState* pGame = pServer->GetGameState();
		if ( pGame )
		{
			bool bGameIsOver = ( pGame->GetNumWinners() > 0 );
			if ( bGameIsOver )
			{
				pServer->InvalidateSaveData();
				pClient->InvalidateSaveData();
			}
			else
			{
				pServer->WriteSaveGameCsvToDisk();
				pClient->WriteSaveGameCsvToDisk();
			}
		}	
	}

	SoCameraManager::DeleteInstance();

	SoInputTouch::GetInstance()->RemoveListener( &_InputTouchListener );
	SoInputGesture::GetInstance()->RemoveListener( &_InputGestureListener );

	#if DCG_IS_TARGET_WIN32
		SoInputMouse::GetInstance()->RemoveListener( &_InputMouseListener );
	#endif

	SoActorManager* pActorManager = SoActorManager::GetInstance();
	pActorManager->RemoveAndDeleteAllActorsNow();


	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();
	pGuiManager->RemoveAndDeleteAllGuis();

	_pBillboardMist->Deinit();
	delete _pBillboardMist;

	_pCustomGuiManager->Deinit();
	delete _pCustomGuiManager;
	_pCustomGuiManager = NULL;

	SoFsmStateMain::GetSharedMainLayer()->removeChild( _pCustomGuiManagerNode, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
	_pCustomGuiManagerNode->release();
	_pCustomGuiManagerNode = NULL;
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmStateMainCore::Update( float dt )
{
	SoFsmState::Update( dt );

	SoActorManager* pActorManager = SoActorManager::GetInstance();

	unsigned int uiReturnHash = 0;

	_pBillboardMist->Update( dt );
	_pCustomGuiManager->Update( dt );


	TempDebugUpdate( dt );

	switch ( _eCurrentSubState )
	{
		case eSubState_CreateNewGame:
		{
			_iCameraZoomState = 7;
			SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
			SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
			SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();
			if ( pCameraGameplay )
			{
				pCameraGameplay->JumpToStandardPositionAndZoom();
			}


			if ( !_pLayer )
			{
				_pLayer = SoFsmMainCoreLayer::create();
				_pLayer->retain();
				CCDirector::sharedDirector()->getRunningScene()->addChild(_pLayer);
			}



			//-----------------------
			// client/server setup varies depending 
			// on game connection type.
			bool bSaveDataExists = SoGameGlobals::DoesSaveGameExist();

			if ( _eGameConnectionType == eGameConnectionType_Offline && bSaveDataExists && _bLoadGameIfPossible )
			{
				//-----------------------
				// Create the network communications object.
				SoGameGlobals::CreateNetworkCommsOffline();
				SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();

				//-----------------------
				// Create the game server
				NeCsvIn CsvInServerData;

				char acFullPathAndFilename1[ 512 ] = "";
				SoUtils::CreateFullPathWithRelativePathAndFilename( acFullPathAndFilename1, SoUtils::AddRelativePathToSaveData( "SavedData_Server.csv" ) );
				CsvInServerData.ReadCsvFile( acFullPathAndFilename1 );

				CsvInServerData.MakeHashIndicesFromRowValuesAtCol( 0 );
				hash hServerId = CsvInServerData.GetIntByHashAndInt( JpHash( "_hServerId" ), 1, -1 );
				hash hMasterRngSeed = CsvInServerData.GetIntByHashAndInt( JpHash( "MasterRngSeed" ), 1, -1 );
				SoGameGlobals::CreateServerForLoadGame( hServerId, hMasterRngSeed );

				_bIsVisibleStock = CsvInServerData.GetBoolByHashAndInt( JpHash( "_bIsVisibleStock" ), 1, true );
				_bIsVisibleMoney = CsvInServerData.GetBoolByHashAndInt( JpHash( "_bIsVisibleMoney" ), 1, false );
				SoGameGlobals::GetServer()->GetGameState()->SetIsVisibleStocks( _bIsVisibleStock );
				SoGameGlobals::GetServer()->GetGameState()->SetIsVisibleMoney( _bIsVisibleMoney );


				//-----------------------
				// Create the local client
				NeCsvIn CsvInClientData;

				char acFullPathAndFilename2[ 512 ] = "";
				SoUtils::CreateFullPathWithRelativePathAndFilename( acFullPathAndFilename2, SoUtils::AddRelativePathToSaveData( "SavedData_Client.csv" ) );
				CsvInClientData.ReadCsvFile( acFullPathAndFilename2 );

				CsvInClientData.MakeHashIndicesFromRowValuesAtCol( 0 );
				hash hClientId = CsvInClientData.GetIntByHashAndInt( JpHash( "_hClientId" ), 1, -1 );
				_pClient = SoGameGlobals::CreateClient();
				_pClient->SetClientId( hClientId );
				_pClient->SetServerId( hServerId );
				_pClient->GetGameState()->SetOwningClientId( hClientId );


				//-----------------------
				// load all the game messages
				NeCsvIn CsvInMessageData;
				char acFullPathAndFilename3[ 512 ] = "";
				SoUtils::CreateFullPathWithRelativePathAndFilename( acFullPathAndFilename3, SoUtils::AddRelativePathToSaveData( "SavedData_Messages.csv" ) );
				CsvInMessageData.ReadCsvFile( acFullPathAndFilename3 );


				//-----------------------
				// Send every saved message to the server.  
				// This is the key step that actually restores the saved game.
				SoGameServer* pServer = SoGameGlobals::GetServer();
				for ( int i = 1; i < CsvInMessageData.GetNumRows(); i ++ )
				{
					SoGameMessage Message( hServerId, -1, Msg::TYPE_LAST );
					Message.ReadFromCsv( &CsvInMessageData, i );

					// As the game is played forward, we want the clients/players to NOT respond to commands
					// from the server, because we already know what the responses will be.  Those
					// responses are part of the saved data.  Except, if the very last saved message is a
					// command to a player, we DO want that player to respond.  So for the last message
					// we say that loading is finished so that the clients/players respond normally.
					if ( i == CsvInMessageData.GetNumRows() - 1 )
					{
						pServer->GetGameState()->SetIsLoadingGame( false );
						_pClient->GetGameState()->SetIsLoadingGame( false );
						SoGameGlobals::SetIsLoadingGame( false );

						// Some of the message handling done during the course of restoring the game will have
						// changed the master RNGs.  But we want the master RNGs to be restored to their saved values,
						// and right before any new moves are about to be played.  This code does just that.
						unsigned int hServerMasterRngSeedHigh = CsvInServerData.GetIntByHashAndInt( JpHash( "MasterRngSeedHigh" ), 1, -1 );
						unsigned int hServerMasterRngSeedLow = CsvInServerData.GetIntByHashAndInt( JpHash( "MasterRngSeedLow" ), 1, -1 );
						unsigned int hClientMasterRngSeedHigh = CsvInClientData.GetIntByHashAndInt( JpHash( "MasterRngSeedHigh" ), 1, -1 );
						unsigned int hClientMasterRngSeedLow = CsvInClientData.GetIntByHashAndInt( JpHash( "MasterRngSeedLow" ), 1, -1 );
						pServer->GetGameState()->GetMasterRng()->SetHigh( hServerMasterRngSeedHigh );
						pServer->GetGameState()->GetMasterRng()->SetLow( hServerMasterRngSeedLow );
						_pClient->GetGameState()->GetMasterRng()->SetHigh( hClientMasterRngSeedHigh );
						_pClient->GetGameState()->GetMasterRng()->SetLow( hClientMasterRngSeedLow );
					}
					else
					{
						pServer->GetGameState()->SetIsLoadingGame( true );
						_pClient->GetGameState()->SetIsLoadingGame( true );
						SoGameGlobals::SetIsLoadingGame( true );
					}


					// Send the message to the server, and let the server and client update
					// a few times to process the result of the message.
					pNetwork->SendMessageToServer( Message );

					for ( int j = 0; j < 5; j ++ )
					{
						// This detail makes sure the the message accounting is up to date at all times.
						// Without this, a game may load, but have messages be out of sync, and be unable to
						// continue.
						int iLastMessageNum = pServer->GetLastMessageNumReceivedFromClient( _pClient->GetClientId() );
						_pClient->SetLastMessageNumSentToServer( iLastMessageNum );

						SoGameGlobals::Update( dt );
					}
				}

				//-----------------------
				// Some final fail-safe settings.  This is also done above, but just in case...
				pServer->GetGameState()->SetIsLoadingGame( false );
				_pClient->GetGameState()->SetIsLoadingGame( false );
				SoGameGlobals::SetIsLoadingGame( false );

				int iLastMessageNum = pServer->GetLastMessageNumReceivedFromClient( _pClient->GetClientId() );
				_pClient->SetLastMessageNumSentToServer( iLastMessageNum );


				//-----------------------
				// Synchronize all the visible actors as needed, so that they match
				// the current game state.
				SoActorManager* pActorManager = SoActorManager::GetInstance();
				SoActorList* pAllActors = pActorManager->GetActorList();
				SoActorList::Iterator ActorIter( pAllActors );

				while ( ActorIter.Next() )
				{
					SoActor* pActor = ActorIter.GetNodeData();
					SoActorVisible* pActorVisible = pActor->GetAsVisible();
					if ( pActorVisible )
					{
						pActorVisible->SynchronizeAfterLoad();
					}
				}

				_eCurrentSubState = eSubState_Main;
			}
			else if ( _eGameConnectionType == eGameConnectionType_Offline )
			{
				//-----------------------
				// Create the network communications object.
				SoGameGlobals::CreateNetworkCommsOffline();

				//-----------------------
				// Create the game server
				SoGameGlobals::CreateServer( "" );
				SoGameGlobals::GetServer()->GetGameState()->SetIsVisibleStocks( _bIsVisibleStock );
				SoGameGlobals::GetServer()->GetGameState()->SetIsVisibleMoney( _bIsVisibleMoney );

				//-----------------------
				// Create the local client
				_pClient = SoGameGlobals::CreateClient();
				_pClient->ConnectToServer( "" );


				//-----------------------
				// add all the players
				NeConfig::Line* pLine1 = SoConfigManager::GetInstance()->FindLine( "sPlayerType1" );
				if ( pLine1 )
					CreatePlayersBasedOnIniFile( _pClient );
				else
					CreatePlayersBasedOnInGameSetup( _pClient );


				//-----------------------
				// Send the start game message
				SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();

				// hack! - this won't work when we need to support multiple local servers.
				hash hServerId = pNetwork->FindAvailableServer(); 

				SoGameMessage Message( hServerId, -1, Msg::TYPE_SVR_COMMAND_START_GAME );
				pNetwork->SendMessageToServer( Message );

				_eCurrentSubState = eSubState_Main;
			}
			else if ( _eGameConnectionType == eGameConnectionType_Online_Server )
			{
				SoGameGlobals::CreateNetworkCommsOnline();

				SoGameGlobals::CreateServer( _sOnlineServerName );
				SoGameGlobals::GetServer()->GetGameState()->SetIsVisibleStocks( _bIsVisibleStock );
				SoGameGlobals::GetServer()->GetGameState()->SetIsVisibleMoney( _bIsVisibleMoney );

				_pClient = SoGameGlobals::CreateClient();
				_pClient->ConnectToServer( _sOnlineServerName );

				_bOnlineServerGamesLocalHumanPlayerAdded = false;
				_iPlayerAddingState = 0;
				_eCurrentSubState = eSubState_Online_Server_Add_Players;

				SoGameGlobals::Update( dt );

			}
			else if ( _eGameConnectionType == eGameConnectionType_Online_Client )
			{
				SoGameGlobals::CreateNetworkCommsOnline();

				_pClient = SoGameGlobals::CreateClient();

				_pClient->ConnectToServer( _sOnlineServerName );

				_pClient->CreatePlayerHuman();

				_eCurrentSubState = eSubState_Main;
			}


			break;
		}

		case eSubState_Online_Server_Add_Players:
		{
			bool bDoneAddingPlayers = OnlineServerGameAddPlayers();

			if ( bDoneAddingPlayers )
			{
				SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();

				SoGameServer* pServer = SoGameGlobals::GetServer();
				SoGameMessage Message( pServer->GetServerId(), -1, Msg::TYPE_SVR_COMMAND_START_GAME );
				pNetwork->SendMessageToServer( Message );

				_eCurrentSubState = eSubState_Main;
			}


			SoGameGlobals::Update( dt );

			break;
		}

		case eSubState_Main:
		{
			MainGameUpdate( dt );
			break;
		}

		case eSubState_FadeInInit:
		{   
			_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_OPAQUE_TO_TRANSPARENT );
			_pScreenFader->BeginFade();

			_eCurrentSubState = eSubState_FadeInUpdate;

			break;
		}

		case eSubState_FadeInUpdate:
		{
			// This makes sure the actors are instanced and ready, as we fade in.
			pActorManager->Update( 0.0f );

			_pScreenFader->Update( dt );
			if ( _pScreenFader->IsDone() )
			{
				_eCurrentSubState = eSubState_Main;
			}

			break;
		}

		case eSubState_ExitToShellInit:
		{
			_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_TRANSPARENT_TO_OPAQUE );
			_pScreenFader->BeginFade();

			_eCurrentSubState = eSubState_ExitToShellUpdate;

			break;
		}

		case eSubState_ExitToShellUpdate:
		{
			_pScreenFader->Update( dt );
			if ( _pScreenFader->IsDone() )
			{
				uiReturnHash = 0x5a130bf8 /* "GoToShell" */;
			}

			break;
		}

		default:
			break;
	}

	UpdateCameraZoomSetting( dt );

	UpdateInput( dt );

	#if DCG_IS_TARGET_WIN32
		_eLastInputPressed = OIS::KC_UNASSIGNED;
	#endif

	return uiReturnHash;
}



/******************************************************************************
 FUNCTION:  AddPlayers
******************************************************************************/
bool SoFsmStateMainCore::OnlineServerGameAddPlayers()
{
	//-----------------------
	// Local functions
	struct Local
	{
		// This function creates all non-human players.
		// If the player specified is human, then it only creates the first human requested.
		// This function returns TRUE if a player was created.
		static bool CreatePlayer( SoGameClient* pClient, hash hPlayerType, bool* pbOnlineServerGamesLocalHumanPlayerAdded )
		{
			if ( hPlayerType == JpHash( "AiEasy" ) )
			{
				pClient->CreatePlayerMctsRandom();
				return true;
			}
			else if ( hPlayerType == JpHash( "AiMedium" ) )
			{
				pClient->CreatePlayerAi();
				return true;
			}
			else if ( hPlayerType == JpHash( "AiMediumLevel2" ) )
			{
				pClient->CreatePlayerAiLevel2();
				return true;
			}
			else if ( hPlayerType == JpHash( "Human" ) )
			{
				if ( !(*pbOnlineServerGamesLocalHumanPlayerAdded) )
				{
					pClient->CreatePlayerHuman();
					*pbOnlineServerGamesLocalHumanPlayerAdded = true;
					return true;
				}
			}

			return false;
		}
	};


	//-----------------------
	SoGameServer* pServer = SoGameGlobals::GetServer();

	switch ( _iPlayerAddingState )
	{
		case 0:
		{
			// Essentially, here we either create a player (if it is non-human, or the first human)
			// or we wait for wait for the server to say that a player was created by another client.
			if ( Local::CreatePlayer( _pClient, _hPlayerType1, &_bOnlineServerGamesLocalHumanPlayerAdded ) )
				_iPlayerAddingState ++;
			else if ( pServer->GetNumPlayersInGame() == 1 )
				_iPlayerAddingState ++;
			else if ( _hPlayerType1 == JpHash( "None" ) )
				_iPlayerAddingState ++;

			break;
		}
		case 1:
		{
			if ( Local::CreatePlayer( _pClient, _hPlayerType2, &_bOnlineServerGamesLocalHumanPlayerAdded ) )
				_iPlayerAddingState ++;
			else if ( pServer->GetNumPlayersInGame() == 2 )
				_iPlayerAddingState ++;
			else if ( _hPlayerType2 == JpHash( "None" ) )
				_iPlayerAddingState ++;

			break;
		}
		case 2:
		{
			if ( Local::CreatePlayer( _pClient, _hPlayerType3, &_bOnlineServerGamesLocalHumanPlayerAdded ) )
				_iPlayerAddingState ++;
			else if ( pServer->GetNumPlayersInGame() == 3 )
				_iPlayerAddingState ++;
			else if ( _hPlayerType3 == JpHash( "None" ) )
				_iPlayerAddingState ++;

			break;
		}
		case 3:
		{
			if ( Local::CreatePlayer( _pClient, _hPlayerType4, &_bOnlineServerGamesLocalHumanPlayerAdded ) )
				_iPlayerAddingState ++;
			else if ( pServer->GetNumPlayersInGame() == 4 )
				_iPlayerAddingState ++;
			else if ( _hPlayerType4 == JpHash( "None" ) )
				_iPlayerAddingState ++;

			break;
		}
		case 4:
		{
			if ( Local::CreatePlayer( _pClient, _hPlayerType5, &_bOnlineServerGamesLocalHumanPlayerAdded ) )
				_iPlayerAddingState ++;
			else if ( pServer->GetNumPlayersInGame() == 5 )
				_iPlayerAddingState ++;
			else if ( _hPlayerType5 == JpHash( "None" ) )
				_iPlayerAddingState ++;

			break;
		}
		case 5:
		{
			if ( Local::CreatePlayer( _pClient, _hPlayerType6, &_bOnlineServerGamesLocalHumanPlayerAdded ) )
				_iPlayerAddingState ++;
			else if ( pServer->GetNumPlayersInGame() == 6 )
				_iPlayerAddingState ++;
			else if ( _hPlayerType6 == JpHash( "None" ) )
				_iPlayerAddingState ++;

			break;
		}
		case 6:
			return true;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  MainGameUpdate
******************************************************************************/
void SoFsmStateMainCore::MainGameUpdate( float dt )
{

	SoCameraManager::GetInstance()->Update( dt );

	_pSkyPlane->Update( dt );
	SoActorManager::GetInstance()->Update( dt );

	SoActorSorter::GetInstance()->Update( dt );
	SoLuaState::GetInstance()->Update( dt );
	SoKickerManager::GetInstance()->Update( dt );
	SoHud::GetInstance()->Update( dt );
	SoHudSwooshNoticeManager::GetInstance()->Update( dt );
	SoParticleManager::GetInstance()->Update( dt );
	SoInputDispatcher::GetInstance()->Update( dt );
	SoGuiManager::GetInstance()->Update( dt );

	SoGameGlobals::Update( dt );

	SoScriptManager::GetInstance()->Update( dt );

	// This is a HACK.  Parking spots are not actors, and because of this
	// they don't get updated with the actors.  Previously they were getting
	// updated by the SoGameBitPlayerHuman that owned them, but this meant
	// that in some situations ParkingSpots were getting updates at undesirable
	// times.  One such situation was when loading a game.  The parking spots
	// were getting updated because the player was getting updated.  Thus they
	// were made visible too early, and much earlier than the visible actors.
	// The proper fix for this is to make ParkingSpots actual actors, but
	// for now, moving the update for them out of the PlayerHuman's control
	// is a good enough fix.  mdomowicz 2015_12_06
	UpdateWorkerParkingSpots( dt );
}



/******************************************************************************
 FUNCTION:  UpdateWorkerParkingSpots
******************************************************************************/
void SoFsmStateMainCore::UpdateWorkerParkingSpots( float dt )
{
	SoGameClientList* pClientList = SoGameGlobals::GetClientList();
	SoGameClientList::Iterator ClientIter( pClientList );
	while ( ClientIter.Next() )
	{
		SoGameClient* pClient = ClientIter.GetNodeData();
		
		SoGameBitPlayerArray* pPlayerArray = pClient->GetGameState()->GetPlayerArray();
		SoGameBitPlayerArray::Iterator PlayerIter( pPlayerArray );
		while ( PlayerIter.Next() )
		{
			SoGameBitPlayer* pPlayer = PlayerIter.GetData();
			SoGameBitPlayerHuman* pPlayerHuman = pPlayer->GetAsPlayerHuman();
			if ( pPlayerHuman )
			{
				pPlayerHuman->UpdateAllParkingSpots( dt );
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  CreatePlayersBasedOnIniFile
******************************************************************************/
void SoFsmStateMainCore::CreatePlayersBasedOnIniFile( SoGameClient* pClient )
{
	NeConfig::Line* pLine1 = SoConfigManager::GetInstance()->FindLine( "sPlayerType1" );
	NeConfig::Line* pLine2 = SoConfigManager::GetInstance()->FindLine( "sPlayerType2" );
	NeConfig::Line* pLine3 = SoConfigManager::GetInstance()->FindLine( "sPlayerType3" );
	NeConfig::Line* pLine4 = SoConfigManager::GetInstance()->FindLine( "sPlayerType4" );
	NeConfig::Line* pLine5 = SoConfigManager::GetInstance()->FindLine( "sPlayerType5" );
	NeConfig::Line* pLine6 = SoConfigManager::GetInstance()->FindLine( "sPlayerType6" );

	hash hPlayerType1 = 0;
	hash hPlayerType2 = 0;
	hash hPlayerType3 = 0;
	hash hPlayerType4 = 0;
	hash hPlayerType5 = 0;
	hash hPlayerType6 = 0;

	if ( pLine1 )
		hPlayerType1 = JpHash( pLine1->GetString( 1 ) );
	if ( pLine2 )
		hPlayerType2 = JpHash( pLine2->GetString( 1 ) );
	if ( pLine3 )
		hPlayerType3 = JpHash( pLine3->GetString( 1 ) );
	if ( pLine4 )
		hPlayerType4 = JpHash( pLine4->GetString( 1 ) );
	if ( pLine5 )
		hPlayerType5 = JpHash( pLine5->GetString( 1 ) );
	if ( pLine6 )
		hPlayerType6 = JpHash( pLine6->GetString( 1 ) );


	if ( hPlayerType1 == 0xc2e36c54 /* "mcts" */ )
		pClient->CreatePlayerMctsRandom();
	else if ( hPlayerType1 == 0x4424f79f /* "ai" */ )
		pClient->CreatePlayerAi();
	else if ( hPlayerType1 == 0x714daa7a /* "human" */ )
		pClient->CreatePlayerHuman();

	if ( hPlayerType2 == 0xc2e36c54 /* "mcts" */ )
		pClient->CreatePlayerMctsRandom();
	else if ( hPlayerType2 == 0x4424f79f /* "ai" */ )
		pClient->CreatePlayerAi();
	else if ( hPlayerType2 == 0x714daa7a /* "human" */ )
		pClient->CreatePlayerHuman();

	if ( hPlayerType3 == 0xc2e36c54 /* "mcts" */ )
		pClient->CreatePlayerMctsRandom();
	else if ( hPlayerType3 == 0x4424f79f /* "ai" */ )
		pClient->CreatePlayerAi();
	else if ( hPlayerType3 == 0x714daa7a /* "human" */ )
		pClient->CreatePlayerHuman();

	if ( hPlayerType4 == 0xc2e36c54 /* "mcts" */ )
		pClient->CreatePlayerMctsRandom();
	else if ( hPlayerType4 == 0x4424f79f /* "ai" */ )
		pClient->CreatePlayerAi();
	else if ( hPlayerType4 == 0x714daa7a /* "human" */ )
		pClient->CreatePlayerHuman();

	if ( hPlayerType5 == 0xc2e36c54 /* "mcts" */ )
		pClient->CreatePlayerMctsRandom();
	else if ( hPlayerType5 == 0x4424f79f /* "ai" */ )
		pClient->CreatePlayerAi();
	else if ( hPlayerType5 == 0x714daa7a /* "human" */ )
		pClient->CreatePlayerHuman();

	if ( hPlayerType6 == 0xc2e36c54 /* "mcts" */ )
		pClient->CreatePlayerMctsRandom();
	else if ( hPlayerType6 == 0x4424f79f /* "ai" */ )
		pClient->CreatePlayerAi();
	else if ( hPlayerType6 == 0x714daa7a /* "human" */ )
		pClient->CreatePlayerHuman();
}



/******************************************************************************
 FUNCTION:  CreatePlayersBasedOnInGameSetup
******************************************************************************/
void SoFsmStateMainCore::CreatePlayersBasedOnInGameSetup( SoGameClient* pClient )
{
	if ( _hPlayerType1 == JpHash( "AiEasy" ) )
		pClient->CreatePlayerMctsRandom();
	else if ( _hPlayerType1 == JpHash( "AiMedium" ) )
		pClient->CreatePlayerAi();
	else if ( _hPlayerType1 == JpHash( "AiMediumLevel2" ) )
		pClient->CreatePlayerAiLevel2();
	else if ( _hPlayerType1 == JpHash( "Human" ) )
		pClient->CreatePlayerHuman();

	if ( _hPlayerType2 == JpHash( "AiEasy" ) )
		pClient->CreatePlayerMctsRandom();
	else if ( _hPlayerType2 == JpHash( "AiMedium" ) )
		pClient->CreatePlayerAi();
	else if ( _hPlayerType2 == JpHash( "AiMediumLevel2" ) )
		pClient->CreatePlayerAiLevel2();
	else if ( _hPlayerType2 == JpHash( "Human" ) )
		pClient->CreatePlayerHuman();

	if ( _hPlayerType3 == JpHash( "AiEasy" ) )
		pClient->CreatePlayerMctsRandom();
	else if ( _hPlayerType3 == JpHash( "AiMedium" ) )
		pClient->CreatePlayerAi();
	else if ( _hPlayerType3 == JpHash( "AiMediumLevel2" ) )
		pClient->CreatePlayerAiLevel2();
	else if ( _hPlayerType3 == JpHash( "Human" ) )
		pClient->CreatePlayerHuman();

	if ( _hPlayerType4 == JpHash( "AiEasy" ) )
		pClient->CreatePlayerMctsRandom();
	else if ( _hPlayerType4 == JpHash( "AiMedium" ) )
		pClient->CreatePlayerAi();
	else if ( _hPlayerType4 == JpHash( "AiMediumLevel2" ) )
		pClient->CreatePlayerAiLevel2();
	else if ( _hPlayerType4 == JpHash( "Human" ) )
		pClient->CreatePlayerHuman();

	if ( _hPlayerType5 == JpHash( "AiEasy" ) )
		pClient->CreatePlayerMctsRandom();
	else if ( _hPlayerType5 == JpHash( "AiMedium" ) )
		pClient->CreatePlayerAi();
	else if ( _hPlayerType5 == JpHash( "AiMediumLevel2" ) )
		pClient->CreatePlayerAiLevel2();
	else if ( _hPlayerType5 == JpHash( "Human" ) )
		pClient->CreatePlayerHuman();

	if ( _hPlayerType6 == JpHash( "AiEasy" ) )
		pClient->CreatePlayerMctsRandom();
	else if ( _hPlayerType6 == JpHash( "AiMedium" ) )
		pClient->CreatePlayerAi();
	else if ( _hPlayerType6 == JpHash( "AiMediumLevel2" ) )
		pClient->CreatePlayerAiLevel2();
	else if ( _hPlayerType6 == JpHash( "Human" ) )
		pClient->CreatePlayerHuman();
}



/******************************************************************************
 FUNCTION:  AllowFarOutZoom
******************************************************************************/
bool SoFsmStateMainCore::AllowFarOutZoom()
{
	bool bAllowFarOutZoom = false;
	NeConfig::Line* pLine1 = SoConfigManager::GetInstance()->FindLine( "bAllowFarOutZoom" );
	if ( pLine1 )
		bAllowFarOutZoom = pLine1->GetBool( 1 );

	return bAllowFarOutZoom;
}



/******************************************************************************
 FUNCTION:  UpdateCameraZoomSetting
******************************************************************************/
void SoFsmStateMainCore::UpdateCameraZoomSetting( float dt )
{
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
	SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();

	SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();

	bool bConsoleIsActive = ( pConsole && pConsole->IsShowing() );

	float fL0 = 0.1f;   // Need bAllowFarOutZoom enabled to reach this zoom
	float fL1 = 0.17f;
	float fL2 = 0.20f;
	float fL3 = 0.25f;
	float fL4 = 0.35f;
	float fL5 = 0.45f;
	float fL6 = 0.55f;
	float fL7 = 0.65f;

	#if DCG_IS_TARGET_WIN32
		if ( !bConsoleIsActive )
		{
			switch ( _iCameraZoomState )
			{
				case 0:
				{
					if ( _eLastInputPressed == OIS::KC_EQUALS )
					{
						pCameraGameplay->SetTargetZoom( fL1 );
						_iCameraZoomState ++;
					}
					break;
				}

				case 1:
				{
					if ( _eLastInputPressed == OIS::KC_EQUALS )
					{
						pCameraGameplay->SetTargetZoom( fL2 );
						_iCameraZoomState ++;
					}
					else if ( ( _eLastInputPressed == OIS::KC_MINUS ) && AllowFarOutZoom() )
					{
						pCameraGameplay->SetTargetZoom( fL0 );
						_iCameraZoomState --;
					}
					break;
				}

				case 2:
				{
					if ( _eLastInputPressed == OIS::KC_EQUALS )
					{
						pCameraGameplay->SetTargetZoom( fL3 );
						_iCameraZoomState ++;
					}
					else if ( ( _eLastInputPressed == OIS::KC_MINUS ) )
					{
						pCameraGameplay->SetTargetZoom( fL1 );
						_iCameraZoomState --;
					}
					break;
				}

				case 3:
				{
					if ( _eLastInputPressed == OIS::KC_EQUALS )
					{
						pCameraGameplay->SetTargetZoom( fL4 );
						_iCameraZoomState ++;
					}
					else if ( _eLastInputPressed == OIS::KC_MINUS )
					{
						pCameraGameplay->SetTargetZoom( fL2 );
						_iCameraZoomState --;
					}
					break;
				}

				case 4:
				{
					if ( _eLastInputPressed == OIS::KC_EQUALS )
					{
						pCameraGameplay->SetTargetZoom( fL5 );
						_iCameraZoomState ++;
					}
					else if ( _eLastInputPressed == OIS::KC_MINUS )
					{
						pCameraGameplay->SetTargetZoom( fL3 );
						_iCameraZoomState --;
					}
					break;
				}

				case 5:
				{
					if ( _eLastInputPressed == OIS::KC_EQUALS )
					{
						pCameraGameplay->SetTargetZoom( fL6 );
						_iCameraZoomState ++;
					}
					else if ( _eLastInputPressed == OIS::KC_MINUS )
					{
						pCameraGameplay->SetTargetZoom( fL4 );
						_iCameraZoomState --;
					}
					break;
				}

				case 6:
				{
					if ( _eLastInputPressed == OIS::KC_EQUALS )
					{
						pCameraGameplay->SetTargetZoom( fL7 );
						_iCameraZoomState ++;
					}
					else if ( _eLastInputPressed == OIS::KC_MINUS )
					{
						pCameraGameplay->SetTargetZoom( fL5 );
						_iCameraZoomState --;
					}
					break;
				}

				case 7:
				{
					if ( _eLastInputPressed == OIS::KC_MINUS )
					{
						pCameraGameplay->SetTargetZoom( fL6 );
						_iCameraZoomState --;
					}
					break;
				}
			}		
		}
	#else 

	#endif
}



/******************************************************************************
 FUNCTION:  UpdateInput
******************************************************************************/
void SoFsmStateMainCore::UpdateInput( float dt )
{
// 	if ( _eLastInputPressed == OIS::KC_ESCAPE )
// 		_eCurrentSubState = eSubState_ExitToResultsInit;
}



/******************************************************************************
 FUNCTION:  LoadGame
******************************************************************************/
void SoFsmStateMainCore::LoadGame()
{
	SoSaveSystem SaveSystem;
	SaveSystem.LoadGame();

	int iPreviousNodeId = -1;
	PostLoadOrGenerateProcess( iPreviousNodeId );
}



/******************************************************************************
 FUNCTION:  PostLoadOrGenerateProcess
******************************************************************************/
void SoFsmStateMainCore::PostLoadOrGenerateProcess( int iNodeIdOfPreviousLevel )
{
	//-----------------------
	// reset the camera
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	if ( pCameraManager )
	{
		SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
		SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();
		pCameraGameplay->SetZoom( SoUtils::GetSpriteZoom() );
		pCameraGameplay->ResetFocus();
	}
}



/******************************************************************************
 FUNCTION:  OnKeyPressed
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void SoFsmStateMainCore::OnKeyPressed( OIS::KeyCode Key )
	{
		_eLastInputPressed = Key;
	}
#endif


/******************************************************************************
 FUNCTION:  OnKeyReleased
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void SoFsmStateMainCore::OnKeyReleased( OIS::KeyCode Key )
	{

	}
#endif



/******************************************************************************
 FUNCTION:  OnTouchesDown
******************************************************************************/
bool SoFsmStateMainCore::OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) 
{
	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoFsmStateMainCore::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{	
	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesMove
******************************************************************************/
bool SoFsmStateMainCore::OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) 
{
	return false;
}



/******************************************************************************
 FUNCTION:  OnGestureDown
 *****************************************************************************/
bool SoFsmStateMainCore::OnGestureDown( SoGesture* pGesture )
{
	return false;
}



/******************************************************************************
 FUNCTION:  OnGestureUp
 *****************************************************************************/
bool SoFsmStateMainCore::OnGestureUp( SoGesture* pGesture )
{
	return false;
}



/******************************************************************************
 FUNCTION:  OnGestureMove
 *****************************************************************************/
bool SoFsmStateMainCore::OnGestureMove( SoGesture* pGesture )
{
	if ( _bIsZoomEnabled )
	{
		switch ( pGesture->_eGestureType )
		{
			case SoGesture::GESTURE_TYPE_ZOOM:
			case SoGesture::GESTURE_TYPE_PINCH:
			{
				float fDelta = 0.5f * ( pGesture->_vPositionDeltaPrevious[ 0 ].Length() + pGesture->_vPositionDeltaPrevious[ 1 ].Length() );
				fDelta *= GESTURE_ZOOM_SPEED * SoUtils::GetiPadRatio();

				if ( pGesture->_eGestureType == SoGesture::GESTURE_TYPE_PINCH )
					fDelta *= -1.0f;
				
				_fCameraZoom = Clamp( _fCameraZoom + fDelta, 0.25f, 2.0f );

				SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
				SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
				SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();
				pCameraGameplay->SetTargetZoom( _fCameraZoom );
				
				//CCLOG( "Delta: %f", fDelta );
				//CCLOG( "CameraZoom: %f", _fCameraZoom );

				return true;
				break;
			}
			default:
				break;
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnKeyPressed
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	bool SoFsmStateMainCore::OnMouseMoved( OIS::MouseEvent arg )
	{
		if ( _bIsZoomEnabled )
		{
			// Experimenting with disabling player control of the camera.
			// This seems better for Achoir.  mdomowicz 2014_02_02
			/*
			if ( arg.state.Z.rel > 0 )
				_eLastInputPressed = OIS::KC_EQUALS;
			else if ( arg.state.Z.rel < 0 )
				_eLastInputPressed = OIS::KC_MINUS;
			*/

			//if ( arg.state.Z.rel != 0 )
			//	CCLOG( "Z: %i", arg.state.Z.rel );

			return true;
		}

		return false;
	}
#endif



/******************************************************************************
 FUNCTION:  OnMessageFromGui
******************************************************************************/
void SoFsmStateMainCore::OnMessageFromGui( SoGuiMessage* pMessage )
{
	if ( pMessage )
	{
		if ( pMessage->GetType() == SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER )
		{
			hash hData = pMessage->FirstHash();

			if ( hData == JpHash( "StartGameButtonPressed" ) )
			{
				SoGuiManager* pGuiManager = SoGuiManager::GetInstance();
				pGuiManager->SendMessageToGui( _gGuiCoreMainLauncher, &SoGuiMessage::_BasicTransitionOut );
			}


			if ( hData == JpHash( "WinCoreMainLauncher_QuitGameButtonPressed" ) )
			{
				#if DCG_IS_TARGET_WIN32
					SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();
					pConsole->WriteFullHistoryToDisk();
					pConsole->ClearFullHistory();
				#endif

				_eCurrentSubState = eSubState_ExitToShellInit;
			}
		}

	}
}



/******************************************************************************
 FUNCTION:  TempDebugUpdate
******************************************************************************/
void SoFsmStateMainCore::TempDebugUpdate( float dt )
{
	SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();

	#if DCG_IS_TARGET_WIN32
		// Disable debug features for the final build!
		#ifdef _FINAL
			return;
		#endif

		if ( pConsole && !pConsole->IsShowing() )
		{
			if 
			( 
				_eLastInputPressed == OIS::KC_1 ||
				_eLastInputPressed == OIS::KC_2 ||
				_eLastInputPressed == OIS::KC_3 || 
				_eLastInputPressed == OIS::KC_4 || 
				_eLastInputPressed == OIS::KC_5 || 
				_eLastInputPressed == OIS::KC_6 ||
				_eLastInputPressed == OIS::KC_7 || 
				_eLastInputPressed == OIS::KC_8 || 
				_eLastInputPressed == OIS::KC_9 ||
				_eLastInputPressed == OIS::KC_0 ||

				_eLastInputPressed == OIS::KC_SYSRQ ||
				_eLastInputPressed == OIS::KC_P ||
				_eLastInputPressed == OIS::KC_O
			)
			{
				if ( _eLastInputPressed == OIS::KC_SYSRQ || _eLastInputPressed == OIS::KC_P )
					SoUtils::TakeScreenshot();

				if ( _eLastInputPressed == OIS::KC_O )
					CCTextureCache::sharedTextureCache()->dumpCachedTextureInfo();

			}
		}
	#endif // DCG_IS_TARGET_WIN32

	// Disable debug features for the final build!
	#ifdef _FINAL
		return;
	#endif

	if ( pConsole && pConsole->GetDebugSkipToLevelNumber() != -1 )
	{
	}
}












/******************************************************************************
 FUNCTION:  SoFsmMainCoreLayer
******************************************************************************/
SoFsmMainCoreLayer::SoFsmMainCoreLayer()
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmMainCoreLayer
******************************************************************************/
SoFsmMainCoreLayer::~SoFsmMainCoreLayer()
{
}


/******************************************************************************
 FUNCTION:  init
******************************************************************************/
bool SoFsmMainCoreLayer::init()
{
	if ( CCLayerColor::initWithColor( ccc4(0,0,0,0) ) )
	{
		CCDirector::sharedDirector()->setProjection(kCCDirectorProjection2D);
		CCSize size = CCDirector::sharedDirector()->getWinSize();

		_Console.Init( this );
		_Console.SetMaxOutputLines( 100 );

		scheduleUpdate();

		return true;
	}


	return false;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmMainCoreLayer::Deinit()
{
	_Console.Deinit();

	removeAllChildrenWithCleanup( DCG_CLEANUP_ALL_RUNNING_ACTIONS );
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoFsmMainCoreLayer::update( float dt )
{
	_Console.Update( dt );
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoFsmMainCoreLayer* SoFsmMainCoreLayer::create()
{
	SoFsmMainCoreLayer* pRet = new SoFsmMainCoreLayer();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}
