/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoConsole.h"
#include "SoFsmState.h"
#include "SoGuiManager.h"
#include "SoHudLog.h"
#include "SoInputGesture.h"
#include "SoInputMouse.h"
#include "SoInputTouch.h"
#include "SoScreenFader.h"
#include "SoUiButton.h"


class SoBillboardMist;
class SoFsmMainCoreLayer;
class SoGameClient;
class SoSkyPlane;
class SoSpritePlus;


class SoFsmStateMainCore : public SoFsmState
{
	public:

		//-----------------------
		// public enums
		enum eGameConnectionType
		{
			eGameConnectionType_Offline,
			eGameConnectionType_Online_Server,
			eGameConnectionType_Online_Client,
			eGameConnectionType_Last
		};


		enum eSubState
		{
			eSubState_CreateNewGame,
			eSubState_Main,
			eSubState_GameMenu,
			eSubState_FadeInInit,
			eSubState_FadeInUpdate,
			eSubState_ExitToShellInit,
			eSubState_ExitToShellUpdate,
			eSubState_SwitchMapInit,
			eSubState_SwitchMap,
			eSubState_Online_Server_Add_Players,
			eSubState_Last
		};


		//-----------------------
		// public functions

		SoFsmStateMainCore();

		virtual ~SoFsmStateMainCore();


		virtual void Init();
		virtual void Deinit();
		virtual hash Update( float dt );
		void         UpdateInput( float dt );


		// Because SoFsmStateMainCore is getting input apart from the InputDispatcher,
		// we don't have central control over when this class receives input.
		// So we need this hack to control when zooming is available.
		static void SetIsZoomEnabled( bool b ) { _bIsZoomEnabled = b; }

		static eGameConnectionType GetGameConnectionType() { return _eGameConnectionType; }
		static void SetGameConnectionType( eGameConnectionType e ) { _eGameConnectionType = e; }
		static void SetOnlineServerName( std::string s ) { _sOnlineServerName = s; }
		static void SetLoadGameIfPossible( bool b ) { _bLoadGameIfPossible = b; }
		static void SetPlayerType1( hash h ) { _hPlayerType1 = h; }
		static void SetPlayerType2( hash h ) { _hPlayerType2 = h; }
		static void SetPlayerType3( hash h ) { _hPlayerType3 = h; }
		static void SetPlayerType4( hash h ) { _hPlayerType4 = h; }
		static void SetPlayerType5( hash h ) { _hPlayerType5 = h; }
		static void SetPlayerType6( hash h ) { _hPlayerType6 = h; }

		static void SetIsVisibleStock( bool b ) { _bIsVisibleStock = b; }
		static void SetIsVisibleMoney( bool b ) { _bIsVisibleMoney = b; }


		static void LoadGame();
		static void PostLoadOrGenerateProcess( int iNodeIdOfPreviousLevel );

		static SoGuiManager* GetCustomGuiManager() { return _pCustomGuiManager; }

		#if DCG_IS_TARGET_WIN32
			// SoInputKeyboardListener callbacks
			virtual void OnKeyPressed( OIS::KeyCode Key );
			virtual void OnKeyReleased( OIS::KeyCode Key );
		
			virtual bool OnMousePressed( OIS::MouseEvent arg, OIS::MouseButtonID id ) { return false; }
			virtual bool OnMouseReleased( OIS::MouseEvent arg, OIS::MouseButtonID id ) { return false; }
			virtual bool OnMouseMoved( OIS::MouseEvent arg );
		#endif

		// SoInputTouch callbacks
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList );
		
		// SoInputGesture callbacks
		virtual bool OnGestureDown( SoGesture* pGesture );
		virtual bool OnGestureUp( SoGesture* pGesture );
		virtual bool OnGestureMove( SoGesture* pGesture );

		// SoGui callbacks
		virtual void OnMessageFromGui( SoGuiMessage* pMessage );

	
	private:

		//-----------------------
		// private variables

		SoFsmMainCoreLayer* _pLayer;
		eSubState _eCurrentSubState;

		#if DCG_IS_TARGET_WIN32
			OIS::KeyCode _eLastInputPressed;

			SoInputMouseListener< SoFsmStateMainCore > _InputMouseListener;
		#endif

		SoInputGestureListener< SoFsmStateMainCore > _InputGestureListener;
		SoInputTouchListener< SoFsmStateMainCore > _InputTouchListener;

		SoScreenFader* _pScreenFader;

		SoSpritePlus* _pGameOverSplash;
		float _fGameOverSplashTimer;

		SoSpritePlus* _pGameWonSplash;
		float _fGameWonSplashTimer;

		SoBillboardMist* _pBillboardMist;

		int _iCameraZoomState;
		float _fCameraZoom;

		SoSkyPlane* _pSkyPlane;

		SoGuiListener< SoFsmStateMainCore > _GuiListener;
		guid _gGuiCoreMainLauncher;

		static hash _hPlayerType1;
		static hash _hPlayerType2;
		static hash _hPlayerType3;
		static hash _hPlayerType4;
		static hash _hPlayerType5;
		static hash _hPlayerType6;

		static bool _bIsVisibleStock;
		static bool _bIsVisibleMoney;

		static eGameConnectionType _eGameConnectionType;
		static std::string _sOnlineServerName;

		static bool _bIsZoomEnabled;

		static bool _bLoadGameIfPossible;

		SoGameClient* _pClient;
		bool _bOnlineServerGamesLocalHumanPlayerAdded;
		int _iPlayerAddingState;


		CCNode* _pCustomGuiManagerNode;
		static SoGuiManager* _pCustomGuiManager;

		//-----------------------
		// private functions

		void MainGameUpdate( float dt );

		void UpdateWorkerParkingSpots( float dt );

		void CreatePlayersBasedOnIniFile( SoGameClient* pClient );
		void CreatePlayersBasedOnInGameSetup( SoGameClient* pClient );

		void UpdateCameraZoomSetting( float dt );
		void TempDebugUpdate( float dt );


		bool OnlineServerGameAddPlayers();
		bool AllowFarOutZoom();
};






class SoFsmMainCoreLayer : public cocos2d::CCLayerColor
{
	public:

		//-----------------------
		// public functions

		SoFsmMainCoreLayer();
		virtual ~SoFsmMainCoreLayer();

		virtual bool init();
		void Deinit();

		virtual void update( float dt );

		LAYER_NODE_FUNC(SoFsmMainCoreLayer);

		static SoFsmMainCoreLayer* create();


	private:

		//-----------------------
		// private variables
		SoConsole _Console;
};