/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFsmStateMainCore.h"
#include "SoGameGlobals.h"
#include "SoGuiAnimWinNewOrLoadGame.h"
#include "SoGuiAnimWinSetupGame.h"
#include "SoGuiAnimWinSetupPlayers.h"
#include "SoGuiMessage.h"
#include "SoFontDefs.h"
#include "SoFsmStateSetupOfflineGame.h"
#include "SoSoundManager.h"
#include "SoSprite.h"
#include "SoSpriteAnim.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoFsmStateSetupOfflineGame
******************************************************************************/
SoFsmStateSetupOfflineGame::SoFsmStateSetupOfflineGame() :
_pLayer( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmStateSetupOfflineGame
******************************************************************************/
SoFsmStateSetupOfflineGame::~SoFsmStateSetupOfflineGame()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFsmStateSetupOfflineGame::Init()
{
	SoFsmState::Init();

	if ( !SoSoundManager::GetInstance()->GetMusicTrack() )
	{
		SoSoundManager::GetInstance()->LoadMusicTrack( SoUtils::AddRelativePathToSfx( "MusicTrackTitle.ogg" ) );
		SoSoundManager::GetInstance()->PlayMusic();
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmStateSetupOfflineGame::Deinit()
{
	if ( _pLayer )
	{
		_pLayer->Deinit();
		_pLayer->removeFromParentAndCleanup( true );
		_pLayer->release();
		_pLayer = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmStateSetupOfflineGame::Update( float dt )
{
	if ( !_pLayer )
	{
		_pLayer = SoFsmSetupOfflineGameLayer::create();
		_pLayer->retain();
		CCDirector::sharedDirector()->getRunningScene()->addChild(_pLayer);
	}

	if ( _pLayer->IsReadyToExitState() )
	{
		SoFsmStateManager* pStateMgr = GetParentFsmStateManager();
		if ( _pLayer->IsStartGameSelected() || _pLayer->IsLoadGameSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "InGame" ) );
		else if ( _pLayer->IsPlayerSetupBackSelected() || _pLayer->IsNewOrLoadGameBackSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "Shell" ) );
	}

	return 0;
}









/******************************************************************************
 FUNCTION:  SoFsmSetupOfflineGameLayer
******************************************************************************/
SoFsmSetupOfflineGameLayer::SoFsmSetupOfflineGameLayer() : 
_bReadyToExitState( false ),
_bPlayerSetupOkSelected( false ),
_bPlayerSetupBackSelected( false ),
_bGameSetupStartSelected( false ),
_bGameSetupBackSelected( false ),
_bNewGameSelected( false ),
_bLoadGameSelected( false ),
_bNewOrLoadGameBackSelected( false ),
_pScreenFader( NULL ),
_iCurrentState( 0 ),
_gGuiNewOrLoadGame( 0 ),
_gGuiSetupPlayers( 0 ),
_pWinNewOrLoadGame( NULL ),
_pWinSetupPlayers( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmSetupOfflineGameLayer
******************************************************************************/
SoFsmSetupOfflineGameLayer::~SoFsmSetupOfflineGameLayer()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  init
******************************************************************************/
bool SoFsmSetupOfflineGameLayer::init()
{
	if ( CCLayerColor::initWithColor( ccc4(0,0,0,255) ) )
	{
		scheduleUpdate();

		setPosition( 0.0f, 0.0f );

		_bReadyToExitState = false;
		_bPlayerSetupOkSelected = false;
		_bPlayerSetupBackSelected = false;
		_bGameSetupStartSelected = false;
		_bGameSetupBackSelected = false;
		_bNewGameSelected = false;
		_bLoadGameSelected = false;
		_bNewOrLoadGameBackSelected = false;

		CCSize size = CCDirector::sharedDirector()->getWinSize();
		float fPosX = 0.0f;
		float fPosY = 0.0f;


		// Background image
		SoSprite* pSprite = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "LogoBackground2.png" ) );
		pSprite->setPosition( ccp(size.width/2, size.height/2) );

		CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
		if ( SizeInPixels.width > SizeInPixels.height )
		{
			float fExactScreenWidth = SizeInPixels.width / pSprite->getContentSize().width;
			pSprite->setScaleX( fExactScreenWidth );
			pSprite->setScaleY( fExactScreenWidth );
		}
		else
		{
			float fExactScreenHeight = SizeInPixels.height / pSprite->getContentSize().height;
			pSprite->setScaleX( fExactScreenHeight );
			pSprite->setScaleY( fExactScreenHeight );
		}

		addChild(pSprite, 0);


		int iRenderOrder = 0;
		_pScreenFader = SoScreenFader::create();
		_pScreenFader->retain();
		_pScreenFader->Setup( this, iRenderOrder );
		_pScreenFader->SetFadeSpeed( 2.0f );
		_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_OPAQUE_TO_TRANSPARENT );
		_pScreenFader->BeginFade();


		_GuiManager.Init( this, SoGuiManager::INPUT_SOURCE_INTERNAL );
		_GuiManager.SetHeadGuiAbsolutionPositionToScreenCenter();

		_pWinNewOrLoadGame = new SoGuiAnimWinNewOrLoadGame;
		_gGuiNewOrLoadGame = _GuiManager.AddGui( _pWinNewOrLoadGame, NULL );
		_pWinNewOrLoadGame->AddListener( &_GuiListener );

		_pWinSetupPlayers = new SoGuiAnimWinSetupPlayers;
		_gGuiSetupPlayers = _GuiManager.AddGui( _pWinSetupPlayers, NULL );
		_pWinSetupPlayers->AddListener( &_GuiListener );

		_pWinSetupGame = new SoGuiAnimWinSetupGame;
		_gGuiSetupGame = _GuiManager.AddGui( _pWinSetupGame, NULL );
		_pWinSetupGame->AddListener( &_GuiListener );

		_GuiListener.SetOwner( this );

		_WatermarkPreRelease.Init( this );

		return true;
	}


	return false;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmSetupOfflineGameLayer::Deinit()
{
	// This removes and deletes all Guis.  Deleted Guis remove their listeners.
	_GuiManager.Deinit();

	_pWinNewOrLoadGame = NULL;
	_pWinSetupPlayers = NULL;
	_pWinSetupGame = NULL;

	if ( _pScreenFader )
	{
		_pScreenFader->Deinit();
		_pScreenFader->release();
		_pScreenFader = NULL;
	}

	_WatermarkPreRelease.Deinit();

	removeAllChildrenWithCleanup( DCG_CLEANUP_ALL_RUNNING_ACTIONS );
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoFsmSetupOfflineGameLayer::update( float dt )
{
	_GuiManager.Update( dt );

	_pScreenFader->Update( dt );


	const int STATE_INITIAL_FADE_IN = 0;
	const int STATE_NEW_OR_LOAD_GAME_INIT = 10;
	const int STATE_NEW_OR_LOAD_GAME_UPDATE = 20;
	const int STATE_SETUP_PLAYERS_INIT = 30;
	const int STATE_SETUP_PLAYERS_UPDATE = 40;
	const int STATE_SETUP_GAME = 45;
	const int STATE_FADE_OUT_INIT = 50;
	const int STATE_FADE_OUT = 60;


	switch ( _iCurrentState )
	{
		case STATE_INITIAL_FADE_IN:
		{
			if ( _pScreenFader->IsDone() )
			{

				if ( SoGameGlobals::DoesSaveGameExist() )
				{
					_iCurrentState = STATE_NEW_OR_LOAD_GAME_INIT;
				}
				else
				{
					_iCurrentState = STATE_SETUP_PLAYERS_INIT;
				}
			}
			break;
		}

		case STATE_NEW_OR_LOAD_GAME_INIT:
		{
			_bNewGameSelected = false;
			_bLoadGameSelected = false;
			_GuiManager.SendMessageToGui( _gGuiNewOrLoadGame, &SoGuiMessage::_BasicTransitionIn );
			_iCurrentState = STATE_NEW_OR_LOAD_GAME_UPDATE;
			break;
		}


		case STATE_NEW_OR_LOAD_GAME_UPDATE:
		{
			if ( _bNewOrLoadGameBackSelected )
			{
				_GuiManager.SendMessageToGui( _gGuiNewOrLoadGame, &SoGuiMessage::_BasicTransitionOut );
				_iCurrentState = STATE_FADE_OUT_INIT;
			}
			else if ( _bNewGameSelected )
			{
				_GuiManager.SendMessageToGui( _gGuiNewOrLoadGame, &SoGuiMessage::_BasicTransitionOut );
				_iCurrentState = STATE_SETUP_PLAYERS_INIT;
			}
			else if ( _bLoadGameSelected )
			{
				_GuiManager.SendMessageToGui( _gGuiNewOrLoadGame, &SoGuiMessage::_BasicTransitionOut );
				SoFsmStateMainCore::SetGameConnectionType( SoFsmStateMainCore::eGameConnectionType_Offline );
				SoFsmStateMainCore::SetLoadGameIfPossible( true );
				_iCurrentState = STATE_FADE_OUT_INIT;
			}

			break;
		}

		case STATE_SETUP_PLAYERS_INIT:
		{
			_GuiManager.SendMessageToGui( _gGuiSetupPlayers, &SoGuiMessage::_BasicTransitionIn );
			_iCurrentState = STATE_SETUP_PLAYERS_UPDATE;
			break;
		}

		case STATE_SETUP_PLAYERS_UPDATE:
		{
			if ( _bPlayerSetupOkSelected )
			{
				SoFsmStateMainCore::SetGameConnectionType( SoFsmStateMainCore::eGameConnectionType_Offline );
				SoFsmStateMainCore::SetLoadGameIfPossible( false );
				SoFsmStateMainCore::SetPlayerType1( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 0 ) );
				SoFsmStateMainCore::SetPlayerType2( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 1 ) );
				SoFsmStateMainCore::SetPlayerType3( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 2 ) );
				SoFsmStateMainCore::SetPlayerType4( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 3 ) );
				SoFsmStateMainCore::SetPlayerType5( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 4 ) );
				SoFsmStateMainCore::SetPlayerType6( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 5 ) );
			}

			if ( _bPlayerSetupOkSelected )
			{
				_bPlayerSetupOkSelected = false;
				_GuiManager.SendMessageToGui( _gGuiSetupPlayers, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gGuiSetupGame, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_SETUP_GAME;
			}
			else if ( _bPlayerSetupBackSelected )
			{
				_GuiManager.SendMessageToGui( _gGuiSetupPlayers, &SoGuiMessage::_BasicTransitionOut );
				if ( SoGameGlobals::DoesSaveGameExist() )
				{
					_bPlayerSetupBackSelected = false;
					_iCurrentState = STATE_NEW_OR_LOAD_GAME_INIT;						
				}
				else
				{
					_iCurrentState = STATE_FADE_OUT_INIT;						
				}
			}

			break;
		}

		case STATE_SETUP_GAME:
		{
			if ( _bGameSetupBackSelected )
			{
				_bGameSetupBackSelected = false;

				_GuiManager.SendMessageToGui( _gGuiSetupPlayers, &SoGuiMessage::_BasicTransitionIn );
				_GuiManager.SendMessageToGui( _gGuiSetupGame, &SoGuiMessage::_BasicTransitionOut );

				_iCurrentState = STATE_SETUP_PLAYERS_INIT;
			}

			if ( _bGameSetupStartSelected )
			{
				SoFsmStateMainCore::SetIsVisibleStock( _pWinSetupGame->GetComboButtonValue( 0 ) == JpHash( "yes" ) );
				SoFsmStateMainCore::SetIsVisibleMoney( _pWinSetupGame->GetComboButtonValue( 1 ) == JpHash( "yes" ) );

				_GuiManager.SendMessageToGui( _gGuiSetupGame, &SoGuiMessage::_BasicTransitionOut );
				_iCurrentState = STATE_FADE_OUT_INIT;
			}

			break;
		}

		case STATE_FADE_OUT_INIT:
		{
			_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_TRANSPARENT_TO_OPAQUE );
			_pScreenFader->BeginFade();
			_iCurrentState = STATE_FADE_OUT;
			break;
		}
		case STATE_FADE_OUT:
		{
			if ( _pScreenFader->IsDone() )
				_bReadyToExitState = true;

			break;
		}

		default:
			ASSERTF( 0, ( "\nUnknown state." ) );
	}

}



/******************************************************************************
 FUNCTION:  draw
******************************************************************************/
void SoFsmSetupOfflineGameLayer::draw()
{
	CCLayerColor::draw();
}



/******************************************************************************
 FUNCTION:  OnMessageFromGui
******************************************************************************/
void SoFsmSetupOfflineGameLayer::OnMessageFromGui( SoGuiMessage* pMessage )
{
	if ( pMessage )
	{
		if ( pMessage->GetType() == SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER )
		{
			hash hData = pMessage->FirstHash();


			if ( hData == JpHash( "SetupPlayerNextButtonPressed" ) )
			{
				if ( CountValidPlayers() >= 2 )
				{
					_bPlayerSetupOkSelected = true;
				}
			}

			if ( hData == JpHash( "NewGameButtonPressed" ) )
			{
				_bNewGameSelected = true;
			}

			if ( hData == JpHash( "LoadGameButtonPressed" ) )
			{
				_bLoadGameSelected = true;
			}

			if ( hData == JpHash( "SetupPlayerBackButtonPressed" ) )
			{
				_bPlayerSetupBackSelected = true;
			}

			if ( hData == JpHash( "SetupGameStartButtonPressed" ) )
			{
				_bGameSetupStartSelected = true;
			}

			if ( hData == JpHash( "SetupGameBackButtonPressed" ) )
			{
				_bGameSetupBackSelected = true;
			}

			if ( hData == JpHash( "NewOrLoadBackButtonPressed" ) )
			{
				_bNewOrLoadGameBackSelected = true;
			}

		}
	}
}



/******************************************************************************
 FUNCTION:  OnUiButtonDown
******************************************************************************/
void SoFsmSetupOfflineGameLayer::OnUiButtonDown( SoUiButton* pButton )
{
}



/******************************************************************************
 FUNCTION:  OnUiButtonUp
******************************************************************************/
void SoFsmSetupOfflineGameLayer::OnUiButtonUp( SoUiButton* pButton )
{
}



/******************************************************************************
 FUNCTION:  CountValidPlayers
******************************************************************************/
int SoFsmSetupOfflineGameLayer::CountValidPlayers()
{
	int iNumPlayers = 0;

	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 0 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 1 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 2 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 3 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 4 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 5 ) != JpHash( "None" ) )
		iNumPlayers ++;

	return iNumPlayers;
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoFsmSetupOfflineGameLayer* SoFsmSetupOfflineGameLayer::create()
{
	SoFsmSetupOfflineGameLayer* pRet = new SoFsmSetupOfflineGameLayer();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}





















