/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFsmStateMainCore.h"
#include "SoGuiAnimWinCreateOrJoinGame.h"
#include "SoGuiAnimWinEnterGameName.h"
#include "SoGuiAnimWinSetupGame.h"
#include "SoGuiAnimWinSetupPlayers.h"
#include "SoGuiMessage.h"
#include "SoFontDefs.h"
#include "SoFsmStateSetupOnlineGame.h"
#include "SoSoundManager.h"
#include "SoSprite.h"
#include "SoSpriteAnim.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoFsmStateSetupOnlineGame
******************************************************************************/
SoFsmStateSetupOnlineGame::SoFsmStateSetupOnlineGame() :
_pLayer( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmStateSetupOnlineGame
******************************************************************************/
SoFsmStateSetupOnlineGame::~SoFsmStateSetupOnlineGame()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFsmStateSetupOnlineGame::Init()
{
	SoFsmState::Init();

	if ( !SoSoundManager::GetInstance()->GetMusicTrack() )
	{
		SoSoundManager::GetInstance()->LoadMusicTrack( SoUtils::AddRelativePathToSfx( "MusicTrackTitle.ogg" ) );
		SoSoundManager::GetInstance()->PlayMusic();
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmStateSetupOnlineGame::Deinit()
{
	if ( _pLayer )
	{
		_pLayer->Deinit();
		_pLayer->removeFromParentAndCleanup( true );
		_pLayer->release();
		_pLayer = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmStateSetupOnlineGame::Update( float dt )
{
	if ( !_pLayer )
	{
		_pLayer = SoFsmSetupOnlineGameLayer::create();
		_pLayer->retain();
		CCDirector::sharedDirector()->getRunningScene()->addChild(_pLayer);
	}

	if ( _pLayer->IsReadyToExitState() )
	{
		SoFsmStateManager* pStateMgr = GetParentFsmStateManager();
		if ( _pLayer->IsStartGameSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "InGame" ) );
		else if ( _pLayer->IsJoinGameSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "InGame" ) );
		else if ( _pLayer->IsBackSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "Shell" ) );
	}

	return 0;
}









/******************************************************************************
 FUNCTION:  SoFsmSetupOnlineGameLayer
******************************************************************************/
SoFsmSetupOnlineGameLayer::SoFsmSetupOnlineGameLayer() : 
_bReadyToExitState( false ),
_bCreateGameSelected( false ),
_bJoinGameSelected( false ),
_bCreateOrJoinBackSelected( false ),
_bPlayerSetupOkSelected( false ),
_bPlayerSetupBackSelected( false ),
_bGameSetupStartSelected( false ),
_bGameSetupBackSelected( false ),
_bGameNameNextSelected( false ),
_bGameNameBackSelected( false ),
_pScreenFader( NULL ),
_iCurrentState( 0 ),
_gWinSetupPlayers( 0 ),
_gWinSetupGame( 0 ),
_gWinCreateOrJoinGame( 0 ),
_gWinEnterGameName( 0 ),
_pWinSetupPlayers( NULL ),
_pWinSetupGame( NULL ),
_pWinCreateOrJoinGame( NULL ),
_pWinEnterGameName( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmSetupOnlineGameLayer
******************************************************************************/
SoFsmSetupOnlineGameLayer::~SoFsmSetupOnlineGameLayer()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  init
******************************************************************************/
bool SoFsmSetupOnlineGameLayer::init()
{
	if ( CCLayerColor::initWithColor( ccc4(0,0,0,255) ) )
	{
		scheduleUpdate();

		setPosition( 0.0f, 0.0f );

		_bReadyToExitState = false;
		_bCreateGameSelected = false;
		_bJoinGameSelected = false;
		_bCreateOrJoinBackSelected = false;
		_bPlayerSetupOkSelected = false;
		_bPlayerSetupBackSelected = false;
		_bGameSetupStartSelected = false;
		_bGameSetupBackSelected = false;
		_bGameNameNextSelected = false;
		_bGameNameBackSelected = false;

		CCSize size = CCDirector::sharedDirector()->getWinSize();
		float fPosX = 0.0f;
		float fPosY = 0.0f;


		// Background image
		SoSprite* pSprite = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "LogoBackground2.png" ) );
		pSprite->setPosition( ccp(size.width/2, size.height/2) );

		CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
		if ( SizeInPixels.width > SizeInPixels.height )
		{
			float fExactScreenWidth = SizeInPixels.width / pSprite->getContentSize().width;
			pSprite->setScaleX( fExactScreenWidth );
			pSprite->setScaleY( fExactScreenWidth );
		}
		else
		{
			float fExactScreenHeight = SizeInPixels.height / pSprite->getContentSize().height;
			pSprite->setScaleX( fExactScreenHeight );
			pSprite->setScaleY( fExactScreenHeight );
		}

		addChild(pSprite, 0);


		int iRenderOrder = 0;
		_pScreenFader = SoScreenFader::create();
		_pScreenFader->retain();
		_pScreenFader->Setup( this, iRenderOrder );
		_pScreenFader->SetFadeSpeed( 2.0f );
		_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_OPAQUE_TO_TRANSPARENT );
		_pScreenFader->BeginFade();


		_GuiManager.Init( this, SoGuiManager::INPUT_SOURCE_INTERNAL );
		_GuiManager.SetHeadGuiAbsolutionPositionToScreenCenter();

		_pWinSetupPlayers = new SoGuiAnimWinSetupPlayers;
		_gWinSetupPlayers = _GuiManager.AddGui( _pWinSetupPlayers, NULL );
		_pWinSetupPlayers->AddListener( &_GuiListener );

		_pWinSetupGame = new SoGuiAnimWinSetupGame;
		_gWinSetupGame = _GuiManager.AddGui( _pWinSetupGame, NULL );
		_pWinSetupGame->AddListener( &_GuiListener );

		_pWinCreateOrJoinGame = new SoGuiAnimWinCreateOrJoinGame;
		_gWinCreateOrJoinGame = _GuiManager.AddGui( _pWinCreateOrJoinGame, NULL );
		_pWinCreateOrJoinGame->AddListener( &_GuiListener );

		_pWinEnterGameName = new SoGuiAnimWinEnterGameName;
		_gWinEnterGameName = _GuiManager.AddGui( _pWinEnterGameName, NULL );
		_pWinEnterGameName->AddListener( &_GuiListener );


		_GuiListener.SetOwner( this );

		_WatermarkPreRelease.Init( this );

		return true;
	}


	return false;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmSetupOnlineGameLayer::Deinit()
{
	// This removes and deletes all Guis.  Deleted Guis remove their listeners.
	_GuiManager.Deinit();

	_pWinSetupPlayers = NULL;
	_pWinSetupGame = NULL;

	if ( _pScreenFader )
	{
		_pScreenFader->Deinit();
		_pScreenFader->release();
		_pScreenFader = NULL;
	}

	_WatermarkPreRelease.Deinit();

	removeAllChildrenWithCleanup( DCG_CLEANUP_ALL_RUNNING_ACTIONS );
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoFsmSetupOnlineGameLayer::update( float dt )
{
	_GuiManager.Update( dt );

	_pScreenFader->Update( dt );


	const int STATE_INITIAL_FADE_IN = 0;
	const int STATE_MAIN_UPDATE = 10;
	const int STATE_ENTER_NAME_UPDATE = 20;
	const int STATE_SETUP_PLAYERS_UPDATE = 30;
	const int STATE_SETUP_GAME_UPDATE = 40;
	const int STATE_FADE_OUT = 50;

	switch ( _iCurrentState )
	{
		case STATE_INITIAL_FADE_IN:
		{
			if ( _pScreenFader->IsDone() )
			{
				_GuiManager.SendMessageToGui( _gWinCreateOrJoinGame, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_MAIN_UPDATE;
			}
			break;
		}

		case STATE_MAIN_UPDATE:
		{
			if ( _bCreateOrJoinBackSelected )
			{
				_GuiManager.SendMessageToGui( _gWinCreateOrJoinGame, &SoGuiMessage::_BasicTransitionOut );

				_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_TRANSPARENT_TO_OPAQUE );
				_pScreenFader->BeginFade();
				_iCurrentState = STATE_FADE_OUT;
			}

			if ( _bCreateGameSelected )
			{
				_GuiManager.SendMessageToGui( _gWinCreateOrJoinGame, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gWinEnterGameName, &SoGuiMessage::_BasicTransitionIn );

				_pWinEnterGameName->SetIsJoiningServer( false );
				_pWinEnterGameName->SetIsCreatingServer( true );

				_iCurrentState = STATE_ENTER_NAME_UPDATE;
			}

			if ( _bJoinGameSelected )
			{
				_GuiManager.SendMessageToGui( _gWinCreateOrJoinGame, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gWinEnterGameName, &SoGuiMessage::_BasicTransitionIn );

				_pWinEnterGameName->SetIsJoiningServer( true );
				_pWinEnterGameName->SetIsCreatingServer( false );

				_iCurrentState = STATE_ENTER_NAME_UPDATE;
			}

			break;
		}

		case STATE_ENTER_NAME_UPDATE:
		{
			if ( _bGameNameBackSelected )
			{
				_bGameNameBackSelected = false;
				_bJoinGameSelected = false;
				_bCreateGameSelected = false;

				_GuiManager.SendMessageToGui( _gWinEnterGameName, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gWinCreateOrJoinGame, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_MAIN_UPDATE;
			}

			if ( _bGameNameNextSelected )
			{
				_bGameNameNextSelected = false;

				if ( _bCreateGameSelected )
				{ 
					_GuiManager.SendMessageToGui( _gWinEnterGameName, &SoGuiMessage::_BasicTransitionOut );
					_GuiManager.SendMessageToGui( _gWinSetupPlayers, &SoGuiMessage::_BasicTransitionIn );

					_iCurrentState = STATE_SETUP_PLAYERS_UPDATE;
				}
				else if ( _bJoinGameSelected )
				{
					//-----------------------
					// This is where the user has finally committed to Joining a game.
					SoFsmStateMainCore::SetGameConnectionType( SoFsmStateMainCore::eGameConnectionType_Online_Client );
					SoFsmStateMainCore::SetOnlineServerName( _pWinEnterGameName->GetEnteredGameName() );

					_GuiManager.SendMessageToGui( _gWinEnterGameName, &SoGuiMessage::_BasicTransitionOut );

					_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_TRANSPARENT_TO_OPAQUE );
					_pScreenFader->BeginFade();
					_iCurrentState = STATE_FADE_OUT;
				}
			}

			break;
		}

		case STATE_SETUP_PLAYERS_UPDATE:
		{
			if ( _bPlayerSetupBackSelected )
			{
				_bPlayerSetupBackSelected = false;

				_GuiManager.SendMessageToGui( _gWinSetupPlayers, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gWinEnterGameName, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_ENTER_NAME_UPDATE;
			}

			if ( _bPlayerSetupOkSelected )
			{
				_bPlayerSetupOkSelected = false;

				//-----------------------
				// This is where the user has finally committed to Creating a game.
				SoFsmStateMainCore::SetGameConnectionType( SoFsmStateMainCore::eGameConnectionType_Online_Server );
				SoFsmStateMainCore::SetOnlineServerName( _pWinEnterGameName->GetEnteredGameName() );

				SoFsmStateMainCore::SetPlayerType1( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 0 ) );
				SoFsmStateMainCore::SetPlayerType2( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 1 ) );
				SoFsmStateMainCore::SetPlayerType3( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 2 ) );
				SoFsmStateMainCore::SetPlayerType4( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 3 ) );
				SoFsmStateMainCore::SetPlayerType5( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 4 ) );
				SoFsmStateMainCore::SetPlayerType6( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 5 ) );

				_GuiManager.SendMessageToGui( _gWinSetupPlayers, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gWinSetupGame, &SoGuiMessage::_BasicTransitionIn );


				//_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_TRANSPARENT_TO_OPAQUE );
				_iCurrentState = STATE_SETUP_GAME_UPDATE;
			}

			break;
		}

		case STATE_SETUP_GAME_UPDATE:
		{
			if ( _bGameSetupBackSelected )
			{
				_bGameSetupBackSelected = false;

				_GuiManager.SendMessageToGui( _gWinSetupPlayers, &SoGuiMessage::_BasicTransitionIn );
				_GuiManager.SendMessageToGui( _gWinSetupGame, &SoGuiMessage::_BasicTransitionOut );

				_iCurrentState = STATE_SETUP_PLAYERS_UPDATE;
			}

			if ( _bGameSetupStartSelected )
			{
				SoFsmStateMainCore::SetIsVisibleStock( _pWinSetupGame->GetComboButtonValue( 0 ) == JpHash( "yes" ) );
				SoFsmStateMainCore::SetIsVisibleMoney( _pWinSetupGame->GetComboButtonValue( 1 ) == JpHash( "yes" ) );

				_GuiManager.SendMessageToGui( _gWinSetupGame, &SoGuiMessage::_BasicTransitionOut );
				_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_TRANSPARENT_TO_OPAQUE );
				_pScreenFader->BeginFade();
				_iCurrentState = STATE_FADE_OUT;
			}

			break;
		}

		case STATE_FADE_OUT:
		{
			if ( _pScreenFader->IsDone() )
				_bReadyToExitState = true;

			break;
		}

		default:
			ASSERTF( 0, ( "\nUnknown state." ) );
	}

}



/******************************************************************************
 FUNCTION:  draw
******************************************************************************/
void SoFsmSetupOnlineGameLayer::draw()
{
	CCLayerColor::draw();
}



/******************************************************************************
 FUNCTION:  OnMessageFromGui
******************************************************************************/
void SoFsmSetupOnlineGameLayer::OnMessageFromGui( SoGuiMessage* pMessage )
{
	if ( pMessage )
	{
		if ( pMessage->GetType() == SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER )
		{
			hash hData = pMessage->FirstHash();

			if ( hData == JpHash( "CreateGameButtonPressed" ) )
			{
				_bCreateGameSelected = true;
			}

			if ( hData == JpHash( "JoinGameButtonPressed" ) )
			{
				_bJoinGameSelected = true;
			}

			if ( hData == JpHash( "CreateOrJoinBackButtonPressed" ) )
			{
				_bCreateOrJoinBackSelected = true;
			}
			

			if ( hData == JpHash( "SetupPlayerNextButtonPressed" ) )
			{
				if ( CountValidPlayers() >= 2 )
				{
					_bPlayerSetupOkSelected = true;
				}
			}

			if ( hData == JpHash( "SetupPlayerBackButtonPressed" ) )
			{
				_bPlayerSetupBackSelected = true;
			}


			if ( hData == JpHash( "GameNameBackButtonPressed" ) )
			{
				_bGameNameBackSelected = true;
			}

			if ( hData == JpHash( "GameNameNextButtonPressed" ) )
			{
				_bGameNameNextSelected = true;
			}

			if ( hData == JpHash( "SetupGameStartButtonPressed" ) )
			{
				_bGameSetupStartSelected = true;
			}

			if ( hData == JpHash( "SetupGameBackButtonPressed" ) )
			{
				_bGameSetupBackSelected = true;
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  OnUiButtonDown
******************************************************************************/
void SoFsmSetupOnlineGameLayer::OnUiButtonDown( SoUiButton* pButton )
{
}



/******************************************************************************
 FUNCTION:  OnUiButtonUp
******************************************************************************/
void SoFsmSetupOnlineGameLayer::OnUiButtonUp( SoUiButton* pButton )
{
}



/******************************************************************************
 FUNCTION:  CountValidPlayers
******************************************************************************/
int SoFsmSetupOnlineGameLayer::CountValidPlayers()
{
	int iNumPlayers = 0;

	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 0 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 1 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 2 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 3 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 4 ) != JpHash( "None" ) )
		iNumPlayers ++;
	if ( _pWinSetupPlayers->GetPlayerTypeComboButtonValue( 5 ) != JpHash( "None" ) )
		iNumPlayers ++;

	return iNumPlayers;
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoFsmSetupOnlineGameLayer* SoFsmSetupOnlineGameLayer::create()
{
	SoFsmSetupOnlineGameLayer* pRet = new SoFsmSetupOnlineGameLayer();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}





















