/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoGuiAnimWinOptions.h"
#include "SoGuiAnimWinRules.h"
#include "SoGuiAnimWinShellMain.h"
#include "SoGuiAnimWinShellMainMore.h"
#include "SoGuiAnimWinStoreBadges.h"
#include "SoGuiMessage.h"
#include "SoFontDefs.h"
#include "SoFsmStateShell.h"
#include "SoSoundManager.h"
#include "SoSprite.h"
#include "SoSpriteAnim.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoFsmStateShell
******************************************************************************/
SoFsmStateShell::SoFsmStateShell() :
_pLayer( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmStateShell
******************************************************************************/
SoFsmStateShell::~SoFsmStateShell()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFsmStateShell::Init()
{
	SoFsmState::Init();

	if ( !SoSoundManager::GetInstance()->GetMusicTrack() )
	{
		SoSoundManager::GetInstance()->LoadMusicTrack( SoUtils::AddRelativePathToSfx( "MusicTrackTitle.ogg" ) );
		SoSoundManager::GetInstance()->PlayMusic();
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmStateShell::Deinit()
{
	if ( _pLayer )
	{
		_pLayer->Deinit();
		_pLayer->removeFromParentAndCleanup( true );
		_pLayer->release();
		_pLayer = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmStateShell::Update( float dt )
{
	if ( !_pLayer )
	{
		_pLayer = SoFsmShellLayer::create();
		_pLayer->retain();
		CCDirector::sharedDirector()->getRunningScene()->addChild(_pLayer);
	}

	if ( _pLayer->IsReadyToExitState() )
	{
		SoFsmStateManager* pStateMgr = GetParentFsmStateManager();
		if ( _pLayer->IsPlayOfflineSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "SetupOfflineGame" ) );
		else if ( _pLayer->IsPlayOnlineSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "SetupOnlineGame" ) );
		else if ( _pLayer->IsCreditsSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "Credits" ) );
		else if ( _pLayer->IsFacebookSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "Shell" ) );
		else if ( _pLayer->IsTutorialSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "Tutorial" ) );
		else if ( _pLayer->IsBackSelected() )
			pStateMgr->SetStateOnNextUpdate( JpHash( "Shell" ) );
	}

	return 0;
}









/******************************************************************************
 FUNCTION:  SoFsmShellLayer
******************************************************************************/
SoFsmShellLayer::SoFsmShellLayer() : 
_bReadyToExitState( false ),
_bFacebookSelected( false ),
_bPlayOfflineSelected( false ),
_bPlayOnlineSelected( false ),
_bCreditsSelected( false ),
_bTutorialSelected( false ),
_bOptionsSelected( false ),
_bRulesSelected( false ),
_bMoreSelected( false ),
_bBackSelected( false ),
_pScreenFader( NULL ),
_iCurrentState( 0 ),
_gGuiShellMain( 0 ),
_gGuiShellMainMore( 0 ),
_gGuiOptions( 0 ),
_gGuiRules( 0 ),
_gGuiWinStoreBadges( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmShellLayer
******************************************************************************/
SoFsmShellLayer::~SoFsmShellLayer()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  init
******************************************************************************/
bool SoFsmShellLayer::init()
{
	if ( CCLayerColor::initWithColor( ccc4(0,0,0,255) ) )
	{
		scheduleUpdate();

		setPosition( 0.0f, 0.0f );

		_bReadyToExitState = false;
		_bFacebookSelected = false,
		_bPlayOfflineSelected = false;
		_bPlayOnlineSelected = false;
		_bCreditsSelected = false;
		_bTutorialSelected = false;
		_bMoreSelected = false;
		_bBackSelected = false;

		CCSize size = CCDirector::sharedDirector()->getWinSize();
		float fPosX = 0.0f;
		float fPosY = 0.0f;


		// Background image
		SoSprite* pSprite = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "LogoBackground.png" ) );
		pSprite->setPosition( ccp(size.width/2, size.height/2) );

		CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
		if ( SizeInPixels.width > SizeInPixels.height )
		{
			float fExactScreenWidth = SizeInPixels.width / pSprite->getContentSize().width;
			pSprite->setScaleX( fExactScreenWidth );
			pSprite->setScaleY( fExactScreenWidth );
		}
		else
		{
			float fExactScreenHeight = SizeInPixels.height / pSprite->getContentSize().height;
			pSprite->setScaleX( fExactScreenHeight );
			pSprite->setScaleY( fExactScreenHeight );
		}


		addChild(pSprite, 0);


		// Background image
		SoSprite* pLogoSprite = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "Logo.png" ) );
		pLogoSprite->setPosition( ccp(size.width/2, size.height/2 + ( 200.0f * SoUtils::GetiPadRatioY() ) ) );
		pLogoSprite->setScaleX( SoUtils::GetiPadRatioY() );
		pLogoSprite->setScaleY( SoUtils::GetiPadRatioY() );
		addChild( pLogoSprite, 0 );



		_pScreenFader = SoScreenFader::create();
		_pScreenFader->retain();
		_pScreenFader->Setup( this, BASE_RENDER_ORDER_FADER );
		_pScreenFader->SetFadeSpeed( 2.0f );
		_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_OPAQUE_TO_TRANSPARENT );
		_pScreenFader->BeginFade();


		_GuiManager.Init( this, SoGuiManager::INPUT_SOURCE_INTERNAL );
		_GuiManager.SetHeadGuiAbsolutionPositionToScreenCenter();

		SoGuiAnimWinShellMain* pNewWindow = new SoGuiAnimWinShellMain;
		_gGuiShellMain = _GuiManager.AddGui( pNewWindow, NULL );
		pNewWindow->AddListener( &_GuiListener );

		SoGuiAnimWinShellMainMore* pNewWindow2 = new SoGuiAnimWinShellMainMore;
		_gGuiShellMainMore = _GuiManager.AddGui( pNewWindow2, NULL );
		pNewWindow2->AddListener( &_GuiListener );

		SoGuiAnimWinOptions* pNewWindow3 = new SoGuiAnimWinOptions;
		_gGuiOptions = _GuiManager.AddGui( pNewWindow3, NULL );
		pNewWindow3->AddListener( &_GuiListener );

		SoGuiAnimWinRules* pNewWindow4 = new SoGuiAnimWinRules;
		_gGuiRules = _GuiManager.AddGui( pNewWindow4, NULL );
		pNewWindow4->AddListener( &_GuiListener );


		#if DCG_IS_TARGET_WIN32
			SoGuiAnimWinStoreBadges* pNewWindow5 = new SoGuiAnimWinStoreBadges;
			_gGuiWinStoreBadges = _GuiManager.AddGui( pNewWindow5, NULL );
			pNewWindow5->AddListener( &_GuiListener );
			_GuiManager.SendMessageToGui( _gGuiWinStoreBadges, &SoGuiMessage::_BasicTransitionIn );
		#endif

		_GuiListener.SetOwner( this );

		_WatermarkPreRelease.Init( this );
		_WatermarkBuildNumber.Init( this );

		return true;
	}


	return false;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmShellLayer::Deinit()
{
	// This removes and deletes all Guis.  Deleted Guis remove their listeners.
	_GuiManager.Deinit();

	if ( _pScreenFader )
	{
		_pScreenFader->Deinit();
		_pScreenFader->release();
		_pScreenFader = NULL;
	}

	_WatermarkBuildNumber.Deinit();
	_WatermarkPreRelease.Deinit();

	removeAllChildrenWithCleanup( DCG_CLEANUP_ALL_RUNNING_ACTIONS );
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoFsmShellLayer::update( float dt )
{
	_GuiManager.Update( dt );

	_pScreenFader->Update( dt );


	const int STATE_INITIAL_FADE_IN = 0;
	const int STATE_MAIN_UPDATE = 10;
	const int STATE_MAIN_UPDATE_MORE = 15;
	const int STATE_MAIN_UPDATE_OPTIONS = 20;
	const int STATE_MAIN_UPDATE_RULES = 30;
	const int STATE_FADE_OUT = 40;

	switch ( _iCurrentState )
	{
		case STATE_INITIAL_FADE_IN:
		{
			if ( _pScreenFader->IsDone() )
			{
				_GuiManager.SendMessageToGui( _gGuiShellMain, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_MAIN_UPDATE;
			}
			break;
		}

		case STATE_MAIN_UPDATE:
		{
			if ( _bMoreSelected )
			{
				_bMoreSelected = false;

				_GuiManager.SendMessageToGui( _gGuiShellMain, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gGuiShellMainMore, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_MAIN_UPDATE_MORE;
			}
			else if ( _bOptionsSelected )
			{
				_bOptionsSelected = false;

				_GuiManager.SendMessageToGui( _gGuiShellMain, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gGuiOptions, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_MAIN_UPDATE_OPTIONS;
			}
			else if 
			( 
				_bPlayOfflineSelected ||
				_bPlayOnlineSelected ||
				_bTutorialSelected ||
				_bFacebookSelected
			)
			{
				_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_TRANSPARENT_TO_OPAQUE );
				_pScreenFader->BeginFade();
				_iCurrentState = STATE_FADE_OUT;
			}

			break;
		}

		case STATE_MAIN_UPDATE_MORE:
		{
			if ( _bRulesSelected )
			{
				_bRulesSelected = false;

				_GuiManager.SendMessageToGui( _gGuiShellMainMore, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gGuiRules, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_MAIN_UPDATE_RULES;
			}
			else if ( _bCreditsSelected )
			{
				_pScreenFader->SetFadeDirection( SoScreenFader::FADE_DIRECTION_TRANSPARENT_TO_OPAQUE );
				_pScreenFader->BeginFade();
				_iCurrentState = STATE_FADE_OUT;
			}
			else if ( _bBackSelected )
			{
				_bBackSelected = false;

				_GuiManager.SendMessageToGui( _gGuiShellMainMore, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gGuiShellMain, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_MAIN_UPDATE;
			}

			break;
		}

		case STATE_MAIN_UPDATE_OPTIONS:
		{
			if ( _bBackSelected )
			{
				_bBackSelected = false;

				_GuiManager.SendMessageToGui( _gGuiOptions, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gGuiShellMain, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_MAIN_UPDATE;
			}
			break;
		}

		case STATE_MAIN_UPDATE_RULES:
		{
			if ( _bBackSelected )
			{
				_bBackSelected = false;

				_GuiManager.SendMessageToGui( _gGuiRules, &SoGuiMessage::_BasicTransitionOut );
				_GuiManager.SendMessageToGui( _gGuiShellMainMore, &SoGuiMessage::_BasicTransitionIn );

				_iCurrentState = STATE_MAIN_UPDATE_MORE;
			}
			break;
		}

		case STATE_FADE_OUT:
		{
			if ( _pScreenFader->IsDone() )
				_bReadyToExitState = true;

			break;
		}

		default:
			ASSERTF( 0, ( "\nUnknown state." ) );
	}

}



/******************************************************************************
 FUNCTION:  draw
******************************************************************************/
void SoFsmShellLayer::draw()
{
	CCLayerColor::draw();
}



/******************************************************************************
 FUNCTION:  OnMessageFromGui
******************************************************************************/
void SoFsmShellLayer::OnMessageFromGui( SoGuiMessage* pMessage )
{
	if ( pMessage )
	{
		if ( pMessage->GetType() == SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER )
		{
			hash hData = pMessage->FirstHash();
			if ( hData == JpHash( "CreditsButtonPressed" ) )
			{
				_bCreditsSelected = true;
				_pScreenFader->BeginFade();

				_GuiManager.SendMessageToGui( _gGuiShellMainMore, &SoGuiMessage::_BasicTransitionOut );
			}

			if ( hData == JpHash( "MinionGamesButtonPressed" ) )
			{
				_bFacebookSelected = true;
				SoUtils::LaunchBrowserAndOpenUrl( "https://www.facebook.com/miniongames" );
			}

			if ( hData == JpHash( "DcgButtonPressed" ) )
			{
				_bFacebookSelected = true;
				SoUtils::LaunchBrowserAndOpenUrl( "http://www.domowiczcg.com/" );
			}

			if ( hData == JpHash( "AppleAppStoreButtonPressed" ) )
			{
				SoUtils::LaunchBrowserAndOpenUrl( "https://itunes.apple.com/us/app/achoir/id862293130?mt=8" );
			}

			if ( hData == JpHash( "GooglePlayButtonPressed" ) )
			{
				SoUtils::LaunchBrowserAndOpenUrl( "https://play.google.com/store/apps/details?id=com.domowiczcg.achoir" );
			}

			if ( hData == JpHash( "PlayOfflineButtonPressed" ) )
			{
				_bPlayOfflineSelected = true;
				_pScreenFader->BeginFade();

				_GuiManager.SendMessageToGui( _gGuiShellMain, &SoGuiMessage::_BasicTransitionOut );
			}

			if ( hData == JpHash( "PlayOnlineButtonPressed" ) )
			{
				_bPlayOnlineSelected = true;
				_pScreenFader->BeginFade();

				_GuiManager.SendMessageToGui( _gGuiShellMain, &SoGuiMessage::_BasicTransitionOut );
			}

			if ( hData == JpHash( "TutorialButtonPressed" ) )
			{
				_bTutorialSelected = true;
				_pScreenFader->BeginFade();

				_GuiManager.SendMessageToGui( _gGuiShellMain, &SoGuiMessage::_BasicTransitionOut );
			}

			if ( hData == JpHash( "OptionsButtonPressed" ) )
			{
				_bOptionsSelected = true;
			}

			if ( hData == JpHash( "RulesButtonPressed" ) )
			{
				_bRulesSelected = true;
			}

			if ( hData == JpHash( "MoreButtonPressed" ) )
			{
				_bMoreSelected = true;
			}

			if ( hData == JpHash( "BackButtonPressed" ) )
			{
				_bBackSelected = true;
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  OnUiButtonDown
******************************************************************************/
void SoFsmShellLayer::OnUiButtonDown( SoUiButton* pButton )
{
}



/******************************************************************************
 FUNCTION:  OnUiButtonUp
******************************************************************************/
void SoFsmShellLayer::OnUiButtonUp( SoUiButton* pButton )
{
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoFsmShellLayer* SoFsmShellLayer::create()
{
	SoFsmShellLayer* pRet = new SoFsmShellLayer();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}





















