/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFsmStateTutorial.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoFsmStateTutorial
******************************************************************************/
SoFsmStateTutorial::SoFsmStateTutorial() :
_pLayer( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmStateTutorial
******************************************************************************/
SoFsmStateTutorial::~SoFsmStateTutorial()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoFsmStateTutorial::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoFsmStateTutorial::Deinit()
{
	if ( _pLayer )
	{
		_pLayer->removeFromParentAndCleanup( true );
		_pLayer->release();
		_pLayer = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
hash SoFsmStateTutorial::Update( float dt )
{
	if ( !_pLayer )
	{
		_pLayer = SoFsmTutorialLayer::create();
		_pLayer->retain();
		CCDirector::sharedDirector()->getRunningScene()->addChild(_pLayer);
	}

	if ( _pLayer->IsDone() )
	{
		SoFsmStateManager* pStateMgr = GetParentFsmStateManager();
		pStateMgr->SetStateOnNextUpdate( JpHash( "Shell" ) );
	}

	return 0;
}








/******************************************************************************
 FUNCTION:  SoFsmTutorialLayer
******************************************************************************/
SoFsmTutorialLayer::SoFsmTutorialLayer() :
_bTouched( false ),
_fTimer( 0.0f ),
_eState( eSubState_Last )
{
}



/******************************************************************************
 FUNCTION:  ~SoFsmTutorialLayer
******************************************************************************/
SoFsmTutorialLayer::~SoFsmTutorialLayer()
{
}



/******************************************************************************
 FUNCTION:  init
******************************************************************************/
bool SoFsmTutorialLayer::init()
{
	if ( CCLayerColor::initWithColor( ccc4(255,255,255,255) ) )
	{
		_bTouched = false;

		CCSize size = CCDirector::sharedDirector()->getWinSize();

		//--------------------
		// Sprite 1
		_pSplashSprite1 = SoSprite::CreateScaledToDevice( SoUtils::AddRelativePathToTexture( "ImgHighScore.png" ) );
		
		// place the image in the center of the screen		
		_pSplashSprite1->setPosition( ccp(size.width/2, size.height/2) );

		// scale the image so that it's stretched to fill the entire screen
		_pSplashSprite1->setScaleX( size.width / _pSplashSprite1->getContentSize().width );
		_pSplashSprite1->setScaleY( size.height / _pSplashSprite1->getContentSize().height );
		_pSplashSprite1->setVisible( true );

		addChild(_pSplashSprite1, 0);

		scheduleUpdate();
		setTouchEnabled( true );

		_fTimer = 0.0f;
		_eState = eSubState_Screen1;

		return true;
	}


	return false;
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoFsmTutorialLayer::update( float dt )
{
	_fTimer += dt;

	switch ( _eState )
	{
		case eSubState_Screen1:
		{
			if ( _bTouched )
			{
				//_pSplashSprite1->setVisible( false );
				_bTouched = false;
				_eState = eSubState_Done;
			}
			break;
		}
		case eSubState_Done:
		{
		}
	}
}



/******************************************************************************
 FUNCTION:  IsDone
******************************************************************************/
bool SoFsmTutorialLayer::IsDone()
{
	return ( _eState == eSubState_Done );
}



/******************************************************************************
 FUNCTION:  ccTouchesEnded
******************************************************************************/
void SoFsmTutorialLayer::ccTouchesEnded(CCSet *pTouches, CCEvent *pEvent)
{
	_bTouched = true;
}



/******************************************************************************
 FUNCTION:  ccTouchesCancelled
******************************************************************************/
void SoFsmTutorialLayer::ccTouchesCancelled(CCSet *pTouches, CCEvent *pEvent)
{
	ccTouchesEnded(pTouches, pEvent);
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoFsmTutorialLayer* SoFsmTutorialLayer::create()
{
	SoFsmTutorialLayer* pRet = new SoFsmTutorialLayer();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}
