/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"

#include "SoActorManager.h"
#include "SoGameBitPlayer.h"
#include "SoGameBitHotel.h"

#include "mmgrChooser.h"



/******************************************************************************
 FUNCTION:  SoActorCard
******************************************************************************/
SoGameBitHotel::SoGameBitHotel() :
_iHotelId( HOTEL_UNDEFINED_ID ),
_iPlacementTileId( TILE_UNDEFINED_ID ),
_iSize( 0 ),
_iAvailableStock( 0 )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGameBitHotel::Init( CCNode* pParentNode )
{
	SoGameBit::Init( pParentNode );

	_iHotelId = HOTEL_UNDEFINED_ID;
	_iPlacementTileId = TILE_UNDEFINED_ID;
	_iAvailableStock = GAMERULES_NUM_STOCKS_PER_HOTEL;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGameBitHotel::Deinit()
{	
	SoGameBit::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGameBitHotel::Update( float dt )
{
	SoGameBit::Update( dt );
}



/******************************************************************************
 FUNCTION:  AdjustAvailableStock
******************************************************************************/
void SoGameBitHotel::AdjustAvailableStock( int i )
{
	ASSERTF
	( 
		( _iAvailableStock + i ) >= 0 && ( _iAvailableStock + i ) <= GAMERULES_NUM_STOCKS_PER_HOTEL, 
		( "\nUnexpected adjustment of hotel stock count." ) 
	);

	_iAvailableStock = Clamp( _iAvailableStock + i, 0, GAMERULES_NUM_STOCKS_PER_HOTEL ); 
}



/******************************************************************************
 FUNCTION:  GetPricePerStockFromTable
******************************************************************************/
int SoGameBitHotel::GetPricePerStockFromTable( int iHotelId, int iHotelSize )
{
	if ( iHotelSize >= 2 )
	{
		ASSERTF( iHotelId >= 0 && iHotelId < GAMERULES_NUM_HOTELS, ( "\nBad hotel Id." ) );
		iHotelId = Clamp( iHotelId, 0, GAMERULES_NUM_HOTELS - 1 );


		int HotelPrices[ GAMERULES_NUM_HOTELS ][ 9 ] =
		{
			{ 2, 3, 4, 5, 6, 7,  8,  9, 10 },
			{ 2, 3, 4, 5, 6, 7,  8,  9, 10 },
			{ 3, 4, 5, 6, 7, 8,  9, 10, 11 },
			{ 3, 4, 5, 6, 7, 8,  9, 10, 11 },
			{ 3, 4, 5, 6, 7, 8,  9, 10, 11 },
			{ 4, 5, 6, 7, 8, 9, 10, 11, 12 },
			{ 4, 5, 6, 7, 8, 9, 10, 11, 12 }
		};

		int iColumn = GetTableColumnFromHotelSize( iHotelSize );

		return HotelPrices[ iHotelId ][ iColumn ];
	}
	
	return 0;
}



/******************************************************************************
 FUNCTION:  GetMajorityBonusFromTable
******************************************************************************/
int SoGameBitHotel::GetMajorityBonusFromTable( int iHotelId, int iHotelSize )
{
	return ( GetPricePerStockFromTable( iHotelId, iHotelSize ) * 10 );
}



/******************************************************************************
 FUNCTION:  GetMinorityBonusFromTable
******************************************************************************/
int SoGameBitHotel::GetMinorityBonusFromTable( int iHotelId, int iHotelSize )
{
	return ( GetPricePerStockFromTable( iHotelId, iHotelSize ) * 5 );
}



/******************************************************************************
 FUNCTION:  GetTableColumnFromHotelSize
******************************************************************************/
int SoGameBitHotel::GetTableColumnFromHotelSize( int iHotelSize )
{
	int iConvertHotelSizeToTableColumn[ 42 ] = 
	{
		-1,                            // can't have a hotel of size 0
		-1,                            // can't have a hotel of size 1
		0,                             // size: 2
		1,                             // size: 3
		2,                             // size: 4
		3,                             // size: 5
		4, 4, 4, 4, 4,                 // size: 6-10
		5, 5, 5, 5, 5, 5, 5, 5, 5, 5,  // size: 11-20
		6, 6, 6, 6, 6, 6, 6, 6, 6, 6,  // size: 21-30
		7, 7, 7, 7, 7, 7, 7, 7, 7, 7,  // size: 31-40
		8                              // size: 41+
	};

	iHotelSize = Clamp( iHotelSize, 2, 41 );

	return iConvertHotelSizeToTableColumn[ iHotelSize ];
}



/******************************************************************************
 FUNCTION:  GetHotelPrestigeRating
 NOTES:     Intended for use by Ai.  Not needed for the rules.
******************************************************************************/
int SoGameBitHotel::GetHotelPrestigeRating( int iHotelId )
{
	switch ( iHotelId )
	{
		case 0:
		case 1:
			return 1;
		case 2:
		case 3:
		case 4:
			return 2;
		case 5:
		case 6:
			return 3;
		default:
			ASSERTF( 0, ( "\nUnhandled hotel id" ) );
	}

	return -1;
}



/******************************************************************************
 FUNCTION:  MakeCopy
******************************************************************************/
void SoGameBitHotel::MakeCopy( SoGameBitHotel* pSource )
{
	_iHotelId         = pSource->_iHotelId;
	_iPlacementTileId = pSource->_iPlacementTileId;
	_iSize            = pSource->_iSize;
	_iAvailableStock  = pSource->_iAvailableStock;
}
