/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"

#include "SoActorManager.h"
#include "SoGameBitPlayer.h"
#include "SoGameGlobals.h"

#include "mmgrChooser.h"


SoGameBitPlayer::PLAYER_COLOR SoGameBitPlayer::_aePlayerColors[ GAMERULES_MAX_PLAYERS ] = 
{
	PLAYER_COLOR_LAST,
	PLAYER_COLOR_LAST,
	PLAYER_COLOR_LAST,
	PLAYER_COLOR_LAST,
	PLAYER_COLOR_LAST,
	PLAYER_COLOR_LAST
};




/******************************************************************************
 FUNCTION:  SoActorPlayer
******************************************************************************/
SoGameBitPlayer::SoGameBitPlayer() :
_eState( STATE_LAST ),
_eRequestedState( STATE_LAST ),
_iPlayerNumber( PLAYER_UNDEFINED_PLAYER_NUMBER ),
_iMoney( 0 ),
_iNumStocksPurchasedThisTurn( 0 ),
_bMustPrintNonStockPurchaseToLog( true ),
_bIsAllowedToPurchaseStock( false ),
_bIsWinner( false ),
_pOwningGame( NULL ),
_bIsFirstMoveOfPlayersTurn( true ),
_eAiPlayerStrength( AI_PLAYER_STRENGTH_LAST )
{
}



/******************************************************************************
 FUNCTION:  MakeCopy
******************************************************************************/
void SoGameBitPlayer::MakeCopy( SoGameBitPlayer* pSource )
{
	_eState =                      pSource->_eState;
	_eRequestedState =             pSource->_eRequestedState;

	_iPlayerNumber =               pSource->_iPlayerNumber;
	_iMoney =                      pSource->_iMoney;

	_bIsFirstMoveOfPlayersTurn   = pSource->_bIsFirstMoveOfPlayersTurn;
	_iNumStocksPurchasedThisTurn = pSource->_iNumStocksPurchasedThisTurn;
	_bIsAllowedToPurchaseStock   = pSource->_bIsAllowedToPurchaseStock;
	_bIsWinner =                   pSource->_bIsWinner;

	_bMustPrintNonStockPurchaseToLog = pSource->_bMustPrintNonStockPurchaseToLog;

	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
		_aiStockCounts[ i ] = pSource->_aiStockCounts[ i ];
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGameBitPlayer::Init( CCNode* pParentNode )
{
	SoGameBit::Init( pParentNode );

	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
		_aiStockCounts[ i ] = 0;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGameBitPlayer::Deinit()
{	
	SoGameBit::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGameBitPlayer::Update( float dt )
{
	SoGameBit::Update( dt );
}



/******************************************************************************
 FUNCTION:  AdjustMoney
******************************************************************************/
void SoGameBitPlayer::AdjustMoney( int i ) 
{
	ASSERTF( ( _iMoney + i ) >= 0, ( "\nIs player spending more money then he owns?" ) );
	_iMoney = ClampMin( _iMoney + i, 0 );
}



/******************************************************************************
 FUNCTION:  GetStockCount
******************************************************************************/
int SoGameBitPlayer::GetStockCount( int iHotelId )
{
	ASSERTF( iHotelId >= 0 && iHotelId < GAMERULES_NUM_HOTELS, ( "\nBad corporation id provided." ) );

	if ( iHotelId >= 0 && iHotelId < GAMERULES_NUM_HOTELS )
		return _aiStockCounts[ iHotelId ];

	return -1;
}



/******************************************************************************
 FUNCTION:  SetStockCount
******************************************************************************/
void SoGameBitPlayer::SetStockCount( int iHotelId, int iAmount )
{
	ASSERTF( iHotelId >= 0 && iHotelId < GAMERULES_NUM_HOTELS, ( "\nBad corporation id provided." ) );
	ASSERTF( iAmount >= 0 && iAmount <= GAMERULES_NUM_STOCKS_PER_HOTEL, ( "\nStock amount it out of range." ) );

	iAmount = Clamp( iAmount, 0, GAMERULES_NUM_STOCKS_PER_HOTEL );

	if ( iHotelId >= 0 && iHotelId < GAMERULES_NUM_HOTELS )
		_aiStockCounts[ iHotelId ] = iAmount;

}



/******************************************************************************
 FUNCTION:  AdjustStockCount
******************************************************************************/
void SoGameBitPlayer::AdjustStockCount( int iHotelId, int iAdjustment )
{
	ASSERTF( iHotelId >= 0 && iHotelId < GAMERULES_NUM_HOTELS, ( "\nBad corporation id provided." ) );

	if ( iHotelId >= 0 && iHotelId < GAMERULES_NUM_HOTELS )
	{
		_aiStockCounts[ iHotelId ] += iAdjustment;

		int iAmount = _aiStockCounts[ iHotelId ];

		ASSERTF( iAmount >= 0 && iAmount <= GAMERULES_NUM_STOCKS_PER_HOTEL, ( "\nStock amount it out of range." ) );
		_aiStockCounts[ iHotelId ] = Clamp( iAmount, 0, GAMERULES_NUM_STOCKS_PER_HOTEL );
	}
}



/******************************************************************************
 FUNCTION:  AdjustNumStocksPurchasedThisTurn
******************************************************************************/
void SoGameBitPlayer::AdjustNumStocksPurchasedThisTurn( int i )
{
	ASSERTF( ( _iNumStocksPurchasedThisTurn + i ) >= 0 && ( _iNumStocksPurchasedThisTurn + i ) <= 3, ( "\nStock purchases are out of range." ) );
	_iNumStocksPurchasedThisTurn = Clamp( _iNumStocksPurchasedThisTurn + i, 0, 3 );	
}



/******************************************************************************
 FUNCTION:  GetPlayerColorByNumber
******************************************************************************/
SoGameBitPlayer::PLAYER_COLOR SoGameBitPlayer::GetPlayerColorByNumber( int iPlayerNumber )
{
	ASSERTF( iPlayerNumber >= 0 && iPlayerNumber < GAMERULES_MAX_PLAYERS, ( "\niPlayerNumber is out of range" ) );

	if ( iPlayerNumber >= 0 && iPlayerNumber < GAMERULES_MAX_PLAYERS )
		return _aePlayerColors[ iPlayerNumber ];

	return PLAYER_COLOR_LAST;
}



/******************************************************************************
 FUNCTION:  SetPlayerColorByNumber
******************************************************************************/
void SoGameBitPlayer::SetPlayerColorByNumber( int iPlayerNumber, SoGameBitPlayer::PLAYER_COLOR ePlayerColor )
{
	ASSERTF( iPlayerNumber >= 0 && iPlayerNumber < GAMERULES_MAX_PLAYERS, ( "\niPlayerNumber is out of range" ) );

	if ( iPlayerNumber >= 0 && iPlayerNumber < GAMERULES_MAX_PLAYERS )
		_aePlayerColors[ iPlayerNumber ] = ePlayerColor;
}



/******************************************************************************
 FUNCTION:  PrintToConsole
******************************************************************************/
void SoGameBitPlayer::PrintToConsole( const std::string& s )
{
	SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();
	pConsole->AddOutputString( s );
	pConsole->SetCurrentTextColor( JpUtils::_White );
}



/******************************************************************************
 FUNCTION:  CPrintF
******************************************************************************/
void SoGameBitPlayer::CPrintF( const char* pFormat, ... )
{
	if ( strlen( pFormat ) < 2048 )
	{
		char buffer[2048];
		va_list args;
		va_start (args, pFormat);
		vsprintf (buffer,pFormat, args);
		va_end (args);

		PrintToConsole( buffer );
	}
}
