/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"

#include "SoCodec.h"
#include "SoGameBitPlayerAi.h"
#include "SoGameGlobals.h"
#include "SoGameState.h"
#include "SoMctsTree.h"

#include "mmgrChooser.h"



/******************************************************************************
 FUNCTION:  SoGameBitPlayerAi
******************************************************************************/
SoGameBitPlayerAi::SoGameBitPlayerAi() :
_pMctsTree( NULL )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGameBitPlayerAi::Init( CCNode* pParentNode )
{
	SoGameBitPlayer::Init( pParentNode );
	_eState = STATE_INIT;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGameBitPlayerAi::Deinit()
{	
	SoGameBitPlayer::Deinit();

	if ( _pMctsTree )
	{
		delete _pMctsTree;
		_pMctsTree = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGameBitPlayerAi::Update( float dt )
{
	SoGameBitPlayer::Update( dt );

	JpRandom* pRng = _pOwningGame->GetMasterRng();

	switch ( _eState )
	{
		case STATE_INIT:
		{
			CPrintF( "Player %i, waiting...", _iPlayerNumber + 1 );
			_eState = STATE_WAIT;
			break;
		}
		case STATE_WAIT:
		{
			if ( _eRequestedState != STATE_LAST )
			{
				_eState = _eRequestedState;
				_eRequestedState = STATE_LAST;
			}
			break;
		}

		case STATE_SELECT_ACTION_INIT:
		{
			SelectAction();
			break;
		}

		case STATE_SELECT_STOCK_PURCHASE_INIT:
		{
			DoSelectStockPurchase();
			break;
		}

		case STATE_SELECT_NEW_HOTEL_INIT:
		{
			DoSelectNewHotel();
			break;
		}

		case STATE_SELECT_MERGE_SURVIVOR_HOTEL_INIT:
		{
			DoSelectMergeSurviorHotel();
			break;
		}

		case STATE_SELECT_MANAGE_OLD_STOCK_INIT:
		{
			DoManageOldStock();
			break;
		}

		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknown Player state." ) );
	}
}



/******************************************************************************
 FUNCTION:  SelectAction
******************************************************************************/
void SoGameBitPlayerAi::SelectAction()
{
	bool bPlayerHasPlacedTile = !( _pOwningGame->GetLastPlacedTileId() == TILE_UNDEFINED_ID );
	bool bPlayerWasForcedToSkipTilePlacement = ( _pOwningGame->IsLastTilePlacementSkipped() );

	if ( !bPlayerHasPlacedTile && !bPlayerWasForcedToSkipTilePlacement )
	{
		TakeTurnUsingMcts( SoGameState::STATE_CONTINUE_PLAYER_TURN );
	}
	else
	{
		if ( !_pOwningGame->IsLastTurnDeclared() && _pOwningGame->CanDeclareLastTurn() )
		{
			TakeTurnUsingMcts( SoGameState::STATE_CONTINUE_PLAYER_TURN );
		}
		else
		{
			SoGameMessage Message;
			SoCodecPlyrMoveEndTurn Encoder;
			Encoder._iPlayerNumber = GetPlayerNumber();
			Encoder.Encode( &Message, 0, 0 );
			_pOwningGame->SendMessageToServer( Message );

			_eState = STATE_WAIT;
		}
	}
}



/******************************************************************************
 FUNCTION:  DoSelectStockPurchase
******************************************************************************/
void SoGameBitPlayerAi::DoSelectStockPurchase()
{
	bool bPlayerCanPurchaseStocks = false;

	//-----------------------
	// First see if it's even possible for the
	// this player to buy any stocks at all.
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoGameBitHotel* pHotel = _pOwningGame->GetHotelByHotelId( i );

		int iCostPerStock = SoGameBitHotel::GetPricePerStockFromTable( i, pHotel->GetSize() );
		int iNumHotelStocksAvailable = pHotel->GetAvailableStock();

		bool bIsPlaced = pHotel->IsPlaced();
		bool bPlayerHasFunds = ( iCostPerStock <= GetMoney() );
		bool bHotelStocksAreAvailable = ( iNumHotelStocksAvailable > 0 );
		bool bCanBuyThisStock = ( bIsPlaced && bPlayerHasFunds && bHotelStocksAreAvailable );

		if ( bCanBuyThisStock )
		{
			bPlayerCanPurchaseStocks = true;
			break;
		}
	}


	//-----------------------
	// If the player can buy stocks, do mcts processing.
	// Otherwise, just send the no-stock-purchase message.
	if ( bPlayerCanPurchaseStocks )
	{
		TakeTurnUsingMcts( SoGameState::STATE_CONTINUE_PLAYER_TURN );
	}
	else
	{
		SoGameMessage Message;
		SoCodecPlyrMoveStockPurchases StockPurchasesEncoder;
		StockPurchasesEncoder._iPlayerNumber = GetPlayerNumber();
		StockPurchasesEncoder._iAmountOfHotelWithId0 = 0;
		StockPurchasesEncoder._iAmountOfHotelWithId1 = 0;
		StockPurchasesEncoder._iAmountOfHotelWithId2 = 0;
		StockPurchasesEncoder._iAmountOfHotelWithId3 = 0;
		StockPurchasesEncoder._iAmountOfHotelWithId4 = 0;
		StockPurchasesEncoder._iAmountOfHotelWithId5 = 0;
		StockPurchasesEncoder._iAmountOfHotelWithId6 = 0;
		StockPurchasesEncoder.Encode( &Message, 0, 0 );
		_pOwningGame->SendMessageToServer( Message );
		_eState = STATE_WAIT;
	}
}



/******************************************************************************
 FUNCTION:  DoSelectNewHotel
******************************************************************************/
void SoGameBitPlayerAi::DoSelectNewHotel()
{
	SoGameBitHotelList HotelChoices;

	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoGameBitHotel* pHotel = _pOwningGame->GetHotelByHotelId( i );

		if ( !pHotel->IsPlaced() )
		{
			HotelChoices.AddLast( pHotel );
		}
	}

	if ( HotelChoices.GetNumEntries() > 1 )
	{
		TakeTurnUsingMcts( SoGameState::STATE_PLAYER_SELECT_NEW_HOTEL );
	}
	else if ( HotelChoices.GetNumEntries() == 1 )
	{
		// If there is only one hotel left to choose from, we simply pick
		// that hotel, without MCTS processing.

		SoGameBitHotelList::Node* pFirstNode = HotelChoices.GetFirst();
		SoGameBitHotel* pHotel = pFirstNode->GetData();

		if ( pHotel )
		{
			SoGameMessage Message;
			SoCodecPlyrMoveNewHotelPick NewHotelPickEncoder;
			NewHotelPickEncoder._iPlayerNumber = GetPlayerNumber();
			NewHotelPickEncoder._iHotelId = pHotel->GetHotelId();
			NewHotelPickEncoder.Encode( &Message, 0, 0 );
			_pOwningGame->SendMessageToServer( Message );
		}

		_eState = STATE_WAIT;
	}
	else
	{
		ASSERTF( 0, ( "\nThere should always be a valid hotel at this point, else how did we get here?" ) );
	}
}



/******************************************************************************
 FUNCTION:  DoSelectMergeSurviorHotel
******************************************************************************/
void SoGameBitPlayerAi::DoSelectMergeSurviorHotel()
{
	JpRandom* pRng = _pOwningGame->GetMasterRng();
	SoGameBitTile* pLastPlacedTile = _pOwningGame->GetLastPlacedTile();

	bool bMessageSent = false;

	//-----------------------
	// Pick the surviving hotel
	SoGameBitHotelList SurvivorHotels;
	_pOwningGame->BuildListOfPotentialMergeSurvivorHotels( &SurvivorHotels, pLastPlacedTile->GetTileX(), pLastPlacedTile->GetTileY() );

	if ( SurvivorHotels.GetNumEntries() == 1 )
	{
		SoGameBitHotelList::Node* pFirstSurvivorNode = SurvivorHotels.GetFirst();
		SoGameBitHotel* pSurvivorHotel = pFirstSurvivorNode->GetData();

		//-----------------------
		// Then pick the victim hotel
		SoGameBitHotelList VictimHotels;
		_pOwningGame->BuildListOfPotentialMergeVictimHotels( &VictimHotels, pLastPlacedTile->GetTileX(), pLastPlacedTile->GetTileY(), pSurvivorHotel );


		if ( VictimHotels.GetNumEntries() == 1 )
		{
			SoGameBitHotelList::Node* pFirstVictimNode = VictimHotels.GetFirst();
			SoGameBitHotel* pVictimHotel = pFirstVictimNode->GetData();

			//-----------------------
			// Send the result.
			SoGameMessage Message;
			SoCodecPlyrMoveMergeHotels MergeHotelsEncoder;
			MergeHotelsEncoder._iPlayerNumber = GetPlayerNumber();
			MergeHotelsEncoder._iSurvivorHotelId = pSurvivorHotel->GetHotelId(); 
			MergeHotelsEncoder._iVictimHotelId = pVictimHotel->GetHotelId();
			MergeHotelsEncoder.Encode( &Message, 0, 0 );
			_pOwningGame->SendMessageToServer( Message );

			_eState = STATE_WAIT;

			bMessageSent = true;
		}
	}

	if ( !bMessageSent )
	{
		TakeTurnUsingMcts( SoGameState::STATE_MERGE_HOTELS );
	}
}



/******************************************************************************
 FUNCTION:  DoManageOldStock
******************************************************************************/
void SoGameBitPlayerAi::DoManageOldStock()
{
	TakeTurnUsingMcts( SoGameState::STATE_MANAGE_OLD_STOCK );
}



/******************************************************************************
 FUNCTION:  TakeTurnUsingMcts
******************************************************************************/
void SoGameBitPlayerAi::TakeTurnUsingMcts( SoGameState::STATE eServersSoGameState )
{
	_pMctsTree = PrepareMctsTree();

	SoGameMessage* pMessage = _pMctsTree->SearchForMove( this, _pOwningGame, eServersSoGameState );

	if ( pMessage )
	{
		_pOwningGame->SendMessageToServer( *pMessage );

		delete _pMctsTree;
		_pMctsTree = NULL;

		_eState = STATE_WAIT;
	}
}



/******************************************************************************
 FUNCTION:  PrepareMctsTree
******************************************************************************/
SoMctsTree* SoGameBitPlayerAi::PrepareMctsTree()
{
	if ( !_pMctsTree )
	{
		_pMctsTree = new SoMctsTree;
		_pMctsTree->SetMaxSimulations( 150 );
		_pMctsTree->SetNumMovesPerSimulation( 150 );

		/*
		// This is how you would increase the Mcts processing for different Ai levels.
		if ( GetAiPlayerStrength() == AI_PLAYER_STRENGTH_MEDIUM2 )
		{
			_pMctsTree->SetMaxSimulations( 250 );
			_pMctsTree->SetNumMovesPerSimulation( 150 );
		}
		else
		{
			_pMctsTree->SetMaxSimulations( 150 );
			_pMctsTree->SetNumMovesPerSimulation( 150 );
		}
		*/
	}

	return _pMctsTree;
}









