/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"

#include "SoActorManager.h"
#include "SoCameraGameplay.h"
#include "SoFsmStateMain.h"
#include "SoGameBitPlayerHuman.h"
#include "SoGameGlobals.h"
#include "SoGameState.h"
#include "SoGuiAnimWinMessageBox2.h"
#include "SoGuiAnimWinPlayerBar.h"
#include "SoGuiAnimWinPlayerDeclareLastTurn.h"
#include "SoGuiAnimWinPlayerManageOldStock.h"
#include "SoGuiAnimWinPlayerMergeHotelChoice.h"
#include "SoGuiAnimWinPlayerNewHotelChoice.h"
#include "SoGuiAnimWinPlayerStockPurchase.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoHud.h"

#include "mmgrChooser.h"



/******************************************************************************
 FUNCTION:  SoActorPlayerHuman
******************************************************************************/
SoGameBitPlayerHuman::SoGameBitPlayerHuman() :
_gWindowNewHotelChoice( 0 ),
_gWindowMergeHotelChoice( 0 ),
_gWindowStockPurchase( 0 ),
_gWindowManageOldStock( 0 ),
_gWindowDeclareLastTurn( 0 ),
_gWindowMessageBox2( 0 ),
_gWindowUniversityHires( 0 ),
_gWindowParkingSpotInput( 0 ),
_gGuiPlayerBar( 0 ),
_pGuiPlayerBar( NULL ),
_fCancelActionTimer( 0.0f ),
_bIsViewTilesToggleOn( false )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGameBitPlayerHuman::Init( CCNode* pParentNode )
{
	SoGameBitPlayer::Init( pParentNode );
	_eState = STATE_INIT;


	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();
	_pGuiPlayerBar = new SoGuiAnimWinPlayerBar;
	_pGuiPlayerBar->SetPlayer( this );
	_gGuiPlayerBar = pGuiManager->AddGui( _pGuiPlayerBar, NULL );
	

	_GuiListener.SetOwner( this );
	_pGuiPlayerBar->AddListener( &_GuiListener );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGameBitPlayerHuman::Deinit()
{	
	SoGameBitPlayer::Deinit();

	RemoveAndDeleteAllWorkerParkingSpots();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGameBitPlayerHuman::Update( float dt )
{
	SoGameBitPlayer::Update( dt );

	SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();
	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();

	switch ( _eState )
	{
		case STATE_INIT:
		{
			CPrintF( "Player %i, waiting...", _iPlayerNumber + 1 );
			_eState = STATE_WAIT;
			break;
		}
		case STATE_WAIT:
		{
			if ( _eRequestedState != STATE_LAST )
			{
				RemoveAndDeleteAllWorkerParkingSpots();

				_eState = _eRequestedState;
				_eRequestedState = STATE_LAST;
			}
			break;
		}

		case STATE_SELECT_ACTION_INIT:
		{
			SetupCameraOnStartOfPlayersTurn();

			RemoveAndDeleteAllWorkerParkingSpots();

			bool bPlayerHasPlacedTile = ( _pOwningGame->GetLastPlacedTileId() != TILE_UNDEFINED_ID );
			if ( !bPlayerHasPlacedTile )
				CreateParkingSpotsForAvailableActions();

			SoGuiManager* pGuiManager = SoGuiManager::GetInstance();
			pGuiManager->SendMessageToGui( _gGuiPlayerBar, &SoGuiMessage::_BasicTransitionIn );

			_eState = STATE_SELECT_ACTION;

			break;
		}

		case STATE_SELECT_ACTION:
		{		
			bool bPlayerHasPlacedTile = 
			( 
				_pOwningGame->GetLastPlacedTileId() != TILE_UNDEFINED_ID ||
				_pOwningGame->IsLastTilePlacementSkipped()
			);

			bool bPlayerCanPlaceTile = ( _pOwningGame->GetNumPlayableTilesForPlayer( _iPlayerNumber ) > 0 );

			SoWorkerParkingSpot* pClickedParkingSpot = GetParkingSpotClickedByUser();

			if ( !bPlayerHasPlacedTile && !bPlayerCanPlaceTile )
			{
				// If we are here, then the player is unable to make a tile placement.
				// So we automatically send a message to the server, to signal 
				// that this player is intentionally skipping tile placement.
				// Note: even with the 2008 special rule that says a player can replace an
				// unplayable hand with new tiles at the start of his turn, it is still
				// possible that there are no new tiles left!  So no matter what, we still
				// must handle this corner case.  mdomowicz 2014_01_15
				SoCodecPlyrMovePlaceTile TilePlacementEncoder;
				TilePlacementEncoder._iPlayerNumber = GetPlayerNumber();
				TilePlacementEncoder._iPlacedTileId = TILE_UNDEFINED_ID;
				TilePlacementEncoder._bSkippingTilePlacement = true;

				_Message.Clear();
				TilePlacementEncoder.Encode( &_Message, 0, 0 );
				_pOwningGame->SendMessageToServer( _Message );
				_eState = STATE_WAIT;

			}
			else if ( pClickedParkingSpot && !bPlayerHasPlacedTile )
			{
				pClickedParkingSpot->SetIsClickedByUser( false );

				if ( pClickedParkingSpot->GetType() == SoWorkerParkingSpot::TYPE_USABLE )
				{
					SoGameBitTile* pTile = pClickedParkingSpot->GetGameBitTile();

					if ( pTile )
					{
						SoCodecPlyrMovePlaceTile TilePlacementEncoder;
						TilePlacementEncoder._iPlayerNumber = GetPlayerNumber();
						TilePlacementEncoder._iPlacedTileId = pTile->GetId();
						TilePlacementEncoder._bSkippingTilePlacement = false;

						_Message.Clear();
						TilePlacementEncoder.Encode( &_Message, 0, 0 );
						_pOwningGame->SendMessageToServer( _Message );
						_eState = STATE_WAIT;
					}
				}
				else
				{
					SoGameBitTile* pTile = pClickedParkingSpot->GetGameBitTile();

					if ( pTile )
					{
						if ( _pOwningGame->DoesTileCreateIllegalHotelWhenPlaced( pTile->GetTileX(), pTile->GetTileY() ) )
						{
							LaunchWindowMessageBox2( "You cannot place this tile because it would create a new corporation and every corporation exists already." );
							_eState = STATE_MESSAGE_BOX_UPDATE;
						}
						else if ( _pOwningGame->DoesTileMergeSafeHotelsWhenPlaced( pTile->GetTileX(), pTile->GetTileY() ) )
						{
							LaunchWindowMessageBox2( "You cannot place this tile because it would merge two safe corporations.  A safe corporation is any corporation that is 11 or more tiles in size." );
							_eState = STATE_MESSAGE_BOX_UPDATE;
						}
					}
				}
			}
			else if ( _sGuiMessageData == "EndTurnButtonPressed" )
			{
				_eState = STATE_END_TURN;
				_sGuiMessageData = "";
			}
			else if ( _sGuiMessageData == "UndoButtonPressed" )
			{
				_eState = STATE_UNDO;
				_sGuiMessageData = "";
			}
			else if ( _sGuiMessageData == "RestartTurnButtonPressed" )
			{
				_eState = STATE_RESTART_TURN;
				_sGuiMessageData = "";
			}
			else if ( _sGuiMessageData == "BuyStockButtonPressed" )
			{
				_eState = STATE_SELECT_STOCK_PURCHASE_INIT;
				_sGuiMessageData = "";
			}
			else if ( _sGuiMessageData == "DeclareLastTurnButtonPressed" )
			{
				_eState = STATE_DECLARE_LAST_TURN_INIT;
				_sGuiMessageData = "";
			}

			break;
		}

		case STATE_SELECT_NEW_HOTEL_INIT:
		{
			LaunchWindowNewHotelChoice();

			_eState = STATE_SELECT_NEW_HOTEL;
			break;
		}

		case STATE_SELECT_NEW_HOTEL:
		{
			if ( _sGuiMessageData == "NewHotelChoiceWindowResult" )
			{
				_sGuiMessageData = "";
				_Message.Clear();
				_NewHotelPickEncoder._iPlayerNumber = GetPlayerNumber();
				_NewHotelPickEncoder.Encode( &_Message, 0, 0 );
				_pOwningGame->SendMessageToServer( _Message );

				pGuiManager->SendMessageToGui( _gWindowNewHotelChoice, &SoGuiMessage::_BasicTransitionOut );
				_gWindowNewHotelChoice = 0;

				_eState = STATE_WAIT;
			}
			else if ( _sGuiMessageData == "NewHotelChoiceWindowCanceled" )
			{
				_sGuiMessageData = "";
				_NewHotelPickEncoder.Reset();
				pGuiManager->SendMessageToGui( _gWindowNewHotelChoice, &SoGuiMessage::_BasicTransitionOut );
				_gWindowNewHotelChoice = 0;

				_eState = STATE_RESTART_TURN;
			}

			break;
		}

		case STATE_SELECT_MERGE_SURVIVOR_HOTEL_INIT:
		{
			LaunchWindowMergeSurvivorHotelChoice();
			_eState = STATE_SELECT_MERGE_SURVIVOR_HOTEL;

			break;
		}

		case STATE_SELECT_MERGE_SURVIVOR_HOTEL:
		{
			if ( _sGuiMessageData == "MergeHotelChoiceWindowResultSurvivor" )
			{
				_sGuiMessageData = "";
				_Message.Clear();

				pGuiManager->SendMessageToGui( _gWindowMergeHotelChoice, &SoGuiMessage::_BasicTransitionOut );
				_gWindowMergeHotelChoice = 0;

				_eState = STATE_SELECT_MERGE_VICTIM_HOTEL_INIT;
			}
			else if ( _sGuiMessageData == "MergeHotelChoiceWindowCanceled" )
			{
				_sGuiMessageData = "";
				_MergeHotelsEncoder.Reset();
				pGuiManager->SendMessageToGui( _gWindowMergeHotelChoice, &SoGuiMessage::_BasicTransitionOut );
				_gWindowMergeHotelChoice = 0;

				_eState = STATE_RESTART_TURN;
			}

			break;
		}

		case STATE_SELECT_MERGE_VICTIM_HOTEL_INIT:
		{
			LaunchWindowMergeVictimHotelChoice();
			_eState = STATE_SELECT_MERGE_VICTIM_HOTEL;

			break;
		}

		case STATE_SELECT_MERGE_VICTIM_HOTEL:
		{
			if ( _sGuiMessageData == "MergeHotelChoiceWindowResultVictim" )
			{
				_sGuiMessageData = "";
				_Message.Clear();
				_MergeHotelsEncoder._iPlayerNumber = GetPlayerNumber();
				_MergeHotelsEncoder.Encode( &_Message, 0, 0 );
				_pOwningGame->SendMessageToServer( _Message );

				pGuiManager->SendMessageToGui( _gWindowMergeHotelChoice, &SoGuiMessage::_BasicTransitionOut );
				_gWindowMergeHotelChoice = 0;

				_eState = STATE_WAIT;
			}
			else if ( _sGuiMessageData == "MergeHotelChoiceWindowCanceled" )
			{
				_sGuiMessageData = "";
				_MergeHotelsEncoder.Reset();
				pGuiManager->SendMessageToGui( _gWindowMergeHotelChoice, &SoGuiMessage::_BasicTransitionOut );
				_gWindowMergeHotelChoice = 0;

				_eState = STATE_SELECT_MERGE_SURVIVOR_HOTEL_INIT;
			}

			break;
		}

		case STATE_SELECT_STOCK_PURCHASE_INIT:
		{
			if ( GetNumStocksPurchasedThisTurn() < 3 )
			{
				LaunchWindowStockPurchase();

				SoGuiManager* pGuiManager = SoGuiManager::GetInstance();
				pGuiManager->SendMessageToGui( _gGuiPlayerBar, &SoGuiMessage::_BasicTransitionIn );

				_eState = STATE_SELECT_STOCK_PURCHASE;
			}
			else
			{
				// This is only ever reached when debugging, specifically when a player 
				// can place multiple tiles in a single turn.  This prevents the stock 
				// purchase window from opening after every tile placement.  
				// mdomowicz 2013_12_11
				_eState = STATE_SELECT_ACTION_INIT;
			}

			break;
		}

		case STATE_SELECT_STOCK_PURCHASE:
		{
			if ( _sGuiMessageData == "StockPurchaseWindowResult" )
			{
				_sGuiMessageData = "";
				_Message.Clear();
				_StockPurchasesEncoder._iPlayerNumber = GetPlayerNumber();
				_StockPurchasesEncoder.Encode( &_Message, 0, 0 );
				_pOwningGame->SendMessageToServer( _Message );

				pGuiManager->SendMessageToGui( _gWindowStockPurchase, &SoGuiMessage::_BasicTransitionOut );
				_gWindowStockPurchase = 0;

				MoveCameraToStandardPosAndZoom();

				_eState = STATE_WAIT;
			}
			else if ( _sGuiMessageData == "StockPurchaseWindowCanceled" )
			{
				_sGuiMessageData = "";
				_StockPurchasesEncoder.Reset();
				pGuiManager->SendMessageToGui( _gWindowStockPurchase, &SoGuiMessage::_BasicTransitionOut );
				_gWindowStockPurchase = 0;

				_fCancelActionTimer = 0.0f;

				MoveCameraToStandardPosAndZoom();

				_eState = STATE_CANCEL_ACTION_TIMEOUT;
			}

			break;
		}


		case STATE_SELECT_MANAGE_OLD_STOCK_INIT:
		{
			LaunchWindowManageOldStock();

			_ManageOldStockEncoder.Reset();

			_eState = STATE_SELECT_MANAGE_OLD_STOCK;

			break;
		}

		case STATE_SELECT_MANAGE_OLD_STOCK:
		{
			if ( _sGuiMessageData == "ManageOldStockWindowResult" )
			{
				_sGuiMessageData = "";
				_Message.Clear();
				_ManageOldStockEncoder._iPlayerNumber = GetPlayerNumber();
				_ManageOldStockEncoder.Encode( &_Message, 0, 0 );
				_pOwningGame->SendMessageToServer( _Message );

				pGuiManager->SendMessageToGui( _gWindowManageOldStock, &SoGuiMessage::_BasicTransitionOut );
				_gWindowManageOldStock = 0;

				// The logic for when to move back to standard pos/zoom is tricky, because
				// it has to only do so under certain conditions, (like when there is just
				// a single human on the client), to avoid visible jerks due to the camera
				// changing pos/zoom settings quickly in succession.
				// mdomowicz 2014_02_08
				if ( _pOwningGame->GetOwningClient()->GetNumHumanPlayersOnThisClient() == 1 )
				{
					SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();
					if ( pNetwork->GetAsNetworkCommsOnline() )
						MoveCameraToStandardPosAndZoom();
				}

				if ( _pOwningGame->GetCurrentPlayerNumber() != GetPlayerNumber() )
				{
					MoveCameraToStandardPosAndZoom();
				}

				_eState = STATE_WAIT;
			}

			break;
		}

		case STATE_DECLARE_LAST_TURN_INIT:
		{
			LaunchWindowDeclareLastTurn();

			_eState = STATE_DECLARE_LAST_TURN;

			break;
		}

		case STATE_DECLARE_LAST_TURN:
		{
			if ( _sGuiMessageData == "DeclareLastTurnWindowResult" )
			{
				_sGuiMessageData = "";
				_Message.Clear();
				_DeclareLastTurnEncoder._iPlayerNumber = GetPlayerNumber();
				_DeclareLastTurnEncoder.Encode( &_Message, 0, 0 );
				_pOwningGame->SendMessageToServer( _Message );

				pGuiManager->SendMessageToGui( _gWindowDeclareLastTurn, &SoGuiMessage::_BasicTransitionOut );
				_gWindowDeclareLastTurn = 0;

				_eState = STATE_WAIT;
			}
			else if ( _sGuiMessageData == "DeclareLastTurnWindowCanceled" )
			{
				_sGuiMessageData = "";
				_DeclareLastTurnEncoder.Reset();
				pGuiManager->SendMessageToGui( _gWindowDeclareLastTurn, &SoGuiMessage::_BasicTransitionOut );
				_gWindowDeclareLastTurn = 0;

				_fCancelActionTimer = 0.0f;
				_eState = STATE_CANCEL_ACTION_TIMEOUT;
			}

			break;
		}
		case STATE_CANCEL_ACTION_TIMEOUT:
		{
			// When an action is canceled, we remove all the parking spots
			// and then wait for a short delay for the dropped visible worker to
			// move back to its previous position, before resuming the
			// players turn.  If we didn't wait, new parking spots would be
			// created too soon, and the visible worker would immediately "find"
			// the parking spot it was already on, which we don't want.
			// mdomowicz 2013_10_02

			RemoveAndDeleteAllWorkerParkingSpots();

			_fCancelActionTimer += dt;
			if ( _fCancelActionTimer > 1.0f )
			{
				_eState = STATE_SELECT_ACTION_INIT;
			}
			break;
		}

		case STATE_MESSAGE_BOX_UPDATE:
		{
			if ( _sGuiMessageData == "MessageBox2WindowOk" )
			{
				_sGuiMessageData = "";
				pGuiManager->SendMessageToGui( _gWindowMessageBox2, &SoGuiMessage::_BasicTransitionOut );
				_gWindowMessageBox2 = 0;
				_eState = STATE_CANCEL_ACTION_TIMEOUT;
			}

			break;
		}

		case STATE_UNDO:
		{
			// Remove parking spots
			RemoveAndDeleteAllWorkerParkingSpots();

			// Hide the Gui Player Bar for this player
			SoGuiManager* pGuiManager = SoGuiManager::GetInstance();
			pGuiManager->SendMessageToGui( _gGuiPlayerBar, &SoGuiMessage::_BasicTransitionOut );

			// Construct and send the "undo" message
			SoGameMessage M;
			SoCodecPlyrMoveUndo Encoder;
			Encoder._iPlayerNumber = GetPlayerNumber();

			Encoder.Encode( &M, 0, 0 );
			_pOwningGame->SendMessageToServer( M );

			_eState = STATE_WAIT;
			break;
		}

		case STATE_RESTART_TURN:
		{
			// Remove parking spots
			RemoveAndDeleteAllWorkerParkingSpots();

			// reset the view
			MoveCameraToStandardPosAndZoom();

			// Hide the Gui Player Bar for this player
			SoGuiManager* pGuiManager = SoGuiManager::GetInstance();
			pGuiManager->SendMessageToGui( _gGuiPlayerBar, &SoGuiMessage::_BasicTransitionOut );

			// Construct and send the "restart turn" message
			SoGameMessage M;
			SoCodecPlyrMoveRestartTurn Encoder;
			Encoder._iPlayerNumber = GetPlayerNumber();

			Encoder.Encode( &M, 0, 0 );
			_pOwningGame->SendMessageToServer( M );

			_eState = STATE_WAIT;
			break;
		}

		case STATE_END_TURN:
		{
			SetupCameraOnEndOfPlayersTurn();

			// Remove parking spots
			RemoveAndDeleteAllWorkerParkingSpots();

			// Hide the Gui Player Bar for this player
			SoGuiManager* pGuiManager = SoGuiManager::GetInstance();
			pGuiManager->SendMessageToGui( _gGuiPlayerBar, &SoGuiMessage::_BasicTransitionOut );

			// Construct and send the "end turn" message
			SoGameMessage M;
			SoCodecPlyrMoveEndTurn Encoder;
			Encoder._iPlayerNumber = GetPlayerNumber();

			Encoder.Encode( &M, 0, 0 );
			_pOwningGame->SendMessageToServer( M );


			// If there are other humans on the same client, we hide 
			// this players tiles.  Otherwise, other
			// human players may see this player's tiles.  This mainly
			// affects pass-n-play games.  mdomowicz 2013_10_15
			if ( _pOwningGame->GetOwningClient()->GetNumHumanPlayersOnThisClient() > 1 )
			{
				_pGuiPlayerBar->UntoggleViewTilesToggle();
				_bIsViewTilesToggleOn = false;
			}

			_eState = STATE_WAIT;
			break;
		}

		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknown Player state." ) );
	}
}



/******************************************************************************
 FUNCTION:  OnMessageFromGui
******************************************************************************/
void SoGameBitPlayerHuman::OnMessageFromGui( SoGuiMessage* pMessage )
{
	if ( pMessage )
	{
		if ( pMessage->GetType() == SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER )
		{
			hash hData = pMessage->FirstHash();

			if ( hData == JpHash( "EndTurnButtonPressed" ) )
			{
				_sGuiMessageData = "EndTurnButtonPressed";
			}

			if ( hData == JpHash( "UndoButtonPressed" ) )
			{
				_sGuiMessageData = "UndoButtonPressed";
			}

			if ( hData == JpHash( "RestartTurnButtonPressed" ) )
			{
				_sGuiMessageData = "RestartTurnButtonPressed";
			}

			if ( hData == JpHash( "BuyStockButtonPressed" ) )
			{
				_sGuiMessageData = "BuyStockButtonPressed";
			}

			if ( hData == JpHash( "DeclareLastTurnButtonPressed" ) )
			{
				_sGuiMessageData = "DeclareLastTurnButtonPressed";
			}

			if ( hData == JpHash( "NewHotelChoiceWindowResult" ) )
			{
				int iHotelId = pMessage->NextInt();
				_sGuiMessageData = "NewHotelChoiceWindowResult";
				_NewHotelPickEncoder._iHotelId = iHotelId;
			}

			if ( hData == JpHash( "NewHotelChoiceWindowCanceled" ) )
			{
				_sGuiMessageData = "NewHotelChoiceWindowCanceled";
			}

			if ( hData == JpHash( "MergeHotelChoiceWindowResultSurvivor" ) )
			{
				_sGuiMessageData = "MergeHotelChoiceWindowResultSurvivor";
				_MergeHotelsEncoder._iSurvivorHotelId = pMessage->NextInt();
			}

			if ( hData == JpHash( "MergeHotelChoiceWindowResultVictim" ) )
			{
				_sGuiMessageData = "MergeHotelChoiceWindowResultVictim";
				_MergeHotelsEncoder._iVictimHotelId = pMessage->NextInt();
			}

			if ( hData == JpHash( "MergeHotelChoiceWindowCanceled" ) )
			{
				_sGuiMessageData = "MergeHotelChoiceWindowCanceled";
			}

			if ( hData == JpHash( "StockPurchaseWindowResult" ) )
			{
				_sGuiMessageData = "StockPurchaseWindowResult";
				_StockPurchasesEncoder._iAmountOfHotelWithId0 = pMessage->NextInt();
				_StockPurchasesEncoder._iAmountOfHotelWithId1 = pMessage->NextInt();
				_StockPurchasesEncoder._iAmountOfHotelWithId2 = pMessage->NextInt();
				_StockPurchasesEncoder._iAmountOfHotelWithId3 = pMessage->NextInt();
				_StockPurchasesEncoder._iAmountOfHotelWithId4 = pMessage->NextInt();
				_StockPurchasesEncoder._iAmountOfHotelWithId5 = pMessage->NextInt();
				_StockPurchasesEncoder._iAmountOfHotelWithId6 = pMessage->NextInt();
			}

			if ( hData == JpHash( "StockPurchaseWindowCanceled" ) )
			{
				_sGuiMessageData = "StockPurchaseWindowCanceled";
			}

			if ( hData == JpHash( "ManageOldStockWindowResult" ) )
			{
				_sGuiMessageData = "ManageOldStockWindowResult";
				_ManageOldStockEncoder._iNumStocksToKeep = pMessage->NextInt();
				_ManageOldStockEncoder._iNumStocksToSell = pMessage->NextInt();
				_ManageOldStockEncoder._iNumStocksToTrade = pMessage->NextInt();
			}

			if ( hData == JpHash( "DeclareLastTurnWindowResult" ) )
			{
				_sGuiMessageData = "DeclareLastTurnWindowResult";
			}

			if ( hData == JpHash( "DeclareLastTurnWindowCanceled" ) )
			{
				_sGuiMessageData = "DeclareLastTurnWindowCanceled";
			}

			if ( hData == JpHash( "ViewTilesToggleOn" ) )
			{
				_bIsViewTilesToggleOn = true;
			}

			if ( hData == JpHash( "ViewTilesToggleOff" ) )
			{
				_bIsViewTilesToggleOn = false;
			}

			if ( hData == JpHash( "MessageBox2WindowOk" ) )
			{
				_sGuiMessageData = "MessageBox2WindowOk";
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  UpdateAllParkingSpots
******************************************************************************/
void SoGameBitPlayerHuman::UpdateAllParkingSpots( float dt )
{
	SoWorkerParkingSpotList::Iterator Iter( &_WorkerParkingSpotList );
	while ( Iter.Next() )
	{
		SoWorkerParkingSpot* pSpot = Iter.GetNodeData();

		bool bThereIsOnlyOneHumanPlayer = ( _pOwningGame->GetOwningClient()->GetNumHumanPlayersOnThisClient() == 1 );
		bool bIsActivated = ( _bIsViewTilesToggleOn || bThereIsOnlyOneHumanPlayer );

		pSpot->SetIsActivated( bIsActivated );
		pSpot->Update( dt );
	}
}



/******************************************************************************
 FUNCTION:  GetParkingSpotClickedByUser
******************************************************************************/
SoWorkerParkingSpot* SoGameBitPlayerHuman::GetParkingSpotClickedByUser()
{
	SoWorkerParkingSpotList::Iterator Iter( &_WorkerParkingSpotList );
	while ( Iter.Next() )
	{
		SoWorkerParkingSpot* pSpot = Iter.GetNodeData();

		if ( pSpot->IsClickedByUser() )
			return pSpot;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  CreateParkingSpotsForAvailableActions
******************************************************************************/
void SoGameBitPlayerHuman::CreateParkingSpotsForAvailableActions()
{
	SoTileSet* pTileSet = _pOwningGame->GetTileSet();
	SoGameBitTileArray* pAllTiles = pTileSet->GetAllTiles();

	SoGameBitTileArray::Iterator TileIter( pAllTiles );
	while ( TileIter.Next() )
	{
		SoGameBitTile* pTile = TileIter.GetData();
		if ( pTile->GetOwnerPlayerNumber() == GetPlayerNumber() && !pTile->IsPlaced() )
		{
			bool bIsPlacable = 
			(
				!_pOwningGame->DoesTileCreateIllegalHotelWhenPlaced( pTile->GetTileX(), pTile->GetTileY() ) &&
				!_pOwningGame->DoesTileMergeSafeHotelsWhenPlaced( pTile->GetTileX(), pTile->GetTileY() )
			);

			SoWorkerParkingSpot* pNewSpot = new SoWorkerParkingSpot;

			if ( bIsPlacable )
				pNewSpot->Init( SoFsmStateMain::GetSharedMainLayer(), SoWorkerParkingSpot::TYPE_USABLE );
			else
				pNewSpot->Init( SoFsmStateMain::GetSharedMainLayer(), SoWorkerParkingSpot::TYPE_UNUSABLE );

			pNewSpot->SetGameBitTile( pTile );

			_WorkerParkingSpotList.AddLast( pNewSpot );
		}
	}
}



/******************************************************************************
 FUNCTION:  RemoveAndDeleteAllWorkerParkingSpots
******************************************************************************/
void SoGameBitPlayerHuman::RemoveAndDeleteAllWorkerParkingSpots()
{
	SoWorkerParkingSpotList::Iterator Iter( &_WorkerParkingSpotList );
	while ( Iter.Next() )
	{
		SoWorkerParkingSpot* pSpot = Iter.GetNodeData();
		delete pSpot;
	}

	_WorkerParkingSpotList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  LaunchWindowNewHotelChoice
******************************************************************************/
void SoGameBitPlayerHuman::LaunchWindowNewHotelChoice()
{
	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();

	if ( _gWindowNewHotelChoice == 0 )
	{
		SoGuiAnimWinPlayerNewHotelChoice* pNewWindow = new SoGuiAnimWinPlayerNewHotelChoice;
		pNewWindow->SetDeleteOnTransitionOut( true );
		pNewWindow->SetPlayerNumber( GetPlayerNumber() );
		pNewWindow->SetGame( _pOwningGame );
		_gWindowNewHotelChoice = pGuiManager->AddGui( pNewWindow, NULL );

		pNewWindow->AddListener( &_GuiListener );
	}

	pGuiManager->SendMessageToGui( _gWindowNewHotelChoice, &SoGuiMessage::_BasicTransitionIn );


	MoveCameraToOpenWindowPosAndZoom();
}



/******************************************************************************
 FUNCTION:  LaunchWindowMergeSurvivorHotelChoice
******************************************************************************/
void SoGameBitPlayerHuman::LaunchWindowMergeSurvivorHotelChoice()
{
	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();

	if ( _gWindowMergeHotelChoice == 0 )
	{
		SoGuiAnimWinPlayerMergeHotelChoice* pNewWindow = new SoGuiAnimWinPlayerMergeHotelChoice;
		pNewWindow->SetDeleteOnTransitionOut( true );
		pNewWindow->SetPlayerNumber( GetPlayerNumber() );
		pNewWindow->SetGame( _pOwningGame );
		pNewWindow->SetUsageType( SoGuiAnimWinPlayerMergeHotelChoice::USAGE_TYPE_PICK_SURVIVOR_HOTEL );
		_gWindowMergeHotelChoice = pGuiManager->AddGui( pNewWindow, NULL );

		pNewWindow->AddListener( &_GuiListener );
	}

	pGuiManager->SendMessageToGui( _gWindowMergeHotelChoice, &SoGuiMessage::_BasicTransitionIn );

	MoveCameraToOpenWindowPosAndZoom();
}



/******************************************************************************
 FUNCTION:  LaunchWindowMergeVictimHotelChoice
******************************************************************************/
void SoGameBitPlayerHuman::LaunchWindowMergeVictimHotelChoice()
{
	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();

	if ( _gWindowMergeHotelChoice == 0 )
	{
		SoGuiAnimWinPlayerMergeHotelChoice* pNewWindow = new SoGuiAnimWinPlayerMergeHotelChoice;
		pNewWindow->SetDeleteOnTransitionOut( true );
		pNewWindow->SetPlayerNumber( GetPlayerNumber() );
		pNewWindow->SetGame( _pOwningGame );
		pNewWindow->SetSurvivingHotelId( _MergeHotelsEncoder._iSurvivorHotelId );
		pNewWindow->SetUsageType( SoGuiAnimWinPlayerMergeHotelChoice::USAGE_TYPE_PICK_VICTIM_HOTEL );
		_gWindowMergeHotelChoice = pGuiManager->AddGui( pNewWindow, NULL );

		pNewWindow->AddListener( &_GuiListener );
	}

	pGuiManager->SendMessageToGui( _gWindowMergeHotelChoice, &SoGuiMessage::_BasicTransitionIn );

	MoveCameraToOpenWindowPosAndZoom();
}



/******************************************************************************
 FUNCTION:  LaunchWindowStockPurchase
******************************************************************************/
void SoGameBitPlayerHuman::LaunchWindowStockPurchase()
{
	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();

	if ( _gWindowStockPurchase == 0 )
	{
		SoGuiAnimWinPlayerStockPurchase* pNewWindow = new SoGuiAnimWinPlayerStockPurchase;
		pNewWindow->SetDeleteOnTransitionOut( true );
		pNewWindow->SetPlayerNumber( GetPlayerNumber() );
		pNewWindow->SetGame( _pOwningGame );
		_gWindowStockPurchase = pGuiManager->AddGui( pNewWindow, NULL );

		pNewWindow->AddListener( &_GuiListener );
	}

	pGuiManager->SendMessageToGui( _gWindowStockPurchase, &SoGuiMessage::_BasicTransitionIn );

	MoveCameraToOpenWindowPosAndZoom();
}



/******************************************************************************
 FUNCTION:  LaunchWindowManageOldStock
******************************************************************************/
void SoGameBitPlayerHuman::LaunchWindowManageOldStock()
{
	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();

	if ( _gWindowManageOldStock == 0 )
	{
		SoGuiAnimWinPlayerManageOldStock* pNewWindow = new SoGuiAnimWinPlayerManageOldStock;
		pNewWindow->SetDeleteOnTransitionOut( true );
		pNewWindow->SetPlayerNumber( GetPlayerNumber() );
		pNewWindow->SetGame( _pOwningGame );
		_gWindowManageOldStock = pGuiManager->AddGui( pNewWindow, NULL );

		pNewWindow->AddListener( &_GuiListener );
	}

	pGuiManager->SendMessageToGui( _gWindowManageOldStock, &SoGuiMessage::_BasicTransitionIn );

	MoveCameraToOpenWindowPosAndZoom();
}



/******************************************************************************
 FUNCTION:  LaunchWindowDeclareLastTurn
******************************************************************************/
void SoGameBitPlayerHuman::LaunchWindowDeclareLastTurn()
{
	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();

	if ( _gWindowDeclareLastTurn == 0 )
	{
		SoGuiAnimWinPlayerDeclareLastTurn* pNewWindow = new SoGuiAnimWinPlayerDeclareLastTurn;
		pNewWindow->SetDeleteOnTransitionOut( true );
		pNewWindow->SetPlayerNumber( GetPlayerNumber() );
		pNewWindow->SetGame( _pOwningGame );
		_gWindowDeclareLastTurn = pGuiManager->AddGui( pNewWindow, NULL );

		pNewWindow->AddListener( &_GuiListener );
	}

	pGuiManager->SendMessageToGui( _gWindowDeclareLastTurn, &SoGuiMessage::_BasicTransitionIn );
}



/******************************************************************************
 FUNCTION:  LaunchWindowMessageBox2
******************************************************************************/
void SoGameBitPlayerHuman::LaunchWindowMessageBox2( const std::string& sWindowText )
{
	SoGuiManager* pGuiManager = SoGuiManager::GetInstance();

	if ( _gWindowMessageBox2 == 0 )
	{
		SoGuiAnimWinMessageBox2* pNewWindow = new SoGuiAnimWinMessageBox2;
		pNewWindow->SetDeleteOnTransitionOut( true );
		pNewWindow->SetPlayerNumber( GetPlayerNumber() );
		pNewWindow->SetGame( _pOwningGame );
		_gWindowMessageBox2 = pGuiManager->AddGui( pNewWindow, NULL );

		pNewWindow->AddListener( &_GuiListener );
		pNewWindow->SetText( sWindowText );
	}

	pGuiManager->SendMessageToGui( _gWindowMessageBox2, &SoGuiMessage::_BasicTransitionIn );
}



/******************************************************************************
 FUNCTION:  MoveCameraToStandardPosAndZoom
******************************************************************************/
void SoGameBitPlayerHuman::MoveCameraToStandardPosAndZoom()
{
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
	SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();

	if ( pCameraGameplay )
	{
		pCameraGameplay->MoveToStandardPositionAndZoom();
	}

	SoHud::GetInstance()->GetHotelStats()->MoveToStandardPostion();
}



/******************************************************************************
 FUNCTION:  MoveCameraToOpenWindowPosAndZoom
******************************************************************************/
void SoGameBitPlayerHuman::MoveCameraToOpenWindowPosAndZoom()
{
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
	SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();

	if ( pCameraGameplay )
	{
		pCameraGameplay->MoveToOpenWindowPositionAndZoom();
	}

	SoHud::GetInstance()->GetHotelStats()->MoveToWindowOpenPosition();
}



/******************************************************************************
 FUNCTION:  SetupCameraOnStartOfPlayersTurn
******************************************************************************/
void SoGameBitPlayerHuman::SetupCameraOnStartOfPlayersTurn()
{
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
	SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();

	if ( pCameraGameplay )
	{
		if ( _pGuiPlayerBar->IsViewSettingMin() )
		{
			pCameraGameplay->SetStandardPosition( CAMERA_GAMEPLAY_STANDARD_POS_1 );
			pCameraGameplay->SetStandardZoom( CAMERA_GAMEPLAY_STANDARD_ZOOM_1 );

			SoHud::GetInstance()->GetHotelStats()->SetStandardPosition
			(
				HOTEL_STATS_STANDARD_POS_1_X * SoUtils::GetTrueiPadRatioX(),
				HOTEL_STATS_STANDARD_POS_1_Y * SoUtils::GetiPadRatioY()
			);
		}
		else
		{
			pCameraGameplay->SetStandardPosition( CAMERA_GAMEPLAY_STANDARD_POS_2 );
			pCameraGameplay->SetStandardZoom( CAMERA_GAMEPLAY_STANDARD_ZOOM_2 );

			SoHud::GetInstance()->GetHotelStats()->SetStandardPosition
			(
				HOTEL_STATS_STANDARD_POS_2_X * SoUtils::GetTrueiPadRatioX(),
				HOTEL_STATS_STANDARD_POS_2_Y * SoUtils::GetiPadRatioY()
			);
		}
	}

	MoveCameraToStandardPosAndZoom();
}



/******************************************************************************
 FUNCTION:  SetupCameraOnEndOfPlayersTurn
 NOTES:     When the current player ends his turn, if the next player is Ai 
            (ie, not human), then we want to move the camera back to the 
            "minimized" standard position, which shows the board and the 
            stats simultaneously.  This is so that even if the player is in 
            the "max" view which shows only the board, when he ends his turn
            he can still see the stats changes caused by the other Ai players.
            Without this code, the view would stay "maximized", and the
            player would miss a lot of info.

            The reason we do this only for Ai players is because if the
            next player is (also) human, then that player will set the
            camera to that player's current view setting, at the start of
            his turn.  Making view setting changes here, only to have them
            overridden by the next player could lead to ugly camera jumps.
******************************************************************************/
void SoGameBitPlayerHuman::SetupCameraOnEndOfPlayersTurn()
{
	int iNextPlayerNumber = GetPlayerNumber() + 1;
	if ( iNextPlayerNumber == _pOwningGame->GetNumPlayers() )
		iNextPlayerNumber = 0;

	SoGameBitPlayer* pNextPlayer = _pOwningGame->GetPlayerByPlayerNumber( iNextPlayerNumber ); 

	if ( !pNextPlayer->GetAsPlayerHuman() )
	{
		SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
		SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
		SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();

		if ( pCameraGameplay )
		{
			pCameraGameplay->SetStandardPosition( CAMERA_GAMEPLAY_STANDARD_POS_1 );
			pCameraGameplay->SetStandardZoom( CAMERA_GAMEPLAY_STANDARD_ZOOM_1 );

			SoHud::GetInstance()->GetHotelStats()->SetStandardPosition
			(
				HOTEL_STATS_STANDARD_POS_1_X * SoUtils::GetTrueiPadRatioX(),
				HOTEL_STATS_STANDARD_POS_1_Y * SoUtils::GetiPadRatioY()
			);

			MoveCameraToStandardPosAndZoom();
		}
	}
}



/******************************************************************************
 FUNCTION:  PrintToConsole
******************************************************************************/
void SoGameBitPlayerHuman::PrintToConsole( const std::string& s )
{
	SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();
	pConsole->AddOutputString( s );
	pConsole->SetCurrentTextColor( JpUtils::_White );
}



/******************************************************************************
 FUNCTION:  CPrintF
******************************************************************************/
void SoGameBitPlayerHuman::CPrintF( const char* pFormat, ... )
{
	if ( strlen( pFormat ) < 2048 )
	{
		char buffer[2048];
		va_list args;
		va_start (args, pFormat);
		vsprintf (buffer,pFormat, args);
		va_end (args);

		PrintToConsole( buffer );
	}
}
