/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/

#include "stdafx.h"

#include "NeRandomPoolPicker.h"

#include "SoActorManager.h"
#include "SoFsmStateMain.h"
#include "SoGameBitPlayerMcts.h"
#include "SoGameGlobals.h"
#include "SoGameState.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"

#include "mmgrChooser.h"



/******************************************************************************
 FUNCTION:  SoActorPlayerMcts
******************************************************************************/
SoGameBitPlayerMcts::SoGameBitPlayerMcts() :
_eMessageTarget( MESSAGE_TARGET_LAST )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGameBitPlayerMcts::Init( CCNode* pParentNode )
{
	SoGameBitPlayer::Init( pParentNode );
	_eState = STATE_INIT;
	_eMessageTarget = MESSAGE_TARGET_SERVER;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGameBitPlayerMcts::Deinit()
{	
	SoGameBitPlayer::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGameBitPlayerMcts::Update( float dt )
{
	SoGameBitPlayer::Update( dt );

	JpRandom* pRng = _pOwningGame->GetMasterRng();

	switch ( _eState )
	{
		case STATE_INIT:
		{
			CPrintF( "Player %i, waiting...", _iPlayerNumber + 1 );
			_eState = STATE_WAIT;
			break;
		}

		case STATE_WAIT:
		{
			if ( _eRequestedState != STATE_LAST )
			{
				_eState = _eRequestedState;
				_eRequestedState = STATE_LAST;
			}
			break;
		}

		case STATE_SELECT_ACTION_INIT:
		{
			SelectAction();
			break;
		}

		case STATE_SELECT_STOCK_PURCHASE_INIT:
		{
			DoSelectStockPurchase();
			break;
		}

		case STATE_SELECT_NEW_HOTEL_INIT:
		{
			DoSelectNewHotel();
			break;
		}

		case STATE_SELECT_MERGE_SURVIVOR_HOTEL_INIT:
		{
			DoSelectMergeSurviorHotel();
			break;
		}

		case STATE_SELECT_MANAGE_OLD_STOCK_INIT:
		{
			DoManageOldStock();
			break;
		}

		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknown Player state." ) );
	}
}



/******************************************************************************
 FUNCTION:  SelectAction
******************************************************************************/
void SoGameBitPlayerMcts::SelectAction()
{
	if 
	( 
		_pOwningGame->GetLastPlacedTileId() == TILE_UNDEFINED_ID && 
		!_pOwningGame->IsLastTilePlacementSkipped() 
	)
	{
		DoTilePlacement();
	}
	else
	{
		if ( !_pOwningGame->IsLastTurnDeclared() && _pOwningGame->CanDeclareLastTurn() )
		{
			JpRandom* pRng = _pOwningGame->GetMasterRng();
			if ( pRng->GetBool() )
			{
				DoDeclareLastTurn();
			}
			else
			{
				DoEndTurn();
			}
		}
		else
		{
			DoEndTurn();
		}
	}
}



/******************************************************************************
 FUNCTION:  DoTilePlacement
******************************************************************************/
void SoGameBitPlayerMcts::DoTilePlacement()
{
	SoGameBitTileArray PlaceableTilesList( TILESET_MAX_TILES );

	SoTileSet* pTileSet = _pOwningGame->GetTileSet();
	SoGameBitTileArray* pAllTiles = pTileSet->GetAllTiles();


	//-----------------------
	SoGameBitTileArray::Iterator TileIter( pAllTiles );
	while ( TileIter.Next() )
	{
		SoGameBitTile* pTile = TileIter.GetData();
		if ( pTile->GetOwnerPlayerNumber() == GetPlayerNumber() && !pTile->IsPlaced() )
		{
			bool bIsPlacable = 
			(
				!_pOwningGame->DoesTileCreateIllegalHotelWhenPlaced( pTile->GetTileX(), pTile->GetTileY() ) &&
				!_pOwningGame->DoesTileMergeSafeHotelsWhenPlaced( pTile->GetTileX(), pTile->GetTileY() )
			);

			if ( bIsPlacable )
			{
				PlaceableTilesList.AddLast( pTile );
			}
		}
	}

	
	//-----------------------
	JpRandom* pRng = _pOwningGame->GetMasterRng();
	SoGameBitTile* pRandomTile = PlaceableTilesList.GetRandom( pRng );
	if ( pRandomTile )
	{
		_Message.Clear();
		SoCodecPlyrMovePlaceTile TilePlacementEncoder;
		TilePlacementEncoder._iPlayerNumber = GetPlayerNumber();
		TilePlacementEncoder._iPlacedTileId = pRandomTile->GetId();
		TilePlacementEncoder._bSkippingTilePlacement = false;
		TilePlacementEncoder.Encode( &_Message, 0, 0 );

		SendMessageToMessageTarget( _Message );
	}
	else
	{
		//-----------------------
		// If we are here, then the player is unable to make a tile placement.
		// So we automatically send a message to the server, to signal 
		// that this player is intentionally skipping tile placement.
		// Note: even with the 2008 special rule that says a player can replace an
		// unplayable hand with new tiles at the start of his turn, it is still
		// possible that there are no new tiles left!  So no matter what, we still
		// must handle this corner case.  mdomowicz 2014_01_15
		_Message.Clear();
		SoCodecPlyrMovePlaceTile TilePlacementEncoder;
		TilePlacementEncoder._iPlayerNumber = GetPlayerNumber();
		TilePlacementEncoder._iPlacedTileId = TILE_UNDEFINED_ID;
		TilePlacementEncoder._bSkippingTilePlacement = true;
		TilePlacementEncoder.Encode( &_Message, 0, 0 );

		SendMessageToMessageTarget( _Message );
	}



	_eState = STATE_WAIT;
}



/******************************************************************************
 FUNCTION:  DoSelectStockPurchase
******************************************************************************/
void SoGameBitPlayerMcts::DoSelectStockPurchase()
{
	int aiPurchases[ GAMERULES_NUM_HOTELS ];
	memset( aiPurchases, 0, GAMERULES_NUM_HOTELS * sizeof( int ) );

	int iMaxValidHotels = GAMERULES_NUM_HOTELS;
	int aiValidHotels[ GAMERULES_NUM_HOTELS ];

	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
		aiValidHotels[ i ] = i;



	JpRandom* pRng = _pOwningGame->GetMasterRng();

	int iMaxStocksToBuy = pRng->GetInt() % 4;
	int iNumStocksPurchased = 0;
	int iPlayerMoneyAfterPurchase = GetMoney();

	while ( iNumStocksPurchased < iMaxStocksToBuy && iMaxValidHotels > 0 )
	{
		//-----------------------
		// pick a random hotel
		int iChoice = pRng->GetInt() % iMaxValidHotels;
		int iHotelId = aiValidHotels[ iChoice ];
		SoGameBitHotel* pHotel = _pOwningGame->GetHotelByHotelId( iHotelId );

				
		//-----------------------
		// verify that it's purchasable
		int iCostPerStock = SoGameBitHotel::GetPricePerStockFromTable( iHotelId, pHotel->GetSize() );
		int iNumHotelStocksAvailable = pHotel->GetAvailableStock();
		int iNumHotelStocksToBePurchased = aiPurchases[ iHotelId ];

		bool bIsPlaced = pHotel->IsPlaced();
		bool bPlayerHasFunds = ( iCostPerStock <= iPlayerMoneyAfterPurchase );
		bool bHotelStocksAreAvailable = ( iNumHotelStocksToBePurchased < iNumHotelStocksAvailable );
		bool bCanBuyThisStock = ( bIsPlaced && bPlayerHasFunds && bHotelStocksAreAvailable );


		//-----------------------
		// If it is purchasable, buy it.  If not, remove
		// it from the array of possible choices.
		if ( bCanBuyThisStock )
		{
			aiPurchases[ iHotelId ] ++;
			iNumStocksPurchased ++;
			iPlayerMoneyAfterPurchase -= iCostPerStock;
		}
		else
		{
			// Swap the chosen element with the last element.
			int iTemp = aiPurchases[ iChoice ];
			aiValidHotels[ iChoice ] = aiValidHotels[ iMaxValidHotels - 1 ];
			aiValidHotels[ iMaxValidHotels - 1 ] = iTemp;

			// Update the array limit.
			iMaxValidHotels --;
		}
	}

	_Message.Clear();
	SoCodecPlyrMoveStockPurchases StockPurchasesEncoder;
	StockPurchasesEncoder._iPlayerNumber = GetPlayerNumber();
	StockPurchasesEncoder._iAmountOfHotelWithId0 = aiPurchases[ 0 ];
	StockPurchasesEncoder._iAmountOfHotelWithId1 = aiPurchases[ 1 ];
	StockPurchasesEncoder._iAmountOfHotelWithId2 = aiPurchases[ 2 ];
	StockPurchasesEncoder._iAmountOfHotelWithId3 = aiPurchases[ 3 ];
	StockPurchasesEncoder._iAmountOfHotelWithId4 = aiPurchases[ 4 ];
	StockPurchasesEncoder._iAmountOfHotelWithId5 = aiPurchases[ 5 ];
	StockPurchasesEncoder._iAmountOfHotelWithId6 = aiPurchases[ 6 ];
	StockPurchasesEncoder.Encode( &_Message, 0, 0 );
	SendMessageToMessageTarget( _Message );
	_eState = STATE_WAIT;
}



/******************************************************************************
 FUNCTION:  DoSelectNewHotel
******************************************************************************/
void SoGameBitPlayerMcts::DoSelectNewHotel()
{
	SoGameBitHotelList HotelChoices;

	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoGameBitHotel* pHotel = _pOwningGame->GetHotelByHotelId( i );

		if ( !pHotel->IsPlaced() )
		{
			HotelChoices.AddLast( pHotel );
		}
	}

	
	JpRandom* pRng = _pOwningGame->GetMasterRng();
	SoGameBitHotelList::Node* pRandomNode = HotelChoices.GetRandom( pRng );
	SoGameBitHotel* pHotel = pRandomNode->GetData();

	ASSERTF( pHotel, ( "\nThere should always be a valid hotel at this point, else how did we get here?" ) );

	if ( pHotel )
	{
		_Message.Clear();

		SoCodecPlyrMoveNewHotelPick NewHotelPickEncoder;
		NewHotelPickEncoder._iPlayerNumber = GetPlayerNumber();
		NewHotelPickEncoder._iHotelId = pHotel->GetHotelId();
		NewHotelPickEncoder.Encode( &_Message, 0, 0 );

		SendMessageToMessageTarget( _Message );
	}

	_eState = STATE_WAIT;
}



/******************************************************************************
 FUNCTION:  DoSelectMergeSurviorHotel
******************************************************************************/
void SoGameBitPlayerMcts::DoSelectMergeSurviorHotel()
{
	JpRandom* pRng = _pOwningGame->GetMasterRng();
	SoGameBitTile* pLastPlacedTile = _pOwningGame->GetLastPlacedTile();


	//-----------------------
	// Pick the surviving hotel
	SoGameBitHotelList SurvivorHotels;
	_pOwningGame->BuildListOfPotentialMergeSurvivorHotels( &SurvivorHotels, pLastPlacedTile->GetTileX(), pLastPlacedTile->GetTileY() );

	SoGameBitHotelList::Node* pRandomSurvivorNode = SurvivorHotels.GetRandom( pRng );
	SoGameBitHotel* pSurvivorHotel = pRandomSurvivorNode->GetData();


	//-----------------------
	// Then pick the victim hotel
	SoGameBitHotelList VictimHotels;
	_pOwningGame->BuildListOfPotentialMergeVictimHotels( &VictimHotels, pLastPlacedTile->GetTileX(), pLastPlacedTile->GetTileY(), pSurvivorHotel );

	SoGameBitHotelList::Node* pRandomVictimNode = VictimHotels.GetRandom( pRng );
	SoGameBitHotel* pVictimHotel = pRandomVictimNode->GetData();


	//-----------------------
	// Send the result.
	_Message.Clear();

	SoCodecPlyrMoveMergeHotels MergeHotelsEncoder;
	MergeHotelsEncoder._iPlayerNumber = GetPlayerNumber();
	MergeHotelsEncoder._iSurvivorHotelId = pSurvivorHotel->GetHotelId(); 
	MergeHotelsEncoder._iVictimHotelId = pVictimHotel->GetHotelId();
	MergeHotelsEncoder.Encode( &_Message, 0, 0 );
	SendMessageToMessageTarget( _Message );

	_eState = STATE_WAIT;
}



/******************************************************************************
 FUNCTION:  DoManageOldStock
******************************************************************************/
void SoGameBitPlayerMcts::DoManageOldStock()
{
	JpRandom* pRng = _pOwningGame->GetMasterRng();

	int iAmountAvailable = GetStockCount( _pOwningGame->GetLastMergerVictimHotelId() );
	int iAmountToKeep = 0;
	int iAmountToTrade = 0;
	int iAmountToSell = 0;

	SoGameBitHotel* pSurvivingHotel = _pOwningGame->GetLastMergerSurvivorHotel();
	int iAmountSurviorHotelStockAvailable = pSurvivingHotel->GetAvailableStock();


	while ( iAmountAvailable > 0 )
	{
		if ( iAmountAvailable >= 2 && iAmountSurviorHotelStockAvailable > 0 )
		{
			int iChoice = ( pRng->GetInt() % 3 );
			switch ( iChoice )
			{
				case 0:
					iAmountToKeep ++;
					iAmountAvailable --;
					break;
				case 1:
					iAmountToSell ++;
					iAmountAvailable --;
					break;
				case 2:
					iAmountToTrade += 2;
					iAmountAvailable -= 2;
					iAmountSurviorHotelStockAvailable --;
					break;
				default:
					ASSERTF( 0, ( "\nUnhandled case" ) );
			}
		}	
		else
		{
			int iChoice = ( pRng->GetInt() % 2 );
			switch ( iChoice )
			{
				case 0:
					iAmountToKeep ++;
					iAmountAvailable --;
					break;
				case 1:
					iAmountToSell ++;
					iAmountAvailable --;
					break;
				default:
					ASSERTF( 0, ( "\nUnhandled case" ) );
			}
		}
	}


	_Message.Clear();
	SoCodecPlyrManageOldStock ManageOldStockEncoder;
	ManageOldStockEncoder._iPlayerNumber = GetPlayerNumber();
	ManageOldStockEncoder._iNumStocksToKeep = iAmountToKeep;
	ManageOldStockEncoder._iNumStocksToSell = iAmountToSell;
	ManageOldStockEncoder._iNumStocksToTrade = iAmountToTrade;
	ManageOldStockEncoder.Encode( &_Message, 0, 0 );
	SendMessageToMessageTarget( _Message );

	_eState = STATE_WAIT;
}



/******************************************************************************
 FUNCTION:  DoDeclareLastTurn
******************************************************************************/
void SoGameBitPlayerMcts::DoDeclareLastTurn()
{
	_Message.Clear();

	SoCodecPlyrMoveDeclareLastTurn DeclareLastTurnEncoder;
	DeclareLastTurnEncoder._iPlayerNumber = GetPlayerNumber();
	DeclareLastTurnEncoder.Encode( &_Message, 0, 0 );
	SendMessageToMessageTarget( _Message );

	_eState = STATE_WAIT;
}



/******************************************************************************
 FUNCTION:  DoEndTurn
******************************************************************************/
void SoGameBitPlayerMcts::DoEndTurn()
{
	_Message.Clear();
	SoCodecPlyrMoveEndTurn Encoder;
	Encoder._iPlayerNumber = GetPlayerNumber();

	Encoder.Encode( &_Message, 0, 0 );
	SendMessageToMessageTarget( _Message );

	_eState = STATE_WAIT;
}



/******************************************************************************
 FUNCTION:  GetLastSentMessage
******************************************************************************/
SoGameMessage* SoGameBitPlayerMcts::GetLastSentMessage()
{
	return &_LastSentMessage;
}



/******************************************************************************
 FUNCTION:  SendMessageToMessageTarget
******************************************************************************/
void SoGameBitPlayerMcts::SendMessageToMessageTarget( SoGameMessage& Message )
{
	ASSERTF( _eMessageTarget != MESSAGE_TARGET_LAST, ( "\nMessage target not set!" ) );

	if ( _eMessageTarget == MESSAGE_TARGET_SERVER )
		_pOwningGame->SendMessageToServer( Message );

	_LastSentMessage.Clear();
	_LastSentMessage.CopyMessage( Message );
}



/******************************************************************************
 FUNCTION:  IsInMoveSelectionState
******************************************************************************/
bool SoGameBitPlayerMcts::IsInMoveSelectionState()
{
	switch ( GetState() )
	{
		case STATE_SELECT_ACTION_INIT:
		case STATE_SELECT_STOCK_PURCHASE_INIT:
		case STATE_SELECT_NEW_HOTEL_INIT:
		case STATE_SELECT_MERGE_SURVIVOR_HOTEL_INIT:
		case STATE_SELECT_MANAGE_OLD_STOCK_INIT:
			return true;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  AdvanceToMoveSelectionState
******************************************************************************/
void SoGameBitPlayerMcts::AdvanceToMoveSelectionState()
{
	ASSERTF( IsInWaitState(), ( "\nWrong state!  Should not be advancing from this state." ) );

	ASSERTF( _eRequestedState != STATE_LAST, ( "\nDo not know which state to advance to." ) );

	if ( _eRequestedState != STATE_LAST )
	{
		_eState = _eRequestedState;
		_eRequestedState = STATE_LAST;
	}
}



/******************************************************************************
 FUNCTION:  IsInWaitState
******************************************************************************/
bool SoGameBitPlayerMcts::IsInWaitState()
{
	switch ( GetState() )
	{
		case STATE_WAIT:
			return true;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  AdvanceToWaitState
 NOTE:      This function relies on the fact that these states are identical:

                   STATE_WAIT

            Because they are identical, we can blindly set the state to
            STATE_WAIT, without checking if it should be one of the other two.
            mdomowicz 2013_09_03
******************************************************************************/
void SoGameBitPlayerMcts::AdvanceToWaitState()
{
	ASSERTF( IsInMoveSelectionState(), ( "\nNot in correct state!  Should not be advancing to Wait." ) );
	_eState = STATE_WAIT;
	_eRequestedState = STATE_LAST;
}



/******************************************************************************
 FUNCTION:  PrintToConsole
******************************************************************************/
void SoGameBitPlayerMcts::PrintToConsole( const std::string& s )
{
	SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();
	pConsole->AddOutputString( s );
	pConsole->SetCurrentTextColor( JpUtils::_White );
}



/******************************************************************************
 FUNCTION:  CPrintF
******************************************************************************/
void SoGameBitPlayerMcts::CPrintF( const char* pFormat, ... )
{
	if ( strlen( pFormat ) < 2048 )
	{
		char buffer[2048];
		va_list args;
		va_start (args, pFormat);
		vsprintf (buffer,pFormat, args);
		va_end (args);

		PrintToConsole( buffer );
	}
}
