/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "JpHashTable.h"
#include "JpList.h"
#include "JpRandom.h"
#include "SoActor.h"
#include "SoGameMessage.h"


USING_NS_CC;

#define SoGameClientList JpList< SoGameClient >

class SoGameBitPlayer;
class SoGameState;
class SoGuiAnimWinMessageBox;


class SoGameClient
{
	public:

		enum STATE
		{
			STATE_WAIT_FOR_SERVER_ADD,
			STATE_LOBBY,
			STATE_START_NEW_GAME,
			STATE_LAST
		};


		//-----------------------
		// public functions

		SoGameClient();

		virtual ~SoGameClient();

		void Init();

		void Deinit();

		void Update( float dt );

		hash GetServerId() { return _hServerId; }
		void SetServerId( hash h ) { _hServerId = h; }

		hash GetClientId() { return _hClientId; }
		void SetClientId( hash h ) { _hClientId = h; }

		void ConnectToServer( const std::string& sServerName );

		void CreatePlayerHuman();
		void CreatePlayerMctsRandom();
		void CreatePlayerAi();
		void CreatePlayerAiLevel2();

		void SendMessageToServer( SoGameMessage& M );

		SoGameState* GetGameState() { return _pGame; }

		// Note: the number of human players on this client is not the
		// necessarily the same as the number of human players in the game!
		// This client might have 1 of the human players, another client might
		// have another of the human players, etc.
		// mdomowicz 2013_10_15
		int GetNumHumanPlayersOnThisClient() { return _iNumHumanPlayersOnThisClient; }


		// This is mainly for temporarily moving the visible actors around 
		// as a whole, for debug purposes but it may be useful for a feature
		// later too.  mdomowicz 2013_11_25
		static JpVector2f GetGlobalCenterOffset( SoGameState* pGameState );

		// All of this is needed for Save/Load support.
		void WriteSaveGameCsvToDisk();
		void InvalidateSaveData();
		void SetLastMessageNumSentToServer( int i ) { _iLastMessageNumSentToServer = i; }

	private:

		//-----------------------
		// private variables
		hash _hServerId;
		hash _hClientId;

		STATE _eCurrentState;
	
		SoGameState* _pGame;

		int _iNumHumanPlayersOnThisClient;

		SoGuiAnimWinMessageBox* _pGuiMessageBox;
		guid _gGuiMessageBox;
		int _iMessageBoxState;
		float _fMessageBoxTimer;
		float _iMessageBoxPlayerNumber;

		int _iLastMessageNumReceivedFromServer;
		int _iLastMessageNumSentToServer;

		JpRandom _RngForSaveLoadSupport;

		//-----------------------
		// private functions
		void UpdateAnotherPlayerIsActingMessageBox( float dt );

		void CreateVisibleActors();

		void PrintToConsole( const std::string& s );
		void CPrintF( const char* pFormat, ... );

};




