/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "NeCsvIn.h"
#include "NeUtils.h"
#include "SoGameGlobals.h"
#include "SoNetworkCommsOffline.h"
#include "SoNetworkCommsOnline.h"

#include "mmgrChooser.h"


USING_NS_CC;


SoGameServer* SoGameGlobals::_pGameServer = NULL;
SoGameClientList SoGameGlobals::_GameClientList;

SoNetworkComms* SoGameGlobals::_pNetworkComms = NULL;

SoGuiAnimWinLog* SoGameGlobals::_pGuiAnimWindowLog = NULL;

bool SoGameGlobals::_bIsLoadingGame = false;


/******************************************************************************
 FUNCTION:  SoGameGlobals
******************************************************************************/
SoGameGlobals::SoGameGlobals()
{
}



/******************************************************************************
 FUNCTION:  ~SoGameGlobals
******************************************************************************/
SoGameGlobals::~SoGameGlobals()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGameGlobals::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGameGlobals::Deinit()
{
	if ( _pGameServer )
	{
		_pGameServer->Deinit();
		delete _pGameServer;
		_pGameServer = NULL;
	}

	SoGameClientList::Iterator Iter( &_GameClientList );
	while ( Iter.Next() )
	{
		Iter.GetNodeData()->Deinit();
		delete Iter.GetNodeData();
	}

	_GameClientList.RemoveAll();

	if ( _pNetworkComms )
	{
		delete _pNetworkComms;
		_pNetworkComms = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGameGlobals::Update( float dt )
{
	if ( _pNetworkComms )
		_pNetworkComms->Update( dt );

	if ( _pGameServer )
		_pGameServer->Update( dt );

	SoGameClientList::Iterator Iter( &_GameClientList );
	while ( Iter.Next() )
	{
		Iter.GetNodeData()->Update( dt );
	}
}



/******************************************************************************
 FUNCTION:  CreateNetworkCommsOnline
******************************************************************************/
void SoGameGlobals::CreateNetworkCommsOnline()
{
	ASSERTF( !_pNetworkComms, ( "\n_pNetworkCommns is not NULL, but it should be." ) );

	if ( !_pNetworkComms )
		_pNetworkComms = new SoNetworkCommsOnline;
}



/******************************************************************************
 FUNCTION:  CreateNetworkCommsOffline
******************************************************************************/
void SoGameGlobals::CreateNetworkCommsOffline()
{
	ASSERTF( !_pNetworkComms, ( "\n_pNetworkCommns is not NULL, but it should be." ) );

	if ( !_pNetworkComms )
		_pNetworkComms = new SoNetworkCommsOffline;
}



/******************************************************************************
 FUNCTION:  CreateServer
******************************************************************************/
void SoGameGlobals::CreateServer( const std::string& sServerName )
{
	_pGameServer = new SoGameServer;
	_pGameServer->Init( sServerName );
}



/******************************************************************************
 FUNCTION:  CreateServerForLoadGame
******************************************************************************/
void SoGameGlobals::CreateServerForLoadGame( hash hServerId, hash hMasterRngSeed )
{
	_pGameServer = new SoGameServer;
	_pGameServer->InitForLoadGame( hServerId, hMasterRngSeed );
}



/******************************************************************************
 FUNCTION:  CreateClient
******************************************************************************/
SoGameClient* SoGameGlobals::CreateClient()
{
	SoGameClient* pNewClient = new SoGameClient;
	pNewClient->Init();
	_GameClientList.AddLast( pNewClient );
	return pNewClient;
}



/******************************************************************************
 FUNCTION:  DoesSaveGameExist
******************************************************************************/
bool SoGameGlobals::DoesSaveGameExist()
{
	NeCsvIn CsvInServerData;

	char acFullPathAndFilename1[ 512 ] = "";
	SoUtils::CreateFullPathWithRelativePathAndFilename( acFullPathAndFilename1, SoUtils::AddRelativePathToSaveData( "SavedData_Server.csv" ) );

	if ( NeUtils::DoesFileExist( acFullPathAndFilename1 ) ) // this prevents an error popup on android if the file doesn't exist.
	{
		bool bLoadedGame = CsvInServerData.ReadCsvFile( acFullPathAndFilename1 );

		if ( bLoadedGame ) 
		{
			CsvInServerData.MakeHashIndicesFromRowValuesAtCol( 0 );
			hash hServerId = CsvInServerData.GetIntByHashAndInt( JpHash( "_hServerId" ), 1, 0 );
			hash hMasterRngSeed = CsvInServerData.GetIntByHashAndInt( JpHash( "MasterRngSeed" ), 1, 0 );

			return ( hServerId != 0 && hMasterRngSeed != 0 );
		}
	}

	return false;
}



