/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "NeCsvIn.h"
#include "NeCsvOut.h"

#include "SoGameMessage.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGameMessage
******************************************************************************/
SoGameMessage::SoGameMessage() :
_eType( Msg::TYPE_LAST ),
_hClientId( 0 ),
_hServerId( 0 ),
_iMessageNumber( 0 )
{
}



/******************************************************************************
 FUNCTION:  SoGameMessage
******************************************************************************/
SoGameMessage::SoGameMessage( hash hServerId, hash hClientId, Msg::TYPE e ) :
_eType( e ),
_hClientId( hClientId ),
_hServerId( hServerId ),
_iMessageNumber( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoGameMessage
******************************************************************************/
SoGameMessage::~SoGameMessage()
{
	ClearDataList();
}



/******************************************************************************
 FUNCTION:  Clear
******************************************************************************/
void SoGameMessage::Clear()
{
	Set( 0, 0, Msg::TYPE_LAST );
	ClearDataList();
}



/******************************************************************************
 FUNCTION:  Set
******************************************************************************/
void SoGameMessage::Set( hash hServerId, hash hClientId, Msg::TYPE e )
{
	_hServerId = hServerId;
	_hClientId = hClientId;
	_eType = e;
}



/******************************************************************************
 FUNCTION:  AddInt
******************************************************************************/
void SoGameMessage::AddInt( int i )
{
	_DataList.AddLast( new Data( i ) );
}



/******************************************************************************
 FUNCTION:  AddFloat
******************************************************************************/
void SoGameMessage::AddFloat( float f )
{
	_DataList.AddLast( new Data( f ) );
}



/******************************************************************************
 FUNCTION:  AddBool
******************************************************************************/
void SoGameMessage::AddBool( bool b )
{
	int iData = 0;

	if ( b )
		iData = 1;

	AddInt( iData );
}



/******************************************************************************
 FUNCTION:  StartDataRead
******************************************************************************/
void SoGameMessage::StartDataRead()
{
	_pDataListNode = _DataList.GetHead();
}



/******************************************************************************
 FUNCTION:  IsAtDataEnd
******************************************************************************/
bool SoGameMessage::IsAtDataEnd()
{
	return ( _pDataListNode->GetNext() == _DataList.GetHead() );
}



/******************************************************************************
 FUNCTION:  FirstInt
******************************************************************************/
int SoGameMessage::FirstInt()
{
	StartDataRead();
	return NextInt();
}



/******************************************************************************
 FUNCTION:  FirstFloat
******************************************************************************/
float SoGameMessage::FirstFloat()
{
	StartDataRead();
	return NextFloat();
}



/******************************************************************************
 FUNCTION:  FirstBool
******************************************************************************/
bool SoGameMessage::FirstBool()
{
	int iData = FirstInt();
	return ( iData != 0 );
}



/******************************************************************************
 FUNCTION:  NextInt
******************************************************************************/
int SoGameMessage::NextInt()
{
	_pDataListNode = _pDataListNode->GetNext();

	ASSERTF( _pDataListNode != _DataList.GetHead(), ( "\nReading bad data." ) );

	if ( _pDataListNode != _DataList.GetHead() )
	{
		SoGameMessage::Data* pData = _pDataListNode->GetData();
		return pData->GetInt();
	}

	return 0;
}



/******************************************************************************
 FUNCTION:  NextFloat
******************************************************************************/
float SoGameMessage::NextFloat()
{
	_pDataListNode = _pDataListNode->GetNext();

	ASSERTF( _pDataListNode != _DataList.GetHead(), ( "\nReading bad data." ) );

	if ( _pDataListNode != _DataList.GetHead() )
	{
		SoGameMessage::Data* pData = _pDataListNode->GetData();
		return pData->GetFloat();
	}

	return 0.0f;
}



/******************************************************************************
 FUNCTION:  NextBool
******************************************************************************/
bool SoGameMessage::NextBool()
{
	int iData = NextInt();
	return ( iData != 0 );
}



/******************************************************************************
 FUNCTION:  CopyMessage
 NOTES:     To avoid a cascade of required consts, I'm just using a 
            custom copy function, rather than a copy constructor.
            mdomowicz 2013_02_05
******************************************************************************/
void SoGameMessage::CopyMessage( SoGameMessage& ToCopy )
{
	SetType( ToCopy.GetType() );
	SetServerId( ToCopy.GetServerId() );
	SetClientId( ToCopy.GetClientId() );
	SetMessageNumber( ToCopy.GetMessageNumber() );

	SoGameMessageDataList::Iterator Iter( ToCopy.GetDataList() );
	while ( Iter.Next() )
	{
		SoGameMessage::Data* pData = Iter.GetNodeData();
		AddInt( pData->GetInt() );
	}
}



/******************************************************************************
 FUNCTION:  ClearDataList
******************************************************************************/
void SoGameMessage::ClearDataList()
{
	SoGameMessageDataList::Iterator Iter( &_DataList );
	while( Iter.Next() )
	{
		delete Iter.GetNodeData();
	}

	_DataList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  ClearDataList
******************************************************************************/
const char* Msg::GetStringFromType( Msg::TYPE eType )
{
	switch ( eType )
	{
		case Msg::TYPE_CLI_REQUESTS_ADD_TO_SVR:            return "TYPE_CLI_REQUESTS_ADD_TO_SVR"; break;
		case Msg::TYPE_SVR_COMMAND_ADD_CLI:                return "TYPE_SVR_COMMAND_ADD_CLI"; break;               

		case Msg::TYPE_CLI_REQUESTS_SEAT:                  return "TYPE_CLI_REQUESTS_SEAT"; break;                 
		case Msg::TYPE_SVR_COMMAND_SEAT_ASSIGNMNET:        return "TYPE_SVR_COMMAND_SEAT_ASSIGNMNET"; break;       

		case Msg::TYPE_SVR_COMMAND_START_GAME:             return "TYPE_SVR_COMMAND_START_GAME"; break;            
		case Msg::TYPE_SVR_COMMAND_INIT_NEW_GAME:          return "TYPE_SVR_COMMAND_INIT_NEW_GAME"; break;         

		case Msg::TYPE_SVR_COMMAND_PLYR_TAKE_TURN:         return "TYPE_SVR_COMMAND_PLYR_TAKE_TURN"; break;        
		case Msg::TYPE_SVR_COMMAND_PLYR_PURCHASE_STOCK:    return "TYPE_SVR_COMMAND_PLYR_PURCHASE_STOCK"; break;   
		case Msg::TYPE_SVR_COMMAND_PLYR_PICK_NEW_HOTEL:    return "TYPE_SVR_COMMAND_PLYR_PICK_NEW_HOTEL"; break;   
		case Msg::TYPE_SVR_COMMAND_PLYR_PICK_MERGE_HOTELS: return "TYPE_SVR_COMMAND_PLYR_PICK_MERGE_HOTELS"; break;
		case Msg::TYPE_SVR_COMMAND_PLYR_MANAGE_OLD_STOCK:  return "TYPE_SVR_COMMAND_PLYR_MANAGE_OLD_STOCK"; break; 

		case Msg::TYPE_PLYR_MOVE_PLACE_TILE:               return "TYPE_PLYR_MOVE_PLACE_TILE"; break;              
		case Msg::TYPE_PLYR_MOVE_STOCK_PURCHASES:          return "TYPE_PLYR_MOVE_STOCK_PURCHASES"; break;         
		case Msg::TYPE_PLYR_MOVE_NEW_HOTEL_PICK:           return "TYPE_PLYR_MOVE_NEW_HOTEL_PICK"; break;          
		case Msg::TYPE_PLYR_MOVE_MERGE_HOTELS:             return "TYPE_PLYR_MOVE_MERGE_HOTELS"; break;            
		case Msg::TYPE_PLYR_MOVE_MANAGE_OLD_STOCK:         return "TYPE_PLYR_MOVE_MANAGE_OLD_STOCK"; break;        
		case Msg::TYPE_PLYR_MOVE_DECLARE_LAST_TURN:        return "TYPE_PLYR_MOVE_DECLARE_LAST_TURN"; break;       

		case Msg::TYPE_PLYR_MOVE_UNDO:                     return "TYPE_PLYR_MOVE_UNDO"; break;                    
		case Msg::TYPE_PLYR_MOVE_RESTART_TURN:             return "TYPE_PLYR_MOVE_RESTART_TURN"; break;            
		case Msg::TYPE_PLYR_MOVE_END_TURN:                 return "TYPE_PLYR_MOVE_END_TURN"; break;                

		case Msg::TYPE_DEBUG_COMMAND:                      return "TYPE_DEBUG_COMMAND"; break;                     

		case Msg::TYPE_LAST:
			ASSERTF( 0, ( "\nUnknown message type." ) );
	}

	return "Error";
}



/******************************************************************************
 FUNCTION:  WriteToCsv
******************************************************************************/
void SoGameMessage::WriteToCsv( NeCsvOut* pCsvOut )
{
	//-----------------------
	if ( pCsvOut->IsClear() )
	{
		pCsvOut->AddNewRow();
		pCsvOut->AppendStringToLastRow( "ServerId" );
		pCsvOut->AppendStringToLastRow( "ClientId" );
		pCsvOut->AppendStringToLastRow( "MsgType" );
		pCsvOut->AppendStringToLastRow( "MsgNumber" );
		pCsvOut->AppendStringToLastRow( "MsgLength" );
	}


	//-----------------------
	// Add the standard data that every message has
	pCsvOut->AddNewRow();
	pCsvOut->AppendIntToLastRow( (int)_hServerId );
	pCsvOut->AppendIntToLastRow( (int)_hClientId );
	pCsvOut->AppendIntToLastRow( (int)_eType );
	pCsvOut->AppendIntToLastRow( _iMessageNumber );

	//-----------------------
	// Add the number of message-specific entries for this message
	pCsvOut->AppendIntToLastRow( _DataList.GetNumEntries() );

	//-----------------------
	// Add the message specific data
	SoGameMessageDataList::Iterator Iter( &_DataList );
	while ( Iter.Next() )
	{
		SoGameMessage::Data* pData = Iter.GetNodeData();
		pCsvOut->AppendIntToLastRow( pData->GetInt() );
	}
}



/******************************************************************************
 FUNCTION:  ReadFromCsv
******************************************************************************/
void SoGameMessage::ReadFromCsv( NeCsvIn* pCsvIn, int iRowIndex )
{
	//-----------------------
	// read the standard data
	_hServerId                      = pCsvIn->GetInt( iRowIndex, 0, 0 );
	_hClientId                      = pCsvIn->GetInt( iRowIndex, 1, 0 );
	_eType               = (Msg::TYPE)pCsvIn->GetInt( iRowIndex, 2, 0 );
	_iMessageNumber                 = pCsvIn->GetInt( iRowIndex, 3, -1 );
	int iNumMessageData             = pCsvIn->GetInt( iRowIndex, 4, 0 );


	//-----------------------
	// read in the message-specific data
	ClearDataList();

	for ( int i = 0; i < iNumMessageData; i ++ )
	{
		int iData = pCsvIn->GetInt( iRowIndex, 5 + i, 0 );
		AddInt( iData );
	}
}














