/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

USING_NS_CC;


#include "JpHashTable.h"
#include "JpList.h"

#include "SoActorTypes.h"


#define SoGameMessageList JpList< SoGameMessage >
#define SoGameMessageDataList JpList< SoGameMessage::Data >

class NeCsvIn;
class NeCsvOut;

class Msg
{
	public:
		enum TYPE
		{
			// Abbreviations:
			// CLI =  Client
			// SVR =  Server
			// PLYR = Player
			
			TYPE_CLI_REQUESTS_ADD_TO_SVR,                // 0     <-- numbered for debugging network messages.  Please maintain.
			TYPE_SVR_COMMAND_ADD_CLI,                    // 1

			TYPE_CLI_REQUESTS_SEAT,                      // 2
			TYPE_SVR_COMMAND_SEAT_ASSIGNMNET,            // 3

			TYPE_SVR_COMMAND_START_GAME,                 // 4
			TYPE_SVR_COMMAND_INIT_NEW_GAME,              // 5

			TYPE_SVR_COMMAND_PLYR_TAKE_TURN,             // 6
			TYPE_SVR_COMMAND_PLYR_PURCHASE_STOCK,        // 7
			TYPE_SVR_COMMAND_PLYR_PICK_NEW_HOTEL,        // 8
			TYPE_SVR_COMMAND_PLYR_PICK_MERGE_HOTELS,     // 9
			TYPE_SVR_COMMAND_PLYR_MANAGE_OLD_STOCK,      // 10

			TYPE_PLYR_MOVE_PLACE_TILE,                   // 11
			TYPE_PLYR_MOVE_STOCK_PURCHASES,              // 12
			TYPE_PLYR_MOVE_NEW_HOTEL_PICK,               // 13
			TYPE_PLYR_MOVE_MERGE_HOTELS,                 // 14
			TYPE_PLYR_MOVE_MANAGE_OLD_STOCK,             // 15
			TYPE_PLYR_MOVE_DECLARE_LAST_TURN,            // 16

			TYPE_PLYR_MOVE_UNDO,                         // 17
			TYPE_PLYR_MOVE_RESTART_TURN,                 // 18
			TYPE_PLYR_MOVE_END_TURN,                     // 19

			TYPE_DEBUG_COMMAND,                          // 20

			TYPE_LAST
		};

		static const char* GetStringFromType( Msg::TYPE eType );
};



class SoGameMessage
{
	public:

		//-----------------------
		// public classes
		class Data
		{
			public:
				Data() {}
				Data( float f ) { _uValue.f32 = f; }
				Data( int i ) { _uValue.i32 = i; }
				virtual ~Data() {}

				
				int GetInt() { return _uValue.i32; }
				float GetFloat() { return _uValue.f32; }
				
			private:
				union uValue
				{
					int	i32;
					float f32;
				};

				uValue _uValue;
		};


		//-----------------------
		// public functions

		SoGameMessage();
		SoGameMessage( hash hServerId, hash hClientId, Msg::TYPE e );
		virtual ~SoGameMessage();

		void Clear();
		bool IsClear() { return ( _DataList.GetNumEntries() == 0 ); }

		void Set( hash hServerId, hash hClientId, Msg::TYPE e );

		void AddInt( int i );
		void AddHash( hash h ) { AddInt( (int)h ); }
		void AddFloat( float f );
		void AddBool( bool b );

		void StartDataRead();
		bool IsAtDataEnd();

		int FirstInt();
		int NextInt();

		hash FirstHash() { return (hash)FirstInt(); }
		hash NextHash() { return (hash)NextInt(); }

		float FirstFloat();
		float NextFloat();

		bool FirstBool();
		bool NextBool();

		bool IsType( Msg::TYPE e ) { return _eType == e; }
		Msg::TYPE GetType() { return _eType; }
		void SetType( Msg::TYPE e ) { _eType = e; }		

		hash GetServerId() { return _hServerId; }
		void SetServerId( hash h ) { _hServerId = h; }

		int GetClientId() { return _hClientId; }
		void SetClientId( hash h ) { _hClientId = h; }

		int GetMessageNumber() { return _iMessageNumber; }
		void SetMessageNumber( int i ) { _iMessageNumber = i; }

		void CopyMessage( SoGameMessage& ToCopy );

		SoGameMessageDataList* GetDataList() { return &_DataList; }
		void ClearDataList();

		void WriteToCsv( NeCsvOut* pCsvOut );
		void ReadFromCsv( NeCsvIn* pCsvIn, int iRowIndex );


	private:

		//-----------------------
		// private variables
		Msg::TYPE _eType;
		hash _hServerId;
		hash _hClientId;
		int _iMessageNumber;
		SoGameMessageDataList _DataList;
		SoGameMessageDataList::Node* _pDataListNode;
};



