/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "JpHashTable.h"
#include "NeCsvOut.h"
#include "SoGameMessage.h"
#include "SoGameState.h"


USING_NS_CC;


#define SoClientInfoList JpList< ClientInfo >


class SoGameState;


class SoGameServer
{
	public:

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_LOBBY,
			STATE_RUN_GAME,
			STATE_LAST
		};


		//-----------------------
		// public classes
		class ClientInfo
		{
			public:
				ClientInfo() :
				_hClientId( 0 ),
				_iLastMessageNumReceivedFromClient( -1 ),
				_iLastMessageNumSentToClient( -1 )
				{};

				virtual ~ClientInfo() {}

				hash _hClientId;

				int _iLastMessageNumReceivedFromClient;
				int _iLastMessageNumSentToClient;

			private:			
		};


		//-----------------------
		// public functions

		SoGameServer();

		virtual ~SoGameServer();

		void Init( const std::string& sServerName );
		void InitForLoadGame( hash hServerId, hash hMasterRngSeed );

		void Deinit();

		void Update( float dt );

		hash GetServerId() { return _hServerId; }
		hash GenerateServerId();
		static hash GenerateServerIdFromString( const std::string& s );

		int GetNumPlayersInGame() { return _pGame->GetNumPlayers(); }


		// HACK!  Doing this because I'm working fast.  Outside classes should
		// only alter the game state via messages to the server.  TODO: Fix this.
		// mdomowicz 2014_01_21
		SoGameState* GetGameState() { return _pGame; }


		void SendMessageToAllClients( SoGameMessage& Msg );
		void SendMessageToClient( hash hClientId, SoGameMessage& Msg );

		// All of this is needed for Save/Load support.
		void WriteSaveGameCsvToDisk();
		void InvalidateSaveData();
		int GetLastMessageNumReceivedFromClient( hash hClientId );
		void SetIsLoadingGame( bool b ) { _bIsLoadingGame = b; }

	private:

		//-----------------------
		// private variables
		hash _hServerId;

		STATE _eCurrentState;

		SoClientInfoList _ClientInfoList;

		SoGameState* _pGame;		

		NeCsvOut _CsvOutMessageData;

		bool _bIsLoadingGame;


		//-----------------------
		// private functions
		ClientInfo* GetClientInfoFromClientId( hash hClientId );

		void PrintToConsole( const std::string& s );
		void CPrintF( const char* pFormat, ... );
};

