/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"


#include "JpAsciiScreen.h"
#include "JpString.h"
#include "NeCsvIn.h"
#include "SoActorManager.h"
#include "SoActorVisibleHotel.h"
#include "SoActorVisiblePlayer.h"
#include "SoActorVisibleTile.h"
#include "SoConfigManager.h"
#include "SoConsole.h"
#include "SoFsmStateMain.h"
#include "SoGameBitPlayerAi.h"
#include "SoGameBitPlayerDumb.h"
#include "SoGameBitPlayerHuman.h"
#include "SoGameBitPlayerMctsRandom.h"
#include "SoGameBitTile.h"
#include "SoGameClient.h"
#include "SoGameGlobals.h"
#include "SoGameMessage.h"
#include "SoGameServer.h"
#include "SoGameState.h"
#include "SoGameStateUndo.h"
#include "SoGuiAnimWinLog.h"
#include "SoSoundManager.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGame
******************************************************************************/
SoGameState::SoGameState() :
_hClientId( 0 ),
_hServerId( 0 ),
_pServer( NULL ),
_pClient( NULL ),
_eCurrentState( STATE_LAST ),
_bIsLoadingGame( false ),
_iCurrentPlayerNum( -1 ),
_iPreviousPlayerNum( -1 ),
_iNumWinners( 0 ),
_hMasterRngSeed( 0 ),
_iRound( 0 ),
_iNumMergersPerformed( 0 ),
_iNumTilesPlaced( 0 ),
_iBoardWidth( 0 ),
_iBoardHeight( 0 ),
_bIsLastTilePlacementSkipped( false ),
_iLastPlacedTileId( TILE_UNDEFINED_ID ),
_iLastMergerVictimHotelId( HOTEL_UNDEFINED_ID ),
_iLastMergerSurvivorHotelId( HOTEL_UNDEFINED_ID ),
_iLastMergerVictimHotelSize( 0 ),
_bIsVisibleStocks( false ),
_bIsVisibleMoney( false ),
_iPlayerIdThatDeclaredLastTurn( PLAYER_UNDEFINED_PLAYER_NUMBER ),
_bIsGameOver( false ),
_iPlayerNumberCommandedToManageOldStock( PLAYER_UNDEFINED_PLAYER_NUMBER ),
_PlayerArray( GAMERULES_MAX_PLAYERS ),
_bIsLastPlayerMoveAnUndo( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoGame
******************************************************************************/
SoGameState::~SoGameState()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGameState::Init()
{
	SetCurrentState( STATE_WAIT_FOR_GAME_START );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGameState::Deinit()
{
	SoGameBitPlayerArray::Iterator PlayerIter( &_PlayerArray );
	while ( PlayerIter.Next() )
	{
		PlayerIter.GetData()->Deinit();
		delete PlayerIter.GetData();
	}
	_PlayerArray.Clear();


	_TileSet.Deinit();

	DeleteUndoStack();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGameState::Update( float dt )
{
	SoGameBitPlayerArray::Iterator Iter( &_PlayerArray );
	while ( Iter.Next() )
	{
		SoGameBitPlayer* pPlayer = Iter.GetData();
		pPlayer->Update( dt );				
	}
}



/******************************************************************************
 FUNCTION:  UpdateFromServer
******************************************************************************/
bool SoGameState::UpdateFromServer( SoGameMessage* pInMsg )
{
	bool bMessageHandled = false;

	switch ( _eCurrentState )
	{
		case STATE_WAIT_FOR_GAME_START:
		{
			break;
		}
		case STATE_START_NEW_GAME:
		{
			SoGameMessage M;
			SoCodecSvrCommandInitNewGame Encoder;
			Encoder._bIsVisibleStocks = _bIsVisibleStocks;
			Encoder._bIsVisibleMoney = _bIsVisibleMoney;
			Encoder._iStartingPlayer = _MasterRng.GetInt() % GetNumPlayers();
			Encoder.Encode( &M, _hServerId, -1 );

			SendMessageToAllClients( M );

			HandleMessage( &M );

			SetCurrentState( STATE_INITIALIZE_PLAYER_TURNS );
			break;
		}

		case STATE_INITIALIZE_PLAYER_TURNS:
		{
			CPrintF( "====================================================================== Round 1" );
			CPrintF( "Client %x -----------------------------Start turn Player %i:", _hClientId, _iCurrentPlayerNum + 1 );

			SetCurrentState( STATE_START_PLAYER_TURN );
		}

		case STATE_START_PLAYER_TURN:
		{
			SoGameMessage M;
			SoCodecSvrCommandPlyrTakeTurn Encoder;
			Encoder._iPlayerNumber = _iCurrentPlayerNum;

			Encoder.Encode( &M, _hServerId, -1 );
			HandleMessage( &M );
			SendMessageToAllClients( M );

			SetCurrentState( STATE_CONTINUE_PLAYER_TURN );
			break;
		}

		case STATE_CONTINUE_PLAYER_TURN:
		{
			if ( !pInMsg )
				break;

			if ( pInMsg->IsType( Msg::TYPE_PLYR_MOVE_PLACE_TILE ) )
			{
				HandleMessage( pInMsg );
				SendMessageToAllClients( *pInMsg );
				bMessageHandled = true;

				SoCodecPlyrMovePlaceTile Decoder;
				Decoder.Decode( pInMsg );

				if ( Decoder._bSkippingTilePlacement )
				{
					// In rare situations, a player may not have any valid tiles to
					// place.  If this occurs, the skip flag is set, and we detect it
					// here (and in HandleTilePlacementMessage()), and just move on
					// to stock purchasing.  mdomowicz 2014_01_15
					SetCurrentState( STATE_PLAYER_PURCHASE_STOCK_INIT );
				}
				else
				{
					SoGameBitTile* pTile = _TileSet.GetTileById( Decoder._iPlacedTileId );

					if ( _TileSet.DoesTileMergeHotelsWhenPlaced( pTile->GetTileX(), pTile->GetTileY() ) )
					{
						SetCurrentState( STATE_MERGE_HOTELS_INIT );
					}
					else if ( _TileSet.DoesTileCreateNewHotelWhenPlaced( pTile->GetTileX(), pTile->GetTileY() ) )
					{				
						SetCurrentState( STATE_PLAYER_SELECT_NEW_HOTEL_INIT );
					}
					else
					{
						SetCurrentState( STATE_PLAYER_PURCHASE_STOCK_INIT );
					}
				}
			}
			else if 
			( 
				pInMsg->IsType( Msg::TYPE_PLYR_MOVE_END_TURN ) ||
				pInMsg->IsType( Msg::TYPE_PLYR_MOVE_DECLARE_LAST_TURN ) ||
				pInMsg->IsType( Msg::TYPE_PLYR_MOVE_STOCK_PURCHASES ) 
			) 
			{
				HandleMessage( pInMsg );
				SendMessageToAllClients( *pInMsg );
				bMessageHandled = true;

				if ( pInMsg->IsType( Msg::TYPE_PLYR_MOVE_END_TURN ) && IsLastTurnDeclared() )
					SetCurrentState( STATE_LAST );
				else
					SetCurrentState( STATE_START_PLAYER_TURN );
			}
			else if 
			( 
				pInMsg->IsType( Msg::TYPE_PLYR_MOVE_UNDO ) ||
				pInMsg->IsType( Msg::TYPE_PLYR_MOVE_RESTART_TURN ) ||
				pInMsg->IsType( Msg::TYPE_DEBUG_COMMAND ) 
			)
			{
				// Handle the message like normal, but don't change the current state.
				HandleMessage( pInMsg );
				SendMessageToAllClients( *pInMsg );
				bMessageHandled = true;
			}
			else
			{
				ASSERTF( 0, ( "\nUnhandled message." ) );
			}

			break;
		}

		case STATE_PLAYER_PURCHASE_STOCK_INIT:
		{
			// Send the command to purchase stocks, but only
			// if there are stocks available to be purchased.
			if ( AreAnyStocksAvailableForSale() )
			{
				SoGameMessage M;
				SoCodecSvrCommandPlyrPurchaseStock Encoder;
				Encoder._iPlayerNumber = _iCurrentPlayerNum;

				Encoder.Encode( &M, _hServerId, -1 );
				HandleMessage( &M );
				SendMessageToAllClients( M );

				SetCurrentState( STATE_CONTINUE_PLAYER_TURN );
			}
			else
			{
				SetCurrentState( STATE_START_PLAYER_TURN );
			}

			break;
		}

		case STATE_PLAYER_SELECT_NEW_HOTEL_INIT:
		{
			SoGameMessage M;
			SoCodecSvrCommandPlyrPickNewHotel Encoder;
			Encoder._iPlayerNumber = _iCurrentPlayerNum;

			Encoder.Encode( &M, _hServerId, -1 );
			HandleMessage( &M );
			SendMessageToAllClients( M );

			SetCurrentState( STATE_PLAYER_SELECT_NEW_HOTEL );

			break;
		}

		case STATE_PLAYER_SELECT_NEW_HOTEL:
		{
			if ( !pInMsg )
				break;

			if ( pInMsg->IsType( Msg::TYPE_PLYR_MOVE_NEW_HOTEL_PICK ) )
			{
				SoCodecPlyrMoveNewHotelPick Decoder;
				Decoder.Decode( pInMsg );

				HandleMessage( pInMsg );
				SendMessageToAllClients( *pInMsg );

				SetCurrentState( STATE_PLAYER_PURCHASE_STOCK_INIT );

				bMessageHandled = true;
			}
			else if ( pInMsg->IsType( Msg::TYPE_PLYR_MOVE_RESTART_TURN ) )
			{
				// Handle the message like normal, but don't change the current state.
				HandleMessage( pInMsg );
				SendMessageToAllClients( *pInMsg );
				bMessageHandled = true;
			}
			else
			{
				ASSERTF( 0, ( "\nUnhandled message." ) );
			}

			break;
		}

		case STATE_MERGE_HOTELS_INIT:
		{
			SoGameMessage M;
			SoCodecSvrCommandPlyrPickMergeHotels Encoder;
			Encoder._iPlayerNumber = _iCurrentPlayerNum;

			Encoder.Encode( &M, _hServerId, -1 );
			HandleMessage( &M );
			SendMessageToAllClients( M );

			SetCurrentState( STATE_MERGE_HOTELS );

			/*
			// This is the start of code that could be used to detect when
			// the merge survivor/victim are non-ambiguous, and in that case
			// skip the process of asking the player to pick the survivor
			// and victim.  However, I've decided to NOT do this because
			// the windows give the human player a chance to cancel the merge
			// before it happens.  Since undo is disabled after a merge
			// it's good to keep these windows as a last chance move review.
			// mdomowicz 2014_02_10

			SoGameBitTile* pPlacedTile = _TileSet.GetTileById( _iLastPlacedTileId );
			int iTileX = pPlacedTile->GetTileX();
			int iTileY = pPlacedTile->GetTileY();			

			SoGameBitHotelList PotentialSurvivorHotels;
			BuildListOfPotentialMergeSurvivorHotels( &PotentialSurvivorHotels, iTileX, iTileY );
			*/

			break;
		}

		case STATE_MERGE_HOTELS:
		{
			if ( !pInMsg )
				break;

			if ( pInMsg->IsType( Msg::TYPE_PLYR_MOVE_MERGE_HOTELS ) )
			{
				SoCodecPlyrMoveMergeHotels Decoder;
				Decoder.Decode( pInMsg );

				HandleMessage( pInMsg );
				SendMessageToAllClients( *pInMsg );

				SetCurrentState( STATE_MANAGE_OLD_STOCK_INIT );

				bMessageHandled = true;
			}
			else if ( pInMsg->IsType( Msg::TYPE_PLYR_MOVE_RESTART_TURN ) )
			{
				// Handle the message like normal, but don't change the current state.
				HandleMessage( pInMsg );
				SendMessageToAllClients( *pInMsg );
				bMessageHandled = true;
			}
			else
			{
				ASSERTF( 0, ( "\nUnhandled message." ) );
			}
			break;
		}

		case STATE_MANAGE_OLD_STOCK_INIT:
		{
			// To prevent players who do not have any victim stock from
			// having to Manage Old Stock (which is annoying to humans)
			// here we skip all players who do not have victim stock
			int iNextPlayerToManageStock = _iCurrentPlayerNum;

			SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( iNextPlayerToManageStock );
			while ( pPlayer->GetStockCount( _iLastMergerVictimHotelId ) == 0 )
			{
				iNextPlayerToManageStock ++;

				if ( iNextPlayerToManageStock == GetNumPlayers() )
					iNextPlayerToManageStock = 0;

				pPlayer = GetPlayerByPlayerNumber( iNextPlayerToManageStock );

				if ( iNextPlayerToManageStock == _iCurrentPlayerNum )
					break;
			}


			// Send the command
			SoGameMessage M;
			SoCodecSvrCommandPlyrManageOldStock Encoder;
			Encoder._iPlayerNumber = iNextPlayerToManageStock;

			Encoder.Encode( &M, _hServerId, -1 );
			SendMessageToAllClients( M );

			SetCurrentState( STATE_MANAGE_OLD_STOCK );
			break;
		}

		case STATE_MANAGE_OLD_STOCK:
		{
			if ( !pInMsg )
				break;

			if ( pInMsg->IsType( Msg::TYPE_PLYR_MOVE_MANAGE_OLD_STOCK ) )
			{
				HandleMessage( pInMsg );
				SendMessageToAllClients( *pInMsg );
				bMessageHandled = true;


				SoCodecPlyrManageOldStock Decoder;
				Decoder.Decode( pInMsg );

				// Once the current player has managed his old stock, the next player gets to do so.
				int iNextPlayerToManageStock = ( Decoder._iPlayerNumber + 1 );

				// Detect when we need to loop around to lower numbered players
				if ( iNextPlayerToManageStock == GetNumPlayers() )
					iNextPlayerToManageStock = 0;

				
				// To prevent players who do not have any victim stock from
				// having to Manage Old Stock (which is annoying to humans)
				// here we skip all players who do not have victim stock.
				// Note: the logic is slightly different here compared to the init
				// above.  This is intentional.  mdomowicz 2014_02_11
				SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( iNextPlayerToManageStock );
				while 
				( 
					pPlayer->GetStockCount( _iLastMergerVictimHotelId ) == 0 &&
					iNextPlayerToManageStock != _iCurrentPlayerNum
				)
				{
					iNextPlayerToManageStock ++;

					if ( iNextPlayerToManageStock == GetNumPlayers() )
						iNextPlayerToManageStock = 0;

					pPlayer = GetPlayerByPlayerNumber( iNextPlayerToManageStock );
				}


				// If we've given all players a chance to manage their old stock,
				// resume the current turn.
				// Otherwise, give the next player a chance to manage their old stock.
				if ( iNextPlayerToManageStock == _iCurrentPlayerNum )
				{
					// Resume the current turn.  If, after the merger (and after all players have dealt
					// with their old stock), there is another motel to merge, then we do that.
					// Otherwise, we move on, letting the current player purchase stock.

					SoGameBitTile* pTile = GetLastPlacedTile();

					if ( _TileSet.DoesTileMergeHotelsWhenPlaced( pTile->GetTileX(), pTile->GetTileY() ) )
						SetCurrentState( STATE_MERGE_HOTELS_INIT );
					else
						SetCurrentState( STATE_PLAYER_PURCHASE_STOCK_INIT );
				}
				else
				{
					SoGameMessage M;
					SoCodecSvrCommandPlyrManageOldStock Encoder;
					Encoder._iPlayerNumber = iNextPlayerToManageStock;

					Encoder.Encode( &M, _hServerId, -1 );
					SendMessageToAllClients( M );
				}
			}

			break;
		}

		case STATE_LAST:
		{
			break;
		}
	}

	return bMessageHandled;
}



/******************************************************************************
 FUNCTION:  SetCurrentState
******************************************************************************/
void SoGameState::SetCurrentState( STATE e )
{
	_eCurrentState = e;
}



/******************************************************************************
 FUNCTION:  SendMessageToAllClients
******************************************************************************/
void SoGameState::SendMessageToAllClients( SoGameMessage& M )
{
	if ( _pServer )
		_pServer->SendMessageToAllClients( M );
}



/******************************************************************************
 FUNCTION:  SendMessageToServer
******************************************************************************/
void SoGameState::SendMessageToServer( SoGameMessage& M )
{
	if ( _pClient )
		_pClient->SendMessageToServer( M );
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGameState::HandleMessage( SoGameMessage* pM )
{
	if ( !pM )
		return;

	Msg::TYPE eMessage = pM->GetType();

	switch( eMessage )
	{
		case Msg::TYPE_SVR_COMMAND_INIT_NEW_GAME:
		{
			HandleInitNewGameMessage( pM );
			break;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_TAKE_TURN:
		{
			SoCodecSvrCommandPlyrTakeTurn Decoder;
			Decoder.Decode( pM );


			SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( Decoder._iPlayerNumber );

			if ( pPlayer->IsFirstMoveOfPlayersTurn() )
				DetectAndHandleNoPlayableTilesSituation( Decoder._iPlayerNumber );


			if ( !_bIsLoadingGame )
			{
				pPlayer->CommandSelectAction();
			}

			_iCurrentPlayerNum = Decoder._iPlayerNumber;

			if ( _iPreviousPlayerNum != _iCurrentPlayerNum )
			{
				_iPreviousPlayerNum = _iCurrentPlayerNum;
				//PrintToHudLogF( 0, "It is Player %i's turn.", _iCurrentPlayerNum );

				DeleteUndoStack();
			}

			// Notes:
			// 1. Undo has a lot of subtleties with it - especially since it must coordinate with
			//    the human player's state, and also since it must still work with game loading, which
			//    is implemented by playing back the game messages, including the undo messages, while
			//    also not giving any of the players commands.  Ie, game loading is one big special case
			//    and undo has to work under it.  All this to say: be really careful about capturing
			//    undo states in any other place than here.  You must know the implications well, and
			//    do a lot of testing after, if you decide to do captures elsewhere.
			// 2. After an Undo (or Reset Turn) the server sends out a new "take turn" message.  When
			//    this happens we must make sure to not capture another Undo state.
			//
			//    mdomowicz 2015_12_06
			if ( !IsLastPlayerMoveAnUndo() )
				CaptureUndoState();

			break;
		}

		case Msg::TYPE_PLYR_MOVE_UNDO:
		{
			PrintToHudLogF( _iCurrentPlayerNum, "undoes last move." );

			SoGameStateList::Node* pLastNode = _UndoStack.GetLast();
			SoGameStateList::Node* pSecondLastNode = pLastNode->GetPrev();

			SoGameState* pLastGame = pLastNode->GetData();
			SoGameState* pSecondLastGame = pSecondLastNode->GetData();

			if ( pLastGame && pSecondLastGame )
			{
				// The last undo state on the stack is a copy of the current state!  We don't
				// want to undo to that, because...nothing would change.  What we really want is the
				// second to last undo state.
				SoGameStateUndo* pUndoState = (SoGameStateUndo*)pSecondLastGame;
				RestoreFromUndoState( pUndoState );


				// Once the restore occurs, this GameState will have an _eCurrentState = STATE_START_PLAYER_TURN,
				// because that's the STATE the server was in when CaptureUndoState() was called.  But, after restoring
				// we don't want the server to be in STATE_START_PLAYER_TURN, because then that state would run
				// again, sending "take turn" messages out again.  Effectively, STATE_START_PLAYER_TURN would be
				// run twice, which causes all sorts of mayhem.  What we really want is to be in the STATE that
				// comes right after CaptureUndoState() is called.  And this is why we manually set the STATE here.
				// I know this is confusing.  Please see mdomowicz for futher explanation.  mdomowicz 2013_10_13
				//
				// UPDATE: We used to do the this, but it seems simpler to let the server actually go back into
				// STATE_START_PLAYER_TURN, and just request the turn again from the player.  As long as the player
				// is ready for the request (which requires special handling - namely the RestoreFromUndo must
				// make sure to not restore (aka: stomp over), the human player's current _eState and _eRequestedState)
				// this should work fine.  This change was needed to fix a bug where a save game that had Undos would 
				// not load correctly.  mdomowicz 2015_12_06
				//
				//if ( _hServerId != 0 )
				//	SetCurrentState( STATE_CONTINUE_PLAYER_TURN );

				if ( pLastGame )
				{
					_UndoStack.RemoveLast();
					delete pLastGame;
				}
			}

			_bIsLastPlayerMoveAnUndo = true;

			break;
		}

		case Msg::TYPE_PLYR_MOVE_RESTART_TURN:
		{
			PrintToHudLogF( _iCurrentPlayerNum, "restarts turn." );

			SoGameStateList::Node* pFirstNode = _UndoStack.GetFirst();
			SoGameState* pGame = pFirstNode->GetData();
			if ( pGame )
			{
				SoGameStateUndo* pUndoState = (SoGameStateUndo*)pGame;
				RestoreFromUndoState( pUndoState );

				// See the big comment block in the Undo handling for an explanation of why this is needed.
				// UPDATE: this is no longer needed.  See big comment block for updated notes.
				//if ( _hServerId != 0 )
				//	SetCurrentState( STATE_CONTINUE_PLAYER_TURN );

				// Since we're reverting to the first saved UndoState, all the rest are
				// invalid.  So we delete them.
				_UndoStack.Remove( pGame );
				DeleteUndoStack();
				_UndoStack.AddLast( pGame );
			}

			_bIsLastPlayerMoveAnUndo = true;

			break;
		}

		case Msg::TYPE_PLYR_MOVE_END_TURN:
		{
			HandleEndTurnMessage( pM );
			_bIsLastPlayerMoveAnUndo = false;
			break;
		}

		case Msg::TYPE_PLYR_MOVE_PLACE_TILE:
		{
			HandleTilePlacementMessage( pM );
			_bIsLastPlayerMoveAnUndo = false;
			break;
		}

		case Msg::TYPE_PLYR_MOVE_STOCK_PURCHASES:
		{
			HandleStockPurchaseMessage( pM );
			_bIsLastPlayerMoveAnUndo = false;
			break;
		}

		case Msg::TYPE_PLYR_MOVE_NEW_HOTEL_PICK:
		{
			HandleNewHotelPickMessage( pM );
			_bIsLastPlayerMoveAnUndo = false;
			break;
		}

		case Msg::TYPE_PLYR_MOVE_MERGE_HOTELS:
		{
			HandleMergeHotelsMessage( pM );
			_bIsLastPlayerMoveAnUndo = false;
			break;
		}

		case Msg::TYPE_PLYR_MOVE_MANAGE_OLD_STOCK:
		{
			HandleManageOldStockMessage( pM );
			_bIsLastPlayerMoveAnUndo = false;
			break;
		}

		case Msg::TYPE_PLYR_MOVE_DECLARE_LAST_TURN:
		{
			HandleDeclareLastTurnMessage( pM );
			_bIsLastPlayerMoveAnUndo = false;
			break;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_PURCHASE_STOCK:
		{
			SoCodecSvrCommandPlyrPurchaseStock Decoder;
			Decoder.Decode( pM );
			SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( Decoder._iPlayerNumber );

			if ( !_bIsLoadingGame )
			{
				pPlayer->CommandSelectStockPurchases();
			}

			pPlayer->SetIsAllowedToPurchaseStock( true );


			// This CaptureUndoState() occurs right after a player places a tile, and then
			// the server tells that player to purchase stock.
			// Currently, I am adding in support for save/load.  The interaction between
			// Undo, Human Player, Save/Load systems is unfortunately complex (each being
			// complex systems in themselves), and is hard to reason about.  So, while I
			// am quite certain this CaptureUndoState works correctly under normal use
			// (after all, it's been here since 2013), I am less sure about how well it
			// works in conjunction with save/load.  It works in my testing, but only
			// time will tell if it's truly robust.  See other uses and notes with the Undo
			// system, and save/load to learn more.  See also the same in ManhattanE, which
			// is the first DCG game to implement Undo with Save/Load. 
			// mdomowicz 2016_01_31
			if ( !IsLastPlayerMoveAnUndo() )
				CaptureUndoState();

			break;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_PICK_NEW_HOTEL:
		{
			if ( !_bIsLoadingGame )
			{
				SoCodecSvrCommandPlyrPickNewHotel Decoder;
				Decoder.Decode( pM );
				SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( Decoder._iPlayerNumber );
				pPlayer->CommandSelectNewHotel();
			}

			break;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_PICK_MERGE_HOTELS:
		{
			if ( !_bIsLoadingGame )
			{
				SoCodecSvrCommandPlyrPickMergeHotels Decoder;
				Decoder.Decode( pM );
				SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( Decoder._iPlayerNumber );
				pPlayer->CommandSelectMergeHotels();
			}

			break;
		}

		case Msg::TYPE_SVR_COMMAND_PLYR_MANAGE_OLD_STOCK:
		{
			SoCodecSvrCommandPlyrManageOldStock Decoder;
			Decoder.Decode( pM );
			SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( Decoder._iPlayerNumber );

			if ( !_bIsLoadingGame )
			{
				pPlayer->CommandSelectManageOldStock();
			}

			_iPlayerNumberCommandedToManageOldStock = Decoder._iPlayerNumber;

			break;
		}

		case Msg::TYPE_DEBUG_COMMAND:
		{
			SoDebug::HandleDebugMessage( this, pM );
			break;
		}

		default:
			ASSERTF( 0, ( "\nUnhandled GameMessage type." ) );
	}
}



/******************************************************************************
 FUNCTION:  HandleInitNewGameMessage
******************************************************************************/
void SoGameState::HandleInitNewGameMessage( SoGameMessage* pM )
{
	SoCodecSvrCommandInitNewGame Decoder;
	Decoder.Decode( pM );

	_bIsVisibleStocks = Decoder._bIsVisibleStocks;
	_bIsVisibleMoney = Decoder._bIsVisibleMoney;
	_iCurrentPlayerNum = Decoder._iStartingPlayer;
	
	_iBoardWidth = 12;
	_iBoardHeight = 9;

	_HotelSet.Init();

	_iRound = 1;
	_iNumMergersPerformed = 0;
	_iNumTilesPlaced = 0;

	_iNumWinners = 0;

	CreateTiles();

	//-----------------------
	// Place the starting tiles, one for each player.
	// Keep a sorted list, for later reference.
	SoGameBitTileList SortedStartingTiles;
	for ( int i = 0; i < GetNumPlayers(); i ++ )
	{
		SoGameBitTile* pTile = _TileSet.DrawNextTile();
		pTile->SetIsPlaced( true );

		SortedStartingTiles.Add( pTile, pTile->GetId() );
	}

	//-----------------------
	// give each player their starting cash
	SoGameBitPlayerArray::Iterator Iter( &_PlayerArray );
	while ( Iter.Next() )
	{
		SoGameBitPlayer* pPlayer = Iter.GetData();
		pPlayer->SetMoney( GAMERULES_STARTING_MONEY );
	}

	//-----------------------
	// give each player their starting hand of tiles.
	for ( int i = 0; i < GetNumPlayers(); i ++ )
	{
		for ( int j = 0; j < GAMERULES_NUM_TILES_PER_HAND; j ++ )
		{
			SoGameBitTile* pTile = _TileSet.DrawNextTile();
			pTile->SetOwnerPlayerNumber( i );
		}
	}


	//-----------------------
	// Print the initial tile placements to the logs.
	// (per request of Geni Bruhin, via email.)
	std::string sInitialTiles = "Players draw and place initial tiles:";
	SoGameBitTileList::Iterator TileIter( &SortedStartingTiles );
	while ( TileIter.Next() )
	{
		SoGameBitTile* pTile = TileIter.GetNodeData();

		sInitialTiles += " ";
		sInitialTiles += SoActorVisibleTile::GenerateTileLabelTextFromXy( pTile->GetTileX(), pTile->GetTileY() );

		if ( pTile != SortedStartingTiles.GetLast()->GetData() )
			sInitialTiles += ",";
	}
	CPrintF( "Client %x: %s", _hClientId, sInitialTiles.c_str() );
	PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, sInitialTiles.c_str() );


	//-----------------------
	// print additional info to the logs
	SoGameBitTileList::Node* pNearestToA1TileNode = SortedStartingTiles.GetFirst();
	SoGameBitTile* pNearestToA1Tile = pNearestToA1TileNode->GetData();
	std::string sNearestToA1TileName = SoActorVisibleTile::GenerateTileLabelTextFromXy( pNearestToA1Tile->GetTileX(), pNearestToA1Tile->GetTileY() );
	CPrintF( "Client %x,  P%i drew %s, nearest to 1A, and thus goes first.", _hClientId, _iCurrentPlayerNum + 1, sNearestToA1TileName.c_str() );
	PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, "P%i drew %s, nearest to 1A, and thus goes first.", _iCurrentPlayerNum + 1, sNearestToA1TileName.c_str() );

	CPrintF( "Client %x,  The game begins.", _hClientId );
	PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, "The game begins." );
}



/******************************************************************************
 FUNCTION:  HandleEndTurnMessage
******************************************************************************/
void SoGameState::HandleEndTurnMessage( SoGameMessage* pM )
{
	DeleteUndoStack();

	SoCodecPlyrMoveEndTurn Decoder;
	Decoder.Decode( pM );

	int iPlayerNumber = Decoder._iPlayerNumber;
	SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( iPlayerNumber );

	if ( IsLastTurnDeclared() )
	{
		PrintToHudLogF( iPlayerNumber, "ends turn, and ends the game!" );		
	}
	else
	{
		int iNumTilesDiscarded = 0;
		std::string sDiscardedTiles = "";

		//-----------------------
		// Discard all of the players tiles that 
		// would merge safe hotels.  These tiles are no
		// longer valid, and never will be again.
		SoGameBitTileArray* pAllTiles = _TileSet.GetAllTiles();
		SoGameBitTileArray::Iterator TileIter( pAllTiles );
		while ( TileIter.Next() )
		{
			SoGameBitTile* pTile = TileIter.GetData();
			if ( pTile->GetOwnerPlayerNumber() == iPlayerNumber && !pTile->IsPlaced() )
			{
				if ( DoesTileMergeSafeHotelsWhenPlaced( pTile->GetTileX(), pTile->GetTileY() ) )
				{
					_TileSet.DiscardTile( pTile );

					std::string sLabelText = SoActorVisibleTile::GenerateTileLabelTextFromXy( pTile->GetTileX(), pTile->GetTileY() );
					sDiscardedTiles += " ";
					sDiscardedTiles += sLabelText;

					iNumTilesDiscarded ++;
				}
			}
		}

		//-----------------------
		// log the discards	
		if ( ShouldPresentFeedback() )
		{
			std::string sFinalText = "";
			if ( iNumTilesDiscarded > 0 )
			{
				sFinalText += "discarded tile(s):";
				sFinalText += sDiscardedTiles;
				//sFinalText += ".";
				PrintToHudLogF( iPlayerNumber, sFinalText.c_str() );
			}
		}

		//-----------------------
		// Draw new tiles.  1 tile for the end of the turn
		// and 1 tile for each discarded tile.
		int iNumTilesToDraw = iNumTilesDiscarded + 1;
		int iNumTilesDrawn = 0;

		for ( int i = 0; i < iNumTilesToDraw; i ++ )
		{
			SoGameBitTile* pTile = _TileSet.DrawNextTile();
			if ( pTile )
			{
				pTile->SetOwnerPlayerNumber( iPlayerNumber );
				iNumTilesDrawn ++;
			}
		}

		//-----------------------
		// Log the actions
		if ( ShouldPresentFeedback() )
		{
			if ( pPlayer->MustPrintNonStockPurchaseToLog() )
			{
				// There are two ways for the human player to "buy no stocks".
				// He can hit OK in the purchase window with no stocks chosen, which
				// sends the stock purchase message, and will print "player buys no stocks".
				// Or he can hit "cancel", which doesn't send a message, and prints nothing.
				// If the second case happens, we still need to notify all players that 
				// the current player didn't buy anything. Thus we detect and notify here.
				// mdomowicz 2014_02_10
				CPrintF( "Client %x,  Player %i: %s", _hClientId, iPlayerNumber + 1, "buys no stock" );
				PrintToHudLogF( iPlayerNumber, "buys no stock" );
			}

			if ( iNumTilesDrawn == 0 )
			{
				PrintToHudLogF( iPlayerNumber, "ends turn. Does not draw new tile (none left)" );
			}
			else if ( iNumTilesDrawn == 1 )
			{
				if ( iNumTilesDrawn < iNumTilesToDraw )
					PrintToHudLogF( iPlayerNumber, "draws 1 tile, rather than %i (no tiles left). Ends turn", iNumTilesToDraw );
				else
					PrintToHudLogF( iPlayerNumber, "draws new tile and ends turn" );
			}
			else if ( iNumTilesDrawn > 1 )
			{
				if ( iNumTilesDrawn < iNumTilesToDraw )
					PrintToHudLogF( iPlayerNumber, "draws %i tiles, rather than %i (no tiles left). Ends turn", iNumTilesDrawn, iNumTilesToDraw );
				else
					PrintToHudLogF( iPlayerNumber, "draws %i tiles and ends turn", iNumTilesDrawn );
			}
		}
	}
	

	//-----------------------
	// Clear game flags
	_bIsLastTilePlacementSkipped = false;
	_iLastPlacedTileId = TILE_UNDEFINED_ID;
	_iLastMergerVictimHotelId = HOTEL_UNDEFINED_ID;
	_iLastMergerSurvivorHotelId = HOTEL_UNDEFINED_ID;
	_iLastMergerVictimHotelSize = 0;
	
	// Clear player flags so that they're ready for use the next time its this players turn.
	pPlayer->SetNumStocksPurchasedThisTurn( 0 );
	pPlayer->SetMustPrintNonStockPurchaseToLog( true );
	pPlayer->SetIsAllowedToPurchaseStock( false );
	pPlayer->SetIsFirstMoveOfPlayersTurn( true );


	if ( IsLastTurnDeclared() )
	{
		CPrintF( "Client %x -------------------------------End turn Player %i:", _hClientId, iPlayerNumber + 1 );
		ProcessEndOfGame();

		if ( ShouldPresentFeedback() )
			PlaySoundEffect( JpHash( "SfxGameOver.wav" ) );
	}
	else
	{
		CPrintF( "Client %x -------------------------------End turn Player %i:", _hClientId, iPlayerNumber + 1 );

		_iCurrentPlayerNum ++;
		if ( _iCurrentPlayerNum == _PlayerArray.GetNumEntries() )
		{
			_iCurrentPlayerNum = 0;

			if ( _iCurrentPlayerNum == 0 )
			{
				_iRound ++;
				CPrintF( "====================================================================== Round %i", _iRound );
			}
		}

		CPrintF( "Client %x -----------------------------Start turn Player %i:", _hClientId, _iCurrentPlayerNum + 1 );
	}
}



/******************************************************************************
 FUNCTION:  HandleStockPurchaseMessage
******************************************************************************/
void SoGameState::HandleStockPurchaseMessage( SoGameMessage* pM )
{
	struct Local
	{
		static int ProcessPurchase( SoGameState* pGameState, SoGameBitPlayer* pPlayer, int iHotelId, int iAmount )
		{
			if ( iAmount > 0 )
			{
				SoGameBitHotel* pHotel = pGameState->GetHotelByHotelId( iHotelId );
				int iPricePerStock = pHotel->GetPricePerStockFromTable( iHotelId, pHotel->GetSize() );
				int iTotalCost = ( iAmount * iPricePerStock );

				pPlayer->AdjustMoney( -iTotalCost );
				pPlayer->AdjustStockCount( iHotelId, iAmount );
				pPlayer->AdjustNumStocksPurchasedThisTurn( iAmount );

				pHotel->AdjustAvailableStock( -iAmount );

				return iTotalCost;
			}

			return 0;
		}

		static void AppendHotelString( std::string& sResult, int iHotelId, int iAmount, int iTotalPurchases, int& iNumPurchasesPrinted )
		{
			if ( iAmount > 0 )
			{
				std::string sHotelName = SoActorVisibleHotel::GetHoteNameById( iHotelId );
				sResult += " ";
				sResult += JpString::IntToString( iAmount );

				if ( iAmount == 1 )
					sResult += " stock of ";
				else
					sResult += " stocks of ";

				sResult += sHotelName;

				iNumPurchasesPrinted += iAmount;
				if ( iNumPurchasesPrinted < iTotalPurchases )
				{
					sResult += ", ";
				}
				else
				{
					sResult += ".";
				}
			}
		}
	};


	//-----------------------
	// Get the number of stocks purchased for each hotel chain.
	SoCodecPlyrMoveStockPurchases Decoder;
	Decoder.Decode( pM );

	int iPlayerNumber = Decoder._iPlayerNumber;
	int iAmount0 = Decoder._iAmountOfHotelWithId0;
	int iAmount1 = Decoder._iAmountOfHotelWithId1;
	int iAmount2 = Decoder._iAmountOfHotelWithId2;
	int iAmount3 = Decoder._iAmountOfHotelWithId3;
	int iAmount4 = Decoder._iAmountOfHotelWithId4;
	int iAmount5 = Decoder._iAmountOfHotelWithId5;
	int iAmount6 = Decoder._iAmountOfHotelWithId6;


	//-----------------------
	// Purchase the stocks
	SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( iPlayerNumber );

	int iFinalCost = 0;
	iFinalCost += Local::ProcessPurchase( this, pPlayer, 0, iAmount0 );
	iFinalCost += Local::ProcessPurchase( this, pPlayer, 1, iAmount1 );
	iFinalCost += Local::ProcessPurchase( this, pPlayer, 2, iAmount2 );
	iFinalCost += Local::ProcessPurchase( this, pPlayer, 3, iAmount3 );
	iFinalCost += Local::ProcessPurchase( this, pPlayer, 4, iAmount4 );
	iFinalCost += Local::ProcessPurchase( this, pPlayer, 5, iAmount5 );
	iFinalCost += Local::ProcessPurchase( this, pPlayer, 6, iAmount6 );


	//-----------------------
	// report the results
	if ( ShouldPresentFeedback() )
	{
		int iTotalPurchases = ( iAmount0 + iAmount1 + iAmount2 + iAmount3 + iAmount4 + iAmount5 + iAmount6 );

		if ( iTotalPurchases > 0 )
		{
			int iNumPurchasesPrinted = 0;

			std::string sFinalString = "purchases";
			Local::AppendHotelString( sFinalString, 0, iAmount0, iTotalPurchases, iNumPurchasesPrinted );
			Local::AppendHotelString( sFinalString, 1, iAmount1, iTotalPurchases, iNumPurchasesPrinted );
			Local::AppendHotelString( sFinalString, 2, iAmount2, iTotalPurchases, iNumPurchasesPrinted );
			Local::AppendHotelString( sFinalString, 3, iAmount3, iTotalPurchases, iNumPurchasesPrinted );
			Local::AppendHotelString( sFinalString, 4, iAmount4, iTotalPurchases, iNumPurchasesPrinted );
			Local::AppendHotelString( sFinalString, 5, iAmount5, iTotalPurchases, iNumPurchasesPrinted );
			Local::AppendHotelString( sFinalString, 6, iAmount6, iTotalPurchases, iNumPurchasesPrinted );

			sFinalString += " Spends $";
			sFinalString += JpString::IntToString( iFinalCost );

			CPrintF( "Client %x,  Player %i: %s", _hClientId, iPlayerNumber + 1, sFinalString.c_str() );
			PrintToHudLogF( iPlayerNumber, sFinalString.c_str() );
		}
		else
		{
			CPrintF( "Client %x,  Player %i: %s", _hClientId, iPlayerNumber + 1, "buys no stock" );
			PrintToHudLogF( iPlayerNumber, "buys no stock" );
		}

		// If this function was run, a message has already been logged
		// And so there is no longer a need to log anything more.
		pPlayer->SetMustPrintNonStockPurchaseToLog( false );
	}
}



/******************************************************************************
 FUNCTION:  HandleNewHotelPickMessage
******************************************************************************/
void SoGameState::HandleNewHotelPickMessage( SoGameMessage* pM )
{
	SoCodecPlyrMoveNewHotelPick Decoder;
	Decoder.Decode( pM );
	int iPlayerNumber = Decoder._iPlayerNumber;
	int iNewHotelId = Decoder._iHotelId;


	//-----------------------
	// set the current tile and all adjacent tiles to
	// the new hotel id.
	SoGameBitTile* pTile = _TileSet.GetTileById( _iLastPlacedTileId );
	_TileSet.CreateNewHotelChainAtTile( pTile->GetTileX(), pTile->GetTileY(), iNewHotelId );


	//-----------------------
	// place the hotel on the tile
	SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( iNewHotelId );
	pHotel->SetPlacementTileId( pTile->GetId() );


	//-----------------------
	// Give the player a bonus stock for founding the new hotel.
	bool bBonusStockAcquired = false;
	if ( pHotel->GetAvailableStock() > 0 )
	{
		bBonusStockAcquired = true;

		SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( iPlayerNumber );
		pPlayer->AdjustStockCount( iNewHotelId, 1 );
		pHotel->AdjustAvailableStock( -1 );
	}


	//-----------------------
	RecalculateHotelSizes();


	//-----------------------
	// Report to the hug log and console log
	if ( ShouldPresentFeedback() )
	{
		std::string sHotelName = SoActorVisibleHotel::GetHoteNameById( iNewHotelId );
		if ( bBonusStockAcquired )
		{
			CPrintF( "Client %x,  Player %i: Creates new corporate chain: %s. Received 1 bonus stock", _hClientId, iPlayerNumber + 1, sHotelName.c_str() );
			PrintToHudLogF( iPlayerNumber, "creates new corporate chain: %s. Received 1 bonus stock", sHotelName.c_str() );
		}
		else
		{
			CPrintF( "Client %x,  Player %i: Creates new corporate chain: %s. No bonus stock received", _hClientId, iPlayerNumber + 1, sHotelName.c_str() );
			PrintToHudLogF( iPlayerNumber, "creates new corporate chain: %s. No bonus stock received", sHotelName.c_str() );
		}

		PlaySoundEffect( JpHash( "SfxNewCompanyFounded.wav" ) );
	}
}



/******************************************************************************
 FUNCTION:  HandleMergeHotelsMessage
******************************************************************************/
void SoGameState::HandleMergeHotelsMessage( SoGameMessage* pM )
{
	SoCodecPlyrMoveMergeHotels Decoder;
	Decoder.Decode( pM );

	int iPlayerNumber = Decoder._iPlayerNumber;
	int iSurvivorHotelId = Decoder._iSurvivorHotelId;
	int iVictimHotelId = Decoder._iVictimHotelId;

	SoGameBitHotel* pSurvivorHotel = _HotelSet.GetHotelByHotelId( iSurvivorHotelId );
	SoGameBitHotel* pVictimHotel = _HotelSet.GetHotelByHotelId( iVictimHotelId );


	//-----------------------
	// Report the merge to the log.  This should happen before the
	// minority and majority bonuses are given out, because we want
	// the bonus log report to come after this one.
	if ( ShouldPresentFeedback() )
	{
		std::string sSurvivorHotelName = SoActorVisibleHotel::GetHoteNameById( iSurvivorHotelId );
		std::string sVictimHotelName = SoActorVisibleHotel::GetHoteNameById( iVictimHotelId );
		CPrintF( "Client %x,  Player %i: merges hotels. %s absorbs %s", _hClientId, iPlayerNumber + 1, sSurvivorHotelName.c_str(), sVictimHotelName.c_str() );
		PrintToHudLogF( iPlayerNumber, "merges hotels. %s absorbs %s", sSurvivorHotelName.c_str(), sVictimHotelName.c_str() );

		PlaySoundEffect( JpHash( "SfxMerger.wav" ) );
	}


	//-----------------------
	// give the majority and minority stock 
	// holders bonuses for the victim hotel
	GiveMajorityMinorityStockOwnersBonuses( iVictimHotelId );


	//-----------------------
	// Turn the victim hotel tiles into survivor hotel tiles.
	int iNumTiles = _TileSet.GetNumTiles();

	for ( int i = 0; i < iNumTiles; i ++ )
	{
		SoGameBitTile* pCurrentTile = _TileSet.GetTileById( i );

		if ( pCurrentTile->GetHotelId() == iVictimHotelId )
			pCurrentTile->SetHotelId( iSurvivorHotelId );
	}


	//-----------------------
	// The placed tile, and of its neighbor tiles that don't have a
	// hotel id assigned, are all set to the surviving hotel id.
	SoGameBitTile* pLastPlacedTile = GetLastPlacedTile();
	_TileSet.ExpandHotelChainAtTile( pLastPlacedTile->GetTileX(), pLastPlacedTile->GetTileY(), iSurvivorHotelId );


	//-----------------------
	// remove the victim hotel from the board
	pVictimHotel->SetPlacementTileId( TILE_UNDEFINED_ID );


	//-----------------------
	// save key data on the merger, for use when handling old (defunct) stock.
	_iLastMergerVictimHotelId = pVictimHotel->GetHotelId();
	_iLastMergerSurvivorHotelId = pSurvivorHotel->GetHotelId();
	_iLastMergerVictimHotelSize = pVictimHotel->GetSize();
	

	//-----------------------
	RecalculateHotelSizes();
	_iNumMergersPerformed ++;


	//-----------------------
	// Disable undo after a merger, since once the merger happens
	// All parties are notified about which stocks each player owns.
	// This is a problem for "hidden stock" games, but even for revealed
	// stock games, after the merger each player is asked to manage
	// their old stock, and once their decisions are made, the current
	// player has new information about the game, and so allowing him
	// to undo would be unfair.  mdomowicz 2014_02_10
	DeleteUndoStack();
}



/******************************************************************************
 FUNCTION:  HandleManageOldStockMessage
******************************************************************************/
void SoGameState::HandleManageOldStockMessage( SoGameMessage* pM )
{
	SoCodecPlyrManageOldStock Decoder;
	Decoder.Decode( pM );

	int iPlayerNumber = Decoder._iPlayerNumber;
	int iAmountKeep = Decoder._iNumStocksToKeep;
	int iAmountToSell = Decoder._iNumStocksToSell;
	int iAmountToTrade = Decoder._iNumStocksToTrade;

	int iVictimHotelId = GetLastMergerVictimHotelId();
	int iSurvivorHotelId = GetLastMergerSurvivorHotelId();
	int iVictimHotelSize = GetLastMergerVictimHotelSize();	


	SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( iPlayerNumber );
	SoGameBitHotel* pVictimHotel = GetHotelByHotelId( iVictimHotelId );
	SoGameBitHotel* pSurvivorHotel = GetHotelByHotelId( iSurvivorHotelId );

	std::string sVictimHotelName = SoActorVisibleHotel::GetHoteNameById( iVictimHotelId );
	std::string sSurvivorHotelName = SoActorVisibleHotel::GetHoteNameById( iSurvivorHotelId );

	int iProfit = 0;
	int iSurvivorStockGained = 0;

	if ( iAmountToSell > 0 )
	{
		//-----------------------
		// process the sale
		pVictimHotel->AdjustAvailableStock( iAmountToSell );
		pPlayer->AdjustStockCount( iVictimHotelId, -iAmountToSell );

		int iCostPerStock = SoGameBitHotel::GetPricePerStockFromTable( iVictimHotelId, iVictimHotelSize );
		iProfit = ( iCostPerStock * iAmountToSell );

		pPlayer->AdjustMoney( iProfit );
	}
	
	if ( iAmountToTrade > 0 )
	{
		//-----------------------
		// process the trade
		pVictimHotel->AdjustAvailableStock( iAmountToTrade );
		pPlayer->AdjustStockCount( iVictimHotelId, -iAmountToTrade );

		iSurvivorStockGained = iAmountToTrade / 2;

		pSurvivorHotel->AdjustAvailableStock( -iSurvivorStockGained );
		pPlayer->AdjustStockCount( iSurvivorHotelId, iSurvivorStockGained );	
	}

	_iPlayerNumberCommandedToManageOldStock = PLAYER_UNDEFINED_PLAYER_NUMBER;


	//-----------------------
	// This entire IF block is just for reporting
	// the results to the logs.
	if ( ShouldPresentFeedback() )
	{
		if ( iAmountKeep > 0 )
		{
			char acText[ 128 ] = "";

			if ( ( iAmountToSell + iAmountToTrade ) == 0 )
			{
				sprintf
				( 
					acText, 
					"keeps all %i share(s) of their defunct %s stock", 
					iAmountKeep, 
					sVictimHotelName.c_str() 
				);
			}
			else
			{
				sprintf
				( 
					acText, 
					"keeps %i share(s) of their defunct %s stock", 
					iAmountKeep, 
					sVictimHotelName.c_str() 
				);
			}

			CPrintF( "Client %x,  Player %i: %s", _hClientId, iPlayerNumber + 1, acText );
			PrintToHudLogF( iPlayerNumber, acText );
		}

		if ( iAmountToSell > 0 )
		{
			//-----------------------
			// report to the hud log
			std::string sStockOrStocks = "stock";
			if ( iAmountToSell > 1 )
				sStockOrStocks = "stocks";

			char acText[ 128 ] = "";
			if ( ( iAmountKeep + iAmountToTrade ) == 0 )
			{
				sprintf
				( 
					acText, 
					"sells all %i share(s) of %s and earns $%i",
					iAmountToSell, 
					sVictimHotelName.c_str(),
					iProfit 
				);
			}
			else
			{
				sprintf
				( 
					acText, 
					"sells %i %s of %s and earns $%i", 			
					iAmountToSell, 
					sStockOrStocks.c_str(), 
					sVictimHotelName.c_str(),
					iProfit 
				);
			}

			CPrintF( "Client %x,  Player %i: %s", _hClientId, iPlayerNumber + 1, acText );
			PrintToHudLogF( iPlayerNumber, acText );
		}
		
		if ( iAmountToTrade > 0 )
		{
			//-----------------------
			// report to the hud log
			std::string sTradingStockOrStocks = "stock";
			if ( iAmountToTrade > 1 )
				sTradingStockOrStocks = "stocks";

			std::string sGainingStockOrStocks = "stock";
			if ( iSurvivorStockGained > 1 )
				sGainingStockOrStocks = "stocks";


			char acText[ 128 ] = "";
			if ( ( iAmountKeep + iAmountToSell ) == 0 )
			{
				sprintf
				( 
					acText, "trades all %i share(s) of %s for %i %s of %s",
					iAmountToTrade, 
					sVictimHotelName.c_str(),
					iSurvivorStockGained,
					sGainingStockOrStocks.c_str(),
					sSurvivorHotelName.c_str()
				);
			}
			else
			{
				sprintf
				( 
					acText, "trades %i %s of %s for %i %s of %s",
					iAmountToTrade, 
					sTradingStockOrStocks.c_str(), 
					sVictimHotelName.c_str(),
					iSurvivorStockGained,
					sGainingStockOrStocks.c_str(),
					sSurvivorHotelName.c_str()
				);
			}

			CPrintF( "Client %x,  Player %i: %s", _hClientId, iPlayerNumber + 1, acText );
			PrintToHudLogF( iPlayerNumber, acText );
		}
	}
}



/******************************************************************************
 FUNCTION:  HandleDeclareLastTurnMessage
******************************************************************************/
void SoGameState::HandleDeclareLastTurnMessage( SoGameMessage* pM )
{
	SoCodecPlyrMoveDeclareLastTurn Decoder;
	Decoder.Decode( pM );

	_iPlayerIdThatDeclaredLastTurn = Decoder._iPlayerNumber;

	if ( ShouldPresentFeedback() )
	{
		CPrintF( "Client %x,  Player %i: Declares the game over at the end of this turn", _hClientId, _iPlayerIdThatDeclaredLastTurn + 1 );
		PrintToHudLogF( _iPlayerIdThatDeclaredLastTurn, "declares the game over at the end of this turn" );
	}
}



/******************************************************************************
 FUNCTION:  HandleTilePlacementMessage
******************************************************************************/
void SoGameState::HandleTilePlacementMessage( SoGameMessage* pM )
{
	SoCodecPlyrMovePlaceTile Decoder;
	Decoder.Decode( pM );

	int iPlayerNumber = Decoder._iPlayerNumber;
	int iTileId = Decoder._iPlacedTileId;
	bool bSkippingPlacement = Decoder._bSkippingTilePlacement;

	if ( bSkippingPlacement )
	{
		_bIsLastTilePlacementSkipped = true;

		if ( ShouldPresentFeedback() )
		{
			CPrintF( "Client %x,  Player %i: could not place any tiles. Continues turn.", _hClientId, iPlayerNumber + 1 );
			PrintToHudLogF( iPlayerNumber, "could not place any tiles. Continues turn." );
		}
	}
	else
	{
		//-----------------------
		// place the tile
		SoGameBitTile* pTile = _TileSet.GetTileById( iTileId );
		pTile->SetIsPlaced( true );


		//-----------------------
		// Set the hotel id, but only if this tile placement won't result
		// in a hotel merger.
		if ( !_TileSet.DoesTileMergeHotelsWhenPlaced( pTile->GetTileX(), pTile->GetTileY() ) )
		{
			// if the tile is next to a hotel, then we grow the hotel chain
			int iHotelId = _TileSet.FindAdjacentHotelToExpand( pTile->GetTileX(), pTile->GetTileY() );
			_TileSet.ExpandHotelChainAtTile( pTile->GetTileX(), pTile->GetTileY(), iHotelId );
		}


		//-----------------------
		RecalculateHotelSizes();


		//-----------------------
		// remember which tile was just placed.
		_iLastPlacedTileId = pTile->GetId();


		//-----------------------
		// misc bookkeeping
		SoGameBitPlayer* pPlayer = GetPlayerByPlayerNumber( iPlayerNumber );
		pPlayer->SetIsFirstMoveOfPlayersTurn( false );
		_iNumTilesPlaced ++;


		//-----------------------
		// Report to the hug log and console log
		if ( ShouldPresentFeedback() )
		{
			std::string sTileLabelText = SoActorVisibleTile::GenerateTileLabelTextFromXy( pTile->GetTileX(), pTile->GetTileY() );

			CPrintF( "Client %x,  Player %i: places tile: %s", _hClientId, iPlayerNumber + 1, sTileLabelText.c_str() );
			PrintToHudLogF( iPlayerNumber, "places tile: %s", sTileLabelText.c_str() );

			PlaySoundEffect( JpHash( "SfxTilePlaced.wav" ) );
		}
	}
}



/******************************************************************************
 FUNCTION:  ProcessEndOfGame
******************************************************************************/
void SoGameState::ProcessEndOfGame()
{
	//-----------------------
	_bIsGameOver = true;


	//-----------------------
	// Give the majority and minority stock holders bonuses
	// for every hotel that's currently active.
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( i );

		if ( pHotel->IsPlaced() )
		{
			GiveMajorityMinorityStockOwnersBonuses( i );
		}
	}


	//-----------------------
	// Each player then sells their stock back to the bank
	SoGameBitPlayerArray::Iterator PlayerIter( &_PlayerArray );
	while ( PlayerIter.Next() )
	{
		SoGameBitPlayer* pPlayer = PlayerIter.GetData();

		for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
		{
			SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( i );

			if ( pHotel->IsPlaced() )
			{
				int iNumStockOwned = pPlayer->GetStockCount( i );
				int iStockPrice = ( SoGameBitHotel::GetPricePerStockFromTable( i, pHotel->GetSize() ) );
				int iProfit = ( iStockPrice * iNumStockOwned );

				pPlayer->AdjustMoney( iProfit );
			}
		}
	}


	//-----------------------
	// Print the final scores to the game log and hud log.
	std::string sFinalScores = "Final score:";
	PlayerIter.Reset();
	while ( PlayerIter.Next() )
	{
		SoGameBitPlayer* pPlayer = PlayerIter.GetData();

		sFinalScores += "  ";
		sFinalScores += SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( pPlayer->GetPlayerNumber() );
		sFinalScores += ": $";
		sFinalScores += JpString::IntToString( pPlayer->GetMoney() );
	}
	CPrintF( "Client %x,  %s", _hClientId, sFinalScores.c_str() );
	PrintToHudLogF( -1, sFinalScores.c_str() );
	

	//-----------------------
	// find out the largest sum of money owned by
	// any player.
	int iMostMoney = - 1;
	PlayerIter.Reset();
	while ( PlayerIter.Next() )
	{
		SoGameBitPlayer* pPlayer = PlayerIter.GetData();
		if ( pPlayer->GetMoney() > iMostMoney )
			iMostMoney = pPlayer->GetMoney();
	}

	
	//-----------------------
	// Determine how many players are winners
	// and flag each winner.
	_iNumWinners = 0;
	PlayerIter.Reset();
	while ( PlayerIter.Next() )
	{
		SoGameBitPlayer* pPlayer = PlayerIter.GetData();
		if ( pPlayer->GetMoney() == iMostMoney )
		{
			pPlayer->SetIsWinner( true );
			_iNumWinners ++;
		}
	}
	

	//-----------------------
	// print the results to the game log and the hud log
	if ( ShouldPresentFeedback() )
	{
		if ( _iNumWinners > 1 )
		{
			std::string sText = "Players ";

			PlayerIter.Reset();
			while ( PlayerIter.Next() )
			{
				SoGameBitPlayer* pPlayer = PlayerIter.GetData();
				if ( pPlayer->IsWinner() )
				{
					sText += " ";
					sText += SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( pPlayer->GetPlayerNumber() );
				}
			}

			sText += " tie for first place!";
			
			CPrintF( "Client %x,  %s", _hClientId, sText.c_str() );
			PrintToHudLogF( -1, sText.c_str() );
		}
		else
		{
			PlayerIter.Reset();
			while ( PlayerIter.Next() )
			{
				SoGameBitPlayer* pPlayer = PlayerIter.GetData();
				std::string sPlayerName = SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( pPlayer->GetPlayerNumber() );

				if ( pPlayer->IsWinner() )
				{
					CPrintF( "Client %x,  %s: wins the game!", _hClientId, sPlayerName.c_str() );
					PrintToHudLogF( pPlayer->GetPlayerNumber(), "wins the game!" );
					break;
				}
			}
		}


		CPrintF( "Client %x,  Stats: %i Rounds, %i Tile placements, %i Mergers", _hClientId, _iRound, _iNumTilesPlaced, _iNumMergersPerformed );
		PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, "Stats: %i Rounds, %i Tile placements, %i Mergers", _iRound, _iNumTilesPlaced, _iNumMergersPerformed );
	}
}



/******************************************************************************
 FUNCTION:  CalculateMajorityMinorityStockOwnersBonusesBasic
******************************************************************************/
void SoGameState::CalculateMajorityMinorityStockOwnersBonusesBasic( int iHotelId, int* paiBonusesResult )
{
	int iUnused1, iUnused2, iUnused3, iUnused4, iUnused5, iUnused6;
	memset( paiBonusesResult, 0, GAMERULES_MAX_PLAYERS * sizeof( int ) );

	CalculateMajorityMinorityStockOwnersBonuses
	(
		iHotelId, 
		&iUnused1,
		&iUnused2,
		&iUnused3,
		&iUnused4,
		&iUnused5,
		&iUnused6,
		paiBonusesResult
	);
}



/******************************************************************************
 FUNCTION:  CalculateMajorityMinorityStockOwnersBonuses
******************************************************************************/
void SoGameState::CalculateMajorityMinorityStockOwnersBonuses
( 
	int iHotelId, 
	int* piMostStockOwnedResult,
	int* piSecondMostStockOwnedResult,
	int* piNumMinorityStockOwnersResult,
	int* piNumMajorityStockOwnersResult,
	int* piFinalMinorityBonusResult,
	int* piFinalMajorityBonusResult,
	int* paiBonusesResult 
)
{
	//-----------------------
	// Determine what are the "most owned" and "second most owned" 
	// amounts of the hotel stock
	int iMostStockOwned = 0;
	int iSecondMostStockOwned = 0;

	SoGameBitPlayerArray::Iterator PlayerIter( &_PlayerArray );
	while ( PlayerIter.Next() )
	{
		SoGameBitPlayer* pPlayer = PlayerIter.GetData();
		int iNumStocks = pPlayer->GetStockCount( iHotelId );

		if ( iNumStocks > 0 )
		{
			if ( iNumStocks > iMostStockOwned )
			{
				iSecondMostStockOwned = iMostStockOwned;
				iMostStockOwned = iNumStocks;
			}
			else if ( iNumStocks > iSecondMostStockOwned )
			{
				iSecondMostStockOwned = iNumStocks;
			}
		}
	}


	//-----------------------
	// Determine how many players own "most owned" amount of stock
	// and how many players own "second most owned" amount of stock.
	int iNumMajorityStockOwners = 0;
	int iNumMinorityStockOwners = 0;

	PlayerIter.Reset();
	while ( PlayerIter.Next() )
	{
		SoGameBitPlayer* pPlayer = PlayerIter.GetData();
		int iNumStocks = pPlayer->GetStockCount( iHotelId );

		if ( iNumStocks > 0 )
		{
			if ( iNumStocks == iMostStockOwned )
				iNumMajorityStockOwners ++;
			else if ( iNumStocks == iSecondMostStockOwned )
				iNumMinorityStockOwners ++;
		}
	}


	//-----------------------
	// Figure out exactly how much money will go the majority 
	// and minority stock owners.
	SoGameBitHotel *pHotel = GetHotelByHotelId( iHotelId );
	int iMajorityBonus = SoGameBitHotel::GetMajorityBonusFromTable( iHotelId, pHotel->GetSize() );
	int iMinorityBonus = SoGameBitHotel::GetMinorityBonusFromTable( iHotelId, pHotel->GetSize() );

	int iFinalMajorityBonus = 0;
	int iFinalMinorityBonus = 0;

	if ( iNumMajorityStockOwners == 1 )
	{
		if ( iNumMinorityStockOwners == 0 )
		{
			// If there is only 1 stock owner, that players gets both bonuses.
			iFinalMajorityBonus = ( iMajorityBonus + iMinorityBonus );
		}
		else if ( iNumMinorityStockOwners == 1 )
		{
			// If there is only 1 majority stock owner and only 1 minority stock 
			// owner, they each get the bonuses, unaltered.
			iFinalMajorityBonus = iMajorityBonus;
			iFinalMinorityBonus = iMinorityBonus;	
		}
		else if ( iNumMinorityStockOwners > 1 )
		{
			// If there is only 1 majority stock owner, and multiple minority
			// stock owners, then the majority owner gets the full majority bonus
			// and the minority owners split the minority bonus
			iFinalMajorityBonus = iMajorityBonus;
			iFinalMinorityBonus = ( iMinorityBonus / iNumMinorityStockOwners );

			// Rules say that if there is a fractional bonus, round up to the
			// nearest 100 (or in our case, 1).
			if ( ( iMinorityBonus % iNumMinorityStockOwners > 0 ) )
				iFinalMinorityBonus += 1;
		}
	}
	else if ( iNumMajorityStockOwners > 1 )
	{
		// If there are multiple majority stock owners, the owners
		// split the combined bonus.  The minority stock owners get nothing.
		int iCombinedBonus = ( iMajorityBonus + iMinorityBonus );
		iFinalMajorityBonus = ( iCombinedBonus / iNumMajorityStockOwners );

		// Rules say that if there is a fractional bonus, round up to the
		// nearest 100 (or in our case, 1).
		if ( ( iCombinedBonus % iNumMajorityStockOwners > 0 ) )
			iFinalMajorityBonus += 1;
	}


	//-----------------------
	// Assign bonus money to each qualifying player.
	PlayerIter.Reset();
	while ( PlayerIter.Next() )
	{
		SoGameBitPlayer* pPlayer = PlayerIter.GetData();
		paiBonusesResult[ pPlayer->GetPlayerNumber() ] = 0;

		int iNumStocks = pPlayer->GetStockCount( iHotelId );
		if ( iNumStocks > 0 )
		{
			if ( iNumStocks == iMostStockOwned )
				paiBonusesResult[ pPlayer->GetPlayerNumber() ] = iFinalMajorityBonus;
			else if ( iNumStocks == iSecondMostStockOwned )
				paiBonusesResult[ pPlayer->GetPlayerNumber() ] = iFinalMinorityBonus;
		}
	}

	//-----------------------
	// copy back additional results.
	*piMostStockOwnedResult = iMostStockOwned;
	*piSecondMostStockOwnedResult = iSecondMostStockOwned;
	*piNumMinorityStockOwnersResult = iNumMinorityStockOwners;
	*piNumMajorityStockOwnersResult = iNumMajorityStockOwners;
	*piFinalMinorityBonusResult = iFinalMinorityBonus;
	*piFinalMajorityBonusResult = iFinalMajorityBonus;
}



/******************************************************************************
 FUNCTION:  GiveMajorityMinorityStockOwnersBonuses
******************************************************************************/
void SoGameState::GiveMajorityMinorityStockOwnersBonuses( int iHotelId )
{
	int iMostStockOwned = 0;
	int iSecondMostStockOwned = 0;
	int iNumMinorityStockOwners = 0;
	int iNumMajorityStockOwners = 0;
	int iFinalMinorityBonus = 0;
	int iFinalMajorityBonus = 0;
	int aiBonuses[ GAMERULES_MAX_PLAYERS ];
	memset( aiBonuses, 0, GAMERULES_MAX_PLAYERS * sizeof( int ) );

	//-----------------------
	// calculate the bonuses
	CalculateMajorityMinorityStockOwnersBonuses
	(
		iHotelId, 
		&iMostStockOwned,
		&iSecondMostStockOwned,
		&iNumMinorityStockOwners,
		&iNumMajorityStockOwners,
		&iFinalMinorityBonus,
		&iFinalMajorityBonus,
		&aiBonuses[ 0 ]
	);


	//-----------------------
	// apply the bonuses to the players
	SoGameBitPlayerArray::Iterator PlayerIter( &_PlayerArray );
	while ( PlayerIter.Next() )
	{
		SoGameBitPlayer* pPlayer = PlayerIter.GetData();
		int iBonus = aiBonuses[ pPlayer->GetPlayerNumber() ];
		pPlayer->AdjustMoney( iBonus );
	}	


	//-----------------------
	// This entire IF block is just for reporting
	// the results to the logs.
	if ( ShouldPresentFeedback() )
	{
		std::string sStocksOwned = "";
		sStocksOwned += SoActorVisibleHotel::GetHoteNameById( iHotelId );
		sStocksOwned += " stocks owned:";

		PlayerIter.Reset();
		while ( PlayerIter.Next() )
		{
			SoGameBitPlayer* pPlayer = PlayerIter.GetData();
			sStocksOwned += "    ";
			sStocksOwned += SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( pPlayer->GetPlayerNumber() );
			sStocksOwned += ": ";
			sStocksOwned += JpString::IntToString( pPlayer->GetStockCount( iHotelId ) );
		}	
		CPrintF( "Client %x,  %s", _hClientId, sStocksOwned.c_str() );
		PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, sStocksOwned.c_str() );



		std::string sMajorityOwners = "";
		std::string sMinorityOwners = "";

		if ( iNumMajorityStockOwners > 0 )
		{
			int iNumCounted = 0;

			PlayerIter.Reset();
			while ( PlayerIter.Next() )
			{
				SoGameBitPlayer* pPlayer = PlayerIter.GetData();
				if ( pPlayer->GetStockCount( iHotelId ) == iMostStockOwned )
				{
					sMajorityOwners += SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( pPlayer->GetPlayerNumber() );

					iNumCounted ++;

					if ( iNumMajorityStockOwners == 1 )
						break;
					else if ( iNumCounted == iNumMajorityStockOwners )
						break;
					else if ( iNumCounted == iNumMajorityStockOwners - 1 )
						sMajorityOwners	+= " and ";
					else if ( iNumCounted < iNumMajorityStockOwners )
						sMajorityOwners	+= ", ";
				}
			}	
		}

		if ( iNumMinorityStockOwners > 0 )
		{
			int iNumCounted = 0;

			PlayerIter.Reset();
			while ( PlayerIter.Next() )
			{
				SoGameBitPlayer* pPlayer = PlayerIter.GetData();
				if ( pPlayer->GetStockCount( iHotelId ) == iSecondMostStockOwned )
				{
					sMinorityOwners += SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( pPlayer->GetPlayerNumber() );

					iNumCounted ++;

					if ( iNumMinorityStockOwners == 1 )
						break;
					else if ( iNumCounted == iNumMinorityStockOwners )
						break;
					else if ( iNumCounted == iNumMinorityStockOwners - 1 )
						sMinorityOwners	+= " and ";
					else if ( iNumCounted < iNumMinorityStockOwners )
						sMinorityOwners	+= ", ";
				}
			}	
		}


		std::string sHotelName = SoActorVisibleHotel::GetHoteNameById( iHotelId );
		char acText[ 128 ] = "";

		if ( iNumMajorityStockOwners == 1 )
		{
			if ( iNumMinorityStockOwners == 0 )
			{
				// If there is only 1 stock owner, that players gets both bonuses.
				sprintf( acText, "%s is sole owner of %s stock, earns both major and minor bonuses totaling $%i", sMajorityOwners.c_str(), sHotelName.c_str(), iFinalMajorityBonus );
				CPrintF( "Client %x,  %s", _hClientId, acText );
				PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, acText );
			}
			else if ( iNumMinorityStockOwners == 1 )
			{
				// If there is only 1 majority stock owner and only 1 minority stock 
				// owner, they each get the bonuses, unaltered.

				sprintf( acText, "%s is the majority %s stock owner, earns majority bonus of $%i", sMajorityOwners.c_str(), sHotelName.c_str(), iFinalMajorityBonus );
				CPrintF( "Client %x,  %s", _hClientId, acText );
				PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, acText );
				
				sprintf( acText, "%s is the minority %s stock owner, earns minority bonus of $%i", sMinorityOwners.c_str(), sHotelName.c_str(), iFinalMinorityBonus );
				CPrintF( "Client %x,  %s", _hClientId, acText );
				PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, acText );
			}
			else if ( iNumMinorityStockOwners > 1 )
			{
				// If there is only 1 majority stock owner, and multiple minority
				// stock owners, then the majority owner gets the full majority bonus
				// and the minority owners split the minority bonus
				sprintf( acText, "%s is the majority %s stock owner, earns majority bonus of $%i", sMajorityOwners.c_str(), sHotelName.c_str(), iFinalMajorityBonus );
				CPrintF( "Client %x,  %s", _hClientId, acText );
				PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, acText );

				sprintf( acText, "%s are tied for minority %s stock owners, each earns minority bonus of $%i", sMinorityOwners.c_str(), sHotelName.c_str(), iFinalMinorityBonus );
				CPrintF( "Client %x,  %s", _hClientId, acText );
				PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, acText );
			}
		}
		else if ( iNumMajorityStockOwners > 1 )
		{
			// If there are multiple majority stock owners, the owners
			// split the combined bonus.  The minority stock owners get nothing.
			sprintf( acText, "%s are tied for majority %s stock owners, each earns majority bonus of $%i", sMajorityOwners.c_str(), sHotelName.c_str(), iFinalMajorityBonus );
			CPrintF( "Client %x,  %s", _hClientId, acText );
			PrintToHudLogF( PLAYER_UNDEFINED_PLAYER_NUMBER, acText );
		}
	}
}



/******************************************************************************
 FUNCTION:  RecalculateHotelSizes
******************************************************************************/
void SoGameState::RecalculateHotelSizes()
{
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( i );

		if ( pHotel->IsPlaced() )
		{
			int iTileId = pHotel->GetPlacementTileId();
			SoGameBitTile* pTile = _TileSet.GetTileById( iTileId );
			int iSize = _TileSet.GetHotelSizeAtTile( pTile->GetTileX(), pTile->GetTileY() );
			pHotel->SetSize( iSize );
		}
		else
		{
			pHotel->SetSize( 0 );
		}
	}
}



/******************************************************************************
 FUNCTION:  DetectAndHandleNoPlayableTilesSituation
******************************************************************************/
void SoGameState::DetectAndHandleNoPlayableTilesSituation( int iPlayerNumber )
{
	struct Local
	{
		//-----------------------
		static int GetNumTilesInPlayersHand( SoGameState* pGame, int iPlayerNumber )
		{
			int iNumTiles = 0;

			SoGameBitTileArray* pAllTiles = pGame->GetTileSet()->GetAllTiles();
			SoGameBitTileArray::Iterator TileIter( pAllTiles );
			while ( TileIter.Next() )
			{
				SoGameBitTile* pTile = TileIter.GetData();
				if ( pTile->GetOwnerPlayerNumber() == iPlayerNumber && !pTile->IsPlaced() )
				{
					iNumTiles ++;
				}
			}

			return iNumTiles;
		}


		//-----------------------
		static void DiscardPlayersHand( SoGameState* pGame, int iPlayerNumber )
		{
			std::string sDiscardedTiles = "";

			SoGameBitTileArray* pAllTiles = pGame->GetTileSet()->GetAllTiles();
			SoGameBitTileArray::Iterator TileIter( pAllTiles );
			while ( TileIter.Next() )
			{
				SoGameBitTile* pTile = TileIter.GetData();
				if ( pTile->GetOwnerPlayerNumber() == iPlayerNumber && !pTile->IsPlaced() )
				{
					pGame->GetTileSet()->DiscardTile( pTile );

					std::string sLabelText = SoActorVisibleTile::GenerateTileLabelTextFromXy( pTile->GetTileX(), pTile->GetTileY() );
					sDiscardedTiles += " ";
					sDiscardedTiles += sLabelText;
				}
			}

			std::string sFinalText = "";
			sFinalText += "discards full hand of unplayable tiles: ";
			sFinalText += sDiscardedTiles;
			pGame->PrintToHudLogF( iPlayerNumber, sFinalText.c_str() );
		}


		//-----------------------
		static void DrawNewHand( SoGameState* pGame, int iPlayerNumber )
		{
			int iNumTilesDrawn = 0;

			for ( int i = 0; i < GAMERULES_NUM_TILES_PER_HAND; i ++ )
			{
				SoGameBitTile* pTile = pGame->GetTileSet()->DrawNextTile();
				if ( pTile )
				{
					pTile->SetOwnerPlayerNumber( iPlayerNumber );
					iNumTilesDrawn ++;
				}
			}

			pGame->PrintToHudLogF( iPlayerNumber, "draws a new hand." );
		}
	};


	//-----------------------
	// If the player's current hand is full of unplacable
	// tiles, discard his hand, and draw a new one.
	// Repeat this if the new hand is also fully unplacable.
	// This is per the 2008 rules for acquire.
	// See: http://en.wikipedia.org/wiki/Acquire#Corner_Cases
	int iNumTilesInHand = Local::GetNumTilesInPlayersHand( this, iPlayerNumber );
	int iNumPlacableTiles = GetNumPlayableTilesForPlayer( iPlayerNumber );
	
	while ( iNumTilesInHand > 0 && iNumPlacableTiles == 0 )
	{
		Local::DiscardPlayersHand( this, iPlayerNumber );
		Local::DrawNewHand( this, iPlayerNumber );

		iNumTilesInHand = Local::GetNumTilesInPlayersHand( this, iPlayerNumber );
		iNumPlacableTiles = GetNumPlayableTilesForPlayer( iPlayerNumber );
	}
}



/******************************************************************************
 FUNCTION:  BuildListOfPotentialMergeSurvivorHotels
******************************************************************************/
void SoGameState::BuildListOfPotentialMergeSurvivorHotels( SoGameBitHotelList* pResultsList, int iTileX, int iTileY )
{
	BuildListOfLargestAdjacentHotelsExcluding( pResultsList, iTileX, iTileY, NULL );
}



/******************************************************************************
 FUNCTION:  BuildListOfPotentialMergeVictimHotels
******************************************************************************/
void SoGameState::BuildListOfPotentialMergeVictimHotels
( 
	SoGameBitHotelList* pResultsList, 
	int iTileX, 
	int iTileY, 
	SoGameBitHotel* pSurvivingHotel
)
{
	BuildListOfLargestAdjacentHotelsExcluding( pResultsList, iTileX, iTileY, pSurvivingHotel );
}



/******************************************************************************
 FUNCTION:  BuildListOfLargestAdjacentHotelsExcluding
******************************************************************************/
void SoGameState::BuildListOfLargestAdjacentHotelsExcluding
( 
	SoGameBitHotelList* pResultsList, 
	int iTileX, 
	int iTileY, 
	SoGameBitHotel* pExcludedHotel 
)
{
	SoGameBitTile* pTileUp =    _TileSet.GetTileByXy( iTileX + 0, iTileY - 1 );
	SoGameBitTile* pTileDown =  _TileSet.GetTileByXy( iTileX + 0, iTileY + 1 );
	SoGameBitTile* pTileLeft =  _TileSet.GetTileByXy( iTileX - 1, iTileY + 0 );
	SoGameBitTile* pTileRight = _TileSet.GetTileByXy( iTileX + 1, iTileY + 0 );

	SoGameBitHotel* pHotelUp = NULL;
	if ( pTileUp && pTileUp->GetHotelId() != HOTEL_UNDEFINED_ID )
		pHotelUp = _HotelSet.GetHotelByHotelId( pTileUp->GetHotelId() );

	SoGameBitHotel* pHotelDown = NULL;
	if ( pTileDown && pTileDown->GetHotelId() != HOTEL_UNDEFINED_ID )
		pHotelDown = _HotelSet.GetHotelByHotelId( pTileDown->GetHotelId() );

	SoGameBitHotel* pHotelLeft = NULL;
	if ( pTileLeft && pTileLeft->GetHotelId() != HOTEL_UNDEFINED_ID )
		pHotelLeft = _HotelSet.GetHotelByHotelId( pTileLeft->GetHotelId() );

	SoGameBitHotel* pHotelRight = NULL;
	if ( pTileRight && pTileRight->GetHotelId() != HOTEL_UNDEFINED_ID )
		pHotelRight = _HotelSet.GetHotelByHotelId( pTileRight->GetHotelId() );


	SoGameBitHotelList HotelList;

	if ( pHotelUp && pHotelUp != pExcludedHotel )
		HotelList.Add( pHotelUp, pHotelUp->GetSize() );

	if ( pHotelDown && pHotelDown != pExcludedHotel )
		HotelList.Add( pHotelDown, pHotelDown->GetSize() );

	if ( pHotelLeft && pHotelLeft != pExcludedHotel )
		HotelList.Add( pHotelLeft, pHotelLeft->GetSize() );

	if ( pHotelRight && pHotelRight != pExcludedHotel )
		HotelList.Add( pHotelRight, pHotelRight->GetSize() );


	SoGameBitHotel* pLargestHotel = HotelList.GetLast()->GetData();
	int iLargestSize = pLargestHotel->GetSize();

	SoGameBitHotelList::Iterator Iter( &HotelList );
	while ( Iter.Next() )
	{
		SoGameBitHotel* pHotel = Iter.GetNodeData();
		if ( pHotel->GetSize() == iLargestSize )
			pResultsList->AddUniquely( pHotel, 0 );
	}
}



/******************************************************************************
 FUNCTION:  GetNumPlayableTilesForPlayer
******************************************************************************/
int SoGameState::GetNumPlayableTilesForPlayer( int iPlayerNumber )
{
	int iNumPlayableTiles = 0;

	SoGameBitTileArray* pAllTiles = _TileSet.GetAllTiles();
	SoGameBitTileArray::Iterator TileIter( pAllTiles );
	while ( TileIter.Next() )
	{
		SoGameBitTile* pTile = TileIter.GetData();
		if ( pTile->GetOwnerPlayerNumber() == iPlayerNumber && !pTile->IsPlaced() )
		{
			bool bTileCreatesIllegalHotel = DoesTileCreateIllegalHotelWhenPlaced( pTile->GetTileX(), pTile->GetTileY() );
			bool bTileMergesSafeHotels = DoesTileMergeSafeHotelsWhenPlaced( pTile->GetTileX(), pTile->GetTileY() );

			bool bIsPlayable = ( !bTileCreatesIllegalHotel && !bTileMergesSafeHotels );

			if ( bIsPlayable )
				iNumPlayableTiles ++;
		}
	}

	return iNumPlayableTiles;
}



/******************************************************************************
 FUNCTION:  AreAnyStocksAvailableForSale
******************************************************************************/
bool SoGameState::AreAnyStocksAvailableForSale()
{
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( i );

		if ( pHotel->IsPlaced() && pHotel->GetAvailableStock() > 0 )
			return true;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  CanDeclareLastTurn
******************************************************************************/
bool SoGameState::CanDeclareLastTurn()
{
	return ( !IsLastTurnDeclared() && ( AreAllActiveHotelsSafe() || IsAnyHotelLargeEnoughToEndGame() ) );
}



/******************************************************************************
 FUNCTION:  AreAllActiveHotelsSafe
******************************************************************************/
bool SoGameState::AreAllActiveHotelsSafe()
{
	bool bAtLeastOneHotelIsActive = false;

	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( i );

		if ( pHotel->IsPlaced() )
		{
			bAtLeastOneHotelIsActive = true;

			if ( !pHotel->IsSafe() )
				return false;
		}
	}

	return bAtLeastOneHotelIsActive;
}



/******************************************************************************
 FUNCTION:  IsAnyHotelLargeEnoughToEndGame
******************************************************************************/
bool SoGameState::IsAnyHotelLargeEnoughToEndGame()
{
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( i );

		if ( pHotel->GetSize() > 40 )
			return true;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  DoesTileMergeSafeHotelsWhenPlaced
******************************************************************************/
bool SoGameState::DoesTileMergeSafeHotelsWhenPlaced( int iTileX, int iTileY )
{
	SoGameBitTile* pUp = _TileSet.GetTileByXy( iTileX, iTileY - 1 );
	SoGameBitTile* pDown = _TileSet.GetTileByXy( iTileX, iTileY + 1 );
	SoGameBitTile* pLeft = _TileSet.GetTileByXy( iTileX - 1, iTileY );
	SoGameBitTile* pRight = _TileSet.GetTileByXy( iTileX + 1, iTileY );


	//-----------------------
	// The logic is a bit strange here.  This is what it's doing:
	//
	// Check the tile in each direction, up, down, left and right,
	// and remember the first one we find that is a safe hotel.
	// 
	// Once we have one safe hotel, then we try to find a second
	// safe hotel, making sure the second find is a different hotel!
	// 
	// As soon as we find such a second, different, hotel, then we 
	// know the given tile will merge two safe hotels.
	//
	// mdomowicz 2013_12_10

	SoGameBitTile* pFirstSafeTile = NULL;

	if ( pUp )
	{
		if ( pUp->IsPlaced() && pUp->GetHotelId() != HOTEL_UNDEFINED_ID )
		{
			SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( pUp->GetHotelId() );
			if ( pHotel->IsSafe() )
				pFirstSafeTile = pUp;
		}
	}

	if ( pDown )
	{
		if ( pDown->IsPlaced() && pDown->GetHotelId() != HOTEL_UNDEFINED_ID )
		{
			SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( pDown->GetHotelId() );
			if ( pHotel->IsSafe() )
			{
				if ( pFirstSafeTile == NULL )
					pFirstSafeTile = pDown;
				else if ( pFirstSafeTile->GetHotelId() != pDown->GetHotelId() )
					return true;
			}
		}
	}

	if ( pLeft )
	{
		if ( pLeft->IsPlaced() && pLeft->GetHotelId() != HOTEL_UNDEFINED_ID )
		{
			SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( pLeft->GetHotelId() );
			if ( pHotel->IsSafe() )
			{
				if ( pFirstSafeTile == NULL )
					pFirstSafeTile = pLeft;
				else if ( pFirstSafeTile->GetHotelId() != pLeft->GetHotelId() )
					return true;
			}
		}
	}


	// If the first safe tile is not set by now, then it doesn't matter
	// what the status of last tile is - it will never cause a bad merger.
	if ( pFirstSafeTile == NULL ) 
		return false;


	if ( pRight )
	{
		if ( pRight->IsPlaced() && pRight->GetHotelId() != HOTEL_UNDEFINED_ID )
		{
			SoGameBitHotel* pHotel = _HotelSet.GetHotelByHotelId( pRight->GetHotelId() );
			if ( pHotel->IsSafe() )
			{
				if ( pFirstSafeTile->GetHotelId() != pRight->GetHotelId() )
					return true;
			}
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  DoesTileCreateIllegalHotelWhenPlaced
******************************************************************************/
bool SoGameState::DoesTileCreateIllegalHotelWhenPlaced( int iTileX, int iTileY )
{
	//-----------------------
	// An illegal hotel is one that would be created
	// by the tile placement, when all possible hotels are
	// already created (placed).  Thus, if any hotels
	// are not yet placed, then the given tile cannot create
	// an illegal hotel.
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		if ( !_HotelSet.GetHotelByHotelId( i )->IsPlaced() )
			return false;
	}

	//-----------------------
	// If we get here, and a new hotel is created, then
	// it is an illegal hotel.
	return _TileSet.DoesTileCreateNewHotelWhenPlaced( iTileX, iTileY );
}



/******************************************************************************
 FUNCTION:  GetGameProgress
******************************************************************************/
SoGameState::GAME_PROGRESS SoGameState::GetGameProgress()
{
	/*
	The following data was gathered from automated self-playthroughs (all Ai players)
	From this (admittedly light) empirical evidence, it seems the number of mergers is
	pretty consistent, and should be good enough for our needs here.  Data is kept
	here in case we want to change the basis of our calculation later.

				Players   Rounds   Tiles   Mergers
					3       22      63        6  
					3       18      52        8
					3       18      52        6
					3       23      65        8
					3       21      61        6

					4       15      58        5
					4       17      64        7
					4       16      58        5
					4       15      59        7
					4       17      62        8

					5       12      57        6
					5       11      51        5
					5       12      58        7
					5       11      50        5
					5       12      58        5

					6       10      56        7
					6       9       50        6
					6       10      55        7
					6       10      56        8
					6       10      54        4
	*/

	if ( _iNumMergersPerformed <= 2 )
		return GAME_PROGRESS_EARLY_GAME;
	else if ( _iNumMergersPerformed <= 4 )
		return GAME_PROGRESS_MID_GAME;

	return GAME_PROGRESS_END_GAME;
}



/******************************************************************************
 FUNCTION:  AddPlayerAi
******************************************************************************/
SoGameBitPlayer* SoGameState::AddPlayerAi()
{
	if ( _PlayerArray.GetNumEntries() < GAMERULES_MAX_PLAYERS )
	{
		SoGameBitPlayerAi* pNewPlayer = new SoGameBitPlayerAi;
		pNewPlayer->Init( SoActorManager::GetInstance()->GetParentNode() );
		pNewPlayer->SetOwningGame( this );
		pNewPlayer->SetPlayerNumber( _PlayerArray.GetNumEntries() );
		_PlayerArray.AddLast( pNewPlayer );
		return pNewPlayer;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  AddPlayerMctsRandom
******************************************************************************/
SoGameBitPlayer* SoGameState::AddPlayerMctsRandom()
{
	if ( _PlayerArray.GetNumEntries() < GAMERULES_MAX_PLAYERS )
	{
		SoGameBitPlayerMctsRandom* pNewPlayer = new SoGameBitPlayerMctsRandom;
		pNewPlayer->Init( SoActorManager::GetInstance()->GetParentNode() );
		pNewPlayer->SetOwningGame( this );
		pNewPlayer->SetPlayerNumber( _PlayerArray.GetNumEntries() );
		_PlayerArray.AddLast( pNewPlayer );
		return pNewPlayer;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  AddPlayerDumb
******************************************************************************/
SoGameBitPlayer* SoGameState::AddPlayerDumb()
{
	if ( _PlayerArray.GetNumEntries() < GAMERULES_MAX_PLAYERS )
	{
		SoGameBitPlayerDumb* pNewPlayer = new SoGameBitPlayerDumb;
		pNewPlayer->Init( SoActorManager::GetInstance()->GetParentNode() );
		pNewPlayer->SetOwningGame( this );
		pNewPlayer->SetPlayerNumber( _PlayerArray.GetNumEntries() );
		_PlayerArray.AddLast( pNewPlayer );
		return pNewPlayer;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  AddPlayerHuman
******************************************************************************/
SoGameBitPlayer* SoGameState::AddPlayerHuman()
{
	if ( _PlayerArray.GetNumEntries() < GAMERULES_MAX_PLAYERS )
	{
		SoGameBitPlayerHuman* pNewPlayer = new SoGameBitPlayerHuman;
		pNewPlayer->Init( SoActorManager::GetInstance()->GetParentNode() );
		pNewPlayer->SetOwningGame( this );
		pNewPlayer->SetPlayerNumber( _PlayerArray.GetNumEntries() );
		_PlayerArray.AddLast( pNewPlayer );
		return pNewPlayer;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  CreateTiles
******************************************************************************/
void SoGameState::CreateTiles()
{
	_TileSet.CreateTiles( _iBoardWidth, _iBoardHeight );

	hash hRngSeed = SoUtils::GenerateDerivedRngSeed( _hMasterRngSeed, "TileSetRng" );
	_TileSet.NewGameShuffle( hRngSeed );
}



/******************************************************************************
 FUNCTION:  SortPlayerArrayByPlayerNumber
******************************************************************************/
void SoGameState::SortPlayerArrayByPlayerNumber()
{
	//-----------------------
	// copy the players to a temp list, using the player
	// number as the sort key.
	SoGameBitPlayerList TempList;
	SoGameBitPlayerArray::Iterator Iter( &_PlayerArray );
	while ( Iter.Next() )
	{
		SoGameBitPlayer* pPlayer = Iter.GetData();
		TempList.Add( pPlayer, pPlayer->GetPlayerNumber() );
	}

	//-----------------------
	// empty the player list
	_PlayerArray.Clear();


	//-----------------------
	// add the players back to the player list, using the
	// player number as the sort key
	SoGameBitPlayerList::Iterator Iter2( &TempList );
	while ( Iter2.Next() )
	{
		SoGameBitPlayer* pPlayer = Iter2.GetNodeData();
		_PlayerArray.AddLast( pPlayer );
	}
}



/******************************************************************************
 FUNCTION:  GetPlayerByPlayerNumber
******************************************************************************/
SoGameBitPlayer* SoGameState::GetPlayerByPlayerNumber( int i )
{
	return _PlayerArray.GetAt( i );
}



/******************************************************************************
 FUNCTION:  GetHotelByHotelId
******************************************************************************/
SoGameBitHotel* SoGameState::GetHotelByHotelId( int iHotelId )
{
	return _HotelSet.GetHotelByHotelId( iHotelId );
}



/******************************************************************************
 FUNCTION:  SeedMasterRng
******************************************************************************/
void SoGameState::SeedMasterRng( hash h ) 
{ 
	_hMasterRngSeed = h; 
	_MasterRng.Seed( h );
}



/******************************************************************************
 FUNCTION:  MakeCopyHelperForMcts
 NOTE:      This does not copy all the data.  Just the SoGame specific stuff
            that derived classes don't have access to.

            This is mainly for convenience, so I don't need to write
            accessor functions for every variable.

            In fact, this is really written specifically for 
            SoGameMcts::MakeCopy().  If you're calling it from someplace
            else, double check that it does everything you want it to do.
******************************************************************************/
void SoGameState::MakeCopyHelperForMcts( SoGameState* pSource )
{
	//-----------------------
	// copy the non-aggregate data
	_iBoardWidth =                   pSource->_iBoardWidth;
	_iBoardHeight =                  pSource->_iBoardHeight;
	_eCurrentState =                 pSource->_eCurrentState;
	_iCurrentPlayerNum =             pSource->_iCurrentPlayerNum;
	_iNumWinners =                   pSource->_iNumWinners;
	_hMasterRngSeed =                pSource->_hMasterRngSeed;
	_iRound =                        pSource->_iRound;
	_iNumMergersPerformed =          pSource->_iNumMergersPerformed;
	_iNumTilesPlaced =               pSource->_iNumTilesPlaced;
	_bIsLastTilePlacementSkipped =   pSource->_bIsLastTilePlacementSkipped;
	_iLastPlacedTileId =             pSource->_iLastPlacedTileId;
	_iLastMergerVictimHotelId =      pSource->_iLastMergerVictimHotelId;
	_iLastMergerSurvivorHotelId =    pSource->_iLastMergerSurvivorHotelId;
	_iLastMergerVictimHotelSize =    pSource->_iLastMergerVictimHotelSize;
	_iPlayerIdThatDeclaredLastTurn = pSource->_iPlayerIdThatDeclaredLastTurn;
	_bIsGameOver =                   pSource->_bIsGameOver;
	_bIsVisibleStocks =              pSource->_bIsVisibleStocks;
	_bIsVisibleMoney =               pSource->_bIsVisibleMoney;
	_bIsLastPlayerMoveAnUndo =       pSource->_bIsLastPlayerMoveAnUndo;

	_iPlayerNumberCommandedToManageOldStock = pSource->	_iPlayerNumberCommandedToManageOldStock;

	_MasterRng.SetHigh( pSource->_MasterRng.GetHigh() );
	_MasterRng.SetLow( pSource->_MasterRng.GetLow() );


	//-----------------------
	// These aren't copied.
	_hClientId = 0;
	_hServerId = 0;
	_pClient = NULL;
	_pServer = NULL;


	//-----------------------
	// copy the aggregate data
	_TileSet.MakeCopy( &pSource->_TileSet );
	_HotelSet.MakeCopy( &pSource->_HotelSet );
}



/******************************************************************************
 FUNCTION:  MakeCopyHelperForUndo
******************************************************************************/
void SoGameState::MakeCopyHelperForUndo( SoGameState* pSource )
{
	MakeCopyHelperForMcts( pSource );
}



/******************************************************************************
 FUNCTION:  RestoreFromUndoState
******************************************************************************/
void SoGameState::RestoreFromUndoState( SoGameState* pSource )
{
	//-----------------------
	// restore the non-aggregate data
	_iBoardWidth =                   pSource->_iBoardWidth;
	_iBoardHeight =                  pSource->_iBoardHeight;
	_eCurrentState =                 pSource->_eCurrentState;
	_iCurrentPlayerNum =             pSource->_iCurrentPlayerNum;
	_iNumWinners =                   pSource->_iNumWinners;
	_hMasterRngSeed =                pSource->_hMasterRngSeed;
	_iRound =                        pSource->_iRound;
	_iNumMergersPerformed =          pSource->_iNumMergersPerformed;
	_iNumTilesPlaced =               pSource->_iNumTilesPlaced;
	_bIsLastTilePlacementSkipped =   pSource->_bIsLastTilePlacementSkipped;
	_iLastPlacedTileId =             pSource->_iLastPlacedTileId;
	_iLastMergerVictimHotelId =      pSource->_iLastMergerVictimHotelId;
	_iLastMergerSurvivorHotelId =    pSource->_iLastMergerSurvivorHotelId;
	_iLastMergerVictimHotelSize =    pSource->_iLastMergerVictimHotelSize;
	_iPlayerIdThatDeclaredLastTurn = pSource->_iPlayerIdThatDeclaredLastTurn;
	_bIsGameOver =                   pSource->_bIsGameOver;
	_bIsVisibleStocks =              pSource->_bIsVisibleStocks;
	_bIsVisibleMoney =               pSource->_bIsVisibleMoney;
	_bIsLastPlayerMoveAnUndo =       pSource->_bIsLastPlayerMoveAnUndo;

	_iPlayerNumberCommandedToManageOldStock = pSource->	_iPlayerNumberCommandedToManageOldStock;

	_MasterRng.SetHigh( pSource->_MasterRng.GetHigh() );
	_MasterRng.SetLow( pSource->_MasterRng.GetLow() );


	//-----------------------
	// These aren't restored.
	// _hClientId = 0;
	// _hServerId = 0;
	// _pClient = NULL;
	// _pServer = NULL;


	//-----------------------
	// Restore the simple aggregate data
	_HotelSet.MakeCopy( &pSource->_HotelSet );



	//-----------------------
	// Restore the complex and list-based aggregate data.
	//
	// Note, for many aggregate classes (e.g. TileSet), we can't call their MakeCopy
	// function directly, but must instead copy each internal object manually.  This is because
	// MakeCopy for these aggregate classes creates new instances of the objects they
	// manage, but in the case of restoring from undo data, we don't want new objects created, 
	// since the objects already exist.
	// mdomowicz 2013_11_25
	//
	// Note, because each of these sections does a repeated linear search (GetById, etc), these 
	// restores are probably much slower than they need to be.  However, doing searches is more 
	// robust, as it does not rely on any guarantee that source and destination have their data 
	// in the same sort order.  As of this writing, the source and dest probably *are* in the
	// same sort order, but I'd rather play it safe for now.
	// mdomowicz 2013_10_12
	int iNumTiles = ( _iBoardWidth * _iBoardHeight );
	for ( int i = 0; i < iNumTiles; i ++ )
	{
		_TileSet.GetTileById( i )->MakeCopy( pSource->_TileSet.GetTileById( i ) );
	}

	SoGameBitPlayerArray::Iterator PlayerIter( &_PlayerArray );
	while ( PlayerIter.Next() )
	{
		SoGameBitPlayer* pPlayer = PlayerIter.GetData();

		SoGameBitPlayer::STATE eState = pPlayer->GetState();
		SoGameBitPlayer::STATE eRequestedState = pPlayer->GetRequestedState();

		pPlayer->MakeCopy( pSource->GetPlayerByPlayerNumber( pPlayer->GetPlayerNumber() ) );

		// The _eState and _eRequestedState of the Human player is manually controlled to	
		// to work in coordination with Undo/ResetTurn, and so we don't want the restored
		// data version of these variables.  So after they get restored, we unrestore them.
		// mdomowicz 2015_12_06
		if ( pPlayer->GetAsPlayerHuman() )
		{
			pPlayer->SetState( eState );
			pPlayer->SetRequestedState( eState );
		}
	}
}



/******************************************************************************
 FUNCTION:  DeleteUndoStack
******************************************************************************/
void SoGameState::DeleteUndoStack()
{
	SoGameStateList::Iterator GameStateIter( &_UndoStack );

	while ( GameStateIter.Next() )
		delete GameStateIter.GetNodeData();

	_UndoStack.RemoveAll();
}



/******************************************************************************
 FUNCTION:  CaptureUndoState
******************************************************************************/
void SoGameState::CaptureUndoState()
{
	SoGameStateUndo* pGameCopy = new SoGameStateUndo;
	pGameCopy->MakeCopy( this );
	_UndoStack.AddLast( pGameCopy );
}



/******************************************************************************
 FUNCTION:  UndoStackHasEnoughDataForUndo
******************************************************************************/
bool SoGameState::UndoStackHasEnoughDataForUndo()
{
	// There must be two states on the undo stack for Undo to work.  See
	// the implementation of undo for an explanation.
	return ( _UndoStack.GetNumEntries() >= 2 );
}



/******************************************************************************
 FUNCTION:  ShouldPresentFeedback
******************************************************************************/
bool SoGameState::ShouldPresentFeedback()
{
	return ( GetAsGameStateMcts() == NULL );
}



/******************************************************************************
 FUNCTION:  PlaySoundEffect
******************************************************************************/
void SoGameState::PlaySoundEffect( hash hSoundEffect )
{
	if ( _hClientId != 0 && ShouldPresentFeedback() )
	{
		if ( !_bIsLoadingGame )
			SoSoundManager::GetInstance()->PlaySoundEffect( hSoundEffect );
	}
}



/******************************************************************************
 FUNCTION:  RenderBoardToDebugAsciiScreen
 NOTES:     This function prints the current board layout to the output window
            and also shows the tiles each player owns.
            This is useful when debugging requires stepping through the code
            and the actual graphically rendered output is not viewable.
******************************************************************************/
void SoGameState::RenderBoardToDebugAsciiScreen()
{
	//-----------------------
	// Setup an ascii screen
	JpAsciiScreen AsciiScreen;
	ASSERTF( _iBoardWidth > 0 && _iBoardHeight > 0, ( "\nBad board dimensions" ) );
	AsciiScreen.Init( _iBoardWidth, _iBoardHeight );
	AsciiScreen.Clear();


	//-----------------------
	// Plot each placed tile
	SoGameBitTileArray* pAllTiles = _TileSet.GetAllTiles();
	SoGameBitTileArray::Iterator TileIter( pAllTiles );
	while ( TileIter.Next() )
	{
		SoGameBitTile* pTile = TileIter.GetData();

		//-----------------------
		// plot numbers to represent tiles owned by players
		// but not yet placed.
		if ( pTile->GetOwnerPlayerNumber() != PLAYER_UNDEFINED_PLAYER_NUMBER && !pTile->IsPlaced() )
		{
			ASSERTF( GAMERULES_MAX_PLAYERS == 6, ( "\nThis array needs to be updated to fit new max players value" ) );

			char acNumbers[ GAMERULES_MAX_PLAYERS ] = { '0', '1', '2', '3', '4', '5' };
			char acNumber = acNumbers[ pTile->GetOwnerPlayerNumber() ];

			AsciiScreen.Plot( pTile->GetTileX(), pTile->GetTileY(), acNumber );
		}

		//-----------------------
		// plot letters to represent existing hotels
		// or plot an 'X' for non-hotel tiles.
		if ( pTile->IsPlaced() )
		{
			if ( pTile->GetHotelId() == HOTEL_UNDEFINED_ID )
			{
				AsciiScreen.Plot( pTile->GetTileX(), pTile->GetTileY(), 'X' );
			}		
			else
			{
				ASSERTF( GAMERULES_NUM_HOTELS == 7, ( "\nThis array needs to be updated to fit new num hotels value" ) );

				char acLetters[ GAMERULES_NUM_HOTELS ] = { 'a', 'b', 'c', 'd', 'e', 'f', 'g' };
				char acLetter = acLetters[ pTile->GetHotelId() ];

				AsciiScreen.Plot( pTile->GetTileX(), pTile->GetTileY(), acLetter );
			}
		}
	}


	//-----------------------
	AsciiScreen.Render();
	AsciiScreen.Deinit();
}



/******************************************************************************
 FUNCTION:  PrintToConsole
******************************************************************************/
void SoGameState::PrintToConsole( const std::string& s )
{
	NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "bLogGameForServerOnly" );
	bool bLogServerOnly = ( pLine && pLine->GetBool( 1 ) );

	bool bLogGame = 
	(
		( bLogServerOnly && _hServerId != 0 ) ||
		!bLogServerOnly
	);


	if ( bLogGame )
	{
		SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();

		std::string sPrefix = "";

		if ( _hServerId != 0 )
		{
			pConsole->SetCurrentTextColor( JpUtils::_Yellow );
			sPrefix = "S: ";
		}
		else if ( _hClientId != 0 )
		{
			pConsole->SetCurrentTextColor( JpUtils::_Red );
			sPrefix = "C: ";
		}


		pConsole->AddOutputString( sPrefix + s );
		pConsole->SetCurrentTextColor( JpUtils::_White );

		CCLOG( s.c_str() );
	}
}



/******************************************************************************
 FUNCTION:  CPrintF
******************************************************************************/
void SoGameState::CPrintF( const char* pFormat, ... )
{
	if ( strlen( pFormat ) < 2048 )
	{
		char buffer[2048];
		va_list args;
		va_start (args, pFormat);
		vsprintf (buffer,pFormat, args);
		va_end (args);

		PrintToConsole( buffer );
	}
}



/******************************************************************************
 FUNCTION:  PrintToHudLog
******************************************************************************/
void SoGameState::PrintToHudLog( int iPlayerNumber, const std::string& s )
{
	SoHudLog* pLog = SoFsmMainLayer::GetSharedHudLog();
	SoGuiAnimWinLog* pWinLog = SoGameGlobals::GetGuiAnimWinLog();

	if ( iPlayerNumber >= 0 )
	{
		std::string sPlayerName = SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( iPlayerNumber );

		char buffer[2048];
		sprintf( buffer, "%s %s", sPlayerName.c_str(), s.c_str() );

		pLog->AddMessageF( SoActorVisiblePlayer::GetColorByPlayerNumber( iPlayerNumber), buffer );
		pWinLog->AddMessageF( SoActorVisiblePlayer::GetColorByPlayerNumber( iPlayerNumber), buffer );
	}
	else
	{
		pLog->AddMessageF( JpUtils::_White, s.c_str() );
		pWinLog->AddMessageF( JpUtils::_White, s.c_str() );
	}
}



/******************************************************************************
 FUNCTION:  PrintToHudLogF
******************************************************************************/
void SoGameState::PrintToHudLogF( int iPlayerNumber, const char* pFormat, ... )
{
	if ( _pClient != 0 )
	{
		if ( strlen( pFormat ) < 2048 )
		{
			char buffer[2048];
			va_list args;
			va_start (args, pFormat);
			vsprintf (buffer,pFormat, args);
			va_end (args);

			PrintToHudLog( iPlayerNumber, buffer );
		}
	}
}
