/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoCardDeck.h"
#include "SoGameBitPlayer.h"
#include "SoHotelSet.h"
#include "SoTileSet.h"


#define SoGameStateList JpList< SoGameState >


USING_NS_CC;

class SoGameClient;
class SoGameMessage;
class SoGameServer;
class SoGameStateMcts;


class SoGameState
{
	public:

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_WAIT_FOR_GAME_START,
			STATE_START_NEW_GAME,

			STATE_INITIALIZE_PLAYER_TURNS,
			STATE_START_PLAYER_TURN,
			STATE_CONTINUE_PLAYER_TURN,

			STATE_PLAYER_PURCHASE_STOCK_INIT,

			STATE_PLAYER_SELECT_NEW_HOTEL_INIT,
			STATE_PLAYER_SELECT_NEW_HOTEL,

			STATE_MERGE_HOTELS_INIT,
			STATE_MERGE_HOTELS,

			STATE_MANAGE_OLD_STOCK_INIT,
			STATE_MANAGE_OLD_STOCK,

			STATE_LAST
		};

		enum GAME_PROGRESS
		{
			GAME_PROGRESS_EARLY_GAME,
			GAME_PROGRESS_MID_GAME,
			GAME_PROGRESS_END_GAME,
			GAME_PROGRESS_LAST
		};


		//-----------------------
		// public functions

		SoGameState();

		virtual ~SoGameState();

		void Init();

		void Deinit();

		void Update( float dt );

		bool UpdateFromServer( SoGameMessage* pInMsg );

		void SetIsLoadingGame( bool b ) { _bIsLoadingGame = b; }

		void HandleMessage( SoGameMessage* pM );

		virtual SoGameStateMcts* GetAsGameStateMcts() { return NULL; }

		SoGameClient* GetOwningClient() { return _pClient; }
		void SetOwningClientId( hash h ) { _hClientId = h; }
		void SetOwningClient( SoGameClient* p ) { _pClient = p; }

		void SetOwningServerId( hash h ) { _hServerId = h; }
		void SetOwningServer( SoGameServer* p ) { _pServer = p; }

		JpRandom* GetMasterRng() { return &_MasterRng; }
		hash GetMasterRngSeed() { return _hMasterRngSeed; }
		void SeedMasterRng( hash h );

		void StartNewGameGame() { _eCurrentState = STATE_START_NEW_GAME; }

		SoGameBitPlayer* AddPlayerAi();
		SoGameBitPlayer* AddPlayerMctsRandom();
		SoGameBitPlayer* AddPlayerDumb();
		SoGameBitPlayer* AddPlayerHuman();

		int GetRound() { return _iRound; }
		int GetNumMergersPerformed() { return _iNumMergersPerformed; }
		int GetNumTilesPlaced() { return _iNumTilesPlaced; }

		SoGameBitPlayer* GetCurrentPlayer() { return GetPlayerByPlayerNumber( _iCurrentPlayerNum ); }
		int GetCurrentPlayerNumber() { return _iCurrentPlayerNum; }

		int GetNumPlayers() { return _PlayerArray.GetNumEntries(); }
		SoGameBitPlayerArray* GetPlayerArray() { return &_PlayerArray; }
		void SortPlayerArrayByPlayerNumber();
		SoGameBitPlayer* GetPlayerByPlayerNumber( int i );

		SoGameBitHotel* GetHotelByHotelId( int iHotelId );

		SoTileSet* GetTileSet() { return &_TileSet; }
		SoHotelSet* GetHotelSet() { return &_HotelSet; }

		bool IsLastTilePlacementSkipped() { return _bIsLastTilePlacementSkipped; }

		int GetLastPlacedTileId() { return _iLastPlacedTileId; }
		SoGameBitTile* GetLastPlacedTile() { return _TileSet.GetTileById( _iLastPlacedTileId ); }

		int GetLastMergerVictimHotelId() { return _iLastMergerVictimHotelId; }
		SoGameBitHotel* GetLastMergerVictimHotel() { return _HotelSet.GetHotelByHotelId( _iLastMergerVictimHotelId ); }

		int GetLastMergerSurvivorHotelId() { return _iLastMergerSurvivorHotelId; }
		SoGameBitHotel* GetLastMergerSurvivorHotel() { return _HotelSet.GetHotelByHotelId( _iLastMergerSurvivorHotelId ); }

		int GetLastMergerVictimHotelSize() { return _iLastMergerVictimHotelSize; }

		int GetNumWinners() { return _iNumWinners; }

		virtual void SendMessageToAllClients( SoGameMessage& M );
		virtual void SendMessageToServer( SoGameMessage& M );

		void BuildListOfPotentialMergeSurvivorHotels( SoGameBitHotelList* pResultsList, int iTileX, int iTileY );
		void BuildListOfPotentialMergeVictimHotels( SoGameBitHotelList* pResultsList, int iTileX, int iTileY, SoGameBitHotel* pSurvivingHotel );
		void BuildListOfLargestAdjacentHotelsExcluding( SoGameBitHotelList* pResultsList, int iTileX, int iTileY, SoGameBitHotel* pExcludedHotel );

		int GetNumPlayableTilesForPlayer( int iPlayerNumber );

		bool AreAnyStocksAvailableForSale();

		bool CanDeclareLastTurn();
		bool IsLastTurnDeclared() { return ( _iPlayerIdThatDeclaredLastTurn != PLAYER_UNDEFINED_PLAYER_NUMBER ); }
		int GetPlayerIdThatDeclaredLastTurn() { return _iPlayerIdThatDeclaredLastTurn; }
		bool IsGameOver() { return _bIsGameOver; }

		bool AreAllActiveHotelsSafe();
		bool IsAnyHotelLargeEnoughToEndGame();

		bool DoesTileMergeSafeHotelsWhenPlaced( int iTileX, int iTileY );
		bool DoesTileCreateIllegalHotelWhenPlaced( int iTileX, int iTileY );

		bool UndoStackHasEnoughDataForUndo();
		bool IsLastPlayerMoveAnUndo() { return _bIsLastPlayerMoveAnUndo; }

		bool ShouldPresentFeedback();

		int GetBoardWidth() { return _iBoardWidth; }
		int GetBoardHeight() { return _iBoardHeight; }

		bool IsVisibleStocks() { return _bIsVisibleStocks; }
		void SetIsVisibleStocks( bool b ) { _bIsVisibleStocks = b; }

		bool IsVisibleMoney() { return _bIsVisibleMoney; }
		void SetIsVisibleMoney( bool b ) { _bIsVisibleMoney = b; }

		int GetPlayerNumberCommandedToManageOldStock() { return _iPlayerNumberCommandedToManageOldStock; }

		void CalculateMajorityMinorityStockOwnersBonusesBasic( int iHotelId, int* paiBonusesResult );

		GAME_PROGRESS GetGameProgress();

		void RenderBoardToDebugAsciiScreen();


	protected:

		//-----------------------
		// protected variables

		STATE _eCurrentState;

		bool _bIsLoadingGame;

		int _iBoardWidth;
		int _iBoardHeight;

		int _iCurrentPlayerNum;
		int _iPreviousPlayerNum;

		SoGameBitPlayerArray _PlayerArray;

		SoTileSet _TileSet;
		SoHotelSet _HotelSet;

		bool _bIsLastTilePlacementSkipped;
		int _iLastPlacedTileId;
		int _iLastMergerVictimHotelId;
		int _iLastMergerSurvivorHotelId;
		int _iLastMergerVictimHotelSize;

		bool _bIsVisibleStocks;
		bool _bIsVisibleMoney;

		int _iPlayerIdThatDeclaredLastTurn;
		bool _bIsGameOver;

		int _iNumWinners;

		hash _hClientId;
		hash _hServerId;

		int _iRound;
		int _iNumMergersPerformed;
		int _iNumTilesPlaced;

		int _iPlayerNumberCommandedToManageOldStock;

		SoGameClient* _pClient;
		SoGameServer* _pServer;

		hash _hMasterRngSeed;
		JpRandom _MasterRng;

		SoGameStateList _UndoStack;
		bool _bIsLastPlayerMoveAnUndo;

		//-----------------------
		// protected functions
		void SetCurrentState( STATE e );

		void CreateTiles();

		virtual void PrintToConsole( const std::string& s );
		virtual void CPrintF( const char* pFormat, ... );

		virtual void PrintToHudLog( int iPlayerNumber, const std::string& s );
		virtual void PrintToHudLogF( int iPlayerNumber, const char* pFormat, ... );


		void HandleInitNewGameMessage( SoGameMessage* pM );
		void HandleEndTurnMessage( SoGameMessage* pM );
		void HandleStockPurchaseMessage( SoGameMessage* pM );
		void HandleNewHotelPickMessage( SoGameMessage* pM );
		void HandleMergeHotelsMessage( SoGameMessage* pM );
		void HandleManageOldStockMessage( SoGameMessage* pM );
		void HandleDeclareLastTurnMessage( SoGameMessage* pM );
		void HandleTilePlacementMessage( SoGameMessage* pM );

		void ProcessEndOfGame();


		void CalculateMajorityMinorityStockOwnersBonuses
		( 
			int iHotelId, 
			int* piMostStockOwnedResult,
			int* piSecondMostStockOwnedResult,
			int* piNumMinorityStockOwnersResult,
			int* piNumMajorityStockOwnersResult,
			int* piFinalMinorityBonusResult,
			int* piFinalMajorityBonusResult,
			int* paiBonusesResult 
		);
		void GiveMajorityMinorityStockOwnersBonuses( int iHotelId );
		void RecalculateHotelSizes();

		void DetectAndHandleNoPlayableTilesSituation( int iPlayerNumber );

		void MakeCopyHelperForMcts( SoGameState* pSource );
		void MakeCopyHelperForUndo( SoGameState* pSource );

		void RestoreFromUndoState( SoGameState* pSource );
		void DeleteUndoStack();
		virtual void CaptureUndoState();

		void PlaySoundEffect( hash hSoundEffect );
};




