/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoGui.h"
#include "SoGuiManager.h"
#include "SoLabelBmFont.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGui
******************************************************************************/
SoGui::SoGui() :
_pParentNode( NULL ),
_pClipNode( NULL ),
_vRelativePos( 0.0f, 0.0f ),
_vAbsolutePos( 0.0f, 0.0f ),
_fWidth( 0.0f ),
_fHeight( 0.0f ),
_fTargetWidth( 0.0f ),
_fTargetHeight( 0.0f ),
_fClipLeftX( 0.0f ),
_fClipBottomY( 0.0f ),
_fClipWidth( 0.0f ),
_fClipHeight( 0.0f ),
_gGuid( 0 ),
_gParentGuid( 0 ),
_pGuiManager( NULL ),
_SpriteColor( JpUtils::_White ),
_TextColor( JpUtils::_White ),
_fSpriteAlpha( 1.0f ),
_fTargetSpriteAlpha( 1.0f ),
_fTextAlpha( 1.0f ),
_fTargetTextAlpha( 1.0f ),
_bListenerListLocked( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoGui
******************************************************************************/
SoGui::~SoGui()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGui::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	_pParentNode = pParentNode;

	_pClipNode = new SoClipNode;
	_pParentNode->addChild( _pClipNode );


	// Not sure if this is strictly necessary, but adding it just in case.
	// mdomowicz 2013_10_03
	SoUtils::SetRenderZOrder( _pParentNode, _pClipNode, BASE_RENDER_ORDER_GUI );


	SetGuiManager( pGuiManager );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGui::Deinit()
{
	SoGui* pParentGui = GetParentGui();
	if ( pParentGui )
	{
		pParentGui->RemoveChildGui( this );
	}

	SoChildGuiGuidList::Iterator Iter( &_ChildGuiGuidList );
	while ( Iter.Next() )
	{
		guid gChild = (guid)Iter.GetNodeData();
		SoGui* pGui = _pGuiManager->GetGuiByGuid( gChild );
		_pGuiManager->RemoveAndDeleteGui( pGui );
	}

	_ChildGuiGuidList.RemoveAll();

	RemoveAndDestroyAllSprites();
	RemoveAndDestroyAllLabels();

	RemoveAllListeners();

	RemoveAndDestroyNode( _pParentNode, _pClipNode );
	_pClipNode = NULL;
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGui::Update( float dt )
{
	
	//-----------------------
	// Update this Gui's absolute pos using the parent's absolute pos.
	// This works because the GuiManager calls Update hierarchically.
	SoGui* pParent = GetParentGui();
	if( !pParent )
		pParent = _pGuiManager->GetHeadGui();

	JpVector2f vParentsAbsolutePos = pParent->GetAbsolutePos();

	_vAbsolutePos = vParentsAbsolutePos + _vRelativePos;

	_pClipNode->SetClippingRegion
	( 
		_vAbsolutePos.X + _fClipLeftX,
		_vAbsolutePos.Y + _fClipBottomY, 
		_fClipWidth, 
		_fClipHeight 
	);
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGui::HandleMessage( SoGuiMessage* pMessage )
{
	SendMessageToAllChildren( pMessage );
}



/******************************************************************************
 FUNCTION:  SetSpriteAlpha
******************************************************************************/
void SoGui::SetSpriteAlpha( float f ) 
{ 
	_fSpriteAlpha = f; 
	_fTargetSpriteAlpha = f;

	SoSpriteList::Iterator Iter( &_SpriteList );
	while ( Iter.Next() )
	{
		SoSprite* pSprite = Iter.GetNodeData();
		pSprite->setOpacity( _fSpriteAlpha * 255.0f );
	}
}



/******************************************************************************
 FUNCTION:  SetSpriteColor
******************************************************************************/
void SoGui::SetSpriteColor( const ccColor3B& Color ) 
{ 
	_SpriteColor = Color; 

	SoSpriteList::Iterator Iter( &_SpriteList );
	while ( Iter.Next() )
	{
		SoSprite* pSprite = Iter.GetNodeData();
		pSprite->setColor( Color );
	}
}



/******************************************************************************
 FUNCTION:  SetTextAlpha
******************************************************************************/
void SoGui::SetTextAlpha( float f ) 
{ 
	_fTextAlpha = f; 
	_fTargetTextAlpha = f;

	SoLabelBmFontList::Iterator Iter2( &_LabelList );
	while ( Iter2.Next() )
	{
		SoLabelBmFont* pLabel = Iter2.GetNodeData();
		pLabel->setOpacity( _fTextAlpha * 255.0f );
	}
}



/******************************************************************************
 FUNCTION:  SetTextColor
******************************************************************************/
void SoGui::SetTextColor( const ccColor3B& Color ) 
{ 
	_TextColor = Color; 

	SoLabelBmFontList::Iterator Iter2( &_LabelList );
	while ( Iter2.Next() )
	{
		SoLabelBmFont* pLabel = Iter2.GetNodeData();
		pLabel->setColor( Color );
	}
}



/******************************************************************************
 FUNCTION:  SetWidth
******************************************************************************/
void SoGui::SetWidth( float f ) 
{ 
	_fWidth = f; 
	_fTargetWidth = f;
}



/******************************************************************************
 FUNCTION:  SetHeight
******************************************************************************/
void SoGui::SetHeight( float f )
{
	_fHeight = f; 
	_fTargetHeight = f;
}



/******************************************************************************
 FUNCTION:  SetWidthHeightFromSprite
******************************************************************************/
void SoGui::SetWidthHeightFromSprite( SoSprite* pSprite )
{
	if ( pSprite )
	{
		// scale the image so that it's stretched to fill the entire window
		SetWidth( pSprite->getContentSize().width * SoUtils::GetiPadRatioX() );
		SetHeight( pSprite->getContentSize().height * SoUtils::GetiPadRatioY() );
	}
}



/******************************************************************************
 FUNCTION:  EnableClipping
******************************************************************************/
void SoGui::EnableClipping( bool b )
{
	ASSERTF( _pClipNode, ( "\nClipnode not set.  You must initialize this gui first." ) );
	if ( _pClipNode )
	{
		_pClipNode->EnableClipping( b );
	}
}



/******************************************************************************
 FUNCTION:  SetClippingRegion
******************************************************************************/
void SoGui::SetClippingRegion( float fLeftX, float fBottomY, float fWidth, float fHeight )
{
	_fClipLeftX = fLeftX;
	_fClipBottomY = fBottomY;
	_fClipWidth = fWidth;
	_fClipHeight = fHeight;


	ASSERTF( _pClipNode, ( "\nClipnode not set.  You must initialize this gui first." ) );
	if ( _pClipNode )
	{
		_pClipNode->SetClippingRegion
		( 
			_vAbsolutePos.X + _fClipLeftX,
			_vAbsolutePos.Y + _fClipBottomY, 
			_fClipWidth, 
			_fClipHeight 
		);
	}
}



/******************************************************************************
 FUNCTION:  SendMessageToParent
******************************************************************************/
void SoGui::SendMessageToParent( SoGuiMessage* pMessage )
{
	_pGuiManager->SendMessageToGui( _gParentGuid, pMessage );
}



/******************************************************************************
 FUNCTION:  SendMessageToSelf
******************************************************************************/
void SoGui::SendMessageToSelf( SoGuiMessage* pMessage )
{
	_pGuiManager->SendMessageToGui( GetGuid(), pMessage );
}



/******************************************************************************
 FUNCTION:  SendMessageToChild
******************************************************************************/
void SoGui::SendMessageToChild( guid gChild, SoGuiMessage* pMessage )
{
	_pGuiManager->SendMessageToGui( gChild, pMessage );
}



/******************************************************************************
 FUNCTION:  SendMessageToAllChildren
******************************************************************************/
void SoGui::SendMessageToAllChildren( SoGuiMessage* pMessage )
{
	SoChildGuiGuidList::Iterator Iter( &_ChildGuiGuidList );
	while ( Iter.Next() )
	{
		guid gChild = (guid)Iter.GetNodeData();
		SendMessageToChild( gChild, pMessage );
	}
}



/******************************************************************************
 FUNCTION:  GetParentGui
******************************************************************************/
void SoGui::SetPartentGui( SoGui* p )
{
	if ( p )
	{
		_gParentGuid = p->GetGuid();
	}
}



/******************************************************************************
 FUNCTION:  GetParentGui
******************************************************************************/
SoGui* SoGui::GetParentGui()
{
	if ( _gParentGuid != 0 )
		return _pGuiManager->GetGuiByGuid( _gParentGuid );

	return NULL;
}



/******************************************************************************
 FUNCTION:  AddChildGui
******************************************************************************/
void SoGui::AddChildGui( SoGui* pGui )
{
	if ( pGui )
	{
		pGui->SetPartentGui( this );
		_ChildGuiGuidList.AddLast( (guid*)pGui->GetGuid() );
	}
}



/******************************************************************************
 FUNCTION:  RemoveChildGui
******************************************************************************/
void SoGui::RemoveChildGui( SoGui* pGui )
{
	if ( pGui )
	{
		pGui->SetPartentGui( NULL );
		_ChildGuiGuidList.Remove( (guid*)pGui->GetGuid() );
	}
}



/******************************************************************************
 FUNCTION:  SendMessageToListeners
******************************************************************************/
void SoGui::SendMessageToListeners( SoGuiMessage* pMessage )
{
	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();
		pListener->OnMessageFromGui( pMessage );
	}

	_bListenerListLocked = false;
}



/******************************************************************************
 FUNCTION:  AddListener
******************************************************************************/
void SoGui::AddListener( Listener* pListener )
{
	ASSERTF( _bListenerListLocked == false, ("Can't add while list is locked.") )
	if ( _bListenerListLocked == false )
		_ListenerList.AddUniquely( pListener, 0 );
}



/******************************************************************************
 FUNCTION:  RemoveListener
******************************************************************************/
void SoGui::RemoveListener( Listener* pListener )
{
	ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
	if ( _bListenerListLocked == false )
		_ListenerList.Remove( pListener );
}



/******************************************************************************
 FUNCTION:  RemoveAllListeners
******************************************************************************/
void SoGui::RemoveAllListeners()
{
	ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
	if ( _bListenerListLocked == false )
		_ListenerList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  CreateSprite
******************************************************************************/
SoSprite* SoGui::CreateSprite( const std::string& sPathAndFilename, int iRenderOrder )
{
	SoSprite* pSprite = NULL;

	pSprite = SoSprite::CreateScaledToDevice( sPathAndFilename.c_str() );

	pSprite->retain();
	pSprite->setPosition( ccp( 0.0f, 0.0f ) );
	_pClipNode->addChild( pSprite );
	pSprite->setVisible( false );

	SoUtils::SetRenderZOrder( _pClipNode, pSprite, iRenderOrder );

	_SpriteList.AddLast( pSprite );

	return pSprite;
}



/******************************************************************************
 FUNCTION:  RemoveAndDestroySprite
******************************************************************************/
void SoGui::RemoveAndDestroySprite( SoSprite* pSprite )
{
	if ( pSprite )
	{
		_SpriteList.Remove( pSprite );
		RemoveAndDestroyNode( _pClipNode, pSprite );
		pSprite = NULL;
	}
}



/******************************************************************************
 FUNCTION:  RemoveAndDestroyAllSprites
******************************************************************************/
void SoGui::RemoveAndDestroyAllSprites()
{
	SoSpriteList::Iterator Iter( &_SpriteList );
	while ( Iter.Next() )
	{
		SoSprite* pSprite = Iter.GetNodeData();
		RemoveAndDestroyNode( _pClipNode, pSprite );		
	}

	_SpriteList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  RemoveAndDestroyLabel
******************************************************************************/
void SoGui::RemoveAndDestroyLabel( SoLabelBmFont* pLabel )
{
	if ( pLabel )
	{
		_LabelList.Remove( pLabel );
		RemoveAndDestroyNode( _pClipNode, pLabel );
		pLabel = NULL;
	}
}



/******************************************************************************
 FUNCTION:  RemoveAndDestroyAllLabels
******************************************************************************/
void SoGui::RemoveAndDestroyAllLabels()
{
	SoLabelBmFontList::Iterator Iter( &_LabelList );
	while ( Iter.Next() )
	{
		SoLabelBmFont* pLabel = Iter.GetNodeData();
		RemoveAndDestroyNode( _pClipNode, pLabel );		
	}

	_LabelList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  CreateBmFontLabel
******************************************************************************/
SoLabelBmFont* SoGui::CreateBmFontLabel( const std::string& sText, const std::string& sFontName, cocos2d::CCTextAlignment Alignment )
{
	CCSize size = CCDirector::sharedDirector()->getWinSize();

	SoLabelBmFont* pLabel = SoLabelBmFont::create
	(
		sText.c_str(), 
		SoUtils::AddRelativePathToFont( sFontName.c_str() ), 
		size.width, 
		Alignment
	);

	CCPoint AnchorPoint = CCPoint( 0, 0 );
	if ( Alignment == kCCTextAlignmentCenter )
		AnchorPoint = CCPoint( 0.5f, 0.5f );

	pLabel->setColor( _TextColor );
	pLabel->setOpacity( 255.0f );
	pLabel->retain();
	pLabel->setAnchorPoint( AnchorPoint );
	pLabel->setScaleX( SoUtils::GetiPadRatioX() );
	pLabel->setScaleY( SoUtils::GetiPadRatioY() );

	_pClipNode->addChild(pLabel, BASE_RENDER_ORDER_GUI + 1 );

	_LabelList.AddLast( pLabel );

	return pLabel;
}



/******************************************************************************
 FUNCTION:  RemoveAndDestroyNode
******************************************************************************/
void SoGui::RemoveAndDestroyNode( CCNode* pParent, CCNode* pNodeToDestroy )
{
	ASSERTF( pParent, ( "\npParent is NULL!" ) );
	if ( pParent && pNodeToDestroy )
	{
		pParent->removeChild( pNodeToDestroy, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		pNodeToDestroy->release();
	}
}



/******************************************************************************
 FUNCTION:  TouchIsInside
******************************************************************************/
bool SoGui::TouchIsInside( CCTouch* pTouch, const JpVector2f& vGuiCenter, float fExtraMarginInPixels )
{
	CCPoint ToutchPos = pTouch->locationInView();
	ToutchPos = CCDirector::sharedDirector()->convertToGL(ToutchPos);

	float fX = vGuiCenter.X;
	float fY = vGuiCenter.Y;

	float fHalfWidth = _fWidth / 2.0f;
	float fHalfHeight = _fHeight / 2.0f;
	fHalfWidth += fExtraMarginInPixels;
	fHalfHeight += fExtraMarginInPixels;

	bool bIsTouchInside = 
	( 
		ToutchPos.x > fX - fHalfWidth && 
		ToutchPos.x < fX + fHalfWidth && 
		ToutchPos.y > fY - fHalfHeight && 
		ToutchPos.y < fY + fHalfHeight
	);

	return bIsTouchInside;
}



/******************************************************************************
 FUNCTION:  ScaleSpriteToSizeOfGui
******************************************************************************/
void SoGui::ScaleSpriteToSizeOfGui( SoSprite* pSprite )
{
	if ( pSprite )
	{
		pSprite->setScaleX( _fWidth / pSprite->getContentSize().width );
		pSprite->setScaleY( _fHeight / pSprite->getContentSize().height );
	}
}



/******************************************************************************
 FUNCTION:  SetPositionToCenterOfParent
******************************************************************************/
void SoGui::SetPositionToCenterOfParent()
{
	SetRelativePos( JpVector2f( 0.0f, 0.0f ) );
}


















