/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"


#include "JpList.h"
#include "JpHashTable.h"

#include "SoClipNode.h"
#include "SoInputGesture.h"
#include "SoInputMouse.h"
#include "SoInputTouch.h"
#include "SoLabelBmFont.h"
#include "SoSprite.h"


#define SoChildGuiGuidList JpList< guid >

USING_NS_CC;



class SoGuiAnimButton;
class SoGuiAnimComboButton;
class SoGuiAnimSlider;
class SoGuiAnimToggle;
class SoGuiManager;
class SoGuiMessage;



class SoGui
{
	public:

		//-----------------------
		// public enum and classes
		class Listener
		{
			public:
				Listener() {}
				virtual ~Listener() {}
				virtual void OnMessageFromGui( SoGuiMessage* pMessage ) {}
		};



		//-----------------------
		// public functions

		SoGui();

		virtual ~SoGui();

		virtual void Init( CCNode* pParentNode, SoGuiManager* pGuiManager );

		virtual void Deinit();

		virtual void Update( float dt );

		virtual void HandleMessage( SoGuiMessage* pMessage );

		virtual SoGuiAnimButton* GetAsAnimButton() { return NULL; }
		virtual SoGuiAnimComboButton* GetAsComboButton() { return NULL; }
		virtual SoGuiAnimSlider* GetAsAnimSlider() { return NULL; }
		virtual SoGuiAnimToggle* GetAsAnimToggle() { return NULL; }

		virtual void SetSpriteAlpha( float f );
		virtual const ccColor3B& GetSpriteColor() { return _SpriteColor; }
		virtual void SetSpriteColor( const ccColor3B& Color );

		virtual void SetTextAlpha( float f );
		virtual const ccColor3B& GetTextColor() { return _TextColor; }
		void SetTextColor( const ccColor3B& Color );

		void SetRelativePos( const JpVector2f& v ) { _vRelativePos = v; }
		JpVector2f GetRelativePos() { return _vRelativePos; }

		void SetAbsolutePos( const JpVector2f& v ) { _vAbsolutePos = v; }
		JpVector2f GetAbsolutePos() { return _vAbsolutePos; }

		float GetWidth() { return _fWidth; }
		void SetWidth( float f );
		float GetHeight() { return _fHeight; }
		void SetHeight( float f );
		void SetWidthHeightFromSprite( SoSprite* pSprite );

		void EnableClipping( bool b );
		void SetClippingRegion( float fLeftX, float fBottomY, float fWidth, float fHeight );


		void SendMessageToParent( SoGuiMessage* pMessage );
		void SendMessageToSelf( SoGuiMessage* pMessage );

		void SendMessageToChild( guid gChild, SoGuiMessage* pMessage );
		void SendMessageToAllChildren( SoGuiMessage* pMessage );

		void SetGuid( guid g ) { _gGuid = g; }
		guid GetGuid() { return _gGuid; }

		SoGui* GetParentGui();
		void SetPartentGui( SoGui* p );

		void AddChildGui( SoGui* pGui );
		void RemoveChildGui( SoGui* pGui );

		SoChildGuiGuidList* GetChildGuiGuidList() { return &_ChildGuiGuidList; }

		void SetGuiManager( SoGuiManager* p ) { _pGuiManager = p; }
		SoGuiManager* GetGuiManager() { return _pGuiManager; }


		// For communicating with classes outside of the gui system
		void SendMessageToListeners( SoGuiMessage* pMessage );
		void AddListener( Listener* pListener );
		void RemoveListener( Listener* pListener );
		void RemoveAllListeners();

		// SoInputTouch callbacks
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) { return false; }
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList ) { return false; }
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) { return false; }

		// SoInputGesture callbacks
		virtual bool OnGestureDown( SoGesture* pGesture ) { return false; }
		virtual bool OnGestureUp( SoGesture* pGesture )   { return false; }
		virtual bool OnGestureMove( SoGesture* pGesture ) { return false; }

		// SoInputKeypad callbacks
		virtual bool OnKeypadBackClicked() { return false; }
		virtual bool OnKeypadMenuClicked() { return false; }

		// SoInputMouse callbacks
		#if DCG_IS_TARGET_WIN32
			virtual bool OnMousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) { return false; }
			virtual bool OnMouseReleased( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) { return false; }
			virtual bool OnMouseMoved( const OIS::MouseEvent &arg ) { return false; }
		#endif

	protected:

		//-----------------------
		// protected variables
		SoClipNode* _pClipNode;

		JpVector2f _vRelativePos;
		JpVector2f _vAbsolutePos;

		float _fWidth;
		float _fHeight;
		float _fTargetWidth;
		float _fTargetHeight;

		float _fClipLeftX;
		float _fClipBottomY;
		float _fClipWidth;
		float _fClipHeight;

		guid _gGuid;
		guid _gParentGuid;

		SoGuiManager* _pGuiManager;

		SoChildGuiGuidList _ChildGuiGuidList;

		float _fSpriteAlpha;
		float _fTargetSpriteAlpha;
		ccColor3B _SpriteColor;

		float _fTextAlpha;
		float _fTargetTextAlpha;
		ccColor3B _TextColor;

		SoSpriteList _SpriteList;
		SoLabelBmFontList _LabelList;

		JpList< Listener > _ListenerList;
		bool _bListenerListLocked;


		//-----------------------
		// protected functions
		SoSprite* CreateSprite( const std::string& sPathAndFilename, int iRenderOrder );
		void RemoveAndDestroySprite( SoSprite* pSprite );
		void RemoveAndDestroyAllSprites();

		SoLabelBmFont* CreateBmFontLabel( const std::string& sText, const std::string& sFontName, cocos2d::CCTextAlignment Alignment = kCCTextAlignmentLeft );
		void RemoveAndDestroyLabel( SoLabelBmFont* pLabel );
		void RemoveAndDestroyAllLabels();

		void RemoveAndDestroyNode( CCNode* pParent, CCNode* pNodeToDestroy );

		bool TouchIsInside( CCTouch* pTouch, const JpVector2f& vGuiCenter, float fExtraMarginInPixels );

		void ScaleSpriteToSizeOfGui( SoSprite* pSprite );

		void SetPositionToCenterOfParent();


	private:

		//-----------------------
		// private variables

		// Derived classes should not use _pParentNode directly.
		// Use _pClipNode instead.  mdomowicz 2013_10_03
		CCNode* _pParentNode;
};




template< class TYPE >
class SoGuiListener : public SoGui::Listener
{
	public:

		SoGuiListener() : _pOwner( NULL ) {}

		~SoGuiListener() {}

		void SetOwner( TYPE* p ) { _pOwner = p; }

		virtual void OnMessageFromGui( SoGuiMessage* pMessage ) { _pOwner->OnMessageFromGui( pMessage ); }

	private:

		TYPE* _pOwner;

		SoGuiListener( SoGuiListener& Copy ) { ASSERTF( 0, ( "\nCannot copy instances of this class" ) ); }
};
