/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoGui.h"

USING_NS_CC;


class SoGuiAnim : public SoGui
{
	public:

		//-----------------------
		// public enums
		enum TRANSITION_STYLE
		{
			TRANSITION_STYLE_FADE_IN_OUT,
			TRANSITION_STYLE_POP_UP,
			TRANSITION_STYLE_NONE,
			TRANSITION_STYLE_TYPE_LAST
		};

		enum TRANSITION_STATE
		{
			TRANSITION_STATE_INIT,
			TRANSITION_STATE_ANIM_IN,
			TRANSITION_STATE_SHOW,
			TRANSITION_STATE_ANIM_OUT,
			TRANSITION_STATE_HIDE,
			TRANSITION_STATE_LAST	
		};

		//-----------------------
		// public functions

		SoGuiAnim();

		virtual ~SoGuiAnim();

		virtual void Init( CCNode* pParentNode, SoGuiManager* pGuiManager );

		virtual void Deinit();

		virtual void Update( float dt );

		virtual void HandleMessage( SoGuiMessage* pMessage );

		SoSprite* GetBackgroundSprite() { return _pSpriteBackground; }
		void CreateBackgroundSprite( const std::string& sPathAndFilename );

		void SetTransitionStyle( TRANSITION_STYLE e ) { _eTransitionStyle = e; }

		void SetTransitionSpeed( float f ) { _fTransitionSpeed = f; }

		void SetDeleteOnTransitionOut( bool b ) { _bDeleteOnTransitionOut = b; }

		bool IsHiding() { return ( _eTransitionState == TRANSITION_STATE_HIDE ); }
		bool IsShowing() { return ( _eTransitionState == TRANSITION_STATE_SHOW ); }

	protected:

		//-----------------------
		// protected variables
		SoSprite* _pSpriteBackground;

		TRANSITION_STYLE _eTransitionStyle;
		TRANSITION_STATE _eTransitionState;

		float _fTransitionTimer;
		float _fTransitionSpeed;

		bool _bDeleteOnTransitionOut;

		//-----------------------
		// private functions
		void TransitionIn();
		void TransitionOut();

		void UpdateStyleFadeInOut( float dt );
		void UpdateStylePopup( float dt );
		void UpdateStyleNone( float dt );

		void SetSpriteAlphaToPercentOfTarget( float fPercent );
		void SetTextAlphaToPercentOfTarget( float fPercent );
};




