/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCamera.h"
#include "SoGuiAnimButton.h"
#include "SoGuiMessage.h"
#include "SoSoundManager.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoGuiAnimButton
******************************************************************************/
SoGuiAnimButton::SoGuiAnimButton() :
_pSpriteDown( NULL ),
_pSpriteUp( NULL ),
_pSpriteDisabled( NULL ),
_eState( STATE_UNPRESSED ),
_eStateToSwitchTo( STATE_UNPRESSED ),
_hSoundEffectOnButtonUp( 0 ),
_hSoundEffectOnButtonDown( 0 ),
_eButtonDownImageOption( BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE ),
_eButtonDisabledImageOption( BUTTON_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY ),
_bUseWorldSpacePositioning( false ),
_vWorldPosition( 0.0f, 0.0f ),
_iUserIntValue( 0 ),
_bIsAttractAnimEnabled( false ),
_fAttractAnimTimer( 0.0f )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimButton
******************************************************************************/
SoGuiAnimButton::~SoGuiAnimButton()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimButton::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnim::Init( pParentNode, pGuiManager );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimButton::Deinit()
{
	SoGuiAnim::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimButton::Update( float dt )
{
	SoGuiAnim::Update( dt );


	switch ( _eState )
	{
		case STATE_UNPRESSED:
		{
			if ( _eButtonDownImageOption == BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN )
			{
				if ( _pSpriteUp )
				{
					_pSpriteUp->setVisible( true );
					_pSpriteUp->setOpacity( _fSpriteAlpha * 255.0f );

					float fScaleX = ( _fWidth / _pSpriteUp->getContentSize().width );
					float fScaleY = ( _fHeight / _pSpriteUp->getContentSize().height );

					_pSpriteUp->setScaleX( fScaleX );
					_pSpriteUp->setScaleY( fScaleY );
				}
			}
			else if ( _eButtonDownImageOption == BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE )
			{
				if ( _pSpriteUp )
					_pSpriteUp->setVisible( true );

				if ( _pSpriteDown )
					_pSpriteDown->setVisible( false );
			}

			if ( _pSpriteDisabled )
				_pSpriteDisabled->setVisible( false );

			if ( _eStateToSwitchTo != _eState )
			{
				_eState = _eStateToSwitchTo;
			}

			break;
		}

		case STATE_PRESSED:
		{
			if ( _eButtonDownImageOption == BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN )
			{
				if ( _pSpriteUp )
				{
					float fScaleX = ( _fWidth / _pSpriteUp->getContentSize().width );
					float fScaleY = ( _fHeight / _pSpriteUp->getContentSize().height );

					_pSpriteUp->setScaleX( fScaleX * 0.9f );
					_pSpriteUp->setScaleY( fScaleY * 0.9f );
				}
			}
			else if ( _eButtonDownImageOption == BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE )
			{
				if ( _pSpriteUp )
					_pSpriteUp->setVisible( false );

				if ( _pSpriteDown )
					_pSpriteDown->setVisible( true );
			}

			if ( _pSpriteDisabled )
				_pSpriteDisabled->setVisible( false );

			if ( _eStateToSwitchTo != _eState )
			{
				_eState = _eStateToSwitchTo;
			}
			break;
		}

		case STATE_DISABLED:
		{
			if ( _pSpriteUp )
				_pSpriteUp->setVisible( false );

			if ( _pSpriteDown )
				_pSpriteDown->setVisible( false );

			if ( _eButtonDisabledImageOption == BUTTON_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY )
			{
				if ( _pSpriteUp )
				{
					_pSpriteUp->setVisible( true );
					_pSpriteUp->setOpacity( 0.25f * _fSpriteAlpha * 255.0f );
				}
			}
			else if ( _eButtonDisabledImageOption == BUTTON_DISABLED_IMAGE_OPTION_USE_IMAGE )
			{
				if ( _pSpriteDisabled )
					_pSpriteDisabled->setVisible( true );
			}
			else if ( _eButtonDisabledImageOption == BUTTON_DISABLED_IMAGE_OPTION_HIDE )
			{
				if ( _pSpriteDisabled )
					_pSpriteDisabled->setVisible( false );
			}
			break;
		}

		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknown button state." ) );
	}



	JpVector2f vFinal = _vAbsolutePos;


	if ( _bUseWorldSpacePositioning )
	{
		vFinal = GetScreenPositionFromWorldSpace( _vWorldPosition );
	}

	if ( _pSpriteDown )
	{
		_pSpriteDown->SetPosition( vFinal );
	}

	if ( _pSpriteUp )
	{
		_pSpriteUp->SetPosition( vFinal );
	}

	if ( _pSpriteDisabled )
	{
		_pSpriteDisabled->SetPosition( vFinal );
	}

	if ( _bIsAttractAnimEnabled )
	{
		if ( _pSpriteUp )
		{
			_fAttractAnimTimer += dt;
			float fScaleDelta = 0.1f * SoUtils::GetiPadRatio() * fabsf( sinf( _fAttractAnimTimer * 3.0f ) ); 
			_pSpriteUp->setScaleX( _pSpriteUp->getScaleX() + fScaleDelta );
			_pSpriteUp->setScaleY( _pSpriteUp->getScaleY() + fScaleDelta );
		}
	}
	else
	{
		_fAttractAnimTimer = 0.0f;
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimButton::HandleMessage( SoGuiMessage* pMessage )
{
	SoGuiAnim::HandleMessage( pMessage );
}




/******************************************************************************
 FUNCTION:  Setup
******************************************************************************/
void SoGuiAnimButton::Setup( const SetupParams& Params )
{
	if ( Params._sPathAndFilenameButtonUp != "" )
		CreateSpriteUp( Params._sPathAndFilenameButtonUp );

	if ( Params._sPathAndFilenameButtonDown != "" )
		CreateSpriteDown( Params._sPathAndFilenameButtonDown );

	if ( Params._sPathAndFilenameButtonDisabled != "" )
		CreateSpriteDisabled( Params._sPathAndFilenameButtonDisabled );

	SetRelativePos( Params._vRelativePos );

	_hSoundEffectOnButtonUp = Params._hSoundEffectOnButtonUp;
	_hSoundEffectOnButtonDown = Params._hSoundEffectOnButtonDown;

	_eButtonDownImageOption = Params._eButtonDownImageOption;
	_eButtonDisabledImageOption = Params._eButtonDisabledImageOption;
}



/******************************************************************************
 FUNCTION:  IsUsable
******************************************************************************/
bool SoGuiAnimButton::IsUsable()
{
	return
	(
		//IsActive() &&
		_eState != STATE_DISABLED &&
		_eTransitionState != TRANSITION_STATE_INIT &&
		_eTransitionState != TRANSITION_STATE_HIDE &&
		_eTransitionState != TRANSITION_STATE_ANIM_IN &&
		_eTransitionState != TRANSITION_STATE_ANIM_OUT &&
		_eTransitionState != TRANSITION_STATE_LAST
	);
}



/******************************************************************************
 FUNCTION:  Enable
******************************************************************************/
void SoGuiAnimButton::Enable( bool b )
{ 
	if ( b )
	{
		if ( _eState == STATE_DISABLED )
			_eState = STATE_UNPRESSED;
	}
	else
		_eState = STATE_DISABLED;
}



/******************************************************************************
 FUNCTION:  CreateSpriteUp
******************************************************************************/
void SoGuiAnimButton::CreateSpriteUp( const std::string& sPathAndFilename ) 
{ 
	_pSpriteUp = CreateSprite( sPathAndFilename, BASE_RENDER_ORDER_GUI_BUTTON ); 
	SetWidthHeightFromSprite( _pSpriteUp );  // temp!  we should find a better way to do this.  mdomowicz 2013_07_06
	_pSpriteBackground = _pSpriteUp;
}



/******************************************************************************
 FUNCTION:  CreateSpriteDown
******************************************************************************/
void SoGuiAnimButton::CreateSpriteDown( const std::string& sPathAndFilename )
{
	_pSpriteDown = CreateSprite( sPathAndFilename, BASE_RENDER_ORDER_GUI_BUTTON ); 
}



/******************************************************************************
 FUNCTION:  CreateSpriteDisabled
******************************************************************************/
void SoGuiAnimButton::CreateSpriteDisabled( const std::string& sPathAndFilename )
{
	_pSpriteDisabled = CreateSprite( sPathAndFilename, BASE_RENDER_ORDER_GUI_BUTTON ); 
}



/******************************************************************************
 FUNCTION:  OnTouchesDown
******************************************************************************/
bool SoGuiAnimButton::OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) 
{
	float fMargin = 0.0f;

	if ( IsUp() && IsUsable() )
	{
		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		SoCcTouchList::Iterator Iter( pNonConstTouchList );
		while ( Iter.Next() )
		{
			CCTouch* pTouch = Iter.GetNodeData();

			JpVector2f vFinalPos = _vAbsolutePos;
			if ( _bUseWorldSpacePositioning )
				vFinalPos = GetScreenPositionFromWorldSpace( _vWorldPosition );			

			if ( TouchIsInside( pTouch, vFinalPos, fMargin ) )
			{
				SetDown();
				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_BUTTON_ON_DOWN, GetGuid() );
				SendMessageToParent( &GuiMessage );

				SoSoundManager::GetInstance()->PlaySoundEffect( _hSoundEffectOnButtonDown );

				return true;
			}
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoGuiAnimButton::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{
	float fMargin = 0.0f;

	if ( IsDown() && IsUsable() )
	{
		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		SoCcTouchList::Iterator Iter( pNonConstTouchList );
		while ( Iter.Next() )
		{
			CCTouch* pTouch = Iter.GetNodeData();

			SetUp();

			JpVector2f vFinalPos = _vAbsolutePos;
			if ( _bUseWorldSpacePositioning )
				vFinalPos = GetScreenPositionFromWorldSpace( _vWorldPosition );			

			if ( TouchIsInside( pTouch, vFinalPos, fMargin ) )
			{
				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_BUTTON_ON_UP, GetGuid() );
				SendMessageToParent( &GuiMessage );

				SoSoundManager::GetInstance()->PlaySoundEffect( _hSoundEffectOnButtonUp );

				return true;
			}
			else
			{
				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE, GetGuid() );
				SendMessageToParent( &GuiMessage );

				return true;
			}
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesMove
******************************************************************************/
bool SoGuiAnimButton::OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) 
{
	return false;
}



/******************************************************************************
 FUNCTION:  GetScreenPositionFromWorldSpace
******************************************************************************/
JpVector2f SoGuiAnimButton::GetScreenPositionFromWorldSpace( JpVector2f vWorldPosition )
{
	JpVector2f vScreenPos( 0.0f, 0.0f );

	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	if ( pCameraManager )
	{
		SoCamera* pCamera = pCameraManager->GetCurrentCamera();

		if ( pCamera )
			vScreenPos = pCamera->TransformWorldToScreenSpace( vWorldPosition );
	}

	return vScreenPos;
}
