/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoGuiAnim.h"

USING_NS_CC;


#define SoGuiAnimButtonList JpList< SoGuiAnimButton >


class SoGuiAnimButton : public SoGuiAnim
{
	public:

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_UNPRESSED,
			STATE_PRESSED,
			STATE_DISABLED,
			STATE_LAST
		};

		enum BUTTON_DOWN_IMAGE_OPTION
		{
			BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN,
			BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE,
			BUTTON_DOWN_IMAGE_OPTION_NO_EFFECT,
			BUTTON_DOWN_IMAGE_OPTION_LAST
		};

		enum BUTTON_DISABLED_IMAGE_OPTION
		{
			BUTTON_DISABLED_IMAGE_OPTION_HIDE,
			BUTTON_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY,
			BUTTON_DISABLED_IMAGE_OPTION_USE_IMAGE,
			BUTTON_DISABLED_IMAGE_OPTION_LAST
		};


		//-----------------------
		// public classes
		class SetupParams
		{
			public:
				SetupParams() :
				_vRelativePos( 0.0f, 0.0f ),
				_hSoundEffectOnButtonUp( 0 ),
				_hSoundEffectOnButtonDown( 0 ),
				_eButtonDownImageOption( BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE ),
				_eButtonDisabledImageOption( BUTTON_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY )
				{};

				virtual ~SetupParams(){};
				std::string _sPathAndFilenameButtonUp;
				std::string _sPathAndFilenameButtonDown;
				std::string _sPathAndFilenameButtonDisabled;
				JpVector2f _vRelativePos;
				hash _hSoundEffectOnButtonUp;
				hash _hSoundEffectOnButtonDown;
				BUTTON_DOWN_IMAGE_OPTION _eButtonDownImageOption;
				BUTTON_DISABLED_IMAGE_OPTION _eButtonDisabledImageOption;
		};


		//-----------------------
		// public functions

		SoGuiAnimButton();

		virtual ~SoGuiAnimButton();

		virtual void Init( CCNode* pParentNode, SoGuiManager* pGuiManager );

		virtual void Deinit();

		virtual void Update( float dt );

		virtual void HandleMessage( SoGuiMessage* pMessage );

		virtual SoGuiAnimButton* GetAsAnimButton() { return this; }

		void Setup( const SetupParams& Params );


		bool IsUsable();
		bool IsEnabled() { return ( _eState != STATE_DISABLED ); }
		void Enable( bool b );

		bool IsDown() { return _eState == STATE_PRESSED; }
		void SetDown() { _eStateToSwitchTo = STATE_PRESSED; }

		bool IsUp() { return _eState == STATE_UNPRESSED; }
		void SetUp() { _eStateToSwitchTo = STATE_UNPRESSED; }

		bool IsAttractAnimEnabled() { return _bIsAttractAnimEnabled; }
		void SetIsAttractAnimEnabled( bool b ) { _bIsAttractAnimEnabled = b; }

		void SetUseWorldSpacePositioning( bool b ) { _bUseWorldSpacePositioning = b; }
		void SetWorldPosition( const JpVector2f& v ) { _vWorldPosition = v; }

		void CreateSpriteUp( const std::string& sPathAndFilename ); 
		void CreateSpriteDown( const std::string& sPathAndFilename );
		void CreateSpriteDisabled( const std::string& sPathAndFilename );

		hash GetSoundEffectOnButtonUp() { return _hSoundEffectOnButtonUp; }
		hash GetSoundEffectOnButtonDown() { return _hSoundEffectOnButtonDown; }

		// This is a "user data" variable that can be used whenever the user
		// needs to identify a button in a specific way or pass data.
		int GetUserIntValue() { return _iUserIntValue; }
		void SetUserIntValue( int i ) { _iUserIntValue = i; }

		// SoInputTouch callbacks
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList );


	protected:

		//-----------------------
		// protected variables

		SoSprite* _pSpriteDown;
		SoSprite* _pSpriteUp;
		SoSprite* _pSpriteDisabled;

		hash _hSoundEffectOnButtonUp;
		hash _hSoundEffectOnButtonDown;


		BUTTON_DOWN_IMAGE_OPTION _eButtonDownImageOption;
		BUTTON_DISABLED_IMAGE_OPTION _eButtonDisabledImageOption;

		STATE _eState;
		STATE _eStateToSwitchTo;


		bool _bUseWorldSpacePositioning;
		JpVector2f _vWorldPosition;

		int _iUserIntValue;


		bool _bIsAttractAnimEnabled;
		float _fAttractAnimTimer;

		//-----------------------
		// protected functions
		JpVector2f GetScreenPositionFromWorldSpace( JpVector2f vWorldPosition );
};




