/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCamera.h"
#include "SoGuiAnimComboButton.h"
#include "SoGuiMessage.h"
#include "SoSoundManager.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoGuiAnimComboButton
******************************************************************************/
SoGuiAnimComboButton::SoGuiAnimComboButton() :
_eState( STATE_UNPRESSED ),
_eStateToSwitchTo( STATE_UNPRESSED ),
_hSoundEffectOnButtonUp( 0 ),
_hSoundEffectOnButtonDown( 0 ),
_eButtonDownImageOption( BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE ),
_bUseWorldSpacePositioning( false ),
_vWorldPosition( 0.0f, 0.0f ),
_pCurrentComboValue( NULL ),
_pPreviousComboValue( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimComboButton
******************************************************************************/
SoGuiAnimComboButton::~SoGuiAnimComboButton()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimComboButton::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnim::Init( pParentNode, pGuiManager );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimComboButton::Deinit()
{
	SoGuiAnim::Deinit();


	SoComboButtonValueList::Iterator Iter( &_ComboValueList );
	while ( Iter.Next() )
	{
		delete Iter.GetNodeData();
	}
	_ComboValueList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimComboButton::Update( float dt )
{
	SoGuiAnim::Update( dt );

	SoSprite* pSpriteUp = NULL;
	SoSprite* pSpriteDown = NULL;
	SoSprite* pSpriteDisabled = NULL;


	if ( _pCurrentComboValue )
	{
		pSpriteUp = _pCurrentComboValue->GetSpriteUp();
		pSpriteDown = _pCurrentComboValue->GetSpriteDown();
		pSpriteDisabled = _pCurrentComboValue->GetSpriteDisabled();
		_pSpriteBackground = pSpriteUp;
	}
	

	switch ( _eState )
	{
		case STATE_UNPRESSED:
		{
			if ( _eButtonDownImageOption == BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN )
			{
				if ( pSpriteUp )
				{
					pSpriteUp->setVisible( true );
					pSpriteUp->setOpacity( _fSpriteAlpha * 255.0f );

					float fScaleX = ( _fWidth / pSpriteUp->getContentSize().width );
					float fScaleY = ( _fHeight / pSpriteUp->getContentSize().height );

					pSpriteUp->setScaleX( fScaleX );
					pSpriteUp->setScaleY( fScaleY );
				}
			}
			else if ( _eButtonDownImageOption == BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE )
			{
				if ( pSpriteUp )
					pSpriteUp->setVisible( true );

				if ( pSpriteDown )
					pSpriteDown->setVisible( false );
			}

			if ( pSpriteDisabled )
				pSpriteDisabled->setVisible( false );

			if ( _eStateToSwitchTo != _eState )
			{
				_eState = _eStateToSwitchTo;
			}

			break;
		}

		case STATE_PRESSED:
		{
			if ( _eButtonDownImageOption == BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN )
			{
				if ( pSpriteUp )
				{
					float fScaleX = ( _fWidth / pSpriteUp->getContentSize().width );
					float fScaleY = ( _fHeight / pSpriteUp->getContentSize().height );

					pSpriteUp->setScaleX( fScaleX * 0.9f );
					pSpriteUp->setScaleY( fScaleY * 0.9f );
				}
			}
			else if ( _eButtonDownImageOption == BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE )
			{
				if ( pSpriteUp )
					pSpriteUp->setVisible( false );

				if ( pSpriteDown )
					pSpriteDown->setVisible( true );
			}

			if ( pSpriteDisabled )
				pSpriteDisabled->setVisible( false );

			if ( _eStateToSwitchTo != _eState )
			{
				_eState = _eStateToSwitchTo;
			}
			break;
		}

		case STATE_DISABLED:
		{
			if ( pSpriteUp )
				pSpriteUp->setVisible( false );

			if ( pSpriteDown )
				pSpriteDown->setVisible( false );

			if ( _eButtonDisabledImageOption == BUTTON_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY )
			{
				if ( pSpriteUp )
				{
					pSpriteUp->setVisible( true );
					pSpriteUp->setOpacity( 0.5f * _fSpriteAlpha * 255.0f );
				}
			}
			else if ( _eButtonDisabledImageOption == BUTTON_DISABLED_IMAGE_OPTION_USE_IMAGE )
			{
				if ( pSpriteDisabled )
					pSpriteDisabled->setVisible( true );
			}
			else if ( _eButtonDisabledImageOption == BUTTON_DISABLED_IMAGE_OPTION_HIDE )
			{
				if ( pSpriteDisabled )
					pSpriteDisabled->setVisible( false );
			}
		
			break;
		}

		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknown button state." ) );
	}



	JpVector2f vFinal = _vAbsolutePos;


	if ( _bUseWorldSpacePositioning )
	{
		vFinal = GetScreenPositionFromWorldSpace( _vWorldPosition );
	}

	if ( pSpriteDown )
	{
		pSpriteDown->SetPosition( vFinal );
	}

	if ( pSpriteUp )
	{
		pSpriteUp->SetPosition( vFinal );
	}

	if ( pSpriteDisabled )
	{
		pSpriteDisabled->SetPosition( vFinal );
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimComboButton::HandleMessage( SoGuiMessage* pMessage )
{
	SoGuiAnim::HandleMessage( pMessage );
}




/******************************************************************************
 FUNCTION:  Setup
******************************************************************************/
void SoGuiAnimComboButton::Setup( const SetupParams& Params )
{
	SetRelativePos( Params._vRelativePos );

	_hSoundEffectOnButtonUp = Params._hSoundEffectOnButtonUp;
	_hSoundEffectOnButtonDown = Params._hSoundEffectOnButtonDown;

	_eButtonDownImageOption = Params._eButtonDownImageOption;
	_eButtonDisabledImageOption = Params._eButtonDisabledImageOption;
}



/******************************************************************************
 FUNCTION:  SetupAndAddComboButtonValue
******************************************************************************/
void SoGuiAnimComboButton::SetupAndAddComboButtonValue
( 				
	hash hValue,
	const std::string& sPathAndFilenameButtonUp,
	const std::string& sPathAndFilenameButtonDown,
	const std::string& sPathAndFilenameButtonDisabled
)
{
	SoComboButtonValue* pNewValue = new SoComboButtonValue;

	pNewValue->SetValue( hValue );

	if ( sPathAndFilenameButtonUp != "" )
	{
		SoSprite* pSpriteUp = CreateSprite( sPathAndFilenameButtonUp, BASE_RENDER_ORDER_GUI_BUTTON );
		pNewValue->SetSpriteUp( pSpriteUp );
	}

	if ( sPathAndFilenameButtonDown != "" )
	{
		SoSprite* pSpriteDown = CreateSprite( sPathAndFilenameButtonDown, BASE_RENDER_ORDER_GUI_BUTTON );
		pNewValue->SetSpriteDown( pSpriteDown );
	}

	if ( sPathAndFilenameButtonDisabled != "" )
	{
		SoSprite* pSpriteDisabled = CreateSprite( sPathAndFilenameButtonDisabled, BASE_RENDER_ORDER_GUI_BUTTON );
		pNewValue->SetSpriteDisabled( pSpriteDisabled );
	}

	_ComboValueList.AddLast( pNewValue );

	if ( !_pCurrentComboValue )
	{
		_pCurrentComboValue = pNewValue;
		_pSpriteBackground = pNewValue->GetSpriteUp();
		SetWidthHeightFromSprite( _pSpriteBackground ); 
	}
}



/******************************************************************************
 FUNCTION:  IsUsable
******************************************************************************/
bool SoGuiAnimComboButton::IsUsable()
{
	return
	(
		//IsActive() &&
		_eState != STATE_DISABLED &&
		_eTransitionState != TRANSITION_STATE_INIT &&
		_eTransitionState != TRANSITION_STATE_HIDE &&
		_eTransitionState != TRANSITION_STATE_ANIM_IN &&
		_eTransitionState != TRANSITION_STATE_ANIM_OUT &&
		_eTransitionState != TRANSITION_STATE_LAST
	);
}



/******************************************************************************
 FUNCTION:  Enable
******************************************************************************/
void SoGuiAnimComboButton::Enable( bool b )
{ 
	if ( b )
	{
		if ( _eState == STATE_DISABLED )
			_eState = STATE_UNPRESSED;
	}
	else
		_eState = STATE_DISABLED;
}



/******************************************************************************
 FUNCTION:  OnTouchesDown
******************************************************************************/
bool SoGuiAnimComboButton::OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) 
{
	float fMargin = 0.0f;

	if ( IsUp() && IsUsable() )
	{
		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		SoCcTouchList::Iterator Iter( pNonConstTouchList );
		while ( Iter.Next() )
		{
			CCTouch* pTouch = Iter.GetNodeData();

			JpVector2f vFinalPos = _vAbsolutePos;
			if ( _bUseWorldSpacePositioning )
				vFinalPos = GetScreenPositionFromWorldSpace( _vWorldPosition );			

			if ( TouchIsInside( pTouch, vFinalPos, fMargin ) )
			{
				SetDown();
				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_COMBO_BUTTON_ON_DOWN, GetGuid() );
				SendMessageToParent( &GuiMessage );

				SoSoundManager::GetInstance()->PlaySoundEffect( _hSoundEffectOnButtonDown );

				return true;
			}
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoGuiAnimComboButton::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{
	float fMargin = 0.0f;

	if ( IsDown() && IsUsable() )
	{
		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		SoCcTouchList::Iterator Iter( pNonConstTouchList );
		while ( Iter.Next() )
		{
			CCTouch* pTouch = Iter.GetNodeData();

			SetUp();

			JpVector2f vFinalPos = _vAbsolutePos;
			if ( _bUseWorldSpacePositioning )
				vFinalPos = GetScreenPositionFromWorldSpace( _vWorldPosition );			

			if ( TouchIsInside( pTouch, vFinalPos, fMargin ) )
			{
				// Switch to the next button value before we send the message - else the button value sent
				// is not the value showing.
				SwitchToNextButtonValue();

				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_COMBO_BUTTON_ON_UP, GetGuid() );
				GuiMessage.AddHash( _pCurrentComboValue->GetValue() );
				SendMessageToParent( &GuiMessage );

				SoSoundManager::GetInstance()->PlaySoundEffect( _hSoundEffectOnButtonUp );

				return true;
			}
			else
			{
				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_COMBO_BUTTON_ON_UP_TOUCH_OUTSIDE, GetGuid() );
				SendMessageToParent( &GuiMessage );

				return true;
			}
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesMove
******************************************************************************/
bool SoGuiAnimComboButton::OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) 
{
	return false;
}



/******************************************************************************
 FUNCTION:  SwitchToNextButtonValue
******************************************************************************/
void SoGuiAnimComboButton::SwitchToNextButtonValue()
{
	SoComboButtonValueList::Node* pCurrentNode = _ComboValueList.Find( _pCurrentComboValue );
	if ( pCurrentNode )
	{
		SoComboButtonValue* pCurrentValue = pCurrentNode->GetData();
		SoSprite* pUp = pCurrentValue->GetSpriteUp();
		SoSprite* pDown = pCurrentValue->GetSpriteDown();
		SoSprite* pDisabled = pCurrentValue->GetSpriteDisabled();
		pUp->setOpacity( 255 );
		pUp->setVisible( false );


		SoComboButtonValueList::Node* pNextNode = pCurrentNode->GetNext();

		// skip over the head node, if we hit it.
		if ( pNextNode == _ComboValueList.GetHead() )
			pNextNode = pNextNode->GetNext();

		_pPreviousComboValue = _pCurrentComboValue;
		_pCurrentComboValue = pNextNode->GetData();

		_pSpriteBackground = pCurrentValue->GetSpriteUp();
		SetWidthHeightFromSprite( _pSpriteBackground ); 		
	}
}



/******************************************************************************
 FUNCTION:  GetScreenPositionFromWorldSpace
******************************************************************************/
JpVector2f SoGuiAnimComboButton::GetScreenPositionFromWorldSpace( JpVector2f vWorldPosition )
{
	JpVector2f vScreenPos( 0.0f, 0.0f );

	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	if ( pCameraManager )
	{
		SoCamera* pCamera = pCameraManager->GetCurrentCamera();

		if ( pCamera )
			vScreenPos = pCamera->TransformWorldToScreenSpace( vWorldPosition );
	}

	return vScreenPos;
}
