/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoGuiAnim.h"

USING_NS_CC;

#define SoComboButtonValueList JpList< SoComboButtonValue >


class SoComboButtonValue;

class SoGuiAnimComboButton : public SoGuiAnim
{
	public:

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_UNPRESSED,
			STATE_PRESSED,
			STATE_DISABLED,
			STATE_LAST
		};

		enum BUTTON_DOWN_IMAGE_OPTION
		{
			BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN,
			BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE,
			BUTTON_DOWN_IMAGE_OPTION_LAST
		};

		enum BUTTON_DISABLED_IMAGE_OPTION
		{
			BUTTON_DISABLED_IMAGE_OPTION_HIDE,
			BUTTON_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY,
			BUTTON_DISABLED_IMAGE_OPTION_USE_IMAGE,
			BUTTON_DISABLED_IMAGE_OPTION_LAST
		};


		//-----------------------
		// public classes
		class SetupParams
		{
			public:
				SetupParams() :
				_vRelativePos( 0.0f, 0.0f ),
				_hSoundEffectOnButtonUp( 0 ),
				_hSoundEffectOnButtonDown( 0 ),
				_eButtonDownImageOption( BUTTON_DOWN_IMAGE_OPTION_USE_IMAGE ),
				_eButtonDisabledImageOption( BUTTON_DISABLED_IMAGE_OPTION_USE_IMAGE )
				{};

				virtual ~SetupParams(){};
				JpVector2f _vRelativePos;
				hash _hSoundEffectOnButtonUp;
				hash _hSoundEffectOnButtonDown;
				BUTTON_DOWN_IMAGE_OPTION _eButtonDownImageOption;
				BUTTON_DISABLED_IMAGE_OPTION _eButtonDisabledImageOption;
		};


		//-----------------------
		// public functions

		SoGuiAnimComboButton();

		virtual ~SoGuiAnimComboButton();

		virtual void Init( CCNode* pParentNode, SoGuiManager* pGuiManager );

		virtual void Deinit();

		virtual void Update( float dt );

		virtual void HandleMessage( SoGuiMessage* pMessage );

		virtual SoGuiAnimComboButton* GetAsComboButton() { return this; }

		SoComboButtonValue* GetCurrentComboButtonValue() { return _pCurrentComboValue; }
		SoComboButtonValue* GetPreviousComboButtonValue() { return _pPreviousComboValue; }

		void Setup( const SetupParams& Params );
		void SetupAndAddComboButtonValue
		( 				
			hash hValue,
			const std::string& sPathAndFilenameButtonUp,
			const std::string& sPathAndFilenameButtonDown,
			const std::string& sPathAndFilenameButtonDisabled
		);

		bool IsUsable();
		void Enable( bool b );

		bool IsDown() { return _eState == STATE_PRESSED; }
		void SetDown() { _eStateToSwitchTo = STATE_PRESSED; }

		bool IsUp() { return _eState == STATE_UNPRESSED; }
		void SetUp() { _eStateToSwitchTo = STATE_UNPRESSED; }

		void SetUseWorldSpacePositioning( bool b ) { _bUseWorldSpacePositioning = b; }
		void SetWorldPosition( const JpVector2f& v ) { _vWorldPosition = v; }


		// SoInputTouch callbacks
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList );


	protected:

		//-----------------------
		// protected variables

		SoComboButtonValueList _ComboValueList;
		SoComboButtonValue* _pCurrentComboValue;
		SoComboButtonValue* _pPreviousComboValue;

		hash _hSoundEffectOnButtonUp;
		hash _hSoundEffectOnButtonDown;


		BUTTON_DOWN_IMAGE_OPTION _eButtonDownImageOption;
		BUTTON_DISABLED_IMAGE_OPTION _eButtonDisabledImageOption;

		STATE _eState;
		STATE _eStateToSwitchTo;


		bool _bUseWorldSpacePositioning;
		JpVector2f _vWorldPosition;

		//-----------------------
		// protected functions

		void SwitchToNextButtonValue();

		JpVector2f GetScreenPositionFromWorldSpace( JpVector2f vWorldPosition );
};



class SoComboButtonValue
{
	public:

		//-----------------------
		// public functions
		SoComboButtonValue() :
		_hValue( 0 ),
		_pSpriteUp( NULL ),
		_pSpriteDown( NULL ),
		_pSpriteDisabled( NULL )
		{};

		virtual ~SoComboButtonValue() {};

		hash GetValue() { return _hValue; }
		void SetValue( hash h ) { _hValue = h; }

		SoSprite* GetSpriteUp() { return _pSpriteUp; }
		void SetSpriteUp( SoSprite* p ) { _pSpriteUp = p; }

		SoSprite* GetSpriteDown() { return _pSpriteDown; }
		void SetSpriteDown( SoSprite* p ) { _pSpriteDown = p; }
		
		SoSprite* GetSpriteDisabled() { return _pSpriteDisabled; }
		void SetSpriteDisabled( SoSprite* p ) { _pSpriteDisabled = p; }

	private:

		//-----------------------
		// private variables
		hash _hValue;
		SoSprite* _pSpriteUp;
		SoSprite* _pSpriteDown;
		SoSprite* _pSpriteDisabled;
};
