/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCamera.h"
#include "SoFontDefs.h"
#include "SoGuiAnimSlider.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoSoundManager.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoGuiAnimSlider
******************************************************************************/
SoGuiAnimSlider::SoGuiAnimSlider() :
_bUseWorldSpacePositioning( false ),
_vWorldPosition( 0.0f, 0.0f ),
_vButtonPosCurrent( 0.0f, 0.0f ),
_vButtonPosCentered( 0.0f, 0.0f ),
_fLength( 0.0f ),
_fValue( 0.0f ),
_bIsDisabled( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimSlider
******************************************************************************/
SoGuiAnimSlider::~SoGuiAnimSlider()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimSlider::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnim::Init( pParentNode, pGuiManager );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimSlider::Deinit()
{
	SoGuiAnim::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimSlider::Update( float dt )
{
	SoGuiAnim::Update( dt );

	JpVector2f vFinal = _vAbsolutePos;

	if ( _bUseWorldSpacePositioning )
	{
		vFinal = GetScreenPositionFromWorldSpace( _vWorldPosition );
	}

	_pLabel->SetPosition( vFinal + _vLabelPosition );

	if ( _bIsDisabled )
	{
		SoGuiAnimButton* pButton = _pGuiManager->GetGuiAnimButtonByGuid( _gSliderButton );
		pButton->Enable( false );
		_pSpriteBackground->setOpacity( _fSpriteAlpha * 0.5f * 255.0f );
		_pLabel->setOpacity( _fTextAlpha * 0.5f * 255.0f );
	}
	else
	{
		SoGuiAnimButton* pButton = _pGuiManager->GetGuiAnimButtonByGuid( _gSliderButton );
		pButton->Enable( true );
		_pSpriteBackground->setOpacity( _fSpriteAlpha * 255.0f );
		_pLabel->setOpacity( _fTextAlpha * 255.0f );
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimSlider::HandleMessage( SoGuiMessage* pMessage )
{
	SoGuiAnim::HandleMessage( pMessage );
}



/******************************************************************************
 FUNCTION:  Setup
******************************************************************************/
void SoGuiAnimSlider::Setup( const SetupParams& Params )
{
	_vButtonPosCentered = Params._vRelativePos;
	_vButtonPosCentered.Y = 0.0f;
	_vButtonPosCurrent = Params._vRelativePos;
	_vButtonPosCurrent.Y = 0.0f;
	_vRelativePos = Params._vRelativePos;
	_fValue = Params._fInitialValue;
	_fLength = Params._fLength;
	_vLabelPosition =  Params._vLabelPosition;
	_bIsDisabled = Params._bIsDisabled;

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( Params._sBackgroundFilename.c_str() ) );

	_pLabel = CreateBmFontLabel( Params._sLabel, SoFontDefs::GetGuiDefault(), kCCTextAlignmentCenter );
	_pLabel->setOpacity( 0 );

	_gSliderButton = _pGuiManager->CreateAndAddAnimButton( Params._ButtonParams, this );

	SetValue( _fValue );
}



/******************************************************************************
 FUNCTION:  OnTouchesDown
******************************************************************************/
bool SoGuiAnimSlider::OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) 
{
	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoGuiAnimSlider::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{
	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesMove
******************************************************************************/
bool SoGuiAnimSlider::OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) 
{
	if ( IsShowing() && !_bIsDisabled )
	{
		SoGuiAnimButton* pButton = _pGuiManager->GetGuiAnimButtonByGuid( _gSliderButton );
		float fHalfLength = _fLength * 0.5f;

		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;

		if ( pNonConstTouchList->GetNumEntries() > 0 )
		{
			CCTouch* pTouch = pNonConstTouchList->GetAtIndex( 0 )->GetData();

			CCPoint vRel = pTouch->previousLocationInView() - pTouch->locationInView();

			if ( pButton->IsDown() )
			{
				float fNewValue = _vButtonPosCurrent.X - ( vRel.x / SoUtils::GetiPadRatioX() );
				
				float fMinX = _vButtonPosCentered.X - fHalfLength;
				float fMaxX = _vButtonPosCentered.X + fHalfLength;

				fNewValue = Clamp( fNewValue, fMinX, fMaxX );

				_vButtonPosCurrent.X = fNewValue;

				pButton->SetRelativePos( JpVector2f( SoUtils::GetiPadRatioX() * fNewValue, _vButtonPosCentered.Y ) );

				// Normalize the value to a '0 through 1' value
				fNewValue -= _vButtonPosCentered.X;
				fNewValue += fHalfLength;
				fNewValue /= _fLength;

				_fValue = fNewValue;

				//CCLOG( "SliderValue: %f", _fValue );
				return true;
			}
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  GetScreenPositionFromWorldSpace
******************************************************************************/
JpVector2f SoGuiAnimSlider::GetScreenPositionFromWorldSpace( JpVector2f vWorldPosition )
{
	JpVector2f vScreenPos( 0.0f, 0.0f );

	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	if ( pCameraManager )
	{
		SoCamera* pCamera = pCameraManager->GetCurrentCamera();

		if ( pCamera )
			vScreenPos = pCamera->TransformWorldToScreenSpace( vWorldPosition );
	}

	return vScreenPos;
}



/******************************************************************************
 FUNCTION:  SetValue
******************************************************************************/
void SoGuiAnimSlider::SetValue( float fValue )
{
	SoGuiAnimButton* pButton = _pGuiManager->GetGuiAnimButtonByGuid( _gSliderButton );
	float fHalfLength = _fLength * 0.5f;

	float fPosition = _fLength * _fValue;
	fPosition -= fHalfLength;
	fPosition += _vButtonPosCentered.X;

	JpVector2f vButtonPosition( fPosition, _vButtonPosCentered.Y );
	JpVector2f vButtonPositionScaled( fPosition * SoUtils::GetiPadRatioX(), _vButtonPosCentered.Y );

	pButton->SetRelativePos( vButtonPositionScaled );

	_vButtonPosCurrent = vButtonPosition;
	_fValue = fValue;
}



/******************************************************************************
 FUNCTION:  SetLabelText
******************************************************************************/
void SoGuiAnimSlider::SetLabelText( std::string& sText )
{
	if ( _pLabel )
		_pLabel->setString( sText.c_str() );
	
}
