/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoGuiAnimTextEntry.h"
#include "SoGuiMessage.h"
#include "SoLabelBmFont.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiTextEntry
******************************************************************************/
SoGuiAnimTextEntry::SoGuiAnimTextEntry() :
_pLabelTextField( NULL ),
_pLabelCursor( NULL ),
_bClearInitialTextOnFirstInput( true ),
_bIsFirstInput( true ),
_iMaxCharacters( 128 ),
_eTextJustification( TEXT_JUSTIFICATION_LEFT ),
_fCursorBlinkTimer( 0.0f ),
_bIsTextBeingEdited( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiTextEntry
******************************************************************************/
SoGuiAnimTextEntry::~SoGuiAnimTextEntry()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimTextEntry::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnim::Init( pParentNode, pGuiManager );

	_InputVirtualKeyboardListener.SetOwner( this );
	SoInputVirtualKeyboard::GetInstance()->AddListener( &_InputVirtualKeyboardListener );

	_bIsTextBeingEdited = false;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimTextEntry::Deinit()
{
	if ( _pClipNode )
	{
		if ( _pLabelTextField )
			_pClipNode->removeChild( _pLabelTextField, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
	}

	if ( _pLabelTextField )
	{
		_pLabelTextField->release();
		_pLabelTextField = NULL;
	}


	// Note: this has to come after _pLabelTextField is removed from _pClipNode
	// because _pClipNode is destroyed in the base class Deinit.
	// mdomowicz 2013_10_03
	SoGuiAnim::Deinit();


	SoInputVirtualKeyboard::GetInstance()->RemoveListener( &_InputVirtualKeyboardListener );
}



/******************************************************************************
 FUNCTION:  Setup
******************************************************************************/
void SoGuiAnimTextEntry::Setup( const char* pacInitialText, const char* pacBmFontName )
{
	SetInitialText( pacInitialText );
	SetFontName( pacBmFontName );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimTextEntry::Update( float dt )
{
	SoGuiAnim::Update( dt );

	if ( !_pLabelTextField )
	{
		_pLabelTextField = SoLabelBmFont::create
		(
			_sInitialText.c_str(), 
			SoUtils::AddRelativePathToFont( _sFontName.c_str() ), 
			_fWidth, 
			kCCTextAlignmentLeft
		);
		_pLabelTextField->setOpacity( 255.0f );
		_pLabelTextField->retain();
		_pClipNode->addChild( _pLabelTextField, 0 );

		if ( _eTextJustification == TEXT_JUSTIFICATION_LEFT )
			_pLabelTextField->setAnchorPoint( CCPoint( 0.0f, 0.5f ) );
		else if ( _eTextJustification == TEXT_JUSTIFICATION_CENTER )
			_pLabelTextField->setAnchorPoint( CCPoint( 0.5f, 0.5f ) );

		SoUtils::SetRenderZOrder( _pClipNode, _pLabelTextField, BASE_RENDER_ORDER_GUI_TEXT_ENTRY );
	}

	if ( _eTextJustification == TEXT_JUSTIFICATION_LEFT )
		_pLabelTextField->setPosition( ccp( _vAbsolutePos.X - ( _fWidth / 2.0f ), _vAbsolutePos.Y ) );
	else if ( _eTextJustification == TEXT_JUSTIFICATION_CENTER )
		_pLabelTextField->setPosition( ccp( _vAbsolutePos.X, _vAbsolutePos.Y ) );

	_pLabelTextField->setWidth( _fWidth );
	_pLabelTextField->setScaleX( SoUtils::GetiPadRatioX() );
	_pLabelTextField->setScaleY( SoUtils::GetiPadRatioY() );

	if ( _eTransitionState == TRANSITION_STATE_SHOW )
		_pLabelTextField->setOpacity( 255.0f );
	else
		_pLabelTextField->setOpacity( 0.0f );


	UpdateCursor( dt );
}



/******************************************************************************
 FUNCTION:  UpdateCursor
******************************************************************************/
void SoGuiAnimTextEntry::UpdateCursor( float dt )
{
	if ( !_pLabelCursor )
	{
		_pLabelCursor = CreateBmFontLabel( " ", _sFontName );
		_pLabelCursor->setScaleX( SoUtils::GetiPadRatioX() );
		_pLabelCursor->setScaleY( SoUtils::GetiPadRatioY() );
		_pLabelCursor->setAnchorPoint( CCPoint( 0, 0.5f ) );
	}


	if ( !_bIsTextBeingEdited )
	{
		_pLabelCursor->setVisible( false );
	}
	else
	{
		_pLabelCursor->setVisible( true );

		if ( _eTransitionState == TRANSITION_STATE_SHOW )
			_pLabelCursor->setOpacity( 255.0f );
		else
			_pLabelCursor->setOpacity( 0.0f );


		_fCursorBlinkTimer += dt;

		if ( _fCursorBlinkTimer > 0.5f )
		{
			_fCursorBlinkTimer -= 0.5f;

			if ( JpHash( _pLabelCursor->getString() ) == JpHash( " " ) )
				_pLabelCursor->setString( "_" );
			else
				_pLabelCursor->setString( " " );
		}


		if ( _eTextJustification == TEXT_JUSTIFICATION_LEFT )
		{
			float fX = _vAbsolutePos.X + _pLabelTextField->GetCalculatedWidthAllRes();
			fX -= ( _fWidth / 2.0f );

			_pLabelCursor->setPosition( fX, _vAbsolutePos.Y );
		}
		else if ( _eTextJustification == TEXT_JUSTIFICATION_CENTER )
		{
			float fX = _vAbsolutePos.X + ( _pLabelTextField->GetCalculatedWidthAllRes() / 2.0f );
			_pLabelCursor->setPosition( fX, _vAbsolutePos.Y );
		}
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimTextEntry::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			CloseVirtualKeyboard();
			_bIsTextBeingEdited = false;

			SoGuiAnim::HandleMessage( pMessage );
			break;
		}
		default:
		{
			SoGuiAnim::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  SetPositionAndSize
******************************************************************************/
void SoGuiAnimTextEntry::SetPositionAndSize
( 
	float fBottomLeftX, 
	float fBottomLeftY, 
	float fWidth, 
	float fHeight 
)
{
	_fWidth = fWidth;
	_fHeight = fHeight;
	_fTargetWidth = _fWidth;
	_fTargetHeight = _fHeight;
	SetRelativePos( JpVector2f( fBottomLeftX, fBottomLeftY ) );
}





/******************************************************************************
 FUNCTION:  OnVirtualKeyPressed
******************************************************************************/
void SoGuiAnimTextEntry::OnVirtualKeyPressed( char cKey )
{
	if ( cKey == '\n' )
	{
		CloseVirtualKeyboard();
		_bIsTextBeingEdited = false;

		// If the user hits "enter" when the text field is empty,
		// set the text field back to the initial text, and 
		// act as if the next input is the first input again
		std::string sText( _pLabelTextField->getString() );
		if ( sText.length() == 0 )
		{
			_bIsFirstInput = true;
			_pLabelTextField->setString( _sInitialText.c_str() );
		}
	}
	else
	{
		if ( _bIsFirstInput && _bClearInitialTextOnFirstInput )
		{
			_bIsFirstInput = false;
            
            // Converting to a string like this ensures there is a
            // null terminator, and prevents a crash in setString.
            std::string sTemp = "";
            sTemp += cKey;
            
			_pLabelTextField->setString( sTemp.c_str() );
		}
		else
		{
			std::string sText( _pLabelTextField->getString() );
			sText += cKey;

			if ( sText.length() <= (size_t)_iMaxCharacters )
				_pLabelTextField->setString( sText.c_str() );
		}
	}

}



/******************************************************************************
 FUNCTION:  OnVirtualKeyDeleteBackward
******************************************************************************/
void SoGuiAnimTextEntry::OnVirtualKeyDeleteBackward()
{
	std::string sText( _pLabelTextField->getString() );

	if ( _bIsFirstInput && _bClearInitialTextOnFirstInput )
	{
		_bIsFirstInput = false;
		_pLabelTextField->setString( "" );
	}
	else
	{
		if ( sText.length() > 0 )
		{
			std::string::iterator Iter = sText.end() - 1;
			sText.erase( Iter );
			_pLabelTextField->setString( sText.c_str() );
		}
	}
}



/******************************************************************************
 FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoGuiAnimTextEntry::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{
	float fMargin = 0.0f;

	if ( IsUsable() )
	{
		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		SoCcTouchList::Iterator Iter( pNonConstTouchList );
		while ( Iter.Next() )
		{
			CCTouch* pTouch = Iter.GetNodeData();

			JpVector2f vFinalPos = _vAbsolutePos;

			if ( TouchIsInside( pTouch, vFinalPos, fMargin ) )
			{
				OpenVirtualKeyboard();
				_bIsTextBeingEdited = true;
	
				return true;
			}
		}
	}

	return false;
}




/******************************************************************************
 FUNCTION:  IsUsable
******************************************************************************/
bool SoGuiAnimTextEntry::IsUsable()
{
	return
	(
		_eTransitionState != TRANSITION_STATE_INIT &&
		_eTransitionState != TRANSITION_STATE_HIDE &&
		_eTransitionState != TRANSITION_STATE_ANIM_IN &&
		_eTransitionState != TRANSITION_STATE_ANIM_OUT &&
		_eTransitionState != TRANSITION_STATE_LAST
	);
}



/******************************************************************************
 FUNCTION:  SetContentTextToInitialText
******************************************************************************/
void SoGuiAnimTextEntry::SetContentTextToInitialText()
{
	if ( _pLabelTextField )
		_pLabelTextField->setString( _sInitialText.c_str() );
}



/******************************************************************************
 FUNCTION:  GetContentText
******************************************************************************/
const char* SoGuiAnimTextEntry::GetContentText()
{
	return _pLabelTextField->getString();
}



/******************************************************************************
 FUNCTION:  ClearContentText
******************************************************************************/
void SoGuiAnimTextEntry::ClearContentText()
{
	if ( _pLabelTextField )
		_pLabelTextField->setString( "" );
}



/******************************************************************************
 FUNCTION:  OpenVirtualKeyboard
******************************************************************************/
void SoGuiAnimTextEntry::OpenVirtualKeyboard()
{
	SoInputVirtualKeyboard* pVirtualKeyboard = SoInputVirtualKeyboard::GetInstance();
	if ( pVirtualKeyboard )
		pVirtualKeyboard->BeginTextEntryShowVirtualKeyboardIfNeeded();
}



/******************************************************************************
 FUNCTION:  CloseVirtualKeyboard
******************************************************************************/
void SoGuiAnimTextEntry::CloseVirtualKeyboard()
{
	SoInputVirtualKeyboard* pVirtualKeyboard = SoInputVirtualKeyboard::GetInstance();
	if ( pVirtualKeyboard )
		pVirtualKeyboard->EndTextEntryHideVirtualKeyboardIfNeeded();
}
