/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoGuiAnim.h"
#include "SoInputVirtualKeyboard.h"

USING_NS_CC;


class SoLabelBmFont;

class SoGuiAnimTextEntry : public SoGuiAnim
{
	public:

		//-----------------------
		// public enums
		enum TEXT_JUSTIFICATION
		{
			TEXT_JUSTIFICATION_LEFT,
			TEXT_JUSTIFICATION_CENTER,
			TEXT_JUSTIFICATION_LAST
		};


		//-----------------------
		// public functions

		SoGuiAnimTextEntry();

		virtual ~SoGuiAnimTextEntry();

		virtual void Init( CCNode* pParentNode, SoGuiManager* pGuiManager );

		virtual void Deinit();

		virtual void Update( float dt );

		virtual void HandleMessage( SoGuiMessage* pMessage );

		void Setup( const char* pacInitialText, const char* pacBmFontName );

		bool IsUsable();

		void SetPositionAndSize( float fBottomLeftX, float fBottomLeftY, float fWidth, float fHeight );

		bool GetClearInitialTextOnFirstInput() { return _bClearInitialTextOnFirstInput; }
		void SetClearInitialTextOnFirstInput( bool b ) { _bClearInitialTextOnFirstInput = b; }

		int GetMaxCharacters() { return _iMaxCharacters; }
		void SetMaxCharacters( int i ) { _iMaxCharacters = i; }

		SoLabelBmFont* GetLabelTextField() { return _pLabelTextField; }

		const std::string& GetInitialText() { return _sInitialText; }
		void SetInitialText( const std::string& sInitialText ) { _sInitialText = sInitialText; }
		void SetContentTextToInitialText();

		const std::string& GetFontName() { return _sFontName; }
		void SetFontName( const std::string& sFontName ) { _sFontName = sFontName; }

		void SetTextJustification( TEXT_JUSTIFICATION e ) { _eTextJustification = e; }

		const char* GetContentText();
		void ClearContentText();

		// SoInputVirtualKeyboard callbacks
		virtual void OnVirtualKeyPressed( char cKey );
		virtual void OnVirtualKeyReleased( char cKey ) {};
		virtual void OnVirtualKeyDeleteBackward();

		// SoInputTouch callbacks
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) { return false; }
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) { return false; }

	private:

		//-----------------------
		// private variables

		int _iMaxCharacters;

		SoLabelBmFont* _pLabelTextField;
		SoLabelBmFont* _pLabelCursor;

		std::string _sFontName;
		std::string _sInitialText;

		TEXT_JUSTIFICATION _eTextJustification;

		bool _bClearInitialTextOnFirstInput;
		bool _bIsFirstInput;

		SoInputVirtualKeyboardListener< SoGuiAnimTextEntry > _InputVirtualKeyboardListener;


		float _fCursorBlinkTimer;
		bool _bIsTextBeingEdited;

		//-----------------------
		// private functions
		void UpdateCursor( float dt );

		void OpenVirtualKeyboard();
		void CloseVirtualKeyboard();
};




