/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoGuiAnimToggle.h"
#include "SoGuiMessage.h"
#include "SoSoundManager.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoGuiAnimToggle
******************************************************************************/
SoGuiAnimToggle::SoGuiAnimToggle() :
_pSpriteOn( NULL ),
_pSpriteOff( NULL ),
_pSpriteDisabled( NULL ),
_eState( STATE_TOGGLE_OFF ),
_eStateToSwitchTo( STATE_TOGGLE_OFF ),
_hSoundEffectOnToggleOff( 0 ),
_hSoundEffectOnToggleOn( 0 ),
_eToggleOnImageOption( TOGGLE_ON_IMAGE_OPTION_USE_IMAGE ),
_eToggleDisabledImageOption( TOGGLE_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimToggle
******************************************************************************/
SoGuiAnimToggle::~SoGuiAnimToggle()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimToggle::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnim::Init( pParentNode, pGuiManager );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimToggle::Deinit()
{
	SoGuiAnim::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimToggle::Update( float dt )
{
	SoGuiAnim::Update( dt );


	switch ( _eState )
	{
		case STATE_TOGGLE_OFF:
		{
			if ( _eToggleOnImageOption == TOGGLE_ON_IMAGE_OPTION_SCALE_THE_OFF_IMAGE_DOWN )
			{
				if ( _pSpriteOff )
				{
					float fScaleX = ( _fWidth / _pSpriteOff->getContentSize().width );
					float fScaleY = ( _fHeight / _pSpriteOff->getContentSize().height );

					_pSpriteOff->setScaleX( fScaleX );
					_pSpriteOff->setScaleY( fScaleY );
					_pSpriteOff->setOpacity( _fSpriteAlpha * 255.0f );
				}
			}
			else if ( _eToggleOnImageOption == TOGGLE_ON_IMAGE_OPTION_USE_IMAGE )
			{
				if ( _pSpriteOff )
				{
					_pSpriteOff->setVisible( true );
					_pSpriteOff->setOpacity( _fSpriteAlpha * 255.0f );
				}

				if ( _pSpriteOn )
					_pSpriteOn->setVisible( false );
			}

			if ( _pSpriteDisabled )
				_pSpriteDisabled->setVisible( false );

			if ( _eStateToSwitchTo != _eState )
			{
				_eState = _eStateToSwitchTo;
			}

			break;
		}

		case STATE_PRESSED_BUT_NOT_YET_ON:
		case STATE_TOGGLE_ON:
		{
			if ( _eToggleOnImageOption == TOGGLE_ON_IMAGE_OPTION_SCALE_THE_OFF_IMAGE_DOWN )
			{
				if ( _pSpriteOff )
				{
					float fScaleX = ( _fWidth / _pSpriteOff->getContentSize().width );
					float fScaleY = ( _fHeight / _pSpriteOff->getContentSize().height );

					_pSpriteOff->setScaleX( fScaleX * 0.9f );
					_pSpriteOff->setScaleY( fScaleY * 0.9f );
				}
			}
			else if ( _eToggleOnImageOption == TOGGLE_ON_IMAGE_OPTION_USE_IMAGE )
			{
				if ( _pSpriteOff )
					_pSpriteOff->setVisible( false );

				if ( _pSpriteOn )
					_pSpriteOn->setVisible( true );
			}

			if ( _pSpriteDisabled )
				_pSpriteDisabled->setVisible( false );

			if ( _eStateToSwitchTo != _eState )
			{
				_eState = _eStateToSwitchTo;
			}
			break;
		}

		case STATE_DISABLED:
		{
			if ( _pSpriteOff )
				_pSpriteOff->setVisible( false );

			if ( _pSpriteOn )
				_pSpriteOn->setVisible( false );


			if ( _eToggleDisabledImageOption == TOGGLE_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY )
			{
				if ( _pSpriteOff )
				{
					_pSpriteOff->setVisible( true );
					_pSpriteOff->setOpacity( 0.5f * _fSpriteAlpha * 255.0f );
				}
			}
			else if ( _eToggleDisabledImageOption == TOGGLE_DISABLED_IMAGE_OPTION_USE_IMAGE )
			{
				if ( _pSpriteDisabled )
					_pSpriteDisabled->setVisible( true );
			}
			else if ( _eToggleDisabledImageOption == TOGGLE_DISABLED_IMAGE_OPTION_HIDE )
			{
				if ( _pSpriteDisabled )
					_pSpriteDisabled->setVisible( false );
			}
		
			break;
		}

		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknown button state." ) );
	}


	if ( _pSpriteOn )
	{
		_pSpriteOn->SetPosition( _vAbsolutePos );
	}

	if ( _pSpriteOff )
	{
		_pSpriteOff->SetPosition( _vAbsolutePos );
	}

	if ( _pSpriteDisabled )
	{
		_pSpriteDisabled->SetPosition( _vAbsolutePos );
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimToggle::HandleMessage( SoGuiMessage* pMessage )
{
	SoGuiAnim::HandleMessage( pMessage );
}




/******************************************************************************
 FUNCTION:  Setup
******************************************************************************/
void SoGuiAnimToggle::Setup( const SetupParams& Params )
{
	if ( Params._sPathAndFilenameToggleOff != "" )
		CreateSpriteOff( Params._sPathAndFilenameToggleOff );

	if ( Params._sPathAndFilenameToggleOn != "" )
		CreateSpriteOn( Params._sPathAndFilenameToggleOn );

	if ( Params._sPathAndFilenameToggleDisabled != "" )
		CreateSpriteDisabled( Params._sPathAndFilenameToggleDisabled );

	SetRelativePos( Params._vRelativePos );

	_hSoundEffectOnToggleOff = Params._hSoundEffectOnToggleOff;
	_hSoundEffectOnToggleOn = Params._hSoundEffectOnToggleOn;

	_eToggleOnImageOption = Params._eToggleOnImageOption;
	_eToggleDisabledImageOption = Params._eToggleDisabledImageOption;
}



/******************************************************************************
 FUNCTION:  IsUsable
******************************************************************************/
bool SoGuiAnimToggle::IsUsable()
{
	return
	(
		//IsActive() &&
		_eState != STATE_DISABLED &&
		_eTransitionState != TRANSITION_STATE_INIT &&
		_eTransitionState != TRANSITION_STATE_HIDE &&
		_eTransitionState != TRANSITION_STATE_ANIM_IN &&
		_eTransitionState != TRANSITION_STATE_ANIM_OUT &&
		_eTransitionState != TRANSITION_STATE_LAST
	);
}



/******************************************************************************
 FUNCTION:  Enable
******************************************************************************/
void SoGuiAnimToggle::Enable( bool b )
{ 
	if ( b )
		_eState = STATE_TOGGLE_OFF;
	else
		_eState = STATE_DISABLED;
}



/******************************************************************************
 FUNCTION:  CreateSpriteOff
******************************************************************************/
void SoGuiAnimToggle::CreateSpriteOff( const std::string& sPathAndFilename ) 
{ 
	_pSpriteOff = CreateSprite( sPathAndFilename, BASE_RENDER_ORDER_GUI_TOGGLE ); 
	SetWidthHeightFromSprite( _pSpriteOff );  // temp!  we should find a better way to do this.  mdomowicz 2013_07_06
	_pSpriteBackground = _pSpriteOff;
}



/******************************************************************************
 FUNCTION:  CreateSpriteOn
******************************************************************************/
void SoGuiAnimToggle::CreateSpriteOn( const std::string& sPathAndFilename )
{
	_pSpriteOn = CreateSprite( sPathAndFilename, BASE_RENDER_ORDER_GUI_TOGGLE ); 
}



/******************************************************************************
 FUNCTION:  CreateSpriteDisabled
******************************************************************************/
void SoGuiAnimToggle::CreateSpriteDisabled( const std::string& sPathAndFilename )
{
	_pSpriteDisabled = CreateSprite( sPathAndFilename, BASE_RENDER_ORDER_GUI_TOGGLE ); 
}



/******************************************************************************
 FUNCTION:  OnTouchesDown
******************************************************************************/
bool SoGuiAnimToggle::OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) 
{
	float fMargin = 0.0f;

	if ( IsOff() && IsUsable() )
	{
		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		SoCcTouchList::Iterator Iter( pNonConstTouchList );
		while ( Iter.Next() )
		{
			CCTouch* pTouch = Iter.GetNodeData();

			if ( TouchIsInside( pTouch, _vAbsolutePos, fMargin ) )
			{
				_eStateToSwitchTo = STATE_PRESSED_BUT_NOT_YET_ON;
				return true;
			}
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoGuiAnimToggle::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{
	float fMargin = 0.0f;

	if ( IsUsable() )
	{
		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		SoCcTouchList::Iterator Iter( pNonConstTouchList );
		while ( Iter.Next() )
		{
			CCTouch* pTouch = Iter.GetNodeData();

			if ( TouchIsInside( pTouch, _vAbsolutePos, fMargin ) )
			{
				if ( IsOn() )
				{
					SetOff();

					SoSoundManager::GetInstance()->PlaySoundEffect( _hSoundEffectOnToggleOff );
				}
				else if ( IsOff() )
				{
					SetOn();

					SoSoundManager::GetInstance()->PlaySoundEffect( _hSoundEffectOnToggleOn );
				}

				return true;
			}
			else
			{
				if ( _eState == STATE_PRESSED_BUT_NOT_YET_ON )
				{
					_eStateToSwitchTo = STATE_TOGGLE_OFF;
				}
			}
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesMove
******************************************************************************/
bool SoGuiAnimToggle::OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) 
{
	return false;
}



/******************************************************************************
 FUNCTION:  SetOn
******************************************************************************/
void SoGuiAnimToggle::SetOn()
{ 
	_eStateToSwitchTo = STATE_TOGGLE_ON; 

	SoGuiMessage GuiMessage( SoGuiMessage::TYPE_TOGGLE_ON_ON, GetGuid() );
	SendMessageToParent( &GuiMessage );
}



/******************************************************************************
 FUNCTION:  SetOff
******************************************************************************/
void SoGuiAnimToggle::SetOff() 
{ 
	_eStateToSwitchTo = STATE_TOGGLE_OFF; 

	SoGuiMessage GuiMessage( SoGuiMessage::TYPE_TOGGLE_ON_OFF, GetGuid() );
	SendMessageToParent( &GuiMessage );
}
