/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoGuiAnim.h"

USING_NS_CC;


class SoGuiAnimToggle : public SoGuiAnim
{
	public:

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_TOGGLE_OFF,
			STATE_PRESSED_BUT_NOT_YET_ON,
			STATE_TOGGLE_ON,
			STATE_DISABLED,
			STATE_LAST
		};

		enum TOGGLE_ON_IMAGE_OPTION
		{
			TOGGLE_ON_IMAGE_OPTION_SCALE_THE_OFF_IMAGE_DOWN,
			TOGGLE_ON_IMAGE_OPTION_USE_IMAGE,
			TOGGLE_ON_IMAGE_OPTION_LAST
		};

		enum TOGGLE_DISABLED_IMAGE_OPTION
		{
			TOGGLE_DISABLED_IMAGE_OPTION_HIDE,
			TOGGLE_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY,
			TOGGLE_DISABLED_IMAGE_OPTION_USE_IMAGE,
			TOGGLE_DISABLED_IMAGE_OPTION_LAST
		};


		//-----------------------
		// public classes
		class SetupParams
		{
			public:
				SetupParams() :
				_vRelativePos( 0.0f, 0.0f ),
				_hSoundEffectOnToggleOff( 0 ),
				_hSoundEffectOnToggleOn( 0 ),
				_eToggleOnImageOption( TOGGLE_ON_IMAGE_OPTION_USE_IMAGE ),
				_eToggleDisabledImageOption( TOGGLE_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY )
				{};

				virtual ~SetupParams(){};
				std::string _sPathAndFilenameToggleOff;
				std::string _sPathAndFilenameToggleOn;
				std::string _sPathAndFilenameToggleDisabled;
				JpVector2f _vRelativePos;
				hash _hSoundEffectOnToggleOff;
				hash _hSoundEffectOnToggleOn;
				TOGGLE_ON_IMAGE_OPTION _eToggleOnImageOption;
				TOGGLE_DISABLED_IMAGE_OPTION _eToggleDisabledImageOption;
		};


		//-----------------------
		// public functions

		SoGuiAnimToggle();

		virtual ~SoGuiAnimToggle();

		virtual void Init( CCNode* pParentNode, SoGuiManager* pGuiManager );

		virtual void Deinit();

		virtual void Update( float dt );

		virtual void HandleMessage( SoGuiMessage* pMessage );

		virtual SoGuiAnimToggle* GetAsAnimToggle() { return this; }

		void Setup( const SetupParams& Params );


		bool IsUsable();
		bool IsEnabled() { return ( _eState != STATE_DISABLED ); }
		void Enable( bool b );

		bool IsOn() { return _eState == STATE_TOGGLE_ON; }
		void SetOn();

		bool IsOff() { return ( _eState == STATE_TOGGLE_OFF || _eState == STATE_PRESSED_BUT_NOT_YET_ON ); }
		void SetOff();


		void CreateSpriteOff( const std::string& sPathAndFilename ); 
		void CreateSpriteOn( const std::string& sPathAndFilename );
		void CreateSpriteDisabled( const std::string& sPathAndFilename );


		// SoInputTouch callbacks
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList );


	protected:

		//-----------------------
		// protected variables

		SoSprite* _pSpriteOn;
		SoSprite* _pSpriteOff;
		SoSprite* _pSpriteDisabled;

		hash _hSoundEffectOnToggleOff;
		hash _hSoundEffectOnToggleOn;


		TOGGLE_ON_IMAGE_OPTION _eToggleOnImageOption;
		TOGGLE_DISABLED_IMAGE_OPTION _eToggleDisabledImageOption;

		STATE _eState;
		STATE _eStateToSwitchTo;

};




