/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "NeConfig.h"

#include "SoCamera.h"
#include "SoFontDefs.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinConfirm.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


#define CONFIRM_WINDOW_WIDTH 500.0f
#define CONFIRM_WINDOW_HEIGHT 345.0f


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinConfirm::SoGuiAnimWinConfirm() :
_gButtonCancel( 0 ),
_gButtonOk( 0 ),
_pLabel( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinConfirm::~SoGuiAnimWinConfirm()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinConfirm::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgLarge.png" ) );

	SetWidth( CONFIRM_WINDOW_WIDTH * SoUtils::GetTrueiPadRatioX() );
	SetHeight( CONFIRM_WINDOW_HEIGHT * SoUtils::GetiPadRatioY() );

	SetSpriteAlpha( 0.85f );

	SetPositionToCenterOfParent();

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	


	//-----------------------
	// window text
	_pLabel = CreateBmFontLabel( "", SoFontDefs::GetGuiDefaultLarge(), kCCTextAlignmentCenter );
	_pLabel->setOpacity( 255 );
	_pLabel->setVisible( false );
	_pLabel->setAnchorPoint( ccp( 0.5f, 0.5f ) );


	//-----------------------
	// Cancel Button
	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonCancelUp.png" );
	ButtonParams._vRelativePos.Init( -130.0f * SoUtils::GetiPadRatioX(), -110.0f * SoUtils::GetiPadRatioY() );
	_gButtonCancel = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	//-----------------------
	// Ok Button
	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonOkUp.png" );
	ButtonParams._vRelativePos.Init( 130.0f * SoUtils::GetiPadRatioX(), -110.0f * SoUtils::GetiPadRatioY() );
	_gButtonOk = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinConfirm::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinConfirm::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	JpVector2f vLabelPos = _vAbsolutePos;
	vLabelPos.Y += 50.0f * SoUtils::GetiPadRatioY();
	_pLabel->SetPosition( vLabelPos );
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinConfirm::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonCancel )
			{
				HandleWindowExit();
			}
			else if ( pMessage->GetSendersGuid() == _gButtonOk )
			{
				HandleConfirm();
			}

			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_IN:
		{
			if 
			( 
				GetParentGui() == false ||
				pMessage->GetSendersGuid() == GetParentGui()->GetGuid() 
			)
			{
				_pLabel->setVisible( true );

				SoGuiAnimWin::HandleMessage( pMessage );
			}
			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			if 
			( 
				GetParentGui() == false || 
				pMessage->GetSendersGuid() == GetParentGui()->GetGuid() 
			)
			{
				_pLabel->setVisible( false );

				SoGuiAnimWin::HandleMessage( pMessage );
			}
			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}




/******************************************************************************
 FUNCTION:  HandleWindowExit
******************************************************************************/
void SoGuiAnimWinConfirm::HandleWindowExit()
{
	// This confirm window can be used by gui's or by non-guis.  Therefore we support two
	// messsage passing code-paths.
	// mdomowicz 2015_12_28

	SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
	GuiMessageForListener.AddHash( JpHash( "WinConfirm_CancelButtonPressed" ) );
	SendMessageToListeners( &GuiMessageForListener );

	if ( GetParentGui() )
	{
		SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
		GuiMessageForParent.AddHash( JpHash( "WinConfirm_CancelButtonPressed" ) );
		SendMessageToParent( &GuiMessageForParent );
	}
}



/******************************************************************************
 FUNCTION:  HandleConfirm
******************************************************************************/
void SoGuiAnimWinConfirm::HandleConfirm()
{
	// This confirm window can be used by gui's or by non-guis.  Therefore we support two
	// messsage passing code-paths.
	// mdomowicz 2015_12_28

	SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
	GuiMessageForListener.AddHash( JpHash( "WinConfirm_OkButtonPressed" ) );
	SendMessageToListeners( &GuiMessageForListener );

	if ( GetParentGui() )
	{
		SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
		GuiMessageForParent.AddHash( JpHash( "WinConfirm_OkButtonPressed" ) );
		SendMessageToParent( &GuiMessageForParent );
	}
}



/******************************************************************************
 FUNCTION:  SoGuiAnimWinConfirm
******************************************************************************/
void SoGuiAnimWinConfirm::SetWindowText( const std::string& s )
{
	if ( _pLabel )
	{
		_pLabel->setString( s.c_str() );
	}
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoGuiAnimWinConfirm::OnKeypadBackClicked()
{
	if ( IsShowing() )
	{
		HandleWindowExit();
	}

	return true;
}
