/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinCoreMain.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


bool SoGuiAnimWinCoreMain::_bIsCoreMainWindowOpen = false;



/******************************************************************************
 FUNCTION:  SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinCoreMain::SoGuiAnimWinCoreMain() :
_gButtonOptions( 0 ),
_gButtonRules( 0 ),
_gButtonQuitGame( 0 ),
_gButtonCloseWindow( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinCoreMain::~SoGuiAnimWinCoreMain()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinCoreMain::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgLarge.png" ) );

	SetSpriteAlpha( 0.85f );
	//SetPositionToCenterOfParent();
	//SetWidth( 600.0f );
	//SetHeight( 100.0f );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	



	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonCloseWindowUp.png" );
	ButtonParams._vRelativePos.Init( 340.0f * SoUtils::GetiPadRatioX(), 230.0f * SoUtils::GetiPadRatioY() );
	_gButtonCloseWindow = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonOptionsUp.png" );
	ButtonParams._vRelativePos.Init( 0.0f * SoUtils::GetiPadRatioX(), 105.0f * SoUtils::GetiPadRatioY() );
	_gButtonOptions = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonRuleBookUp.png" );
	ButtonParams._vRelativePos.Init( 0.0f * SoUtils::GetiPadRatioX(), 30.0f * SoUtils::GetiPadRatioY() );
	_gButtonRules = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonQuitGameUp.png" );
	ButtonParams._vRelativePos.Init( 0.0f * SoUtils::GetiPadRatioX(), -125.0f * SoUtils::GetiPadRatioY() );
	_gButtonQuitGame = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	_bIsCoreMainWindowOpen = false;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinCoreMain::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinCoreMain::Update( float dt )
{
	SoGuiAnimWin::Update( dt );
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinCoreMain::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonQuitGame )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
				GuiMessageForListener.AddHash( JpHash( "QuitGameButtonPressed" ) );
				SendMessageToParent( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonOptions )
			{
				SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
				GuiMessageForParent.AddHash( JpHash( "OptionsButtonPressed" ) );
				SendMessageToParent( &GuiMessageForParent );
			}

			if ( pMessage->GetSendersGuid() == _gButtonRules )
			{
				SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
				GuiMessageForParent.AddHash( JpHash( "RulesButtonPressed" ) );
				SendMessageToParent( &GuiMessageForParent );
			}

			if ( pMessage->GetSendersGuid() == _gButtonCloseWindow )
			{
				SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
				GuiMessageForParent.AddHash( JpHash( "CloseWindowButtonPressed" ) );
				SendMessageToParent( &GuiMessageForParent );
			}

			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_IN:
		{
			if ( pMessage->GetSendersGuid() == GetParentGui()->GetGuid() )
			{
				SoGuiAnimWin::HandleMessage( pMessage );
				_bIsCoreMainWindowOpen = true;
			}
			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			if ( pMessage->GetSendersGuid() == GetParentGui()->GetGuid() )
			{
				SoGuiAnimWin::HandleMessage( pMessage );
				_bIsCoreMainWindowOpen = false;
			}
			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoGuiAnimWinCoreMain::OnKeypadBackClicked()
{
	if ( IsShowing() )
	{
		SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
		GuiMessageForParent.AddHash( JpHash( "CloseWindowButtonPressed" ) );
		SendMessageToParent( &GuiMessageForParent );
	}

	return true;
}
