/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFsmStateMainCore.h"
#include "SoGameGlobals.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinConfirm.h"
#include "SoGuiAnimWinCoreMain.h"
#include "SoGuiAnimWinCoreMainLauncher.h"
#include "SoGuiAnimWinLog.h"
#include "SoGuiAnimWinOptions.h"
#include "SoGuiAnimWinRules.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiAnimWinCoreMainLauncher
******************************************************************************/
SoGuiAnimWinCoreMainLauncher::SoGuiAnimWinCoreMainLauncher() :
#if GUI_CHAT_ENABLED
	_pGuiChat( NULL ),
#endif
_gButtonQuestionMark( 0 ),
_gWindowCoreMain( 0 ),
_gGuiOptions( 0 ),
_gGuiRules( 0 ),
_gButtonViewLog( 0 ),
_bKeypadBackButtonPressed( false ),
_bQuestionMarkButtonPressed( false ),
_bOptionsButtonPressed( false ),
_bRulesButtonPressed( false ),
_bCloseWindowButtonPressed( false ),
_bBackButtonPressed( false ),
_bViewLogButtonPressed( false ),
_bQuitGameButtonPressed( false ),
_bCancelButtonPressed( false ),
_bOkButtonPressed( false ),
_iCurrentState( 0 ),
_pWinComfirm( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimWinCoreMainLauncher
******************************************************************************/
SoGuiAnimWinCoreMainLauncher::~SoGuiAnimWinCoreMainLauncher()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinCoreMainLauncher::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	SetRelativePos( JpVector2f( 0.0f * SoUtils::GetiPadRatioX(), 0.0f  * SoUtils::GetiPadRatioY() ) );
	SetWidth( 100.0f );
	SetHeight( 100.0f );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	

	#if GUI_CHAT_ENABLED
		_pGuiChat = new SoGuiChat;
		pGuiManager->AddGui( _pGuiChat, this );
		_pGuiChat->SetRelativePos( JpVector2f( 0.0f, 350.0f * SoUtils::GetiPadRatioY() ) );
	#endif

	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonQuestionMarkUp.png" );
	ButtonParams._vRelativePos.Init( 468.0f * SoUtils::GetTrueiPadRatioX(), 341.0f * SoUtils::GetiPadRatioY() );
	_gButtonQuestionMark = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonViewLogUp.png" );
	ButtonParams._vRelativePos.Init( 468.0f * SoUtils::GetTrueiPadRatioX(), 271.0f * SoUtils::GetiPadRatioY() );
	_gButtonViewLog = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	SoGuiAnimWinCoreMain* pNewWindow = new SoGuiAnimWinCoreMain;
	_gWindowCoreMain = pGuiManager->AddGui( pNewWindow, this );
	pNewWindow->SetRelativePos( JpVector2f( 0.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() ) );

	SoGuiAnimWinOptions* pNewWindow2 = new SoGuiAnimWinOptions;
	_gGuiOptions = pGuiManager->AddGui( pNewWindow2, this );
	pNewWindow2->SetRelativePos( JpVector2f( 0.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() ) );

	SoGuiAnimWinRules* pNewWindow3 = new SoGuiAnimWinRules;
	_gGuiRules = pGuiManager->AddGui( pNewWindow3, this );
	pNewWindow3->SetRelativePos( JpVector2f( 0.0f * SoUtils::GetiPadRatioX(), 50.0f * SoUtils::GetiPadRatioY() ) );

	SoGuiAnimWinLog* pNewWindow4 = new SoGuiAnimWinLog;
	_gWindowLog = pGuiManager->AddGui( pNewWindow4, this );
	pNewWindow4->SetRelativePos( JpVector2f( 0.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() ) );
	SoGameGlobals::SetGuiAnimWinLog( pNewWindow4 );

	_pWinComfirm = new SoGuiAnimWinConfirm;
	_gGuiConfirmQuit = pGuiManager->AddGui( _pWinComfirm, this );
	_pWinComfirm->SetRelativePos( JpVector2f( 0.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() ) );


	_bViewLogButtonPressed = false;
	_bKeypadBackButtonPressed = false;
	_bQuestionMarkButtonPressed = false;
	_bOptionsButtonPressed = false;
	_bRulesButtonPressed = false;
	_bCloseWindowButtonPressed = false;
	_bBackButtonPressed = false;
	_bQuitGameButtonPressed = false;
	_bCancelButtonPressed = false;
	_bOkButtonPressed = false;
	_iCurrentState = 0;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinCoreMainLauncher::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinCoreMainLauncher::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	const int STATE_RESET = 0;
	const int STATE_CLOSED = 5;
	const int STATE_MAIN_MENU = 10;
	const int STATE_OPTIONS_MENU = 20;
	const int STATE_RULES = 30;
	const int STATE_CONFIRM_QUIT = 40;
	const int STATE_LOG = 50;

	switch ( _iCurrentState )
	{
		case STATE_RESET:
		{
			SoGuiAnimButton* pQuestionMarkButton = _pGuiManager->GetGuiAnimButtonByGuid( _gButtonQuestionMark );
			SoGuiAnimButton* pViewLogButton = _pGuiManager->GetGuiAnimButtonByGuid( _gButtonViewLog );
			pQuestionMarkButton->Enable( true );
			pViewLogButton->Enable( true );

			_iCurrentState = STATE_CLOSED;

			break;
		}
		case STATE_CLOSED:
		{
			if ( _bQuestionMarkButtonPressed || _bKeypadBackButtonPressed )
			{
				SoGuiAnimButton* pQuestionMarkButton = _pGuiManager->GetGuiAnimButtonByGuid( _gButtonQuestionMark );
				SoGuiAnimButton* pViewLogButton = _pGuiManager->GetGuiAnimButtonByGuid( _gButtonViewLog );
				pQuestionMarkButton->Enable( false );
				pViewLogButton->Enable( false );

				_bKeypadBackButtonPressed = false;
				_bQuestionMarkButtonPressed = false;
				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_TRANSITION_IN, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowCoreMain, &GuiMessage );
				_iCurrentState = STATE_MAIN_MENU;
			}
			if ( _bViewLogButtonPressed )
			{
				SoGuiAnimButton* pQuestionMarkButton = _pGuiManager->GetGuiAnimButtonByGuid( _gButtonQuestionMark );
				SoGuiAnimButton* pViewLogButton = _pGuiManager->GetGuiAnimButtonByGuid( _gButtonViewLog );
				pQuestionMarkButton->Enable( false );
				pViewLogButton->Enable( false );


				_bViewLogButtonPressed = false;
				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_TRANSITION_IN, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowLog, &GuiMessage );
				_iCurrentState = STATE_LOG;
			}

			break;
		}

		case STATE_MAIN_MENU:
		{
			if ( _bCloseWindowButtonPressed )
			{
				_bCloseWindowButtonPressed = false;
				_bQuestionMarkButtonPressed = false;
				_bKeypadBackButtonPressed = false;

				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_TRANSITION_OUT, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowCoreMain, &GuiMessage );

				_iCurrentState = STATE_RESET;
			}

			if ( _bOptionsButtonPressed )
			{
				_bOptionsButtonPressed = false;

				SoGuiMessage GuiMessageTransitionOut( SoGuiMessage::TYPE_TRANSITION_OUT, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowCoreMain, &GuiMessageTransitionOut );

				SoGuiMessage GuiMessageTransitionIn( SoGuiMessage::TYPE_TRANSITION_IN, GetGuid() );
				_pGuiManager->SendMessageToGui( _gGuiOptions, &GuiMessageTransitionIn );

				_iCurrentState = STATE_OPTIONS_MENU;
			}

			if ( _bRulesButtonPressed )
			{
				_bRulesButtonPressed = false;

				SoGuiMessage GuiMessageTransitionOut( SoGuiMessage::TYPE_TRANSITION_OUT, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowCoreMain, &GuiMessageTransitionOut );

				SoGuiMessage GuiMessageTransitionIn( SoGuiMessage::TYPE_TRANSITION_IN, GetGuid() );
				_pGuiManager->SendMessageToGui( _gGuiRules, &GuiMessageTransitionIn );

				_iCurrentState = STATE_RULES;
			}

			if ( _bQuitGameButtonPressed )
			{
				_bQuitGameButtonPressed = false;

				SoGuiMessage GuiMessageTransitionOut( SoGuiMessage::TYPE_TRANSITION_OUT, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowCoreMain, &GuiMessageTransitionOut );

				SoGuiMessage GuiMessageTransitionIn( SoGuiMessage::TYPE_TRANSITION_IN, GetGuid() );
				_pGuiManager->SendMessageToGui( _gGuiConfirmQuit, &GuiMessageTransitionIn );

				if ( SoFsmStateMainCore::GetGameConnectionType() == SoFsmStateMainCore::eGameConnectionType_Offline )
				{
					SoGameServer* pServer = SoGameGlobals::GetServer();
					SoGameState* pGame = pServer->GetGameState();
					bool bGameIsOver = ( pGame && pGame->GetNumWinners() > 0 );

					if ( bGameIsOver )
						_pWinComfirm->SetWindowText( "Quit game?" );
					else
						_pWinComfirm->SetWindowText( "Auto-save and quit game?" );
				}
				else
				{
					_pWinComfirm->SetWindowText( "Quit game?" );
				}

				_iCurrentState = STATE_CONFIRM_QUIT;
			}

			break;
		}

		case STATE_OPTIONS_MENU:
		{
			if ( _bBackButtonPressed )
			{
				_bBackButtonPressed = false;

				SoGuiMessage GuiMessageTransitionOut( SoGuiMessage::TYPE_TRANSITION_OUT, GetGuid() );
				_pGuiManager->SendMessageToGui( _gGuiOptions, &GuiMessageTransitionOut );

				SoGuiMessage GuiMessageTransitionIn( SoGuiMessage::TYPE_TRANSITION_IN, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowCoreMain, &GuiMessageTransitionIn );

				_iCurrentState = STATE_MAIN_MENU;
			}
			break;
		}

		case STATE_RULES:
		{
			if ( _bBackButtonPressed )
			{
				_bBackButtonPressed = false;

				SoGuiMessage GuiMessageTransitionOut( SoGuiMessage::TYPE_TRANSITION_OUT, GetGuid() );
				_pGuiManager->SendMessageToGui( _gGuiRules, &GuiMessageTransitionOut );

				SoGuiMessage GuiMessageTransitionIn( SoGuiMessage::TYPE_TRANSITION_IN, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowCoreMain, &GuiMessageTransitionIn );

				_iCurrentState = STATE_MAIN_MENU;
			}
			break;
		}

		case STATE_CONFIRM_QUIT:
		{
			if ( _bCancelButtonPressed )
			{
				_bCancelButtonPressed = false;

				SoGuiMessage GuiMessageTransitionOut( SoGuiMessage::TYPE_TRANSITION_OUT, GetGuid() );
				_pGuiManager->SendMessageToGui( _gGuiConfirmQuit, &GuiMessageTransitionOut );

				SoGuiMessage GuiMessageTransitionIn( SoGuiMessage::TYPE_TRANSITION_IN, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowCoreMain, &GuiMessageTransitionIn );

				_iCurrentState = STATE_MAIN_MENU;
			}
			else if ( _bOkButtonPressed )
			{
				_bOkButtonPressed = false;

				SoGuiMessage GuiMessageTransitionOut( SoGuiMessage::TYPE_TRANSITION_OUT, GetGuid() );
				_pGuiManager->SendMessageToGui( _gGuiConfirmQuit, &GuiMessageTransitionOut );

				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "WinCoreMainLauncher_QuitGameButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			break;
		}

		case STATE_LOG:
		{
			if ( _bCloseWindowButtonPressed )
			{
				_bCloseWindowButtonPressed = false;
				_bViewLogButtonPressed = false;

				SoGuiMessage GuiMessage( SoGuiMessage::TYPE_TRANSITION_OUT, GetGuid() );
				_pGuiManager->SendMessageToGui( _gWindowLog, &GuiMessage );

				_iCurrentState = STATE_RESET;
			}
			break;
		}

		default:
			ASSERTF( 0, ( "\nUnknown state." ) );
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinCoreMainLauncher::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonQuestionMark )
			{
				_bQuestionMarkButtonPressed = true;
			}

			if ( pMessage->GetSendersGuid() == _gButtonViewLog )
			{
				_bViewLogButtonPressed = true;
			}

			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}

		case SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT:
		{
			hash hData = pMessage->FirstHash();
			if ( hData == JpHash( "OptionsButtonPressed" ) )
				_bOptionsButtonPressed = true;

			if ( hData == JpHash( "RulesButtonPressed" ) )
				_bRulesButtonPressed = true;

			if ( hData == JpHash( "QuitGameButtonPressed" ) )
				_bQuitGameButtonPressed = true;

			if ( hData == JpHash( "CloseWindowButtonPressed" ) )
				_bCloseWindowButtonPressed = true;

			if ( hData == JpHash( "BackButtonPressed" ) )
				_bBackButtonPressed = true;

			if ( hData == JpHash( "WinConfirm_CancelButtonPressed" ) )
				_bCancelButtonPressed = true;

			if ( hData == JpHash( "WinConfirm_OkButtonPressed" ) )
				_bOkButtonPressed = true;

			break;
		}

		case SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER:
		{
			SendMessageToListeners( pMessage );
			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoGuiAnimWinCoreMainLauncher::OnKeypadBackClicked()
{
	if ( !SoGuiAnimWinLog::IsLogWindowOpen() )
		_bKeypadBackButtonPressed = true;

	return true;
}
