/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpString.h"

#include "SoFontDefs.h"
#include "SoGameServer.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimTextEntry.h"
#include "SoGuiAnimWinEnterGameName.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinEnterGameName::SoGuiAnimWinEnterGameName() :
_pLabel( NULL ),
_pLabelOnError( NULL ),
_bIsCreatingServer( false ),
_bIsJoiningServer( false ),
_pTextEntry( NULL ),
_gTextEntry( 0 ),
_gButtonBack( 0 ),
_gButtonNext( 0 ),
_gButtonClearText( 0 ),
_bNextPressed( false ),
_eState( STATE_LAST )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinEnterGameName::~SoGuiAnimWinEnterGameName()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinEnterGameName::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	_bIsCreatingServer = false;
	_bIsJoiningServer = false;

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgLarge.png" ) );

	SetSpriteAlpha( 0.85f );

	SetPositionToCenterOfParent();
	//SetWidth( 600.0f );
	//SetHeight( 100.0f );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	



	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;


	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonClearTextUp.png" );
	ButtonParams._vRelativePos.Init( 200.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	_gButtonClearText = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonBackUp.png" );
	ButtonParams._vRelativePos.Init( -200 * SoUtils::GetiPadRatioX(), -170 * SoUtils::GetiPadRatioY() );
	_gButtonBack = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonNextUp.png" );
	ButtonParams._vRelativePos.Init( 200 * SoUtils::GetiPadRatioX(), -170 * SoUtils::GetiPadRatioY() );
	_gButtonNext = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	std::string sStartingServerName = GenerateRandomServerName();

	_pTextEntry = new SoGuiAnimTextEntry;
	_pTextEntry->Setup( sStartingServerName.c_str(), SoFontDefs::GetGuiDefault() );
	_gTextEntry = _pGuiManager->AddGui( _pTextEntry, this );
	_pTextEntry->CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiTextEntryBg.png" ) );
	_pTextEntry->SetMaxCharacters( 20 );
	_pTextEntry->SetRelativePos( JpVector2f( 0.0f, 0.0f ) );
	_pTextEntry->SetSpriteAlpha( 0.75f );
	_pTextEntry->SetTextJustification( SoGuiAnimTextEntry::TEXT_JUSTIFICATION_CENTER );

	SoSprite* pTextEntryBgSprite = _pTextEntry->GetBackgroundSprite();
	pTextEntryBgSprite->setColor( ccc3( 0, 0, 0 ) );


	_pLabel = CreateBmFontLabel( "", SoFontDefs::GetGuiDefaultLarge() );
	_pLabel->setOpacity( 255 );
	_pLabel->setVisible( false );

	_pLabelOnError = CreateBmFontLabel( "", SoFontDefs::GetGuiDefault() );
	_pLabelOnError->setOpacity( 255 );
	_pLabelOnError->setVisible( false );


	_bNextPressed = false;
	_eState = STATE_WAIT;


	_CurlListener.SetOwner( this );
	SoCurl::GetInstance()->AddListener( &_CurlListener );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinEnterGameName::Deinit()
{
	SoGuiAnimWin::Deinit();

	SoCurl::GetInstance()->RemoveListener( &_CurlListener );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinEnterGameName::Update( float dt )
{
	SoGuiAnimWin::Update( dt );


	switch ( _eState )
	{
		case STATE_WAIT:
		{
			if ( _bNextPressed )
			{
				std::string sEnteredName = GetEnteredGameName();
				if ( sEnteredName != "" )
				{
					if ( _bIsCreatingServer )
						_eState = STATE_TEST_SERVER_EXISTS;
					else if ( _bIsJoiningServer )
						_eState = STATE_TEST_IS_SERVER_JOINABLE;
				}

				_pLabelOnError->setString( "" );
				_bNextPressed = false;
			}

			break;
		}

		case STATE_TEST_SERVER_EXISTS:
		{
			SoCurl* pCurl = SoCurl::GetInstance();

			if ( pCurl->IsReadyForNextQuery() )
			{
				hash hServerId = SoGameServer::GenerateServerIdFromString( GetEnteredGameName() );

				std::string sData = "Operation=GetIsServerNameTaken";
				sData += "&ServerId=";
				sData += JpString::HashToString( hServerId );

				pCurl->SetUrlToPhpFile( CURL_URL_MAIN_PHP );
				pCurl->DoGet( sData.c_str() );

				_sCurlResult = "";
				_eState = STATE_WAIT_FOR_TEST_SERVER_EXISTS_RESULT;
			}

			break;
		}
	
		case STATE_WAIT_FOR_TEST_SERVER_EXISTS_RESULT:
		{
			if ( _sCurlResult == "IsServerNameTaken=true" )
			{
				_pLabelOnError->setString( "That game name is taken.  Please specify another name." );
				_pLabelOnError->setVisible( true );
				_sCurlResult = "";
				_eState = STATE_WAIT;
			}
			else if ( _sCurlResult == "IsServerNameTaken=false" )
			{
				// Only after we've verified that the server name is open, do we
				// Tell the gui listeners that the Next button was pressed.
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "GameNameNextButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );

				_sCurlResult = "";
				_eState = STATE_WAIT;
			}
			break;
		}

		case STATE_TEST_IS_SERVER_JOINABLE:
		{
			SoCurl* pCurl = SoCurl::GetInstance();

			if ( pCurl->IsReadyForNextQuery() )
			{
				hash hServerId = SoGameServer::GenerateServerIdFromString( GetEnteredGameName() );

				std::string sData = "Operation=GetServerStatus";
				sData += "&ServerId=";
				sData += JpString::HashToString( hServerId );

				pCurl->SetUrlToPhpFile( CURL_URL_MAIN_PHP );
				pCurl->DoGet( sData.c_str() );

				_sCurlResult = "";
				_eState = STATE_WAIT_FOR_TEST_IS_SERVER_JOINABLE_RESULT;
			}

			break;
		}

		case STATE_WAIT_FOR_TEST_IS_SERVER_JOINABLE_RESULT:
		{
			if ( _sCurlResult == "ServerStatus=noserver" )
			{
				_pLabelOnError->setString( "That game does not exist.  Please specify another name." );
				_pLabelOnError->setVisible( true );
				_sCurlResult = "";
				_eState = STATE_WAIT;
			}
			else if ( _sCurlResult == "ServerStatus=game_started" )
			{
				_pLabelOnError->setString( "That game is full.  Please specify another name." );
				_pLabelOnError->setVisible( true );
				_sCurlResult = "";
				_eState = STATE_WAIT;
			}
			else if ( _sCurlResult == "ServerStatus=lobby" )
			{
				// Only after we've verified that the server exists, and is in a lobby state, do we
				// Tell the gui listeners that the Next button was pressed.
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "GameNameNextButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );

				_sCurlResult = "";
				_eState = STATE_WAIT;
			}

			break;
		}

		default:
			ASSERTF( 0, ( "\nUnknown state" ) )
	}



	JpVector2f vLabelPos = _vAbsolutePos;
	vLabelPos.X -= ( _pLabel->GetCalculatedWidthAllRes() / 2.0f );
	vLabelPos.Y += 100.0f * SoUtils::GetiPadRatioY();
	_pLabel->SetPosition( vLabelPos );

	vLabelPos = _vAbsolutePos;
	vLabelPos.X -= ( _pLabelOnError->GetCalculatedWidthAllRes() / 2.0f );
	vLabelPos.Y += 40.0f * SoUtils::GetiPadRatioY();
	_pLabelOnError->SetPosition( vLabelPos );


	if ( _bIsCreatingServer )
		_pLabel->setString( "Enter the name of the game to create." );
	else if ( _bIsJoiningServer )
		_pLabel->setString( "Enter the name of the game to join." );

}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinEnterGameName::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_TRANSITION_IN:
		{
			_pLabel->setVisible( true );

			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			_pLabel->setVisible( false );

			_pLabelOnError->setString( "" );
			_pLabelOnError->setVisible( false );

			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonClearText )
			{
				_pTextEntry->ClearContentText();
			}

			if ( pMessage->GetSendersGuid() == _gButtonBack )
			{
				_bIsCreatingServer = false;
				_bIsJoiningServer = false;

				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "GameNameBackButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonNext )
			{
				// We don't here immediately signal to Gui Listeners that Next was pressed.
				// Instead we signal to Gui Listeners in the Update, and only after the entered 
				// server name has been validated.
				_bNextPressed = true;
			}

			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}
		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  SetIsCreatingServer
******************************************************************************/
void SoGuiAnimWinEnterGameName::SetIsCreatingServer( bool b )
{
	_bIsCreatingServer = b; 

	if ( _bIsCreatingServer )
	{
		if ( _pTextEntry )
		{
			std::string sStartingServerName = GenerateRandomServerName();		
			_pTextEntry->SetInitialText( sStartingServerName.c_str() );
			_pTextEntry->SetContentTextToInitialText();
		}
	}
}



/******************************************************************************
 FUNCTION:  SetIsJoiningServer
******************************************************************************/
void SoGuiAnimWinEnterGameName::SetIsJoiningServer( bool b )
{
	_bIsJoiningServer = b; 

	if ( _bIsJoiningServer )
	{
		if ( _pTextEntry )
		{
			_pTextEntry->SetInitialText( "" );
			_pTextEntry->SetContentTextToInitialText();
		}
	}
}



/******************************************************************************
 FUNCTION:  GetEnteredGameName
******************************************************************************/
const char* SoGuiAnimWinEnterGameName::GetEnteredGameName()
{
	if ( _pTextEntry )
	{
		return _pTextEntry->GetContentText();
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  GenerateRandomServerName
******************************************************************************/
std::string SoGuiAnimWinEnterGameName::GenerateRandomServerName()
{
	char acPool[] = 
	{
		'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 
		'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'
	};

	std::string sResult = "";

	for ( int i = 0; i < 8; i ++ )
	{
		int iIndex = JpRandom::G.GetInt() % 26;
		sResult += acPool[ iIndex ];
	}

	return sResult;
}



/******************************************************************************
 FUNCTION:  OnCurlDataReceived
******************************************************************************/
void SoGuiAnimWinEnterGameName::OnCurlDataReceived( const std::string& sData )
{
	_sCurlResult = sData;
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoGuiAnimWinEnterGameName::OnKeypadBackClicked()
{
	if ( IsShowing() )
	{
		_bIsCreatingServer = false;
		_bIsJoiningServer = false;

		SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
		GuiMessageForListener.AddHash( JpHash( "GameNameBackButtonPressed" ) );
		SendMessageToListeners( &GuiMessageForListener );
	}

	return true;
}
