/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoConsole.h"
#include "SoFontDefs.h"
#include "SoFsmStateMain.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinLog.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;

#define ANIM_WIN_LOG_TEXT_POS_X  -450.0f


bool SoGuiAnimWinLog::_bIsLogWindowOpen = false;


/******************************************************************************
 FUNCTION:  SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinLog::SoGuiAnimWinLog() :
_pLogTextClipNode( NULL ),
_gButtonCloseWindow( 0 ),
_iMaxMessagesBeforeFifoRemoval( -1 ),
_bIsVisible( false ),
_fLogX( 0.0f ),
_fLogY( 0.0f ),
_fSpeedMax( 50.0f ),
_fDrag( 0.9f ),
_vVelocity( 0.0f, 0.0f ),
_vUserOffset( 0.0f, 0.0f ),
_fTotalLogHeight( 0.0f )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinLog::~SoGuiAnimWinLog()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinLog::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgLarge.png" ) );

	SetSpriteAlpha( 0.85f );

	//SetPositionToCenterOfParent();
	SetWidth( 975.0f * SoUtils::GetTrueiPadRatioX() );
	//SetHeight( 100.0f );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	

	_fLogX = ANIM_WIN_LOG_TEXT_POS_X * SoUtils::GetTrueiPadRatioX();
	_fLogY = -225.0f * SoUtils::GetiPadRatioY();


	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonCloseWindowUp.png" );
	ButtonParams._vRelativePos.Init( 440.0f * SoUtils::GetTrueiPadRatioX(), 230.0f * SoUtils::GetiPadRatioY() );
	_gButtonCloseWindow = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	// Using the regular clip node is bad because it clips the entire window (ie
	// the sprite background and everything).  So we use our own clip node.
	// This is not at all ideal, and eventually we should fix the normal clipnode
	// to work in a more usable way. mdomowicz 2014_02_20
	_pLogTextClipNode = new SoClipNode;
	_pClipNode->addChild( _pLogTextClipNode );
	SoUtils::SetRenderZOrder( _pClipNode, _pLogTextClipNode, BASE_RENDER_ORDER_GUI + 1 );

	_pLogTextClipNode->EnableClipping( true );

	_bIsLogWindowOpen = false;


	// There is a performance problem when there are too many messages in the log.
	// So for now, I'm limiting how many messages the log can have.  A better
	// solution would be to keep all messages and just manage which ones
	// have associated renderable text (too much renderable text is what causes
	// the slowdown) more appropriately.  But that is for another time.
	// mdomowicz 2014_07_19
	SetMaxMessagesBeforeFifoRemoval( 200 );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinLog::Deinit()
{
	Clear();
	RemoveAndDestroyNode( _pClipNode, _pLogTextClipNode );
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinLog::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	const int iClipWindowWidth = 910.0f;
	const int iClipWindowHeight = 479.0f;

	_pLogTextClipNode->SetClippingRegion
	( 
		_vAbsolutePos.X + ( ( ANIM_WIN_LOG_TEXT_POS_X ) * SoUtils::GetTrueiPadRatioX() ) - ( 8.0f * SoUtils::GetTrueiPadRatioX() ),
		_vAbsolutePos.Y - ( 238.0f ) * SoUtils::GetiPadRatioY(),
		iClipWindowWidth * SoUtils::GetTrueiPadRatioX(), 
		iClipWindowHeight * SoUtils::GetiPadRatioY() 
	);

	float fYMinClamp = 0.0f;
	if ( _fTotalLogHeight > iClipWindowHeight * SoUtils::GetiPadRatioY() )
	{
		fYMinClamp = -( _fTotalLogHeight - iClipWindowHeight * SoUtils::GetiPadRatioY() );
		fYMinClamp -= 15.0f * SoUtils::GetiPadRatioY(); // fudge factor.
	}

	_vUserOffset += _vVelocity;
	_vVelocity -= _vVelocity * ( _fDrag * dt );

	_vUserOffset.X = Clamp( _vUserOffset.X, 0.0f, 0.0f );
	_vUserOffset.Y = Clamp( _vUserOffset.Y, fYMinClamp, 0.0f * SoUtils::GetiPadRatioY() );


	if ( _bIsVisible )
	{
		_fTotalLogHeight = 0.0f;

		SoLogMessageList::Iterator Iter( &_LogMessageList );
		while( Iter.Prev() )
		{
			SoLogMessage* pMessage = Iter.GetNodeData();

			pMessage->Update( dt );

			float fX = _vAbsolutePos.X + _fLogX;
			float fY = _vAbsolutePos.Y + _fLogY + _fTotalLogHeight + _vUserOffset.Y;

			pMessage->SetPosition( fX, fY );


			float fLineHeight = pMessage->GetBmFontLabel()->GetCalculatedHeight() * SoUtils::GetiPadRatioY();
			_fTotalLogHeight += fLineHeight;
		}
	}


	// If there is a limit to how many lines can be stored in the log,
	// remove any lines that are over that limit, in FIFO order.
	if ( _iMaxMessagesBeforeFifoRemoval > 0 )
	{
		while ( _LogMessageList.GetNumEntries() > 100 )
		{
			SoLogMessageList::Node* pFirst = _LogMessageList.GetFirst();
			SoLogMessage* pMessage = pFirst->GetData();
			delete pMessage;
			_LogMessageList.RemoveFirst();
		}
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinLog::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonCloseWindow )
			{
				SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
				GuiMessageForParent.AddHash( JpHash( "CloseWindowButtonPressed" ) );
				SendMessageToParent( &GuiMessageForParent );
			}

			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_IN:
		{
			if ( pMessage->GetSendersGuid() == GetParentGui()->GetGuid() )
			{
				_vUserOffset.Zero();
				_vVelocity.Zero();

				SoGuiAnimWin::HandleMessage( pMessage );
				SetVisible( true );

				_bIsLogWindowOpen = true;

				// Hide the hud log when the window log is open.
				SoHudLog* pLog = SoFsmMainLayer::GetSharedHudLog();
				pLog->SetVisible( false );
			}


			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			if ( pMessage->GetSendersGuid() == GetParentGui()->GetGuid() )
			{
				SoGuiAnimWin::HandleMessage( pMessage );
				SetVisible( false );

				_bIsLogWindowOpen = false;

				// Show the hud log when the window log is closed.
				SoHudLog* pLog = SoFsmMainLayer::GetSharedHudLog();
				pLog->SetVisible( true );
			}
			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  Clear
******************************************************************************/
void SoGuiAnimWinLog::Clear()
{
	SoLogMessageList::Iterator Iter( &_LogMessageList );
	while( Iter.Prev() )
	{
		SoLogMessage* pMessage = Iter.GetNodeData();
		_pLogTextClipNode->removeChild( pMessage->GetBmFontLabel(), DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		pMessage->Deinit();
	}

	_LogMessageList.DeleteAll();
}



/******************************************************************************
 FUNCTION:  AddMessage
******************************************************************************/
void SoGuiAnimWinLog::AddMessage( const std::string& sMessage, const ccColor3B& Color )
{
	if ( _pLogTextClipNode )
	{
		SoLogMessage* pMessage = new SoLogMessage;
		
		SoLabelBmFont* pLabel = pMessage->CreateBmFontLabel( sMessage, SoFontDefs::GetWinLog() );
		pLabel->setWidth( 915.0f * SoUtils::GetTrueiPadRatioX() );
		pLabel->setVisible( true );
		_pLogTextClipNode->addChild(pLabel, 1);
		SoUtils::SetRenderZOrder( _pLogTextClipNode, pLabel, BASE_RENDER_ORDER_LOG );

		pMessage->SetColor( Color );
		pMessage->SetMessageShowDuration( -1 );


		_LogMessageList.AddLast( pMessage );
	}
}



/******************************************************************************
 FUNCTION:  AddMessageF
******************************************************************************/
void SoGuiAnimWinLog::AddMessageF( const ccColor3B& Color, const char* pFormat, ... )
{
	if ( strlen( pFormat ) < 2048 )
	{
		char buffer[2048];
		va_list args;
		va_start (args, pFormat);
		vsprintf (buffer,pFormat, args);
		va_end (args);

		AddMessage( buffer, Color );
	}
}



/******************************************************************************
 FUNCTION:  SetPosition
******************************************************************************/
void SoGuiAnimWinLog::SetPosition
( 
	float fBottomLeftX, 
	float fBottomLeftY
)
{
	_fLogX = fBottomLeftX;
	_fLogY = fBottomLeftY;
}



/******************************************************************************
 FUNCTION:  SetVisible
******************************************************************************/
void SoGuiAnimWinLog::SetVisible( bool b )
{
	_bIsVisible = b;

	SoLogMessageList::Iterator Iter( &_LogMessageList );
	while( Iter.Prev() )
	{
		SoLogMessage* pMessage = Iter.GetNodeData();
		pMessage->GetBmFontLabel()->setVisible( _bIsVisible );
	}	
}



/******************************************************************************
 FUNCTION:  OnTouchesMove
******************************************************************************/
bool SoGuiAnimWinLog::OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) 
{
	if ( IsShowing() )
	{
		_vVelocity.Init( 0.0f, 0.0f );
		JpVector2f vDelta( 0.0f, 0.0f );

		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		if ( pNonConstTouchList->GetNumEntries() == 1 )
		{
			CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
			vDelta = SoInputTouch::TouchPosToScreenPos( pTouch0->locationInView() ) - SoInputTouch::TouchPosToScreenPos( pTouch0->previousLocationInView() );

			_vUserOffset.X += vDelta.X;
			_vUserOffset.Y += vDelta.Y;

			// This gets used on 'OnTouchesUp' to calculate a momentum
			_vTouchPosPrev = pTouch0->previousLocationInView();

			return true;
		}
	}

	// The input is considered "handled" no matter what, if this window is open.
	// This helps SoInputDispatcher, and SoGuiManager prevent input conflicts.
	return IsShowing();
}



/******************************************************************************
FUNCTION:  OnTouchesDown
******************************************************************************/
bool SoGuiAnimWinLog::OnTouchesDown( const SoCcTouchList* pSoCcTouchList )
{
	if ( IsShowing() )
	{
		_vTouchPosPrev.x = 0.0f;
		_vTouchPosPrev.y = 0.0f;

		return true;
	}

	// The input is considered "handled" no matter what, if this window is open.
	// This helps SoInputDispatcher, and SoGuiManager prevent input conflicts.
	return IsShowing();
}



/******************************************************************************
FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoGuiAnimWinLog::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{
	if ( IsShowing() )
	{
		JpVector2f vDelta( 0.0f, 0.0f );

		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		if ( pNonConstTouchList->GetNumEntries() > 0 )
		{
			if ( _vTouchPosPrev.x != 0.0f && _vTouchPosPrev.y != 0.0f )
			{
				CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
				vDelta = SoInputTouch::TouchPosToScreenPos( _vTouchPosPrev  ) - SoInputTouch::TouchPosToScreenPos( pTouch0->locationInView() );

				_vVelocity.X -= vDelta.X;
				_vVelocity.Y -= vDelta.Y;

				if ( Length( _vVelocity ) > _fSpeedMax )
				{
					_vVelocity.Normalize();

					_vVelocity *= _fSpeedMax;
				}
				if ( Length( _vVelocity ) < 5.0f )
					_vVelocity.Init( 0.0f, 0.0f );

				return true;
			}
		}
	}

	// The input is considered "handled" no matter what, if this window is open.
	// This helps SoInputDispatcher, and SoGuiManager prevent input conflicts.
	return IsShowing();
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoGuiAnimWinLog::OnKeypadBackClicked()
{
	if ( IsShowing() )
	{
		SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
		GuiMessageForParent.AddHash( JpHash( "CloseWindowButtonPressed" ) );
		SendMessageToParent( &GuiMessageForParent );
	}

	return true;
}






