/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoGuiAnimWin.h"
#include "SoHudLog.h"

USING_NS_CC;


class SoGuiAnimWinLog : public SoGuiAnimWin
{
	public:

		//-----------------------
		// public functions

		SoGuiAnimWinLog();

		virtual ~SoGuiAnimWinLog();

		virtual void Init( CCNode* pParentNode, SoGuiManager* pGuiManager );

		virtual void Deinit();

		virtual void Update( float dt );

		virtual void HandleMessage( SoGuiMessage* pMessage );

		void Clear();

		void AddMessage( const std::string& sMessage, const ccColor3B& Color );
		void AddMessageF( const ccColor3B& Color, const char* pFormat, ... );

		void SetPosition( float fBottomLeftX, float fBottomLeftY );

		// use a negative value for unlimited messages.
		void SetMaxMessagesBeforeFifoRemoval( int i ) { _iMaxMessagesBeforeFifoRemoval = i; }

		bool IsVisible() { return _bIsVisible; }
		void SetVisible( bool b );


		// This is a really hacky way for other menus to know if this window
		// is open.  mdomowicz 2014_02_24
		static bool IsLogWindowOpen() { return _bIsLogWindowOpen; }


		// If this window is showing, then all input is considered "handled".
		// This helps SoGuiManager and SoInputDispatcher control when input should/shouldn't be
		// sent to other systems.  mdomowicz 2013_11_03
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList );


		// SoInputKeypad callbacks
		virtual bool OnKeypadBackClicked();
		virtual bool OnKeypadMenuClicked() { return false; }


	protected:

		//-----------------------
		// protected variables
		SoClipNode* _pLogTextClipNode;

		guid _gButtonCloseWindow;

		SoLogMessageList _LogMessageList;

		int _iMaxMessagesBeforeFifoRemoval;

		bool _bIsVisible;

		float _fLogX;
		float _fLogY;

		float _fSpeedMax;
		float _fDrag;
		JpVector2f _vVelocity;
		JpVector2f _vUserOffset;

		float _fTotalLogHeight;

		CCPoint _vTouchPosPrev;

		static bool _bIsLogWindowOpen;
};




