/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActorVisiblePlayer.h"
#include "SoFontDefs.h"
#include "SoGameBitPlayer.h"
#include "SoGameState.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinMessageBox2.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiAnimWinMessageBox2
******************************************************************************/
SoGuiAnimWinMessageBox2::SoGuiAnimWinMessageBox2() :
_gButtonOk( 0 ),
_gButtonCancel( 0 ),
_iPlayerNumber( -1 ),
_pGame( NULL ),
_pLabel( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimWinMessageBox2
******************************************************************************/
SoGuiAnimWinMessageBox2::~SoGuiAnimWinMessageBox2()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinMessageBox2::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgMedium.png" ) );

	SetSpriteAlpha( 0.85f );
	//SetPositionToCenterOfParent();

	SetRelativePos( JpVector2f( 0.0f, -90.0f * SoUtils::GetiPadRatioY() ) );

	//SetWidth( 922.0f * SoUtils::GetiPadRatioX() );
	//SetHeight( 253.0f * SoUtils::GetiPadRatioY() );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	



	//-----------------------
	// create common ButtonParams
	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;


	//-----------------------
	// Add an "OK" button.
	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonOkUp.png" );
	ButtonParams._vRelativePos.Init( 0.0f * SoUtils::GetiPadRatioX(), -65.0f * SoUtils::GetiPadRatioY() );
	_gButtonOk = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	//-----------------------
	// Add an "cancel" button.
	//ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonCancelUp.png" );
	//ButtonParams._vRelativePos.Init( -50.0f * SoUtils::GetiPadRatioX(), -65.0f * SoUtils::GetiPadRatioY() );
	//_gButtonCancel = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinMessageBox2::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinMessageBox2::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	JpVector2f vOffset;
	SoGameBitPlayer* pPlayer = _pGame->GetPlayerByPlayerNumber( _iPlayerNumber );


	vOffset.X = 0.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = 50.0f * SoUtils::GetiPadRatioY();
	_pLabel->SetPosition( _vAbsolutePos + vOffset );

}



/******************************************************************************
 FUNCTION:  SetText
******************************************************************************/
void SoGuiAnimWinMessageBox2::SetText( const std::string& s )
{
	if ( _pLabel )
		RemoveAndDestroyLabel( _pLabel );

	_pLabel = CreateBmFontLabel( s, SoFontDefs::GetGuiDefault(), kCCTextAlignmentCenter );
	_pLabel->setWidth( 750.0f * SoUtils::GetiPadRatioX() );
	_pLabel->setOpacity( 0 );
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinMessageBox2::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonOk )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "MessageBox2WindowOk" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}
			else if ( _gButtonCancel == pMessage->GetSendersGuid() )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "MessageBox2WindowCancel" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}

