/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "NeConfig.h"

#include "SoConfigManager.h"
#include "SoFontDefs.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinOptions.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoSoundManager.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


bool SoGuiAnimWinOptions::_bIsOptionsWindowOpen = false;


/******************************************************************************
 FUNCTION:  SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinOptions::SoGuiAnimWinOptions() :
_gButtonBack( 0 ),
_gMusicVolumeSlider( 0 ),
_gSfxVolumeSlider( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinOptions::~SoGuiAnimWinOptions()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinOptions::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgLarge.png" ) );

	SetSpriteAlpha( 0.85f );

	SetPositionToCenterOfParent();

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	


	//-----------------------
	// Resolution Data
	CCSize MaxSize = CCSize( JpUtils::GetMaxFullScreenResolution().X, JpUtils::GetMaxFullScreenResolution().Y );
	CCSize CurSize = CCDirector::sharedDirector()->getWinSizeInPixels();
	_eMaximumResolution = VectorToResolutionType( JpVector2i( MaxSize.width, MaxSize.height ) );
	_eCurrentResolution = VectorToResolutionType( JpVector2i( CurSize.width, CurSize.height ) );


	//-----------------------
	// Back Button
	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonBackUp.png" );
	ButtonParams._vRelativePos.Init( 0.0f * SoUtils::GetiPadRatioX(), -190.0f * SoUtils::GetiPadRatioY() );
	_gButtonBack = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	//-----------------------
	// Music Volume Slider
	SoGuiAnimSlider::SetupParams SliderParams;
	SliderParams._sBackgroundFilename = "GuiSliderBackground.png";
	SliderParams._vRelativePos.Init( 0.0f, 160.0f * SoUtils::GetiPadRatioY() );
	SliderParams._fInitialValue = SoSoundManager::GetInstance()->GetVolumeMusic();
	SliderParams._fLength = 256.0f;
	SliderParams._sLabel = "Music Volume";
	SliderParams._vLabelPosition.Init( 0.0f, 55.0f * SoUtils::GetiPadRatioY() );

	SliderParams._ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	SliderParams._ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;
	SliderParams._ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiSliderBar.png" );

	_gMusicVolumeSlider = _pGuiManager->CreateAndAddAnimSlider( SliderParams, this );


	//-----------------------
	// Sfx Volume Slider
	SliderParams._vRelativePos.Init( 0.0f, 30.0f * SoUtils::GetiPadRatioY() );
	SliderParams._fInitialValue = SoSoundManager::GetInstance()->GetVolumeEffects();
	SliderParams._sLabel = "Sound Effects Volume";
	SliderParams._vLabelPosition.Init( 0.0f, 55.0f * SoUtils::GetiPadRatioY() );

	SliderParams._ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	SliderParams._ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;
	SliderParams._ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiSliderBar.png" );

	_gSfxVolumeSlider = _pGuiManager->CreateAndAddAnimSlider( SliderParams, this );


	//-----------------------
	// Resolution Slider
	SliderParams._vRelativePos.Init( 0.0f, -100.0f * SoUtils::GetiPadRatioY() );
	SliderParams._fInitialValue = ResolutionTypeToSliderValue( _eCurrentResolution, _eMaximumResolution );
	SliderParams._sLabel = "Display Resolution";
	SliderParams._vLabelPosition.Init( 0.0f, 55.0f * SoUtils::GetiPadRatioY() );

	SliderParams._ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	SliderParams._ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;
	SliderParams._ButtonParams._eButtonDisabledImageOption = SoGuiAnimButton::BUTTON_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY;
	SliderParams._ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiSliderBar.png" );

	SliderParams._bIsDisabled = true;

	#if DCG_IS_TARGET_WIN32
		SliderParams._bIsDisabled = false;
	#endif

	_gResolutionSlider = _pGuiManager->CreateAndAddAnimSlider( SliderParams, this );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinOptions::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinOptions::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	float fMusicVolume = _pGuiManager->GetGuiAnimSliderByGuid( _gMusicVolumeSlider )->GetValue();
	SoSoundManager::GetInstance()->SetVolumeMusic( fMusicVolume );

	float fSfxVolume = _pGuiManager->GetGuiAnimSliderByGuid( _gSfxVolumeSlider )->GetValue();
	SoSoundManager::GetInstance()->SetVolumeEffects( fSfxVolume );

	#if DCG_IS_TARGET_WIN32
		SoGuiAnimSlider* pResolutionSliderGui= _pGuiManager->GetGuiAnimSliderByGuid( _gResolutionSlider );
		float fResolutionSilderValue = pResolutionSliderGui->GetValue();
		_eDesiredResolution = SliderValueToResolutionType( fResolutionSilderValue, _eMaximumResolution );
		JpVector2i vDesiredResolution = ResolutionTypeToVector( _eDesiredResolution );


		char acText[ 64 ];
		if ( _eCurrentResolution == _eDesiredResolution )
			sprintf( acText, "Display Size (%i x %i)", vDesiredResolution.X, vDesiredResolution.Y );
		else
			sprintf( acText, "Display Size (%i x %i after reset)", vDesiredResolution.X, vDesiredResolution.Y );


		pResolutionSliderGui->GetAsAnimSlider()->SetLabelText( std::string( acText ) );
	#endif
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinOptions::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonBack )
			{
				HandleWindowExit();
			}

			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_IN:
		{
			if 
			( 
				GetParentGui() == false ||
				pMessage->GetSendersGuid() == GetParentGui()->GetGuid() 
			)
			{
				SoGuiAnimWin::HandleMessage( pMessage );
				_bIsOptionsWindowOpen = true;
			}
			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			if 
			( 
				GetParentGui() == false || 
				pMessage->GetSendersGuid() == GetParentGui()->GetGuid() 
			)
			{
				SoGuiAnimWin::HandleMessage( pMessage );
				_bIsOptionsWindowOpen = false;
			}
			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  SaveSettingsToDisk
 NOTES:     Save the settings to disk, so that they are reloaded on startup.
******************************************************************************/
void SoGuiAnimWinOptions::SaveSettingsToDisk()
{
	NeConfig* pConfig = SoConfigManager::GetInstance()->GetEndUserSettings();

	NeConfig::Line* pLine = pConfig->FindOrCreateLine( "fVolumeMusic" );
	pConfig->ClearLineKeepingFirstItem( pLine );
	pConfig->AppendFloatToLine( pLine, SoSoundManager::GetInstance()->GetVolumeMusic() );

	pLine = pConfig->FindOrCreateLine( "fVolumeEffects" );
	pConfig->ClearLineKeepingFirstItem( pLine );
	pConfig->AppendFloatToLine( pLine, SoSoundManager::GetInstance()->GetVolumeEffects() );

	#if DCG_IS_TARGET_WIN32
		pLine = pConfig->FindOrCreateLine( "iResolution" );
		pConfig->ClearLineKeepingFirstItem( pLine );
		pConfig->AppendIntToLine( pLine, ResolutionTypeToVector( _eDesiredResolution ).X );
		pConfig->AppendIntToLine( pLine, ResolutionTypeToVector( _eDesiredResolution ).Y );
	#endif

	SoConfigManager::GetInstance()->WriteEndUserSettingsToDisk();
}



/******************************************************************************
 FUNCTION:  ResolutionTypeToVector
******************************************************************************/
JpVector2i SoGuiAnimWinOptions::ResolutionTypeToVector( RESOLUTION_TYPE eType )
{
	switch ( eType )
	{
		case RESOLUTION_TYPE_480x320:   return JpVector2i(  480, 320  );
		case RESOLUTION_TYPE_640x480:   return JpVector2i(  640, 480  );
		case RESOLUTION_TYPE_800x600:   return JpVector2i(  800, 600  );
		case RESOLUTION_TYPE_1024x768:  return JpVector2i( 1024, 768  );
		case RESOLUTION_TYPE_1152x648:  return JpVector2i( 1152, 648  );
		case RESOLUTION_TYPE_1152x864:  return JpVector2i( 1152, 864  );
		case RESOLUTION_TYPE_1280x720:  return JpVector2i( 1280, 720  );
		case RESOLUTION_TYPE_1280x768:  return JpVector2i( 1280, 768  );
		case RESOLUTION_TYPE_1280x800:  return JpVector2i( 1280, 800  );
		case RESOLUTION_TYPE_1280x960:  return JpVector2i( 1280, 960  );
		case RESOLUTION_TYPE_1280x1024: return JpVector2i( 1280, 1024 );
		case RESOLUTION_TYPE_1600x900:  return JpVector2i( 1600, 900  );
		case RESOLUTION_TYPE_1600x1200: return JpVector2i( 1600, 1200 );
		case RESOLUTION_TYPE_1920x1080: return JpVector2i( 1920, 1080 );
		case RESOLUTION_TYPE_1920x1200: return JpVector2i( 1920, 1200 );
		case RESOLUTION_TYPE_2048x1536: return JpVector2i( 2048, 1536 );
		case RESOLUTION_TYPE_2560x1600: return JpVector2i( 2560, 1600 );
		case RESOLUTION_TYPE_3840x2160: return JpVector2i( 3840, 2160 );
		case RESOLUTION_TYPE_4096x3072: return JpVector2i( 4096, 3072 );
		case RESOLUTION_TYPE_4096x3112: return JpVector2i( 4096, 3112 );
		case RESOLUTION_TYPE_8192x4608: return JpVector2i( 8192, 4608 );
		case RESOLUTION_TYPE_LAST:  
		default:
			ASSERTF( 0, ("Error") );  break;
	}
	
	return JpVector2i( 0, 0 );
}



/******************************************************************************
 FUNCTION:  VectorToResolutionType
******************************************************************************/
SoGuiAnimWinOptions::RESOLUTION_TYPE SoGuiAnimWinOptions::VectorToResolutionType( JpVector2i vRes )
{	
	if ( vRes.X <= 480 )
		return RESOLUTION_TYPE_480x320;
	else if ( vRes.X <= 640 )
		return RESOLUTION_TYPE_640x480;
	else if ( vRes.X <= 800 )
		return RESOLUTION_TYPE_800x600;
	else if ( vRes.X <= 1024 )
		return RESOLUTION_TYPE_1024x768;
	else if ( vRes.X <= 1152 )
	{
		if ( vRes.Y <= 648 )
			return RESOLUTION_TYPE_1152x648;
		else if ( vRes.Y <= 864 )
			return RESOLUTION_TYPE_1152x864;
	}
	else if ( vRes.X <= 1280 )
	{
		if ( vRes.Y <= 720 )
			return RESOLUTION_TYPE_1280x720;
		else if ( vRes.Y <= 768 )
			return RESOLUTION_TYPE_1280x768;
		else if ( vRes.Y <= 800 )
			return RESOLUTION_TYPE_1280x800;
		else if ( vRes.Y <= 960 )
			return RESOLUTION_TYPE_1280x960;
		else if ( vRes.Y <= 1024 )
			return RESOLUTION_TYPE_1280x1024;
	}
	else if ( vRes.X <= 1600 )
	{
		if ( vRes.Y <= 900 )
			return RESOLUTION_TYPE_1600x900;
		else if ( vRes.Y <= 1200 )
			return RESOLUTION_TYPE_1600x1200;
	}
	else if ( vRes.X <= 1920 )
	{
		if ( vRes.Y <= 1080 )
			return RESOLUTION_TYPE_1920x1080;
		else if ( vRes.Y <= 1200 )
			return RESOLUTION_TYPE_1920x1200;
	}
	else if ( vRes.X <= 2048 )
		return RESOLUTION_TYPE_2048x1536;
	else if ( vRes.X <= 2560 )
		return RESOLUTION_TYPE_2560x1600;
	else if ( vRes.X <= 3840 )
		return RESOLUTION_TYPE_3840x2160;
	else if ( vRes.X <= 4096 )
	{
		if ( vRes.Y <= 3072 )
			return RESOLUTION_TYPE_4096x3072;
		else if ( vRes.Y <= 3112 )
			return RESOLUTION_TYPE_4096x3112;
	}
	else if ( vRes.X <= 8192 )
		return RESOLUTION_TYPE_8192x4608;
	
	return RESOLUTION_TYPE_LAST;
}



/******************************************************************************
 FUNCTION:  ResolutionTypeToString
******************************************************************************/
char* SoGuiAnimWinOptions::ResolutionTypeToString( RESOLUTION_TYPE eType )
{
	switch ( eType )
	{
		case RESOLUTION_TYPE_480x320:   return "480 x 320";
		case RESOLUTION_TYPE_640x480:   return "640 x 480";
		case RESOLUTION_TYPE_800x600:   return "800 x 600";
		case RESOLUTION_TYPE_1024x768:  return "1024 x 768";
		case RESOLUTION_TYPE_1152x648:  return "1152 x 648";
		case RESOLUTION_TYPE_1152x864:  return "1152 x 864";
		case RESOLUTION_TYPE_1280x720:  return "1280 x 720";
		case RESOLUTION_TYPE_1280x768:  return "1280 x 768";
		case RESOLUTION_TYPE_1280x800:  return "1280 x 800";
		case RESOLUTION_TYPE_1280x960:  return "1280 x 960";
		case RESOLUTION_TYPE_1280x1024: return "1280 x 1024";
		case RESOLUTION_TYPE_1600x900:  return "1600 x 900";
		case RESOLUTION_TYPE_1600x1200: return "1600 x 1200";
		case RESOLUTION_TYPE_1920x1080: return "1920 x 1080";
		case RESOLUTION_TYPE_1920x1200: return "1920 x 1200";
		case RESOLUTION_TYPE_2048x1536: return "2048 x 1536";
		case RESOLUTION_TYPE_2560x1600: return "2560 x 1600";
		case RESOLUTION_TYPE_3840x2160: return "3840 x 2160";
		case RESOLUTION_TYPE_4096x3072: return "4096 x 3072";
		case RESOLUTION_TYPE_4096x3112: return "4096 x 3112";
		case RESOLUTION_TYPE_8192x4608: return "8192 x 4608";
		case RESOLUTION_TYPE_LAST:  
		default:
			ASSERTF( 0, ("Error") );  break;
	}
	
	return "Error";
}



/******************************************************************************
 FUNCTION:  ResolutionTypeToVector
******************************************************************************/
SoGuiAnimWinOptions::RESOLUTION_TYPE SoGuiAnimWinOptions::SliderValueToResolutionType( float fSliderValue, RESOLUTION_TYPE eMaxResolutionAvailable )
{
	float fMaxRes = (float)(eMaxResolutionAvailable);

	float fClosest = fSliderValue * eMaxResolutionAvailable;

	int iClosest = (int)fClosest;

	return (RESOLUTION_TYPE)iClosest;
}



/******************************************************************************
 FUNCTION:  ResolutionTypeToSliderValue
******************************************************************************/
float SoGuiAnimWinOptions::ResolutionTypeToSliderValue( RESOLUTION_TYPE eCurrentResolution, RESOLUTION_TYPE eMaxResolutionAvailable )
{
	float fMaxRes = (float)(eMaxResolutionAvailable);

	if ( fMaxRes > 0.0f )
	{
		float fCurRes = (float)(eCurrentResolution);

		float fSlider = Clamp( fCurRes / fMaxRes, 0.0f, 1.0f );

		return fSlider;
	}

	return 0.0f;
}



/******************************************************************************
 FUNCTION:  HandleWindowExit
******************************************************************************/
void SoGuiAnimWinOptions::HandleWindowExit()
{
	// This options window is used by SoFsmStateShell as well as SoGuiAnimWinCoreMainLauncher.
	// Because SoFsmStateShell isn't a gui, it must use the listener system to get message back.
	// Because SoGuiAnimWinCoreMainLauncher is a gui, it uses the internal gui messaging system.
	// This is why there are two message passing methods used here.
	// mdomowicz 2013_10_03

	SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
	GuiMessageForListener.AddHash( JpHash( "BackButtonPressed" ) );
	SendMessageToListeners( &GuiMessageForListener );

	if ( GetParentGui() )
	{
		SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
		GuiMessageForParent.AddHash( JpHash( "BackButtonPressed" ) );
		SendMessageToParent( &GuiMessageForParent );
	}

	SaveSettingsToDisk();
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoGuiAnimWinOptions::OnKeypadBackClicked()
{
	if ( IsShowing() )
	{
		HandleWindowExit();
	}

	return true;
}
