/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoGuiAnimWin.h"

USING_NS_CC;


class SoGuiAnimWinOptions : public SoGuiAnimWin
{
	public:

		enum RESOLUTION_TYPE
		{
			RESOLUTION_TYPE_480x320,
			RESOLUTION_TYPE_640x480,
			RESOLUTION_TYPE_800x600,
			RESOLUTION_TYPE_1024x768,
			RESOLUTION_TYPE_1152x648,
			RESOLUTION_TYPE_1152x864,
			RESOLUTION_TYPE_1280x720,
			RESOLUTION_TYPE_1280x768,
			RESOLUTION_TYPE_1280x800,
			RESOLUTION_TYPE_1280x960,
			RESOLUTION_TYPE_1280x1024,
			RESOLUTION_TYPE_1600x900,
			RESOLUTION_TYPE_1600x1200,
			RESOLUTION_TYPE_1920x1080,
			RESOLUTION_TYPE_1920x1200,
			RESOLUTION_TYPE_2048x1536,
			RESOLUTION_TYPE_2560x1600,
			RESOLUTION_TYPE_3840x2160,  // 4k HDTV
			RESOLUTION_TYPE_4096x3072,  // 4k 4:3
			RESOLUTION_TYPE_4096x3112,  // 4k Full aperture
			RESOLUTION_TYPE_8192x4608,  // Made up 8k res for future-proofing, 16:9
			RESOLUTION_TYPE_LAST
		};

		//-----------------------
		// public functions

		SoGuiAnimWinOptions();

		virtual ~SoGuiAnimWinOptions();

		virtual void Init( CCNode* pParentNode, SoGuiManager* pGuiManager );

		virtual void Deinit();

		virtual void Update( float dt );

		virtual void HandleMessage( SoGuiMessage* pMessage );

		// This is a really hacky way for other menus to know if this window
		// is open.  mdomowicz 2014_02_24
		static bool IsOptionsWindowOpen() { return _bIsOptionsWindowOpen; }


		// If this window is showing, then all input is considered "handled".
		// This helps SoGuiManager and SoInputDispatcher control when input should/shouldn't be
		// sent to other systems.  mdomowicz 2013_11_03
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) { return IsShowing(); }
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList )  { return IsShowing(); }
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList ) { return IsShowing(); }


		// SoInputKeypad callbacks
		virtual bool OnKeypadBackClicked();
		virtual bool OnKeypadMenuClicked() { return false; }


		//-----------------------
		// Resolution Functions
		SoGuiAnimWinOptions::RESOLUTION_TYPE VectorToResolutionType( JpVector2i vRes );
		SoGuiAnimWinOptions::RESOLUTION_TYPE SliderValueToResolutionType( float fSliderValue, RESOLUTION_TYPE eMaxResolutionAvailable );

		float      ResolutionTypeToSliderValue( RESOLUTION_TYPE eCurrentResolution, RESOLUTION_TYPE eMaxResolutionAvailable );
		JpVector2i ResolutionTypeToVector( RESOLUTION_TYPE eType );
		char*      ResolutionTypeToString( RESOLUTION_TYPE eType );


	protected:

		//-----------------------
		// protected variables

		guid _gButtonBack;
		guid _gMusicVolumeSlider;
		guid _gSfxVolumeSlider;
		guid _gResolutionSlider;

		RESOLUTION_TYPE _eCurrentResolution;
		RESOLUTION_TYPE _eMaximumResolution;
		RESOLUTION_TYPE _eDesiredResolution;

		static bool _bIsOptionsWindowOpen;


		//-----------------------
		// protected functions
		void SaveSettingsToDisk();

		void HandleWindowExit();
};




