/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCamera.h"
#include "SoCameraGameplay.h"
#include "SoConsole.h"
#include "SoGameBitPlayer.h"
#include "SoGameClient.h"
#include "SoGameState.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinCoreMain.h"
#include "SoGuiAnimWinPlayerBar.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoHud.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiAnimWinPlayerBar
******************************************************************************/
SoGuiAnimWinPlayerBar::SoGuiAnimWinPlayerBar() :
_gButtonEndTurn( 0 ),
_gWindowCoreMain( 0 ),
_gButtonUndo( 0 ),
_gButtonResetTurn( 0 ),
_gButtonNegotiate( 0 ),
_gButtonBuildBomb( 0 ),
_gButtonBuyStock( 0 ),
_gButtonDeclareLastTurn( 0 ),
_gToggleViewTiles( 0 ),
_pPlayer( NULL ),
_iViewTilesToggleState( 0 ),
_gComboButtonViewSetting( 0 ),
_bIsViewSettingMin( true )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimWinPlayerBar
******************************************************************************/
SoGuiAnimWinPlayerBar::~SoGuiAnimWinPlayerBar()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinPlayerBar::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	SetRelativePos( JpVector2f( -462.0f * SoUtils::GetTrueiPadRatioX(), -334.0f  * SoUtils::GetiPadRatioY() ) );
	SetWidth( 100.0f );
	SetHeight( 100.0f );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	



	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;
	ButtonParams._eButtonDisabledImageOption = SoGuiAnimButton::BUTTON_DISABLED_IMAGE_OPTION_USE_IMAGE;

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonEndTurnUp.png" );
	ButtonParams._sPathAndFilenameButtonDisabled = SoUtils::AddRelativePathToTexture( "GuiButtonEndTurnDisabled.png" );
	ButtonParams._vRelativePos.Init( 0.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	_gButtonEndTurn = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );



	SoGuiAnimButton::SetupParams ButtonParams2;
	ButtonParams2._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams2._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;
	ButtonParams2._eButtonDisabledImageOption = SoGuiAnimButton::BUTTON_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY;

	//ButtonParams2._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonUndoUp.png" );
	//ButtonParams2._vRelativePos.Init( 76.0f * 1.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	//_gButtonUndo = _pGuiManager->CreateAndAddAnimButton( ButtonParams2, this );

	ButtonParams2._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonResetTurnUp.png" );
	ButtonParams2._vRelativePos.Init( 76.0f * 1.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	_gButtonResetTurn = _pGuiManager->CreateAndAddAnimButton( ButtonParams2, this );

	ButtonParams2._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonBuyStockUp.png" );
	ButtonParams2._vRelativePos.Init( 76.0f * 2.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	_gButtonBuyStock = _pGuiManager->CreateAndAddAnimButton( ButtonParams2, this );

	ButtonParams2._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonDeclareLastTurnUp.png" );
	ButtonParams2._vRelativePos.Init( 76.0f * 3.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	_gButtonDeclareLastTurn = _pGuiManager->CreateAndAddAnimButton( ButtonParams2, this );

	SoGuiAnimWinCoreMain* pNewWindow = new SoGuiAnimWinCoreMain;
	_gWindowCoreMain = pGuiManager->AddGui( pNewWindow, this );
	pNewWindow->SetRelativePos( JpVector2f( -468.0f * SoUtils::GetiPadRatioX(), -341.0f * SoUtils::GetiPadRatioY() ) );


	SoGuiAnimComboButton::SetupParams ComboButtonParams;
	ComboButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ComboButtonParams._eButtonDownImageOption = SoGuiAnimComboButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;
	ComboButtonParams._eButtonDisabledImageOption = SoGuiAnimComboButton::BUTTON_DISABLED_IMAGE_OPTION_PARTIAL_TRANSPARENCY;

	ComboButtonParams._vRelativePos.Init( 76.0f * 4.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	_gComboButtonViewSetting = _pGuiManager->CreateAndAddAnimComboButton( ComboButtonParams, this );
	SoGuiAnimComboButton* pComboButton1 = (SoGuiAnimComboButton*)_pGuiManager->GetGuiByGuid( _gComboButtonViewSetting );

	pComboButton1->SetupAndAddComboButtonValue( JpHash( "SwitchViewToMax" ), SoUtils::AddRelativePathToTexture( "GuiButtonSwitchViewToMax.png" ), "", "" );
	pComboButton1->SetupAndAddComboButtonValue( JpHash( "SwitchViewToMin" ), SoUtils::AddRelativePathToTexture( "GuiButtonSwitchViewToMin.png" ), "", "" );


	SoGuiAnimToggle::SetupParams ToggleParams;
	ToggleParams._hSoundEffectOnToggleOn = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ToggleParams._eToggleOnImageOption = SoGuiAnimToggle::TOGGLE_ON_IMAGE_OPTION_SCALE_THE_OFF_IMAGE_DOWN;

	ToggleParams._sPathAndFilenameToggleOff = SoUtils::AddRelativePathToTexture( "GuiButtonViewTilesOwnedUp.png" );
	ToggleParams._vRelativePos.Init( 76.0f * 5.0f * SoUtils::GetiPadRatioX(), 0.0f );

	_gToggleViewTiles = _pGuiManager->CreateAndAddAnimToggle( ToggleParams, this );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinPlayerBar::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinPlayerBar::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	SoGui* pGuiEndTurn = _pGuiManager->GetGuiByGuid( _gButtonEndTurn );
	SoGuiAnimButton* pButtonEndTurn = pGuiEndTurn->GetAsAnimButton();

	//SoGui* pGuiUndo = _pGuiManager->GetGuiByGuid( _gButtonUndo );
	//SoGuiAnimButton* pButtonUndo = pGuiUndo->GetAsAnimButton();

	SoGui* pGuiResetTurn = _pGuiManager->GetGuiByGuid( _gButtonResetTurn );
	SoGuiAnimButton* pButtonResetTurn = pGuiResetTurn->GetAsAnimButton();

	SoGui* pGuiBuyStock = _pGuiManager->GetGuiByGuid( _gButtonBuyStock );
	SoGuiAnimButton* pButtonBuyStock = pGuiBuyStock->GetAsAnimButton();

	SoGui* pGuiDeclareLastTurn = _pGuiManager->GetGuiByGuid( _gButtonDeclareLastTurn );
	SoGuiAnimButton* pButtonDeclareLastTurn = pGuiDeclareLastTurn->GetAsAnimButton();

	SoGuiAnimComboButton* pComboButtonViewSetting = (SoGuiAnimComboButton*)_pGuiManager->GetGuiByGuid( _gComboButtonViewSetting );

	SoGameState* pGame = _pPlayer->GetOwningGame();

	bool bPlayerHasPlacedTile = ( pGame->GetLastPlacedTileId() != TILE_UNDEFINED_ID );
	bool bStocksAreAvailableForSale = pGame->AreAnyStocksAvailableForSale();
	bool bIsFirstMove = _pPlayer->IsFirstMoveOfPlayersTurn();
	bool bIsInSelectActionState = ( _pPlayer->GetState() == SoGameBitPlayer::STATE_SELECT_ACTION );
	bool bUndoDataExists = pGame->UndoStackHasEnoughDataForUndo();

	pButtonEndTurn->Enable( bIsInSelectActionState && bPlayerHasPlacedTile );
	//pButtonUndo->Enable( bIsInSelectActionState && bUndoDataExists );
	pButtonResetTurn->Enable( bIsInSelectActionState && bUndoDataExists );

	pButtonBuyStock->Enable
	( 
		bIsInSelectActionState && 
		bStocksAreAvailableForSale && 
		_pPlayer->IsAllowedToPurchaseStock() &&
		_pPlayer->GetNumStocksPurchasedThisTurn() < 3
	);

	pButtonDeclareLastTurn->Enable( bIsInSelectActionState );

	pComboButtonViewSetting->Enable( bIsInSelectActionState );


	// For now, we animate the End Turn button
	// whenever it is active.  May want to nuance this later.
	pButtonEndTurn->SetIsAttractAnimEnabled( pButtonEndTurn->IsEnabled()  );


	UpdateViewToggle( dt );


}



/******************************************************************************
 FUNCTION:  UpdateViewToggle
******************************************************************************/
void SoGuiAnimWinPlayerBar::UpdateViewToggle( float dt )
{
	SoGameState* pGame = _pPlayer->GetOwningGame();
	SoGui* pGuiToggleViewTiles = _pGuiManager->GetGuiByGuid( _gToggleViewTiles );
	SoGuiAnimToggle* pToggleViewTiles = pGuiToggleViewTiles->GetAsAnimToggle();

	const int TOGGLE_STATE_ENABLED = 0;
	const int TOGGLE_STATE_DISABLED = 10;
	const int TOGGLE_STATE_DISABLED_FOR_MANAGING_STOCK = 20;

	switch ( _iViewTilesToggleState )
	{
		case TOGGLE_STATE_ENABLED:
		{
			if ( !DoesPlayerOwnAnyTiles() )
			{
				pToggleViewTiles->Enable( false );
				_iViewTilesToggleState = TOGGLE_STATE_DISABLED; 
			}

			if 
			( 
				pGame->GetPlayerNumberCommandedToManageOldStock() != PLAYER_UNDEFINED_PLAYER_NUMBER &&
				pGame->GetPlayerNumberCommandedToManageOldStock() != _pPlayer->GetPlayerNumber() 
			)
			{
				pToggleViewTiles->Enable( false );
				pToggleViewTiles->SetOff();
				_iViewTilesToggleState = TOGGLE_STATE_DISABLED_FOR_MANAGING_STOCK;
			}

			break;
		}

		case TOGGLE_STATE_DISABLED:
		{
			if ( DoesPlayerOwnAnyTiles() )
			{
				pToggleViewTiles->Enable( true );
				_iViewTilesToggleState = TOGGLE_STATE_ENABLED; 
			}

			break;
		}

		case TOGGLE_STATE_DISABLED_FOR_MANAGING_STOCK:
		{
			if 
			( 
				pGame->GetPlayerNumberCommandedToManageOldStock() == PLAYER_UNDEFINED_PLAYER_NUMBER ||
				pGame->GetPlayerNumberCommandedToManageOldStock() == _pPlayer->GetPlayerNumber() 
			)
			{
				pToggleViewTiles->Enable( true );
				_iViewTilesToggleState = TOGGLE_STATE_ENABLED; 
			}

			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinPlayerBar::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_TOGGLE_ON_ON:
		{
			if ( pMessage->GetSendersGuid() == _gToggleViewTiles )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "ViewTilesToggleOn" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}
			break;
		}
		case SoGuiMessage::TYPE_TOGGLE_ON_OFF:
		{
			if ( pMessage->GetSendersGuid() == _gToggleViewTiles )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "ViewTilesToggleOff" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonEndTurn )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "EndTurnButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonUndo )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "UndoButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonResetTurn )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "RestartTurnButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonBuyStock )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "BuyStockButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonDeclareLastTurn )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "DeclareLastTurnButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}

		case SoGuiMessage::TYPE_COMBO_BUTTON_ON_UP:
		{
			SoGuiAnimComboButton* pComboButtonViewSetting = (SoGuiAnimComboButton*)_pGuiManager->GetGuiByGuid( _gComboButtonViewSetting );

			if ( pComboButtonViewSetting )
			{
				SoComboButtonValue* pValue = pComboButtonViewSetting->GetPreviousComboButtonValue();

				_bIsViewSettingMin = ( pValue->GetValue() == JpHash( "SwitchViewToMin" ) );

				SwitchViewToDesiredStandardPosition( _bIsViewSettingMin );
			}

			break;
		}

		case SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER:
		{
			SendMessageToListeners( pMessage );
			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_IN:
		{
			TransitionIn();

			if ( _pPlayer->GetOwningGame()->GetOwningClient()->GetNumHumanPlayersOnThisClient() == 1 )
				SendMessageToAllChildrenExcept( pMessage, _gToggleViewTiles );
			else
				SendMessageToAllChildren( pMessage );

			break;
		}
		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			TransitionOut();

			if ( _pPlayer->GetOwningGame()->GetOwningClient()->GetNumHumanPlayersOnThisClient() == 1 )
				SendMessageToAllChildrenExcept( pMessage, _gToggleViewTiles );
			else
				SendMessageToAllChildren( pMessage );

			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  UntoggleViewTilesToggle
******************************************************************************/
void SoGuiAnimWinPlayerBar::UntoggleViewTilesToggle()
{
	SoGui* pGuiToggleViewTiles = _pGuiManager->GetGuiByGuid( _gToggleViewTiles );
	SoGuiAnimToggle* pToggleViewTiles = pGuiToggleViewTiles->GetAsAnimToggle();
	pToggleViewTiles->SetOff();
}



/******************************************************************************
 FUNCTION:  SendMessageToAllChildrenExcept
******************************************************************************/
void SoGuiAnimWinPlayerBar::SendMessageToAllChildrenExcept( SoGuiMessage* pMessage, guid gExceptThisGui )
{
	SoChildGuiGuidList::Iterator Iter( &_ChildGuiGuidList );
	while ( Iter.Next() )
	{
		guid gChild = (guid)Iter.GetNodeData();

		if ( gChild != gExceptThisGui )
			SendMessageToChild( gChild, pMessage );
	}
}



/******************************************************************************
 FUNCTION:  DoesPlayerOwnAnyTiles
******************************************************************************/
bool SoGuiAnimWinPlayerBar::DoesPlayerOwnAnyTiles()
{
	SoGameState* pGame = _pPlayer->GetOwningGame();
	SoTileSet* pTileSet = pGame->GetTileSet();
	
	SoGameBitTileArray* pAllTilesList = pTileSet->GetAllTiles();
	SoGameBitTileArray::Iterator Iter( pAllTilesList );
	while ( Iter.Next() )
	{
		SoGameBitTile* pTile = Iter.GetData();

		if ( pTile->GetOwnerPlayerNumber() == _pPlayer->GetPlayerNumber() )
		{
			return true;
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  SwitchViewToDesiredStandardPosition
******************************************************************************/
void SoGuiAnimWinPlayerBar::SwitchViewToDesiredStandardPosition( bool bIsViewSettingMin )
{
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
	SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();

	if ( pCameraGameplay )
	{
		if ( bIsViewSettingMin )
		{
			pCameraGameplay->SetStandardPosition( CAMERA_GAMEPLAY_STANDARD_POS_1 );
			pCameraGameplay->SetStandardZoom( CAMERA_GAMEPLAY_STANDARD_ZOOM_1 );
			pCameraGameplay->MoveToStandardPositionAndZoom();

			SoHud::GetInstance()->GetHotelStats()->SetStandardPosition
			(
				HOTEL_STATS_STANDARD_POS_1_X * SoUtils::GetTrueiPadRatioX(),
				HOTEL_STATS_STANDARD_POS_1_Y * SoUtils::GetiPadRatioY()
			);
			SoHud::GetInstance()->GetHotelStats()->MoveToStandardPostion();
		}
		else 
		{
			pCameraGameplay->SetStandardPosition( CAMERA_GAMEPLAY_STANDARD_POS_2 );
			pCameraGameplay->SetStandardZoom( CAMERA_GAMEPLAY_STANDARD_ZOOM_2 );
			pCameraGameplay->MoveToStandardPositionAndZoom();

			SoHud::GetInstance()->GetHotelStats()->SetStandardPosition
			(
				HOTEL_STATS_STANDARD_POS_2_X * SoUtils::GetTrueiPadRatioX(),
				HOTEL_STATS_STANDARD_POS_2_Y * SoUtils::GetiPadRatioY()
			);			
			SoHud::GetInstance()->GetHotelStats()->MoveToStandardPostion();
		}
	}
}
