/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActorVisiblePlayer.h"
#include "SoFontDefs.h"
#include "SoGameBitPlayer.h"
#include "SoGameState.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinPlayerDeclareLastTurn.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiAnimWinPlayerDeclareLastTurn
******************************************************************************/
SoGuiAnimWinPlayerDeclareLastTurn::SoGuiAnimWinPlayerDeclareLastTurn() :
_gButtonOk( 0 ),
_gButtonCancel( 0 ),
_iPlayerNumber( -1 ),
_pGame( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimWinPlayerDeclareLastTurn
******************************************************************************/
SoGuiAnimWinPlayerDeclareLastTurn::~SoGuiAnimWinPlayerDeclareLastTurn()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinPlayerDeclareLastTurn::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgMedium.png" ) );

	SetSpriteAlpha( 0.85f );

	//SetPositionToCenterOfParent();
	SetRelativePos( JpVector2f( 0.0f, -90.0f * SoUtils::GetiPadRatioY() ) );

	//SetWidth( 922.0f * SoUtils::GetiPadRatioX() );
	//SetHeight( 253.0f * SoUtils::GetiPadRatioY() );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	



	//-----------------------
	// create common ButtonParams
	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;


	//-----------------------
	// set the window text
	char acTemp[ 256 ] = "";
	if ( _pGame->CanDeclareLastTurn() )
	{
		std::string sPlayerName = SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( _iPlayerNumber );

		if ( _pGame->AreAllActiveHotelsSafe() && _pGame->IsAnyHotelLargeEnoughToEndGame() )
			sprintf( acTemp, "All active corporations are safe (larger than 10 tiles) and at least one corporation is larger than 40 tiles.\n%s, shall the game end at the end of your turn?", sPlayerName.c_str() );
		else if ( _pGame->AreAllActiveHotelsSafe() )
			sprintf( acTemp, "All active corporations are safe (larger than 10 tiles).\n%s, shall the game end at the end of your turn?", sPlayerName.c_str() );
		else if ( _pGame->IsAnyHotelLargeEnoughToEndGame() )
			sprintf( acTemp, "At least one corporation is larger than 40 tiles.\n%s, shall the game end at the end of your turn?", sPlayerName.c_str() );
	}
	else
	{
		sprintf( acTemp, "To make your current turn the last turn of the game, either all active corporations must be safe (larger than 10 tiles) or at least one corporation must be larger than 40 tiles.  Neither condition is met." );
	}


	_pLabel = CreateBmFontLabel( acTemp, SoFontDefs::GetGuiDefault(), kCCTextAlignmentCenter );
	_pLabel->setWidth( 750.0f * SoUtils::GetiPadRatioX() );
	_pLabel->setOpacity( 0 );



	if ( _pGame->CanDeclareLastTurn() )
	{
		//-----------------------
		// Add an "OK" button.
		ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonOkUp.png" );
		ButtonParams._vRelativePos.Init( 50.0f * SoUtils::GetiPadRatioX(), -65.0f * SoUtils::GetiPadRatioY() );
		_gButtonOk = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

		//-----------------------
		// Add an "cancel" button.
		ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonCancelUp.png" );
		ButtonParams._vRelativePos.Init( -50.0f * SoUtils::GetiPadRatioX(), -65.0f * SoUtils::GetiPadRatioY() );
		_gButtonCancel = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );
	}
	else
	{
		//-----------------------
		// Add an "OK" button.
		ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonOkUp.png" );
		ButtonParams._vRelativePos.Init( 0.0f * SoUtils::GetiPadRatioX(), -65.0f * SoUtils::GetiPadRatioY() );
		_gButtonOk = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinPlayerDeclareLastTurn::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinPlayerDeclareLastTurn::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	JpVector2f vOffset;
	SoGameBitPlayer* pPlayer = _pGame->GetPlayerByPlayerNumber( _iPlayerNumber );


	vOffset.X = 0.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = 50.0f * SoUtils::GetiPadRatioY();
	_pLabel->SetPosition( _vAbsolutePos + vOffset );

}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinPlayerDeclareLastTurn::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonOk )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );

				// If the player can declare the last turn, then send the OK result, normally.
				// But if he can't, then treat the OK as if it were CANCEL.
				if ( _pGame->CanDeclareLastTurn() )
					GuiMessageForListener.AddHash( JpHash( "DeclareLastTurnWindowResult" ) );
				else
					GuiMessageForListener.AddHash( JpHash( "DeclareLastTurnWindowCanceled" ) );

				SendMessageToListeners( &GuiMessageForListener );
			}
			else if ( _gButtonCancel == pMessage->GetSendersGuid() )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "DeclareLastTurnWindowCanceled" ) );
				
				SendMessageToListeners( &GuiMessageForListener );
			}

			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}

