/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActorVisibleHotel.h"
#include "SoActorVisiblePlayer.h"
#include "SoFontDefs.h"
#include "SoGameBitPlayer.h"
#include "SoGameState.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinCoreMain.h"
#include "SoGuiAnimWinLog.h"
#include "SoGuiAnimWinOptions.h"
#include "SoGuiAnimWinPlayerManageOldStock.h"
#include "SoGuiAnimWinRules.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiAnimWinPlayerManageOldStock
******************************************************************************/
SoGuiAnimWinPlayerManageOldStock::SoGuiAnimWinPlayerManageOldStock() :
_gButtonOk( 0 ),
_gButtonCancel( 0 ),
_gButtonSellUp( 0 ),
_gButtonSellDown( 0 ),
_gButtonTradeUp( 0 ),
_gButtonTradeDown( 0 ),
_iPlayerNumber( -1 ),
_iAmountAvailable( 0 ),
_iAmountToKeep( 0 ),
_iAmountToSell( 0 ),
_iAmountToTrade( 0 ),
_iAmountSurviorHotelStockAvailable( 0 ),
_pGame( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimWinPlayerManageOldStock
******************************************************************************/
SoGuiAnimWinPlayerManageOldStock::~SoGuiAnimWinPlayerManageOldStock()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinPlayerManageOldStock::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgSmall.png" ) );

	SetSpriteAlpha( 0.85f );

	//SetPositionToCenterOfParent();
	SetRelativePos( JpVector2f( 0.0f, -115.0f * SoUtils::GetiPadRatioY() ) );

	//SetWidth( 922.0f * SoUtils::GetiPadRatioX() );
	//SetHeight( 200.0f * SoUtils::GetiPadRatioY() );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	



	//-----------------------
	// Print the window text

	std::string sPlayerName = SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( _iPlayerNumber );
	std::string sHotelName = SoActorVisibleHotel::GetHoteNameById( _pGame->GetLastMergerVictimHotelId() );

	char acTemp[ 256 ] = "";
	sprintf( acTemp, "%s, keep, sell, or trade the stock for the defunct corporation, %s.", sPlayerName.c_str(), sHotelName.c_str() );
	_pLabel = CreateBmFontLabel( acTemp, SoFontDefs::GetGuiDefault() );
	_pLabel->setOpacity( 0 );

	_pLabelKeep = CreateBmFontLabel( "Keep", SoFontDefs::GetGuiDefault() );
	_pLabelKeep->setOpacity( 0 );

	int iCostPerStock = SoGameBitHotel::GetPricePerStockFromTable( _pGame->GetLastMergerVictimHotelId(), _pGame->GetLastMergerVictimHotelSize() );
	sprintf( acTemp, "Sell ($%i each)", iCostPerStock );
	_pLabelSell = CreateBmFontLabel( acTemp, SoFontDefs::GetGuiDefault() );
	_pLabelSell->setOpacity( 0 );

	_pLabelTrade = CreateBmFontLabel( "Trade (2:1)", SoFontDefs::GetGuiDefault() );
	_pLabelTrade->setOpacity( 0 );


	//-----------------------
	// Add an "OK" button.
	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonOkUp.png" );
	ButtonParams._vRelativePos.Init( 420 * SoUtils::GetiPadRatioX(), -145.0f * SoUtils::GetiPadRatioY() );
	_gButtonOk = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	//-----------------------
	// Add up and down arrow buttons
	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonArrowUp.png" );
	ButtonParams._vRelativePos.Init( -80.0f * SoUtils::GetiPadRatioX(), -15.0f * SoUtils::GetiPadRatioY() );
	_gButtonSellUp = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonArrowDown.png" );
	ButtonParams._vRelativePos.Init( -80.0f * SoUtils::GetiPadRatioX(), -70.0f * SoUtils::GetiPadRatioY() );
	_gButtonSellDown = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonArrowUp.png" );
	ButtonParams._vRelativePos.Init( 220.0f * SoUtils::GetiPadRatioX(), -15.0f * SoUtils::GetiPadRatioY() );
	_gButtonTradeUp = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonArrowDown.png" );
	ButtonParams._vRelativePos.Init( 220.0f * SoUtils::GetiPadRatioX(), -70.0f * SoUtils::GetiPadRatioY() );
	_gButtonTradeDown = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	//-----------------------
	// Get and store some key values
	SoGameBitPlayer* pPlayer = _pGame->GetPlayerByPlayerNumber( _iPlayerNumber );

	_iAmountAvailable = pPlayer->GetStockCount( _pGame->GetLastMergerVictimHotelId() );
	_iAmountToKeep = _iAmountAvailable;
	_iAmountToSell = 0;
	_iAmountToTrade = 0;

	SoGameBitHotel* pSurvivingHotel = _pGame->GetLastMergerSurvivorHotel();
	_iAmountSurviorHotelStockAvailable = pSurvivingHotel->GetAvailableStock();


	//-----------------------
	// initialize/prepare the HudNumbers
	JpVector2f vNumberPos( 0.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	_HudNumberKeep.Init( pParentNode, 6, SoFontDefs::GetBoardSpot() );
	_HudNumberKeep.SetRenderZOrder( BASE_RENDER_ORDER_GUI_BUTTON );
	_HudNumberKeep.SetPosition( vNumberPos );
	_HudNumberKeep.SetCurrentNumber( _iAmountAvailable );
	_HudNumberKeep.SetSpacing( 20.0f );
	_HudNumberKeep.SetAnchorPoint( 0.0f, 1.0f );

	vNumberPos.Init( 0.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	_HudNumberSell.Init( pParentNode, 6, SoFontDefs::GetBoardSpot() );
	_HudNumberSell.SetRenderZOrder( BASE_RENDER_ORDER_GUI_BUTTON );
	_HudNumberSell.SetPosition( vNumberPos );
	_HudNumberSell.SetCurrentNumber( _iAmountToSell );
	_HudNumberSell.SetSpacing( 20.0f );
	_HudNumberSell.SetAnchorPoint( 0.0f, 1.0f );

	vNumberPos.Init( 0.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	_HudNumberTrade.Init( pParentNode, 6, SoFontDefs::GetBoardSpot() );
	_HudNumberTrade.SetRenderZOrder( BASE_RENDER_ORDER_GUI_BUTTON );
	_HudNumberTrade.SetPosition( vNumberPos );
	_HudNumberTrade.SetCurrentNumber( _iAmountToTrade );
	_HudNumberTrade.SetSpacing( 20.0f );
	_HudNumberTrade.SetAnchorPoint( 0.0f, 1.0f );



	/*
	Disabled the cancel button for now, because using it leaves the server's GameState in a bad state.
	May fix this up later, but for now it's fine like this.  mdomowicz 2013_12_02
	if ( _pGame->GetCurrentPlayerNumber() == _iPlayerNumber )
	{
		ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonCancelUp.png" );
		ButtonParams._vRelativePos.Init( 350 * SoUtils::GetiPadRatioX(), -165.0f * SoUtils::GetiPadRatioY() );
		_gButtonCancel = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );
	}
	*/
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinPlayerManageOldStock::Deinit()
{
	SoGuiAnimWin::Deinit();

	_HudNumberKeep.Deinit();
	_HudNumberSell.Deinit();
	_HudNumberTrade.Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinPlayerManageOldStock::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	JpVector2f vOffset = _vAbsolutePos;
	vOffset.X = -400.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = 50.0f * SoUtils::GetiPadRatioY();
	_pLabel->SetPosition( _vAbsolutePos + vOffset );


	vOffset.X = -350.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = 10.0f * SoUtils::GetiPadRatioY();
	_pLabelKeep->SetPosition( _vAbsolutePos + vOffset );

	vOffset.X = -100.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = 10.0f * SoUtils::GetiPadRatioY();
	_pLabelSell->SetPosition( _vAbsolutePos + vOffset );


	vOffset.X = 200.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = 10.0f * SoUtils::GetiPadRatioY();
	_pLabelTrade->SetPosition( _vAbsolutePos + vOffset );


	vOffset.X = -330.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = -15.0f * SoUtils::GetiPadRatioY();
	_HudNumberKeep.SetTargetNumber( _iAmountToKeep );
	_HudNumberKeep.Update( dt );
	_HudNumberKeep.SetPosition( _vAbsolutePos + vOffset );

	vOffset.X = -10.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = -15.0f * SoUtils::GetiPadRatioY();
	_HudNumberSell.SetTargetNumber( _iAmountToSell );
	_HudNumberSell.Update( dt );
	_HudNumberSell.SetPosition( _vAbsolutePos + vOffset );


	vOffset.X = 290.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = -15.0f * SoUtils::GetiPadRatioY();
	_HudNumberTrade.SetTargetNumber( _iAmountToTrade );
	_HudNumberTrade.Update( dt );
	_HudNumberTrade.SetPosition( _vAbsolutePos + vOffset );


	// This is an ultra hack way of hiding the hud numbers when higher priority windows
	// are visible.  Without this, the numbers from this window would appear to be on
	// top of the higher priority windows.
	// The critical flaw in the gui system is that it doesn't handle relative render
	// orders between different guis.  An overhaul will likely be necessary to fix this.
	// For the sake of timeliness, I'm going with the hack for now.  mdomowicz 2015_10_10
	if
	( 
		SoGuiAnimWinCoreMain::IsCoreMainWindowOpen() || 
		SoGuiAnimWinLog::IsLogWindowOpen() ||
		SoGuiAnimWinOptions::IsOptionsWindowOpen() ||
		SoGuiAnimWinRules::IsRulesWindowOpen()
	)
	{
		_HudNumberKeep.SetAlpha( 0.0f );
		_HudNumberSell.SetAlpha( 0.0f );
		_HudNumberTrade.SetAlpha( 0.0f );
	}
	else
	{
		_HudNumberKeep.SetAlpha( 1.0f );
		_HudNumberSell.SetAlpha( 1.0f );
		_HudNumberTrade.SetAlpha( 1.0f );
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinPlayerManageOldStock::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonOk )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "ManageOldStockWindowResult" ) );

				GuiMessageForListener.AddInt( _iAmountToKeep );
				GuiMessageForListener.AddInt( _iAmountToSell );
				GuiMessageForListener.AddInt( _iAmountToTrade );

				SendMessageToListeners( &GuiMessageForListener );
			}
			else if ( _gButtonCancel == pMessage->GetSendersGuid() )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "ManageOldStrockWindowCanceled" ) );
				
				SendMessageToListeners( &GuiMessageForListener );
			}
			else if ( _gButtonSellUp == pMessage->GetSendersGuid() )
			{
				if ( _iAmountToKeep > 0 )
				{
					_iAmountToSell ++;
					_iAmountToKeep --;
				}
			}
			else if ( _gButtonSellDown == pMessage->GetSendersGuid() )
			{
				if ( _iAmountToSell > 0 )
				{
					_iAmountToSell --;
					_iAmountToKeep ++;
				}
			}
			else if ( _gButtonTradeUp == pMessage->GetSendersGuid() )
			{
				if ( _iAmountToKeep >= 2 )
				{
					if ( _iAmountSurviorHotelStockAvailable > 0 )
					{
						_iAmountToTrade += 2;
						_iAmountToKeep -= 2;
						_iAmountSurviorHotelStockAvailable --;
					}
				}
			}
			else if ( _gButtonTradeDown == pMessage->GetSendersGuid() )
			{
				if ( _iAmountToTrade >= 2 )
				{
					_iAmountToTrade -= 2;
					_iAmountToKeep += 2;
					_iAmountSurviorHotelStockAvailable ++;
				}
			}

			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



