/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActorVisibleHotel.h"
#include "SoActorVisiblePlayer.h"
#include "SoFontDefs.h"
#include "SoGameBitPlayer.h"
#include "SoGameState.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinPlayerMergeHotelChoice.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiAnimWinPlayerMergeHotelChoice
******************************************************************************/
SoGuiAnimWinPlayerMergeHotelChoice::SoGuiAnimWinPlayerMergeHotelChoice() :
_gButtonOk( 0 ),
_gButtonCancel( 0 ),
_iPlayerNumber( -1 ),
_pGame( NULL ),
_iSurvivingHotelId( HOTEL_UNDEFINED_ID ),
_eUsageType( USAGE_TYPE_LAST )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimWinPlayerMergeHotelChoice
******************************************************************************/
SoGuiAnimWinPlayerMergeHotelChoice::~SoGuiAnimWinPlayerMergeHotelChoice()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinPlayerMergeHotelChoice::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgSmall.png" ) );

	SetSpriteAlpha( 0.85f );

	//SetPositionToCenterOfParent();
	SetRelativePos( JpVector2f( 0.0f, -115.0f * SoUtils::GetiPadRatioY() ) );

	//SetWidth( 922.0f * SoUtils::GetiPadRatioX() );
	//SetHeight( 200.0f * SoUtils::GetiPadRatioY() );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	



	//-----------------------
	// clear data arrays.
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		_agToggles[ i ] = 0;
	}


	//-----------------------
	// create common ToggleParams
	SoGuiAnimToggle::SetupParams ToggleParams;
	ToggleParams._hSoundEffectOnToggleOn = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ToggleParams._eToggleOnImageOption = SoGuiAnimToggle::TOGGLE_ON_IMAGE_OPTION_SCALE_THE_OFF_IMAGE_DOWN;



	//-----------------------
	// select the hotels which need toggles
	SoGameBitHotelList SelectedHotels;

	if ( _eUsageType == USAGE_TYPE_PICK_SURVIVOR_HOTEL )
	{
		SoGameBitTile* pLastPlacedTile = _pGame->GetLastPlacedTile();
		_pGame->BuildListOfPotentialMergeSurvivorHotels( &SelectedHotels, pLastPlacedTile->GetTileX(), pLastPlacedTile->GetTileY() );
	}
	else
	{
		ASSERTF( _iSurvivingHotelId != HOTEL_UNDEFINED_ID, ( "\nThis usage type for this window requires the surviving hotel to be set." ) );

		if ( _iSurvivingHotelId != HOTEL_UNDEFINED_ID )
		{
			SoGameBitHotel* pSurvivingHotel = _pGame->GetHotelByHotelId( _iSurvivingHotelId );
			SoGameBitTile* pLastPlacedTile = _pGame->GetLastPlacedTile();
			_pGame->BuildListOfPotentialMergeVictimHotels( &SelectedHotels, pLastPlacedTile->GetTileX(), pLastPlacedTile->GetTileY(), pSurvivingHotel );
		}
	}



	//-----------------------
	// create the toggles for the selected hotels.
	SoGameBitHotelList::Iterator HotelsIter( &SelectedHotels );
	while ( HotelsIter.Next() )
	{
		SoGameBitHotel* pHotel = HotelsIter.GetNodeData();

		int iHotelId = pHotel->GetHotelId();

		char acTemp[ 128 ] = "";
		sprintf( acTemp, "HotelTile%i.png", iHotelId );

		ToggleParams._sPathAndFilenameToggleOff = SoUtils::AddRelativePathToTexture( acTemp );

		int iX = -377.0f + ( 125.0f * iHotelId );
		
		ToggleParams._vRelativePos.Init( iX * SoUtils::GetiPadRatioX(), -15.0f * SoUtils::GetiPadRatioY() );

		guid gGui = _pGuiManager->CreateAndAddAnimToggle( ToggleParams, this );

		SoGui* pGui = _pGuiManager->GetGuiByGuid( gGui );

		float fWidth = 80.0f;
		float fHeight = fWidth * SoActorVisibleHotel::GetSpriteHeightToWidthRatio( iHotelId );

		pGui->SetWidth( fWidth * SoUtils::GetiPadRatioX() );
		pGui->SetHeight( fHeight * SoUtils::GetiPadRatioY() );
		//pGui->SetSpriteColor( SoActorVisibleHotel::GetHotelColorById( pHotel->GetHotelId() ) );

		_agToggles[ iHotelId ] = gGui;
	}



	//-----------------------
	// Print the window text
	std::string sPlayerName = SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( _iPlayerNumber );
	char acTemp[ 256 ] = "";

	if ( _eUsageType == USAGE_TYPE_PICK_SURVIVOR_HOTEL )
		sprintf( acTemp, "%s, select the corporation that survives the merger.", sPlayerName.c_str() );
	else
		sprintf( acTemp, "%s, select the corporation that is absorbed.", sPlayerName.c_str() );

	_pLabel = CreateBmFontLabel( acTemp, SoFontDefs::GetGuiDefault() );
	_pLabel->setOpacity( 0 );



	//-----------------------
	// Add an "OK" button.
	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonOkUp.png" );
	ButtonParams._vRelativePos.Init( 420 * SoUtils::GetiPadRatioX(), -145.0f * SoUtils::GetiPadRatioY() );
	_gButtonOk = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonCancelUp.png" );
	ButtonParams._vRelativePos.Init( 350 * SoUtils::GetiPadRatioX(), -145.0f * SoUtils::GetiPadRatioY() );
	_gButtonCancel = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinPlayerMergeHotelChoice::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinPlayerMergeHotelChoice::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	JpVector2f vLabelPos = _vAbsolutePos;
	vLabelPos.X -= 400.0f * SoUtils::GetiPadRatioX();
	vLabelPos.Y += 50.0f * SoUtils::GetiPadRatioY();

	_pLabel->SetPosition( vLabelPos );


	//-----------------------
	// Disable the OK button until a valid selection is made.	
	SoGuiAnimButton* pButtonOk = _pGuiManager->GetGuiAnimButtonByGuid( _gButtonOk );
	pButtonOk->Enable( false );
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		if ( IsToggleOn( _agToggles[ i ] ) )
			pButtonOk->Enable( true );
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinPlayerMergeHotelChoice::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_TOGGLE_ON_OFF:
		{
			break;
		}
		case SoGuiMessage::TYPE_TOGGLE_ON_ON:
		{
			UnToggleAllGuisExcept( pMessage->GetSendersGuid() );

			break;
		}

		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonOk )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );

				if ( _eUsageType == USAGE_TYPE_PICK_SURVIVOR_HOTEL )
					GuiMessageForListener.AddHash( JpHash( "MergeHotelChoiceWindowResultSurvivor" ) );
				else
					GuiMessageForListener.AddHash( JpHash( "MergeHotelChoiceWindowResultVictim" ) );


				for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
				{
					if ( IsToggleOn( _agToggles[ i ] ) )
					{
						GuiMessageForListener.AddInt( i );
						SendMessageToListeners( &GuiMessageForListener );
					}
				}
			}
			else if ( _gButtonCancel == pMessage->GetSendersGuid() )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "MergeHotelChoiceWindowCanceled" ) );
				
				SendMessageToListeners( &GuiMessageForListener );
			}

			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  UnToggleAllGuisExcept
******************************************************************************/
void SoGuiAnimWinPlayerMergeHotelChoice::UnToggleAllGuisExcept( guid gExceptionGui )
{
	struct Local
	{
		static void UnToggle( SoGuiManager* pManager, guid gToggle )
		{
			SoGui* pGui = pManager->GetGuiByGuid( gToggle );
			if ( pGui )
			{
				SoGuiAnimToggle* pToggle = pGui->GetAsAnimToggle();
				if ( pToggle )
				{
					pToggle->SetOff();
				}
			}
		}
	};

	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		if ( _agToggles[ i ] != gExceptionGui )
			Local::UnToggle( _pGuiManager, _agToggles[ i ] );
	}
}



/******************************************************************************
 FUNCTION:  IsToggleOn
******************************************************************************/
bool SoGuiAnimWinPlayerMergeHotelChoice::IsToggleOn( guid gGui )
{
	SoGui* pGui = _pGuiManager->GetGuiByGuid( gGui );
	if ( pGui )
	{
		SoGuiAnimToggle* pToggle = pGui->GetAsAnimToggle();
		if ( pToggle )
		{
			return pToggle->IsOn();
		}
	}

	return false;
}
