/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActorVisibleHotel.h"
#include "SoActorVisiblePlayer.h"
#include "SoFontDefs.h"
#include "SoGameBitPlayer.h"
#include "SoGameState.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinPlayerStockPurchase.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiAnimWinPlayerStockPurchase
******************************************************************************/
SoGuiAnimWinPlayerStockPurchase::SoGuiAnimWinPlayerStockPurchase() :
_gButtonOk( 0 ),
_gButtonCancel( 0 ),
_iPlayerNumber( -1 ),
_iPlayerMoney( 0 ),
_pGame( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiAnimWinPlayerStockPurchase
******************************************************************************/
SoGuiAnimWinPlayerStockPurchase::~SoGuiAnimWinPlayerStockPurchase()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinPlayerStockPurchase::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgSmall.png" ) );

	SetSpriteAlpha( 0.85f );

	//SetPositionToCenterOfParent();
	SetRelativePos( JpVector2f( 0.0f, -115.0f * SoUtils::GetiPadRatioY() ) );

	//SetWidth( 922.0f * SoUtils::GetiPadRatioX() );
	//SetHeight( 200.0f * SoUtils::GetiPadRatioY() );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	



	//-----------------------
	// create common ButtonParams
	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;


	//-----------------------
	// Find the cards that are dealt (face up), and create buttons for them.
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoGameBitHotel* pHotel = _pGame->GetHotelByHotelId( i );

		char acTemp[ 128 ] = "";
		sprintf( acTemp, "HotelTile%i.png", i );

		ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( acTemp );

		int iX = -377.0f + ( 125.0f * i );
		int iY = 20.0f;

		ButtonParams._vRelativePos.Init( iX * SoUtils::GetiPadRatioX(), iY * SoUtils::GetiPadRatioY() );

		guid gGui = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

		SoGuiAnimButton* pButton = _pGuiManager->GetGuiAnimButtonByGuid( gGui );

		float fWidth = 40.0f;
		float fHeight = fWidth * SoActorVisibleHotel::GetSpriteHeightToWidthRatio( i );

		pButton->SetWidth( fWidth * SoUtils::GetiPadRatioX() );
		pButton->SetHeight( fHeight * SoUtils::GetiPadRatioY() );
		//pButton->SetSpriteColor( SoActorVisibleHotel::GetHotelColorById( pHotel->GetHotelId() ) );
		pButton->SetUserIntValue( i );

		if ( !pHotel->IsPlaced() )
			pButton->Enable( false );

		_AvailableStockButtonsList.AddLast( pButton );
	}


	//-----------------------
	// set the window text
	std::string sPlayerName = SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( _iPlayerNumber );

	char acTemp[ 256 ] = "";
	sprintf( acTemp, "%s, select stocks to purchase.", sPlayerName.c_str() );
	_pLabel = CreateBmFontLabel( acTemp, SoFontDefs::GetGuiDefault() );
	_pLabel->setOpacity( 0 );


	//-----------------------
	// Add additional labels
	_pLabelMoneyAvailable = CreateBmFontLabel( "Available Money:", SoFontDefs::GetGuiDefault() );
	_pLabelMoneyAvailable->setOpacity( 0 );
	_pLabelMoneyAvailable->setAnchorPoint( ccp( 1.0f, 0.0f ) );

	_pLabelTotalCost = CreateBmFontLabel( "Total Cost:", SoFontDefs::GetGuiDefault() );
	_pLabelTotalCost->setOpacity( 0 );
	_pLabelTotalCost->setAnchorPoint( ccp( 1.0f, 0.0f ) );


	//-----------------------
	// Add an "OK" button.
	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonOkUp.png" );
	ButtonParams._vRelativePos.Init( 420 * SoUtils::GetiPadRatioX(), -145.0f * SoUtils::GetiPadRatioY() );
	_gButtonOk = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	//-----------------------
	// Add an "cancel" button.
	if ( _pGame->GetCurrentPlayerNumber() == _iPlayerNumber )
	{
		ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonCancelUp.png" );
		ButtonParams._vRelativePos.Init( 350 * SoUtils::GetiPadRatioX(), -145.0f * SoUtils::GetiPadRatioY() );
		_gButtonCancel = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );
	}


	// We get and cache the player's money during window init so that
	// when the player's money changes (after he hits OK and purchases stock)
	// the shown values don't change while the window is closing (which looks
	// buggy).  mdomowicz 2014_03_08
	SoGameBitPlayer* pPlayer = _pGame->GetPlayerByPlayerNumber( _iPlayerNumber );
	_iPlayerMoney = pPlayer->GetMoney();
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinPlayerStockPurchase::Deinit()
{
	SoGuiAnimWin::Deinit();

	_AvailableStockButtonsList.RemoveAll();
	_PurchasedStockButtonsList.RemoveAll();
	_ToBeDeletedStockButtonsList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinPlayerStockPurchase::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	JpVector2f vOffset;
	SoGameBitPlayer* pPlayer = _pGame->GetPlayerByPlayerNumber( _iPlayerNumber );



	//-----------------------
	// Calculate the total cost for all stocks purchased so far.
	int iTotalCost = 0;

	SoGuiAnimButtonList::Iterator PurchasedButtonIter( &_PurchasedStockButtonsList );
	while ( PurchasedButtonIter.Next() )
	{
		SoGuiAnimButton* pButton = PurchasedButtonIter.GetNodeData();

		int iHotelId = pButton->GetUserIntValue();
		SoGameBitHotel* pHotel = _pGame->GetHotelByHotelId( iHotelId );
		int iCostPerStock = SoGameBitHotel::GetPricePerStockFromTable( iHotelId, pHotel->GetSize() );

		iTotalCost += iCostPerStock;
	}

	int iPlayerMoneyAfterPurchase = _iPlayerMoney - iTotalCost;



	//-----------------------
	// Update each purchased stock button
	int iNumButtons = 0;
	PurchasedButtonIter.Reset();
	while ( PurchasedButtonIter.Next() )
	{
		SoGuiAnimButton* pButton = PurchasedButtonIter.GetNodeData();

		vOffset.X = ( -330.0f * SoUtils::GetiPadRatioX() ) + ( 100.0f * iNumButtons ) * SoUtils::GetiPadRatioX();
		vOffset.Y = -50.0f * SoUtils::GetiPadRatioY();

		pButton->SetRelativePos( vOffset );
		iNumButtons ++;


		if ( pButton->IsHiding() )
			_ToBeDeletedStockButtonsList.AddLast( pButton );
	}


	//-----------------------
	// Any buttons flagged for deletion are removed here.
	SoGuiAnimButtonList::Iterator ToDeleteButtonIter( &_ToBeDeletedStockButtonsList );
	while ( ToDeleteButtonIter.Next() )
	{
		SoGuiAnimButton* pButton = ToDeleteButtonIter.GetNodeData();
		_PurchasedStockButtonsList.Remove( pButton );

		RemoveChildGui( pButton );
		_pGuiManager->RemoveAndDeleteGui( pButton );
	}
	_ToBeDeletedStockButtonsList.RemoveAll();



	//-----------------------
	// We need to count how many stocks of each hotel 
	// are currently selected for purchase, so that
	// we can later check to make sure we aren't allowing
	// the player to purchase more than are available.  
	// Eg. If there are only 2 stocks available, then we 
	// can't let the player buy 3.
	int aiPurchaseTallies[ GAMERULES_NUM_HOTELS ];
	memset( aiPurchaseTallies, 0, GAMERULES_NUM_HOTELS * sizeof( int ) );

	PurchasedButtonIter.Reset();
	while ( PurchasedButtonIter.Next() )
	{
		SoGuiAnimButton* pButton = PurchasedButtonIter.GetNodeData();
		int iHotelId = pButton->GetUserIntValue();
		ASSERTF( iHotelId >= 0 && iHotelId < GAMERULES_NUM_HOTELS, ( "\nBad Hotel Id" ) );

		if ( iHotelId >= 0 && iHotelId < GAMERULES_NUM_HOTELS )
			aiPurchaseTallies[ iHotelId ] ++;
	}

	//-----------------------
	// Update the buttons used to purchase the stock.
	// These buttons should disable/enable based on if the player has
	// enough money, and so on.
	SoGuiAnimButtonList::Iterator AvailableButtonsIter( &_AvailableStockButtonsList );
	while ( AvailableButtonsIter.Next() )
	{
		SoGuiAnimButton* pButton = AvailableButtonsIter.GetNodeData();

		int iHotelId = pButton->GetUserIntValue();
		SoGameBitHotel* pHotel = _pGame->GetHotelByHotelId( iHotelId );

		int iCostPerStock = SoGameBitHotel::GetPricePerStockFromTable( iHotelId, pHotel->GetSize() );
		int iNumHotelStocksAvailable = pHotel->GetAvailableStock();
		int iNumHotelStocksToBePurchased = aiPurchaseTallies[ iHotelId ];
		int iTotalNumStocksToBePurchased = _PurchasedStockButtonsList.GetNumEntries();

		bool bIsPlaced = pHotel->IsPlaced();
		bool bPlayerHasFunds = ( iCostPerStock <= iPlayerMoneyAfterPurchase );
		bool bHotelStocksAreAvailable = ( iNumHotelStocksToBePurchased < iNumHotelStocksAvailable );
		bool bPlayerCanPurchaseMore = ( ( pPlayer->GetNumStocksPurchasedThisTurn() + iTotalNumStocksToBePurchased ) < 3 );
		bool bEnabled = ( bIsPlaced && bPlayerHasFunds && bHotelStocksAreAvailable && bPlayerCanPurchaseMore );

		pButton->Enable( bEnabled );

		if ( bEnabled )
			pButton->SetSpriteColor( JpUtils::_White );
		else
			pButton->SetSpriteColor( JpUtils::_Grey128 );
	}



	//-----------------------
	// Update the positions and text of all the labels.
	vOffset.X = -400.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = 50.0f * SoUtils::GetiPadRatioY();
	_pLabel->SetPosition( _vAbsolutePos + vOffset );


	char acTemp[ 256 ] = "";
	sprintf( acTemp, "Available Money: $%i", iPlayerMoneyAfterPurchase );
	vOffset.X = 300.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = -40.0f * SoUtils::GetiPadRatioY();
	_pLabelMoneyAvailable->setString( acTemp );
	_pLabelMoneyAvailable->SetPosition( _vAbsolutePos + vOffset );

	sprintf( acTemp, "Total Cost: $%i", iTotalCost );
	vOffset.X = 300.0f * SoUtils::GetiPadRatioX();
	vOffset.Y = -70.0f * SoUtils::GetiPadRatioY();
	_pLabelTotalCost->setString( acTemp );
	_pLabelTotalCost->SetPosition( _vAbsolutePos + vOffset );

}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinPlayerStockPurchase::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonOk )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );


				GuiMessageForListener.AddHash( JpHash( "StockPurchaseWindowResult" ) );

				//-----------------------
				// prepare an array to track how many of each hotel was purchased
				int aiHotelIdPurcahseAmounts[ GAMERULES_NUM_HOTELS ];
				for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
					aiHotelIdPurcahseAmounts[ i ] = 0;


				//-----------------------
				// tally the purchases
				SoGuiAnimButtonList::Iterator ButtonIter( &_PurchasedStockButtonsList );
				while ( ButtonIter.Next() )
				{
					SoGuiAnimButton* pButton = ButtonIter.GetNodeData();
					int iHotelId = pButton->GetUserIntValue();
					aiHotelIdPurcahseAmounts[ iHotelId ] ++;
				}

				//-----------------------
				// add the purchase values to the message.
				for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
					GuiMessageForListener.AddInt( aiHotelIdPurcahseAmounts[ i ] );

				SendMessageToListeners( &GuiMessageForListener );
			}
			else if ( _gButtonCancel == pMessage->GetSendersGuid() )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "StockPurchaseWindowCanceled" ) );
				
				SendMessageToListeners( &GuiMessageForListener );
			}
			else
			{
				//-----------------------
				// if any of the hotel buttons are pressed, add new buttons to represent
				// the hotel stock being purchased.
				SoGuiAnimButtonList::Iterator AvailableButtonsIter( &_AvailableStockButtonsList );
				while ( AvailableButtonsIter.Next() )
				{
					SoGuiAnimButton* pButton = AvailableButtonsIter.GetNodeData();

					if ( pButton->GetGuid() == pMessage->GetSendersGuid() )
						AddNewButtonForPurchasedHotel( pButton->GetUserIntValue() );
				}


				//-----------------------
				// If any of the "purchased stock" buttons are pressed, transition them out.
				// The update() loop will delete them, once they are no longer visible.
				SoGuiAnimButtonList::Iterator ButtonIter( &_PurchasedStockButtonsList );
				while ( ButtonIter.Next() )
				{
					SoGuiAnimButton* pButton = ButtonIter.GetNodeData();

					if ( pButton->GetGuid() == pMessage->GetSendersGuid() )
						_pGuiManager->SendMessageToGui( pButton->GetGuid(), &SoGuiMessage::_BasicTransitionOut );
				}
			}


			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  AddNewButtonForPurchasedHotel
******************************************************************************/
void SoGuiAnimWinPlayerStockPurchase::AddNewButtonForPurchasedHotel( int iHotelId )
{
	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;


	char acTemp[ 128 ] = "";
	sprintf( acTemp, "HotelTile%i.png", iHotelId );
	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( acTemp );


	ButtonParams._vRelativePos.Init( 0.0f * SoUtils::GetiPadRatioX(), 0.0f * SoUtils::GetiPadRatioY() );
	guid gButton = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	SoGuiAnimButton* pButton = _pGuiManager->GetGuiAnimButtonByGuid( gButton );
	_PurchasedStockButtonsList.AddLast( pButton );

	float fWidth = 55.0f;
	float fHeight = fWidth * SoActorVisibleHotel::GetSpriteHeightToWidthRatio( iHotelId );

	pButton->SetWidth( fWidth * SoUtils::GetiPadRatioX() );
	pButton->SetHeight( fHeight * SoUtils::GetiPadRatioY() );
	pButton->SetSpriteColor( JpUtils::_White );
	pButton->SetUserIntValue( iHotelId );

	_pGuiManager->SendMessageToGui( gButton, &SoGuiMessage::_BasicTransitionIn );

}
