/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActorManager.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinRules.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"

#define MINIMUM_USER_SCALE 1.1f
#define RULES_WINDOW_WIDTH 922.0f
#define RULES_WINDOW_HEIGHT 650.0f

#define VERTICAL_OFFSET 50.0f

USING_NS_CC;


bool SoGuiAnimWinRules::_bIsRulesWindowOpen = false;



/******************************************************************************
 FUNCTION:  SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinRules::SoGuiAnimWinRules() :
_gButtonBack( 0 ),
_iRulebookPageNumber( 1 ),
_vTouchPosPrev( 0.0f, 0.0f ),
_vVelocity( 0.0f, 0.0f ),
_vUserOffset( 0.0f, 0.0f ),
_fUserScale( 1.0f ),
_fScaleTimer( 0.0f ),
_fSpeedMax( 50.0f ),
_fDrag( 0.9f ),
_bMultitouchGestureRecognized( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinRules::~SoGuiAnimWinRules()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinRules::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgLarge.png" ) );

	SetSpriteAlpha( 0.85f );

	//SetPositionToCenterOfParent();
	SetRelativePos( JpVector2f( 0.0f, VERTICAL_OFFSET * SoUtils::GetiPadRatioY() ) );

	SetWidth( RULES_WINDOW_WIDTH * SoUtils::GetTrueiPadRatioX() );
	SetHeight( RULES_WINDOW_HEIGHT * SoUtils::GetiPadRatioY() );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	


	// Custom clip node
	_pRulebookClipNode = new SoClipNode;
	_pClipNode->addChild( _pRulebookClipNode );
	SoUtils::SetRenderZOrder( _pClipNode, _pRulebookClipNode, BASE_RENDER_ORDER_GUI + 1 );

	_pRulebookClipNode->EnableClipping( true );

	_fScaleMin = MINIMUM_USER_SCALE * SoUtils::GetiPadRatioY();
	_fScaleMax = 2.0f * SoUtils::GetiPadRatioY();

	_fUserScale = MINIMUM_USER_SCALE * SoUtils::GetiPadRatioY();

	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;


	// ButtonBack
	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonBackUp.png" );
	ButtonParams._vRelativePos.Init( 0 * SoUtils::GetiPadRatioX(), -315 * SoUtils::GetiPadRatioY() );
	_gButtonBack = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	// ButtonPrevPage
	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonRulesPrevUp.png" );
	ButtonParams._vRelativePos.Init( -300 * SoUtils::GetiPadRatioX(), -315 * SoUtils::GetiPadRatioY() );
	_gButtonPrevPage = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	// ButtonNextPage
	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonRulesNextUp.png" );
	ButtonParams._vRelativePos.Init( 300 * SoUtils::GetiPadRatioX(), -315 * SoUtils::GetiPadRatioY() );
	_gButtonNextPage = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );


	// Rulebook sprite
	_pRulebookSprite = CreateSprite( SoUtils::AddRelativePathToTexture( "RulesPage01.png" ), BASE_RENDER_ORDER_GUI );
	_pRulebookSprite->setScale( _fUserScale );


	// Set the initial position so that the we see the top of the page first.
	_vUserOffset.Init( 0.0f, -9000.0f );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinRules::Deinit()
{
	RemoveAndDestroyNode( _pClipNode, _pRulebookClipNode );

	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinRules::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	_pRulebookClipNode->SetClippingRegion
	( 
		_vAbsolutePos.X - ( ( RULES_WINDOW_WIDTH - 60.0f ) / 2.0f ) * SoUtils::GetTrueiPadRatioX(),
		_vAbsolutePos.Y - ( ( RULES_WINDOW_HEIGHT - 75.0f ) / 2.0f ) * SoUtils::GetiPadRatioY(),
		865.0f * SoUtils::GetTrueiPadRatioX(), 
		578.0f * SoUtils::GetiPadRatioY() 
	);


	// If this menu is not visible, don't use input to move sprites around
	if ( _eTransitionState != TRANSITION_STATE_SHOW )
	{
		_pRulebookSprite->setVisible( false );

		ZeroUserValues();
	}
	else
	{
		_pRulebookSprite->setVisible( true );
	}


	// Calculate scale
	_fScaleTimer = Clamp( _fScaleTimer + dt, 0.0f, 1.0f );
	float fNewScale = Interpolate( _pRulebookSprite->getScale(), _fUserScale, _fScaleTimer );


	// Calculate offset
	_vUserOffset += _vVelocity;
	_vVelocity -= _vVelocity * ( _fDrag * dt );




	float fScaleCurDif = ClampMin( fNewScale - _fScaleMin, 0.0f );
	float fScaleMaxDif = _fScaleMax - _fScaleMin;
	float fScalePercent = fScaleCurDif / fScaleMaxDif;

	float fClampX = Interpolate
	( 
		0.0f, 
		350.0f * SoUtils::GetTrueiPadRatioX(), 
		fScalePercent 
	);

	float fClampY = Interpolate
	( 
		280.0f * SoUtils::GetiPadRatioY(), 
		750.0f * SoUtils::GetiPadRatioY(), 
		fScalePercent 
	);

	// Needed so that the top and bottom of the rules are flush with the clip region.
	// This is because the window was moved up by 50.
	// mdomowicz 2014_02_15
	float fWindowPosCompensation = -( VERTICAL_OFFSET * SoUtils::GetiPadRatioY() );

	_vUserOffset.X = Clamp( _vUserOffset.X, -fClampX, fClampX );
	_vUserOffset.Y = Clamp( _vUserOffset.Y, -fClampY + fWindowPosCompensation, fClampY + fWindowPosCompensation );


	// Opacity
	int iRulebookOpacity = _pSpriteBackground->getOpacity() * 2;
	char cRulebookOpacity = (unsigned char)Clamp( iRulebookOpacity, 0, 255 );


	// Set the actual values
	_pRulebookSprite->setScale( fNewScale );
	_pRulebookSprite->setOpacity( cRulebookOpacity );
	_pRulebookSprite->SetPosition( GetAbsolutePos() + GetRelativePos() + _vUserOffset );
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinRules::HandleMessage( SoGuiMessage* pMessage )
{
	ZeroUserValues();

	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonBack )
			{
				HandleWindowExit();
			}
			else if ( pMessage->GetSendersGuid() == _gButtonNextPage )
			{
				_iRulebookPageNumber = Clamp( _iRulebookPageNumber + 1, 1, 4 );

				RemoveAndDestroySprite( _pRulebookSprite );

				_pRulebookSprite = CreateSprite( GetSpriteFromPageNumber( _iRulebookPageNumber ), BASE_RENDER_ORDER_GUI );
				_pRulebookSprite->setScale( 0.9f );
				_pRulebookSprite->setVisible( true );
				_pRulebookSprite->SetPosition( GetAbsolutePos() );

				if ( _iRulebookPageNumber > 1 )
				{
					SoGuiAnimButton* pButtonPrev = _pGuiManager->GetGuiByGuid( _gButtonPrevPage )->GetAsAnimButton();
					pButtonPrev->Enable( true );
				}

				if ( _iRulebookPageNumber >= 4 )
				{
					SoGuiAnimButton* pButtonPrev = _pGuiManager->GetGuiByGuid( _gButtonNextPage )->GetAsAnimButton();
					pButtonPrev->Enable( false );
				}
			}
			else if ( pMessage->GetSendersGuid() == _gButtonPrevPage )
			{
				_iRulebookPageNumber = Clamp( _iRulebookPageNumber - 1, 1, 4 );

				RemoveAndDestroySprite( _pRulebookSprite );

				_pRulebookSprite = CreateSprite( GetSpriteFromPageNumber( _iRulebookPageNumber ), BASE_RENDER_ORDER_GUI );
				_pRulebookSprite->setScale( 0.9f );
				_pRulebookSprite->setVisible( true );
				_pRulebookSprite->SetPosition( GetAbsolutePos() );

				if ( _iRulebookPageNumber <= 1 )
				{
					SoGuiAnimButton* pButtonPrev = _pGuiManager->GetGuiByGuid( _gButtonPrevPage )->GetAsAnimButton();
					pButtonPrev->Enable( false );
				}

				if ( _iRulebookPageNumber < 4 )
				{
					SoGuiAnimButton* pButtonPrev = _pGuiManager->GetGuiByGuid( _gButtonNextPage )->GetAsAnimButton();
					pButtonPrev->Enable( true );
				}
			}

			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_IN:
		{
			if 
			( 
				GetParentGui() == false ||
				pMessage->GetSendersGuid() == GetParentGui()->GetGuid() 
			)
			{
				SoGuiAnimWin::HandleMessage( pMessage );
				_bIsRulesWindowOpen = true;
			}

			if ( _iRulebookPageNumber <= 1 )
			{
				SoGuiAnimButton* pButtonPrev = _pGuiManager->GetGuiByGuid( _gButtonPrevPage )->GetAsAnimButton();
				pButtonPrev->Enable( false );
			}

			break;
		}

		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			if 
			( 
				GetParentGui() == false || 
				pMessage->GetSendersGuid() == GetParentGui()->GetGuid() 
			)
			{
				SoGuiAnimWin::HandleMessage( pMessage );
				_bIsRulesWindowOpen = false;
			}

			_pRulebookSprite->setVisible( false );
			break;
		}

		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  OnMouseMoved
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	bool SoGuiAnimWinRules::OnMouseMoved( const OIS::MouseEvent &MouseEventArg )
	{
		if ( IsShowing() )
		{
			if ( MouseEventArg.state.Z.rel > 0 )
			{
				_fUserScale = Clamp( _fUserScale + ( 0.5f * SoUtils::GetiPadRatioY() ), MINIMUM_USER_SCALE * SoUtils::GetiPadRatioY(), _fScaleMax );
				_fScaleTimer = 0.0f;

				ZeroUserValues();
			}
			else if ( MouseEventArg.state.Z.rel < 0 )
			{
				_fUserScale = Clamp( _fUserScale - ( 0.5f * SoUtils::GetiPadRatioY() ), MINIMUM_USER_SCALE * SoUtils::GetiPadRatioY(), _fScaleMax );
				_fScaleTimer = 0.0f;

				ZeroUserValues();
			}

			//if ( MouseEventArg.state.Z.rel != 0 )
			//	CCLOG( "Z: %i", MouseEventArg.state.Z.rel );
			return true;
		}

		return false;
	}
#endif



/******************************************************************************
 FUNCTION:  OnTouchesMove
******************************************************************************/
bool SoGuiAnimWinRules::OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) 
{
	if ( IsShowing() )
	{
		if ( _bMultitouchGestureRecognized == false )
		{
			_vVelocity.Init( 0.0f, 0.0f );
			JpVector2f vDelta( 0.0f, 0.0f );

			SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
			if ( pNonConstTouchList->GetNumEntries() == 1 )
			{
				CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
				vDelta = SoInputTouch::TouchPosToScreenPos( pTouch0->locationInView() ) - SoInputTouch::TouchPosToScreenPos( pTouch0->previousLocationInView() );

				_vUserOffset.X += vDelta.X;
				_vUserOffset.Y += vDelta.Y;

				// This gets used on 'OnTouchesUp' to calculate a momentum
				_vTouchPosPrev = pTouch0->previousLocationInView();

				return true;
			}

			//CCLOG( "X: %f    Y: %f", _vUserOffset.X), _vUserOffset.Y );
		}
	}

	// The input is considered "handled" no matter what, if this window is open.
	// This helps SoInputDispatcher, and SoGuiManager prevent input conflicts.
	return IsShowing();
}



/******************************************************************************
FUNCTION:  OnTouchesDown
******************************************************************************/
bool SoGuiAnimWinRules::OnTouchesDown( const SoCcTouchList* pSoCcTouchList )
{
	if ( IsShowing() )
	{
		_vTouchPosPrev.x = 0.0f;
		_vTouchPosPrev.y = 0.0f;

		return true;
	}

	// The input is considered "handled" no matter what, if this window is open.
	// This helps SoInputDispatcher, and SoGuiManager prevent input conflicts.
	return IsShowing();
}



/******************************************************************************
FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoGuiAnimWinRules::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{
	if ( IsShowing() )
	{
		JpVector2f vDelta( 0.0f, 0.0f );

		SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
		if ( pNonConstTouchList->GetNumEntries() > 0 )
		{
			if ( _vTouchPosPrev.x != 0.0f && _vTouchPosPrev.y != 0.0f )
			{
				CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
				vDelta = SoInputTouch::TouchPosToScreenPos( _vTouchPosPrev  ) - SoInputTouch::TouchPosToScreenPos( pTouch0->locationInView() );

				_vVelocity.X -= vDelta.X;
				_vVelocity.Y -= vDelta.Y;

				if ( Length( _vVelocity ) > _fSpeedMax )
				{
					_vVelocity.Normalize();

					_vVelocity *= _fSpeedMax;
				}
				if ( Length( _vVelocity ) < 5.0f )
					_vVelocity.Init( 0.0f, 0.0f );

				return true;
			}
			
			_bMultitouchGestureRecognized = false;
		}
	}

	// The input is considered "handled" no matter what, if this window is open.
	// This helps SoInputDispatcher, and SoGuiManager prevent input conflicts.
	return IsShowing();
}



/******************************************************************************
 FUNCTION:  OnGestureMove
 *****************************************************************************/
bool SoGuiAnimWinRules::OnGestureMove( SoGesture* pGesture )
{
	switch ( pGesture->_eGestureType )
	{
		case SoGesture::GESTURE_TYPE_ZOOM:
		case SoGesture::GESTURE_TYPE_PINCH:
		{
			_bMultitouchGestureRecognized = true;
			ZeroUserValues();
			
			float fDelta = 0.5f * ( pGesture->_vPositionDeltaPrevious[ 0 ].Length() + pGesture->_vPositionDeltaPrevious[ 1 ].Length() );
			fDelta *= GESTURE_ZOOM_SPEED;

			if ( pGesture->_eGestureType == SoGesture::GESTURE_TYPE_PINCH )
				fDelta *= -1.0f;
			
			_fUserScale = Clamp( _fUserScale + fDelta, MINIMUM_USER_SCALE, _fScaleMax );
		
			//CCLOG( "Delta: %f", fDelta );

			return true;
			break;
		}
		default:
			break;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  ZeroUserValues
******************************************************************************/
void SoGuiAnimWinRules::ZeroUserValues()
{
	_vVelocity.Init( 0.0f, 0.0f );
	_vTouchPosPrev.x = 0.0f;
	_vTouchPosPrev.y = 0.0f;
}



/******************************************************************************
 FUNCTION:  GetSpriteFromPageNumber
******************************************************************************/
std::string SoGuiAnimWinRules::GetSpriteFromPageNumber( int iPageNumber )
{
	std::string sSpriteFilename = "";

	switch ( iPageNumber )
	{
		case 1:
			sSpriteFilename = "RulesPage01.png";  break;
		case 2:
			sSpriteFilename = "RulesPage02.png";  break;
		case 3:
			sSpriteFilename = "RulesPage03.png";  break;
		case 4:
			sSpriteFilename = "RulesPage04.png";  break;
			
		default:  
			ASSERTF( 0, (" Error " ) );  break;				
	}

	sSpriteFilename = SoUtils::AddRelativePathToTexture( sSpriteFilename.c_str() );

	return sSpriteFilename;
}



/******************************************************************************
 FUNCTION:  CreateSprite
******************************************************************************/
SoSprite* SoGuiAnimWinRules::CreateSprite( const std::string& sPathAndFilename, int iRenderOrder )
{
	SoSprite* pSprite = NULL;

	pSprite = SoSprite::CreateScaledToDevice( sPathAndFilename.c_str() );

	pSprite->retain();
	pSprite->SetPosition( GetAbsolutePos() + _vUserOffset );
	_pRulebookClipNode->addChild( pSprite );
	pSprite->setVisible( false );

	SoUtils::SetRenderZOrder( _pRulebookClipNode, pSprite, iRenderOrder );

	_SpriteList.AddLast( pSprite );

	return pSprite;
}



/******************************************************************************
 FUNCTION:  RemoveAndDestroySprite
******************************************************************************/
void SoGuiAnimWinRules::RemoveAndDestroySprite( SoSprite* pSprite )
{
	if ( pSprite )
	{
		_SpriteList.Remove( pSprite );
		RemoveAndDestroyNode( _pRulebookClipNode, pSprite );
		pSprite = NULL;
	}
}



/******************************************************************************
 FUNCTION:  HandleWindowExit
******************************************************************************/
void SoGuiAnimWinRules::HandleWindowExit()
{
	// This options window is used by SoFsmStateShell as well as SoGuiAnimWinCoreMainLauncher.
	// Because SoFsmStateShell isn't a gui, it must use the listener system to get message back.
	// Because SoGuiAnimWinCoreMainLauncher is a gui, it uses the internal gui messaging system.
	// This is why there are two message passing methods used here.
	// mdomowicz 2013_10_03

	SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
	GuiMessageForListener.AddHash( JpHash( "BackButtonPressed" ) );
	SendMessageToListeners( &GuiMessageForListener );

	if ( GetParentGui() )
	{
		SoGuiMessage GuiMessageForParent( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_PARENT );
		GuiMessageForParent.AddHash( JpHash( "BackButtonPressed" ) );
		SendMessageToParent( &GuiMessageForParent );
	}
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoGuiAnimWinRules::OnKeypadBackClicked()
{
	if ( IsShowing() )
	{
		HandleWindowExit();
	}

	return true;
}












