/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFontDefs.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimComboButton.h"
#include "SoGuiAnimWinSetupGame.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinSetupGame::SoGuiAnimWinSetupGame() :
_pLabel( NULL ),
_gButtonStart( 0 ),
_gButtonBack( 0 ),
_pLabelHideStock( NULL ),
_pLabelHideMoney( NULL ),
_gHideStock( 0 ),
_gHideMoney( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinSetupGame::~SoGuiAnimWinSetupGame()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinSetupGame::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgLarge.png" ) );

	SetSpriteAlpha( 0.85f );

	SetPositionToCenterOfParent();
	//SetWidth( 600.0f );
	//SetHeight( 100.0f );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_POP_UP );	



	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonStartUp.png" );
	ButtonParams._vRelativePos.Init( 200 * SoUtils::GetiPadRatioX(), -170 * SoUtils::GetiPadRatioY() );
	_gButtonStart = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonBackUp.png" );
	ButtonParams._vRelativePos.Init( -200 * SoUtils::GetiPadRatioX(), -170 * SoUtils::GetiPadRatioY() );
	_gButtonBack = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );



	_pLabel = CreateBmFontLabel( "Rules Options", SoFontDefs::GetGuiDefaultLarge() );
	_pLabel->setOpacity( 0 );
	//_pLabel->setVisible( false );

	_pLabelHideStock = CreateBmFontLabel( "", SoFontDefs::GetGuiDefaultLarge() );
	_pLabelHideStock->setOpacity( 0 );
	_pLabelHideStock->setString( "Show Stocks? " );
	_pLabelHideStock->setAnchorPoint( ccp( 0.5f, 0.5f ) );

	_pLabelHideMoney = CreateBmFontLabel( "", SoFontDefs::GetGuiDefaultLarge() );
	_pLabelHideMoney->setOpacity( 0 );
	_pLabelHideMoney->setString( "Show Money? " );
	_pLabelHideMoney->setAnchorPoint( ccp( 0.5f, 0.5f ) );


	SoGuiAnimComboButton::SetupParams ComboButtonParams;
	ComboButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ComboButtonParams._eButtonDownImageOption = SoGuiAnimComboButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;

	ComboButtonParams._vRelativePos.Init( 130 * SoUtils::GetiPadRatioX(), 55 * SoUtils::GetiPadRatioY() );
	_gHideStock = _pGuiManager->CreateAndAddAnimComboButton( ComboButtonParams, this );
	SoGuiAnimComboButton* pComboButton1 = (SoGuiAnimComboButton*)_pGuiManager->GetGuiByGuid( _gHideStock );

	ComboButtonParams._vRelativePos.Init( 130 * SoUtils::GetiPadRatioX(), -15 * SoUtils::GetiPadRatioY() );
	_gHideMoney = _pGuiManager->CreateAndAddAnimComboButton( ComboButtonParams, this );
	SoGuiAnimComboButton* pComboButton2 = (SoGuiAnimComboButton*)_pGuiManager->GetGuiByGuid( _gHideMoney );

	pComboButton1->SetupAndAddComboButtonValue( JpHash( "yes" ), SoUtils::AddRelativePathToTexture( "GuiButtonYesUp.png" ), "", "" );
	pComboButton1->SetupAndAddComboButtonValue( JpHash( "no" ), SoUtils::AddRelativePathToTexture( "GuiButtonNoUp.png" ), "", "" );

	pComboButton2->SetupAndAddComboButtonValue( JpHash( "no" ), SoUtils::AddRelativePathToTexture( "GuiButtonNoUp.png" ), "", "" );
	pComboButton2->SetupAndAddComboButtonValue( JpHash( "yes" ), SoUtils::AddRelativePathToTexture( "GuiButtonYesUp.png" ), "", "" );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinSetupGame::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinSetupGame::Update( float dt )
{
	SoGuiAnimWin::Update( dt );

	JpVector2f vLabelPos = _vAbsolutePos;
	vLabelPos.X -= 145.0f * SoUtils::GetiPadRatioX();
	vLabelPos.Y += 55.0f * SoUtils::GetiPadRatioY();

	_pLabelHideStock->SetPosition( vLabelPos );

	vLabelPos.Y -= 70.0f * SoUtils::GetiPadRatioY();
	_pLabelHideMoney->SetPosition( vLabelPos );


	vLabelPos = _vAbsolutePos;
	vLabelPos.X -= ( _pLabel->GetCalculatedWidthAllRes() / 2.0f );
	vLabelPos.Y += 135.0f * SoUtils::GetiPadRatioY();
	_pLabel->SetPosition( vLabelPos );
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinSetupGame::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonStart )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "SetupGameStartButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonBack )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "SetupGameBackButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}
		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  GetComboButtonValue
******************************************************************************/
hash SoGuiAnimWinSetupGame::GetComboButtonValue( int iButtonNumber )
{
	SoGui* pGui = NULL;

	switch ( iButtonNumber )
	{
		case 0: { pGui = _pGuiManager->GetGuiByGuid( _gHideStock ); break; }
		case 1: { pGui = _pGuiManager->GetGuiByGuid( _gHideMoney ); break; }
		default:
			ASSERTF( 0, ( "\nBad button number." ) );
	}


	SoGuiAnimComboButton* pComboButton = NULL;
	pComboButton = pGui->GetAsComboButton();

	if ( pComboButton )
	{
		SoComboButtonValue* pValue = pComboButton->GetCurrentComboButtonValue();
		if ( pValue )
			return pValue->GetValue();
	}

	return 0;
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoGuiAnimWinSetupGame::OnKeypadBackClicked()
{
	if ( IsShowing() )
	{
		SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
		GuiMessageForListener.AddHash( JpHash( "SetupGameBackButtonPressed" ) );
		SendMessageToListeners( &GuiMessageForListener );
	}

	return true;
}
