/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoGuiAnimButton.h"
#include "SoGuiAnimWinShellMain.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinShellMain::SoGuiAnimWinShellMain() :
_gButtonPlayOnline( 0 ),
_gButtonTutorial( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiWindowShellMain
******************************************************************************/
SoGuiAnimWinShellMain::~SoGuiAnimWinShellMain()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiAnimWinShellMain::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGuiAnimWin::Init( pParentNode, pGuiManager );

	//CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiWindowBgLarge.png" ) );

	SetSpriteAlpha( 0.85f );
	//SetPositionToCenterOfParent();
	SetRelativePos( JpVector2f( 0.0f, -100.0f * SoUtils::GetiPadRatioY() ) );

	//SetWidth( 600.0f );
	//SetHeight( 100.0f );

	SetTransitionStyle( SoGuiAnim::TRANSITION_STYLE_NONE );	



	SoGuiAnimButton::SetupParams ButtonParams;
	ButtonParams._hSoundEffectOnButtonUp = ( 0x5f1a7f2a /* "SfxSelect2.wav" */ );
	ButtonParams._eButtonDownImageOption = SoGuiAnimButton::BUTTON_DOWN_IMAGE_OPTION_SCALE_THE_UP_IMAGE_DOWN;


	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonPlayOnlineUp.png" );
	ButtonParams._vRelativePos.Init( 0 * SoUtils::GetiPadRatioX(), 102.5f * SoUtils::GetiPadRatioY() );
	_gButtonPlayOnline = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonPlayOfflineUp.png" );
	ButtonParams._vRelativePos.Init( 0 * SoUtils::GetiPadRatioX(), 27.5f * SoUtils::GetiPadRatioY() );
	_gButtonPlayOffline = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonOptionsUp.png" );
	ButtonParams._vRelativePos.Init( 0 * SoUtils::GetiPadRatioX(), -47.5f * SoUtils::GetiPadRatioY() );
	_gButtonOptions = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );

	ButtonParams._sPathAndFilenameButtonUp = SoUtils::AddRelativePathToTexture( "GuiButtonMoreUp.png" );
	ButtonParams._vRelativePos.Init( 0 * SoUtils::GetiPadRatioX(), -122.5f * SoUtils::GetiPadRatioY() );
	_gButtonMore = _pGuiManager->CreateAndAddAnimButton( ButtonParams, this );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiAnimWinShellMain::Deinit()
{
	SoGuiAnimWin::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiAnimWinShellMain::Update( float dt )
{
	SoGuiAnimWin::Update( dt );
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiAnimWinShellMain::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			if ( pMessage->GetSendersGuid() == _gButtonPlayOnline )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "PlayOnlineButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonPlayOffline )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "PlayOfflineButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonTutorial )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "TutorialButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonOptions )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "OptionsButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}

			if ( pMessage->GetSendersGuid() == _gButtonMore )
			{
				SoGuiMessage GuiMessageForListener( SoGuiMessage::TYPE_CUSTOM_DATA_FOR_LISTENER );
				GuiMessageForListener.AddHash( JpHash( "MoreButtonPressed" ) );
				SendMessageToListeners( &GuiMessageForListener );
			}


			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}
		default:
		{
			SoGuiAnimWin::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoGuiAnimWinShellMain::OnKeypadBackClicked()
{
	#if DCG_IS_TARGET_ANDROID
		if ( IsShowing() )
			CCDirector::sharedDirector()->end();
	#endif

	return true;
}
