/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoFontDefs.h"
#include "SoGameGlobals.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimTextEntry.h"
#include "SoGuiChat.h"
#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoNetworkComms.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoGuiChat
******************************************************************************/
SoGuiChat::SoGuiChat() :
_pTextEntry( NULL ),
_gTextEntry( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiChat
******************************************************************************/
SoGuiChat::~SoGuiChat()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiChat::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGui::Init( pParentNode, pGuiManager );

	SetSpriteAlpha( 0.5f );

	SetPositionToCenterOfParent();

	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	_HudLog.SetParentNode( pParentNode );
	_HudLog.SetFontName( SoFontDefs::GetTestSmall() );
	_HudLog.SetPositionAndSize( 0, 0, SizeInPixels.width, SizeInPixels.height );
	_HudLog.SetMessageShowDuration( -1.0f );
	_HudLog.SetFillHeight( 185.0f * SoUtils::GetiPadRatioY() );
	SetMaxOutputLines( 10 );

	_pTextEntry = new SoGuiAnimTextEntry;
	_pTextEntry->Setup( "", SoFontDefs::GetGuiDefault() );
	_gTextEntry = _pGuiManager->AddGui( _pTextEntry, this );
	_pTextEntry->CreateBackgroundSprite( SoUtils::AddRelativePathToTexture( "GuiTextEntryBg.png" ) );
	_pTextEntry->SetMaxCharacters( 20 );
	_pTextEntry->SetRelativePos( JpVector2f( 0.0f, 0.0f ) );
	_pTextEntry->SetSpriteAlpha( 0.25f );
	_pTextEntry->SetTextJustification( SoGuiAnimTextEntry::TEXT_JUSTIFICATION_LEFT );

	SoSprite* pTextEntryBgSprite = _pTextEntry->GetBackgroundSprite();
	pTextEntryBgSprite->setColor( ccc3( 0, 0, 0 ) );


	_InputVirtualKeyboardListener.SetOwner( this );
	SoInputVirtualKeyboard::GetInstance()->AddListener( &_InputVirtualKeyboardListener );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiChat::Deinit()
{
	SoGui::Deinit();

	_HudLog.Deinit();

	SoInputVirtualKeyboard::GetInstance()->RemoveListener( &_InputVirtualKeyboardListener );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiChat::Update( float dt )
{
	SoGui::Update( dt );

	float fX = _vAbsolutePos.X - ( ( _pTextEntry->GetWidth() / 2.0f ) * SoUtils::GetiPadRatioX() );
	float fY = _vAbsolutePos.Y - 220.0f * SoUtils::GetiPadRatioY();
	_HudLog.SetPosition( fX, fY );
	_HudLog.Update( dt );


	SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();
	SoNetworkChatText* pChatText = pNetwork->GetNextChatText();
	if ( pChatText )
	{
		_HudLog.AddMessage( pChatText->GetText().c_str(), JpUtils::_White );
		pNetwork->RemoveAndDeleteChatText( pChatText );
	}
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiChat::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_TRANSITION_IN:
		{
			SoGui::HandleMessage( pMessage );
			break;
		}
		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			SoGui::HandleMessage( pMessage );
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_DOWN:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP:
		{
			break;
		}
		case SoGuiMessage::TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE:
		{
			break;
		}
		default:
		{
			SoGui::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  OnVirtualKeyPressed
******************************************************************************/
void SoGuiChat::OnVirtualKeyPressed( char cKey )
{
	if ( cKey == '\n' )
	{
		std::string sText = _pTextEntry->GetContentText();
	
		if ( sText.size() > 0 )
		{
			SoNetworkComms* pNetwork = SoGameGlobals::GetNetworkComms();
			pNetwork->SendChatText( sText );

			_pTextEntry->ClearContentText();
		}
	}
}


