/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"


#include "JpHashTable.h"
#include "JpList.h"
#include "SoGui.h"
#include "SoGuiAnimButton.h"
#include "SoGuiAnimComboButton.h"
#include "SoGuiAnimSlider.h"
#include "SoGuiAnimToggle.h"
#include "SoInputDispatcher.h"
#include "SoInputGesture.h"
#include "SoInputKeypad.h"
#include "SoInputMouse.h"
#include "SoInputTouch.h"


#define MAX_GUIS_PER_MANAGER 512

#define SoGuiList JpList< SoGui >
#define SoGuiTable JpHashTable< SoGui, MAX_GUIS_PER_MANAGER * 2 >

USING_NS_CC;


class SoGuiManager
{
	public:

		//-----------------------
		// public enums

		// This enum is for specifying where keyboard/mouse/joystick/etc input data
		// comes from.  If it's "dispatcher", then input comes from SoInputDispatcher through
		// OnInput().  If it's "internal", then input comes from this classes own input callbacks.
		enum INPUT_SOURCE
		{
			INPUT_SOURCE_DISPATCHER,
			INPUT_SOURCE_INTERNAL,
			INPUT_SOURCE_LAST
		};

		//-----------------------
		// public functions

		SoGuiManager();

		virtual ~SoGuiManager();

		void Init( CCNode* pParentNode, INPUT_SOURCE eInputSource );

		void Deinit();

		void Update( float dt );

		void SetInputSource( INPUT_SOURCE e ) { _eInputSource = e; }

		guid AddGui( SoGui* pGuiToAdd, SoGui* pParentGui );
		guid CreateAndAddAnimButton( const SoGuiAnimButton::SetupParams& Params, SoGui* pParentGui );
		guid CreateAndAddAnimComboButton( const SoGuiAnimComboButton::SetupParams& Params, SoGui* pParentGui );
		guid CreateAndAddAnimSlider( const SoGuiAnimSlider::SetupParams& Params, SoGui* pParentGui );
		guid CreateAndAddAnimToggle( const SoGuiAnimToggle::SetupParams& Params, SoGui* pParentGui );

		void RemoveAndDeleteGui( SoGui* pGui );
		void RemoveAndDeleteAllGuis();
		void RemoveAndDeleteAllGuisNow();
		void DeleteAllGuisInPendingDeletionList();

		guid GenerateGuid();
		SoGui* GetGuiByGuid( guid g );
		SoGuiAnimButton* GetGuiAnimButtonByGuid( guid g ) { return GetGuiByGuid( g )->GetAsAnimButton(); }
		SoGuiAnimSlider* GetGuiAnimSliderByGuid( guid g ) { return GetGuiByGuid( g )->GetAsAnimSlider(); }

		SoGui* GetHeadGui() { return &_HeadGui; }
		void SetHeadGuiAbsolutionPositionToScreenCenter();

		void SendMessageToGui( guid gGui, SoGuiMessage* pMessage );

		// SoInputTouch callbacks
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList );

		// SoInputMouse callbacks
		#if DCG_IS_TARGET_WIN32
			virtual bool OnMousePressed( const OIS::MouseEvent &MouseEventArg, OIS::MouseButtonID MouseButtonId )  { return false; }
			virtual bool OnMouseReleased( const OIS::MouseEvent &MouseEventArg, OIS::MouseButtonID MouseButtonId ) { return false; }
			virtual bool OnMouseMoved( const OIS::MouseEvent &MouseEventArg );
		#endif

		// SoInputGesture callbacks
		virtual bool OnGestureDown( SoGesture* pGesture );
		virtual bool OnGestureUp( SoGesture* pGesture );
		virtual bool OnGestureMove( SoGesture* pGesture );

		// SoInputKeypad callbacks
		virtual bool OnKeypadBackClicked();
		virtual bool OnKeypadMenuClicked();


		// This is for connecting to SoInputDispatcher
		virtual bool OnInput( SoInputDispatcher::InputPacket* pInput );

		static SoGuiManager* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();


	private:

		//-----------------------
		// private variables

		static SoGuiManager* _pInstance;

		CCNode* _pParentNode;
		SoGui _HeadGui;

		SoGuiList _GuiList;
		SoGuiList _PendingDeletionList;

		SoGuiTable _GuiGuidTable;


		SoInputTouchListener< SoGuiManager > _InputTouchListener;
		SoInputGestureListener< SoGuiManager > _InputGestureListener;

		#if DCG_IS_TARGET_WIN32
			SoInputMouseListener< SoGuiManager > _InputMouseListener;
		#endif

		SoInputKeypadListener< SoGuiManager > _InputKeypadListener;


		INPUT_SOURCE _eInputSource;
};




