/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoGuiMessage.h"

#include "mmgrChooser.h"


USING_NS_CC;


SoGuiMessage SoGuiMessage::_BasicTransitionIn( SoGuiMessage::TYPE_TRANSITION_IN );
SoGuiMessage SoGuiMessage::_BasicTransitionOut( SoGuiMessage::TYPE_TRANSITION_OUT );




/******************************************************************************
 FUNCTION:  SoGameMessage
******************************************************************************/
SoGuiMessage::SoGuiMessage() :
_eType( TYPE_LAST ),
_gSendersGuid( 0 )
{
}



/******************************************************************************
 FUNCTION:  SoGameMessage
******************************************************************************/
SoGuiMessage::SoGuiMessage( TYPE eMessageType ) :
_eType( eMessageType ),
_gSendersGuid( 0 )
{
}



/******************************************************************************
 FUNCTION:  SoGameMessage
******************************************************************************/
SoGuiMessage::SoGuiMessage( TYPE eMessageType, guid gSendersGuid ) :
_eType( eMessageType ),
_gSendersGuid( gSendersGuid )
{
}



/******************************************************************************
 FUNCTION:  ~SoGameMessage
******************************************************************************/
SoGuiMessage::~SoGuiMessage()
{
	ClearDataList();
}



/******************************************************************************
 FUNCTION:  Clear
******************************************************************************/
void SoGuiMessage::Clear()
{
	SetType( TYPE_LAST );
	ClearDataList();
}



/******************************************************************************
 FUNCTION:  AddInt
******************************************************************************/
void SoGuiMessage::AddInt( int i )
{
	_DataList.AddLast( new Data( i ) );
}



/******************************************************************************
 FUNCTION:  AddFloat
******************************************************************************/
void SoGuiMessage::AddFloat( float f )
{
	_DataList.AddLast( new Data( f ) );
}



/******************************************************************************
 FUNCTION:  AddBool
******************************************************************************/
void SoGuiMessage::AddBool( bool b )
{
	int iData = 0;

	if ( b )
		iData = 1;

	AddInt( iData );
}



/******************************************************************************
 FUNCTION:  StartDataRead
******************************************************************************/
void SoGuiMessage::StartDataRead()
{
	_pDataListNode = _DataList.GetHead();
}



/******************************************************************************
 FUNCTION:  IsAtDataEnd
******************************************************************************/
bool SoGuiMessage::IsAtDataEnd()
{
	return ( _pDataListNode->GetNext() == _DataList.GetHead() );
}



/******************************************************************************
 FUNCTION:  FirstInt
******************************************************************************/
int SoGuiMessage::FirstInt()
{
	StartDataRead();
	return NextInt();
}



/******************************************************************************
 FUNCTION:  FirstFloat
******************************************************************************/
float SoGuiMessage::FirstFloat()
{
	StartDataRead();
	return NextFloat();
}



/******************************************************************************
 FUNCTION:  FirstBool
******************************************************************************/
bool SoGuiMessage::FirstBool()
{
	int iData = FirstInt();
	return ( iData != 0 );
}



/******************************************************************************
 FUNCTION:  NextInt
******************************************************************************/
int SoGuiMessage::NextInt()
{
	_pDataListNode = _pDataListNode->GetNext();

	ASSERTF( _pDataListNode != _DataList.GetHead(), ( "\nReading bad data." ) );

	if ( _pDataListNode != _DataList.GetHead() )
	{
		SoGuiMessage::Data* pData = _pDataListNode->GetData();
		return pData->GetInt();
	}

	return 0;
}



/******************************************************************************
 FUNCTION:  NextFloat
******************************************************************************/
float SoGuiMessage::NextFloat()
{
	_pDataListNode = _pDataListNode->GetNext();

	ASSERTF( _pDataListNode != _DataList.GetHead(), ( "\nReading bad data." ) );

	if ( _pDataListNode != _DataList.GetHead() )
	{
		SoGuiMessage::Data* pData = _pDataListNode->GetData();
		return pData->GetFloat();
	}

	return 0.0f;
}



/******************************************************************************
 FUNCTION:  NextBool
******************************************************************************/
bool SoGuiMessage::NextBool()
{
	int iData = NextInt();
	return ( iData != 0 );
}



/******************************************************************************
 FUNCTION:  CopyMessage
 NOTES:     To avoid a cascade of required consts, I'm just using a 
            custom copy function, rather than a copy constructor.
            mdomowicz 2013_02_05
******************************************************************************/
void SoGuiMessage::CopyMessage( SoGuiMessage& ToCopy )
{
	SetType( ToCopy.GetType() );

	SoGuiMessageDataList::Iterator Iter( ToCopy.GetDataList() );
	while ( Iter.Next() )
	{
		SoGuiMessage::Data* pData = Iter.GetNodeData();
		AddInt( pData->GetInt() );
	}
}



/******************************************************************************
 FUNCTION:  ClearDataList
******************************************************************************/
void SoGuiMessage::ClearDataList()
{
	SoGuiMessageDataList::Iterator Iter( &_DataList );
	while( Iter.Next() )
	{
		delete Iter.GetNodeData();
	}

	_DataList.RemoveAll();
}