/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

USING_NS_CC;


#include "JpHashTable.h"
#include "JpList.h"

#define SoGuiMessageList JpList< SoGuiMessage >
#define SoGuiMessageDataList JpList< SoGuiMessage::Data >


class SoGuiMessage
{
	public:

		//-----------------------
		// public enums and classes
		enum TYPE
		{
			TYPE_BUTTON_ON_DOWN,
			TYPE_BUTTON_ON_RUB,
			TYPE_BUTTON_ON_UP,
			TYPE_BUTTON_ON_UP_TOUCH_OUTSIDE,
			TYPE_COMBO_BUTTON_ON_DOWN,
			TYPE_COMBO_BUTTON_ON_RUB,
			TYPE_COMBO_BUTTON_ON_UP,
			TYPE_COMBO_BUTTON_ON_UP_TOUCH_OUTSIDE,
			TYPE_CUSTOM_DATA_FOR_LISTENER,
			TYPE_CUSTOM_DATA_FOR_PARENT,
			TYPE_MOUSE_WHEEL_DOWN,
			TYPE_MOUSE_WHEEL_UP,
			TYPE_TOGGLE_ON_ON,
			TYPE_TOGGLE_ON_OFF,
			TYPE_TRANSITION_IN,
			TYPE_TRANSITION_OUT,
			TYPE_LAST
		};


		class Data
		{
			public:
				Data() {}
				Data( float f ) { _uValue.f32 = f; }
				Data( int i ) { _uValue.i32 = i; }
				virtual ~Data() {}

				
				int GetInt() { return _uValue.i32; }
				float GetFloat() { return _uValue.f32; }
				
			private:
				union uValue
				{
					int	i32;
					float f32;
				};

				uValue _uValue;
		};


		//-----------------------
		// public variables
		static SoGuiMessage _BasicTransitionIn;
		static SoGuiMessage _BasicTransitionOut;


		//-----------------------
		// public functions

		SoGuiMessage();
		SoGuiMessage( TYPE eMessageType );
		SoGuiMessage( TYPE eMessageType, guid gSendersGuid );

		virtual ~SoGuiMessage();

		void Clear();

		guid GetSendersGuid() { return _gSendersGuid; }
		void SetSendersGuid( guid g ) { _gSendersGuid = g; }

		void AddInt( int i );
		void AddHash( hash h ) { AddInt( (int)h ); }
		void AddFloat( float f );
		void AddBool( bool b );

		void StartDataRead();
		bool IsAtDataEnd();

		int FirstInt();
		int NextInt();

		hash FirstHash() { return (hash)FirstInt(); }
		hash NextHash() { return (hash)NextInt(); }

		float FirstFloat();
		float NextFloat();

		bool FirstBool();
		bool NextBool();


		bool IsType( TYPE e ) { return _eType == e; }
		TYPE GetType() { return _eType; }
		void SetType( TYPE e ) { _eType = e; }		

		void CopyMessage( SoGuiMessage& ToCopy );

		SoGuiMessageDataList* GetDataList() { return &_DataList; }
		void ClearDataList();

	private:

		//-----------------------
		// private variables
		TYPE _eType;
		guid _gSendersGuid;
		SoGuiMessageDataList _DataList;
		SoGuiMessageDataList::Node* _pDataListNode;
};


