/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoGuiManager.h"
#include "SoGuiMessage.h"
#include "SoGuiScrollText.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoGuiScrollText
******************************************************************************/
SoGuiScrollText::SoGuiScrollText() :
_fScrollY( 0.0f ),
_vLineAnchorPoint( 0.0f, 0.0f ),
_fAutoScrollSpeedInPixelsPerSecond( 0.0f )
{
}



/******************************************************************************
 FUNCTION:  ~SoGuiScrollText
******************************************************************************/
SoGuiScrollText::~SoGuiScrollText()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoGuiScrollText::Init( CCNode* pParentNode, SoGuiManager* pGuiManager )
{
	SoGui::Init( pParentNode, pGuiManager );

	_fScrollY = 0.0f;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoGuiScrollText::Deinit()
{
	SoGui::Deinit();

	SoGuiScrollTextLineList::Iterator Iter( &_LineList );
	while ( Iter.Next() )
		delete Iter.GetNodeData();
	_LineList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoGuiScrollText::Update( float dt )
{
	SoGui::Update( dt );

	_fScrollY += dt * _fAutoScrollSpeedInPixelsPerSecond;

	float fTotalOffsetY = 0.0f;

	SoGuiScrollTextLineList::Iterator Iter( &_LineList );
	while ( Iter.Next() )
	{
		Line* pLine = Iter.GetNodeData();
		pLine->Update( dt );


		JpVector2f vPosition = _vAbsolutePos;

		fTotalOffsetY += pLine->GetTextHeight();
		fTotalOffsetY += pLine->GetExtraLineSpaceY();

		vPosition.Y -= fTotalOffsetY;
		vPosition.Y += _fScrollY;

		pLine->SetPosition( vPosition );

	}


	// Clamp the scroll to the height of the text
	if ( _fScrollY > fTotalOffsetY )
		_fScrollY = fTotalOffsetY;
}



/******************************************************************************
 FUNCTION:  HandleMessage
******************************************************************************/
void SoGuiScrollText::HandleMessage( SoGuiMessage* pMessage )
{
	switch ( pMessage->GetType() )
	{
		case SoGuiMessage::TYPE_TRANSITION_IN:
		{
			TransitionIn();
			SendMessageToAllChildren( pMessage );
			break;
		}
		case SoGuiMessage::TYPE_TRANSITION_OUT:
		{
			TransitionOut();
			SendMessageToAllChildren( pMessage );
			break;
		}
		default:
		{
			SoGui::HandleMessage( pMessage );
			break;
		}
	}
}



/******************************************************************************
 FUNCTION:  AddLineTypeA
******************************************************************************/
void SoGuiScrollText::AddLineTypeA( const std::string& sText )
{
	Line* pNewLine = new Line;
	pNewLine->SetOwner( this );
	pNewLine->SetText( sText, _sDefaultFontTypeA );
	_LineList.AddLast( pNewLine );
}



/******************************************************************************
 FUNCTION:  AddLineTypeB
******************************************************************************/
void SoGuiScrollText::AddLineTypeB( const std::string& sText )
{
	Line* pNewLine = new Line;
	pNewLine->SetOwner( this );
	pNewLine->SetText( sText, _sDefaultFontTypeB );
	_LineList.AddLast( pNewLine );
}



/******************************************************************************
 FUNCTION:  AddLineTypeC
******************************************************************************/
void SoGuiScrollText::AddLineTypeC( const std::string& sText )
{
	Line* pNewLine = new Line;
	pNewLine->SetOwner( this );
	pNewLine->SetText( sText, _sDefaultFontTypeC );
	_LineList.AddLast( pNewLine );
}



/******************************************************************************
 FUNCTION:  AddLineBlankCustomY
******************************************************************************/
void SoGuiScrollText::AddLineBlankCustomY( float fHeight )
{
	Line* pNewLine = new Line;
	pNewLine->SetOwner( this );
	pNewLine->SetExtraLineSpaceY( fHeight );
	_LineList.AddLast( pNewLine );
}



/******************************************************************************
 FUNCTION:  TransitionIn
******************************************************************************/
void SoGuiScrollText::TransitionIn()
{
}



/******************************************************************************
 FUNCTION:  TransitionOut
******************************************************************************/
void SoGuiScrollText::TransitionOut()
{
}



/******************************************************************************
 FUNCTION:  OnTouchesDown
******************************************************************************/
bool SoGuiScrollText::OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) 
{
	float fMargin = 0.0f;

	SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
	SoCcTouchList::Iterator Iter( pNonConstTouchList );
	while ( Iter.Next() )
	{
		CCTouch* pTouch = Iter.GetNodeData();
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoGuiScrollText::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{
	float fMargin = 0.0f;

	SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pSoCcTouchList;
	SoCcTouchList::Iterator Iter( pNonConstTouchList );
	while ( Iter.Next() )
	{
		CCTouch* pTouch = Iter.GetNodeData();
	}

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesMove
******************************************************************************/
bool SoGuiScrollText::OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) 
{
	return false;
}










/******************************************************************************
 FUNCTION:  Line::Init
******************************************************************************/
void SoGuiScrollText::Line::Init() 
{
}



/******************************************************************************
 FUNCTION:  Line::Deinit
******************************************************************************/
void SoGuiScrollText::Line::Deinit() 
{
}



/******************************************************************************
 FUNCTION:  Line::Update
******************************************************************************/
void SoGuiScrollText::Line::Update( float dt ) 
{
}



/******************************************************************************
 FUNCTION:  Line::SetText
******************************************************************************/
void SoGuiScrollText::Line::SetText( const std::string& sText, const std::string& sFontName ) 
{ 
	_sText = sText; 
	_pLabel = _pOwner->CreateBmFontLabel( sText, sFontName );

	_pLabel->setAnchorPoint( SoUtils::ToCcp( _pOwner->GetLineAnchorPoint() ) );
}



/******************************************************************************
 FUNCTION:  Line::SetPosition
******************************************************************************/
void SoGuiScrollText::Line::SetPosition( const JpVector2f& v )
{
	if ( _pLabel )
		_pLabel->SetPosition( v );
}



/******************************************************************************
 FUNCTION:  Line::GetTextHeight
******************************************************************************/
float SoGuiScrollText::Line::GetTextHeight()
{
	if ( _pLabel )
		return _pLabel->GetCalculatedHeightAllRes();

	return 0.0f;
}
