/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoGui.h"

USING_NS_CC;


#define SoGuiScrollTextLineList  JpList< SoGuiScrollText::Line >


class SoGuiScrollText : public SoGui
{
	public:

		//-----------------------
		// public classes
		class Line
		{
			public:
				Line() :
				_pLabel( NULL ),
				_pOwner( NULL ),
				_fExtraLineSpaceY( 0.0f )
				{ 
					Init(); 
				}

				virtual ~Line() { Deinit(); }

				void Init(); 
				void Deinit();
				void Update( float dt );

				void SetOwner( SoGuiScrollText* p ) { _pOwner = p; }

				void SetText( const std::string& sText, const std::string& sFontName );

				void SetPosition( const JpVector2f& v );

				float GetTextHeight();

				float GetExtraLineSpaceY() { return _fExtraLineSpaceY; }
				void SetExtraLineSpaceY( float f ) { _fExtraLineSpaceY = f; }

			private:

				std::string _sText;
				SoLabelBmFont* _pLabel;
				SoGuiScrollText* _pOwner;

				float _fExtraLineSpaceY;
		};


		//-----------------------
		// public functions

		SoGuiScrollText();

		virtual ~SoGuiScrollText();

		virtual void Init( CCNode* pParentNode, SoGuiManager* pGuiManager );

		virtual void Deinit();

		virtual void Update( float dt );

		virtual void HandleMessage( SoGuiMessage* pMessage );

		void AddLineTypeA( const std::string& sText );
		void AddLineBlankTypeA() { AddLineTypeA( " " ); }

		void AddLineTypeB( const std::string& sText );
		void AddLineBlankTypeB() { AddLineTypeB( " " ); }

		void AddLineTypeC( const std::string& sText );
		void AddLineBlankTypeC() { AddLineTypeC( " " ); }

		void AddLineBlankCustomY( float fHeight );

		const JpVector2f& GetLineAnchorPoint() { return _vLineAnchorPoint; }
		void SetLineAnchorPoint( const JpVector2f& v ) { _vLineAnchorPoint = v; }

		const std::string& GetDefaultFontTypeA() { return _sDefaultFontTypeA; }
		void SetDefaultFontTypeA( const std::string& s ) { _sDefaultFontTypeA = s; }

		const std::string& GetDefaultFontTypeB() { return _sDefaultFontTypeB; }
		void SetDefaultFontTypeB( const std::string& s ) { _sDefaultFontTypeB = s; }

		const std::string& GetDefaultFontTypeC() { return _sDefaultFontTypeC; }
		void SetDefaultFontTypeC( const std::string& s ) { _sDefaultFontTypeC = s; }

		void SetAutoScrollSpeedInPixelPerSecond ( float f ) { _fAutoScrollSpeedInPixelsPerSecond = f; }


		// SoInputTouch callbacks
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList );

	protected:

		//-----------------------
		// protected variables
		SoGuiScrollTextLineList _LineList;

		std::string _sDefaultFontTypeA;
		std::string _sDefaultFontTypeB;
		std::string _sDefaultFontTypeC;

		float _fScrollY;
		float _fAutoScrollSpeedInPixelsPerSecond;

		JpVector2f _vLineAnchorPoint;

		//-----------------------
		// protected functions
		void TransitionIn();
		void TransitionOut();
};




