/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCamera.h"
#include "SoFsmStateMain.h"
#include "SoHangdown.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


SoHangdownManager* SoHangdownManager::_pInstance = NULL;


/******************************************************************************
 FUNCTION:  SoHangdown
******************************************************************************/
SoHangdown::SoHangdown()
{
}



/******************************************************************************
 FUNCTION:  ~SoHangdown
******************************************************************************/
SoHangdown::~SoHangdown()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoHangdown::Init( CCNode* pParentNode, const char* pFilename )
{
	_pParentNode = pParentNode;
	_pSprite = CreateSprite( _pParentNode, SoUtils::AddRelativePathToTexture( pFilename ) );
	SoUtils::SetRenderZOrder( _pParentNode, _pSprite, BASE_RENDER_ORDER_HANGDOWN );

	_vScale.X = Interpolate( 1.2f, 2.2f, JpRandom::G.GetFloat() );
	_vScale.Y = Interpolate( 1.6f, 3.0f, JpRandom::G.GetFloat() );
	_pSprite->setScaleX( _vScale.X * SoUtils::GetiPadRatioX() * SoUtils::GetSpriteZoom() );
	_pSprite->setScaleY( _vScale.Y * SoUtils::GetiPadRatioY() * SoUtils::GetSpriteZoom() );

	/*
	SoGameDungeon* pDungeon = SoGameDungeon::GetInstance();
	SoTileMap* pMap = pDungeon->GetCurrentTileMap();

	_vPosition.X = JpRandom::G.GetInt()%pMap->GetMapWidth();
	_vPosition.Y = JpRandom::G.GetInt()%pMap->GetMapHeight();
	*/
	_vPosition.X = JpRandom::G.GetInt()%64;
	_vPosition.Y = JpRandom::G.GetInt()%64;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoHangdown::Deinit()
{
	_pParentNode->removeChild( _pSprite, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
	_pSprite->release();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHangdown::Update( float dt )
{
	_pSprite->setVisible( true );


	// Here we find the pixel position of the center of the current map
	// This is the center of the world, and we want to place the plane there.
	//SoGameDungeon* pDungeon = SoGameDungeon::GetInstance();
	//SoTileMap* pMap = pDungeon->GetCurrentTileMap();
	//JpVector2f vPos = pMap->GetTileByXz( _vPosition.X, _vPosition.Y )->GetWorldPositionFromMap();
	JpVector2f vPos( _vPosition.X, _vPosition.Y );

	// Here we find the plane's camera relative position, however, unlike with the normal sprites
	// once we know the position relative to the camera, we scale the position down.  This has the
	// effect of making the sprite move slower relative to the camera, and thus seem
	// farther away.
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->GetCurrentCamera();
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	float fRate = 2.0f;
	JpVector2f vCameraSpacePos = ( vPos - pCamera->GetPosition() ) * fRate;
	vCameraSpacePos.X *= SoUtils::GetSpriteZoom() * SoUtils::GetiPadRatioX();
	vCameraSpacePos.Y *= SoUtils::GetSpriteZoom() * SoUtils::GetiPadRatioY();

	JpVector2f vScreenOffset( ( SizeInPixels.width / 2.0f ), ( SizeInPixels.height / 2.0f ) );

	JpVector2f vFinal( vCameraSpacePos + vScreenOffset );

	_pSprite->SetPosition( vFinal );
	_pSprite->setScaleX( _vScale.X * SoUtils::GetiPadRatioX() * SoUtils::GetSpriteZoom() );
	_pSprite->setScaleY( _vScale.Y * SoUtils::GetiPadRatioY() * SoUtils::GetSpriteZoom() );
}



/******************************************************************************
 FUNCTION:  CreateSprite
******************************************************************************/
SoSprite* SoHangdown::CreateSprite( CCNode* pParent, const char* pacPathAndFilename )
{
	SoSprite* pSprite = SoSprite::CreateScaledToDevice( pacPathAndFilename );
	pSprite->retain();
	pParent->addChild( pSprite );
	pSprite->setVisible( false );
	pSprite->setAnchorPoint( ccp( 0.5f, 0.5f ) );

	return pSprite;
}



///////////////////////////////////////////////////////////////////////////////////////////////////////



/******************************************************************************
 FUNCTION:  Constructor
******************************************************************************/
SoHangdownManager::SoHangdownManager()
{
	_pInstance = NULL;
	_pParentNode = NULL;
	_bIsInited = false;
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoHangdownManager::CreateInstance()
{
	ASSERTF( !_pInstance, ( "\nCan't create a second instance." ) )
	if ( !_pInstance )
	{
		_pInstance = new SoHangdownManager;
	}
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoHangdownManager::DeleteInstance()
{
	if ( _pInstance )
	{
		_pInstance->DeleteAllHangdowns();

		delete _pInstance;
		_pInstance = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHangdownManager::Update( float dt )
{
	if ( _pParentNode && !_bIsInited )
	{
		_bIsInited = true;
	}

	//-----------------------------------------------------------

	SoHangdownList* pList = _pInstance->GetHangdownList();
	SoHangdownList::Iterator Iter( pList );
	while ( Iter.Next() )
	{
		SoHangdown* pPs = Iter.GetNodeData();

		pPs->Update( dt );
	}
}



/******************************************************************************
 FUNCTION:  DeleteAllParticleSystems
******************************************************************************/
void SoHangdownManager::DeleteAllHangdowns()
{
		SoHangdownList* pList = _pInstance->GetHangdownList();
		SoHangdownList::Iterator Iter( pList );
		while ( Iter.Next() )
		{
			SoHangdown* pPs = Iter.GetNodeData();

			delete pPs;
		}

		pList->RemoveAll();
}



/******************************************************************************
 FUNCTION:  CreateHangdowns
******************************************************************************/
void SoHangdownManager::CreateHangdowns( char* sFilename, int iNum )
{
	CCLayer* pSharedLayer = SoFsmStateMain::GetSharedMainLayer();

	if ( pSharedLayer )
	{
		for ( int i = 0; i < iNum; i ++ )
		{
			SoHangdown* pHangdown = new SoHangdown;
			pHangdown->Init( pSharedLayer, sFilename );

			GetHangdownList()->AddLast( pHangdown );
		}
	}
}












