/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoHudButtons.h"
#include "SoSoundManager.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoHudButtons
******************************************************************************/
SoHudButtons::SoHudButtons() :
_eState( STATE_LAST ),
_pParentNode( NULL ),
_pTargetingButton( NULL ),
_pCancelTargetingButton( NULL ),
_pFireButton( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoHudButtons
******************************************************************************/
SoHudButtons::~SoHudButtons()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoHudButtons::Init( CCNode* pParentNode )
{
	_eState = STATE_HERO_MOVEMENT;


	_UiButtonManager.Init( pParentNode );
	float fPosX = SoUtils::iPadPixelToPixelX( 0.0f );
	float fPosY = SoUtils::iPadPixelToPixelY( 0.0f );
	_UiButtonManager.setPosition( fPosX, fPosY );
	SoUtils::SetRenderZOrder( pParentNode, &_UiButtonManager, BASE_RENDER_ORDER_HUD );


	//---------------------
	// Targeting Button
	_pTargetingButton = _UiButtonManager.CreateButton
	( 
		"TargetingButton",
		SoUtils::AddRelativePathToTexture( "UiButtonHudTargeting.png" ), 
		SoUtils::AddRelativePathToTexture( "UiButtonHudTargetingPressed.png" )
	);
	fPosX = SoUtils::iPadPixelToPixelX( 512.0f );
	fPosY = SoUtils::iPadPixelToPixelY( 64.0f );
	_pTargetingButton->setPosition( fPosX, fPosY );
	_pTargetingButton->Hide();
	_pTargetingButton->TransitionIn();


	//---------------------
	// Cancel Targeting Button
	_pCancelTargetingButton = _UiButtonManager.CreateButton
	( 
		"CancelTargetingButton",
		SoUtils::AddRelativePathToTexture( "UiButtonHudCancelTargeting.png" ), 
		SoUtils::AddRelativePathToTexture( "UiButtonHudCancelTargetingPressed.png" )
	);
	fPosX = SoUtils::iPadPixelToPixelX( 512.0f );
	fPosY = SoUtils::iPadPixelToPixelY( 128.0f );
	_pCancelTargetingButton->setPosition( fPosX, fPosY );
	_pCancelTargetingButton->Hide();
	//_pCancelTargetingButton->TransitionIn();

	//---------------------
	// Fire Button
	_pFireButton = _UiButtonManager.CreateButton
	( 
		"FireButton",
		SoUtils::AddRelativePathToTexture( "UiButtonHudFire.png" ), 
		SoUtils::AddRelativePathToTexture( "UiButtonHudFirePressed.png" )
	);
	fPosX = SoUtils::iPadPixelToPixelX( 512.0f );
	fPosY = SoUtils::iPadPixelToPixelY( 64.0f );
	_pFireButton->setPosition( fPosX, fPosY );
	_pFireButton->Hide();
	//_pFireButton->TransitionIn();

}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoHudButtons::Deinit()
{
	_UiButtonManager.Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHudButtons::Update( float dt )
{
	_UiButtonManager.Update( dt );
}



/******************************************************************************
 FUNCTION:  OnInput
******************************************************************************/
void SoHudButtons::OnInput( SoInputDispatcher::InputPacket* pInput )
{
	SoInputDispatcher::InputPacket* pI = pInput; // for conciseness


	switch ( _eState )
	{
		case STATE_HERO_MOVEMENT:
		{
			bool bBeginTargeting = ( pI->_pButtonUp && pI->_pButtonUp->GetButtonName() == "TargetingButton" );
			#if DCG_IS_TARGET_WIN32
				bBeginTargeting |= pI->_eKeyPressed == OIS::KC_F;
			#endif

			if ( bBeginTargeting )
			{
				_pTargetingButton->TransitionOut();
				_pCancelTargetingButton->TransitionIn();
				_pFireButton->TransitionIn();
				_eState = STATE_TARGETING;
				SoSoundManager::GetInstance()->PlaySoundEffectCsvEventRandomPitch( SoSoundManager::TYPE_ONTOGGLETARGETINGON, NULL );
			}

			break;
		}
		case STATE_TARGETING:
		{
			bool bDoneTargeting = false;
			bDoneTargeting |= ( pI->_pButtonUp && pI->_pButtonUp->GetButtonName() == "FireButton" );
			bDoneTargeting |= ( pI->_pButtonUp && pI->_pButtonUp->GetButtonName() == "CancelTargetingButton" );
			#if DCG_IS_TARGET_WIN32
				bDoneTargeting |= pI->_eKeyPressed == OIS::KC_F;
				bDoneTargeting |= pI->_eKeyPressed == OIS::KC_ESCAPE;
			#endif

			if ( bDoneTargeting )
			{
				_pTargetingButton->TransitionIn();
				_pCancelTargetingButton->TransitionOut();
				_pFireButton->TransitionOut();
				_eState = STATE_HERO_MOVEMENT;
				SoSoundManager::GetInstance()->PlaySoundEffectCsvEventRandomPitch( SoSoundManager::TYPE_ONTOGGLETARGETINGOFF, NULL );
		}

			break;
		}

		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknonwn state" ) );
	}
}



/******************************************************************************
 FUNCTION:  AddListener
******************************************************************************/
void SoHudButtons::AddListener( SoUiButtonManager::Listener* pListener )
{
	_UiButtonManager.AddListener( pListener );
}



/******************************************************************************
 FUNCTION:  RemoveListener
******************************************************************************/
void SoHudButtons::RemoveListener( SoUiButtonManager::Listener* pListener )
{
	_UiButtonManager.RemoveListener( pListener );
}






