/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpString.h"

#include "SoActor.h"
#include "SoActorVisibleHotel.h"
#include "SoActorVisiblePlayer.h"
#include "SoFontDefs.h"
#include "SoGameBitHotel.h"
#include "SoGameBitPlayerHuman.h"
#include "SoGameClient.h"
#include "SoGameState.h"
#include "SoHudHotelStats.h"
#include "SoLabelBmFont.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoHudHotelStatLine
******************************************************************************/
SoHudHotelStatLine::SoHudHotelStatLine() :
_pParentNode( NULL ),
_vPosition( 0.0f, 0.0f )
{
}



/******************************************************************************
 FUNCTION:  ~SoHudHotelStatLine
******************************************************************************/
SoHudHotelStatLine::~SoHudHotelStatLine()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoHudHotelStatLine::Init( CCNode* pParentNode )
{
	_pParentNode = pParentNode;

	_vPosition.Init( 0.0f, 0.0f );

	_HudNumberSize.Init( pParentNode, 6, SoFontDefs::GetHotelStats() );
	_HudNumberSize.SetPosition( _vPosition );
	_HudNumberSize.SetCurrentNumber( 0 );
	_HudNumberSize.SetAnchorPoint( 0.5f, 0.5f );
	_HudNumberSize.SetSpacing( 10.0f );

	_HudNumberMajorBonus.Init( pParentNode, 6, SoFontDefs::GetHotelStats() );
	_HudNumberMajorBonus.SetPosition( _vPosition );
	_HudNumberMajorBonus.SetCurrentNumber( 0 );
	_HudNumberMajorBonus.SetAnchorPoint( 0.5f, 0.5f );
	_HudNumberMajorBonus.SetSpacing( 10.0f );

	_HudNumberMinorBonus.Init( pParentNode, 6, SoFontDefs::GetHotelStats() );
	_HudNumberMinorBonus.SetPosition( _vPosition );
	_HudNumberMinorBonus.SetCurrentNumber( 0 );
	_HudNumberMinorBonus.SetAnchorPoint( 0.5f, 0.5f );
	_HudNumberMinorBonus.SetSpacing( 10.0f );

	_HudNumberSharePrice.Init( pParentNode, 6, SoFontDefs::GetHotelStats() );
	_HudNumberSharePrice.SetPosition( _vPosition );
	_HudNumberSharePrice.SetCurrentNumber( 1 );
	_HudNumberSharePrice.SetAnchorPoint( 0.5f, 0.5f );
	_HudNumberSharePrice.SetSpacing( 10.0f );

	_HudNumberSharesLeft.Init( pParentNode, 6, SoFontDefs::GetHotelStats() );
	_HudNumberSharesLeft.SetPosition( _vPosition );
	_HudNumberSharesLeft.SetCurrentNumber( 0 );
	_HudNumberSharesLeft.SetAnchorPoint( 0.5f, 0.5f );
	_HudNumberSharesLeft.SetSpacing( 10.0f );

	for ( int i = 0; i < GAMERULES_MAX_PLAYERS; i ++ )
	{
		_aHudNumberPlayers[ i ].Init( pParentNode, 6, SoFontDefs::GetHotelStats() );
		_aHudNumberPlayers[ i ].SetPosition( SoUtils::_v2fOffWorld );
		_aHudNumberPlayers[ i ].SetCurrentNumber( 0 );
		_aHudNumberPlayers[ i ].SetAnchorPoint( 0.5f, 0.5f );
		_aHudNumberPlayers[ i ].SetSpacing( 10.0f );
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoHudHotelStatLine::Deinit()
{
	_HudNumberSize.Deinit();
	_HudNumberMajorBonus.Deinit();
	_HudNumberMinorBonus.Deinit();
	_HudNumberSharePrice.Deinit();
	_HudNumberSharesLeft.Deinit();

	for ( int i = 0; i < GAMERULES_MAX_PLAYERS; i ++ )
		_aHudNumberPlayers[ i ].Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHudHotelStatLine::Update( float dt )
{
	JpVector2f vOffset( 0.0f, 0.0f );

	vOffset.X += 0.0f * SoUtils::GetTrueiPadRatioX();
	vOffset.Y -= 0.0f * SoUtils::GetiPadRatioY();
	_HudNumberSize.SetPosition( _vPosition + vOffset );

	float fDeltaY = -30.0f * SoUtils::GetiPadRatioY();

	vOffset.Y += fDeltaY;
	_HudNumberMajorBonus.SetPosition( _vPosition + vOffset );

	vOffset.Y += fDeltaY;
	_HudNumberMinorBonus.SetPosition( _vPosition + vOffset );

	vOffset.Y += fDeltaY;
	_HudNumberSharePrice.SetPosition( _vPosition + vOffset );

	vOffset.Y += fDeltaY;
	_HudNumberSharesLeft.SetPosition( _vPosition + vOffset );


	if ( _pGame )
	{
		SoGameBitHotel* pHotel = _pGame->GetHotelByHotelId( _iHotelId );
		_HudNumberSharesLeft.SetTargetNumber( pHotel->GetAvailableStock() ); 

		_HudNumberSize.SetTargetNumber( pHotel->GetSize() );

		_HudNumberMajorBonus.SetTargetNumber( pHotel->GetMajorityBonusFromTable( _iHotelId, pHotel->GetSize() ) );

		_HudNumberMinorBonus.SetTargetNumber( pHotel->GetMinorityBonusFromTable( _iHotelId, pHotel->GetSize() ) );

		if ( pHotel->GetHotelId() != HOTEL_UNDEFINED_ID )
			_HudNumberSharePrice.SetTargetNumber( pHotel->GetPricePerStockFromTable( pHotel->GetHotelId(), pHotel->GetSize() ) );

		//-----------------------
		// If the hotel chain exists, make the numbers full opacity.
		// Otherwise dim them.
		if ( pHotel->IsPlaced() )
		{
			_HudNumberSharesLeft.SetAlpha( 1.0f );
			_HudNumberSize.SetAlpha( 1.0f );
			_HudNumberMajorBonus.SetAlpha( 1.0f );
			_HudNumberMinorBonus.SetAlpha( 1.0f );
			_HudNumberSharePrice.SetAlpha( 1.0f );
		}
		else
		{
			_HudNumberSharesLeft.SetAlpha( 0.15f );
			_HudNumberSize.SetAlpha( 0.15f );
			_HudNumberMajorBonus.SetAlpha( 0.15f );
			_HudNumberMinorBonus.SetAlpha( 0.15f );
			_HudNumberSharePrice.SetAlpha( 0.15f );
		}

		//-----------------------
		// Determine what are the "most owned" and "second most owned" 
		// amounts of the hotel stock
		int iMostStockOwned = 0;
		int iSecondMostStockOwned = 0;

		SoGameBitPlayerArray::Iterator PlayerIter( _pGame->GetPlayerArray() );
		while ( PlayerIter.Next() )
		{
			SoGameBitPlayer* pPlayer = PlayerIter.GetData();
			int iNumStocks = pPlayer->GetStockCount( _iHotelId );

			if ( iNumStocks > 0 )
			{
				if ( iNumStocks > iMostStockOwned )
				{
					iSecondMostStockOwned = iMostStockOwned;
					iMostStockOwned = iNumStocks;
				}
				else if ( iNumStocks > iSecondMostStockOwned )
				{
					iSecondMostStockOwned = iNumStocks;
				}
			}
		}

		for ( int i = 0; i < GAMERULES_MAX_PLAYERS; i ++ )
		{
			SoGameBitPlayer* pPlayer = _pGame->GetPlayerByPlayerNumber( i );

			if ( pPlayer )
			{
				if ( SoHudHotelStats::IsPlayersStockInfoVisible( _pGame, pPlayer ) )
				{
					SoHudNumber* pNumber = &_aHudNumberPlayers[ i ];

					vOffset.Y += fDeltaY;

					pNumber->SetVisible( true );
					pNumber->SetPosition( _vPosition + vOffset );

					int iStocksOwned = pPlayer->GetStockCount( _iHotelId );
					_aHudNumberPlayers[ i ].SetTargetNumber( iStocksOwned );

					if ( _pGame->IsVisibleStocks() )
					{
						if ( pPlayer->GetStockCount( _iHotelId ) == 0 )
						{
							pNumber->SetColor( JpUtils::_White );
							pNumber->SetScale( 1.0f * SoUtils::GetiPadRatio() );
							pNumber->SetSpacing( 10.0f );
							pNumber->SetAlpha( 0.15f );
						}
						else if ( pPlayer->GetStockCount( _iHotelId ) == iMostStockOwned )
						{
							pNumber->SetColor( JpUtils::_Green );
							pNumber->SetScale( 1.35f * SoUtils::GetiPadRatio() );
							pNumber->SetSpacing( 13.5f );
							pNumber->SetAlpha( 1.0f );
						}
						else if ( pPlayer->GetStockCount( _iHotelId ) == iSecondMostStockOwned )
						{
							pNumber->SetColor( JpUtils::_Red );
							pNumber->SetScale( 1.0f * SoUtils::GetiPadRatio() );
							pNumber->SetSpacing( 10.0f );
							pNumber->SetAlpha( 1.0f );
						}
						else
						{
							pNumber->SetColor( JpUtils::_White );
							pNumber->SetScale( 1.0f * SoUtils::GetiPadRatio() );
							pNumber->SetSpacing( 10.0f );
							pNumber->SetAlpha( 1.0f );
						}
					}
					else
					{
						if ( pPlayer->GetStockCount( _iHotelId ) == 0 )
						{
							pNumber->SetColor( JpUtils::_White );
							pNumber->SetScale( 1.0f * SoUtils::GetiPadRatio() );
							pNumber->SetSpacing( 10.0f );
							pNumber->SetAlpha( 0.15f );
						}
						else
						{
							pNumber->SetColor( JpUtils::_White );
							pNumber->SetScale( 1.0f * SoUtils::GetiPadRatio() );
							pNumber->SetSpacing( 10.0f );
							pNumber->SetAlpha( 1.0f );
						}
					}
				}
				else
				{
					_aHudNumberPlayers[ i ].SetVisible( false );
				}
			}
		}
	}


	for ( int i = 0; i < GAMERULES_MAX_PLAYERS; i ++ )
		_aHudNumberPlayers[ i ].Update( dt );

	_HudNumberSize.Update( dt );
	_HudNumberMajorBonus.Update( dt );
	_HudNumberMinorBonus.Update( dt );
	_HudNumberSharePrice.Update( dt );
	_HudNumberSharesLeft.Update( dt );
}











/******************************************************************************
 FUNCTION:  SoHudHotelStats
******************************************************************************/
SoHudHotelStats::SoHudHotelStats() :
_pParentNode( NULL ),
_pGame( NULL ),
_vPosition( 0.0f, 0.0f ),
_vTargetPosition( 0.0f, 0.0f ),
_pLabelSize( NULL ),
_pLabelAvailableShares( NULL ),
_pLabelPricePerShare( NULL ),
_pLabelMajorBonus( NULL ),
_pLabelMinorBonus( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoHudHotelStats
******************************************************************************/
SoHudHotelStats::~SoHudHotelStats()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoHudHotelStats::Init( CCNode* pParentNode )
{
	_pParentNode = pParentNode;


	_vStandardPosition.Init
	(
		HOTEL_STATS_STANDARD_POS_1_X * SoUtils::GetTrueiPadRatioX(),
		HOTEL_STATS_STANDARD_POS_1_Y * SoUtils::GetiPadRatioY()
	);


	JumpToStandardPosition();

	_pLabelSize = SoUtils::CreateBmFontLabel( _pParentNode, "Size", SoFontDefs::GetHotelStats(), kCCTextAlignmentLeft );
	_pLabelAvailableShares = SoUtils::CreateBmFontLabel( _pParentNode, "Bank", SoFontDefs::GetHotelStats(), kCCTextAlignmentLeft );
	_pLabelPricePerShare = SoUtils::CreateBmFontLabel( _pParentNode, "Cost", SoFontDefs::GetHotelStats(), kCCTextAlignmentLeft );
	_pLabelMajorBonus= SoUtils::CreateBmFontLabel( _pParentNode, "Lrg $", SoFontDefs::GetHotelStats(), kCCTextAlignmentLeft );
	_pLabelMinorBonus= SoUtils::CreateBmFontLabel( _pParentNode, "Sm $", SoFontDefs::GetHotelStats(), kCCTextAlignmentLeft );


	SoUtils::SetRenderZOrder( _pParentNode, _pLabelSize, BASE_RENDER_ORDER_HUD );
	SoUtils::SetRenderZOrder( _pParentNode, _pLabelAvailableShares, BASE_RENDER_ORDER_HUD );
	SoUtils::SetRenderZOrder( _pParentNode, _pLabelPricePerShare, BASE_RENDER_ORDER_HUD );
	SoUtils::SetRenderZOrder( _pParentNode, _pLabelMajorBonus, BASE_RENDER_ORDER_HUD );
	SoUtils::SetRenderZOrder( _pParentNode, _pLabelMinorBonus, BASE_RENDER_ORDER_HUD );


	for ( int i = 0; i < GAMERULES_MAX_PLAYERS; i ++ )
	{
		std::string sPlayerName = SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( i );
		_apLabelPlayers[ i ] = SoUtils::CreateBmFontLabel( _pParentNode, sPlayerName, SoFontDefs::GetHotelStats(), kCCTextAlignmentLeft );
		SoUtils::SetRenderZOrder( _pParentNode, _apLabelPlayers[ i ], BASE_RENDER_ORDER_HUD );
	}


	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		std::string sFilename = "HudHotelIcon";
		sFilename += JpString::IntToString( i );
		sFilename += ".png";

		_apHotelIconSprites[ i ] = SoUtils::CreateSprite( _pParentNode, SoUtils::AddRelativePathToTexture( sFilename.c_str() ) );


		_apHotelIconSprites[ i ]->SetPosition( _vPosition );
		_apHotelIconSprites[ i ]->setVisible( false );

		// using the iPadY ratio works better on super wide aspect ratio displays.
		_apHotelIconSprites[ i ]->setScale( 0.40f * SoUtils::GetiPadRatioY() ); 

		SoUtils::SetRenderZOrder( _pParentNode, _apHotelIconSprites[ i ], BASE_RENDER_ORDER_HUD );
	}

	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		std::string sFilename = "HudHotelBubble";
		sFilename += JpString::IntToString( i );
		sFilename += ".png";

		_apHotelBubbleSprites[ i ] = SoUtils::CreateSprite( _pParentNode, SoUtils::AddRelativePathToTexture( sFilename.c_str() ) );

		_apHotelBubbleSprites[ i ]->SetPosition( _vPosition );
		_apHotelBubbleSprites[ i ]->setVisible( false );
		_apHotelBubbleSprites[ i ]->setScale( 0.40f * SoUtils::GetiPadRatio() );
		SoUtils::SetRenderZOrder( _pParentNode, _apHotelBubbleSprites[ i ], BASE_RENDER_ORDER_HUD );
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoHudHotelStats::Deinit()
{
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoUtils::RemoveAndDestroyNode( _pParentNode, _apHotelIconSprites[ i ] );
		SoUtils::RemoveAndDestroyNode( _pParentNode, _apHotelBubbleSprites[ i ] );
	}

	SoHudHotelStatLineList::Iterator LineIter( &_StatLineList );
	while ( LineIter.Next() )
	{
		SoHudHotelStatLine* pLine = LineIter.GetNodeData();
		pLine->Deinit();
		delete pLine;
	}

	_StatLineList.RemoveAll();


	SoUtils::RemoveAndDestroyNode( _pParentNode, _pLabelSize );
	SoUtils::RemoveAndDestroyNode( _pParentNode, _pLabelAvailableShares );
	SoUtils::RemoveAndDestroyNode( _pParentNode, _pLabelPricePerShare );
	SoUtils::RemoveAndDestroyNode( _pParentNode, _pLabelMajorBonus );
	SoUtils::RemoveAndDestroyNode( _pParentNode, _pLabelMinorBonus );

	for ( int i = 0; i < GAMERULES_MAX_PLAYERS; i ++ )
		SoUtils::RemoveAndDestroyNode( _pParentNode, _apLabelPlayers[ i ] );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHudHotelStats::Update( float dt )
{
	_vPosition = InterpolateSCurve( _vPosition, _vTargetPosition, 1.0f, 4.0f, 10.0f * dt );

	//-----------------------
	// create the stat lines
	if ( _pGame && _StatLineList.GetNumEntries() == 0 )
	{
		for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
		{
			SoHudHotelStatLine* pNewLine = new SoHudHotelStatLine;
			pNewLine->Init( _pParentNode );
			pNewLine->SetHotelId( i );
			pNewLine->SetGameState( _pGame );
			_StatLineList.AddLast( pNewLine );
		}
	}



	//-----------------------
	// position the hotel icons
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoSprite* pSprite = _apHotelIconSprites[ i ];

		pSprite->setVisible( true );

		JpVector2f vOffset( 60.0f * SoUtils::GetTrueiPadRatioX(), 0.0f );

		vOffset.X += i * 40.0f * SoUtils::GetTrueiPadRatioX();
		vOffset.Y = 0.0f * SoUtils::GetiPadRatioY();
		pSprite->setVisible( true );
		pSprite->SetPosition( _vPosition + vOffset);


		// If the hotel chain exists, make the icon fully opaque.
		// Else make it partially alphaed out.
		if ( _pGame->GetHotelByHotelId( i )->IsPlaced() )
			pSprite->setOpacity( 255 );
		else
			pSprite->setOpacity( 0.5f * 255.0f );
	}

	//-----------------------
	// position the hotel name bubbles
	for ( int i = 0; i < GAMERULES_NUM_HOTELS; i ++ )
	{
		SoSprite* pSprite = _apHotelBubbleSprites[ i ];

		pSprite->setVisible( true );

		JpVector2f vOffset( 60.0f * SoUtils::GetTrueiPadRatioX(), 0.0f );

		vOffset.X += i * 40.0f * SoUtils::GetTrueiPadRatioX();
		vOffset.Y += 70.0f * SoUtils::GetiPadRatioY();
		pSprite->setVisible( true );
		pSprite->SetPosition( _vPosition + vOffset);


		// If the hotel chain exists, make the icon fully opaque.
		// Else make it partially alphaed out.
		if ( _pGame->GetHotelByHotelId( i )->IsPlaced() )
			pSprite->setOpacity( 255 );
		else
			pSprite->setOpacity( 0.25f * 255.0f );
	}


	//-----------------------
	// position the row labels

	_pLabelSize->setVisible( true );
	_pLabelAvailableShares->setVisible( true );
	_pLabelPricePerShare->setVisible( true );
	_pLabelMajorBonus->setVisible( true );
	_pLabelMinorBonus->setVisible( true );

	float fDeltaY = -30.0f * SoUtils::GetiPadRatioY();

	JpVector2f vOffset( 0.0f, -20.0f * SoUtils::GetiPadRatioY() );

	vOffset.Y += fDeltaY;
	_pLabelSize->SetPosition( _vPosition + vOffset );

	vOffset.Y += fDeltaY;
	_pLabelMajorBonus->SetPosition( _vPosition + vOffset );

	vOffset.Y += fDeltaY;
	_pLabelMinorBonus->SetPosition( _vPosition + vOffset );

	vOffset.Y += fDeltaY;
	_pLabelPricePerShare->SetPosition( _vPosition + vOffset );

	vOffset.Y += fDeltaY;
	_pLabelAvailableShares->SetPosition( _vPosition + vOffset );



	if ( _pGame )
	{
		for ( int i = 0; i < GAMERULES_MAX_PLAYERS; i ++ )
		{
			SoGameBitPlayer* pPlayer = _pGame->GetPlayerByPlayerNumber( i );

			if ( IsPlayersStockInfoVisible( _pGame, pPlayer ) )
			{
				_apLabelPlayers[ i ]->setVisible( true );

				vOffset.Y += fDeltaY;
				_apLabelPlayers[ i ]->SetPosition( _vPosition + vOffset );
			}
			else
			{
				_apLabelPlayers[ i ]->setVisible( false );
			}
		}
	}



	//-----------------------
	// position the stat lines
	float fDeltaX = 40.0f * SoUtils::GetTrueiPadRatioX();
	float fCurrentOffsetX = 60.0f * SoUtils::GetTrueiPadRatioX();;

	SoHudHotelStatLineList::Iterator LineIter( &_StatLineList );
	while ( LineIter.Next() )
	{
		SoHudHotelStatLine* pLine = LineIter.GetNodeData();
	
		JpVector2f vPos = _vPosition;
		vPos.X += fCurrentOffsetX;
		vPos.Y -= 40.0f * SoUtils::GetiPadRatioY();

		pLine->SetPosition( vPos );

		fCurrentOffsetX += fDeltaX;

		pLine->Update( dt );
	}
}



/******************************************************************************
 FUNCTION:  MoveToStandardPostion
******************************************************************************/
void SoHudHotelStats::MoveToStandardPostion()
{
	MoveToTargetPosition( _vStandardPosition );
}



/******************************************************************************
 FUNCTION:  JumpToStandardPosition
******************************************************************************/
void SoHudHotelStats::JumpToStandardPosition()
{
	JumpToTargetPosition( _vStandardPosition );
}



/******************************************************************************
 FUNCTION:  MoveToWindowOpenPosition
******************************************************************************/
void SoHudHotelStats::MoveToWindowOpenPosition()
{
	float fX = 560.0f * SoUtils::GetTrueiPadRatioX();
	float fY = 740.0f * SoUtils::GetiPadRatioY();
	JpVector2f vHotelStatsPos( fX, fY );
	MoveToTargetPosition( vHotelStatsPos );
}



/******************************************************************************
 FUNCTION:  IsPlayersStockInfoVisible
******************************************************************************/
bool SoHudHotelStats::IsPlayersStockInfoVisible( SoGameState* pGame, SoGameBitPlayer* pPlayer )
{
	bool bStockIsVisible = false;

	if ( pPlayer )
	{
		if ( pGame->IsVisibleStocks() || pGame->IsGameOver() )
		{
			bStockIsVisible = true;
		}
		else
		{
			SoGameBitPlayerHuman* pPlayerHuman = pPlayer->GetAsPlayerHuman();

			bool bThisPlayerIsHuman = ( pPlayerHuman != NULL );
			bool bThisPlayerIsCurrentPlayer = ( pGame->GetCurrentPlayerNumber() == pPlayer->GetPlayerNumber() );
			bool bViewTileIsToggledOn = ( pPlayerHuman && pPlayerHuman->GetIsViewTilesToggleOn() );

			int iNumHumansOnClient = pGame->GetOwningClient()->GetNumHumanPlayersOnThisClient();

			if ( iNumHumansOnClient == 1 )
			{
				bStockIsVisible = bThisPlayerIsHuman;
			}
			else if ( iNumHumansOnClient > 1 )
			{
				bStockIsVisible = ( bThisPlayerIsCurrentPlayer && bViewTileIsToggledOn );
			}
		}
	}

	return bStockIsVisible;
}
