/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoHudNumber.h"
#include "SoUtils.h"

USING_NS_CC;


#define HOTEL_STATS_STANDARD_POS_1_X  675.0f
#define HOTEL_STATS_STANDARD_POS_1_Y  520.0f
#define HOTEL_STATS_STANDARD_POS_2_X  1200.0f
#define HOTEL_STATS_STANDARD_POS_2_Y  520.0f


#define SoHudHotelStatLineList JpList< SoHudHotelStatLine >

class SoGameBitPlayer;
class SoGameState;
class SoSprite;


class SoHudHotelStatLine
{
	public:

		//-----------------------
		// public functions

		SoHudHotelStatLine();

		virtual ~SoHudHotelStatLine();

		void Init( CCNode* pParentNode );

		void Deinit();

		void Update( float dt );

		const JpVector2f& GetPosition() { return _vPosition; }
		void SetPosition( const JpVector2f& v ) { _vPosition = v; }

		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }

		SoGameState* GetGameState() { return _pGame; }
		void SetGameState( SoGameState* p ) { _pGame = p; }

		void SetHotelId( int i ) { _iHotelId = i; }
		int GetHotelId() { return _iHotelId; }


	private:

		//-----------------------
		// private variables
		CCNode* _pParentNode;
		SoGameState* _pGame;

		JpVector2f _vPosition;

		SoHudNumber _HudNumberSize;
		SoHudNumber _HudNumberMajorBonus;
		SoHudNumber _HudNumberMinorBonus;
		SoHudNumber _HudNumberSharePrice;
		SoHudNumber _HudNumberSharesLeft;

		SoHudNumber _aHudNumberPlayers[ GAMERULES_MAX_PLAYERS ];

		int _iHotelId;
};




class SoHudHotelStats
{
	public:

		//-----------------------
		// public functions

		SoHudHotelStats();

		virtual ~SoHudHotelStats();

		void Init( CCNode* pParentNode );

		void Deinit();

		void Update( float dt );

		const JpVector2f& GetPosition() { return _vPosition; }
		void SetPosition( const JpVector2f& v ) { _vPosition = v; }

		void MoveToTargetPosition( const JpVector2f& v ) { _vTargetPosition = v; }
		void JumpToTargetPosition( const JpVector2f& v ) { _vPosition = _vTargetPosition = v; }

		void MoveToStandardPostion();
		void JumpToStandardPosition();

		void MoveToWindowOpenPosition();

		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }

		SoGameState* GetGameState() { return _pGame; }
		void SetGameState( SoGameState* p ) { _pGame = p; }

		void SetStandardPosition( float fX, float fY ) { _vStandardPosition.Init( fX, fY ); }

		static bool IsPlayersStockInfoVisible( SoGameState* pGame, SoGameBitPlayer* pPlayer );

	private:

		//-----------------------
		// private variables
		CCNode* _pParentNode;

		SoSprite* _apHotelIconSprites[ GAMERULES_NUM_HOTELS ];
		SoSprite* _apHotelBubbleSprites[ GAMERULES_NUM_HOTELS ];

		SoGameState* _pGame;

		JpVector2f _vPosition;
		JpVector2f _vTargetPosition;

		JpVector2f _vStandardPosition;

		SoHudHotelStatLineList _StatLineList;


		SoLabelBmFont* _pLabelSize;
		SoLabelBmFont* _pLabelAvailableShares;
		SoLabelBmFont* _pLabelPricePerShare;
		SoLabelBmFont* _pLabelMajorBonus;
		SoLabelBmFont* _pLabelMinorBonus;
		SoLabelBmFont* _apLabelPlayers[ GAMERULES_MAX_PLAYERS ];
};
