/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "JpList.h"
#include "NeCsvOut.h"

USING_NS_CC;


#define SoLogMessageList JpList< SoLogMessage >

class SoLabelBmFont;

class SoLogMessage
{
	public:
		//-----------------------
		// public enums
		enum STATE
		{
			STATE_ANIMATE_IN,
			STATE_SHOW,
			STATE_ANIMATE_OUT,
			STATE_DONE,
			STATE_LAST
		};

		//-----------------------
		// public functions
		SoLogMessage();

		virtual ~SoLogMessage();

		void Deinit();
		void Update( float dt );

		JpVector2f GetPosition();
		void SetPosition( float fX, float fY );

		STATE GetCurrentState() { return _eCurrentState; }

		bool IsDone() { return _eCurrentState == STATE_DONE; }

		const ccColor3B& GetColor() { return _Color; }
		void SetColor( const ccColor3B& Color ) { _Color = Color; }

		SoLabelBmFont* CreateBmFontLabel( const std::string& sText, const std::string& sFontName );
		void DestroyBmFontLabel();
		SoLabelBmFont* GetBmFontLabel() { return _pBmFontLabel; }

		float GetMessageShowDuration() { return _fMessageShowDuration; }
		void SetMessageShowDuration( float fSeconds ) { _fMessageShowDuration = fSeconds; }

	private:

		//-----------------------
		// private variables
		std::string _sText;

		float _fTimer;
		float _fMessageShowDuration;

		SoLabelBmFont* _pBmFontLabel;

		ccColor3B _Color;

		STATE _eCurrentState;
};




class SoHudLog
{
	public:

		//-----------------------
		// public functions

		SoHudLog();

		virtual ~SoHudLog();

		void Init();

		void Deinit();

		void Update( float dt );

		void Clear();

		void AddMessage( const std::string& sMessage, const ccColor3B& Color );
		void AddMessageF( const ccColor3B& Color, const char* pFormat, ... );

		void SetPosition( float fBottomLeftX, float fBottomLeftY );
		void SetPositionAndSize( float fBottomLeftX, float fBottomLeftY, float fWidth, float fHeight );

		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }

		const std::string& GetFontName() { return _sFontName; }
		void SetFontName( const std::string& sFontName ) { _sFontName = sFontName; }

		// negative duration means, "show forever"
		float GetMessageShowDuration() { return _fMessageShowDuration; }
		void SetMessageShowDuration( float fSeconds ) { _fMessageShowDuration = fSeconds; }

		// use a negative value for unlimited messages.
		void SetMaxMessagesBeforeFifoRemoval( int i ) { _iMaxMessagesBeforeFifoRemoval = i; }

		bool IsVisible() { return _bIsVisible; }
		void SetVisible( bool b );

		void WriteFullHistoryToDisk();
		void ClearFullHistory() { _FullHistoryCsv.Clear(); }

		// See AdjustHeightsForFillHeight() implementation for an explanation for what 
		// this does, and when/why you might want to use it.
		void SetFillHeight( float f ) { _fFillHeight = f; }

	private:

		//-----------------------
		// private variables
		bool _bIsVisible;	

		SoLogMessageList _LogMessageList;

		float _fX;
		float _fY;
		float _fWidth;
		float _fHeight;
		float _fMessageShowDuration;
		int _iMaxMessagesBeforeFifoRemoval;

		float _fFillHeight;

		std::string _sFontName;

		NeCsvOut _FullHistoryCsv;

		CCNode* _pParentNode;


		//-----------------------
		// private functions
		float GetHeightOfAllEntries();
		void AdjustHeightsForFillHeight();
};




