/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpString.h"

#include "SoActor.h"
#include "SoHudNumber.h"
#include "SoLabelBmFont.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoHudDigit
******************************************************************************/
SoHudDigit::SoHudDigit() :
_pParentNode( NULL ),
_pOwnerNumber( NULL ),
_vNumberRelativePos( 0.0f, 0.0f ),
_fAnchorX( 0.0f ),
_fAnchorY( 0.0f ),
_fTimer( 0.0f ),
_eState( STATE_LAST ),
_iCurrentDigit( 0 ),
_iTargetDigit( 0 ),
_pCurrentDigit( NULL ),
_pTargetDigit( NULL ),
_bIsALeadingZero( false ),
_iRenderZOrder( -1 ),
_fMaxAlpha( 1.0f ),
_bIsVisible( true )
{
}



/******************************************************************************
 FUNCTION:  ~SoHudDigit
******************************************************************************/
SoHudDigit::~SoHudDigit()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoHudDigit::Init( CCNode* pParentNode )
{
	_pParentNode = pParentNode;

	_fTimer = 0.0f;
	_eState = STATE_WAIT;

	_iRenderZOrder = BASE_RENDER_ORDER_HUD;
	_Color = JpUtils::_White;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoHudDigit::Deinit()
{
	if ( _pCurrentDigit )
	{
		DestroyBmFontLabel( _pCurrentDigit );
		_pCurrentDigit = NULL;
	}

	if ( _pTargetDigit )
	{
		DestroyBmFontLabel( _pTargetDigit );
		_pTargetDigit = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHudDigit::Update( float dt )
{
	if ( _pCurrentDigit )
		_pCurrentDigit->setAnchorPoint( ccp( _fAnchorX, _fAnchorY ) );

	if ( _pTargetDigit )
		_pTargetDigit->setAnchorPoint( ccp( _fAnchorX, _fAnchorY ) );


	if ( _bIsALeadingZero && !_pOwnerNumber->ShowLeadingZeros() )
	{
		_pCurrentDigit->setVisible( false );
	}
	else
	{
		JpVector2f vParentPos = _pOwnerNumber->GetPosition();
		JpVector2f vPos = vParentPos + _vNumberRelativePos;

		_fTimer += dt;

		switch ( _eState )
		{
			case STATE_WAIT:
			{
				_pCurrentDigit->SetPosition( vPos );
				_pCurrentDigit->setVisible( true && _bIsVisible );
				_pCurrentDigit->setOpacity( _fMaxAlpha * 255.0f );

				if ( _iCurrentDigit != _iTargetDigit )
				{
					_fTimer = 0.0f;
					_pTargetDigit = CreateDigitLabel( _iTargetDigit );

					_eState = STATE_ANIMATE_TO_TARGET;
				}
				break;
			}

			case STATE_ANIMATE_TO_TARGET:
			{
				_pCurrentDigit->setVisible( true && _bIsVisible );
				_pTargetDigit->setVisible( true && _bIsVisible );

				float fFastInterpolator = Square( _fTimer * 15.0f );
				fFastInterpolator = Min( fFastInterpolator, 1.0f );


				float fSpeed = 5.0f;
				float fCurrentDigitAlpha = 1.0f - fFastInterpolator;
				fCurrentDigitAlpha = Clamp( fCurrentDigitAlpha, 0.0f, _fMaxAlpha );
				_pCurrentDigit->setOpacity( fCurrentDigitAlpha * 255.0f );

				float fTargetDigitAlpha = fFastInterpolator;
				fTargetDigitAlpha = Clamp( fTargetDigitAlpha, 0.0f, _fMaxAlpha );
				_pTargetDigit->setOpacity( fTargetDigitAlpha * 255.0f );


				float fVerticalMoveDistance = 10.0f * SoUtils::GetiPadRatioY();

				float fTargetStartY = vPos.Y + fVerticalMoveDistance;
				float fTargetEndY = vPos.Y;

				JpVector2f vTargetPos = vPos;
				vTargetPos.Y = Interpolate( fTargetStartY, fTargetEndY, fFastInterpolator );
				_pTargetDigit->SetPosition( vTargetPos );


				float fCurrentStartY = vPos.Y;
				float fCurrentEndY = vPos.Y - fVerticalMoveDistance;

				JpVector2f vCurrentPos = vPos;
				vCurrentPos.Y = Interpolate( fCurrentStartY, fCurrentEndY, fFastInterpolator );
				_pCurrentDigit->SetPosition( vCurrentPos );

				if ( fCurrentDigitAlpha == 0.0f )
				{
					DestroyBmFontLabel( _pCurrentDigit );

					_iCurrentDigit = _iTargetDigit;
					_pCurrentDigit = _pTargetDigit;
					_pTargetDigit = NULL;
					_fTimer = 0.0f;
					_eState = STATE_WAIT;
				}

				break;
			}

			default:
				ASSERTF( 0, ( "\nunknown state" ) );
		}	
	}
}



/******************************************************************************
 FUNCTION:  SetCurrentDigit
******************************************************************************/
void SoHudDigit::SetCurrentDigit( int i )
{
	_iCurrentDigit = i;

	if ( _pCurrentDigit )
	{
		DestroyBmFontLabel( _pCurrentDigit );
		_pCurrentDigit = NULL;
	}

	_pCurrentDigit = CreateDigitLabel( i );
}



/******************************************************************************
 FUNCTION:  SetTargetDigit
******************************************************************************/
void SoHudDigit::SetTargetDigit( int i )
{
	_iTargetDigit = i;
}



/******************************************************************************
 FUNCTION:  SetRenderZOrder
******************************************************************************/
void SoHudDigit::SetRenderZOrder( int iZWhereLargerMeansCloser )
{
	_iRenderZOrder = iZWhereLargerMeansCloser;

	if ( _pCurrentDigit )
		SoUtils::SetRenderZOrder( _pParentNode, _pCurrentDigit, iZWhereLargerMeansCloser );

	if ( _pTargetDigit )
		SoUtils::SetRenderZOrder( _pParentNode, _pTargetDigit, iZWhereLargerMeansCloser );
}



/******************************************************************************
 FUNCTION:  SetColor
******************************************************************************/
void SoHudDigit::SetColor( const ccColor3B& Color )
{
	_Color = Color;

	if ( _pCurrentDigit )
		_pCurrentDigit->setColor( Color );

	if ( _pTargetDigit )
		_pTargetDigit->setColor( Color );
}



/******************************************************************************
 FUNCTION:  SetVisible
******************************************************************************/
void SoHudDigit::SetVisible( bool b )
{
	_bIsVisible = b;
}



/******************************************************************************
 FUNCTION:  SetScale
******************************************************************************/
void SoHudDigit::SetScale( float f )
{
	if ( _pCurrentDigit )
		_pCurrentDigit->setScale( f );

	if ( _pTargetDigit )
		_pTargetDigit->setScale( f );
}



/******************************************************************************
 FUNCTION:  SetAlpha
******************************************************************************/
void SoHudDigit::SetAlpha( float f )
{
	_fMaxAlpha = f;
}



/******************************************************************************
 FUNCTION:  SetAnchorPoint
******************************************************************************/
void SoHudDigit::SetAnchorPoint( float fX, float fY )
{
	_fAnchorX = fX;
	_fAnchorY = fY;
}



/******************************************************************************
 FUNCTION:  CreateDigitLabel
******************************************************************************/
SoLabelBmFont* SoHudDigit::CreateDigitLabel( int iDigit )
{
	std::string sNumber = JpString::IntToString( iDigit );
	SoLabelBmFont* pLabel = CreateBmFontLabel( sNumber );

	JpVector2f vParentPos = _pOwnerNumber->GetPosition();
	JpVector2f vPos = vParentPos + _vNumberRelativePos;
	pLabel->SetPosition( vPos );

	
	SoUtils::SetRenderZOrder( _pParentNode, pLabel, _iRenderZOrder );
	pLabel->setColor( _Color );

	return pLabel;
}



/******************************************************************************
 FUNCTION:  CreateBmFontLabel
******************************************************************************/
SoLabelBmFont* SoHudDigit::CreateBmFontLabel( const std::string& sText )
{
	CCSize size = CCDirector::sharedDirector()->getWinSize();

	SoLabelBmFont* pLabel = SoLabelBmFont::create
	(
		sText.c_str(), 
		SoUtils::AddRelativePathToFont( _sFontName.c_str() ), 
		size.width, 
		kCCTextAlignmentLeft
	);
	pLabel->setVisible( false );
	pLabel->setOpacity( 255.0f );
	pLabel->retain();
	pLabel->setAnchorPoint( CCPoint( 0, 1 ) );
	pLabel->setScaleX( SoUtils::GetiPadRatioX() );
	pLabel->setScaleY( SoUtils::GetiPadRatioY() );

	_pParentNode->addChild( pLabel );
	SoUtils::SetRenderZOrder( _pParentNode, pLabel, _iRenderZOrder );

	return pLabel;
}



/******************************************************************************
 FUNCTION:  DestroyBmFontLabel
******************************************************************************/
void SoHudDigit::DestroyBmFontLabel( SoLabelBmFont* p )
{
	_pParentNode->removeChild( p, DCG_CLEANUP_ALL_RUNNING_ACTIONS );

	if ( p )
		p->release();
}










/******************************************************************************
 FUNCTION:  SoHudNumber
******************************************************************************/
SoHudNumber::SoHudNumber() :
_pParentNode( NULL ),
_iCurrentNumber( 0 ),
_iTargetNumber( 0 ),
_vPosition( 0.0f, 0.0f ),
_eState( STATE_LAST ),
_iMaxDigits( 0 ),
_bShowLeadingZeros( false ),
_fSpacing( 0.0f ),
_iRenderZOrder( -1 )
{
}



/******************************************************************************
 FUNCTION:  ~SoHudNumber
******************************************************************************/
SoHudNumber::~SoHudNumber()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoHudNumber::Init( CCNode* pParentNode, int iMaxDigitsInNumber, const std::string& sFontName )
{
	_pParentNode = pParentNode;

	_iMaxDigits = iMaxDigitsInNumber;
	_iRenderZOrder = BASE_RENDER_ORDER_HUD;
	_Color = JpUtils::_White;


	float fCurrentPos = 0.0f;
		
	for ( int i = 0; i < iMaxDigitsInNumber; i ++ )
	{
		SoHudDigit* pNewDigit = new SoHudDigit;
		pNewDigit->Init( pParentNode );
		pNewDigit->SetOwner( this );
		pNewDigit->SetFontName( sFontName );
		pNewDigit->SetCurrentDigit( 0 );
		pNewDigit->SetTargetDigit( 0 );
		pNewDigit->SetRenderZOrder( _iRenderZOrder );
		pNewDigit->SetColor( _Color );
		_DigitList.AddLast( pNewDigit );
	}

	_eState = STATE_WAIT;

	// Here we're just using a guessed value.  User should call this function
	// with the real value.
	RecalculateDigitSpacing( 20.0f ); 
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoHudNumber::Deinit()
{
	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		pDigit->Deinit();
		delete pDigit;
	}

	_DigitList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHudNumber::Update( float dt )
{
	FlagLeadingZeros();
	RecalculateDigitSpacing( _fSpacing );


	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		pDigit->Update( dt );
	}


	switch( _eState )
	{
		case STATE_WAIT:
		{
			if ( _iCurrentNumber != _iTargetNumber )
			{
				_eState = STATE_ANIMATE_INIT;
			}
			break;
		}

		case STATE_ANIMATE_INIT:
		{
			if ( _iCurrentNumber == _iTargetNumber )
			{
				_eState = STATE_WAIT;
			}
			else
			{
				float fDelta = 0.2f * ( _iTargetNumber - _iCurrentNumber );

				if ( fDelta > 0.0f )
					fDelta = Max( fDelta, 1.0f );
				else
					fDelta = Min( fDelta, -1.0f );

				_iCurrentNumber += (int)fDelta;

				SetNumberToAnimateTo( _iCurrentNumber );

				_eState = STATE_ANIMATE;
			}

			break;
		}

		case STATE_ANIMATE:
		{
			if ( IsDoneAnimating() )
			{
				_eState = STATE_ANIMATE_INIT;
			}

			break;
		}

		default:
			ASSERTF( 0, ( "\nUnknown state" ) );
	}
}



/******************************************************************************
 FUNCTION:  IsDoneAnimating
******************************************************************************/
bool SoHudNumber::IsDoneAnimating()
{
	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
	
		if ( pDigit->IsAnimating() )
			return false;
	}

	return true;
}



/******************************************************************************
 FUNCTION:  FlagLeadingZeros
******************************************************************************/
void SoHudNumber::FlagLeadingZeros()
{
	SoHudDigitList::Iterator Iter( &_DigitList );

	//-----------------------
	// flag all digits as not leading zeros
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		pDigit->SetIsALeadingZero( false );
	}

	//-----------------------
	// Then loop through and flag all zeros as leading
	// zeros, stopping when we hit the first non-zero
	Iter.Reset();
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		if ( pDigit->GetTargetDigit() == 0 )
			pDigit->SetIsALeadingZero( true );
		else
			break;
	}

	//-----------------------
	// Make sure the last digit is never flagged as a
	// leading zero.  This ensures that when the full
	// number is 0 (ie 000000), at least one zero is 
	// rendered.
	SoHudDigitList::Node* pLastNode = _DigitList.GetLast();
	if ( pLastNode != _DigitList.GetHead() )
	{
		SoHudDigit* pDigit = pLastNode->GetData();
		pDigit->SetIsALeadingZero( false );
	}
}



/******************************************************************************
 FUNCTION:  RecalculateDigitSpacing
******************************************************************************/
void SoHudNumber::RecalculateDigitSpacing( float fSpacing )
{
	SoHudDigitList::Iterator Iter( &_DigitList );

	float fCurrentX = 0.0f;

	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
	
		if ( _bShowLeadingZeros || !pDigit->IsALeadingZero() )
		{
			JpVector2f vDigitPos( fCurrentX, 0.0f );

			pDigit->SetNumberRelativePos( vDigitPos );

			fCurrentX += ( fSpacing * SoUtils::GetiPadRatioX() );
		}
	}
}



/******************************************************************************
 FUNCTION:  SetCurrentNumber
******************************************************************************/
void SoHudNumber::SetCurrentNumber( int i )
{
	_iCurrentNumber = i;
	_iTargetNumber = i;

	int iTemp = i;
	int iPlace = ( _iMaxDigits - 1 );

	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		ASSERTF( iPlace >= 0, ( "\nHudNumber calculation logic is faulty." ) );

		if ( iPlace >= 0 )
		{
			SoHudDigit* pDigit = Iter.GetNodeData();

			int iDivisor = powf( 10, iPlace );

			int iPlaceValue = ( iTemp / iDivisor );
			pDigit->SetCurrentDigit( iPlaceValue );
			pDigit->SetTargetDigit( iPlaceValue );

			iTemp -= iPlaceValue * ( iDivisor );
			iPlace --;
		}
	}
}



/******************************************************************************
 FUNCTION:  SetNumberToAnimateTo
******************************************************************************/
void SoHudNumber::SetNumberToAnimateTo( int i )
{
	int iTemp = i;
	int iPlace = ( _iMaxDigits - 1 );

	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		ASSERTF( iPlace >= 0, ( "\nHudNumber calculation logic is faulty." ) );

		if ( iPlace >= 0 )
		{
			SoHudDigit* pDigit = Iter.GetNodeData();

			int iDivisor = powf( 10, iPlace );

			int iPlaceValue = ( iTemp / iDivisor );
			pDigit->SetTargetDigit( iPlaceValue );

			iTemp -= iPlaceValue * ( iDivisor );
			iPlace --;
		}
	}
}



/******************************************************************************
 FUNCTION:  SetRenderZOrder
******************************************************************************/
void SoHudNumber::SetRenderZOrder( int iZWhereLargerMeansCloser )
{
	_iRenderZOrder = iZWhereLargerMeansCloser;

	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		pDigit->SetRenderZOrder( _iRenderZOrder );
	}
}



/******************************************************************************
 FUNCTION:  SetColor
******************************************************************************/
void SoHudNumber::SetColor( const ccColor3B& Color )
{
	_Color = Color;

	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		pDigit->SetColor( Color );
	}
}



/******************************************************************************
 FUNCTION:  SetVisible
******************************************************************************/
void SoHudNumber::SetVisible( bool b )
{
	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		pDigit->SetVisible( b );
	}
}



/******************************************************************************
 FUNCTION:  SetScale
******************************************************************************/
void SoHudNumber::SetScale( float f )
{
	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		pDigit->SetScale( f );
	}
}



/******************************************************************************
 FUNCTION:  SetAlpha
******************************************************************************/
void SoHudNumber::SetAlpha( float f )
{
	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		pDigit->SetAlpha( f );
	}
}



/******************************************************************************
 FUNCTION:  SetAnchorPoint
******************************************************************************/
void SoHudNumber::SetAnchorPoint( float fX, float fY )
{
	SoHudDigitList::Iterator Iter( &_DigitList );
	while ( Iter.Next() )
	{
		SoHudDigit* pDigit = Iter.GetNodeData();
		pDigit->SetAnchorPoint( fX, fY );
	}
}











