/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "NeCsvIn.h"

#include "JpHashTable.h"
#include "JpList.h"

USING_NS_CC;


#define SoHudDigitList    JpList< SoHudDigit >


class SoActor;
class SoLabelBmFont;
class SoHudNumber;


class SoHudDigit
{
	public:
		//-----------------------
		// public enums
		enum STATE
		{
			STATE_WAIT,
			STATE_ANIMATE_TO_TARGET,
			STATE_LAST
		};


		//-----------------------
		// public functions

		SoHudDigit();

		virtual ~SoHudDigit();

		void Init( CCNode* pParentNode );

		void SetOwner( SoHudNumber* p ) { _pOwnerNumber = p; }

		void Deinit();

		void Update( float dt );

		STATE GetState() { return _eState; }

		bool IsAnimating() { return _eState == STATE_ANIMATE_TO_TARGET; }
		
		int GetCurrentDigit() { return _iCurrentDigit; }
		void SetCurrentDigit( int i );
	
		int GetTargetDigit() { return _iTargetDigit; }
		void SetTargetDigit( int i );

		void SetFontName( const std::string& s ) { _sFontName = s; }

		void SetNumberRelativePos( const JpVector2f& v ) { _vNumberRelativePos = v; }

		bool IsALeadingZero() { return _bIsALeadingZero; }
		void SetIsALeadingZero( bool b ) { _bIsALeadingZero = b; }

		void SetRenderZOrder( int iZWhereLargerMeansCloser );
		void SetColor( const ccColor3B& Color );
		void SetVisible( bool b );
		void SetScale( float f );
		void SetAlpha( float f );
		void SetAnchorPoint( float fX, float fY );

	private:

		//-----------------------
		// private variables
		int _iCurrentDigit;
		int _iTargetDigit;

		SoLabelBmFont* _pCurrentDigit;
		SoLabelBmFont* _pTargetDigit;

		std::string _sFontName;

		CCNode* _pParentNode;

		STATE _eState;
		float _fTimer;

		SoHudNumber* _pOwnerNumber;
		JpVector2f _vNumberRelativePos;

		float _fAnchorX;
		float _fAnchorY;

		bool _bIsALeadingZero;

		int _iRenderZOrder;
		ccColor3B _Color;

		float _fMaxAlpha;
		bool _bIsVisible;

		//-----------------------
		// private functions
		SoLabelBmFont* CreateDigitLabel( int iDigit );

		SoLabelBmFont* CreateBmFontLabel( const std::string& sText );
		void DestroyBmFontLabel( SoLabelBmFont* p );
};




class SoHudNumber
{
	public:

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_WAIT,
			STATE_ANIMATE_INIT,
			STATE_ANIMATE,
			STATE_LAST
		};


		//-----------------------
		// public functions

		SoHudNumber();

		virtual ~SoHudNumber();

		void Init( CCNode* pParentNode, int iMaxDigitsInNumber, const std::string& sFontName );

		void Deinit();

		void Update( float dt );

		const JpVector2f& GetPosition() { return _vPosition; }
		void SetPosition( const JpVector2f& v ) { _vPosition = v; }

		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }

		void SetSpacing( float f ) { _fSpacing = f; }

		void SetCurrentNumber( int i );
		void SetTargetNumber( int i ) { _iTargetNumber = i; }
		int  GetTargetNumber() { return _iTargetNumber; }

		bool ShowLeadingZeros() { return _bShowLeadingZeros; }
		void SetShowLeadingZeros( bool b ) { _bShowLeadingZeros = b; }

		void SetRenderZOrder( int iZWhereLargerMeansCloser );

		void SetColor( const ccColor3B& Color );
		void SetVisible( bool b );
		void SetScale( float f );
		void SetAlpha( float f );
		void SetAnchorPoint( float fX, float fY );

	private:

		//-----------------------
		// private variables
		CCNode* _pParentNode;
		SoHudDigitList _DigitList;

		int _iCurrentNumber;
		int _iTargetNumber;

		JpVector2f _vPosition;

		STATE _eState;

		int _iMaxDigits;

		bool _bShowLeadingZeros;

		float _fSpacing;

		int _iRenderZOrder;
		ccColor3B _Color;

		//-----------------------
		// private functions
		
		void SetNumberToAnimateTo( int i );
		bool IsDoneAnimating();

		void FlagLeadingZeros();
		void RecalculateDigitSpacing( float fSpacing );
};





















