/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpString.h"

#include "SoActor.h"
#include "SoActorVisiblePlayer.h"
#include "SoCamera.h"
#include "SoCameraGameplay.h"
#include "SoConsole.h"
#include "SoFontDefs.h"
#include "SoGameBitPlayer.h"
#include "SoGameBitPlayerHuman.h"
#include "SoGameClient.h"
#include "SoGameState.h"
#include "SoHudPlayerGuide.h"
#include "SoLabelBmFont.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


#define PLAYER_GUIDE_ANIMATION_HEIGHT_OFFSET 30.0f


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoHudPlayerGuide
******************************************************************************/
SoHudPlayerGuide::SoHudPlayerGuide() :
_pParentNode( NULL ),
_pPortraitSprite( NULL ),
_pTrophySprite( NULL ),
_vPosition( 0.0f, 0.0f ),
_pPlayer( NULL ),
_pPlayerLabel( NULL ),
_vPlayerLabelOffset( 0.0f, 0.0f ),
_pDollarSignLabel( NULL ),
_iTouchState( 0 ),
_fTimer( 0.0f ),
_fOffsetY( 0.0f ),
_iAnimState( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoHudPlayerGuide
******************************************************************************/
SoHudPlayerGuide::~SoHudPlayerGuide()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoHudPlayerGuide::Init( CCNode* pParentNode )
{
	_pParentNode = pParentNode;

	float fX = 40.0f * SoUtils::GetiPadRatioX();
	float fY = 765.0f * SoUtils::GetiPadRatioY();
	_vPosition.Init( fX, fY );

	_HudNumberMoney.Init( pParentNode, 6, SoFontDefs::GetHudPlayerGuideScore() );
	_HudNumberMoney.SetSpacing( 15.0f );
	_HudNumberMoney.SetPosition( _vPosition );
	_HudNumberMoney.SetCurrentNumber( 10 );
	_HudNumberMoney.SetRenderZOrder( BASE_RENDER_ORDER_HUD + 1 );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoHudPlayerGuide::Deinit()
{
	_HudNumberMoney.Deinit();

	SoUtils::RemoveAndDestroyNode( _pParentNode, _pPortraitSprite );
	_pPortraitSprite = NULL;

	if ( _pTrophySprite )
	{
		SoUtils::RemoveAndDestroyNode( _pParentNode, _pTrophySprite );
		_pTrophySprite = NULL;
	}

	if ( _pPlayerLabel )
	{
		SoUtils::RemoveAndDestroyNode( _pParentNode, _pPlayerLabel );
		_pPlayerLabel = NULL;
	}

	if ( _pDollarSignLabel )
	{
		SoUtils::RemoveAndDestroyNode( _pParentNode, _pDollarSignLabel );
		_pDollarSignLabel = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHudPlayerGuide::Update( float dt )
{
	CreateAssets();


	JpVector2f vFinalPos = _vPosition;


	//-----------------------
	// This state machine handles animating the portrait image when it
	// is the players turn.
	if ( _pPlayer )
	{
		SoGameState* pGame = _pPlayer->GetOwningGame();

		_fTimer += dt;

		const int ANIM_STATE_WAIT = 0;
		const int ANIM_STATE_ANIMATE = 10;
		const int ANIM_STATE_COOLDOWN = 20;

		switch ( _iAnimState )
		{
			case ANIM_STATE_WAIT:
			{
				if 
				( 
					pGame->GetCurrentPlayerNumber() == _pPlayer->GetPlayerNumber() &&
					!pGame->IsGameOver()
				)
				{
					_fTimer = 0.0f;
					_iAnimState = ANIM_STATE_ANIMATE;
				}

				break;
			}

			case ANIM_STATE_ANIMATE:
			{
				_fOffsetY = InterpolateFastInSlowOut( 0.0f, PLAYER_GUIDE_ANIMATION_HEIGHT_OFFSET * SoUtils::GetiPadRatioY(), 1.0f, _fTimer * 2.5f );
				vFinalPos.Y += _fOffsetY;

				if 
				( 
					pGame->GetCurrentPlayerNumber() != _pPlayer->GetPlayerNumber() ||
					pGame->IsGameOver()
				)
				{
					_fTimer = 0.0f;
					_iAnimState = ANIM_STATE_COOLDOWN;
				}

				break;
			}

			case ANIM_STATE_COOLDOWN:
			{
				_fOffsetY -= ( _fTimer * 30.0f );
				_fOffsetY = ClampMin( _fOffsetY, 0.0f );
				vFinalPos.Y += _fOffsetY;

				if ( _fOffsetY == 0.0f )
				{
					_fTimer = 0.0f;
					_iAnimState = ANIM_STATE_WAIT;
				}

				break;
			}
		}
	}


	_pPortraitSprite->setVisible( true );
	_pPortraitSprite->SetPosition( vFinalPos );

	if ( _pPlayerLabel )
	{
		_pPlayerLabel->setVisible( true );
		_pPlayerLabel->SetPosition( vFinalPos + _vPlayerLabelOffset );
	}

	if ( _pPlayer )
	{
		SoGameState* pGame = _pPlayer->GetOwningGame();
		bool bMoneyIsVisible = false;

		if ( pGame->IsVisibleMoney() || pGame->IsGameOver() )
		{
			bMoneyIsVisible = true;
		}
		else
		{
			SoGameBitPlayerHuman* pPlayerHuman = _pPlayer->GetAsPlayerHuman();

			bool bThisPlayerIsHuman = ( pPlayerHuman != NULL );
			bool bThisPlayerIsCurrentPlayer = ( pGame->GetCurrentPlayerNumber() == _pPlayer->GetPlayerNumber() );
			bool bViewTileIsToggledOn = ( pPlayerHuman && pPlayerHuman->GetIsViewTilesToggleOn() );

			int iNumHumansOnClient = pGame->GetOwningClient()->GetNumHumanPlayersOnThisClient();

			if ( iNumHumansOnClient == 1 )
			{
				bMoneyIsVisible = bThisPlayerIsHuman;
			}
			else if ( iNumHumansOnClient > 1 )
			{
				bMoneyIsVisible = ( bThisPlayerIsCurrentPlayer && bViewTileIsToggledOn );
			}
		}


		JpVector2f vMoneyPos = vFinalPos;
		vMoneyPos.X += -30.0f * SoUtils::GetiPadRatioX();
		vMoneyPos.Y += 33.0f * SoUtils::GetiPadRatioY();
		_HudNumberMoney.SetVisible( bMoneyIsVisible );
		_HudNumberMoney.Update( dt );
		_HudNumberMoney.SetPosition( vMoneyPos );
		_HudNumberMoney.SetTargetNumber( _pPlayer->GetMoney() );


		if ( _pDollarSignLabel )
		{
			vMoneyPos.X += -5.0f * SoUtils::GetiPadRatioX();
			vMoneyPos.Y += 20.0f * SoUtils::GetiPadRatioY();

			_pDollarSignLabel->setVisible( bMoneyIsVisible );
			_pDollarSignLabel->SetPosition( vMoneyPos );
		}

		if ( pGame->IsGameOver() && _pPlayer->IsWinner() )
		{
			if ( !_pTrophySprite )
			{
				_pTrophySprite = SoUtils::CreateSprite( _pParentNode, SoUtils::AddRelativePathToTexture( "Star.png" ) );
				_pTrophySprite->setScale( 0.5f );
				_pTrophySprite->setVisible( true );
				SoUtils::SetRenderZOrder( _pParentNode, _pTrophySprite, BASE_RENDER_ORDER_HUD );			
				_fTimer = 0.0f;
			}


			JpVector2f vTrophyPos = vFinalPos;
			vTrophyPos.Y += 90.0f * SoUtils::GetiPadRatioY();

			float fOffsetY = sinf( _fTimer * 7.0f ) * 20.0f * SoUtils::GetiPadRatioY();
			fOffsetY = ClampMin( fOffsetY, 0.0f );
			vTrophyPos.Y += fOffsetY;

			_pTrophySprite->SetPosition( vTrophyPos );
		}
	}
}



/******************************************************************************
 FUNCTION:  CreateAssets
******************************************************************************/
void SoHudPlayerGuide::CreateAssets()
{
	if ( _pPlayer && !_pPortraitSprite )
	{
		SoGameBitPlayer::PLAYER_COLOR ePlayerColor = SoGameBitPlayer::GetPlayerColorByNumber( _pPlayer->GetPlayerNumber() );

		std::string sFilename = "";
		switch ( ePlayerColor )
		{
			case SoGameBitPlayer::PLAYER_COLOR_BLUE:   sFilename = "PlayerGuidePortraitBlue.png"; break;
			case SoGameBitPlayer::PLAYER_COLOR_GREEN:  sFilename = "PlayerGuidePortraitGreen.png"; break;
			case SoGameBitPlayer::PLAYER_COLOR_PURPLE: sFilename = "PlayerGuidePortraitPurple.png"; break;
			case SoGameBitPlayer::PLAYER_COLOR_RED:    sFilename = "PlayerGuidePortraitRed.png"; break;
			case SoGameBitPlayer::PLAYER_COLOR_WHITE:  sFilename = "PlayerGuidePortraitWhite.png"; break;
			case SoGameBitPlayer::PLAYER_COLOR_YELLOW: sFilename = "PlayerGuidePortraitYellow.png"; break;
			default:
				ASSERTF( 0, ( "\nUnknown player color." ) );
		}

		_pPortraitSprite = SoUtils::CreateSprite( _pParentNode, SoUtils::AddRelativePathToTexture( sFilename.c_str() ) );
		_pPortraitSprite->SetPosition( _vPosition );
		_pPortraitSprite->setVisible( false );
		SoUtils::SetRenderZOrder( _pParentNode, _pPortraitSprite, BASE_RENDER_ORDER_HUD );
	}

	if ( _pPlayer && !_pPlayerLabel )
	{
		char acText[ 32 ] = "";
		std::string sPlayerName = SoActorVisiblePlayer::GetPlayerNameByPlayerNumber( _pPlayer->GetPlayerNumber() );

		if ( _pPlayer->GetAsPlayerHuman() )
		{
			bool bThereIsOnlyOneHumanPlayer = ( _pPlayer->GetOwningGame()->GetOwningClient()->GetNumHumanPlayersOnThisClient() == 1 );

			if ( bThereIsOnlyOneHumanPlayer )
			{
				sprintf( acText, "%s\n(YOU)", sPlayerName.c_str() );
				_vPlayerLabelOffset.Init( 0.0f, -50.0f * SoUtils::GetiPadRatioY() );
			}
			else
			{
				sprintf( acText, "%s\nLOCAL", sPlayerName.c_str() );
				_vPlayerLabelOffset.Init( 0.0f, -50.0f * SoUtils::GetiPadRatioY() );
			}
		}
		else
		{
			sprintf( acText, "%s", sPlayerName.c_str() );
			_vPlayerLabelOffset.Init( 0.0f, -43.0f * SoUtils::GetiPadRatioY() );
		}

		_pPlayerLabel = SoUtils::CreateBmFontLabel( _pParentNode, acText, SoFontDefs::GetHudPlayerGuide(), kCCTextAlignmentCenter );
		_pPlayerLabel->setWidth( 1024.0f * SoUtils::GetiPadRatioX() );
		_pPlayerLabel->setOpacity( 255 );
		_pPlayerLabel->setScale( 0.75f * SoUtils::GetiPadRatio() );
		SoUtils::SetRenderZOrder( _pParentNode, _pPlayerLabel, BASE_RENDER_ORDER_HUD + 1 );
	}

	if ( !_pDollarSignLabel )
	{
		_pDollarSignLabel = SoUtils::CreateBmFontLabel( _pParentNode, "$", SoFontDefs::GetHudPlayerGuide(), kCCTextAlignmentCenter );
		_pDollarSignLabel->setWidth( 1024.0f * SoUtils::GetiPadRatioX() );
		_pDollarSignLabel->setOpacity( 255 );
		_pDollarSignLabel->setScale( 0.75f * SoUtils::GetiPadRatio() );
		SoUtils::SetRenderZOrder( _pParentNode, _pDollarSignLabel, BASE_RENDER_ORDER_HUD + 1 );
	}
}



/******************************************************************************
 FUNCTION:  OnInput
******************************************************************************/
void SoHudPlayerGuide::OnInput( SoInputDispatcher::InputPacket* pInput )
{
	const int STATE_UP = 0;
	const int STATE_DOWN = 10;

	switch ( _iTouchState )
	{
		case STATE_UP:
		{
			if ( pInput->_pSoCcTouchesDownList )
			{
				// get the position of the touch, in screen space
				JpVector2f vScreenPos( 0.0f, 0.0f );
				SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pInput->_pSoCcTouchesDownList;
				if ( pNonConstTouchList->GetNumEntries() > 0 )
				{
					CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
					vScreenPos = SoInputTouch::TouchPosToScreenPos( pTouch0->locationInView() );
				}

				if ( IsPositionInside( vScreenPos ) )
					_iTouchState = STATE_DOWN;
			}

			break;
		}

		case STATE_DOWN:
		{
			if ( pInput->_pSoCcTouchesUpList )
			{
				// get the position of the touch, in screen space
				JpVector2f vScreenPos( 0.0f, 0.0f );
				SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pInput->_pSoCcTouchesUpList;
				if ( pNonConstTouchList->GetNumEntries() > 0 )
				{
					CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
					vScreenPos = SoInputTouch::TouchPosToScreenPos( pTouch0->locationInView() );
				}

				if ( IsPositionInside( vScreenPos ) )
				{
					SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
					SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
					SoCameraGameplay* pCameraGameplay = pCamera->GetAsGameplay();

					//JpVector2f vTargetPos = SoActorVisiblePlayerBoard::GetBoardWorldPositionForPlayer( _pPlayer->GetPlayerNumber() );
					//pCameraGameplay->MoveCameraTargetPosition( vTargetPos );
				}

				_iTouchState = STATE_UP;
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  IsPositionInside
******************************************************************************/
bool SoHudPlayerGuide::IsPositionInside( const JpVector2f& vScreenPos )
{
	return IsPositionInsideSprite( vScreenPos, _pPortraitSprite );
}



/******************************************************************************
 FUNCTION:  IsPositionInsideSprite
******************************************************************************/
bool SoHudPlayerGuide::IsPositionInsideSprite( const JpVector2f& vScreenPos, SoSprite* pSprite )
{
	if ( pSprite )
	{
		float fHalfWidth = ( pSprite->getContentSize().width / 2 );
		float fHalfHeight = ( pSprite->getContentSize().height / 2 );

		return 
		( 
			vScreenPos.X > ( _vPosition.X - fHalfWidth ) &&
			vScreenPos.X < ( _vPosition.X + fHalfWidth ) &&
			vScreenPos.Y > ( _vPosition.Y - fHalfHeight ) &&
			vScreenPos.Y < ( _vPosition.Y + fHalfHeight )
		);
	}

	return false;
}



/******************************************************************************
 FUNCTION:  GetHudPositionFromPlayerNumber
******************************************************************************/
JpVector2f SoHudPlayerGuide::GetHudPositionFromPlayerNumber( int iPlayerNumber )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	int iScreenWidth = SizeInPixels.width;;

	JpVector2f vGuidePosition;
	vGuidePosition.X = iScreenWidth - ( 50.0f * SoUtils::GetTrueiPadRatioX() );
	vGuidePosition.Y = 75.0f * SoUtils::GetiPadRatioY();

	vGuidePosition.X -= 70.0f * SoUtils::GetiPadRatioX() * ( 5 - iPlayerNumber );
	return vGuidePosition;
}
