/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActor.h"
#include "SoHudSwooshNotice.h"
#include "SoSoundManager.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


SoHudSwooshNoticeManager* SoHudSwooshNoticeManager::_pInstance = NULL;



/******************************************************************************
 FUNCTION:  SoHudSwooshNotice
******************************************************************************/
SoHudSwooshNotice::SoHudSwooshNotice() :
_pParentNode( NULL ),
_pSprite( NULL ),
_fTimer( 0.0f ),
_fShowDuration( 0.0f ),
_eState( STATE_LAST ),
_vPos1( 0.0f, 0.0f ),
_vPos2( 0.0f, 0.0f ),
_vPos3( 0.0f, 0.0f ),
_hImageFilenameHash( 0 ),
_fQuickFadeOutTimer( 0.0f ),
_bQuickFadeOut( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoHudSwooshNotice
******************************************************************************/
SoHudSwooshNotice::~SoHudSwooshNotice()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoHudSwooshNotice::Init( CCNode* pParentNode, const std::string& sPortriteSpriteFilename )
{
	_pParentNode = pParentNode;
	_pSprite = SoUtils::CreateSprite( _pParentNode, sPortriteSpriteFilename.c_str() );
	SoUtils::SetRenderZOrder( _pParentNode, _pSprite, BASE_RENDER_ORDER_HUD_NOTICE );

	_fTimer = 0.0f;
	_eState = STATE_ANIMATE_IN;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoHudSwooshNotice::Deinit()
{
	_pParentNode->removeChild( _pSprite, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
	_pSprite->release();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHudSwooshNotice::Update( float dt )
{
	_fTimer += dt;

	switch ( _eState )
	{
		case STATE_ANIMATE_IN:
		{
			_pSprite->setVisible( true );

			float fTween = _fTimer * 1.5f;

			if ( _eType == TYPE_MIDDLE_LEFT_TO_SCREEN_CENTER )
				_vPos2.X += 20.0f * dt;
			else if ( _eType == TYPE_MIDDLE_RIGHT_TO_SCREEN_CENTER )
				_vPos2.X -= 20.0f * dt;

			float fX = InterpolateSCurve( _vPos1.X, _vPos2.X, 1.0f, 8.0f, fTween );
			float fY = InterpolateSCurve( _vPos1.Y, _vPos2.Y, 1.0f, 8.0f, fTween );

			_pSprite->setPositionX( fX );
			_pSprite->setPositionY( fY );

			if ( fTween > 2.0f )
			{
				_vPos2.Init( fX, fY );
				_fTimer = 0.0f;

				if ( _fShowDuration > 0.0f )
					_eState = STATE_SHOW;
				else
					_eState = STATE_ANIMATE_OUT;
			}

			break;
		}

		case STATE_SHOW:
		{
			if ( _fTimer > _fShowDuration )
			{
				_fTimer = 0.0f;
				_eState = STATE_ANIMATE_OUT;
			}

			break;
		}

		case STATE_ANIMATE_OUT:
		{
			float fTween = _fTimer * 3.0f;
			float fX = InterpolateSCurve( _vPos2.X, _vPos3.X, 1.0f, 4.0f, fTween );
			float fY = InterpolateSCurve( _vPos2.Y, _vPos3.Y, 1.0f, 4.0f, fTween );

			_pSprite->setPositionX( fX );
			_pSprite->setPositionY( fY );

			if ( fTween > 1.0f )
			{
				_fTimer = 0.0f;
				_eState = STATE_DONE;
			}

			break;
		}

		case STATE_ANIMATE_OUT_EARLY_INIT:
		{
			// Set the middle point to the current position so that
			// we can interpolate out from the sprites current position.
			_vPos2 = _pSprite->GetPosition();

			_fTimer = 0.0f;
			_eState = STATE_ANIMATE_OUT_EARLY;

			break;
		}
		case STATE_ANIMATE_OUT_EARLY:
		{
			float fTween = _fTimer * 5.0f;
			float fX = InterpolateSCurve( _vPos2.X, _vPos3.X, 1.0f, 4.0f, fTween );
			float fY = InterpolateSCurve( _vPos2.Y, _vPos3.Y, 1.0f, 4.0f, fTween );

			_pSprite->setPositionX( fX );
			_pSprite->setPositionY( fY );

			if ( fTween > 1.0f )
			{
				_fTimer = 0.0f;
				_eState = STATE_DONE;
			}
			break;
		}

		case STATE_DONE:
		{
			break;
		}
		default:
			ASSERTF( 0, ( "\nunknown state" ) );
	}	

	if ( _bQuickFadeOut )
	{
		_fQuickFadeOutTimer += dt * 5.0f;
		float fAlpha = ( 1.0f - _fQuickFadeOutTimer );
		fAlpha = Clamp( fAlpha, 0.0f, 1.0f );

		_pSprite->setOpacity( 255.0f * fAlpha );
	}
}



/******************************************************************************
 FUNCTION:  QuickExit
******************************************************************************/
void SoHudSwooshNotice::QuickExit( QUICK_EXIT_STYLE eQuickExitStyle )
{
	if ( eQuickExitStyle == QUICK_EXIT_STYLE_ANIMATE_OUT)
		_eState = STATE_ANIMATE_OUT_EARLY_INIT;
	else if ( eQuickExitStyle == QUICK_EXIT_STYLE_FADE_OUT )
		_bQuickFadeOut = true;
}



/******************************************************************************
 FUNCTION:  Renew
******************************************************************************/
void SoHudSwooshNotice::Renew()
{
	// Reset the timer if we are currently in the "show" state.
	if ( _eState == STATE_SHOW )
		_fTimer = 0.0f;
}



/******************************************************************************
 FUNCTION:  IsAnimatingOut
******************************************************************************/
bool SoHudSwooshNotice::IsAnimatingOut()
{
	return
	(
		_eState == STATE_ANIMATE_OUT ||
		_eState == STATE_ANIMATE_OUT_EARLY_INIT ||
		_eState == STATE_ANIMATE_OUT_EARLY ||
		_eState == STATE_DONE
	);
}



/******************************************************************************
 FUNCTION:  SetType
******************************************************************************/
void SoHudSwooshNotice::SetType( TYPE e )
{
	_eType = e;
	switch ( _eType )
	{
		case TYPE_TOP_LEFT:
		{
			_vPos1.Init( 1512.0f, 600.0f );
			_vPos2.Init( 180.0f, 600.0f );
			_vPos3.Init( -512.0f, 600.0f );
			break;
		}

		case TYPE_BOTTOM_RIGHT:
		{
			_vPos1.Init( -512.0f, 168.0f );
			_vPos2.Init( 844.0f, 168.0f );
			_vPos3.Init( 1512.0f, 168.0f );
			break;
		}

		case TYPE_MIDDLE_LEFT_TO_SCREEN_CENTER:
		{
			_vPos1.Init( -512.0f, 384.0f );
			_vPos2.Init( 512.0f, 384.0f );
			_vPos3.Init( 1512.0f, 384.0f );
			break;
		}

		case TYPE_MIDDLE_RIGHT_TO_SCREEN_CENTER:
		{
			_vPos1.Init( 1512.0f, 384.0f );
			_vPos2.Init( 512.0f, 384.0f );
			_vPos3.Init( -512.0f, 384.0f );
			break;
		}

		case TYPE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknown swoosh notice type" ) );
	}

	
	_vPos1.X *= SoUtils::GetTrueiPadRatioX();
	_vPos1.Y *= SoUtils::GetiPadRatioY();

	_vPos2.X *= SoUtils::GetTrueiPadRatioX();
	_vPos2.Y *= SoUtils::GetiPadRatioY();

	_vPos3.X *= SoUtils::GetTrueiPadRatioX();
	_vPos3.Y *= SoUtils::GetiPadRatioY();

	_pSprite->SetPosition( _vPos1 );
}








/******************************************************************************
 FUNCTION:  SoHudSwooshNoticeManager
******************************************************************************/
SoHudSwooshNoticeManager::SoHudSwooshNoticeManager() :
_pParentNode( NULL ),
_pNotice1( NULL ),
_pNotice2( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoHudSwooshNoticeManager
******************************************************************************/
SoHudSwooshNoticeManager::~SoHudSwooshNoticeManager()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoHudSwooshNoticeManager::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoHudSwooshNoticeManager::Deinit()
{
	SoHudSwooshNoticeList::Iterator Iter( &_NoticeList );
	while ( Iter.Next() )
	{
		SoHudSwooshNotice* pNotice = Iter.GetNodeData();
		pNotice->Deinit();
		delete pNotice;
	}

	_NoticeList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoHudSwooshNoticeManager::Update( float dt )
{
	SoHudSwooshNoticeList::Iterator Iter( &_NoticeList );
	while ( Iter.Next() )
	{
		SoHudSwooshNotice* pNotice = Iter.GetNodeData();

		if ( pNotice->IsDone() )
		{
			_NoticeList.Remove( pNotice );
			pNotice->Deinit();
			delete pNotice;

			if ( _pNotice1 == pNotice )
				_pNotice1 = NULL;

			if ( _pNotice2 == pNotice )
				_pNotice2 = NULL;
		}
		else
		{
			pNotice->Update( dt );
		}
	}
}



/******************************************************************************
 FUNCTION:  AddNotices
******************************************************************************/
void SoHudSwooshNoticeManager::AddNotices( const std::string& sImageFilename1, const std::string& sImageFilename2, float fShowDuration )
{
	// Any noticess that are currently showing are told to animate out 
	// as fast as possible.

	hash hImage1 = JpHash( sImageFilename1.c_str() );
	hash hImage2 = JpHash( sImageFilename2.c_str() );
	

	bool bNew1Needed = true;
	bool bNew2Needed = true;
	

	if ( _pNotice1 )
	{
		// If the current image1 is already showing...
		if ( _pNotice1->GetImageFilenameHash() == hImage1 )
		{
			// and it's not already on its way out
			if ( !_pNotice1->IsAnimatingOut() )
			{
				// tell it to stick around, 
				_pNotice1->Renew();

				// and make sure we don't make a new notice
				bNew1Needed = false;
			}
		}
	}

	if ( bNew1Needed )
	{
		if ( _pNotice1 )
			_pNotice1->QuickExit( SoHudSwooshNotice::QUICK_EXIT_STYLE_FADE_OUT );

		SoHudSwooshNotice* pNewNotice1 = new SoHudSwooshNotice;
		pNewNotice1->Init( _pParentNode, sImageFilename1.c_str() );
		pNewNotice1->SetType( SoHudSwooshNotice::TYPE_MIDDLE_LEFT_TO_SCREEN_CENTER );
		pNewNotice1->SetShowDuration( fShowDuration );
		_NoticeList.AddLast( pNewNotice1 );
		_pNotice1 = pNewNotice1;
	}

	if ( _pNotice2 )
	{
		// If the current image2 is already showing...
		if ( _pNotice2->GetImageFilenameHash() == hImage2 )
		{
			// and it's not already on its way out
			if ( !_pNotice2->IsAnimatingOut() )
			{
				// tell it to stick around, 
				_pNotice2->Renew();

				// and make sure we don't make a new notice
				bNew2Needed = false;
			}
		}
	}

	if ( bNew2Needed )
	{
		if ( _pNotice2 )
			_pNotice2->QuickExit( SoHudSwooshNotice::QUICK_EXIT_STYLE_FADE_OUT );

		SoHudSwooshNotice* pNewNotice2 = new SoHudSwooshNotice;
		pNewNotice2->Init( _pParentNode, sImageFilename2.c_str() );
		pNewNotice2->SetType( SoHudSwooshNotice::TYPE_MIDDLE_RIGHT_TO_SCREEN_CENTER );
		pNewNotice2->SetShowDuration( fShowDuration );
		_NoticeList.AddLast( pNewNotice2 );
		_pNotice2 = pNewNotice2;
	}


	SoSoundManager::GetInstance()->PlaySoundEffect( JpHash( "SfxNextPlayersTurn.wav" ) );
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoHudSwooshNoticeManager::CreateInstance()
{
	ASSERTF( _pInstance == NULL, ( "\nCan't create more than one instance.  Not allowed." ) );

	if ( !_pInstance )
	{
		_pInstance = new SoHudSwooshNoticeManager;
		_pInstance->Init();
	}
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoHudSwooshNoticeManager::DeleteInstance()
{
	if ( _pInstance )
	{
		delete _pInstance;
		_pInstance = NULL;
	}
}


