/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "NeCsvIn.h"

#include "JpHashTable.h"
#include "JpList.h"

USING_NS_CC;


#define SoHudSwooshNoticeList    JpList< SoHudSwooshNotice >


class SoActor;
class SoSprite;

class SoHudSwooshNotice
{
	public:
		//-----------------------
		// public enums
		enum STATE
		{
			STATE_ANIMATE_IN,
			STATE_SHOW,
			STATE_ANIMATE_OUT,
			STATE_ANIMATE_OUT_EARLY_INIT,
			STATE_ANIMATE_OUT_EARLY,
			STATE_DONE,
			STATE_LAST
		};

		enum TYPE
		{
			TYPE_BOTTOM_RIGHT,
			TYPE_TOP_LEFT,
			TYPE_MIDDLE_LEFT_TO_SCREEN_CENTER,
			TYPE_MIDDLE_RIGHT_TO_SCREEN_CENTER,
			TYPE_LAST
		};

		enum QUICK_EXIT_STYLE
		{
			QUICK_EXIT_STYLE_ANIMATE_OUT,
			QUICK_EXIT_STYLE_FADE_OUT,
			QUICK_EXIT_STYLE_LAST
		};

		//-----------------------
		// public functions

		SoHudSwooshNotice();

		virtual ~SoHudSwooshNotice();

		void Init( CCNode* pParentNode, const std::string& sPortriteSpriteFilename );

		void Deinit();

		void Update( float dt );

		void QuickExit( QUICK_EXIT_STYLE eQuickExitStyle );
		void Renew();

		bool IsDone() { return _eState == STATE_DONE; }
		bool IsAnimatingOut();

		void SetType( TYPE e );

		STATE GetState() { return _eState; }

		hash GetImageFilenameHash() { return _hImageFilenameHash; }

		void SetShowDuration( float f ) { _fShowDuration = f; }


	private:

		//-----------------------
		// private variables
		SoSprite* _pSprite;
		CCNode* _pParentNode;

		STATE _eState;
		float _fTimer;
		float _fShowDuration;

		JpVector2f _vPos1;
		JpVector2f _vPos2;
		JpVector2f _vPos3;

		TYPE _eType;

		hash _hImageFilenameHash;

		float _fQuickFadeOutTimer;
		bool _bQuickFadeOut;
};




class SoHudSwooshNoticeManager
{
	public:

		//-----------------------
		// public functions

		SoHudSwooshNoticeManager();

		virtual ~SoHudSwooshNoticeManager();

		void Init();

		void Deinit();

		void Update( float dt );

		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }

		void AddNotices( const std::string& sImageFilename1, const std::string& sImageFilename2, float fShowDuration );

		static SoHudSwooshNoticeManager* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();

	private:

		//-----------------------
		// private variables
		static SoHudSwooshNoticeManager* _pInstance;

		CCNode* _pParentNode;
		SoHudSwooshNoticeList _NoticeList;

		SoHudSwooshNotice* _pNotice1;
		SoHudSwooshNotice* _pNotice2;

};




