/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpString.h"

#include "SoActor.h"
#include "SoActorSorter.h"
#include "SoActorVisible.h"
#include "SoCamera.h"
#include "SoCameraGameplay.h"
#include "SoFontDefs.h"
#include "SoFsmStateMainCore.h"
#include "SoGameBit.h"
#include "SoGameBitPlayerHuman.h"
#include "SoGameClient.h"
#include "SoGameGlobals.h"
#include "SoHud.h"
#include "SoHudPlayerGuide.h"
#include "SoInputDispatcher.h"
#include "SoLabelBmFont.h"
#include "SoSprite.h"
#include "SoUtils.h"
#include "SoWorkerParkingSpot.h"

#include "mmgrChooser.h"


USING_NS_CC;


SoInputDispatcher* SoInputDispatcher::_pInstance = NULL;




/******************************************************************************
 FUNCTION:  SoInputDispatcher
******************************************************************************/
SoInputDispatcher::SoInputDispatcher() :
_pParentNode( NULL ),
_eState( STATE_LAST ),
_iNumHudButtonsDown( 0 ),
_pMovingActor( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoInputDispatcher
******************************************************************************/
SoInputDispatcher::~SoInputDispatcher()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoInputDispatcher::Init( CCNode* pParentNode )
{
	_pParentNode = pParentNode;

	_InputTouchListener.SetOwner( this );
	SoInputTouch::GetInstance()->AddListener( &_InputTouchListener );
	
	_InputGestureListener.SetOwner( this );
	SoInputGesture::GetInstance()->AddListener( &_InputGestureListener );

	_InputKeypadListener.SetOwner( this );
	SoInputKeypad::GetInstance()->AddListener( &_InputKeypadListener );

	#if DCG_IS_TARGET_WIN32
		_InputKeyboardListener.SetOwner( this );
		SoInputKeyboard::GetInstance()->AddListener( &_InputKeyboardListener );
		_eLastInputPressed = OIS::KC_UNASSIGNED;

		_InputMouseListener.SetOwner( this );
		SoInputMouse::GetInstance()->AddListener( &_InputMouseListener );
	#endif


	_UiButtonListener.SetOwner( this );
	SoHud* pHud = SoHud::GetInstance();
	pHud->GetHudButtons()->AddListener( &_UiButtonListener );	

	_eState = STATE_CAMERA_PANNING;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoInputDispatcher::Deinit()
{
	SoHud* pHud = SoHud::GetInstance();
	pHud->GetHudButtons()->RemoveListener( &_UiButtonListener );	

	SoInputTouch::GetInstance()->RemoveListener( &_InputTouchListener );
	SoInputKeypad::GetInstance()->RemoveListener( &_InputKeypadListener );

	#if DCG_IS_TARGET_WIN32
		SoInputKeyboard::GetInstance()->RemoveListener( &_InputKeyboardListener );
		SoInputMouse::GetInstance()->RemoveListener( &_InputMouseListener );
	#endif

	SoInputTouch::GetInstance()->RemoveListener( &_InputTouchListener );
	SoInputGesture::GetInstance()->RemoveListener( &_InputGestureListener );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoInputDispatcher::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  MainDisptach
******************************************************************************/
void SoInputDispatcher::MainDisptach( InputPacket* pInputPacket )
{
	SoHud* pHud = SoHud::GetInstance();
	SoHudButtons* pHudButtons = pHud->GetHudButtons();
	InputPacket* pI = pInputPacket; // for conciseness


	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCameraGameplay* pCameraGameplay = NULL;

	if ( pCameraManager )
	{
		SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );
		pCameraGameplay = pCamera->GetAsGameplay();
	}


	switch ( _eState )
	{
		case STATE_CAMERA_PANNING:
		{
			bool bCustomGuiProcessedInput = false;
			if ( SoFsmStateMainCore::GetCustomGuiManager() )
				bCustomGuiProcessedInput |= SoFsmStateMainCore::GetCustomGuiManager()->OnInput( pInputPacket );

			bool bGuiProcessedInput =false;
			if ( !bCustomGuiProcessedInput )
				bGuiProcessedInput |= SoGuiManager::GetInstance()->OnInput( pInputPacket );


			if ( !bGuiProcessedInput && !bCustomGuiProcessedInput )
			{
				// Experimenting with disabling player control of the camera.
				// This seems better for Achoir.  mdomowicz 2014_02_02
				//if ( pCameraGameplay )
				//	pCameraGameplay->OnInput( pI );


				bool bParkingSpotIsPressed = SendInputToParkingSpots( pI );


				SoHudPlayerGuideList::Iterator Iter( SoHud::GetInstance()->GetHudPlayerGuideList() );
				while ( Iter.Next() )
				{
					SoHudPlayerGuide* pGuide = Iter.GetNodeData();
					pGuide->OnInput( pI );
				}


				_pMovingActor = DetectTouchedDownOnActor( pI );

				if ( !bParkingSpotIsPressed && _pMovingActor )
				{
					_pMovingActor->OnInput( pI );
					_eState = STATE_MOVING_ACTOR;
				}

			}
			break;
		}
		case STATE_MOVING_ACTOR:
		{
			if ( pCameraGameplay )
			{
				if ( _pMovingActor->GetAsVisibleTile() )
				{
					pCameraGameplay->SetState( SoCameraGameplay::STATE_EDGE_PAN );
					pCameraGameplay->OnInput( pI );
				}
			}

			_pMovingActor->OnInput( pI );

			if ( pI->_pSoCcTouchesUpList )
			{
				if ( pCameraGameplay )
					pCameraGameplay->SetState( SoCameraGameplay::STATE_STANDARD );

				_eState = STATE_CAMERA_PANNING;
			}

			break;
		}
		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\unknown state" ) );
	}
}



/******************************************************************************
 FUNCTION:  SendInputToParkingSpots
******************************************************************************/
bool SoInputDispatcher::SendInputToParkingSpots( InputPacket* pInputPacket )
{
	int iNumParkingSpotsPressed = 0;

	SoGameClientList* pClientList = SoGameGlobals::GetClientList();

	// This needs to be fixed if we ever want to support more than one local client.
	ASSERTF( pClientList->GetNumEntries() <= 1, ( "\nRight now this only works for one local client." ) );

	SoGameClientList::Node* pFirstNode = pClientList->GetFirst();
	SoGameClient* pClient = pFirstNode->GetData();
	if ( pClient )
	{
		SoGameState* pGame = pClient->GetGameState();

		if ( pGame )
		{
			SoGameBitPlayerArray* pPlayerArray = pGame->GetPlayerArray();
			SoGameBitPlayerArray::Iterator PlayerIter( pPlayerArray );

			while ( PlayerIter.Next() )
			{
				SoGameBit* pGameBit = PlayerIter.GetData();
				SoGameBitPlayerHuman* pPlayerHuman = pGameBit->GetAsPlayerHuman();

				if ( pPlayerHuman )
				{
					SoWorkerParkingSpotList* pParkingSpotList = pPlayerHuman->GetWorkerParkingSpotList();
					SoWorkerParkingSpotList::Iterator ParkingSpotIter( pParkingSpotList );

					while ( ParkingSpotIter.Next() )
					{
						SoWorkerParkingSpot* pSpot = ParkingSpotIter.GetNodeData();
						if ( pSpot->OnInput( pInputPacket ) )
						{
							iNumParkingSpotsPressed ++;
						}	
					}		
				}
			}
		}
	}

	return ( iNumParkingSpotsPressed > 0 );
}



/******************************************************************************
 FUNCTION:  DetectTouchedDownOnActor
******************************************************************************/
SoActorVisible* SoInputDispatcher::DetectTouchedDownOnActor( InputPacket* pInputPacket )
{
	InputPacket* pI = pInputPacket; // for conciseness

	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();

	if ( pCameraManager )
	{
		SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );

		if ( pI->_pSoCcTouchesDownList )
		{
			//-----------------------
			// get the position of the touch, in screen space
			JpVector2f vScreenPos( 0.0f, 0.0f );
			SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pI->_pSoCcTouchesDownList;
			if ( pNonConstTouchList->GetNumEntries() > 0 )
			{
				CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
				vScreenPos = SoInputTouch::TouchPosToScreenPos( pTouch0->locationInView() );
			}

			//-----------------------
			// convert to world space.
			JpVector2f vWorldPos = pCamera->TransformScreenToWorldSpace( vScreenPos );



			//-----------------------
			// Get the list of movable actors and copy it into a new list.
			// Sort the actors in the new list by the actors' render order.
			SoActorList* pMovableActors = SoActorSorter::GetInstance()->GetMovableActorsList();
			SoActorList::Iterator Iter( pMovableActors );

			SoActorList MovableActorsSorted;
			while ( Iter.Next() )
			{
				SoActor* pActor = Iter.GetNodeData();
				SoActorVisible* pActorVisible = pActor->GetAsVisible();
				if ( pActorVisible )
				{
					MovableActorsSorted.AddLast( pActorVisible );
					SoActorList::Node* pLast = MovableActorsSorted.GetLast();
					pLast->SetSortKey( pActorVisible->GetRenderOrder() );
				}
			}
			MovableActorsSorted.QuickSort();


			//-----------------------
			// Now iterate through the sorted actor list.
			// As soon as we detect the touch is "inside" one of the movable actors, return that actor.
			// Note that we iterate backwards - this is because the QuickSort() above sorts
			// by render order, farthest to nearest, but we want the nearest actor returned first.
			SoActorList::Iterator SortedIter( &MovableActorsSorted );

			while ( SortedIter.Prev() )
			{
				SoActor* pActor = SortedIter.GetNodeData();
				SoActorVisible* pActorVisible = pActor->GetAsVisible();

				if ( pActorVisible )
				{
					if ( pActorVisible->IsPositionInside( vWorldPos ) )
					{
						return pActorVisible;
						break;
					}
				}
			}
		}
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  OnUiButtonDown
******************************************************************************/
void SoInputDispatcher::OnUiButtonDown( SoUiButton* pButton )
{
	InputPacket Input;
	Input._pButtonDown = pButton;
	MainDisptach( &Input );
}



/******************************************************************************
 FUNCTION:  OnUiButtonUp
******************************************************************************/
void SoInputDispatcher::OnUiButtonUp( SoUiButton* pButton )
{
	InputPacket Input;
	Input._pButtonUp = pButton;
	MainDisptach( &Input );
}



/******************************************************************************
 FUNCTION:  OnUiButtonUpTouchOutside
******************************************************************************/
void SoInputDispatcher::OnUiButtonUpTouchOutside( SoUiButton* pButton )
{
	InputPacket Input;
	Input._pButtonUpTouchOutside = pButton;
	MainDisptach( &Input );
}



/******************************************************************************
 FUNCTION:  OnTouchesDown
******************************************************************************/
bool SoInputDispatcher::OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) 
{
	InputPacket Input;
	Input._pSoCcTouchesDownList = pSoCcTouchList;
	MainDisptach( &Input );

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesUp
******************************************************************************/
bool SoInputDispatcher::OnTouchesUp( const SoCcTouchList* pSoCcTouchList )
{
	InputPacket Input;
	Input._pSoCcTouchesUpList = pSoCcTouchList;
	MainDisptach( &Input );

	return false;
}



/******************************************************************************
 FUNCTION:  OnTouchesMove
******************************************************************************/
bool SoInputDispatcher::OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) 
{
	InputPacket Input;
	Input._pSoCcTouchesMoveList = pSoCcTouchList;
	MainDisptach( &Input );

	return false;
}



/******************************************************************************
 FUNCTION:  OnGestureDown
 ******************************************************************************/
bool SoInputDispatcher::OnGestureDown( SoGesture* pGesture )
{
	InputPacket Input;
	Input._pGestureDown = pGesture;
	MainDisptach( &Input );

	return false;
}



/******************************************************************************
 FUNCTION:  OnGestureUp
 ******************************************************************************/
bool SoInputDispatcher::OnGestureUp( SoGesture* pGesture )
{
	InputPacket Input;
	Input._pGestureUp = pGesture;
	MainDisptach( &Input );

	return false;
}



/******************************************************************************
 FUNCTION:  OnGestureMove
 ******************************************************************************/
bool SoInputDispatcher::OnGestureMove( SoGesture* pGesture )
{
	InputPacket Input;
	Input._pGestureMove = pGesture;
	MainDisptach( &Input );

	return false;
}



/******************************************************************************
 FUNCTION:  OnKeypadBackClicked
******************************************************************************/
bool SoInputDispatcher::OnKeypadBackClicked()
{
	InputPacket Input;
	Input._bKeypadBackButtonPressed = true;
	MainDisptach( &Input );

	return false;
}



/******************************************************************************
 FUNCTION:  OnKeypadMenuClicked
******************************************************************************/
bool SoInputDispatcher::OnKeypadMenuClicked()
{
	InputPacket Input;
	Input._bKeypadMenuButtonPressed = true;
	MainDisptach( &Input );

	return false;
}



/******************************************************************************
 FUNCTION:  OnKeyPressed
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void SoInputDispatcher::OnKeyPressed( OIS::KeyCode Key )
	{
		InputPacket Input;
		Input._eKeyPressed = Key;
		MainDisptach( &Input );
	}
#endif



/******************************************************************************
 FUNCTION:  OnKeyReleased
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	void SoInputDispatcher::OnKeyReleased( OIS::KeyCode Key )
	{
		InputPacket Input;
		Input._eKeyReleased = Key;
		MainDisptach( &Input );
	}
#endif



/******************************************************************************
 FUNCTION:  OnKeyPressed
******************************************************************************/
#if DCG_IS_TARGET_WIN32
	bool SoInputDispatcher::OnMouseMoved( OIS::MouseEvent arg )
	{
		InputPacket Input;
		Input._iMouseMovedX = arg.state.X.rel;
		Input._iMouseMovedY = arg.state.Y.rel;
		Input._iMouseMovedZ = arg.state.Z.rel;
		Input._pMouseEvent = &arg;
		MainDisptach( &Input );

		return false;
	}
#endif


		
/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoInputDispatcher::CreateInstance()
{
	ASSERTF( _pInstance == NULL, ( "\nCan't create more than one instance.  Not allowed." ) );

	if ( !_pInstance )
		_pInstance = new SoInputDispatcher;
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoInputDispatcher::DeleteInstance()
{
	if ( _pInstance )
	{
		_pInstance->Deinit();
		delete _pInstance;
		_pInstance = NULL;
	}
}









/******************************************************************************
 FUNCTION:  InputPacket
******************************************************************************/
SoInputDispatcher::InputPacket::InputPacket() :
#if DCG_IS_TARGET_WIN32
	_pMouseEvent( NULL ),
	_eKeyPressed( OIS::KC_UNASSIGNED ), 
	_eKeyReleased( OIS::KC_UNASSIGNED ),
#endif
_pSoCcTouchesDownList( NULL ),
_pSoCcTouchesUpList( NULL ),
_pSoCcTouchesMoveList( NULL ),
_pGestureDown( NULL ),
_pGestureUp( NULL ),
_pGestureMove( NULL ),
_pButtonDown( NULL ),
_pButtonUp( NULL ),
_pButtonUpTouchOutside( NULL ),
_pButtonRub( NULL ),
_iMouseMovedX( 0 ),
_iMouseMovedY( 0 ),
_iMouseMovedZ( 0 ),
_bKeypadBackButtonPressed( false ),
_bKeypadMenuButtonPressed( false )
{
}


