/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "JpHashTable.h"
#include "JpList.h"

#include "SoInputGesture.h"
#include "SoInputKeyboard.h"
#include "SoInputKeypad.h"
#include "SoInputMouse.h"
#include "SoInputTouch.h"
#include "SoUiButton.h"


USING_NS_CC;

class SoActorVisible;


class SoInputDispatcher
{
	public:

		//-----------------------
		// public enums and classes
		enum STATE
		{
			STATE_CAMERA_PANNING,
			STATE_MOVING_ACTOR,
			STATE_LAST
		};

		class InputPacket
		{
			public:
				InputPacket();
				virtual ~InputPacket() {}

				#if DCG_IS_TARGET_WIN32
					OIS::KeyCode _eKeyPressed; 
					OIS::KeyCode _eKeyReleased;
				#endif

				const SoCcTouchList* _pSoCcTouchesDownList;
				const SoCcTouchList* _pSoCcTouchesUpList;
				const SoCcTouchList* _pSoCcTouchesMoveList;
				SoGesture* _pGestureDown;
				SoGesture* _pGestureUp;
				SoGesture* _pGestureMove;
				SoUiButton* _pButtonDown;
				SoUiButton* _pButtonUp;
				SoUiButton* _pButtonUpTouchOutside;
				SoUiButton* _pButtonRub;
				int _iMouseMovedX;
				int _iMouseMovedY;
				int _iMouseMovedZ;
				bool _bKeypadBackButtonPressed;
				bool _bKeypadMenuButtonPressed;

				#if DCG_IS_TARGET_WIN32
					OIS::MouseEvent* _pMouseEvent;
				#endif				
		};



		//-----------------------
		// public functions

		SoInputDispatcher();

		virtual ~SoInputDispatcher();

		void Init( CCNode* pParentNode );

		void Deinit();

		void Update( float dt );

		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }

		// button listener callbacks
		virtual void OnUiButtonDown( SoUiButton* pButton );
		virtual void OnUiButtonUp( SoUiButton* pButton );
		virtual void OnUiButtonUpTouchOutside( SoUiButton* pButton );
		virtual void OnUiButtonRub( SoUiButton* pButton ) {};


		// SoInputTouch callbacks
		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList );
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList );

		// SoInputGesture callbacks
		virtual bool OnGestureDown( SoGesture* pGesture );
		virtual bool OnGestureUp( SoGesture* pGesture );
		virtual bool OnGestureMove( SoGesture* pGesture );
	
		// SoInputKeypad callbacks
		virtual bool OnKeypadBackClicked();
		virtual bool OnKeypadMenuClicked();

		#if DCG_IS_TARGET_WIN32
			// SoInputKeyboardListener callbacks
			virtual void OnKeyPressed( OIS::KeyCode Key );
			virtual void OnKeyReleased( OIS::KeyCode Key );

			virtual bool OnMousePressed( OIS::MouseEvent arg, OIS::MouseButtonID id ) { return false; }
			virtual bool OnMouseReleased( OIS::MouseEvent arg, OIS::MouseButtonID id ) { return false; }
			virtual bool OnMouseMoved( OIS::MouseEvent arg );
		#endif


		static SoInputDispatcher* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();

	private:

		//-----------------------
		// private variables
		static SoInputDispatcher* _pInstance;

		STATE _eState;

		CCNode* _pParentNode;

		int _iNumHudButtonsDown;

		#if DCG_IS_TARGET_WIN32
			SoInputMouseListener< SoInputDispatcher > _InputMouseListener;
			SoInputKeyboardListener< SoInputDispatcher > _InputKeyboardListener;
			OIS::KeyCode _eLastInputPressed;
		#endif

		SoInputTouchListener< SoInputDispatcher > _InputTouchListener;
		SoInputGestureListener< SoInputDispatcher > _InputGestureListener;
		SoUiButtonListener< SoInputDispatcher > _UiButtonListener;
		SoInputKeypadListener< SoInputDispatcher > _InputKeypadListener;


		SoActorVisible* _pMovingActor;


		//-----------------------
		// private functions
		void MainDisptach( InputPacket* pInputPacket );

		bool SendInputToParkingSpots( InputPacket* pInputPacket );
		SoActorVisible* DetectTouchedDownOnActor( InputPacket* pInputPacket );
};




