/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#if DCG_IS_TARGET_WIN32

#include "cocos2d.h"
#include "OIS.h"

#include "JpList.h"

USING_NS_CC;



class SoInputOisManager;
#define SoKeyRepeatList              JpList< SoKeyRepeat >
#define SoKeysToRepeatList           JpList< int >
#define SoKeyRepeatCooldownDefault   0.25f



class SoKeyRepeat
{
	public:
		SoKeyRepeat() 
		{
			_KeyCode = OIS::KC_MEDIASELECT;
			_bIsDown = false;
		};
		virtual ~SoKeyRepeat() {}

		OIS::KeyCode GetKeyCode() { return _KeyCode; }
		void         SetKeyCode( OIS::KeyCode e ) { _KeyCode = e; }

		bool  GetIsDown() { return _bIsDown; }
		void  SetIsDown( bool b ) { _bIsDown = b; }

		float GetRepeatTimer() { return _fRepeatTimer; }
		void SetRepeatTimer( float f ) { _fRepeatTimer = f; }

	private:

		OIS::KeyCode _KeyCode;

		float _fRepeatTimer;
		bool  _bIsDown;
};



class SoInputKeyboard : public OIS::KeyListener
{
	public:

		//-----------------------
		// public enum and classes
		class Listener
		{
			public:
				Listener() { _fKeyRepeatCooldown = SoKeyRepeatCooldownDefault; }
				virtual ~Listener();
				virtual void OnKeyPressed( OIS::KeyCode Key ) {}
				virtual void OnKeyReleased( OIS::KeyCode Key ) {}
				
				virtual void SetLastInputPressed( OIS::KeyCode Key ) {}
				virtual void SetLastInputReleased( OIS::KeyCode Key ) {}

				void Update( float dt );

				void EnableRepeatForKey( OIS::KeyCode Key ) { _KeysToRepeatList.AddLast( (int*)Key ); }

				bool UseRepeat( OIS::KeyCode Key );
				void UpdateRepeats( float dt );
				void BroadcastRepeats();
				void DestroyKeyRepeastList();

				float GetKeyRepeatCooldown() { return _fKeyRepeatCooldown; }
				void SetKeyRepeatCooldown( float f ) { _fKeyRepeatCooldown = f; }
				

				SoKeysToRepeatList* GetKeysToRepeatList() { return &_KeysToRepeatList; }
				SoKeyRepeatList*    GetRepeatList()       { return &_RepeatList; }


				SoKeysToRepeatList _KeysToRepeatList;
				SoKeyRepeatList    _RepeatList;


			private:

				float _fKeyRepeatCooldown;
		};


		//-----------------------
		// public functions

		SoInputKeyboard();

		virtual ~SoInputKeyboard();

		void Init();

		void Deinit();

		void Update( float dt );

		void AddListener( Listener* pListener );
		void RemoveListener( Listener* pListener );
		void RemoveAllListeners();

		// overrides from OIS::KeyListener
		virtual bool keyPressed( const OIS::KeyEvent &arg );
		virtual bool keyReleased( const OIS::KeyEvent &arg );


		static SoInputKeyboard* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();


	private:

		//-----------------------
		// private variables

		static SoInputKeyboard* _pInstance;

		JpList< Listener > _ListenerList;
		bool _bListenerListLocked;
};




template< class TYPE >
class SoInputKeyboardListener : public SoInputKeyboard::Listener
{
	public:
		//-----------------------
		// public functions

		SoInputKeyboardListener() : _pOwner( NULL ) {}
		~SoInputKeyboardListener() {}
		void SetOwner( TYPE* p ) { _pOwner = p; }

		virtual void OnKeyPressed( OIS::KeyCode Key )
		{ 
			_eLastInputPressed = Key;
			_pOwner->OnKeyPressed( Key ); 
		}
		virtual void OnKeyReleased( OIS::KeyCode Key )
		{ 
			_eLastInputReleased = Key;
			_pOwner->OnKeyReleased( Key ); 
		}


		OIS::KeyCode GetLastInputPressed() { return _eLastInputPressed; }
		OIS::KeyCode GetLastInputReleased() { return _eLastInputReleased; }

		void SetLastInputPressed( OIS::KeyCode Key ) { _eLastInputPressed = Key; }
		void SetLastInputReleased( OIS::KeyCode Key ) { _eLastInputReleased = Key; }


	private:

		//-----------------------
		// private variables.
		TYPE* _pOwner;
		OIS::KeyCode _eLastInputPressed;
		OIS::KeyCode _eLastInputReleased;


		//-----------------------
		// private functions.
		SoInputKeyboardListener( SoInputKeyboardListener& Copy )
		{
			ASSERTF( 0, ( "\nCannot copy instances of this class" ) );
		}
};



#endif // DCG_IS_TARGET_WIN32



