/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "NeCsvIn.h"

#include "SoDrawDebug.h"
#include "SoInputKeypad.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"



SoInputKeypad* SoInputKeypad::_pInstance = NULL;



/******************************************************************************
 FUNCTION:  SoInputKeypad
******************************************************************************/
SoInputKeypad::SoInputKeypad() :
_bListenerListLocked( false ),
_pParentNode( NULL )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoInputKeypad::Init( CCNode* pParentNode )
{
	_pParentNode = pParentNode;

	if ( pParentNode )
		pParentNode->addChild( this );

	setKeypadEnabled( true );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoInputKeypad::Deinit()
{
	if ( _pParentNode )
	{
		_pParentNode->removeChild( this, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pParentNode = NULL;
	}

	RemoveAllListeners();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoInputKeypad::Update( float dt )
{
	if ( !_pParentNode )
	{
		_pParentNode = CCDirector::sharedDirector()->getRunningScene();
		_pParentNode->addChild( this );
	}
}




/******************************************************************************
 FUNCTION:  BroadcastKeypadBackClicked
******************************************************************************/
void SoInputKeypad::BroadcastKeypadBackClicked()
{
	//-----------------------
	// The event handler we are calling can do anything it likes...even attempt to
	// add or remove to the listener list.  That would be bad since we are in 
	// the middle of walking through the list.  So we lock the list to prevent tampering.

	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();
		pListener->OnKeypadBackClicked();
	}

	_bListenerListLocked = false;
}



/******************************************************************************
 FUNCTION:  BroadcastKeypadMenuClicked
******************************************************************************/
void SoInputKeypad::BroadcastKeypadMenuClicked()
{
	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();
		pListener->OnKeypadMenuClicked();
	}

	_bListenerListLocked = false;
}



/******************************************************************************
 FUNCTION:  keyBackClicked
******************************************************************************/
void SoInputKeypad::keyBackClicked()
{
	BroadcastKeypadBackClicked();
}



/******************************************************************************
 FUNCTION:  keyMenuClicked
******************************************************************************/
void SoInputKeypad::keyMenuClicked()
{
	BroadcastKeypadMenuClicked();
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoInputKeypad::CreateInstance()
{
	ASSERTF( _pInstance == NULL, ( "\nCan't create more than one instance.  Not allowed." ) );

	if ( !_pInstance )
		_pInstance = new SoInputKeypad;
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoInputKeypad::DeleteInstance()
{
	if ( _pInstance )
	{
		_pInstance->Deinit();
		delete _pInstance;
		_pInstance = NULL;
	}
}






