/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#include <string>

#include "cocos2d.h"

#include "JpList.h"



USING_NS_CC;


/*
	This class is for managing input from the keypad.
	The "Keypad" mainly refers to Android devices and their Menu and Back buttons.
	Although I think this term is confusing, it is the terminology used by
	the cocos2dx libraries, and so I think it's best to stick to it.
	mdomowicz 2014_07_21
*/



// Note, we derive from CCLayer so we can receive Keypad input callbacks
class SoInputKeypad : public CCLayer  
{
	public:

		//-----------------------
		// public enum and classes
		class Listener
		{
			public:
				Listener() {}
				virtual ~Listener() {}
				virtual bool OnKeypadBackClicked() { return false; }
				virtual bool OnKeypadMenuClicked() { return false; }
		};



		//-----------------------
		// public functions

		SoInputKeypad();
		virtual ~SoInputKeypad() {};

		void Init( CCNode* pParentNode );
		void Deinit();
		void Update( float dt );
	
		void AddListener( Listener* pListener )
		{
			ASSERTF( _bListenerListLocked == false, ("Can't add while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.AddUniquely( pListener, 0 );
		}

		void RemoveListener( Listener* pListener )
		{
			ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.Remove( pListener );
		}
		
		void RemoveAllListeners()
		{
			ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.RemoveAll();
		}


		virtual void keyBackClicked();
		virtual void keyMenuClicked();


		static SoInputKeypad* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();

	private:


		//-----------------------
		// private variables
		static SoInputKeypad* _pInstance;


		JpList< Listener > _ListenerList;
		bool _bListenerListLocked;

		CCNode* _pParentNode;



		//-----------------------
		// private functions
		void BroadcastKeypadBackClicked();
		void BroadcastKeypadMenuClicked();
};





template< class TYPE >
class SoInputKeypadListener : public SoInputKeypad::Listener
{
	public:
		//-----------------------
		// public functions

		SoInputKeypadListener() : _pOwner( NULL ) {}
		~SoInputKeypadListener() {}
		void SetOwner( TYPE* p ) { _pOwner = p; }

		virtual bool OnKeypadBackClicked() { return _pOwner->OnKeypadBackClicked(); }
		virtual bool OnKeypadMenuClicked() { return _pOwner->OnKeypadMenuClicked(); }

	private:

		//-----------------------
		// private variables.
		TYPE* _pOwner;


		//-----------------------
		// private functions.
		SoInputKeypadListener( SoInputKeypadListener& Copy )
		{
			ASSERTF( 0, ( "\nCannot copy instances of this class" ) );
		}
};
