/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#if DCG_IS_TARGET_WIN32

#include "JpDebug.h"
#include "SoInputMouse.h"
#include "SoInputOisManager.h"

#include "mmgrChooser.h"


USING_NS_CC;


SoInputMouse* SoInputMouse::_pInstance = NULL;



/******************************************************************************
 FUNCTION:  SoInputMouse
******************************************************************************/
SoInputMouse::SoInputMouse() :
_bListenerListLocked( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoInputMouse
******************************************************************************/
SoInputMouse::~SoInputMouse()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoInputMouse::Init()
{
	SoInputOisManager* pOisInputManager = SoInputOisManager::GetInstance();

	ASSERTF( pOisInputManager, ( "\nNo OIS Input manager exists!  Create it before initializing this class." ) );

	if ( pOisInputManager )
		pOisInputManager->SetMouseListener( this );

	ShowCursor( true );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoInputMouse::Deinit()
{
	SoInputOisManager* pOisInputManager = SoInputOisManager::GetInstance();

	ASSERTF( pOisInputManager, ( "\nNo OIS Input manager exists!" ) );

	if ( pOisInputManager )
		pOisInputManager->SetMouseListener( NULL );
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoInputMouse::CreateInstance()
{
	ASSERTF( _pInstance == NULL, ( "\nCan't create more than one instance.  Not allowed." ) );

	if ( !_pInstance )
		_pInstance = new SoInputMouse;
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoInputMouse::DeleteInstance()
{
	if ( _pInstance )
	{
		delete _pInstance;
		_pInstance = NULL;
	}
}



/******************************************************************************
 FUNCTION:  AddListener
******************************************************************************/
void SoInputMouse::AddListener( Listener* pListener )
{
	ASSERTF( _bListenerListLocked == false, ("Can't add while list is locked.") )
	if ( _bListenerListLocked == false )
		_ListenerList.AddUniquely( pListener, 0 );
}



/******************************************************************************
 FUNCTION:  RemoveListener
******************************************************************************/
void SoInputMouse::RemoveListener( Listener* pListener )
{
	ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
	if ( _bListenerListLocked == false )
	{
		_ListenerList.Remove( pListener );
	}
}



/******************************************************************************
 FUNCTION:  RemoveAllListeners
******************************************************************************/
void SoInputMouse::RemoveAllListeners()
{
	ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
	if ( _bListenerListLocked == false )
	{
		_ListenerList.RemoveAll();
	}
}



/******************************************************************************
 FUNCTION:  mouseMoved
******************************************************************************/
bool SoInputMouse::mouseMoved( const OIS::MouseEvent &arg ) 
{
	//-----------------------
	// The event handler we are calling can do anything it likes...even attempt to
	// add or remove to the listener list.  That would be bad since we are in 
	// the middle of walking through the list.  So we lock the list to prevent tampering.
	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();
		pListener->OnMouseMoved( arg );
	}

	_bListenerListLocked = false;

	return true;
}



/******************************************************************************
 FUNCTION:  mousePressed
******************************************************************************/
bool SoInputMouse::mousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) 
{
	return true;
}



/******************************************************************************
 FUNCTION:  mouseReleased
******************************************************************************/
bool SoInputMouse::mouseReleased( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) 
{
	return true;
}



#endif // DCG_IS_TARGET_WIN32



























