/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#if DCG_IS_TARGET_WIN32

#include "cocos2d.h"
#include "OIS.h"

#include "JpList.h"

USING_NS_CC;



class SoInputOisManager;



class SoInputMouse : public OIS::MouseListener
{
	public:

		//-----------------------
		// public enum and classes
		class Listener
		{
			public:
				Listener() {}
				virtual ~Listener() {}
				virtual bool OnMousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) { return false; }
				virtual bool OnMouseReleased( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) { return false; }
				virtual bool OnMouseMoved( const OIS::MouseEvent &arg ) { return false; }
		};


		//-----------------------
		// public functions

		SoInputMouse();

		virtual ~SoInputMouse();

		void Init();
		void Deinit();

		void AddListener( Listener* pListener );
		void RemoveListener( Listener* pListener );
		void RemoveAllListeners();

		// overrides from OIS::MouseListener
		virtual bool mouseMoved( const OIS::MouseEvent &arg );
		virtual bool mousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id );
		virtual bool mouseReleased( const OIS::MouseEvent &arg, OIS::MouseButtonID id );

		static SoInputMouse* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();


	private:

		//-----------------------
		// private variables

		static SoInputMouse* _pInstance;

		JpList< Listener > _ListenerList;
		bool _bListenerListLocked;
};




template< class TYPE >
class SoInputMouseListener : public SoInputMouse::Listener
{
	public:
		//-----------------------
		// public functions

		SoInputMouseListener(): _pOwner(NULL) {}
		~SoInputMouseListener() {}
		void SetOwner( TYPE* p ) { _pOwner = p; }

		virtual bool OnMousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) { return _pOwner->OnMousePressed( arg, id ); }
		virtual bool OnMouseReleased( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) { return _pOwner->OnMouseReleased( arg, id ); }
		virtual bool OnMouseMoved( const OIS::MouseEvent &arg ) { return _pOwner->OnMouseMoved( arg ); }


	private:
		TYPE* _pOwner;

		//-----------------------
		// private functions.
		SoInputMouseListener( SoInputMouseListener& Copy )
		{
			ASSERTF( 0, ( "\nCannot copy instances of this class" ) );
		}
};



#endif // DCG_IS_TARGET_WIN32



