/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#if DCG_IS_TARGET_WIN32

#include "OIS.h"

#include "windows.h"


#include "JpDebug.h"
#include "JpUtils.h"
#include "SoInputOisManager.h"

#include "mmgrChooser.h"


USING_NS_CC;

SoInputOisManager* SoInputOisManager::_pInstance = NULL;


// Taken from the OIS example code, and adjusted.
#if DCG_IS_TARGET_LINUX
  Display *xDisp = 0;
  Window xWin = 0;
#elif DCG_IS_TARGET_OSX
  WindowRef mWin = 0;
#endif



char* SoInputOisManager::_pDeviceType[ 6 ] = 
{
	"OISUnknown", 
	"OISKeyboard", 
	"OISMouse", 
	"OISJoyStick",
	"OISTablet", 
	"OISOther"
};





/******************************************************************************
 CLASS:  EventHandler
 NOTES:  This is an example OIS input event handler taken from the OIS sample
         code.  I leave this here as reference.
         mdomowicz 2013_01_18
******************************************************************************/
class EventHandler : public OIS::KeyListener, public OIS::MouseListener, public OIS::JoyStickListener
{
public:
	EventHandler() {}
	~EventHandler() {}
	bool keyPressed( const OIS::KeyEvent &arg ) {
		std::cout << " KeyPressed {" << arg.key
			<< ", " << ((OIS::Keyboard*)(arg.device))->getAsString(arg.key)
			<< "} || Character (" << (char)arg.text << ")" << std::endl;
		return true;
	}
	bool keyReleased( const OIS::KeyEvent &arg ) {
		//if( arg.key == KC_ESCAPE || arg.key == KC_Q )
		//	appRunning = false;
		std::cout << "KeyReleased {" << ((OIS::Keyboard*)(arg.device))->getAsString(arg.key) << "}\n";
		return true;
	}
	bool mouseMoved( const OIS::MouseEvent &arg ) {
		const OIS::MouseState& s = arg.state;
		std::cout << "\nMouseMoved: Abs("
				  << s.X.abs << ", " << s.Y.abs << ", " << s.Z.abs << ") Rel("
				  << s.X.rel << ", " << s.Y.rel << ", " << s.Z.rel << ")";
		return true;
	}
	bool mousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) {
		const OIS::MouseState& s = arg.state;
		std::cout << "\nMouse button #" << id << " pressed. Abs("
				  << s.X.abs << ", " << s.Y.abs << ", " << s.Z.abs << ") Rel("
				  << s.X.rel << ", " << s.Y.rel << ", " << s.Z.rel << ")";
		return true;
	}
	bool mouseReleased( const OIS::MouseEvent &arg, OIS::MouseButtonID id ) {
		const OIS::MouseState& s = arg.state;
		std::cout << "\nMouse button #" << id << " released. Abs("
				  << s.X.abs << ", " << s.Y.abs << ", " << s.Z.abs << ") Rel("
				  << s.X.rel << ", " << s.Y.rel << ", " << s.Z.rel << ")";
		return true;
	}
	bool buttonPressed( const OIS::JoyStickEvent &arg, int button ) {
		std::cout << std::endl << arg.device->vendor() << ". Button Pressed # " << button;
		return true;
	}
	bool buttonReleased( const OIS::JoyStickEvent &arg, int button ) {
		std::cout << std::endl << arg.device->vendor() << ". Button Released # " << button;
		return true;
	}
	bool axisMoved( const OIS::JoyStickEvent &arg, int axis )
	{
		//Provide a little dead zone
		if( arg.state.mAxes[axis].abs > 2500 || arg.state.mAxes[axis].abs < -2500 )
			std::cout << std::endl << arg.device->vendor() << ". Axis # " << axis << " Value: " << arg.state.mAxes[axis].abs;
		return true;
	}
	bool povMoved( const OIS::JoyStickEvent &arg, int pov )
	{
		std::cout << std::endl << arg.device->vendor() << ". POV" << pov << " ";

		if( arg.state.mPOV[pov].direction & OIS::Pov::North ) //Going up
			std::cout << "North";
		else if( arg.state.mPOV[pov].direction & OIS::Pov::South ) //Going down
			std::cout << "South";

		if( arg.state.mPOV[pov].direction & OIS::Pov::East ) //Going right
			std::cout << "East";
		else if( arg.state.mPOV[pov].direction & OIS::Pov::West ) //Going left
			std::cout << "West";

		if( arg.state.mPOV[pov].direction == OIS::Pov::Centered ) //stopped/centered out
			std::cout << "Centered";
		return true;
	}

	bool vector3Moved( const OIS::JoyStickEvent &arg, int index)
	{
		std::cout.precision(2);
		std::cout.flags(std::ios::fixed | std::ios::right);
		std::cout << std::endl << arg.device->vendor() << ". Orientation # " << index 
			<< " X Value: " << arg.state.mVectors[index].x
			<< " Y Value: " << arg.state.mVectors[index].y
			<< " Z Value: " << arg.state.mVectors[index].z;
		std::cout.precision();
		std::cout.flags();
		return true;
	}
};

//Create a global instance
EventHandler handler;




/******************************************************************************
 FUNCTION:  SoInputOisManager
******************************************************************************/
SoInputOisManager::SoInputOisManager() :
_pOisInputManager( NULL ),
_pOisKeyboard( NULL ),
_pOisMouse( NULL )
{
	_pOisJoysticks[ 0 ] = 0;
	_pOisJoysticks[ 1 ] = 0;
	_pOisJoysticks[ 2 ] = 0;
	_pOisJoysticks[ 3 ] = 0;
}



/******************************************************************************
 FUNCTION:  ~SoInputOisManager
******************************************************************************/
SoInputOisManager::~SoInputOisManager()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
 NOTES:     The majority of this code is taken from the OIS sample code.
******************************************************************************/
void SoInputOisManager::Init()
{
	OIS::ParamList pl;

	#if defined OIS_WIN32_PLATFORM
		CCEGLView& mainWnd = CCEGLView::sharedOpenGLView();
		HWND hWnd = mainWnd.getHWnd();
		ShowWindow(mainWnd.getHWnd(), SW_SHOW);

		std::ostringstream wnd;
		wnd << (size_t)hWnd;

		pl.insert(std::make_pair( std::string("WINDOW"), wnd.str() ));

		//Default mode is foreground exclusive..but, we want to show mouse - so nonexclusive
		pl.insert(std::make_pair(std::string("w32_mouse"), std::string("DISCL_FOREGROUND" )));
		pl.insert(std::make_pair(std::string("w32_mouse"), std::string("DISCL_NONEXCLUSIVE")));
	#elif defined OIS_LINUX_PLATFORM
		//Connects to default X window
		if( !(xDisp = XOpenDisplay(0)) )
			OIS_EXCEPT(E_General, "Error opening X!");
		//Create a window
		xWin = XCreateSimpleWindow(xDisp,DefaultRootWindow(xDisp), 0,0, 100,100, 0, 0, 0);
		//bind our connection to that window
		XMapWindow(xDisp, xWin);
		//Select what events we want to listen to locally
		XSelectInput(xDisp, xWin, StructureNotifyMask);
		XEvent evtent;
		do
		{
			XNextEvent(xDisp, &evtent);
		} while(evtent.type != MapNotify);

		std::ostringstream wnd;
		wnd << xWin;

		pl.insert(std::make_pair(std::string("WINDOW"), wnd.str()));

		//For this demo, show mouse and do not grab (confine to window)
		//pl.insert(std::make_pair(std::string("x11_mouse_grab"), std::string("false")));
		//pl.insert(std::make_pair(std::string("x11_mouse_hide"), std::string("false")));
	#elif defined OIS_APPLE_PLATFORM
		// create the window rect in global coords
		::Rect windowRect;
		windowRect.left = 0;
		windowRect.top = 0;
		windowRect.right = 300;
		windowRect.bottom = 300;
	    
		// set the default attributes for the window
		WindowAttributes windowAttrs = kWindowStandardDocumentAttributes
			| kWindowStandardHandlerAttribute 
			| kWindowInWindowMenuAttribute
			| kWindowHideOnFullScreenAttribute;
	    
		// Create the window
		CreateNewWindow(kDocumentWindowClass, windowAttrs, &windowRect, &mWin);
	    
		// Color the window background black
		SetThemeWindowBackground (mWin, kThemeBrushBlack, true);
	    
		// Set the title of our window
		CFStringRef titleRef = CFStringCreateWithCString( kCFAllocatorDefault, "OIS Input", kCFStringEncodingASCII );
		SetWindowTitleWithCFString( mWin, titleRef );
	    
		// Center our window on the screen
		RepositionWindow( mWin, NULL, kWindowCenterOnMainScreen );
	    
		// Install the event handler for the window
		InstallStandardEventHandler(GetWindowEventTarget(mWin));
	    
		// This will give our window focus, and not lock it to the terminal
		ProcessSerialNumber psn = { 0, kCurrentProcess };
		TransformProcessType( &psn, kProcessTransformToForegroundApplication );
		SetFrontProcess(&psn);
	    
		// Display and select our window
		ShowWindow(mWin);
		SelectWindow(mWin);

		std::ostringstream wnd;
		wnd << (unsigned int)mWin; //cast to int so it gets encoded correctly (else it gets stored as a hex string)
		std::cout << "WindowRef: " << mWin << " WindowRef as int: " << wnd.str() << "\n";
		pl.insert(std::make_pair(std::string("WINDOW"), wnd.str()));
	#endif

	//This never returns null.. it will raise an exception on errors
	_pOisInputManager = OIS::InputManager::createInputSystem(pl);

	//Lets enable all addons that were compiled in:
	_pOisInputManager->enableAddOnFactory(OIS::InputManager::AddOn_All);

	//Print debugging information
	unsigned int v = _pOisInputManager->getVersionNumber();
	std::cout << "OIS Version: " << (v>>16 ) << "." << ((v>>8) & 0x000000FF) << "." << (v & 0x000000FF)
		<< "\nRelease Name: " << _pOisInputManager->getVersionName()
		<< "\nManager: " << _pOisInputManager->inputSystemName()
		<< "\nTotal Keyboards: " << _pOisInputManager->getNumberOfDevices(OIS::OISKeyboard)
		<< "\nTotal Mice: " << _pOisInputManager->getNumberOfDevices(OIS::OISMouse)
		<< "\nTotal JoySticks: " << _pOisInputManager->getNumberOfDevices(OIS::OISJoyStick);

	//List all devices
	OIS::DeviceList list = _pOisInputManager->listFreeDevices();
	for( OIS::DeviceList::iterator i = list.begin(); i != list.end(); ++i )
		std::cout << "\n\tDevice: " << _pDeviceType[i->first] << " Vendor: " << i->second;


	_pOisKeyboard = (OIS::Keyboard*)_pOisInputManager->createInputObject( OIS::OISKeyboard, true );
	// The example used the event handler above, but we replaced it with our own (SoInputKeyboard)
	// mdomowicz 2013_01_18
	//_pOisKeyboard->setEventCallback( &handler );


	// This code enables the OIS Mouse system.  But when I use it, it makes the mouse cursor disappear
	// on windows.  Since Cocos2d already does mouse handling, for now I'm just disabling the mouse.
	// mdomowicz 2013_01_17
	
	_pOisMouse = (OIS::Mouse*)_pOisInputManager->createInputObject( OIS::OISMouse, true );
	/*
	g_m->setEventCallback( &handler );
	const OIS::MouseState &ms = g_m->getMouseState();
	ms.width = 100;
	ms.height = 100;
	*/

	try
	{
		int numSticks = Min(_pOisInputManager->getNumberOfDevices(OIS::OISJoyStick), DCG_MAX_OIS_JOYSTICKS);
		for( int i = 0; i < numSticks; ++i )
		{
			_pOisJoysticks[i] = (OIS::JoyStick*)_pOisInputManager->createInputObject( OIS::OISJoyStick, true );
			_pOisJoysticks[i]->setEventCallback( &handler );
			std::cout << "\n\nCreating Joystick " << (i + 1)
				<< "\n\tAxes: " << _pOisJoysticks[i]->getNumberOfComponents(OIS::OIS_Axis)
				<< "\n\tSliders: " << _pOisJoysticks[i]->getNumberOfComponents(OIS::OIS_Slider)
				<< "\n\tPOV/HATs: " << _pOisJoysticks[i]->getNumberOfComponents(OIS::OIS_POV)
				<< "\n\tButtons: " << _pOisJoysticks[i]->getNumberOfComponents(OIS::OIS_Button)
				<< "\n\tVector3: " << _pOisJoysticks[i]->getNumberOfComponents(OIS::OIS_Vector3);
		}
	}
	catch(OIS::Exception &ex)
	{
		std::cout << "\nException raised on joystick creation: " << ex.eText << std::endl;
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoInputOisManager::Deinit()
{
	if ( _pOisKeyboard )
	{
		_pOisInputManager->destroyInputObject( _pOisKeyboard );
		_pOisKeyboard = NULL;
	}

	if ( _pOisMouse )
	{
		_pOisInputManager->destroyInputObject( _pOisMouse );
		_pOisMouse = NULL;
	}

	for ( int i = 0; i < DCG_MAX_OIS_JOYSTICKS; i ++ )
	{
		if ( _pOisJoysticks[ i ] )
		{
			_pOisInputManager->destroyInputObject( _pOisJoysticks[ i ] );
			_pOisJoysticks[ i ] = NULL;
		}
	}

	OIS::InputManager::destroyInputSystem( _pOisInputManager );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoInputOisManager::Update( float dt )
{
	if( _pOisKeyboard )
		_pOisKeyboard->capture();

	if( _pOisMouse )
		_pOisMouse->capture();

	for( int i = 0; i < DCG_MAX_OIS_JOYSTICKS ; ++i )
	{
		if( _pOisJoysticks[i] )
		{
			_pOisJoysticks[i]->capture();
		}
	}
}



/******************************************************************************
 FUNCTION:  SetKeyboardListener
******************************************************************************/
void SoInputOisManager::SetKeyboardListener( OIS::KeyListener* pKeyboardListener )
{
	ASSERTF( _pOisKeyboard, ( "\nOIS Keyboard no created!" ) );

	if ( _pOisKeyboard )
	{
		if ( pKeyboardListener )
		{
			ASSERTF( _pOisKeyboard->getEventCallback() == 0, ( "\nOIS callback already set.  Only one allowed according to the OIS header comments." ) );
			if ( _pOisKeyboard->getEventCallback() == 0 )
				_pOisKeyboard->setEventCallback( pKeyboardListener );
		}
		else
		{
			_pOisKeyboard->setEventCallback( NULL );
		}
	}
}



/******************************************************************************
 FUNCTION:  SetMouseListener
******************************************************************************/
void SoInputOisManager::SetMouseListener( OIS::MouseListener* pMouseListener )
{
	ASSERTF( _pOisMouse, ( "\nOIS Keyboard no created!" ) );

	if ( _pOisMouse )
	{
		if ( pMouseListener )
		{
			ASSERTF( _pOisMouse->getEventCallback() == 0, ( "\nOIS callback already set.  Only one allowed according to the OIS header comments." ) );
			if ( _pOisMouse->getEventCallback() == 0 )
				_pOisMouse->setEventCallback( pMouseListener );
		}
		else
		{
			_pOisMouse->setEventCallback( NULL );
		}
	}
}



/******************************************************************************
 FUNCTION:  GetCharacterFromKeyCode
******************************************************************************/
char SoInputOisManager::GetCharacterFromKeyCode( OIS::KeyCode eKeyCode )
{
	switch ( eKeyCode )
	{
		case OIS::KC_UNASSIGNED   : return 0;
		case OIS::KC_ESCAPE       : return 0;
		case OIS::KC_1            : return '1';
		case OIS::KC_2            : return '2';
		case OIS::KC_3            : return '3';
		case OIS::KC_4            : return '4';
		case OIS::KC_5            : return '5';
		case OIS::KC_6            : return '6';
		case OIS::KC_7            : return '7';
		case OIS::KC_8            : return '8';
		case OIS::KC_9            : return '9';
		case OIS::KC_0            : return '0';
		case OIS::KC_MINUS        : return '-';
		case OIS::KC_EQUALS       : return '=';
		case OIS::KC_BACK         : return 0;
		case OIS::KC_TAB          : return 0;
		case OIS::KC_Q            : return 'q';
		case OIS::KC_W            : return 'w';
		case OIS::KC_E            : return 'e';
		case OIS::KC_R            : return 'r';
		case OIS::KC_T            : return 't';
		case OIS::KC_Y            : return 'y';
		case OIS::KC_U            : return 'u';
		case OIS::KC_I            : return 'i';
		case OIS::KC_O            : return 'o';
		case OIS::KC_P            : return 'p';
		case OIS::KC_LBRACKET     : return '[';
		case OIS::KC_RBRACKET     : return ']';
		case OIS::KC_RETURN       : return 0;
		case OIS::KC_LCONTROL     : return 0;
		case OIS::KC_A            : return 'a';
		case OIS::KC_S            : return 's';
		case OIS::KC_D            : return 'd';
		case OIS::KC_F            : return 'f';
		case OIS::KC_G            : return 'g';
		case OIS::KC_H            : return 'h';
		case OIS::KC_J            : return 'j';
		case OIS::KC_K            : return 'k';
		case OIS::KC_L            : return 'l';
		case OIS::KC_SEMICOLON    : return ';';
		case OIS::KC_APOSTROPHE   : return '\'';
		case OIS::KC_GRAVE        : return '`';
		case OIS::KC_LSHIFT       : return 0;
		case OIS::KC_BACKSLASH    : return '\\';
		case OIS::KC_Z            : return 'z';
		case OIS::KC_X            : return 'x';
		case OIS::KC_C            : return 'c';
		case OIS::KC_V            : return 'v';
		case OIS::KC_B            : return 'b';
		case OIS::KC_N            : return 'n';
		case OIS::KC_M            : return 'm';
		case OIS::KC_COMMA        : return ',';
		case OIS::KC_PERIOD       : return '.';
		case OIS::KC_SLASH        : return '/';
		case OIS::KC_RSHIFT       : return 0;
		case OIS::KC_MULTIPLY     : return '*';
		case OIS::KC_LMENU        : return 0;
		case OIS::KC_SPACE        : return ' ';
		case OIS::KC_CAPITAL      : return 0;
		case OIS::KC_F1           : return 0;
		case OIS::KC_F2           : return 0;
		case OIS::KC_F3           : return 0;
		case OIS::KC_F4           : return 0;
		case OIS::KC_F5           : return 0;
		case OIS::KC_F6           : return 0;
		case OIS::KC_F7           : return 0;
		case OIS::KC_F8           : return 0;
		case OIS::KC_F9           : return 0;
		case OIS::KC_F10          : return 0;
		case OIS::KC_NUMLOCK      : return 0;
		case OIS::KC_SCROLL       : return 0;
		case OIS::KC_NUMPAD7      : return '7';
		case OIS::KC_NUMPAD8      : return '8';
		case OIS::KC_NUMPAD9      : return '9';
		case OIS::KC_SUBTRACT     : return '-';
		case OIS::KC_NUMPAD4      : return '4';
		case OIS::KC_NUMPAD5      : return '5';
		case OIS::KC_NUMPAD6      : return '6';
		case OIS::KC_ADD          : return '+';
		case OIS::KC_NUMPAD1      : return '1';
		case OIS::KC_NUMPAD2      : return '2';
		case OIS::KC_NUMPAD3      : return '3';
		case OIS::KC_NUMPAD0      : return '0';
		case OIS::KC_DECIMAL      : return '.';
		case OIS::KC_OEM_102      : return 0;
		case OIS::KC_F11          : return 0;
		case OIS::KC_F12          : return 0;
		case OIS::KC_F13          : return 0;
		case OIS::KC_F14          : return 0;
		case OIS::KC_F15          : return 0;
		case OIS::KC_KANA         : return 0;
		case OIS::KC_ABNT_C1      : return 0;
		case OIS::KC_CONVERT      : return 0;
		case OIS::KC_NOCONVERT    : return 0;
		case OIS::KC_YEN          : return 0;
		case OIS::KC_ABNT_C2      : return 0;
		case OIS::KC_NUMPADEQUALS : return 0;
		case OIS::KC_PREVTRACK    : return 0;
		case OIS::KC_AT           : return '@';
		case OIS::KC_COLON        : return ':';
		case OIS::KC_UNDERLINE    : return '_';
		case OIS::KC_KANJI        : return 0;
		case OIS::KC_STOP         : return 0;
		case OIS::KC_AX           : return 0;
		case OIS::KC_UNLABELED    : return 0;
		case OIS::KC_NEXTTRACK    : return 0;
		case OIS::KC_NUMPADENTER  : return 0;
		case OIS::KC_RCONTROL     : return 0;
		case OIS::KC_MUTE         : return 0;
		case OIS::KC_CALCULATOR   : return 0;
		case OIS::KC_PLAYPAUSE    : return 0;
		case OIS::KC_MEDIASTOP    : return 0;
		case OIS::KC_VOLUMEDOWN   : return 0;
		case OIS::KC_VOLUMEUP     : return 0;
		case OIS::KC_WEBHOME      : return 0;
		case OIS::KC_NUMPADCOMMA  : return 0;
		case OIS::KC_DIVIDE       : return '/';
		case OIS::KC_SYSRQ        : return 0;
		case OIS::KC_RMENU        : return 0;
		case OIS::KC_PAUSE        : return 0;
		case OIS::KC_HOME         : return 0;
		case OIS::KC_UP           : return 0;
		case OIS::KC_PGUP         : return 0;
		case OIS::KC_LEFT         : return 0;
		case OIS::KC_RIGHT        : return 0;
		case OIS::KC_END          : return 0;
		case OIS::KC_DOWN         : return 0;
		case OIS::KC_PGDOWN       : return 0;
		case OIS::KC_INSERT       : return 0;
		case OIS::KC_DELETE       : return 0;
		case OIS::KC_LWIN         : return 0;
		case OIS::KC_RWIN         : return 0;
		case OIS::KC_APPS         : return 0;
		case OIS::KC_POWER        : return 0;
		case OIS::KC_SLEEP        : return 0;
		case OIS::KC_WAKE         : return 0;
		case OIS::KC_WEBSEARCH    : return 0;
		case OIS::KC_WEBFAVORITES : return 0;
		case OIS::KC_WEBREFRESH   : return 0;
		case OIS::KC_WEBSTOP      : return 0;
		case OIS::KC_WEBFORWARD   : return 0;
		case OIS::KC_WEBBACK      : return 0;
		case OIS::KC_MYCOMPUTER   : return 0;
		case OIS::KC_MAIL         : return 0;
		case OIS::KC_MEDIASELECT  : return 0;
	}

	return 0;
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoInputOisManager::CreateInstance()
{
	ASSERTF( _pInstance == NULL, ( "\nCan't create more than one instance.  Not allowed." ) );

	if ( !_pInstance )
		_pInstance = new SoInputOisManager;
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoInputOisManager::DeleteInstance()
{
	if ( _pInstance )
	{
		delete _pInstance;
		_pInstance = NULL;
	}
}


#endif // DCG_IS_TARGET_WIN32
