/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#include <string>

#include "cocos2d.h"

#include "JpList.h"



USING_NS_CC;



#define SoCcTouchList JpList< CCTouch >



// Note, we derive from CCLayer so we can receive Touch input callbacks
class SoInputTouch : public CCLayer  
{
	public:

		//-----------------------
		// public enum and classes
		class Listener
		{
			public:
				Listener() {}
				virtual ~Listener() {}
				virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) { return false; }
				virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList ) { return false; }
				virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) { return false; }
		};



		//-----------------------
		// public functions

		SoInputTouch();
		virtual ~SoInputTouch() {};

		void Init( CCNode* pParentNode );
		void Deinit();
		void Update( float dt );
	
		void AddListener( Listener* pListener )
		{
			ASSERTF( _bListenerListLocked == false, ("Can't add while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.AddUniquely( pListener, 0 );
		}

		void RemoveListener( Listener* pListener )
		{
			ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.Remove( pListener );
		}
		
		void RemoveAllListeners()
		{
			ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.RemoveAll();
		}


		virtual void ccTouchesBegan(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);
		virtual void ccTouchesMoved(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);
		virtual void ccTouchesEnded(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);
		virtual void ccTouchesCancelled(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);


		static JpVector2f TouchPosToScreenPos( const CCPoint& v );

		static SoInputTouch* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();

	private:


		//-----------------------
		// private variables
		static SoInputTouch* _pInstance;


		JpList< Listener > _ListenerList;
		bool _bListenerListLocked;

		CCNode* _pParentNode;

		SoCcTouchList _TouchList;
	

		//-----------------------
		// private functions
		void CopyCcToucheSetToSoTouchList( SoCcTouchList* pSoCcTouchList, cocos2d::CCSet *pTouches );

		void BroadcastTouchesDown( const SoCcTouchList* pSoCcTouchList );
		void BroadcastTouchesUp( const SoCcTouchList* pSoCcTouchList );
		void BroadcastTouchesMove( const SoCcTouchList* pSoCcTouchList );
};





template< class TYPE >
class SoInputTouchListener : public SoInputTouch::Listener
{
	public:
		//-----------------------
		// public functions

		SoInputTouchListener() : _pOwner( NULL ) {}
		~SoInputTouchListener() {}
		void SetOwner( TYPE* p ) { _pOwner = p; }

		virtual bool OnTouchesDown( const SoCcTouchList* pSoCcTouchList ) { return _pOwner->OnTouchesDown( pSoCcTouchList ); }
		virtual bool OnTouchesUp( const SoCcTouchList* pSoCcTouchList ) { return _pOwner->OnTouchesUp( pSoCcTouchList ); }
		virtual bool OnTouchesMove( const SoCcTouchList* pSoCcTouchList ) { return _pOwner->OnTouchesMove( pSoCcTouchList ); }

	private:

		//-----------------------
		// private variables.
		TYPE* _pOwner;


		//-----------------------
		// private functions.
		SoInputTouchListener( SoInputTouchListener& Copy )
		{
			ASSERTF( 0, ( "\nCannot copy instances of this class" ) );
		}
};
