/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoInputVirtualKeyboard.h"
#include "SoLabelBmFont.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



SoInputVirtualKeyboard* SoInputVirtualKeyboard::_pInstance = NULL;



/******************************************************************************
 FUNCTION:  SoInputVirtualKeyboard
******************************************************************************/
SoInputVirtualKeyboard::SoInputVirtualKeyboard() :
_bListenerListLocked( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoInputVirtualKeyboard
******************************************************************************/
SoInputVirtualKeyboard::~SoInputVirtualKeyboard()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoInputVirtualKeyboard::Init()
{
	_bListenerListLocked = false;
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoInputVirtualKeyboard::Deinit()
{
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoInputVirtualKeyboard::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  attachWithIME
******************************************************************************/
bool SoInputVirtualKeyboard::attachWithIME()
{
    bool bRet = CCIMEDelegate::attachWithIME();
    if (bRet)
    {
        // open keyboard
        CCEGLView * pGlView = CCDirector::sharedDirector()->getOpenGLView();
        if (pGlView)
        {
            pGlView->setIMEKeyboardState(true);
        }
    }
    return bRet;
}



/******************************************************************************
 FUNCTION:  detachWithIME
******************************************************************************/
bool SoInputVirtualKeyboard::detachWithIME()
{
    bool bRet = CCIMEDelegate::detachWithIME();
    if (bRet)
    {
        // close keyboard
        CCEGLView * pGlView = CCDirector::sharedDirector()->getOpenGLView();
        if (pGlView)
        {
            pGlView->setIMEKeyboardState(false);
        }
    }
    return bRet;
}



/******************************************************************************
 FUNCTION:  insertText
******************************************************************************/
void SoInputVirtualKeyboard::insertText(const char * text, int len)
{
	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();

		int i = 0;
		char c = text[ i ];

		while ( c != 0 )
		{
			pListener->OnVirtualKeyPressed( c );

			i ++;
			c = text[ i ];
		}
	}

	_bListenerListLocked = false;
}



/******************************************************************************
 FUNCTION:  deleteBackward
******************************************************************************/
void SoInputVirtualKeyboard::deleteBackward()
{
	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();
		pListener->OnVirtualKeyDeleteBackward();
	}

	_bListenerListLocked = false;
}




/******************************************************************************
 FUNCTION:  getContentText
******************************************************************************/
const char* SoInputVirtualKeyboard::getContentText()
{
	return NULL;
}



/******************************************************************************
 FUNCTION:  CreateInstance
 ******************************************************************************/
void SoInputVirtualKeyboard::CreateInstance()
{
	ASSERTF( _pInstance == NULL, ( "\nCan't create more than one instance.  Not allowed." ) );
	
	if ( !_pInstance )
		_pInstance = new SoInputVirtualKeyboard;
}



/******************************************************************************
 FUNCTION:  DeleteInstance
 ******************************************************************************/
void SoInputVirtualKeyboard::DeleteInstance()
{
	if ( _pInstance )
	{
		delete _pInstance;
		_pInstance = NULL;
	}
}


















