/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"
#include "JpList.h"



USING_NS_CC;





class SoInputVirtualKeyboard : public CCIMEDelegate
{
	public:
		//-----------------------
		// public enum and classes
		class Listener
		{
			public:
				Listener() {}
				virtual ~Listener() {};
				virtual void OnVirtualKeyPressed( char cKey ) {}
				virtual void OnVirtualKeyReleased( char cKey ) {}
				virtual void OnVirtualKeyDeleteBackward() {}
				
				virtual void SetLastInputPressed( char cKey ) {}
				virtual void SetLastInputReleased( char cKey ) {}

				
				void Update( float dt );
		};
		
	

		//-----------------------
		// public functions

		SoInputVirtualKeyboard();

		virtual ~SoInputVirtualKeyboard();

		void Init();
		void Deinit();

		void Update( float dt );

		bool BeginTextEntryShowVirtualKeyboardIfNeeded() { return attachWithIME(); }
		bool EndTextEntryHideVirtualKeyboardIfNeeded() { return detachWithIME(); }


		void AddListener( Listener* pListener )
		{
			ASSERTF( _bListenerListLocked == false, ("Can't add while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.AddUniquely( pListener, 0 );
		}
		
		void RemoveListener( Listener* pListener )
		{
			ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.Remove( pListener );
		}
		
		void RemoveAllListeners()
		{
			ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.RemoveAll();
		}
	
	
		// CCIMEDelegate overrides
		virtual bool attachWithIME();
		virtual bool detachWithIME();
		virtual bool canAttachWithIME() { return true; }
		virtual bool canDetachWithIME() { return true; }
		virtual void insertText(const char * text, int len);
		virtual void deleteBackward();
		virtual const char * getContentText();
		
		
		static SoInputVirtualKeyboard* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();
		
		
	private:
		
		//-----------------------
		// private variables
		
		static SoInputVirtualKeyboard* _pInstance;

		JpList< Listener > _ListenerList;
		bool _bListenerListLocked;
};



template< class TYPE >
class SoInputVirtualKeyboardListener : public SoInputVirtualKeyboard::Listener
{
	public:
		//-----------------------
		// public functions
		
		SoInputVirtualKeyboardListener() : _pOwner( NULL ) {}
		~SoInputVirtualKeyboardListener() {}
		
		void SetOwner( TYPE* p ) { _pOwner = p; }
		
		virtual void OnVirtualKeyPressed( char Key )
		{
			_eLastInputPressed = Key;
			_pOwner->OnVirtualKeyPressed( Key );
		}
		virtual void OnVirtualKeyReleased( char Key )
		{
			_eLastInputReleased = Key;
			_pOwner->OnVirtualKeyReleased( Key );
		}
		virtual void OnVirtualKeyDeleteBackward()
		{
			_pOwner->OnVirtualKeyDeleteBackward();
		}
		
		char GetLastInputPressed() { return _eLastInputPressed; }
		char GetLastInputReleased() { return _eLastInputReleased; }
		
		void SetLastInputPressed( char Key ) { _eLastInputPressed = Key; }
		void SetLastInputReleased( char Key ) { _eLastInputReleased = Key; }
		
		
	private:
		
		//-----------------------
		// private variables.
		TYPE* _pOwner;
		char _eLastInputPressed;
		char _eLastInputReleased;
		
		
		//-----------------------
		// private functions.
		SoInputVirtualKeyboardListener( SoInputVirtualKeyboardListener& Copy )
		{
			ASSERTF( 0, ( "\nCannot copy instances of this class" ) );
		}
};

