/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActorVisible.h"
#include "SoActorVisiblePlayer.h"
#include "SoCamera.h"
#include "SoFontDefs.h"
#include "SoLabelBmFont.h"
#include "SoKicker.h"

#include "mmgrChooser.h"




SoKickerManager*  SoKickerManager::_pInstance = NULL;



/******************************************************************************
***************************************************  CLASS:  SoKickerCharacter
******************************************************************************/



/******************************************************************************
 FUNCTION:  SoKickerCharacter
******************************************************************************/
SoKickerCharacter::SoKickerCharacter() :
_pParentNode( NULL ),
_fTimer( 0.0f ),
_pOwnerKicker( NULL ),
_vKickerRelativePos( 0.0f, 0.0f ),
_pBmFontLabel( NULL ),
_eState( STATE_LAST )
{
}



/******************************************************************************
 FUNCTION:  SoKickerCharacter
******************************************************************************/
SoKickerCharacter::~SoKickerCharacter()
{
	DestroyBmFontLabel();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoKickerCharacter::Init( CCNode* pParentNode, char c )
{
	char acText[ 2 ];
	acText[ 0 ] = c;
	acText[ 1 ] = 0;
	_pBmFontLabel = CreateBmFontLabel( acText, SoFontDefs::GetKicker() );

	_pParentNode = pParentNode;
	_pParentNode->addChild( _pBmFontLabel, BASE_RENDER_ORDER_KICKER );

	_eState = STATE_ANIMATE_IN;
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoKickerCharacter::Update( float dt )
{
	_fTimer += dt;

	float fYOffset = 0.0f;

	switch ( _eState )
	{
		case STATE_ANIMATE_IN:
		{
			_pBmFontLabel->setOpacity( 0 );

			if ( _fTimer > 0.0f )
			{
				_pBmFontLabel->setOpacity( 128 );

				float fAngle = _fTimer * 20.0f;

// HACK! -- I hacked up the Kickers stuff so that it works in screen space.  If you want kickers
// for your game, you probably want to use the version in StarterCocosB.  mdomowicz 2013_10_22

				float fMaxHeight = 15.0f * SoUtils::GetiPadRatioY();// * SoUtils::GetSpriteZoom();  
				fYOffset = fMaxHeight * sinf( fAngle );
				fYOffset = Clamp( fYOffset, 0.0f, fMaxHeight );

				if ( fAngle > JpUtils::_fPi )
				{
					_fTimer = 0.0f;
					_eState = STATE_SHOW;				
				}
			}
			break;
		}
		case STATE_SHOW:
		{
			if ( _fTimer > 1.0f )
			{
				_fTimer = 0.0f;
				_eState = STATE_ANIMATE_OUT;
			}

			break;
		}
		case STATE_ANIMATE_OUT:
		{
			float fAlpha = Interpolate( 255.0f, 0.0f, _fTimer * 3.0f );
			fAlpha = Clamp( fAlpha, 0.0f, 255.0f );
			_pBmFontLabel->setOpacity( fAlpha );

			if ( fAlpha == 0.0f )
			{
				_fTimer = 0.0f;
				_eState = STATE_DONE;
			}

			break;
		}
		case STATE_DONE:
		{
			break;
		}
		default:
			ASSERTF( 0, ( "\nunknown state" ) );
	}	

	

	JpVector2f vKickerPos = _pOwnerKicker->GetPosition();
	JpVector2f vCharacterPos = vKickerPos + _vKickerRelativePos;
	vCharacterPos.Y += fYOffset;

// HACK! -- I hacked up the Kickers stuff so that it works in screen space.  If you want kickers
// for your game, you probably want to use the version in StarterCocosB.  mdomowicz 2013_10_22
	JpVector2f vCameraRelativePos = ( vCharacterPos );
	//JpVector2f vCameraRelativePos = SoCameraManager::GetInstance()->GetCurrentCamera()->TransformWorldToScreenSpace( vCharacterPos );

	_pBmFontLabel->setPosition( vCameraRelativePos.X, vCameraRelativePos.Y );


// HACK! -- I hacked up the Kickers stuff so that it works in screen space.  If you want kickers
// for your game, you probably want to use the version in StarterCocosB.  mdomowicz 2013_10_22
	_pBmFontLabel->setScaleX( SoUtils::GetiPadRatioX() );//* SoUtils::GetSpriteZoom() );
	_pBmFontLabel->setScaleY( SoUtils::GetiPadRatioY() );//* SoUtils::GetSpriteZoom() );
}



/******************************************************************************
 FUNCTION:  GetWidthInPixels
******************************************************************************/
float SoKickerCharacter::GetWidthInPixels()
{
	if ( _pBmFontLabel )
	{
		return _pBmFontLabel->GetCalculatedWidthAllRes();
	}

	return 0.0f;
}



/******************************************************************************
 FUNCTION:  CreateBmFontLabel
******************************************************************************/
SoLabelBmFont* SoKickerCharacter::CreateBmFontLabel( const std::string& sText, const std::string& sFontName )
{
	CCSize size = CCDirector::sharedDirector()->getWinSize();

	SoLabelBmFont* pLabel = SoLabelBmFont::create
	(
		sText.c_str(), 
		SoUtils::AddRelativePathToFont( sFontName.c_str() ), 
		size.width, 
		kCCTextAlignmentLeft
	);
	pLabel->setVisible( true );
	pLabel->setOpacity( 128.0f );
	pLabel->retain();
	pLabel->setAnchorPoint( CCPoint( 0, 0 ) );
	pLabel->setScaleX( SoUtils::GetiPadRatioX() );
	pLabel->setScaleY( SoUtils::GetiPadRatioY() );

	_pBmFontLabel = pLabel;

	return _pBmFontLabel;
}



/******************************************************************************
 FUNCTION:  DestroyBmFontLabel
******************************************************************************/
void SoKickerCharacter::DestroyBmFontLabel()
{
	_pParentNode->removeChild( _pBmFontLabel, DCG_CLEANUP_ALL_RUNNING_ACTIONS );

	if ( _pBmFontLabel )
	{
		_pBmFontLabel->release();
		_pBmFontLabel = NULL;
	}
}






/******************************************************************************
************************************************************  CLASS:  SoKicker
******************************************************************************/



/******************************************************************************
 FUNCTION:  SoKicker
******************************************************************************/
SoKicker::SoKicker() :
_fTimer( 0.0f ),
_bIsDone( false ),
_gOwner( 0 )
{
}



/******************************************************************************
 FUNCTION:  ~SoKicker
******************************************************************************/
SoKicker::~SoKicker()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoKicker::Init( CCNode* pParentNode, guid gActorOwner, std::string sText )
{
	SetOwner( gActorOwner );

	//-----------------------
	// generate a random offset to position the kicker
	// Note: the position is scaled to sprite zoom and res, later in the
	// calculations.  mdomowicz 2013_04_01
	const float fRangeX = 0.0f;
	const float fBaseY = 0.0f;
	const float fRangeY = 0.0f;
	_vOffsetFromOwner.X = Interpolate( -fRangeX, fRangeX, JpRandom::G.GetFloat() );
	_vOffsetFromOwner.Y = fBaseY + Interpolate( 0.0f, fRangeY, JpRandom::G.GetFloat() );
	

	//-----------------------
	// Separate the string into individual characters.
	// Keep track of the total string width in pixels, which we'll need for positioning later.
	float fTotalWidth = 0.0f;

	for ( int i = 0; i < (int)sText.length(); i ++ )
	{
		SoKickerCharacter* pNewChar = new SoKickerCharacter;
		pNewChar->Init( pParentNode, sText[ i ] );
		pNewChar->SetOwner( this );
		_KickerCharacterList.AddLast( pNewChar );

		fTotalWidth += pNewChar->GetWidthInPixels() * SoUtils::GetiPadRatioX();

		// Also set the character timers such that they are offset from each other.
		// This is how we achieve a per-letter effect
		pNewChar->SetTimer( -0.05f * i );
	}


	//-----------------------
	// Set the position of each character so that they are
	// properly kerned and so that the whole string is centered
	float fHalfWidth = fTotalWidth / 2.0f;
	float fCurrentPos = -fHalfWidth;

	SoKickerCharacterList::Iterator Iter( &_KickerCharacterList );
	while ( Iter.Next() )
	{
		SoKickerCharacter* pChar = Iter.GetNodeData();
		JpVector2f vPos( fCurrentPos, 0.0f );
		pChar->SetKickerRelativePos( vPos );
		fCurrentPos += pChar->GetWidthInPixels();
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoKicker::Deinit()
{
	SoKickerCharacterList::Iterator Iter( &_KickerCharacterList );
	while ( Iter.Next() )
	{
		SoKickerCharacter* pKickerChar = Iter.GetNodeData();
		delete pKickerChar;
	}

	_KickerCharacterList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoKicker::Update( float dt )
{
	_fTimer += dt;


	SoActor* pActor = SoActorManager::GetInstance()->GetActorByGuid( _gOwner );
	if ( pActor )
	{
		SoActorVisible* pActorVisible = pActor->GetAsVisible();
		SoActorVisiblePlayer* pActorVisiblePlayer = pActor->GetAsVisiblePlayer();

		if ( pActorVisiblePlayer )
		{
			const float fMoveRate = 10.0f * SoUtils::GetiPadRatioY();
			_vOffsetFromOwner.Y += fMoveRate * dt;
			_vPosition = _vOriginalPosition + _vOffsetFromOwner;
		}
		else if ( pActorVisible )
		{
			JpVector2f vActorPos = pActorVisible->GetWorldPosition();
			const float fMoveRate = 10.0f * SoUtils::GetiPadRatioY();
			_vOffsetFromOwner.Y += fMoveRate * dt;
			_vPosition = vActorPos + _vOffsetFromOwner;
		}
	}



	_bIsDone = true;

	SoKickerCharacterList::Iterator Iter( &_KickerCharacterList );
	while ( Iter.Next() )
	{
		SoKickerCharacter* pKickerChar = Iter.GetNodeData();
		pKickerChar->Update( dt );

		if ( !pKickerChar->IsDone() )
			_bIsDone = false;
	}
}






/******************************************************************************
*****************************************************  CLASS:  SoKickerManager
******************************************************************************/



/******************************************************************************
 FUNCTION:  SoKickerManager
******************************************************************************/
SoKickerManager::SoKickerManager() :
_pParentNode( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoKickerManager
******************************************************************************/
SoKickerManager::~SoKickerManager()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoKickerManager::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoKickerManager::Deinit()
{
	SoKickerList::Iterator Iter( &_KickerList );
	while( Iter.Prev() )
	{
		SoKicker* pKicker = Iter.GetNodeData();
		delete pKicker;
	}

	_KickerList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoKickerManager::Update( float dt )
{
	SoKickerList::Iterator Iter( &_KickerList );
	while( Iter.Prev() )
	{
		SoKicker* pKicker = Iter.GetNodeData();

		if ( pKicker->IsDone() )
		{
			_KickerList.Remove( pKicker );
			delete ( pKicker );
		}
		else
			pKicker->Update( dt );
	}
}



/******************************************************************************
 FUNCTION:  AddKicker
******************************************************************************/
void SoKickerManager::AddKicker( SoActor* pActor, JpVector2f vPosition, const std::string& sMessage )
{
	if ( pActor )
	{
		SoActorVisible* pActorVisible = pActor->GetAsVisible();
		if ( pActorVisible )
		{
			guid gOwner = pActor->GetGuid();

			SoKicker* pNewKicker = new SoKicker;
			pNewKicker->Init( _pParentNode, gOwner, sMessage );
			pNewKicker->SetPosition( vPosition );
			pNewKicker->SetOriginalPosition( vPosition );

			_KickerList.AddLast( pNewKicker );
		}
	}
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoKickerManager::CreateInstance()
{
	ASSERTF( !_pInstance, ( "\nCan't create a second instance." ) )
	if ( !_pInstance )
	{
		_pInstance = new SoKickerManager;
		_pInstance->Init();
	}
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoKickerManager::DeleteInstance()
{
	if ( _pInstance )
	{
		_pInstance->Deinit();
		delete _pInstance;
		_pInstance = NULL;
	}
}





