/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#include "JpList.h"
#include "SoActorManager.h"


USING_NS_CC;


class SoActor;
class SoKicker;
class SoKickerCharacter;
class SoLabelBmFont;


#define SoKickerList JpList< SoKicker >
#define SoKickerCharacterList JpList< SoKickerCharacter >



/******************************************************************************
***************************************************  CLASS:  SoKickerCharacter
******************************************************************************/

class SoKickerCharacter
{
	public: 

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_ANIMATE_IN,
			STATE_SHOW,
			STATE_ANIMATE_OUT,
			STATE_DONE,
			STATE_LAST
		};

		//-----------------------
		// public functions
		SoKickerCharacter();
		virtual ~SoKickerCharacter();

		void Init( CCNode* pParentNode, char c );

		void Update( float dt );

		void SetKickerRelativePos( const JpVector2f& v ) { _vKickerRelativePos = v; }

		void SetOwner( SoKicker* p ) { _pOwnerKicker = p; }

		void SetTimer( float f ) { _fTimer = f; }

		SoLabelBmFont* CreateBmFontLabel( const std::string& sText, const std::string& sFontName );
		void DestroyBmFontLabel();
		SoLabelBmFont* GetBmFontLabel() { return _pBmFontLabel; }

		float GetWidthInPixels();

		bool IsDone() { return _eState == STATE_DONE; }

	private:
		//-----------------------
		// private variables
		SoKicker* _pOwnerKicker;
		float _fTimer;

		SoLabelBmFont* _pBmFontLabel;
		CCNode* _pParentNode;

		STATE _eState;

		JpVector2f _vKickerRelativePos;
};





/******************************************************************************
************************************************************  CLASS:  SoKicker
******************************************************************************/

class SoKicker
{
	public:

		//-----------------------
		// public functions
		SoKicker();
		virtual ~SoKicker();

		void Init( CCNode* pParentNode, guid gActorOwner, std::string sText );
		void Deinit();

		void Update( float dt );

		const JpVector2f& GetPosition() { return _vPosition; }
		void SetPosition( const JpVector2f& v ) { _vPosition = v; }

// HACK! -- I hacked up the Kickers stuff so that it works in screen space.  If you want kickers
// for your game, you probably want to use the version in StarterCocosB.  mdomowicz 2013_10_22
		const JpVector2f& GetOriginalPosition() { return _vOriginalPosition; }
		void SetOriginalPosition( const JpVector2f& v ) { _vOriginalPosition = v; }

		void SetOwner( guid gOwner ) { _gOwner = gOwner; }
		guid GetOwner() { return _gOwner; }

		bool IsDone() { return _bIsDone; }

		SoKickerCharacterList* GetKickerCharacterList() { return &_KickerCharacterList; }
		

	private:

		//-----------------------
		// private variables
		guid _gOwner;
		float _fTimer;
	
		JpVector2f _vPosition;
		JpVector2f _vOriginalPosition;
		JpVector2f _vOffsetFromOwner;

		std::string  _sText;

		JpList< SoKickerCharacter > _KickerCharacterList;

		bool _bIsDone;
};





/******************************************************************************
*****************************************************  CLASS:  SoKickerManager
******************************************************************************/

class SoKickerManager
{
	public:

		//-----------------------
		// public functions
		SoKickerManager();
		virtual ~SoKickerManager();

		void Init();
		void Deinit();

		void Update( float dt );

// HACK! -- I hacked up the Kickers stuff so that it works in screen space.  If you want kickers
// for your game, you probably want to use the version in StarterCocosB.  mdomowicz 2013_10_22
		void AddKicker( SoActor* pActor, JpVector2f vPosition, const std::string& sMessage );

		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }

		static SoKickerManager* GetInstance() {	return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();

	private:

		//-----------------------
		// private variables
	
		SoKickerList _KickerList;

		static SoKickerManager* _pInstance;

		CCNode* _pParentNode;
};




