/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCamera.h"
#include "SoLightOverlay.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoLightOverlay
******************************************************************************/
SoLightOverlay::SoLightOverlay()
{
}



/******************************************************************************
 FUNCTION:  ~SoLightOverlay
******************************************************************************/
SoLightOverlay::~SoLightOverlay()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoLightOverlay::Init( CCNode* pParentNode, const char* pacPathAndFilename )
{
	_pParentNode = pParentNode;
	_pSprite = CreateSprite( _pParentNode, pacPathAndFilename );

	// The sprite is set to visible only in the update.  This prevents a 1-frame flash in which
	// this sprite shows, but nothing else does.
	// mdomowicz 2013_02_19
	_pSprite->setVisible( false );

	_pSprite->setScale( 20.0f * SoUtils::GetiPadRatio() * SoUtils::GetSpriteZoom() );
	SoUtils::SetRenderZOrder( _pParentNode, _pSprite, BASE_RENDER_ORDER_LIGHT_OVERLAY );

	//ccBlendFunc BlendFunc;
	//BlendFunc.src = GL_ONE_MINUS_SRC_COLOR;
	//BlendFunc.dst = GL_SRC_COLOR;
	//_pSprite->setBlendFunc( BlendFunc );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoLightOverlay::Deinit()
{
	_pParentNode->removeChild( _pSprite, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
	_pSprite->release();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoLightOverlay::Update( float dt )
{
	_pSprite->setVisible( true );

	// Here we find the pixel position of the center of the current map
	// This is the center of the world, and we want to place the overlay there.
	/*
	SoGameDungeon* pDungeon = SoGameDungeon::GetInstance();
	SoTileMap* pMap = pDungeon->GetCurrentTileMap();

	int iCenterTileX = pMap->GetMapWidth() / 2;
	int iCenterTileY = pMap->GetMapHeight() / 2;
	*/

	int iCenterTileX = 64 / 2;
	int iCenterTileY = 64 / 2;

	float fSpriteSize = SoUtils::GetBaseSpriteWidth();
	float fSpriteSizeX = fSpriteSize;
	float fSpriteSizeY = fSpriteSize;
	JpVector2f vWorld
	(
		(float)iCenterTileX * fSpriteSizeX,
		(float)iCenterTileY * fSpriteSizeY
	);


	// Here we find the overlays's camera relative position.
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->GetCurrentCamera();
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	JpVector2f vCameraSpacePos = ( vWorld - pCamera->GetPosition() );
	vCameraSpacePos.X *= SoUtils::GetSpriteZoom() * SoUtils::GetiPadRatioX();
	vCameraSpacePos.Y *= SoUtils::GetSpriteZoom() * SoUtils::GetiPadRatioY();

	JpVector2f vScreenOffset( ( SizeInPixels.width / 2.0f ), ( SizeInPixels.height / 2.0f ) );
	JpVector2f vFinal( vCameraSpacePos + vScreenOffset );

	_pSprite->setScale( 20.0f * SoUtils::GetiPadRatio() * SoUtils::GetSpriteZoom() );
	_pSprite->SetPosition( vFinal );
}



/******************************************************************************
 FUNCTION:  CreateSprite
******************************************************************************/
SoSprite* SoLightOverlay::CreateSprite( CCNode* pParent, const char* pacPathAndFilename )
{
	SoSprite* pSprite = SoSprite::CreateScaledToDevice( pacPathAndFilename );
	pSprite->retain();
	pParent->addChild( pSprite );
	pSprite->setVisible( false );
	pSprite->setAnchorPoint( ccp( 0.5f, 0.5f ) );


	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	pSprite->setPosition( ccp( SizeInPixels.width / 2, SizeInPixels.height / 2 ) );
	

	return pSprite;
}
