/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "JpChecksum.h"
#include "JpHashTable.h"
#include "JpList.h"
#include "SoGameStateMcts.h"


USING_NS_CC;



#define SoMctsTreeNodeList  JpList< SoMctsTreeNode >

class SoGameBitPlayerMcts;

class SoMctsTreeNode
{
	public:


		//-----------------------
		// public functions

		SoMctsTreeNode();

		~SoMctsTreeNode();

		void Init();

		SoMctsTreeNode* GetParentNode() { return _pParentNode; }
		void SetParentNode( SoMctsTreeNode* p ) { _pParentNode = p; }


		SoMctsTreeNodeList* GetChildNodeList() { return &_ChildNodeList; }
		void AddChildNode( SoMctsTreeNode* p ) { _ChildNodeList.AddLast( p ); }

		hash GetRngSeed() { return _hRngSeed; }
		void SetRngSeed( hash hSeed ) { _hRngSeed = hSeed; }

		void EvaluateNode( SoGameBitPlayerMcts* pPlayer, SoGameStateMcts* pGameMcts );
		SoMctsTreeNode* CreateOrSelectNodeToEvaluate( SoGameBitPlayerMcts* pPlayerMcts, SoGameStateMcts* pGameMcts ); 

		int GetNumVisits() { return _iNumVisits; }
		void SetNumVisits( int i ) { _iNumVisits = i; }
		void AdjustNumVisits( int i ) { _iNumVisits += i; }

		SoMctsTreeNode* GetChildWithBestScore();

		void RemoveAndDeleteAllChildNodes();

		SoGameMessage* GetGameMessage() { return &_GameMessage; }
		void CopyGameMessage( SoGameMessage* pSource );

		float GetPlayerEvaluation( int iPlayerNumber );
		void SetPlayerEvaluation( int iPlayerNumber, float fScore );
		void AdjustPlayerEvaluation( int iPlayerNumber, float fScore );

		float CalculateNodesValueToPlayer( int iPlayerNumber );

		int GetPlayerNumber() { return _iPlayerNumber; }
		void SetPlayerNumber( int i ) { _iPlayerNumber = i; }

		void SetNumMovesPerSimulation( int i ) { _iNumMovesPerSimulation = i; }

		ChecksumUint32 GetChecksum() { return _GameChecksum; }
		void SetChecksum( ChecksumUint32 ui ) { _GameChecksum = ui; }

		int CalculateTotalSubNodeCount();
		int CalculateMaxDepth();
		int CalculateMaxBreadth();

		static void SetDoChecksumTesting( bool b ) { _bDoChecksumTesting = b; }

	private:

		//-----------------------
		// private variables

		SoMctsTreeNode* _pParentNode;

		hash _hRngSeed;

		SoMctsTreeNodeList _ChildNodeList;

		int _iNumVisits;

		int _iPlayerNumber;

		int _iNumMovesPerSimulation;

		SoGameMessage _GameMessage;


		float _afPlayerEvaluations[ GAMERULES_MAX_PLAYERS ];

		ChecksumUint32 _GameChecksum;

		static bool _bDoChecksumTesting;

		//-----------------------
		// private functions

		SoMctsTreeNode* SelectNodeToEvaluate( int iPlayerNumber );

		SoMctsTreeNode* FindChildNodeWithDuplicateMove( SoGameMessage* pMessage, SoGameStateMcts* pGameMcts );
		bool AreTheseMessagesDuplicates( SoGameMessage* pMessage1, SoGameMessage* pMessage2, SoGameStateMcts* pGameMcts );

		SoGameBitPlayerMcts* FindPlayerCommandedToMove( SoGameStateMcts* pGameMcts );
};



class SoMctsTree
{
	public:

		//-----------------------
		// public functions

		SoMctsTree();

		virtual ~SoMctsTree();

		void Init();

		SoMctsTreeNode* GetHeadNode() { return &_HeadNode; }

		SoGameMessage* SearchForMove( SoGameBitPlayer* pPlayer, SoGameState* pGame, SoGameState::STATE eServersSoGameState );

		void RemoveAndDeleteAllNodes();

		void SetMaxSimulations( int i ) { _iMaxSimulations = i; }
		void SetNumMovesPerSimulation( int i ) { _iNumMovesPerSimulation = i; }

	private:

		//-----------------------
		// private variables

		SoMctsTreeNode _HeadNode;
		SoGameStateMcts _GameMcts;

		int _iMaxSimulations;
		int _iNumSimulations;

		int _iNumMovesPerSimulation;


		//-----------------------
		// private functions
		void DebugPrintDiagnostics( SoGameState* pGame );
};




