/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpDebug.h"
#include "JpString.h"
#include "SoConfigManager.h"
#include "SoCurl.h"
#include "SoFsmStateMain.h"
#include "SoGameGlobals.h"
#include "SoNetworkCommsOnline.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoNetworkCommsOnline
******************************************************************************/
SoNetworkCommsOnline::SoNetworkCommsOnline() :
_iUpdateState( 0 ),
_bPrintNetworkMessages( false ),
_bTellWebServerGameHasStarted( false ),
_bGotWebServersOnConnectMessage( false ),
_fKeepAliveTestTimer( 0.0f )
{
	Init();
}



/******************************************************************************
 FUNCTION:  ~SoNetworkCommsOnline
******************************************************************************/
SoNetworkCommsOnline::~SoNetworkCommsOnline()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoNetworkCommsOnline::Init()
{
	SoNetworkComms::Init();

	_CurlListener.SetOwner( this );
	SoCurl::GetInstance()->AddListener( &_CurlListener );

	_iUpdateState = 0;
	_fKeepAliveTestTimer = 0.0f;
	_bTellWebServerGameHasStarted = false;
	_bGotWebServersOnConnectMessage = false;

	NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "bPrintNetworkMessages" );
	if ( pLine )
		_bPrintNetworkMessages = pLine->GetBool( 1 );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoNetworkCommsOnline::Deinit()
{
	SoNetworkComms::Deinit();

	SoGameMessageList::Iterator Iter1( &_OutgoingMessagesForServers );
	while ( Iter1.Next() )
		delete Iter1.GetNodeData();
	_OutgoingMessagesForServers.RemoveAll();
	
	SoGameMessageList::Iterator Iter2( &_OutgoingMessagesForClients );
	while ( Iter2.Next() )
		delete Iter2.GetNodeData();
	_OutgoingMessagesForClients.RemoveAll();


	SoCurl::GetInstance()->RemoveListener( &_CurlListener );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoNetworkCommsOnline::Update( float dt )
{
	SoNetworkComms::Update( dt );

	_fKeepAliveTestTimer += dt;

	SoCurl* pCurl = SoCurl::GetInstance();

	if ( pCurl->IsReadyForNextQuery() )
	{
		const int GET_MESSAGE_FOR_SERVER = 0;
		const int GET_MESSAGE_FOR_CLIENT = 1;
		const int POST_MESSAGE_TO_SERVER = 2;
		const int POST_MESSAGE_TO_CLIENT = 3;
		const int GET_CHAT_TEXT = 4;
		const int POST_CHAT_TEXT = 5;
		const int POST_MESSAGE_TO_WEB_SERVER = 6;
		const int GET_WEB_SERVERS_ON_CONNECT_MESSAGE = 7;
		const int GET_KEEP_ALIVE_STATUSES = 8;

		switch ( _iUpdateState )
		{
			case GET_MESSAGE_FOR_SERVER:
			{
				SoGameServer* pServer = SoGameGlobals::GetServer();

				if ( pServer )
				{
					std::string sData = "Operation=GetNextMessageForServer";
					sData += "&ServerId=";
					sData += JpString::HashToString( pServer->GetServerId() );

					pCurl->SetUrlToPhpFile( CURL_URL_MAIN_PHP );
					pCurl->DoGet( sData.c_str() );

					if ( _bPrintNetworkMessages )
					{
						CCLOG( "GET for server." );
					}
				}

				_iUpdateState ++;
				break;
			}

			case GET_MESSAGE_FOR_CLIENT:
			{
				SoGameClient* pClient = GetClient();

				if ( pClient )
				{
					std::string sData = "Operation=GetNextMessageForClient";
					sData += "&ServerId=";
					sData += JpString::HashToString( pClient->GetServerId() );
					sData += "&ClientId=";
					sData += JpString::HashToString( pClient->GetClientId() );

					pCurl->SetUrlToPhpFile( CURL_URL_MAIN_PHP );
					pCurl->DoGet( sData.c_str() );

					if ( _bPrintNetworkMessages )
					{
						CCLOG( "GET for client." );
					}
				}

				_iUpdateState ++;
				break;
			}

			case POST_MESSAGE_TO_SERVER:
			{
				SoGameMessageList::Node* pFirstNode = _OutgoingMessagesForServers.GetFirst();
				SoGameMessage* pMessage = pFirstNode->GetData();

				if ( pMessage )
				{
					std::string sData = "Operation=SendMessageToServer";
					AppendCurlDataFromMessage( &sData, *pMessage );

					SoCurl::GetInstance()->SetUrlToPhpFile( CURL_URL_MAIN_PHP );
					SoCurl::GetInstance()->DoPost( sData.c_str() );

					_OutgoingMessagesForServers.Remove( pMessage );


					if ( _bPrintNetworkMessages )
					{
						CCLOG( "POST to server: %s", sData.c_str() );
					}

					delete pMessage;
				}

				_iUpdateState ++;
				break;
			}

			case POST_MESSAGE_TO_CLIENT:
			{
				SoGameMessageList::Node* pFirstNode = _OutgoingMessagesForClients.GetFirst();
				SoGameMessage* pMessage = pFirstNode->GetData();

				if ( pMessage )
				{
					std::string sData = "Operation=SendMessageToClient";
					AppendCurlDataFromMessage( &sData, *pMessage );

					SoCurl::GetInstance()->SetUrlToPhpFile( CURL_URL_MAIN_PHP );
					SoCurl::GetInstance()->DoPost( sData.c_str() );

					_OutgoingMessagesForClients.Remove( pMessage );


					if ( _bPrintNetworkMessages )
					{
						CCLOG( "POST to client: %s", sData.c_str() );
					}

					delete pMessage;
				}

				_iUpdateState ++;
				break;
			}

			case GET_CHAT_TEXT:
			{
				SoGameClient* pClient = GetClient();

				if ( pClient )
				{
					std::string sData = "Operation=GetChatText";
					sData += "&ServerId=";
					sData += JpString::HashToString( pClient->GetServerId() );
					sData += "&ClientId=";
					sData += JpString::HashToString( pClient->GetClientId() );

					pCurl->SetUrlToPhpFile( CURL_URL_CHAT_PHP );
					pCurl->DoGet( sData.c_str() );

					if ( _bPrintNetworkMessages )
					{
						CCLOG( "GET for client." );
					}
				}

				_iUpdateState ++;
				break;
			}

			case POST_CHAT_TEXT:
			{
				SoNetworkChatTextList::Node* pFirstNode = _OutgoingChatText.GetFirst();
				SoNetworkChatText* pChatText = pFirstNode->GetData();

				if ( pChatText )
				{
					std::string sData = "Operation=SendChatText";
					AppendCurlDataFromChatText( &sData, pChatText );

					SoCurl::GetInstance()->SetUrlToPhpFile( CURL_URL_CHAT_PHP );
					SoCurl::GetInstance()->DoPost( sData.c_str() );

					_OutgoingChatText.Remove( pChatText );


					if ( _bPrintNetworkMessages )
					{
						CCLOG( "POST chat text: %s", sData.c_str() );
					}

					delete pChatText;
				}

				_iUpdateState ++;
				break;
			}

			case POST_MESSAGE_TO_WEB_SERVER:
			{
				if ( _bTellWebServerGameHasStarted )
				{
					_bTellWebServerGameHasStarted = false;

					SoGameServer* pServer = SoGameGlobals::GetServer();

					if ( pServer )
					{
						std::string sData = "Operation=PostNewServerStatus";
						sData += "&ServerId=";
						sData += JpString::HashToString( pServer->GetServerId() );
						sData += "&Data=";
						sData += "game_started";

						pCurl->SetUrlToPhpFile( CURL_URL_MAIN_PHP );
						pCurl->DoPost( sData.c_str() );

						if ( _bPrintNetworkMessages )
						{
							CCLOG( "POST message to web server." );
						}
					}
				}

				_iUpdateState ++;
				break;
			}

			case GET_WEB_SERVERS_ON_CONNECT_MESSAGE:
			{
				if ( !_bGotWebServersOnConnectMessage  )
				{
					_bGotWebServersOnConnectMessage = true;

					std::string sData = "Operation=GetWebServersOnConnectMessage";

					pCurl->SetUrlToPhpFile( CURL_URL_MAIN_PHP );
					pCurl->DoGet( sData.c_str() );

					if ( _bPrintNetworkMessages )
					{
						CCLOG( "GET web servers on connect message." );
					}
				}

				_iUpdateState ++;
				break;
			}

			case GET_KEEP_ALIVE_STATUSES:
			{
				if ( _fKeepAliveTestTimer > 5.0f )
				{
					_fKeepAliveTestTimer -= 5.0f;

					SoGameClient* pClient = GetClient();

					if ( pClient )
					{
						std::string sData = "Operation=GetKeepAliveStatus";
						sData += "&ServerId=";
						sData += JpString::HashToString( pClient->GetServerId() );
						sData += "&ClientId=";
						sData += JpString::HashToString( pClient->GetClientId() );

						pCurl->SetUrlToPhpFile( CURL_URL_MAIN_PHP );
						pCurl->DoGet( sData.c_str() );

						if ( _bPrintNetworkMessages )
						{
							CCLOG( "GET keep alive statuses." );
						}
					}
				}

				_iUpdateState ++;
				break;
			}

			default:
			{
				_iUpdateState = 0;
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  FindAvailableServer
******************************************************************************/
hash SoNetworkCommsOnline::FindAvailableServer()
{
	if ( SoGameGlobals::GetServer() )
		return SoGameGlobals::GetServer()->GetServerId();

	return 0;
}




/******************************************************************************
 FUNCTION:  SendMessageToServer
******************************************************************************/
void SoNetworkCommsOnline::SendMessageToServer( SoGameMessage& Message )
{
	SoGameMessage* pNewMessage = new SoGameMessage;
	pNewMessage->CopyMessage( Message );

	_OutgoingMessagesForServers.AddLast( pNewMessage );

}



/******************************************************************************
 FUNCTION:  SendMessageToClient
******************************************************************************/
void SoNetworkCommsOnline::SendMessageToClient( SoGameMessage& Message )
{
	SoGameMessage* pNewMessage = new SoGameMessage;
	pNewMessage->CopyMessage( Message );

	_OutgoingMessagesForClients.AddLast( pNewMessage );
}



/******************************************************************************
 FUNCTION:  GetNextMessageForServer
******************************************************************************/
SoGameMessage* SoNetworkCommsOnline::GetNextMessageForServer( hash hServerId )
{
	SoGameMessageList::Iterator Iter( &_MessagesForServers );
	while ( Iter.Next() )
	{
		SoGameMessage* pMessage = Iter.GetNodeData();
		if ( pMessage->GetServerId() == hServerId )
			return pMessage;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  RemoveAndDeleteMessageForServer
******************************************************************************/
void SoNetworkCommsOnline::RemoveAndDeleteMessageForServer( SoGameMessage* p )
{
	_MessagesForServers.Remove( p );
	delete p;
}




/******************************************************************************
 FUNCTION:  GetNextMessageForClient
******************************************************************************/
SoGameMessage* SoNetworkCommsOnline::GetNextMessageForClient( hash hServerId, hash hClientId )
{
	SoGameMessageList::Iterator Iter( &_MessagesForClients );
	while ( Iter.Next() )
	{
		SoGameMessage* pMessage = Iter.GetNodeData();
		if ( pMessage->GetServerId() == hServerId )
		{
			if ( pMessage->GetClientId() == hClientId )
				return pMessage;
		}
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  RemoveAndDeleteMessageForClient
******************************************************************************/
void SoNetworkCommsOnline::RemoveAndDeleteMessageForClient( SoGameMessage* p )
{
	_MessagesForClients.Remove( p );
	delete p;
}



/******************************************************************************
 FUNCTION:  AppendCurlDataFromMessage
******************************************************************************/
void SoNetworkCommsOnline::AppendCurlDataFromMessage( std::string* pResultString, SoGameMessage& Message )
{
	std::string sData = "";

	sData += "&ServerId=";
	sData += JpString::HashToString( Message.GetServerId() );
	sData += "&ClientId=";
	sData += JpString::HashToString( Message.GetClientId() );


	sData += "&Data=";
	sData += JpString::HashToString( Message.GetServerId() );
	sData += ":";
	sData += JpString::HashToString( Message.GetClientId() );
	sData += ":";
	sData += JpString::IntToString( Message.GetMessageNumber() );
	sData += ":";
	sData += JpString::IntToString( (int)Message.GetType() );

	SoGameMessageDataList::Iterator Iter( Message.GetDataList() );
	while ( Iter.Next() )
	{
		SoGameMessage::Data* pData = Iter.GetNodeData();

		sData += ":";
		sData += JpString::IntToString( pData->GetInt() );
	}

	// ParseMessageFromCurlData requires that the end of the 
	// message ends with a : too.
	sData += ":";

	*pResultString += sData;
}



/******************************************************************************
 FUNCTION:  AppendCurlDataFromChatText
******************************************************************************/
void SoNetworkCommsOnline::AppendCurlDataFromChatText( std::string* pResultString, SoNetworkChatText* pChatText )
{
	SoGameClient* pClient = GetClient();

	if ( pClient )
	{
		std::string sData = "";

		sData += "&ServerId=";
		sData += JpString::HashToString( pClient->GetServerId() );
		sData += "&ClientId=";
		sData += JpString::HashToString( pClient->GetClientId() );

		sData += "&Data=";
		sData += pChatText->GetText();

		*pResultString += sData;
	}
}



/******************************************************************************
 FUNCTION:  OnCurlDataReceived
******************************************************************************/
void SoNetworkCommsOnline::OnCurlDataReceived( const std::string& sData )
{
	if ( _bPrintNetworkMessages )
	{
		CCLOG( "Received: %s", sData.c_str() );
	}


	std::string sMessageType = ExtractMessageTypeFromCurlData( sData );

	if ( sMessageType == "ChatTextForClient" )
	{
		SoNetworkChatText* pChatText = new SoNetworkChatText;
		ParseChatTextFromCurlData( pChatText, sData );
		_IncomingChatText.AddLast( pChatText );
	}
	else if ( sMessageType == "NextMessageForClient" )
	{
		SoGameMessage* pMessage = new SoGameMessage;
		ParseMessageFromCurlData( pMessage, sData );
		_MessagesForClients.Add( pMessage, pMessage->GetMessageNumber() );
	}
	else if ( sMessageType == "NextMessageForServer"  )
	{
		SoGameMessage* pMessage = new SoGameMessage;
		ParseMessageFromCurlData( pMessage, sData );
		_MessagesForServers.Add( pMessage, pMessage->GetMessageNumber() );
	}
	else if ( sMessageType == "KeepAliveStatus" )
	{
		if ( sData == "KeepAliveStatus:PlayerHasDropped" )
		{
			SoHudLog* pLog = SoFsmMainLayer::GetSharedHudLog();
			pLog->AddMessageF( JpUtils::_Red, "A player has disconnected.  The game cannot continue." );
		}
	}
	else if ( sMessageType == "WebServersOnConnectMessage" )
	{
		std::string sMessageData = ExtractMessageDataFromCurlData( sData );
		
		SoHudLog* pLog = SoFsmMainLayer::GetSharedHudLog();
		pLog->AddMessageF( JpUtils::_Yellow, sMessageData.c_str() );
	}
	else
	{
		ASSERTF( 0, ( "\nUnknown message type received from web server." ) );
	}
}



/******************************************************************************
 FUNCTION:  ExtractMessageTypeFromCurlData
******************************************************************************/
std::string SoNetworkCommsOnline::ExtractMessageTypeFromCurlData( const std::string& sData )
{
	const int STATE_READ_WORD = 10;
	const int STATE_DONE = 20;

	std::string sWord = "";

	int iState = STATE_READ_WORD;
	int iSize = sData.size();

	int i = 0;
	while ( i < iSize && iState != STATE_DONE )
	{
		switch ( iState )
		{
			case STATE_READ_WORD:
			{
				char c = sData[ i ];

				if ( c != ':' )
				{
					sWord += c;
					i ++;
				}
				else
					iState = STATE_DONE;

				break;
			}
		}
	}

	return sWord;
}



/******************************************************************************
 FUNCTION:  ExtractMessageDataFromCurlData
******************************************************************************/
std::string SoNetworkCommsOnline::ExtractMessageDataFromCurlData( const std::string& sData )
{
	const int STATE_READ_WORD = 10;
	const int STATE_READ_DATA = 20;

	std::string sResult = "";

	int iState = STATE_READ_WORD;
	int iSize = sData.size();

	int i = 0;
	while ( i < iSize )
	{
		switch ( iState )
		{
			case STATE_READ_WORD:
			{
				if ( sData[ i ] == ':' )
					iState = STATE_READ_DATA;

				i ++;

				break;
			}

			case STATE_READ_DATA:
			{
				sResult += sData[ i ];
				i ++;

				break;
			}
		}
	}


	return sResult;
}



/******************************************************************************
 FUNCTION:  ParseChatTextFromCurlData
******************************************************************************/
bool SoNetworkCommsOnline::ParseChatTextFromCurlData
( 
	SoNetworkChatText* pChatTextResult, 
	const std::string& sData 
)
{
	bool bIsValidMessage = false;

	const int STATE_READ_WORD = 10;
	const int STATE_INTERPRET_WORD = 20;
	const int STATE_READ_CHAT_TEXT = 30;

	std::string sWord = "";
	std::string sChatText = "";

	int iState = STATE_READ_WORD;
	int iSize = sData.size();

	int i = 0;
	while ( i < iSize )
	{
		switch ( iState )
		{
			case STATE_READ_WORD:
			{
				char c = sData[ i ];

				if ( c != ':' )
				{
					sWord += c;
					i ++;
				}
				else
					iState = STATE_INTERPRET_WORD;


				break;
			}

			case STATE_INTERPRET_WORD:
			{
				if ( sWord == "ChatTextForClient" )
				{
					bIsValidMessage = true;

					i ++;
					iState = STATE_READ_CHAT_TEXT;
				}
				else
				{
					i = iSize; // forces an early exit from the WHILE loop.
				}

				break;
			}

			case STATE_READ_CHAT_TEXT:
			{
				char c = sData[ i ];

				sChatText += c;
				i ++;

				break;
			}
		}
	}

	if ( bIsValidMessage )
		pChatTextResult->SetText( sChatText );


	return bIsValidMessage;
}



/******************************************************************************
 FUNCTION:  ParseMessageFromCurlData
******************************************************************************/
bool SoNetworkCommsOnline::ParseMessageFromCurlData
( 
	SoGameMessage* pMessageResult, 
	const std::string& sData 
)
{
	bool bIsValidMessage = true;

	const int STATE_READ_WORD = 10;
	const int STATE_INTERPRET_WORD = 20;
	const int STATE_SKIP_LEADING_COMMENT = 30;

	std::string sWord = "";
	int iNumWordsInterpreted = 0;

	int iState = STATE_READ_WORD;
	int iSize = sData.size();

	int i = 0;
	while ( i < iSize )
	{
		switch ( iState )
		{
			case STATE_READ_WORD:
			{
				char c = sData[ i ];

				if ( c == '*' )
				{
					iState = STATE_SKIP_LEADING_COMMENT;
					i ++;
				}
				else if ( c != ':' )
				{
					sWord += c;
					i ++;
				}
				else
					iState = STATE_INTERPRET_WORD;


				break;
			}

			case STATE_INTERPRET_WORD:
			{
				if ( iNumWordsInterpreted == 0 )
				{
					if ( sWord == "NextMessageForServer" )
						bIsValidMessage = true;
					else if ( sWord == "NextMessageForClient" )
						bIsValidMessage = true;
					else
						bIsValidMessage = false;
				}
				else if ( iNumWordsInterpreted == 1 )
				{
					pMessageResult->SetServerId( JpString::ParseHash( sWord.c_str() ) );
				}
				else if ( iNumWordsInterpreted == 2 )
				{
					pMessageResult->SetClientId( JpString::ParseHash( sWord.c_str() ) );
				}
				else if ( iNumWordsInterpreted == 3 )
				{
					pMessageResult->SetMessageNumber( JpString::ParseInt( sWord.c_str() ) );
				}
				else if ( iNumWordsInterpreted == 4 )
				{
					pMessageResult->SetType( (Msg::TYPE)JpString::ParseInt( sWord.c_str() ) );
				}
				else
				{
					pMessageResult->AddInt( JpString::ParseInt( sWord.c_str() ) );
				}

				sWord.clear();
				iNumWordsInterpreted ++;

				i ++;
				iState = STATE_READ_WORD;

				break;
			}

			case STATE_SKIP_LEADING_COMMENT:
			{
				char c = sData[ i ];

				if ( c == '*' )
				{
					iState = STATE_READ_WORD;
				}

				i ++;

				break;
			}
		}
	}

	bool bSuccess = ( bIsValidMessage && iNumWordsInterpreted > 0 );
	return bSuccess;
}



/******************************************************************************
 FUNCTION:  GetClient
******************************************************************************/
SoGameClient* SoNetworkCommsOnline::GetClient()
{
	SoGameClientList* pClientList = SoGameGlobals::GetClientList();

	// This needs to be fixed if we ever want to support more than one local client.
	ASSERTF( pClientList->GetNumEntries() <= 1, ( "\nRight now this only works for one local client." ) );

	SoGameClientList::Node* pFirstNode = pClientList->GetFirst();
	SoGameClient* pClient = pFirstNode->GetData();

	return pClient;
}
