/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActor.h"
#include "SoActorManager.h"
#include "SoActorVisible.h"
#include "SoCamera.h"
#include "SoHudLog.h"
#include "SoParticleManager.h"

#include "mmgrChooser.h"



SoParticleManager* SoParticleManager::_pInstance = NULL;



/******************************************************************************
 FUNCTION:  addParticle
 NOTE: This is an override, and is the same as the Cocos version
       with the 1 addition of setting the Particle->cameraStartPos.
******************************************************************************/
bool SoCCParticleSystemQuad::addParticle()
{
	if ( this->isFull() )
	{
		return false;
	}

	tCCParticle* Particle = &m_pParticles[ m_uParticleCount ];
	this->initParticle( Particle );
	Particle->cameraStartPos = 
		SoUtils::ToCcp( SoCameraManager::GetInstance()->GetCurrentCamera()->GetPosition() );
	++m_uParticleCount;

	return true;
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoCCParticleSystemQuad * SoCCParticleSystemQuad::create(const char *plistFile)
{
	SoCCParticleSystemQuad *pRet = new SoCCParticleSystemQuad;

	if ( pRet && pRet->initWithFile( plistFile ) )
	{
		pRet->autorelease();
		pRet->retain();
		return pRet;
	}
	CC_SAFE_DELETE(pRet);
	return pRet;
}



/******************************************************************************
 FUNCTION:  updateQuads
******************************************************************************/
void SoCCParticleSystemQuad::updateQuads( float dt )
{
	if ( _pOwner && _pOwner->GetAnimateTexture() )
	{
		int iWidth = _pOwner->GetFrameWidth();

		_pOwner->SetTextureIndex( ( _pOwner->GetTextureIndex() + 1 ) % _pOwner->GetNumFrames() );
		CCRect Rect = CCRectMake( _pOwner->GetTextureIndex() * iWidth, 0, iWidth, _pOwner->GetFrameHeight() );
		_pOwner->GetParticleSystem()->setTextureWithRect( _pOwner->GetParticleSystem()->getTexture(), Rect );
	}
}



/******************************************************************************
 FUNCTION:  updateQuadWithParticle
******************************************************************************/
void SoCCParticleSystemQuad::updateQuadWithParticle( tCCParticle* particle, const CCPoint& newPosition )
{
	CCPoint ccpNewPos = newPosition;

	if ( _pOwner && _pOwner->GetParticleSystem()->isVisible() )
	{
		if ( _pOwner->GetBillboardType() == SoParticleSystem::BILLBOARD_TYPE_ORIENTED_SELF )
		{
 			JpVector2f vDirection( particle->pos.x, -particle->pos.y ); // Flip Y for Cocos!

			particle->rotation = ccpToAngle( ccp( vDirection.X, vDirection.Y ) ) * RAD2DEG;
		}
		if ( _pOwner->GetAttachToActor() )
		{
			SoCamera* pCamera = SoCameraManager::GetInstance()->GetCurrentCamera();

			JpVector2f vDiff = TransformWorldToScreenSpace( pCamera->GetPosition() ) -
				TransformWorldToScreenSpace( SoUtils::ToJpVector2f( particle->cameraStartPos ) );
				
			ccpNewPos -= SoUtils::ToCcp( vDiff );
		}

		CCParticleSystemQuad::updateQuadWithParticle( particle, ccpNewPos );
	}
}



/******************************************************************************
 FUNCTION:  TransformWorldToScreenSpace
******************************************************************************/
JpVector2f SoCCParticleSystemQuad::TransformWorldToScreenSpace( const JpVector2f& vWorldSpacePos )
{
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->GetCurrentCamera();
	
	if ( pCameraManager && pCamera )
	{
		CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

		JpVector2f vCameraSpacePos = vWorldSpacePos - pCamera->GetPosition();
		vCameraSpacePos.X *= (SoUtils::GetSpriteZoom()) * SoUtils::GetiPadRatioX();
		vCameraSpacePos.Y *= (SoUtils::GetSpriteZoom()) * SoUtils::GetiPadRatioY();

		JpVector2f vScreenOffset( ( SizeInPixels.width / 2.0f ), ( SizeInPixels.height / 2.0f ) );

		JpVector2f vFinal = ( vCameraSpacePos + vScreenOffset );
		return vFinal;
	}

	return JpVector2f( 0.0f, 0.0f );
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoCCParticleSystemQuad::update( float dt )
{
	CCParticleSystemQuad::update( dt );
}



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



/******************************************************************************
 FUNCTION:  SoParticleSystem
******************************************************************************/
SoParticleSystem::SoParticleSystem()
{
	Init();
}



/******************************************************************************
 FUNCTION:  ~SoParticleSystem
******************************************************************************/
SoParticleSystem::~SoParticleSystem()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoParticleSystem::Init()
{
	_pSoCCParticleSystem = NULL;
	_pOwner = NULL;
	_eType          = TYPE_LAST;
	_eBillboardType = BILLBOARD_TYPE_LAST; 
	_eEventType     = SoParticleEvent::TYPE_LAST;
	_vWorldPosition = JpVector2f( 0.0f, 0.0f );
	_vWorldPositionStart = JpVector2f( 0.0f, 0.0f );
	_vOffset = JpVector2f( 0.0f, 0.0f );
	_vDirection = JpVector2f( 0.0f, 0.0f );

	_iZorder = BASE_RENDER_ORDER_HUD;

	_bHasBegun = false;
	_bAttachToActor = false;
	_bFlaggedForDelete = false;

	_fTimeToLive = 5.0f;
	_fTimeElapsedIncludeDelay = 0.0f;
	_fTimeElapsedExcludeDelay = 0.0f;
	_fDelay = 0.0f;

	_bAnimateTexture = false;
	_fTimeBetweenFrames = 0.0f;
	_iNumFrames = 0;
	_iTextureIndex = 0;
	_iFrameWidth = 32;
	_iFrameHeight = 32;
}



/******************************************************************************
 FUNCTION:  BeginParticleEffect
******************************************************************************/
void SoParticleSystem::BeginParticleEffect()
{
	LoadParticleFromFile( _sParticleName );

	JpVector2f vScreenPos( 0.0f, 0.0f );

	switch ( _eType )
	{
		case TYPE_ENVIRONMENTAL:
		{
			CCSize vSize = CCDirector::sharedDirector()->getWinSize();
			vScreenPos.X = vSize.width / 2.0f;
			vScreenPos.Y = vSize.height / 2.0f;

			break;
		}
		case TYPE_BASIC:
		case TYPE_EVENT:
		{
			_pSoCCParticleSystem->setScale( SoUtils::GetSpriteZoom() );
			
			float fRadians = ccpToAngle( ccp( _vDirection.X, _vDirection.Y ) );
			_pSoCCParticleSystem->setAngle( fRadians * RAD2DEG );

			JpVector2f vOffset = _vOffset;
			vOffset.Rotate( fRadians );
			vScreenPos = WorldPosToScreenPos( _vWorldPosition + vOffset );	
			_vWorldPositionStart = _vWorldPosition;
			break;
		}
	}


	_pSoCCParticleSystem->setPositionType( kCCPositionTypeFree );
	_pSoCCParticleSystem->setPosition( vScreenPos.X, vScreenPos.Y );
	_pSoCCParticleSystem->setVisible( true );


	if ( _bAnimateTexture )
	{
		int iTextureWidth = _pSoCCParticleSystem->getTexture()->getPixelsWide();
		_iNumFrames =  iTextureWidth / _iFrameWidth;
	}


	_pSoCCParticleSystem->schedule( schedule_selector( SoCCParticleSystemQuad::updateQuads ), _fTimeBetweenFrames );


	CCNode* pNode = SoParticleManager::GetInstance()->GetParentNode();
	pNode->addChild( _pSoCCParticleSystem );

	SoUtils::SetRenderZOrder( pNode, _pSoCCParticleSystem, _iZorder );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoParticleSystem::Update( float dt )
{
	_fTimeElapsedIncludeDelay += dt;

	if ( _fTimeElapsedIncludeDelay >= _fDelay )
	{
		_fTimeElapsedExcludeDelay += dt;

		if ( !_bHasBegun )
		{
			BeginParticleEffect();
			_bHasBegun = true;
		}
		else
		{
			switch ( _eType )
			{
				case TYPE_BASIC:
					UpdateVisibility( dt );
					UpdateParticleBasic( dt );
					break;
				case TYPE_EVENT:
					UpdateVisibility( dt );
					UpdateParticleEvents( dt );
					break;
				case TYPE_ENVIRONMENTAL:
					UpdateEnvironmental( dt );
					break;
				default:
					ASSERTF( 0, ("ERROR") );
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  UpdateEnvironmental
******************************************************************************/
void SoParticleSystem::UpdateEnvironmental( float dt )
{
	// Nothing ATM
}



/******************************************************************************
 FUNCTION:  UpdateParticleEvents
******************************************************************************/
void SoParticleSystem::UpdateParticleEvents( float dt )
{
	if ( _pSoCCParticleSystem && _pSoCCParticleSystem->isVisible() )
	{
		float fRadians = ccpToAngle( ccp( _vDirection.X, _vDirection.Y ) );
		JpVector2f vOffset = _vOffset;
		vOffset.Rotate( fRadians );

		bool bFlagForDelete = false;

		if ( _bAttachToActor )
		{
			if ( _pOwner && SoActorManager::GetInstance()->Exists( _gOwner ) )
			{
				//_pOwnerTile = _pOwner->GetCurrentTile();
			}
			else
			{
				bFlagForDelete = true;
			}
			if ( bFlagForDelete )
				SetFlaggedForDelete( true );
			else
			{
				_vWorldPositionStart = _vWorldPosition;

				SoActorVisible* pActorVisible = _pOwner->GetAsVisible();
				if ( pActorVisible )
					_vWorldPosition = pActorVisible->GetWorldPosition();
			}
		}

		JpVector2f vScreenPos = WorldPosToScreenPos( _vWorldPosition + vOffset  );	

		_pSoCCParticleSystem->setScale( SoUtils::GetSpriteZoom() );
		_pSoCCParticleSystem->setPosition( vScreenPos.X, vScreenPos.Y );
	}
}



/******************************************************************************
 FUNCTION:  UpdateParticleBasic
******************************************************************************/
void SoParticleSystem::UpdateParticleBasic( float dt )
{
	UpdateParticleEvents( dt );
}



/******************************************************************************
 FUNCTION:  UpdateVisibility
******************************************************************************/
void SoParticleSystem::UpdateVisibility( float dt )
{
	if ( _pSoCCParticleSystem )
	{
	}
}



/******************************************************************************
 FUNCTION:  OnDeath
*****************************************************************************/
void SoParticleSystem::OnDeath()
{
	if ( _pSoCCParticleSystem )
	{
		CCNode* pParentNode = SoParticleManager::GetInstance()->GetParentNode();
		pParentNode->removeChild( _pSoCCParticleSystem, DCG_CLEANUP_ALL_RUNNING_ACTIONS );

		_pSoCCParticleSystem->release();

		_pSoCCParticleSystem = NULL;
	}
}



/******************************************************************************
 FUNCTION:  WorldPosToScreenPos
******************************************************************************/
JpVector2f SoParticleSystem::WorldPosToScreenPos( JpVector2f vWorldPos )
{
	// Adjust to screen space
	JpVector2f vScreenPos = SoCameraManager::GetInstance()->GetCurrentCamera()->TransformWorldToScreenSpace( vWorldPos );
	vScreenPos.Y += ( 64.0f * SoUtils::GetSpriteZoom() * SoUtils::GetiPadRatioY() );

	return vScreenPos;
}



/******************************************************************************
 FUNCTION:  LoadInitialSettings
******************************************************************************/
void SoParticleSystem::LoadInitialSettings( char* pParticleName, SoParticleSystem::TYPE eType )
{
	SetParticleName( pParticleName );
	SetType( eType );
}



/******************************************************************************
 FUNCTION:  LoadParticleFromFile
******************************************************************************/
void SoParticleSystem::LoadParticleFromFile( JpString sParticleName )
{
	std::string sFileName = sParticleName.GetCString();
	sFileName.append( ".plist" );

	SoCCParticleSystemQuad* pccPs = SoCCParticleSystemQuad::create( SoUtils::AddRelativePathToParticles( sFileName.c_str() ) );
	pccPs->SetOwner( this );
	
	SetParticleSystem( pccPs );
}



/******************************************************************************
 FUNCTION:  LoadCsvCustomSettings
******************************************************************************/
void SoParticleSystem::LoadCustomCsvSettings( char* pParticleName )
{
	if ( pParticleName )
	{
		NeCsvIn* pCsvIn = SoParticleManager::GetInstance()->GetParticleSettingsCsv();
		int iNumRows = pCsvIn->GetNumRows();

		for ( int i = 1; i < iNumRows; i ++ )
		{
			char* pacPsName    = pCsvIn->GetStringByIntAndHash( i, 0x8a2f32f6 /* "sParticleSystemName" */ );
			float fDelay       = pCsvIn->GetFloatByIntAndHash(  i, 0x1dd0f73c /* "fDelay" */, 0.0f );
			float fTtl         = pCsvIn->GetFloatByIntAndHash(  i, 0x1a95aaab /* "fTimeToLive" */, 5.0f );
			float fXoffset     = pCsvIn->GetFloatByIntAndHash(  i, 0x4f1c9fe0 /* "fXoffset" */, 0.0f );
			float fYoffset     = pCsvIn->GetFloatByIntAndHash(  i, 0xe8342243 /* "fYoffset" */, 0.0f );
			char* pacBbType    = pCsvIn->GetStringByIntAndHash( i, 0xa9aa4a29 /* "sBillboardType" */ );
			bool  bAnimate     = pCsvIn->GetBoolByIntAndHash(  i, 0xc04fac2b /* "bAnimateTexture" */, false );
			float fTimeBetweenFrames = pCsvIn->GetFloatByIntAndHash(  i, 0xfd62eecc /* "fSecondsBetweenFrames" */, 0.0f );
			int   iFrameWidth  = pCsvIn->GetIntByIntAndHash( i, 0x3709ba4b /* "iFrameWidth" */, 32 );
			int   iFrameHeight = pCsvIn->GetIntByIntAndHash( i, 0x48755f26 /* "iFrameHeight" */, 32 );
			bool  bAttach      = pCsvIn->GetBoolByIntAndHash(  i, 0xd672dec2 /* "bAttachToActor" */, false );


			JpString s = GetParticleName();
			if ( GetParticleName().MatchesCString( pacPsName ) )
			{
				// Append the beginning part and uppercase it
				if ( pacBbType )
				{
					std::string sBbType = "BILLBOARD_TYPE_";
					sBbType.append( pacBbType );
					pacBbType = (char*)sBbType.c_str();
					SoUtils::StringToUpper( pacBbType );

					SetBillboardType( GetBillboardTypeFromString( pacBbType ) );
				}

				// Do the actual set
				SetOffset( JpVector2f( fXoffset, fYoffset ) );
				SetDelay( fDelay );
				SetTimeToLive( fTtl );

				SetAnimateTexture( bAnimate );
				SetTimeBetweenFrames( fTimeBetweenFrames );
				SetFrameWidth( iFrameWidth );
				SetFrameHeight( iFrameHeight );
				SetAttachToActor( bAttach );
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  GetTypeFromString
******************************************************************************/
SoParticleEvent::EVENT_TYPE SoParticleEvent::GetTypeFromString( const char* pacString )
{
	unsigned int uiStringHash = JpHash( pacString );

	switch ( uiStringHash )
	{
		case 0xbafa8e90 /* "TYPE_ONHIT" */: return TYPE_ONHIT;
		case 0x251ee673 /* "TYPE_ONDEATH" */: return TYPE_ONDEATH;
		case 0x26f57227 /* "TYPE_ONATTACK" */: return TYPE_ONATTACK;
		case 0xb7cd476c /* "TYPE_ONACTION1" */: return TYPE_ONACTION1;
		case 0x45f8810e /* "TYPE_ONCREATION" */: return TYPE_ONCREATION;
		default: ASSERTF( 0, ( "\nString could not be converted to Actor Type." ) );
	}

	return TYPE_LAST;
}



/******************************************************************************
 FUNCTION:  GetStringFromType
******************************************************************************/
const char* SoParticleEvent::GetStringFromType( SoParticleEvent::EVENT_TYPE eType )
{
	switch ( eType )
	{
		case TYPE_ONHIT: return "TYPE_ONHIT";
		case TYPE_ONDEATH: return "TYPE_ONDEATH";
		case TYPE_ONATTACK: return "TYPE_ONATTACK";
		case TYPE_ONACTION1: return "TYPE_ONACTION1";
		case TYPE_ONCREATION: return "TYPE_ONCREATION";
		default: ASSERTF( 0, ( "\nActor Type not found." ) );
	}

	return NULL;
}


/******************************************************************************
 FUNCTION:  GetBillboardTypeFromString
******************************************************************************/
SoParticleSystem::BILLBOARD_TYPE SoParticleSystem::GetBillboardTypeFromString( const char* pacString )
{
	unsigned int uiStringHash = JpHash( pacString );

	switch ( uiStringHash )
	{
		case 0x3794690a /* "BILLBOARD_TYPE_PERPENDICULAR_COMMON" */: return SoParticleSystem::BILLBOARD_TYPE_PERPENDICULAR_COMMON;
		case 0x8d275295 /* "BILLBOARD_TYPE_PERPENDICULAR_SELF" */: return SoParticleSystem::BILLBOARD_TYPE_PERPENDICULAR_SELF;
		case 0x2436b512 /* "BILLBOARD_TYPE_ORIENTED_COMMON" */: return SoParticleSystem::BILLBOARD_TYPE_ORIENTED_COMMON;
		case 0xf143bd1d /* "BILLBOARD_TYPE_ORIENTED_SELF" */: return SoParticleSystem::BILLBOARD_TYPE_ORIENTED_SELF;
		case 0x17481bb4 /* "BILLBOARD_TYPE_POINT" */: return SoParticleSystem::BILLBOARD_TYPE_POINT;
		case 0x01bdd936 /* "BILLBOARD_TYPE_LAST" */: return SoParticleSystem::BILLBOARD_TYPE_LAST;

		default: ASSERTF( 0, ( "\nString could not be converted." ) );
	}

	return SoParticleSystem::BILLBOARD_TYPE_LAST;
}



/******************************************************************************
 FUNCTION:  GetBillboardStringFromType
******************************************************************************/
const char* SoParticleSystem::GetBillboardStringFromType( SoParticleSystem::BILLBOARD_TYPE eType )
{
	switch ( eType )
	{
		case SoParticleSystem::BILLBOARD_TYPE_PERPENDICULAR_COMMON: return "BILLBOARD_TYPE_PERPENDICULAR_COMMON";
		case SoParticleSystem::BILLBOARD_TYPE_PERPENDICULAR_SELF: return "BILLBOARD_TYPE_PERPENDICULAR_SELF";
		case SoParticleSystem::BILLBOARD_TYPE_ORIENTED_COMMON: return "BILLBOARD_TYPE_ORIENTED_COMMON";
		case SoParticleSystem::BILLBOARD_TYPE_ORIENTED_SELF: return "BILLBOARD_TYPE_ORIENTED_SELF";
		case SoParticleSystem::BILLBOARD_TYPE_POINT: return "BILLBOARD_TYPE_POINT";
		case SoParticleSystem::BILLBOARD_TYPE_LAST: return "BILLBOARD_TYPE_LAST";
		default: ASSERTF( 0, ( "\nType not found." ) );
	}

	return NULL;
}



///////////////////////////////////////////////////////////////////////////////////////////////////////



/******************************************************************************
 FUNCTION:  Constructor
******************************************************************************/
SoParticleManager::SoParticleManager()
{
	_pInstance = NULL;
	_pParentNode = NULL;
	_bIsInited = false;
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoParticleManager::CreateInstance()
{
	ASSERTF( !_pInstance, ( "\nCan't create a second instance." ) )
	if ( !_pInstance )
	{
		_pInstance = new SoParticleManager;
		_pInstance->LoadCsvFiles();
	}
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoParticleManager::DeleteInstance()
{
	if ( _pInstance )
	{
		_pInstance->DeleteAllParticleSystems();

		delete _pInstance;
		_pInstance = NULL;
	}
}



/******************************************************************************
 FUNCTION:  CreateParticleSystemEvent
******************************************************************************/
SoParticleSystem* SoParticleManager::CreateParticleSystem( char* sParticleName, JpVector2f vPosition, JpVector2f vDirection )
{
	if ( sParticleName )
	{
		SoParticleSystem* pPs = new SoParticleSystem;
		
		pPs->LoadInitialSettings( sParticleName, SoParticleSystem::TYPE_BASIC );
		pPs->LoadCustomCsvSettings( sParticleName );

		pPs->SetOwner( NULL );
		pPs->SetOwnerGuid( 0 );
		pPs->SetWorldPosition( vPosition );
		pPs->SetDirection( vDirection );

		pPs->SetZorder( BASE_RENDER_ORDER_PARTICLES );

		_ParticleSystemBasicList.AddLast( pPs );

		return pPs;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  CreateParticleSystemEvent
******************************************************************************/
SoParticleSystem* SoParticleManager::CreateParticleSystemEvent( char* sParticleName, SoActor* pActor, JpVector2f vDirection )
{
	SoParticleSystem* pPs = new SoParticleSystem;
	
	pPs->LoadInitialSettings( sParticleName, SoParticleSystem::TYPE_EVENT );
	pPs->LoadCustomCsvSettings( sParticleName );

	pPs->SetOwner( pActor );
	pPs->SetOwnerGuid( pActor->GetGuid() );

	SoActorVisible* pActorVisible = pActor->GetAsVisible();
	if ( pActorVisible )
	{
		pPs->SetWorldPosition( pActorVisible->GetWorldPosition() );
		pPs->SetDirection( vDirection );

		if ( pActorVisible->GetSprite() )
			pPs->SetZorder( pActorVisible->GetSprite()->getZOrder() );
	}

	return pPs;
}



/******************************************************************************
 FUNCTION:  CreateParticleSystemEnvironment
******************************************************************************/
SoParticleSystem* SoParticleManager::CreateParticleSystemEnvironment( char* sParticleName )
{
	SoParticleSystem* pPs = new SoParticleSystem;

	pPs->LoadInitialSettings( sParticleName, SoParticleSystem::TYPE_ENVIRONMENTAL );
	pPs->LoadCustomCsvSettings( sParticleName );

	return pPs;
}



/******************************************************************************
 FUNCTION:  LoadCsvFiles
******************************************************************************/
void SoParticleManager::LoadCsvFiles()
{
	_ParticleEventsCsv.ReadCsvFile( SoUtils::AddRelativePathToCsv( "ParticleEvents.csv" ) );
	_ParticleEventsCsv.MakeHashIndicesFromColValuesAtRow( 0 );

	_ParticleEnvironmentalCsv.ReadCsvFile( SoUtils::AddRelativePathToCsv( "ParticleEnvironmental.csv" ) );
	_ParticleEnvironmentalCsv.MakeHashIndicesFromColValuesAtRow( 0 );

	_ParticleSettingsCsv.ReadCsvFile( SoUtils::AddRelativePathToCsv( "ParticleSettings.csv" ) );
	_ParticleSettingsCsv.MakeHashIndicesFromColValuesAtRow( 0 );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoParticleManager::Update( float dt )
{
	if ( _pParentNode && !_bIsInited )
	{
		_bIsInited = true;
	}

	SoParticleSystemList* pList = _pInstance->GetParticleEventList();
	SoParticleSystemList::Iterator Iter( pList );
	while ( Iter.Next() )
	{
		SoParticleSystem* pPs = Iter.GetNodeData();

		float fTimeToLive = pPs->GetTimeToLive();

		if ( !pPs->GetFlaggedForDelete() && ( fTimeToLive < 0.0f ) || !pPs->GetHasEnded() )
			pPs->Update( dt );
		else 
			DeleteAndRemoveParticleSystem( pPs );
	}

	//-----------------------------------------------------------

	SoParticleSystemList* pList2 = _pInstance->GetParticleEnvironmentalList();
	SoParticleSystemList::Iterator Iter2( pList2 );
	while ( Iter2.Next() )
	{
		SoParticleSystem* pPs = Iter2.GetNodeData();

		pPs->Update( dt );
	}

	//-----------------------------------------------------------

	SoParticleSystemList* pList3 = _pInstance->GetParticleBasicList();
	SoParticleSystemList::Iterator Iter3( pList3 );
	while ( Iter3.Next() )
	{
		SoParticleSystem* pPs = Iter3.GetNodeData();

		float fTimeToLive = pPs->GetTimeToLive();

		if ( ( fTimeToLive < 0.0f ) || !pPs->GetHasEnded() )
			pPs->Update( dt );
		else 
			DeleteAndRemoveParticleSystem( pPs );
	}
}



/******************************************************************************
 FUNCTION:  DeleteAllParticleSystems
******************************************************************************/
void SoParticleManager::DeleteAllParticleSystems()
{
	SoParticleSystemList* pList = _pInstance->GetParticleEnvironmentalList();
	SoParticleSystemList::Iterator Iter( pList );
	while ( Iter.Next() )
	{
		SoParticleSystem* pPs = Iter.GetNodeData();

		pPs->OnDeath();
		delete pPs;
	}

	pList->RemoveAll();

	//-------------------------------------------------------

	pList = _pInstance->GetParticleEventList();
	SoParticleSystemList::Iterator Iter2( pList );
	while ( Iter2.Next() )
	{
		SoParticleSystem* pPs = Iter2.GetNodeData();

		pPs->OnDeath();
		delete pPs;
	}

	pList->RemoveAll();

	//-------------------------------------------------------

	pList = _pInstance->GetParticleBasicList();
	SoParticleSystemList::Iterator Iter3( pList );
	while ( Iter3.Next() )
	{
		SoParticleSystem* pPs = Iter3.GetNodeData();

		pPs->OnDeath();
		delete pPs;
	}

	pList->RemoveAll();
}



/******************************************************************************
 FUNCTION:  DeleteAndRemoveParticleSystem
******************************************************************************/
void SoParticleManager::DeleteAndRemoveParticleSystem( SoParticleSystem* pPs )
{
	if ( pPs )
	{
		SoParticleSystem::TYPE eType = pPs->GetType();

 		pPs->OnDeath();

		switch ( eType )
		{
			case SoParticleSystem::TYPE_BASIC: 
				_ParticleSystemBasicList.Remove( pPs );
				break;
			case SoParticleSystem::TYPE_EVENT: 
				_ParticleSystemEventList.Remove( pPs );
				break;
			case SoParticleSystem::TYPE_ENVIRONMENTAL: 
				_ParticleSystemEnvironmentList.Remove( pPs );
				break;			
		}

		delete pPs;
	}
}



/******************************************************************************
 FUNCTION:  LoadCsvEnvrionmentalParticle
******************************************************************************/
void SoParticleManager::LoadCsvEnvrionmentalParticle( /* SoGameLevel* pGameLevel */ )
{
	#if 0
		int iNumRows = _ParticleEnvironmentalCsv.GetNumRows();

		for ( int i = 1; i < iNumRows; i ++ )
		{
			char* pacLevel        = _ParticleEnvironmentalCsv.GetStringByIntAndHash( i, 0xe716c7f8 /* "sLevelType" */ );
			char* pacParticleName = _ParticleEnvironmentalCsv.GetStringByIntAndHash( i, 0x8a2f32f6 /* "sParticleSystemName" */ );

			SoUtils::StringToUpper( pacLevel );

			if ( SoGameLevel::GetTypeFromString( pacLevel ) == pGameLevel->GetType() )
			{
				SoParticleSystem* pPs = _pInstance->CreateParticleSystemEnvironment
				( 
					pacParticleName
				);

				GetParticleEnvironmentalList()->AddLast( pPs );
			}
		}
	#endif
}



/******************************************************************************
 FUNCTION:  LoadCsvEventsParticle
******************************************************************************/
void SoParticleManager::LoadCsvEventParticle( SoParticleEvent::EVENT_TYPE eType, SoActor* pActor, JpVector2f vOffset )
{
	int iNumRows = _ParticleEventsCsv.GetNumRows();

	for ( int i = 1; i < iNumRows; i ++ )
	{
		char* pacActorType = _ParticleEventsCsv.GetStringByIntAndHash( i, 0xfbb76ecc /* "SoActorType" */ );
		char* pacEvent     = _ParticleEventsCsv.GetStringByIntAndHash( i, 0xc8489d3e /* "sEvent" */ );
		char* pacPsName    = _ParticleEventsCsv.GetStringByIntAndHash( i, 0x8a2f32f6 /* "sParticleSystemName" */ );

		std::string sEvent = "TYPE_";
		sEvent.append( pacEvent );
		pacEvent = (char*)sEvent.c_str();

		SoUtils::StringToUpper( pacActorType );
		SoUtils::StringToUpper( pacEvent );

		if ( JpHash( pacActorType ) == pActor->GetCsvRowIndex() )
		{
			if ( SoParticleEvent::GetTypeFromString( pacEvent ) == eType )
			{
				SoParticleSystem* pPs = _pInstance->CreateParticleSystemEvent
				( 
					pacPsName, 
					pActor, 
					vOffset
				);
				pPs->SetEventType( SoParticleEvent::GetTypeFromString( pacEvent ) );

				GetParticleEventList()->AddLast( pPs );
			}
		}
	}
}





















