/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#pragma once


#include "stdafx.h"

#include "NeCsvIn.h"

#include "JpList.h"
#include "JpString.h"



class SoActor;
class SoParticleSystem;



/******************************************************************************
*********************************************  CLASS:  CCParticleSystemQuad
******************************************************************************/
class SoCCParticleSystemQuad : public CCParticleSystemQuad
{
	public:
		SoCCParticleSystemQuad() { _pOwner = NULL; }
		virtual ~SoCCParticleSystemQuad() {}

		static SoCCParticleSystemQuad * create(const char *plistFile);
		virtual void updateQuadWithParticle( tCCParticle* particle, const CCPoint& newPosition );
		virtual void updateQuads( float dt );
		virtual void update( float dt );
		virtual bool addParticle();

		SoParticleSystem* GetOwner() { return _pOwner; }
		void SetOwner( SoParticleSystem* p ) { _pOwner = p; }
		JpVector2f TransformWorldToScreenSpace( const JpVector2f& vWorldSpacePos );
		SoParticleSystem* _pOwner;
		JpVector2f _vStartPos;
};










/******************************************************************************
*********************************************  CLASS:  SoParticleEvent
******************************************************************************/
class SoParticleEvent
{
	public:

		enum EVENT_TYPE
		{
			TYPE_ONHIT,
			TYPE_ONDEATH,
			TYPE_ONATTACK,
			TYPE_ONACTION1,
			TYPE_ONCREATION,
			TYPE_LAST
		};

		static EVENT_TYPE GetTypeFromString( const char* pacString );
		static const char* GetStringFromType( EVENT_TYPE eType );
};










/******************************************************************************
*********************************************  CLASS:  SoParticleSystemInstance
******************************************************************************/
class SoParticleSystem
{
	public:

		enum TYPE
		{
			TYPE_BASIC,
			TYPE_EVENT,
			TYPE_ENVIRONMENTAL,
			TYPE_LAST
		};

		enum BILLBOARD_TYPE
		{
			BILLBOARD_TYPE_PERPENDICULAR_COMMON,
			BILLBOARD_TYPE_PERPENDICULAR_SELF,
			BILLBOARD_TYPE_ORIENTED_COMMON,
			BILLBOARD_TYPE_ORIENTED_SELF,
			BILLBOARD_TYPE_POINT,
			BILLBOARD_TYPE_LAST
		};

		enum TRIGGER_TYPE
		{
			TRIGGER_TYPE_ACTION1,
			TRIGGER_TYPE_ONDEATH,
			TRIGGER_TYPE_ONHIT,
			TRIGGER_TYPE_ONATTACK
		};


		//-----------------------
		// public functions
		SoParticleSystem();
		virtual ~SoParticleSystem();

		void Init();
		void Deinit() {}

		void Update( float dt );
		void UpdateParticleBasic( float dt );
		void UpdateEnvironmental( float dt );
		void UpdateParticleEvents( float dt );
		void UpdateVisibility( float dt );
		void OnDeath();

		void LoadInitialSettings( char* pParticleName, SoParticleSystem::TYPE eType );
		void LoadCustomCsvSettings( char* pParticleName );

		SoCCParticleSystemQuad* GetParticleSystem() { return _pSoCCParticleSystem; }
		void SetParticleSystem( SoCCParticleSystemQuad* pParticleSystem ) { _pSoCCParticleSystem = pParticleSystem; }

		void SetParticleName( char* cp ) { _sParticleName = cp; }
		JpString GetParticleName() { return _sParticleName; }

		BILLBOARD_TYPE GetBillboardType() { return _eBillboardType; }
		void SetBillboardType( BILLBOARD_TYPE eType ) { _eBillboardType = eType; }

		TYPE GetType() { return _eType; }
		void SetType( TYPE eType ) { _eType = eType; }

		void     SetOwner( SoActor* pOwner ) { _pOwner = pOwner; }
		SoActor* GetOwner() { return _pOwner; }

		void SetOwnerGuid( guid g ) { _gOwner = g; }
		guid GetOwnerGuid() { return _gOwner; }

		void LoadParticleFromFile( JpString sParticleName );
		void AddToList( JpList< SoParticleSystem >* pList );
		
		void BeginParticleEffect();
		bool GetHasBegun() { return _bHasBegun; }
		bool GetHasEnded() { return ( _fTimeElapsedExcludeDelay > _fTimeToLive ); }

		JpVector2f WorldPosToScreenPos( JpVector2f vWorldPos );
		JpVector2f GetWorldPosition() { return _vWorldPosition; }
		void       SetWorldPosition( JpVector2f v ) { _vWorldPosition = v; }
		JpVector2f GetWorldPositionStart() { return _vWorldPositionStart; }
		void       SetWorldPositionStart( JpVector2f v ) { _vWorldPositionStart = v; }

		JpVector2f GetOffset() { return _vOffset; }
		void       SetOffset( JpVector2f v ) { _vOffset = v; }

		JpVector2f GetDirection() { return _vDirection; }
		void       SetDirection( JpVector2f v ) { _vDirection = v; }

		BILLBOARD_TYPE GetBillboardTypeFromString( const char* pacString );
		const char* GetBillboardStringFromType( BILLBOARD_TYPE eType );
	
		int GetZorder() { return _iZorder; }
		void SetZorder(int i) { _iZorder = i; }

	
		// ----------------------
		// Animated Texture
		int GetTextureIndex() { return _iTextureIndex; }
		void SetTextureIndex(int val) { _iTextureIndex = val; }

		bool GetAnimateTexture() { return _bAnimateTexture; }
		void SetAnimateTexture( bool val ) { _bAnimateTexture = val; }

		float GetTimeBetweenFrames() { return _fTimeBetweenFrames; }
		void SetTimeBetweenFrames( float val ) { _fTimeBetweenFrames = val; }

		int GetFrameWidth() { return _iFrameWidth; }
		void SetFrameWidth( int val ) { _iFrameWidth = val; }

		int GetFrameHeight()  { return _iFrameHeight; }
		void SetFrameHeight( int val ) { _iFrameHeight = val; }

		int GetNumFrames() { return _iNumFrames; }
		void SetNumFrames( int val ) { _iNumFrames = val; }


		// ----------------------
		// Time
		float GetTimeElapsedIncludeDelay() { return _fTimeElapsedIncludeDelay; }
		float GetTimeElapsedExcludeDelay() { return _fTimeElapsedExcludeDelay; }
	
		float GetTimeToLive() { return _fTimeToLive; }
		void  SetTimeToLive( float f ) { _fTimeToLive = f; }
	
		float GetDelay() { return _fDelay; }
		void  SetDelay( float f ) { _fDelay = f; }

		void SetEventType( SoParticleEvent::EVENT_TYPE e ) { _eEventType = e; }
		SoParticleEvent::EVENT_TYPE GetEventType() { return _eEventType; }

		bool GetAttachToActor() { return _bAttachToActor; }
		void SetAttachToActor( bool b ) { _bAttachToActor = b; }

		bool GetFlaggedForDelete() { return _bFlaggedForDelete; }
		void SetFlaggedForDelete( bool b ) { _bFlaggedForDelete = b; }


	private: 

		//-----------------------
		// private variables
		SoCCParticleSystemQuad*     _pSoCCParticleSystem;

		JpString   _sParticleName;
		SoActor*   _pOwner;
		guid       _gOwner;

		JpVector2f _vWorldPosition;
		JpVector2f _vWorldPositionStart;
		JpVector2f _vOffset;
		JpVector2f _vDirection;
		bool       _bHasBegun;
		bool       _bAttachToActor;
		SoParticleEvent::EVENT_TYPE _eEventType;
		BILLBOARD_TYPE _eBillboardType;
		TYPE           _eType;

		
		// ----------------------
		// Animated Texture
		bool  _bAnimateTexture;
		bool  _bFlaggedForDelete;
		int   _iFrameWidth;
		int   _iFrameHeight;
		int   _iTextureIndex;
		int   _iNumFrames;
		int   _iZorder;
		float _fTimeBetweenFrames;


		// ----------------------
		// Time
		float _fDelay;
		float _fTimeToLive;
		float _fTimeElapsedIncludeDelay;
		float _fTimeElapsedExcludeDelay;
};





#define SoParticleSystemList JpList< SoParticleSystem >





/******************************************************************************
**************************************************  CLASS:  SoParticleManager
******************************************************************************/
class SoParticleManager
{
	public:

		//-----------------------
		// public functions
		SoParticleManager();
		virtual ~SoParticleManager() {}

		static SoParticleManager* GetInstance() { return _pInstance; }
		static void CreateInstance();
		static void DeleteInstance();
		static void DeleteAllParticleSystems();

		void Update( float dt );
			
		void LoadCsvEnvrionmentalParticle( /* SoGameLevel* pGameLevel */ );
		void LoadCsvEventParticle( SoParticleEvent::EVENT_TYPE eType, SoActor* pActor, JpVector2f vDirection = JpVector2f( 0.0f, 0.0f ) );

		SoParticleSystem* CreateParticleSystemEnvironment( char* sParticleName );
		SoParticleSystem* CreateParticleSystemEvent( char* sParticleName, SoActor* pActor, JpVector2f vDirection = JpVector2f( 0.0f, 0.0f ) );
		SoParticleSystem* CreateParticleSystem( char* sParticleName, JpVector2f vPosition, JpVector2f vDirection = JpVector2f( 0.0f, 0.0f ) );

		SoParticleSystemList* GetParticleBasicList() { return &_ParticleSystemBasicList; }
		SoParticleSystemList* GetParticleEventList() { return &_ParticleSystemEventList; }
		SoParticleSystemList* GetParticleEnvironmentalList() { return &_ParticleSystemEnvironmentList; }
		
		void SetParentNode( CCNode* p ) { _pParentNode = p; }
		CCNode* GetParentNode() { return _pParentNode; }

		void DeleteAndRemoveParticleSystem( SoParticleSystem* pPs );

		void LoadCsvFiles();
		NeCsvIn* GetParticleSettingsCsv() { return &_ParticleSettingsCsv; }


	private:

		//-----------------------
		// private

		static SoParticleManager* _pInstance;

		SoParticleSystem::BILLBOARD_TYPE GetBillboardTypeFromString( const char* pacString );

		CCNode* _pParentNode;

		bool _bIsInited;

		SoParticleSystemList _ParticleSystemBasicList;
		SoParticleSystemList _ParticleSystemEventList;
		SoParticleSystemList _ParticleSystemEnvironmentList;

		NeCsvIn _ParticleEnvironmentalCsv;
		NeCsvIn _ParticleEventsCsv;
		NeCsvIn _ParticleSettingsCsv;
};















