#include "stdafx.h"



#include "JpRandomStrong.h"

#include "NeSoundEffect.h"

#include "SoConfigManager.h"
#include "SoCurl.h"
#include "SoFontDefs.h"
#include "SoHud.h"
#include "SoInputKeyboard.h"
#include "SoInputKeypad.h"
#include "SoInputMouse.h"
#include "SoInputOisManager.h"
#include "SoInputTouch.h"
#include "SoLoadScreen.h"
#include "SoSceneMain.h"
#include "SoSoundManager.h"
#include "SoTextureUtils.h"
#include "SoUtils.h"

#include "mmgrChooser.h"



USING_NS_CC;




/******************************************************************************
 FUNCTION:  SoSceneMain
******************************************************************************/
SoSceneMain::SoSceneMain()
{
}



/******************************************************************************
 FUNCTION:  ~SoSceneMain
******************************************************************************/
SoSceneMain::~SoSceneMain()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  init
******************************************************************************/
bool SoSceneMain::init()
{
	if( CCScene::init() )
	{
		//-----------------------
		SoUtils::DetermineAndCacheDeviceType();
		JpRandom::G.Seed( (unsigned int)time( NULL ) );
		JpRandomStrong::G.Seed( (unsigned int)time( NULL ) );


		//-----------------------
		// On PC the config manager is created earlier, in main.cpp
		// But on iOS builds main.cpp is not used, so we create the config manager here.
		if ( !SoConfigManager::GetInstance() )
		{
			SoConfigManager::CreateInstance();
			SoConfigManager::GetInstance()->Init();
		}


		NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "fTextureScaler" );
		if ( pLine )
			SoTextureUtils::SetTextureScaler( pLine->GetFloat( 1 ) );

		
		//-----------------------
		// Init audio
		if ( !SoSoundManager::GetInstance() )
		{
			SoSoundManager::CreateInstance();
			SoSoundManager::GetInstance()->LoadSoundEffectsByCsv( SoUtils::AddRelativePathToCsv( "SoundEffects.csv" ) );
			/*
			// Not needed for now, but keeping here as reference on how to load a sound effect.
			// mdomowicz 2012_10_15
			char* pacFile = (char*)CCFileUtils::sharedFileUtils()->fullPathFromRelativePath
			( 
				SoUtils::AddRelativePathToSfx( "SfxTestBeep.wav" )
			);
			NeSoundManager::GetInstance()->LoadSoundEffect( pacFile );
			*/
		}


		//-----------------------
		// add all our FSM States
		_FsmStateManager.StoreState( JpHash( "Boot" ), &_FsmStateBoot );
		_FsmStateManager.StoreState( JpHash( "InGame" ), &_FsmStateMain );
		_FsmStateManager.StoreState( JpHash( "Shell" ), &_FsmStateShell );
		_FsmStateManager.StoreState( JpHash( "SetupOfflineGame" ), &_FsmStateSetupOfflineGame );
		_FsmStateManager.StoreState( JpHash( "SetupOnlineGame" ), &_FsmStateSetupOnlineGame );
		_FsmStateManager.StoreState( JpHash( "Tutorial" ), &_FsmStateTutorial );
		_FsmStateManager.StoreState( JpHash( "Credits" ), &_FsmStateCredits );


		//-----------------------
		// turn on or off debug fps rendering
		pLine = SoConfigManager::GetInstance()->FindLine( "bDisplayStats" );
		CCDirector::sharedDirector()->setDisplayStats( pLine->GetBool( 1 ) );

		#if DCG_IS_TARGET_WIN32
			SoInputOisManager::CreateInstance();
			SoInputOisManager::GetInstance()->Init();

			SoInputKeyboard::CreateInstance();
			SoInputKeyboard::GetInstance()->Init();

			SoInputMouse::CreateInstance();
			SoInputMouse::GetInstance()->Init();
		#else
		#endif

		SoInputVirtualKeyboard::CreateInstance();
		SoInputVirtualKeyboard::GetInstance()->Init();

		SoInputTouch::CreateInstance();
		SoInputTouch::GetInstance()->Init( NULL );

		SoInputGesture::CreateInstance();

		SoInputKeypad::CreateInstance();
		SoInputKeypad::GetInstance()->Init( NULL );

		SoLoadScreen::CreateInstance( this );

		SoCurl::CreateInstance();

		scheduleUpdate();

		//-----------------------
		// Pick and enter the starting FSM state
		pLine = SoConfigManager::GetInstance()->FindLine( "SkipToState" );
		std::string sSkipToState = "";
		if ( pLine )
			sSkipToState = pLine->GetString( 1 );

		if ( _FsmStateManager.FindState( JpHash( sSkipToState.c_str() ) ) )
		{
			_FsmStateManager.SetState( JpHash( sSkipToState.c_str() ) );
		}
		else
		{
			#if DCG_IS_TARGET_IOS
				_FsmStateManager.SetState( JpHash( "Boot" ) );
			#else
				_FsmStateManager.SetState( JpHash( "Boot" ) );
			#endif
		}


		return true;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoSceneMain::Deinit()
{
	_FsmStateManager.SetState( 0 );

	SoCurl::DeleteInstance();

	SoFontDefs::DestroyPreloadedFonts();
	SoLoadScreen::DeleteInstance();

	SoInputKeypad::DeleteInstance();
	SoInputGesture::DeleteInstance();
	SoInputTouch::DeleteInstance();

	SoInputVirtualKeyboard::DeleteInstance();	

	#if DCG_IS_TARGET_WIN32
		SoInputMouse::DeleteInstance();
		SoInputKeyboard::DeleteInstance();
		SoInputOisManager::DeleteInstance();
	#endif
		
	SoSoundManager::DeleteInstance();
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoSceneMain::update( float dt )
{
	SoSoundManager::GetInstance()->Update( dt );

	#if DCG_IS_TARGET_WIN32
		SoInputOisManager::GetInstance()->Update( dt );
		SoInputKeyboard::GetInstance()->Update( dt );
	#endif
	
	SoInputVirtualKeyboard::GetInstance()->Update( dt );	

	SoInputTouch::GetInstance()->Update( dt );
	SoInputGesture::GetInstance()->Update( dt );
	SoInputKeypad::GetInstance()->Update( dt );

	SoCurl::GetInstance()->Update( dt );

	hash hResult = _FsmStateManager.Update( dt );

	if ( hResult == 0x4cc43854 /* "GoToResults" */ )
		_FsmStateManager.SetState( JpHash( "Results" ) );
	if ( hResult == 0x5a130bf8 /* "GoToShell" */ )
		_FsmStateManager.SetState( JpHash( "Shell" ) );
}






















