/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoScreenFader.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoScreenFader
******************************************************************************/
SoScreenFader::SoScreenFader() :
_pParentNode( NULL ),
_fTimer( 0.0f ),
_eFadeDirection( FADE_DIRECTION_TRANSPARENT_TO_OPAQUE ),
_fFadeSpeed( 1.0f ),
_fAlpha( 0.0f ),
_eCurrentState( STATE_WAIT )
{
	_Color.r = 0;
	_Color.g = 0;
	_Color.b = 0;
}



/******************************************************************************
 FUNCTION:  ~SoScreenFader
******************************************************************************/
SoScreenFader::~SoScreenFader()
{
}



/******************************************************************************
 FUNCTION:  Setup
******************************************************************************/
void SoScreenFader::Setup( CCNode* pParentNode, int iRenderOrder )
{
	_eCurrentState = STATE_WAIT;

	SetParentNode( pParentNode );
	pParentNode->addChild( this, iRenderOrder );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoScreenFader::Deinit()
{
	if ( _pParentNode )
	{
		_pParentNode->removeChild( this, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pParentNode = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoScreenFader::Update( float dt )
{
	switch ( _eCurrentState )
	{
		case STATE_WAIT:
			break;

		case STATE_FADE:
			_fTimer += dt;

			if ( _eFadeDirection == FADE_DIRECTION_TRANSPARENT_TO_OPAQUE && _fAlpha == 1.0f )
				_eCurrentState = STATE_DONE;

			if ( _eFadeDirection == FADE_DIRECTION_OPAQUE_TO_TRANSPARENT && _fAlpha == 0.0f )
				_eCurrentState = STATE_DONE;

			break;

		case STATE_DONE:
			break;

		default:
			ASSERTF( 0, ( "\nUnknown state." ) );

	}
}



/******************************************************************************
 FUNCTION:  draw
******************************************************************************/
void SoScreenFader::draw()
{
	CCNode::draw();

	CCSize ScreenSize = CCDirector::sharedDirector()->getWinSize();

    CCPoint aVertices[] =
	{
		ccp( 0, 0 ), 
		ccp( 0, ScreenSize.height ), 
		ccp( ScreenSize.width, ScreenSize.height ), 
		ccp( ScreenSize.width, 0 )
	};

	float fTimer = _fFadeSpeed * _fTimer;
	_fAlpha = Clamp( fTimer, 0.0f, 1.0f );

	if ( _eFadeDirection == FADE_DIRECTION_OPAQUE_TO_TRANSPARENT )
		_fAlpha = Clamp( 1.0f - fTimer, 0.0f, 1.0f );	


	ccColor4F Color = ccc4f( _Color.r, _Color.g, _Color.b, _fAlpha );

    ccDrawSolidPoly( aVertices, 4, Color );
}



/******************************************************************************
 FUNCTION:  BeginFade
******************************************************************************/
void SoScreenFader::BeginFade()
{ 
	_fTimer = 0.0f; 
	_eCurrentState = STATE_FADE;

	if ( _eFadeDirection == FADE_DIRECTION_TRANSPARENT_TO_OPAQUE )
		_fAlpha = 0.0f;
	else
		_fAlpha = 1.0f;
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoScreenFader* SoScreenFader::create()
{
	SoScreenFader* pRet = new SoScreenFader();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}
