/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include <stdio.h>


#include "JpUtils.h"
#include "NeConfig.h"
#include "NeCsvIn.h"
#include "NeTgaIo.h"
#include "SoActorManager.h"
#include "SoSaveSystem.h"
#include "SoScript.h"
#include "SoUtils.h"

#include "TgScriptMain.h"

#include "mmgrChooser.h"



SoScriptManager* SoScriptManager::_pInstance = NULL;




/******************************************************************************
 FUNCTION:  SoScript
******************************************************************************/
SoScript::SoScript() :
_hRngSeed( 0 ),
_pOwnerScriptManager( NULL ),
_bIsEnabled( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoScript
******************************************************************************/
SoScript::~SoScript()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoScript::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoScript::Deinit()
{
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoScript::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  SaveToFile
******************************************************************************/
void SoScript::SaveToFile( SoSaveSystem* pSaveSystem )
{
	NeCsvOut* pCsvWriter = pSaveSystem->GetSharedCsvWriter();

	pCsvWriter->AddNewRow();
	pCsvWriter->AppendStringToLastRow( "_iScriptId" );
	pCsvWriter->AppendIntToLastRow( _iScriptId );

	pCsvWriter->AddNewRow();
	pCsvWriter->AppendStringToLastRow( "_sName" );
	pCsvWriter->AppendStringToLastRow( _sName.c_str() );

	pCsvWriter->AddNewRow();
	pCsvWriter->AppendStringToLastRow( "_hRngSeed" );
	pCsvWriter->AppendUnsignedIntToLastRow( _hRngSeed );

	pCsvWriter->AddNewRow();
	pCsvWriter->AppendStringToLastRow( "_bIsEnabled" );
	pCsvWriter->AppendBoolToLastRow( _bIsEnabled );

	pCsvWriter->AddNewRow();
	pCsvWriter->AppendStringToLastRow( "_Rng.high" );
	pCsvWriter->AppendUnsignedIntToLastRow( _Rng.GetHigh() );

	pCsvWriter->AddNewRow();
	pCsvWriter->AppendStringToLastRow( "_Rng.low" );
	pCsvWriter->AppendUnsignedIntToLastRow( _Rng.GetLow() );
}



/******************************************************************************
 FUNCTION:  LoadFromFile
******************************************************************************/
void SoScript::LoadFromFile( SoSaveSystem* pSaveSystem )
{
	NeCsvIn* pCsvReader = pSaveSystem->GetSharedCsvReader();

	_iScriptId = pCsvReader->GetIntByHashAndInt( 0x828ad947 /* "_iScriptId" */, 1, -1 );
	_sName = pCsvReader->GetStringByHashAndInt( 0x8db06484 /* "_sName" */, 1 );
	_hRngSeed = pCsvReader->GetHashByHashAndInt( 0xea77b66a /* "_hRngSeed" */, 1, 0 );
	_bIsEnabled = pCsvReader->GetBoolByHashAndInt( 0xb82858bd /* "_bIsEnabled" */, 1, false );

	unsigned int uiHigh = pCsvReader->GetHashByHashAndInt( 0xd35f9bd3 /* "_Rng.high" */, 1, 0 );
	unsigned int uiLow = pCsvReader->GetHashByHashAndInt( 0x3eea233f /* "_Rng.low" */, 1, 0 );

	_Rng.SetHigh( uiHigh );
	_Rng.SetLow( uiLow );
}



/******************************************************************************
 FUNCTION:  ActorExistsAndIsAlive
******************************************************************************/
bool SoScript::ActorExistsAndIsAlive( guid gActor )
{
	//SoActorManager* pActorManager = SoActorManager::GetInstance();
	//SoActor* pActor = pActorManager->GetActorByGuid( gActor );
	//return ( pActor && pActor->IsAlive() );
	return true;
}



/******************************************************************************
 FUNCTION:  ActorAdd
******************************************************************************/
guid SoScript::ActorAdd( SoActorTypes::TYPE eActorType, int iActorSubTypeCastToInt )
{
	return 0;
}











/******************************************************************************
 FUNCTION:  SoScriptManager
******************************************************************************/
SoScriptManager::SoScriptManager() :
_bIsGameWon( false ),
_iScriptIdAssigner( SCRIPT_ID_BASE_VALUE )
{
}



/******************************************************************************
 FUNCTION:  ~SoScriptManager
******************************************************************************/
SoScriptManager::~SoScriptManager()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoScriptManager::Init()
{
	SetGameWon( false );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoScriptManager::Deinit()
{
	RemoveAndDeleteAllScriptsNow();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoScriptManager::Update( float dt )
{
	SoScriptList::Iterator Iter( &_ScriptList );
	while ( Iter.Next() )
	{
		SoScript* pScript = Iter.GetNodeData();
		pScript->Update( dt );
	}

	DeleteAllScriptsInPendingDeletionList();
}



/******************************************************************************
 FUNCTION:  FindById
******************************************************************************/
SoScript* SoScriptManager::FindById( int iId )
{
	SoScriptList::Iterator Iter( &_ScriptList );
	while ( Iter.Next() )
	{
		SoScript* pScript = Iter.GetNodeData();
		if ( pScript->GetScriptId() == iId )
			return pScript;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  FindByName
******************************************************************************/
SoScript* SoScriptManager::FindByName( const std::string& sName )
{
	SoScriptList::Iterator Iter( &_ScriptList );
	while ( Iter.Next() )
	{
		SoScript* pScript = Iter.GetNodeData();
		if ( pScript->GetName() == sName )
			return pScript;
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  AddScript
******************************************************************************/
void SoScriptManager::AddScript( SoScript* p )
{
	p->SetOwnerScriptManager( this );

	p->SetScriptId( _iScriptIdAssigner );
	_iScriptIdAssigner ++;

	_ScriptList.AddLast( p );
}



/******************************************************************************
 FUNCTION:  RemoveAndDeleteScript
******************************************************************************/
void SoScriptManager::RemoveAndDeleteScript( SoScript* pScript )
{
	//-----------------------
	// NOTE: we do not call _ScriptList.Remove anymore because
	// RemoveAndDeleteScript is often called from an Script being 
	// updated via SoScriptManager::Update, which walks the _ScriptList, 
	// and if a Script is removed from the _ScriptList while the 
	// ScriptList is being walked, the list could become broken.
	// 2008_07_20 mdomowicz
	//
	// The above comment was written for SoActorManager, but applies here.  
	// 2013_06_07 mdomowicz 

	if ( pScript != NULL )
	{
		_PendingDeletionList.AddUniquely( pScript, 0 );
	}
}



/******************************************************************************
 FUNCTION:  RemoveAndDeleteAllScripts
******************************************************************************/
void SoScriptManager::RemoveAndDeleteAllScripts()
{
	SoScriptList::Iterator Iter( &_ScriptList );
	while ( Iter.Next() )
	{	
		SoScript* pScript = Iter.GetNodeData();
		_PendingDeletionList.AddUniquely( pScript, 0 );
	}

	_ScriptList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  RemoveAndDeleteAllScriptsNow
******************************************************************************/
void SoScriptManager::RemoveAndDeleteAllScriptsNow()
{
	RemoveAndDeleteAllScripts();
	DeleteAllScriptsInPendingDeletionList();
}



/******************************************************************************
 FUNCTION:  DeleteAllScriptsInPendingDeletionList
******************************************************************************/
void SoScriptManager::DeleteAllScriptsInPendingDeletionList()
{
	SoScriptList::Iterator Iter( &_PendingDeletionList );
	while ( Iter.Next() )
	{	
		SoScript* pScript = Iter.GetNodeData();
		delete pScript;
	}

	_PendingDeletionList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  CreateScriptFromType
******************************************************************************/
SoScript* SoScriptManager::CreateScriptFromType( SoScript::TYPE e )
{
	switch ( e )
	{
		case SoScript::TYPE_MAIN: return new TgScriptMain;
	}
	ASSERTF( 0, ( "\nUnknown script type." ) );
	return NULL;
}



/******************************************************************************
 FUNCTION:  SaveToFile
******************************************************************************/
void SoScriptManager::SaveToFile( SoSaveSystem* pSaveSystem )
{
	NeCsvOut* pCsvWriter = pSaveSystem->GetSharedCsvWriter();

	//-----------------------
	// save the ScriptManager state data
	pCsvWriter->Clear();
	pCsvWriter->AddNewRow();
	pCsvWriter->AppendStringToLastRow( "_iScriptIdAssigner" );
	pCsvWriter->AppendIntToLastRow( _iScriptIdAssigner );
	pCsvWriter->WriteToFile( pSaveSystem->ConstructPathAndFileName( "SoScriptManager", 0 ) );

	
	//-----------------------
	// save a list of the scripts
	pCsvWriter->Clear();
	pCsvWriter->AddNewRow();
	pCsvWriter->AppendStringToLastRow( "iType" );
	pCsvWriter->AppendStringToLastRow( "_iScriptId" );
	SoScriptList::Iterator Iter( &_ScriptList );
	while ( Iter.Next() )
	{
		pCsvWriter->AddNewRow();
		SoScript* pScript = Iter.GetNodeData();

		// save so we know what script class to instance when we load.
		pCsvWriter->AppendIntToLastRow( (int)pScript->GetType() );

		// save so we know the exact file to load.
		pCsvWriter->AppendIntToLastRow( pScript->GetScriptId() );
	}

	pCsvWriter->WriteToFile( pSaveSystem->ConstructPathAndFileName( "UgScriptList", 0 ) );


	//-----------------------
	// save each script
	Iter.Reset();
	while ( Iter.Next() )
	{
		SoScript* pScript = Iter.GetNodeData();
		pScript->SaveToFile( pSaveSystem );
	}
}



/******************************************************************************
 FUNCTION:  LoadFromFile
******************************************************************************/
void SoScriptManager::LoadFromFile( SoSaveSystem* pSaveSystem )
{
	//-----------------------
	// load the ScriptManager state data
	NeCsvIn* pCsvReader = pSaveSystem->GetSharedCsvReader();
	pCsvReader->Clear();
	pCsvReader->ReadCsvFile( pSaveSystem->ConstructPathAndFileName( "SoScriptManager", 0 ) );
	pCsvReader->MakeHashIndicesFromRowValuesAtCol( 0 );
	_iScriptIdAssigner = pCsvReader->GetHashByHashAndInt( 0xf3a54429 /* "_iScriptIdAssigner" */, 1, 0 );



	//-----------------------
	// Read the script list.
	// Note we don't use the shared reader here because the 
	// instanced scripts need to use it.
	NeCsvIn CsvReader;
	CsvReader.Clear();
	CsvReader.ReadCsvFile( pSaveSystem->ConstructPathAndFileName( "TgScriptList", 0 ) );
	CsvReader.MakeHashIndicesFromColValuesAtRow( 0 );


	//-----------------------
	// for every script in the list, instance a script object
	// and load it.
	for ( int i = 1; i < CsvReader.GetNumRows(); i ++ )
	{
		//-----------------------
		// instance a script per the loaded script type
		SoScript::TYPE eScriptType = (SoScript::TYPE)CsvReader.GetIntByIntAndHash
		( 
			i, 
			0x0951fccc /* "iType" */, 
			(int)(SoScript::TYPE_LAST)
		);
		SoScript* pNewScript = CreateScriptFromType( eScriptType );


		//-----------------------
		// load and set the script id - with this, the script knows 
		// which file to load.
		int iScriptId = CsvReader.GetIntByIntAndHash( i, 0x828ad947 /* "_iScriptId" */, -1 );
		pNewScript->SetScriptId( iScriptId );


		//-----------------------
		pNewScript->LoadFromFile( pSaveSystem );


		//-----------------------
		// Add the script to the system.  Don't call AddScript, since it assigns a ScriptId,
		// which would clobber the ScriptId we loaded.
		pNewScript->SetOwnerScriptManager( this );
		_ScriptList.AddLast( pNewScript );
	}
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoScriptManager::CreateInstance()
{
	ASSERTF( !_pInstance, ( "\nCan't create a second static instance of this class." ) )
	if ( !_pInstance )
	{
		_pInstance = new SoScriptManager;
		_pInstance->Init();
	}
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoScriptManager::DeleteInstance()
{
	if ( _pInstance )
	{
		_pInstance->Deinit();

		delete _pInstance;
		_pInstance = NULL;
	}
}
