/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#pragma once


#include "stdafx.h"

#include "JpHashTable.h"
#include "JpRandom.h"
#include "SoActor.h"


#define SoScriptList  JpList< SoScript >
#define SCRIPT_ID_BASE_VALUE  1000

class SoSaveSystem;
class SoScriptManager;



class SoScript
{
	public:

		//-----------------------
		// public enums
		enum TYPE
		{
			TYPE_GAMEFLOW,
			TYPE_MAIN,
			TYPE_MISSION,
			TYPE_MISSION_ANCIENT_RUINS,
			TYPE_LAST
		};


		//-----------------------
		// public functions
		SoScript();

		virtual ~SoScript();

		virtual void Init();

		virtual void Deinit();

		virtual void Update( float dt );

		virtual TYPE GetType() { return TYPE_LAST; }

		int GetScriptId() { return _iScriptId; }
		void SetScriptId( int i ) { _iScriptId = i; }

		const std::string& GetName() { return _sName; }
		void SetName( const std::string& s ) { _sName = s; }

		JpRandom* GetRng() { return &_Rng; }

		void SetRngSeed( hash h ) 
		{ 
			_hRngSeed = h; 
			_Rng.Seed( h );
		}

		bool IsEnabled() { return _bIsEnabled; }
		void Enable() { _bIsEnabled = true; }
		void Disable() { _bIsEnabled = false; }

		SoScriptManager* GetOwnerScriptManger() { return _pOwnerScriptManager; }
		void SetOwnerScriptManager( SoScriptManager* p ) { _pOwnerScriptManager = p; }

		virtual void SaveToFile( SoSaveSystem* pSaveSystem );
		virtual void LoadFromFile( SoSaveSystem* pSaveSystem );

	protected:

		//-----------------------
		// private classes
		

		//-----------------------
		// private variables
		int _iScriptId;

		std::string _sName;

		bool _bIsEnabled;

		hash _hRngSeed;
		JpRandom _Rng;

		SoScriptManager* _pOwnerScriptManager;

		//-----------------------
		// private functions
		bool ActorExistsAndIsAlive( guid gActor );

		guid ActorAdd( SoActorTypes::TYPE eActorType, int iActorSubTypeCastToInt );
};





class SoScriptManager
{
	public:

		//-----------------------
		// public functions
		SoScriptManager();
		virtual ~SoScriptManager();

		void Init();

		void Deinit();

		void Update( float dt );


		SoScript* FindById( int iId );
		SoScript* FindByName( const std::string& sName );

		void AddScript( SoScript* p );

		void RemoveAndDeleteScript( SoScript* pScript );
		void RemoveAndDeleteAllScripts();
		void RemoveAndDeleteAllScriptsNow();
		void DeleteAllScriptsInPendingDeletionList();

		bool IsGameWon() { return _bIsGameWon; }
		void SetGameWon( bool b ) { _bIsGameWon = b; }


		void SaveToFile( SoSaveSystem* pSaveSystem );
		void LoadFromFile( SoSaveSystem* pSaveSystem );


		static SoScript* CreateScriptFromType( SoScript::TYPE e );

		static void CreateInstance();
		static void DeleteInstance();
		static SoScriptManager* GetInstance() { return _pInstance; }

	private:


		//-----------------------
		// private variables
		static SoScriptManager* _pInstance;

		int _iScriptIdAssigner;

		SoScriptList _ScriptList;
		SoScriptList _PendingDeletionList;

		bool _bIsGameWon;



		//-----------------------
		// private functions
};




