/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCamera.h"
#include "SoSkyPlane.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


// HACK!
// This was originally the skyplane used for the roguelike projects.  The image
// used was usually scaled up quite a bit, and it would move at a parallax rate
// relative to the actors, to make it look far away.
// However, for Achoir, we want just a normal background, and one that
// works in multiple resolutions.  So, I just hacked up this skyplane
// code.  It no longer moves in parallax, and it is scaled only as much
// as is necessary to cover the entire screen.
// If more elaborate background work is required, it may be best to create
// a dedicated and cleaned up class specifically for that purpose.
// But for now, repurposing the skyplane does the job.
// This is being committed to svn as revision 294.
// mdomowicz 2014_03_07


/******************************************************************************
 FUNCTION:  SoSkyPlane
******************************************************************************/
SoSkyPlane::SoSkyPlane()
{
}



/******************************************************************************
 FUNCTION:  ~SoSkyPlane
******************************************************************************/
SoSkyPlane::~SoSkyPlane()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoSkyPlane::Init( CCNode* pParentNode, const char* pacPathAndFilename )
{
	_pParentNode = pParentNode;
	_pSprite = CreateSprite( _pParentNode, pacPathAndFilename );
	SoUtils::SetRenderZOrder( _pParentNode, _pSprite, BASE_RENDER_ORDER_SKY_PLANE );

	// The sprite is set to visible only in the update.  This prevents a 1-frame flash in which
	// this sprite shows, but nothing else does.
	// mdomowicz 2013_02_19
	_pSprite->setVisible( false );

	// The scale values were found experimentally.  They size a 1024x1024 texture
	// just large enough to prevent the user from seeing the edges of the sky plane.
	// mdomowicz 2013_10_16


	// HACK - See notes at top of file.
	//_pSprite->setScaleX( 2.1f * SoUtils::GetiPadRatioX() );
	//_pSprite->setScaleY( 1.6f * SoUtils::GetiPadRatioY() );

	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	if ( SizeInPixels.width > SizeInPixels.height )
	{
		float fExactScreenWidth = SizeInPixels.width / _pSprite->getContentSize().width;
		_pSprite->setScaleX( fExactScreenWidth );
		_pSprite->setScaleY( fExactScreenWidth );
	}
	else
	{
		float fExactScreenHeight = SizeInPixels.height / _pSprite->getContentSize().height;
		_pSprite->setScaleX( fExactScreenHeight );
		_pSprite->setScaleY( fExactScreenHeight );
	}
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoSkyPlane::Deinit()
{
	_pParentNode->removeChild( _pSprite, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
	_pSprite->release();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoSkyPlane::Update( float dt )
{
	_pSprite->setVisible( true );


	float fSpriteSize = SoUtils::GetBaseSpriteWidth();
	float fSpriteSizeX = fSpriteSize;
	float fSpriteSizeY = fSpriteSize;

	JpVector2f vWorld( 0.0f, 0.0f );


	// Here we find the plane's camera relative position, however, unlike with the normal sprites
	// once we know the position relative to the camera, we scale the position down.  This has the
	// effect of making the sprite move slower relative to the camera, and thus seem
	// farther away.
	// Notice also we don't zoom the sprite.
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->GetCurrentCamera();
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	float fRate = 0.2f;
	JpVector2f vCameraSpacePos = ( vWorld - pCamera->GetPosition() ) * fRate;
	vCameraSpacePos.X *= SoUtils::GetiPadRatioX();
	vCameraSpacePos.Y *= SoUtils::GetiPadRatioY();
	JpVector2f vScreenOffset( ( SizeInPixels.width / 2.0f ), ( SizeInPixels.height / 2.0f ) );

	JpVector2f vFinal( vCameraSpacePos + vScreenOffset );

	// HACK - See notes at top of file.
	//_pSprite->SetPosition( vFinal );
}



/******************************************************************************
 FUNCTION:  CreateSprite
******************************************************************************/
SoSprite* SoSkyPlane::CreateSprite( CCNode* pParent, const char* pacPathAndFilename )
{
	SoSprite* pSprite = SoSprite::CreateScaledToDevice( pacPathAndFilename );
	pSprite->retain();
	pParent->addChild( pSprite );
	pSprite->setVisible( false );
	pSprite->setAnchorPoint( ccp( 0.5f, 0.5f ) );


	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	pSprite->setPosition( ccp( SizeInPixels.width / 2, SizeInPixels.height / 2 ) );
	

	return pSprite;
}
