/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#if DCG_IS_TARGET_WIN32
	#include "windows.h"
#elif DCG_IS_TARGET_IOS
	#include "unistd.h"
#endif


#include "fmod.hpp"
#include "fmod_errors.h"

#include "JpUtils.h"

#include "NeRandomPoolPicker.h"
#include "NeSoundEffect.h"

#include "SoConfigManager.h"
#include "SoConsole.h"
#include "SoFsmStateMain.h"
#include "SoSoundManager.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


SoSoundManager* SoSoundManager::_pInstance = NULL;



/******************************************************************************
 FUNCTION:  SoSoundManager
******************************************************************************/
SoSoundManager::SoSoundManager()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoSoundManager::Init()
{
	NeSoundManager::Init();

	_MusicTracksCsv.ReadCsvFile( SoUtils::AddRelativePathToCsv( "SoundMusicTracks.csv" ) );
	_MusicTracksCsv.MakeHashIndicesFromColValuesAtRow( 0 );
	_MusicTracksCsv.MakeHashIndicesFromRowValuesAtCol( 0 );

	_AmbienceTracksCsv.ReadCsvFile( SoUtils::AddRelativePathToCsv( "SoundAmbienceTracks.csv" ) );
	_AmbienceTracksCsv.MakeHashIndicesFromColValuesAtRow( 0 );
	_AmbienceTracksCsv.MakeHashIndicesFromRowValuesAtCol( 0 );

	_EffectsAttachmentalCsv.ReadCsvFile( SoUtils::AddRelativePathToCsv( "SoundEffectEvents.csv" ) );
	_EffectsAttachmentalCsv.MakeHashIndicesFromColValuesAtRow( 0 );

	NeConfig::Line* pLine1 = SoConfigManager::GetInstance()->FindLine( "fVolumeMusic" );
	NeConfig::Line* pLine2 = SoConfigManager::GetInstance()->FindLine( "fVolumeEffects" );

	if ( pLine1 )
		SetVolumeMusic( pLine1->GetFloat( 1 ) );

	if ( pLine2 )
		SetVolumeEffects( pLine2->GetFloat( 1 ) );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoSoundManager::Deinit()
{
	NeSoundManager::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoSoundManager::Update( float dt )
{
	NeSoundManager::Update( dt );
}



/******************************************************************************
 FUNCTION:  LoadMusicTrack
******************************************************************************/
void SoSoundManager::LoadMusicTrack( const char* pacPathAndFileName )
{
	NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "bPlayMusic" );
	bool bMusicEnabled = ( pLine && pLine->GetBool( 1 ) );

	if ( bMusicEnabled )
	{
		NeSoundManager::LoadMusicTrack( pacPathAndFileName );
	}
}



/******************************************************************************
 FUNCTION:  LoadMusicTrackForLevel
******************************************************************************/
void SoSoundManager::LoadMusicTrackForLevel( /* SoGameLevel::LEVEL_TYPE eLevelType */ )
{
/*
	NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "bPlayMusic" );
	bool bMusicEnabled = ( pLine && pLine->GetBool( 1 ) );

	if ( bMusicEnabled )
	{
		char* pacKey = SoGameLevel::GetStringFromType( eLevelType );
		char* pacFilenameTrack1 = _MusicTracksCsv.GetStringByTextIndices( pacKey, "sFilenameTrack1" );
		char* pacFilenameTrack2 = _MusicTracksCsv.GetStringByTextIndices( pacKey, "sFilenameTrack2" );
		char* pacFilenameTrack3 = _MusicTracksCsv.GetStringByTextIndices( pacKey, "sFilenameTrack3" );

		NeRandomPoolPicker Selector;
		if ( pacFilenameTrack1 )
			Selector.AddValueInt( (int)pacFilenameTrack1, 1 );
		if ( pacFilenameTrack2 )
			Selector.AddValueInt( (int)pacFilenameTrack2, 1 );
		if ( pacFilenameTrack3 )
			Selector.AddValueInt( (int)pacFilenameTrack3, 1 );


		if ( Selector.GetNumEntries() > 0 )
		{
			char* pacSelectedTrack = (char*)Selector.SelectValueFromPool( &JpRandom::G );
			if ( pacSelectedTrack )
			{
				LoadMusicTrack( SoUtils::AddRelativePathToSfx( pacSelectedTrack ) );
			}
		}
	}
*/
}



/******************************************************************************
 FUNCTION:  LoadAmbienceTrackForLevel
******************************************************************************/
void SoSoundManager::LoadAmbienceTrackForLevel( /* SoGameLevel::LEVEL_TYPE eLevelType */ )
{
	/*
	NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "bPlaySoundEffects" );
	bool bAmbienceEnabled = ( pLine && pLine->GetBool( 1 ) );

	if ( bAmbienceEnabled )
	{
		char* pacKey = SoGameLevel::GetStringFromType( eLevelType );
		char* pacFilenameTrack1 = _AmbienceTracksCsv.GetStringByTextIndices( pacKey, "sFilenameTrack1" );
		char* pacFilenameTrack2 = _AmbienceTracksCsv.GetStringByTextIndices( pacKey, "sFilenameTrack2" );
		char* pacFilenameTrack3 = _AmbienceTracksCsv.GetStringByTextIndices( pacKey, "sFilenameTrack3" );

		NeRandomPoolPicker Selector;
		if ( pacFilenameTrack1 )
			Selector.AddValueInt( (int)pacFilenameTrack1, 1 );
		if ( pacFilenameTrack2 )
			Selector.AddValueInt( (int)pacFilenameTrack2, 1 );
		if ( pacFilenameTrack3 )
			Selector.AddValueInt( (int)pacFilenameTrack3, 1 );


		if ( Selector.GetNumEntries() > 0 )
		{
			char* pacSelectedTrack = (char*)Selector.SelectValueFromPool( &JpRandom::G );
			if ( pacSelectedTrack )
			{
				NeSoundEffect* pSound = LoadSoundEffect( SoUtils::AddRelativePathToSfx( pacSelectedTrack ) );
				_sAmbienceTrackFileName = pacSelectedTrack;
			}
		}
	}
	*/
}



/******************************************************************************
 FUNCTION:  UnloadAmbienceTrack
******************************************************************************/
void SoSoundManager::UnloadAmbienceTrack()
{
	UnloadSoundEffect( JpHash( _sAmbienceTrackFileName.c_str() ) );
}



/******************************************************************************
 FUNCTION:  PlayAmbienceTrack
******************************************************************************/
void SoSoundManager::PlayAmbienceTrack()
{
	NeSoundEffect* pSound = GetSoundEffect( JpHash( _sAmbienceTrackFileName.c_str() ) );
	if ( pSound )
	{
		pSound->SetLoopCount( -1 );
		pSound->Play();
		pSound->GetChannel()->setVolume( SoConsoleFunctions::fVolumeEffects );
	}
}



/******************************************************************************
 FUNCTION:  StopAmbienceTrack
******************************************************************************/
void SoSoundManager::StopAmbienceTrack()
{
	NeSoundEffect* pSound = GetSoundEffect( JpHash( _sAmbienceTrackFileName.c_str() ) );
	if ( pSound )
		pSound->Stop();
}



/******************************************************************************
 FUNCTION:  PlaySoundEffect
******************************************************************************/
NeSoundEffect* SoSoundManager::PlaySoundEffect( hash hSoundEffect )
{
	NeSoundEffect* pSfx = SoSoundManager::GetInstance()->GetSoundEffect( hSoundEffect );
	if ( pSfx )
	{
		pSfx->Play();
		pSfx->GetChannel()->setVolume( SoConsoleFunctions::fVolumeEffects );
	}

	return pSfx;
}



/******************************************************************************
 FUNCTION:  LoadSoundEffectsByCsv
******************************************************************************/
void SoSoundManager::LoadSoundEffectsByCsv( const char* pacCsvPathAndFilename )
{
	NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "bPlaySoundEffects" );
	bool bPlaySoundEffects = ( pLine && pLine->GetBool( 1 ) );

	if ( bPlaySoundEffects )
	{
		NeCsvIn CsvReader;
		CsvReader.ReadCsvFile( pacCsvPathAndFilename );
		CsvReader.MakeHashIndicesFromColValuesAtRow( 0 );

		for ( int i = 1; i < CsvReader.GetNumRows(); i ++ )
		{
			char* pacFilename = CsvReader.GetStringByIntAndHash( i, 0x0db47d7f /* "sFilename" */ );

            if ( pacFilename )
            {
                char* pacFile = (char*)CCFileUtils::sharedFileUtils()->fullPathFromRelativePath
                (
                    SoUtils::AddRelativePathToSfx( pacFilename )
                );

                LoadSoundEffect( pacFile );
            }
		}
	}
}



/******************************************************************************
 FUNCTION:  PlaySoundEffectCsvEvent
******************************************************************************/
NeSoundEffect* SoSoundManager::PlaySoundEffectCsvEvent( SoSoundManager::EVENT_TYPE eType, SoActor* pActor )
{
	JpList< NeSoundEffect > _SfxList;

	for ( int i = 1; i < _EffectsAttachmentalCsv.GetNumRows(); i ++ )
	{
		char* pacActorType = 
			_EffectsAttachmentalCsv.GetStringByIntAndHash( i, 0xfbb76ecc /* "SoActorType" */ );
		char* pacEvent     = 
			_EffectsAttachmentalCsv.GetStringByIntAndHash( i, 0xc8489d3e /* "sEvent" */ );
		char* pacSoundEffectFilename = 
			_EffectsAttachmentalCsv.GetStringByIntAndHash( i, 0x67185fb5 /* "sSoundEffectFilename" */ );

		std::string sEvent = "TYPE_";
		sEvent.append( pacEvent );
		pacEvent = (char*)sEvent.c_str();

		SoUtils::StringToUpper( pacActorType );
		SoUtils::StringToUpper( pacEvent );

		if ( pActor )
		{
			if ( JpHash( pacActorType ) == pActor->GetCsvRowIndex() )
			{
				if ( GetTypeFromString( pacEvent ) == eType )
				{
					NeSoundEffect* pEffect =
						GetSoundEffectTable()->Find( JpHash( pacSoundEffectFilename ) );

					if ( pEffect )
						_SfxList.AddLast( pEffect );
					else
					{
						NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "bPlaySoundEffects" );
						bool bPlaySoundEffects = ( pLine && pLine->GetBool( 1 ) );

						if ( bPlaySoundEffects )
						{
							ASSERTF( 0, ("Sound effect file not found!! Spelling? Correct extension?") );
						}
					}
				}
			}
		}
		else
		{
			if ( GetTypeFromString( pacEvent ) == eType )
			{
				NeSoundEffect* pEffect =
					GetSoundEffectTable()->Find( JpHash( pacSoundEffectFilename ) );

				if ( pEffect )
					_SfxList.AddLast( pEffect );
				else
				{
					NeConfig::Line* pLine = SoConfigManager::GetInstance()->FindLine( "bPlaySoundEffects" );
					bool bPlaySoundEffects = ( pLine && pLine->GetBool( 1 ) );

					if ( bPlaySoundEffects )
					{
						ASSERTF( 0, ("Sound effect file not found!! Spelling? Correct extension?") );
					}
				}
			}
		}
	}

	NeSoundEffect* pSfxToPlay = NULL;

	if ( _SfxList.GetNumEntries() )
	{
		if ( _SfxList.GetNumEntries() == 1 )
		{
			JpList< NeSoundEffect >::Node* pNode = _SfxList.GetFirst();
			pSfxToPlay = pNode->GetData();
		}
		else
		{
			int iRand = JpRandom::G.GetInt()%_SfxList.GetNumEntries();
			pSfxToPlay = _SfxList.GetAtIndex( iRand )->GetData();
		}

		pSfxToPlay->Play();
		pSfxToPlay->GetChannel()->setVolume( SoConsoleFunctions::fVolumeEffects );
	}

	return pSfxToPlay;
}



/******************************************************************************
 FUNCTION:  PlaySoundEffectCsvEventRandomPitch
******************************************************************************/
NeSoundEffect* SoSoundManager::PlaySoundEffectCsvEventRandomPitch( SoSoundManager::EVENT_TYPE eType, SoActor* pActor )
{
	NeSoundEffect* pEffect = PlaySoundEffectCsvEvent( eType, pActor );

	if ( pEffect )
	{
		float fFrequency = 1.0f;
		int i = JpRandom::G.GetInt()%3;

		switch ( i )
		{
			case 0: fFrequency = 1.0f; break;
			case 1: fFrequency = 1.0f; break;
			case 2: fFrequency = 1.0f; break;
		}

		// This is the non DSP way of changing frequency.  Keeping it here until we
		// decide which way to go.
		//pEffect->GetChannel()->setFrequency( fFrequency );


		// This is the DSP way of changing the pitch.  This way is better in that it
		// does not change the length of the sound clip.  The downside is it only allows
		// values between 0.5f and 2.0f for fFrequency.
		FMOD::DSP* dsp;
		FMOD_RESULT Result = GetFmodSystem()->createDSPByType(FMOD_DSP_TYPE_PITCHSHIFT, &dsp);
		Result = pEffect->GetChannel()->addDSP(dsp, 0);
		dsp->setParameter(FMOD_DSP_PITCHSHIFT_PITCH, fFrequency );

		// --------------------
		// Show a HudLog message
		//SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();
		//SoHudLog* pHudLog = pConsole->GetHudLog();
		//pHudLog->AddMessageF( JpUtils::_Magenta, "Sound effect pitch change: %.1f%%", fFrequency );
	}

	return pEffect;
}


	
/******************************************************************************
 FUNCTION:  PlaySoundEffectOnHit
******************************************************************************/
NeSoundEffect* SoSoundManager::PlaySoundEffectOnHit( SoActor* pActor, SoActor* pInflictor )
{
	SoSoundManager::EVENT_TYPE eType = TYPE_LAST;
	NeSoundEffect* pSoundEffect = NULL;

	switch ( pInflictor->GetCsvRowIndex() )
	{
		case 0xe894a379 /* "BULLET" */:
			pSoundEffect = SoSoundManager::GetInstance()->PlaySoundEffectCsvEventRandomPitch( SoSoundManager::TYPE_ONHIT_BULLET, pActor ); 
			break;
		case 0x79ca637b /* "MELEE" */:
			pSoundEffect = SoSoundManager::GetInstance()->PlaySoundEffectCsvEventRandomPitch( SoSoundManager::TYPE_ONHIT_MELEE, pActor ); 
			break;
		case 0x0092c223 /* "PET_BASIC" */:
			return NULL;
			break;
		default:
			break;
	};

	if ( !pSoundEffect )
		pSoundEffect = SoSoundManager::GetInstance()->PlaySoundEffectCsvEvent( SoSoundManager::TYPE_ONHIT, pActor );	
	
	if ( !pSoundEffect )
		pSoundEffect = SoSoundManager::GetInstance()->PlaySoundEffect( 0xc34b56a9 /* "SfxDefaultOnHit.mp3" */ );

	return pSoundEffect;
}



/******************************************************************************
 FUNCTION:  PlaySoundEffectOnDeath
******************************************************************************/
NeSoundEffect* SoSoundManager::PlaySoundEffectOnDeath( SoActor* pActor )
{
	NeSoundEffect* pSoundEffect = NULL;

	pSoundEffect = SoSoundManager::GetInstance()->PlaySoundEffectCsvEvent( SoSoundManager::TYPE_ONDEATH, pActor );	
	
	if ( !pSoundEffect )
	{
		pSoundEffect = SoSoundManager::GetInstance()->PlaySoundEffect( 0x6331a411 /* "SfxDefaultDeath.mp3" */ );
	}

	return pSoundEffect;
}



/******************************************************************************
 FUNCTION:  GetTypeFromString
******************************************************************************/
SoSoundManager::EVENT_TYPE SoSoundManager::GetTypeFromString( const char* pacString )
{
	unsigned int uiStringHash = JpHash( pacString );

	switch ( uiStringHash )
	{
		case 0x26f57227 /* "TYPE_ONATTACK" */: return TYPE_ONATTACK;
		case 0xf37d7002 /* "TYPE_ONATTACK_GUN" */: return TYPE_ONATTACK_GUN;
		case 0x251ee673 /* "TYPE_ONDEATH" */: return TYPE_ONDEATH;
		case 0xbafa8e90 /* "TYPE_ONHIT" */: return TYPE_ONHIT;
		case 0x93e7ad31 /* "TYPE_ONHIT_BULLET" */: return TYPE_ONHIT_BULLET;
		case 0x0812e2c3 /* "TYPE_ONHIT_MELEE" */: return TYPE_ONHIT_MELEE;
		case 0x2f30ad7f /* "TYPE_ONPICKUP" */: return TYPE_ONPICKUP;
		case 0xf091ea67 /* "TYPE_ONTOGGLETARGETINGOFF" */: return TYPE_ONTOGGLETARGETINGOFF;
		case 0x8c603ba3 /* "TYPE_ONTOGGLETARGETINGON" */: return TYPE_ONTOGGLETARGETINGON;

		default: ASSERTF( 0, ( "\nString could not be converted to event Type." ) );
	}

	return TYPE_LAST;
}



/******************************************************************************
 FUNCTION:  GetStringFromType
******************************************************************************/
const char* SoSoundManager::GetStringFromType( SoSoundManager::EVENT_TYPE eType )
{
	switch ( eType )
	{
		case TYPE_ONATTACK: return "TYPE_ONATTACK";
		case TYPE_ONATTACK_GUN: return "TYPE_ONATTACK_GUN";
		case TYPE_ONDEATH: return "TYPE_ONDEATH";
		case TYPE_ONHIT: return "TYPE_ONHIT";
		case TYPE_ONHIT_BULLET: return "TYPE_ONHIT_BULLET";
		case TYPE_ONHIT_MELEE: return "TYPE_ONHIT_MELEE";
		case TYPE_ONPICKUP: return "TYPE_ONPICKUP";
		case TYPE_ONTOGGLETARGETINGOFF: return "TYPE_ONTOGGLETARGETINGOFF";
		case TYPE_ONTOGGLETARGETINGON: return "TYPE_ONTOGGLETARGETINGON";
		default: ASSERTF( 0, ( "\nEvent Type not found." ) );
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  CreateInstance
******************************************************************************/
void SoSoundManager::CreateInstance()
{
	ASSERTF( !_pInstance, ( "\nCan't create a second static instance of this class." ) )
	if ( !_pInstance )
	{
		_pInstance = new SoSoundManager;
		_pInstance->Init();
	}
}



/******************************************************************************
 FUNCTION:  DeleteInstance
******************************************************************************/
void SoSoundManager::DeleteInstance()
{
	if ( _pInstance )
	{
		_pInstance->Deinit();

		delete _pInstance;
		_pInstance = NULL;
	}
}



/******************************************************************************
 FUNCTION:  SetVolumeMusic
******************************************************************************/
void SoSoundManager::SetVolumeMusic( float f ) 
{ 
	SoConsoleFunctions::fVolumeMusic = Clamp( f, 0.0f, 1.0f );

	if ( NeSoundManager::GetMusicTrackChannel() )
	{
		NeSoundManager::GetMusicTrackChannel()->setVolume( SoConsoleFunctions::fVolumeMusic );
	}
}




/******************************************************************************
 FUNCTION:  SetEffectsVolume
******************************************************************************/
void SoSoundManager::SetVolumeEffects( float f ) 
{ 
	SoConsoleFunctions::fVolumeEffects = Clamp( f, 0.0f, 1.0f );
}



/******************************************************************************
 FUNCTION:  PlayMusic
******************************************************************************/
void SoSoundManager::PlayMusic()
{
	NeSoundManager::PlayMusic();

	if ( NeSoundManager::GetMusicTrackChannel() )
	{
		NeSoundManager::GetMusicTrackChannel()->setVolume( SoConsoleFunctions::fVolumeMusic );
	}
}













