/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#include "fmod.hpp"
#include "cocos2d.h"

#include "JpHashTable.h"

#include "NeCsvIn.h"
#include "NeSoundManager.h"

#include "SoActor.h"
#include "SoConsole.h"

USING_NS_CC;


class SoSoundManager : public NeSoundManager
{
	public:

		//-----------------------
		// public enums

		enum EVENT_TYPE
		{
			TYPE_ONATTACK,
			TYPE_ONATTACK_GUN,
			TYPE_ONDEATH,
			TYPE_ONHIT,
			TYPE_ONHIT_BULLET,
			TYPE_ONHIT_MELEE,
			TYPE_ONPICKUP,
			TYPE_ONTOGGLETARGETINGOFF,
			TYPE_ONTOGGLETARGETINGON,
			TYPE_LAST
		};


		//-----------------------
		// public functions

		SoSoundManager();

		virtual ~SoSoundManager() {};


		virtual void Init();
		virtual void Deinit();
		virtual void Update( float dt );

		virtual void PlayMusic();

		void PlayAmbienceTrack();
		void StopAmbienceTrack();
		void UnloadAmbienceTrack();

		NeSoundEffect* PlaySoundEffect( hash hSoundEffect );
		NeSoundEffect* PlaySoundEffectCsvEvent( SoSoundManager::EVENT_TYPE eType, SoActor* pActor );
		NeSoundEffect* PlaySoundEffectCsvEventRandomPitch( SoSoundManager::EVENT_TYPE eType, SoActor* pActor );
		NeSoundEffect* PlaySoundEffectOnHit( SoActor* pActor, SoActor* pInflictor );
		NeSoundEffect* PlaySoundEffectOnDeath( SoActor* pActor );

		void LoadAmbienceTrackForLevel( /* SoGameLevel::LEVEL_TYPE eLevelType */ );
		void LoadSoundEffectsByCsv( const char* pacCsvPathAndFilename );
		void LoadMusicTrack( const char* pacPathAndFileName );
		void LoadMusicTrackForLevel( /* SoGameLevel::LEVEL_TYPE eLevelType */ );

		static void CreateInstance();
		static void DeleteInstance();
		static SoSoundManager* GetInstance() { return _pInstance; }
		static SoSoundManager::EVENT_TYPE GetTypeFromString( const char* pacString );
		static const char* GetStringFromType( SoSoundManager::EVENT_TYPE eType );

		float GetVolumeMusic() { return SoConsoleFunctions::fVolumeMusic; }
		void  SetVolumeMusic( float f );

		float GetVolumeEffects() { return SoConsoleFunctions::fVolumeEffects; }
		void  SetVolumeEffects( float f );


	private:

		//-----------------------
		// private variables
		static SoSoundManager* _pInstance;

		NeCsvIn _MusicTracksCsv;
		NeCsvIn _AmbienceTracksCsv;
		NeCsvIn _EffectsAttachmentalCsv;

		std::string _sAmbienceTrackFileName;

		//-----------------------
		// private functions
};




