/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoSprite.h"
#include "SoTextureUtils.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoSprite
******************************************************************************/
SoSprite::SoSprite()
{
}



/******************************************************************************
 FUNCTION:  ~SoSprite
******************************************************************************/
SoSprite::~SoSprite()
{
}



/******************************************************************************
 FUNCTION:  SetPosition
******************************************************************************/
void SoSprite::SetPosition( const JpVector2f& v )
{
	setPosition( ccp(v.X, v.Y ) );
}



/******************************************************************************
 FUNCTION:  GetPosition
******************************************************************************/
JpVector2f SoSprite::GetPosition()
{
	return JpVector2f( getPositionX(), getPositionY() );
}



/******************************************************************************
 FUNCTION:  ScaleToBox
******************************************************************************/
void SoSprite::ScaleToBox
( 
	float fBottomLeftX, 
	float fBottomLeftY, 
	float fTopRightX, 
	float fTopRightY 
)
{
	float fWidth = ( fTopRightX - fBottomLeftX );
	float fHeight = ( fTopRightY - fBottomLeftY );

	setScaleX( fWidth / getContentSize().width );
	setScaleY( fHeight / getContentSize().height );
}



/******************************************************************************
 FUNCTION:  CreateScaledToDevice
******************************************************************************/
SoSprite* SoSprite::CreateScaledToDevice( const char* pacFileName )
{
 	SoSprite* pSprite = SoSprite::create( pacFileName );
	pSprite->setScaleX( SoUtils::GetiPadRatioX() );
	pSprite->setScaleY( SoUtils::GetiPadRatioY() );
	return pSprite;
}



/******************************************************************************
 FUNCTION:  CreateSpriteResampledFromFilename
******************************************************************************/
SoSprite* SoSprite::CreateSpriteResampledFromFilename( const char *pszSpriteFrameName, float fScale )
{
	// We use our custom engine-changed AddUiImageScaled() to load a scaled texture.
	CCTexture2D* pTexture = SoTextureUtils::AddUiImageScaled( pszSpriteFrameName, fScale );

	// Note: no call to autorelease here, since that happens in SoSprite::spriteWithTexture();
	// And double calls to autorelease can cause really bad, mysterious, crash bugs!
	// mdomowicz 2013_02_28
	SoSprite* pSprite = SoSprite::spriteWithTexture( pTexture );
	pSprite->setScaleX( SoUtils::GetiPadRatioX() );
	pSprite->setScaleY( SoUtils::GetiPadRatioY() );

	return pSprite;
}










/******************************************************************************
*******************************************************************************
   Every function below this line is a straight up copy of 
   CCSprite, except that these are for creating SoSprites rather than CCSprites
*******************************************************************************
******************************************************************************/
SoSprite* SoSprite::spriteWithTexture(CCTexture2D *pTexture)
{
    return SoSprite::create(pTexture);
}

SoSprite* SoSprite::create(CCTexture2D *pTexture)
{
    SoSprite *pobSprite = new SoSprite();
    if (pobSprite && pobSprite->initWithTexture(pTexture))
    {
        pobSprite->autorelease();
        return pobSprite;
    }
    CC_SAFE_DELETE(pobSprite);
    return NULL;
}

SoSprite* SoSprite::spriteWithTexture(CCTexture2D *pTexture, const CCRect& rect)
{
    return SoSprite::create(pTexture, rect);
}

SoSprite* SoSprite::create(CCTexture2D *pTexture, const CCRect& rect)
{
    SoSprite *pobSprite = new SoSprite();
    if (pobSprite && pobSprite->initWithTexture(pTexture, rect))
    {
        pobSprite->autorelease();
        return pobSprite;
    }
    CC_SAFE_DELETE(pobSprite);
    return NULL;
}

SoSprite* SoSprite::spriteWithFile(const char *pszFileName)
{
    return SoSprite::create(pszFileName);
}

SoSprite* SoSprite::create(const char *pszFileName)
{
    SoSprite *pobSprite = new SoSprite();
    if (pobSprite && pobSprite->initWithFile(pszFileName))
    {
        pobSprite->autorelease();
        return pobSprite;
    }
    CC_SAFE_DELETE(pobSprite);
    return NULL;
}

SoSprite* SoSprite::spriteWithFile(const char *pszFileName, const CCRect& rect)
{
    return SoSprite::create(pszFileName, rect);
}

SoSprite* SoSprite::create(const char *pszFileName, const CCRect& rect)
{
    SoSprite *pobSprite = new SoSprite();
    if (pobSprite && pobSprite->initWithFile(pszFileName, rect))
    {
        pobSprite->autorelease();
        return pobSprite;
    }
    CC_SAFE_DELETE(pobSprite);
    return NULL;
}

SoSprite* SoSprite::spriteWithSpriteFrame(CCSpriteFrame *pSpriteFrame)
{
    return SoSprite::create(pSpriteFrame);
}

SoSprite* SoSprite::create(CCSpriteFrame *pSpriteFrame)
{
    SoSprite *pobSprite = new SoSprite();
    if (pobSprite && pobSprite->initWithSpriteFrame(pSpriteFrame))
    {
        pobSprite->autorelease();
        return pobSprite;
    }
    CC_SAFE_DELETE(pobSprite);
    return NULL;
}

SoSprite* SoSprite::spriteWithSpriteFrameName(const char *pszSpriteFrameName)
{
    return SoSprite::createWithSpriteFrameName(pszSpriteFrameName);
}

SoSprite* SoSprite::createWithSpriteFrameName(const char *pszSpriteFrameName)
{
    CCSpriteFrame *pFrame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(pszSpriteFrameName);

    char msg[256] = {0};
    sprintf(msg, "Invalid spriteFrameName: %s", pszSpriteFrameName);
    CCAssert(pFrame != NULL, msg);
    return create(pFrame);
}

SoSprite* SoSprite::node()
{
    return SoSprite::create();
}

SoSprite* SoSprite::create()
{
    SoSprite *pSprite = new SoSprite();
    if (pSprite && pSprite->init())
    {
        pSprite->autorelease();
        return pSprite;
    }
    CC_SAFE_DELETE(pSprite);
    return NULL;
}

	
	
























