/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "JpVector.h"


USING_NS_CC;


#define SoSpriteList JpList< SoSprite >

/*
	This class exists so that we can add custom 
	functionality onto CCSprite.
	mdomowicz 2012_09_01
*/

class SoSprite : public CCSprite
{
	public:

		//-----------------------
		// public functions

		SoSprite();

		virtual ~SoSprite();

		void SetPosition( const JpVector2f& v );
		JpVector2f GetPosition();

		void ScaleToBox( float fBottomLeftX, float fBottomLeftY, float fTopRightX, float fTopRightY );


		static SoSprite* CreateScaledToDevice( const char* pacFileName );

		// Creates an sprite who's texture is scaled by the value fScale.
		static SoSprite* CreateSpriteResampledFromFilename( const char *pszSpriteFrameName, float fScale );



		//-----------------------
		// Every public function below this line is a straight up copy of 
		// CCSprite, except that these are for creating SoSprites rather than CCSprites

		/** Creates an sprite with a texture.
		 The rect used will be the size of the texture.
		 The offset will be (0,0).
		 @deprecated: This interface will be deprecated sooner or later.
		 */
		CC_DEPRECATED_ATTRIBUTE static SoSprite* spriteWithTexture(CCTexture2D *pTexture);

		/** Creates an sprite with a texture and a rect.
		 The offset will be (0,0).
		 @deprecated: This interface will be deprecated sooner or later.
		 */
		CC_DEPRECATED_ATTRIBUTE static SoSprite* spriteWithTexture(CCTexture2D *pTexture, const CCRect& rect);

			/** Creates an sprite with a texture.
		 The rect used will be the size of the texture.
		 The offset will be (0,0).
		 */
		static SoSprite* create(CCTexture2D *pTexture);

		/** Creates an sprite with a texture and a rect.
		 The offset will be (0,0).
		 */
		static SoSprite* create(CCTexture2D *pTexture, const CCRect& rect);

		/** Creates an sprite with an sprite frame. 
		@deprecated: This interface will be deprecated sooner or later.
		*/
		CC_DEPRECATED_ATTRIBUTE static SoSprite* spriteWithSpriteFrame(CCSpriteFrame *pSpriteFrame);

		/** Creates an sprite with an sprite frame name.
		 An CCSpriteFrame will be fetched from the CCSpriteFrameCache by name.
		 If the CCSpriteFrame doesn't exist it will raise an exception.
		 @deprecated: This interface will be deprecated sooner or later.
		 @since v0.9
		 */
		CC_DEPRECATED_ATTRIBUTE static SoSprite* spriteWithSpriteFrameName(const char *pszSpriteFrameName);

		/** Creates an sprite with an sprite frame. */
		static SoSprite* create(CCSpriteFrame *pSpriteFrame);

		/** Creates an sprite with an sprite frame name.
		 An CCSpriteFrame will be fetched from the CCSpriteFrameCache by name.
		 If the CCSpriteFrame doesn't exist it will raise an exception.
		 @since v0.9
		 */
		static SoSprite* createWithSpriteFrameName(const char *pszSpriteFrameName);

		/** Creates an sprite with an image filename.
		 The rect used will be the size of the image.
		 The offset will be (0,0).
		 @deprecated: This interface will be deprecated sooner or later.
		 */
		CC_DEPRECATED_ATTRIBUTE static SoSprite* spriteWithFile(const char *pszFileName);

		/** Creates an sprite with an image filename and a rect.
		 The offset will be (0,0).
		 @deprecated: This interface will be deprecated sooner or later.
		 */
		CC_DEPRECATED_ATTRIBUTE static SoSprite* spriteWithFile(const char *pszFileName, const CCRect& rect);
	    
		/** Creates an sprite with an image filename.
		 The rect used will be the size of the image.
		 The offset will be (0,0).
		 */
		static SoSprite* create(const char *pszFileName);

		/** Creates an sprite with an image filename and a rect.
		 The offset will be (0,0).
		 */
		static SoSprite* create(const char *pszFileName, const CCRect& rect);

		/** Creates an sprite.
		@deprecated: This interface will be deprecated sooner or later.
		 */
		CC_DEPRECATED_ATTRIBUTE static SoSprite* node();
		/** Creates an sprite.
		 */
		static SoSprite* create();

	private:

		//-----------------------
		// private variables
};




