//
//  SoSpriteAnim.cpp
//  FlyingFreak
//
//  Created by varadharaj on 16/05/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
//  Updated for cocos2d-2.0-x-2.0.2 by Francis Styck
//



#include "stdafx.h"

#include "JpList.h"
#include "JpString.h"
#include "SoSpriteAnim.h"
#include "SoTextureUtils.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


#define STATE_WAIT                             0
#define STATE_PLAY_FORWARD                     1
#define STATE_PLAY_FORWARD_HANDLE_LAST_FRAME   2
#define STATE_PLAY_BACKWARD                    3
#define STATE_PLAY_BACKWARD_HANDLE_LAST_FRAME  4


#ifdef _DEBUG
	JpHashTable< int, 4096 > SoSpriteAnim::_DebugFrameNameCollisionChecker;
#endif



/******************************************************************************
 FUNCTION:  SoSpriteAnim
******************************************************************************/
SoSpriteAnim::SoSpriteAnim() :
_pfnOnAnimationDoneCallback( 0 ),
_pParentObject( NULL ),
_bIsPlistLoaded( false ),
_fTimer( 0.0f ),
_iIncrementValue( 0 ),
_bIsAnimationRunning( false ),
_iNumFrames( 0 ),
_iNumLoops( -1 ),
_fSecondsPerFrame( 1.0f ),
_bPlayReversedWhenAtEnd( false ),
_CurrentState( STATE_WAIT )
{
}



/******************************************************************************
 FUNCTION:  ~SoSpriteAnim
******************************************************************************/
SoSpriteAnim::~SoSpriteAnim()
{
	if( _bIsPlistLoaded )
		CC_SAFE_DELETE( _pAnimationFrames );
}



/******************************************************************************
 FUNCTION:  addFrames
******************************************************************************/
void SoSpriteAnim::addFrames(const char *pList)
{
	_bIsPlistLoaded = true;
	_pAnimationFrames = new CCArray;

	// IMPORTANT:
	// The sprite frames will be cached AND RETAINED, and they won't be released unless you call
	//     CCSpriteFrameCache::sharedSpriteFrameCache()->removeUnusedSpriteFrames);
	CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();
	cache->addSpriteFramesWithFile(pList);

	const char *pszPath = CCFileUtils::sharedFileUtils()->fullPathFromRelativePath(pList);

	// This didn't compile so I replaced it with a call to "create()", which
	// appears to do the intended work.  create() sets the dictionary
	// to autorelease, which I think is also desired behavior.
	// mdomowicz 2012_09_29
	//
	//CCDictionary *dict = CCDictionary::createWithContentsOfFile(pszPath);
	CCDictionary *dict = CCDictionary::create(pszPath);

	CCDictionary *framesDict = (CCDictionary*)dict->objectForKey(std::string("frames"));

	CCDictElement* pElement = NULL;
	CCDICT_FOREACH(framesDict, pElement)
	{
		CCDictionary* frameDict = (CCDictionary*)pElement->getObject();
		std::string spriteFrameName = pElement->getStrKey();
		CCSpriteFrame* frame = cache->spriteFrameByName( spriteFrameName.c_str() );
		_pAnimationFrames->addObject(frame);
	}

	setDisplayFrame((CCSpriteFrame *)_pAnimationFrames->objectAtIndex(0));
}



/******************************************************************************
 FUNCTION:  addSpriteFramesWithFile
******************************************************************************/
void SoSpriteAnim::addSpriteFramesWithFile( const char* plist, const char* textureFileName )
{
	CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();

	CCAssert(textureFileName, "texture name should not be null");
	CCTexture2D *texture = SoTextureUtils::AddUiImageScaled(textureFileName, SoTextureUtils::GetTextureScaler() );

	if (texture)
	{
		cache->addSpriteFramesWithFile(plist, texture);
	}
	else
	{
		CCLOG("cocos2d: CCSpriteFrameCache: couldn't load texture file. File not found %s", textureFileName);
	}
}



/******************************************************************************
 FUNCTION:  addFrames
******************************************************************************/
int SoSpriteAnim::addFrames(const char *pList, const char *textureFileName)
{
	_bIsPlistLoaded = true;
	_pAnimationFrames = new CCArray;

	// IMPORTANT:
	// The sprite frames will be cached AND RETAINED, and they won't be released unless you call
	//     CCSpriteFrameCache::sharedSpriteFrameCache()->removeUnusedSpriteFrames);
	CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();
	addSpriteFramesWithFile(pList, textureFileName);

	const char *pszPath = CCFileUtils::sharedFileUtils()->fullPathFromRelativePath(pList);

	//-----------------------
	// This didn't compile so I replaced it with a call to "create()", which
	// appears to do the intended work.  create() sets the dictionary
	// to autorelease, which I think is also desired behavior.
	// mdomowicz 2012_09_29
	//
	//CCDictionary *dict = CCDictionary::createWithContentsOfFile(pszPath);
	CCDictionary *dict = CCDictionary::create(pszPath);

	CCDictionary *framesDict = (CCDictionary*)dict->objectForKey(std::string("frames"));

	JpList< CCSpriteFrame > FramesList;

	CCDictElement* pElement = NULL;
	CCDICT_FOREACH(framesDict, pElement)
	{
		CCDictionary* frameDict = (CCDictionary*)pElement->getObject();
		std::string spriteFrameName = pElement->getStrKey();
		CCSpriteFrame* frame = cache->spriteFrameByName( spriteFrameName.c_str() );

		// The following 'if' is for if the texture has been scaled with the texture scaler.
		// If so, then the sprite frame's sizes and offsets must also then be scaled.
		// We want to only do this once, however, no matter how many times we load this frame.
		// Hence the added m_bHasBeenScaled variable to CCSpriteFrame.
		if ( !frame->m_bHasBeenScaled )
		{
		
			CCSize vSize = frame->getOriginalSize();
			vSize.width  *= SoTextureUtils::GetTextureScaler();
			vSize.height *= SoTextureUtils::GetTextureScaler();
			frame->setOriginalSize( vSize );

			CCPoint vOffset = frame->getOffset();
			vOffset.x *= SoTextureUtils::GetTextureScaler();
			vOffset.y *= SoTextureUtils::GetTextureScaler();
			frame->setOffset( vOffset );

			CCRect Rect1 = frame->getRect();
			Rect1.origin.x *= SoTextureUtils::GetTextureScaler();
			Rect1.origin.y *= SoTextureUtils::GetTextureScaler();
			Rect1.size.width *= SoTextureUtils::GetTextureScaler();
			Rect1.size.height *= SoTextureUtils::GetTextureScaler();
			frame->setRect( Rect1 );

			// Also setting these seems to cause an error.  Leaving here just in case.
			//CCSize vSize2 = frame->getOriginalSizeInPixels();
			//vSize2.width  *= SoTextureUtils::GetTextureScaler();
			//vSize2.height *= SoTextureUtils::GetTextureScaler();
			//frame->setOriginalSizeInPixels( vSize2 );

			//CCPoint vOffset2 = frame->getOffsetInPixels();
			//vOffset2.x *= SoTextureUtils::GetTextureScaler();
			//vOffset2.y *= SoTextureUtils::GetTextureScaler();
			//frame->setOffsetInPixels( vOffset2 );

			//CCRect Rect2 = frame->getRectInPixels();
			//Rect2.origin.x *= SoTextureUtils::GetTextureScaler();
			//Rect2.origin.y *= SoTextureUtils::GetTextureScaler();
			//Rect2.size.width *= SoTextureUtils::GetTextureScaler();
			//Rect2.size.height *= SoTextureUtils::GetTextureScaler();
			//frame->setRectInPixels( Rect2 );

			frame->m_bHasBeenScaled = true;
		}


		//-----------------------
		// BUGFIX! 
		// Because CCDICT_FOREACH is not guaranteed to return each element in the
		// order it was added, just adding the frames to a list like this:
		//     m_AnimationFrames->addObject(frame);
		// could mean that the animation frames are played out of order.  In
		// my particular case, the order was correct on PC, but completely
		// incorrect on iOS.  So, my fix is to edit the .plist data to add
		// a "SoFrameNumber" value to each frame, and then add the frames to
		// a JpList sorted by their SoFrameNumber.  With the frames in the
		// JpList in the correct order, we then copy those frames back to the 
		// m_AnimationFrames list.
		// mdomowicz 2012_10_15
		//
		// GOTCHA!
		// Also, note that if you hand edit the .plist to add in the SoFrameNumber data
		// all indentation must be done with spaces, not tabs.  The parser that cocos
		// uses to read the .plist file is pretty lame, and won't properly read or even find
		// the values if they use tabs to indent.
		// mdomowicz 2012_10_15
// 		FramesList.Add( frame, frameDict->valueForKey("SoFrameNumber")->intValue() );
		FramesList.Add( frame, GetFrameNumber( spriteFrameName ) );


		//-----------------------
		// This is to signal when a frame with the name of an already existing frame is
		// being added.  Duplicate frame names are bad because all the frames are stored in a single
		// shared frame dictionary, and names and frames must have a 1:1 correspondence.
		// mdomowicz 2013_03_02
		#ifdef _DEBUG
			/*
			hash hNameHash = JpHash( spriteFrameName.c_str() );
			bool bFrameExistsAlready = _DebugFrameNameCollisionChecker.Exists( hNameHash );
			ASSERTF( !bFrameExistsAlready, ( "\nAn animation frame with this name is already in use!  Duplicates will lead to errors.  Please fix." ) );
			_DebugFrameNameCollisionChecker.Store( hNameHash, 0 );
			*/
		#endif
	}


	//-----------------------
	// Copy the ordered frames in FramesList back to m_AnimationFrames
	JpList< CCSpriteFrame >::Iterator Iter( &FramesList );
	while ( Iter.Next() )
	{
		CCSpriteFrame* frame = Iter.GetNodeData();
		_pAnimationFrames->addObject(frame);
	}


	setDisplayFrame((CCSpriteFrame *)_pAnimationFrames->objectAtIndex(0));


	//-----------------------
	// Original version returned void.  Changed by mdomowcicz.
	_iNumFrames = FramesList.GetNumEntries();
	return _iNumFrames;
}



/******************************************************************************
 FUNCTION:  getFrameNumber
******************************************************************************/
int SoSpriteAnim::GetFrameNumber( std::string sFrameName )
{
	// We use the last two digits before the ".extension" 
	// as the frame number.  (ie:  typical FrameName: "PlayerAnimation03.png")
	// would be '03', or 3, as the frame number.
	char* pacFrameName = (char*)sFrameName.c_str();
	char pacFrameNameNoExt[ 50 ];
	JpUtils::StripExtension( pacFrameNameNoExt, pacFrameName );

	int iIndex = strlen( pacFrameNameNoExt );
	iIndex -= 2;

	// We test to make sure the first digit is a number that is not 
	// zero (which would make it octal).  If it is, we ignore it.
	char acNumber[8];
	acNumber[ 0 ] = pacFrameNameNoExt[ iIndex ];
	if( acNumber[ 0 ] < 49   ||
		acNumber[ 0 ] > 57 )
	{
		iIndex++;
		acNumber[ 0 ] = pacFrameNameNoExt[ iIndex ];
	}

	acNumber[ 1 ] = pacFrameNameNoExt[ iIndex + 1 ];
	acNumber[ 2 ] = 0;

	int iNumber = JpString::ParseInt( acNumber );

	return iNumber;
}



/******************************************************************************
 FUNCTION:  addFrames
******************************************************************************/
void SoSpriteAnim::addFrames(CCArray *frames)
{
	_pAnimationFrames = frames;
	setDisplayFrame((CCSpriteFrame *)_pAnimationFrames->objectAtIndex(0));
}



/******************************************************************************
 FUNCTION:  addFrames
******************************************************************************/
void SoSpriteAnim::addFrames(CCArray *frames, int displayTextureIndex)
{
	_pAnimationFrames = frames;
	setDisplayFrame((CCSpriteFrame *)_pAnimationFrames->objectAtIndex(displayTextureIndex));
}



/******************************************************************************
 FUNCTION:  OnAnimationDone
******************************************************************************/
void SoSpriteAnim::OnAnimationDone()
{
	StopAnimation();

	//Raising Callback.
	if( _pParentObject != 0 && _pfnOnAnimationDoneCallback != 0 )
		(_pParentObject->*_pfnOnAnimationDoneCallback)();
}



/******************************************************************************
 FUNCTION:  update
******************************************************************************/
void SoSpriteAnim::update(float dt)
{
	_fTimer += dt;

	while ( _fTimer >= 0.0f ) 
	{
		_fTimer -= _fSecondsPerFrame;

		switch ( _CurrentState )
		{
			case STATE_WAIT:
			{
				break;
			}
			case STATE_PLAY_FORWARD:
			{
				// Cocos manages the update calls for these animations, and sometimes an update
				// is scheduled even after a stop has been signaled.  That extra update can cause
				// the index to go out of bounds, and hence this clamp.
				// mdomowicz 2013_02_23
				_iCurrentIndex = Clamp( _iCurrentIndex, 0, _iNumFrames - 1 );

				setDisplayFrame( (CCSpriteFrame *)_pAnimationFrames->objectAtIndex( _iCurrentIndex ) );

				_iCurrentIndex ++;


				if ( _iCurrentIndex > _iEndingIndex ) 
				{
					// When we're at the last frame, we must let the frame show
					// for the _fSecondsPerFrame duration before doing anything.
					// So here we switch states, knowing that the next state won't
					// be run until _fSecondsPerFrame seconds have passed, and we
					// do the "on last frame" logic in that next state instead of
					// here. mdomowicz 2013_05_22
					_CurrentState = STATE_PLAY_FORWARD_HANDLE_LAST_FRAME;
				}

				break;
			}

			case STATE_PLAY_FORWARD_HANDLE_LAST_FRAME:
			{
				if( _bPlayReversedWhenAtEnd )
				{
					// As of this writing, a lot of changes were made to the forward playing
					// code to fix a number of subtle playback errors.  Some matching changes were 
					// also made to the backwards playing code, but since backwards playback is not
					// yet needed, I have not tested it yet.  Given the subtle nature of the
					// program flow, I'm certain it does not work and will fixes.
					// mdomowicz 2013_05_22
					ASSERTF( 0, ( "\nThis code is not tested and very likely broken.  Seen comments." ) );


					_iCurrentIndex = ( _iEndingIndex - 1 );
					setDisplayFrame( (CCSpriteFrame *)_pAnimationFrames->objectAtIndex( _iCurrentIndex ) );
					_CurrentState = STATE_PLAY_BACKWARD;
				}
				else
				{
					_iCurrentLoopCount ++;

					if ( _iCurrentLoopCount == _iNumLoops )
					{
						OnAnimationDone();
						_CurrentState = STATE_WAIT;
					}
					else
					{
						_iCurrentIndex = _iStartingIndex;
						setDisplayFrame( (CCSpriteFrame *)_pAnimationFrames->objectAtIndex( _iCurrentIndex ) );
						_CurrentState = STATE_PLAY_FORWARD;
					}
				}

				break;
			}


			case STATE_PLAY_BACKWARD:
			{
				// As of this writing, a lot of changes were made to the forward playing
				// code to fix a number of subtle playback errors.  Some matching changes were 
				// also made to the backwards playing code, but since backwards playback is not
				// yet needed, I have not tested it yet.  Given the subtle nature of the
				// program flow, I'm certain it does not work and will fixes.
				// mdomowicz 2013_05_22
				//ASSERTF( 0, ( "\nThis code is not tested and very likely broken.  Seen comments." ) );

				_iCurrentIndex = Clamp( _iCurrentIndex, 0, _iNumFrames - 1 );
				setDisplayFrame( (CCSpriteFrame *)_pAnimationFrames->objectAtIndex( _iCurrentIndex ) );

				if ( _iCurrentIndex == _iEndingIndex ) 
				{
					_iCurrentLoopCount ++;

					if( _iCurrentLoopCount == _iNumLoops )
						OnAnimationDone();

					_CurrentState = STATE_PLAY_BACKWARD_HANDLE_LAST_FRAME;
				}

				_iCurrentIndex --;
				break;
			}


			case STATE_PLAY_BACKWARD_HANDLE_LAST_FRAME:
			{
				if( _bPlayReversedWhenAtEnd )
				{
					// As of this writing, a lot of changes were made to the forward playing
					// code to fix a number of subtle playback errors.  Some matching changes were 
					// also made to the backwards playing code, but since backwards playback is not
					// yet needed, I have not tested it yet.  Given the subtle nature of the
					// program flow, I'm certain it does not work and will fixes.
					// mdomowicz 2013_05_22
					ASSERTF( 0, ( "\nThis code is not tested and very likely broken.  Seen comments." ) );


					_iCurrentIndex = ( _iEndingIndex - 1 );
					setDisplayFrame( (CCSpriteFrame *)_pAnimationFrames->objectAtIndex( _iCurrentIndex ) );
					_CurrentState = STATE_PLAY_FORWARD;
				}
				else
				{
					_iCurrentLoopCount ++;

					if ( _iCurrentLoopCount == _iNumLoops )
					{
						OnAnimationDone();
						_CurrentState = STATE_WAIT;
					}
					else
					{
						_iCurrentIndex = _iStartingIndex;
						setDisplayFrame( (CCSpriteFrame *)_pAnimationFrames->objectAtIndex( _iCurrentIndex ) );
						_CurrentState = STATE_PLAY_BACKWARD;
					}
				}

				break;
			}


			default:
				ASSERTF( 0, ( "\nUnknown animation state" ) );
		}
	}
}



/******************************************************************************
 FUNCTION:  PlayAnimation
******************************************************************************/
void SoSpriteAnim::PlayAnimation()
{
	PlayAnimationRanged( 0, GetNumFrames() - 1 );
}



/******************************************************************************
 FUNCTION:  PlayAnimationRanged
******************************************************************************/
void SoSpriteAnim::PlayAnimationRanged( int iStartIndex, int iEndIndex )
{
	if( _bIsAnimationRunning )
	{
		StopAnimation();
	}

	//Assign values.
	_bIsAnimationRunning = true;

	_iStartingIndex = iStartIndex ;
	_iEndingIndex = iEndIndex;

	//Initilize values....
	_fTimer = 0.0f;
	_bIsForwardPlayback = true;
	_iCurrentIndex = _iStartingIndex;
	_iCurrentLoopCount = 0;
	_iIncrementValue = 0;

	_CurrentState = STATE_PLAY_FORWARD;


	// Because of thread execution order, we must set the
	// frame immediately, so that the correct image is used
	// even if it's shown before we get to the first true 
	// animation update.  mdomowicz 2013_05_22
	setDisplayFrame( (CCSpriteFrame *)_pAnimationFrames->objectAtIndex( _iCurrentIndex ) );

	scheduleUpdateWithPriority(-1);
	ResumeAnimation();
}



/******************************************************************************
 FUNCTION:  PlayAnimationReverse
******************************************************************************/
void SoSpriteAnim::PlayAnimationReverse()
{
	PlayAnimationReverseRanged( GetNumFrames() - 1, 0 );
}



/******************************************************************************
 FUNCTION:  PlayAnimationReverseRanged
******************************************************************************/
void SoSpriteAnim::PlayAnimationReverseRanged( int iStartIndex, int iEndIndex )
{
	if( _bIsAnimationRunning )
	{
		StopAnimation();
	}

	//Assign values.
	_bIsAnimationRunning = true;

	_iStartingIndex = iStartIndex;
	_iEndingIndex = iEndIndex;

	//Initilize values....
	_fTimer = 0.0f;
	_bIsForwardPlayback = false;
	_iCurrentIndex = _iStartingIndex;
	_iCurrentLoopCount = 0;
	_iIncrementValue = 0;

	_CurrentState = STATE_PLAY_BACKWARD;


	// Because of thread execution order, we must set the
	// frame immediately, so that the correct image is used
	// even if it's shown before we get to the first true 
	// animation update.  mdomowicz 2013_05_22
	setDisplayFrame( (CCSpriteFrame *)_pAnimationFrames->objectAtIndex( _iCurrentIndex ) );

	scheduleUpdateWithPriority(-1);
	ResumeAnimation();
}



/******************************************************************************
 FUNCTION:  StopAnimation
******************************************************************************/
void SoSpriteAnim::StopAnimation()
{
	_bIsAnimationRunning = false;
	unscheduleUpdate();
}



/******************************************************************************
 FUNCTION:  SetNumFramesPerSecond
******************************************************************************/
void SoSpriteAnim::SetNumFramesPerSecond( int i )
{
	if( i != -1 )
		_fSecondsPerFrame = calculateFrameRate( i );
}



/******************************************************************************
 FUNCTION:  IsStopped
******************************************************************************/
bool SoSpriteAnim::IsStopped()
{
	return ( _bIsAnimationRunning == false );
}



/******************************************************************************
 FUNCTION:  HasCompleted
******************************************************************************/
bool SoSpriteAnim::HasCompleted()
{
	return
	( 
		_iCurrentLoopCount == _iNumLoops && 
		_bIsAnimationRunning == false 
	);
}


/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoSpriteAnim* SoSpriteAnim::create()
{
	SoSpriteAnim* pRet = new SoSpriteAnim();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}
