//
//  SoSpriteAnim.h
//  FlyingFreak
//
//  Created by varadharaj on 16/05/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
//  Updated for cocos2d-2.0-x-2.0.2 by Francis Styck
//


/*
	This class is based off of public domain code that was posted here: 
		http://www.cocos2d-x.org/boards/6/topics/1792

	Tweaks were needed to work with this specific version of Cocos2d-x.
	Comments are added where changes were made.

	Update: this version is pretty heavily reworked from the original.  The goal
	was to fix bugs and improve the logic to make it easier to follow/use.

	This class requires sprite sheet and sprite .plist files that are created
	in an external tool.  The tool I used is called TexturePacker and can be
	found here:
		http://www.codeandweb.com/texturepacker


	Below is sample code for how to use this class in this app.


			//-----------------------
			// Do this when creating the sprite animation.

			SoSpriteAnim* _pAdvanceSprite;
			_pAdvanceSprite = SoSpriteAnim::create();
			_pAdvanceSprite->retain();
			_pAdvanceSprite->init();    // Must call init for latest version cocos2d-x

			_pAdvanceSprite->setPosition(ccp(size.width/2,size.height/2));
			addChild(_pAdvanceSprite);

			_pAdvanceSprite->addFrames
			(
				"Textures\\TestAnim.plist",
				"Textures\\TestAnim.png"
			);

			_pAdvanceSprite->SetParentObject( _pParentNode );
			_pAdvanceSprite->SetNumLoops( -1 );
			_pAdvanceSprite->SetNumFramesPerSecond( 10 );
			_pAdvanceSprite->PlayAnimation();


			//-----------------------
			// You can tweak the sprite and the animation changes

			_pAdvanceSprite->setScaleX( 0.5f );


			//-----------------------
			// Do this when destroying the sprite animation

			pTheParentNode->removeChild( _pAdvanceSprite, DCG_CLEANUP_ALL_RUNNING_ACTIONS );

			if ( _pAdvanceSprite )
			{
				_pAdvanceSprite->release();
				_pAdvanceSprite = NULL;
			}


	mdomowicz 2012_09_29 - original notes
    mdomowicz 2013_02_22 - updated notes.



	Note also that there is a bug in addFrames that I had to fix by adding
	a SoFrameNumber value to every frame in the animation's .plist
	See ::addFrames for more details on this fix, and what must be added
	to the .plist, and why.
	mdomowicz 2012_10_15

*/

#pragma once

#include "cocos2d.h"


#include "JpHashTable.h"

using namespace cocos2d;


#define SoSpriteAnimTable64 JpHashTable< SoSpriteAnim, 64 >


class SoSpriteAnim : public CCSprite
{
	public:
		//-----------------------
		// public functions
		SoSpriteAnim();
		virtual ~SoSpriteAnim();

		// Add Frames to run Animation.
		// See implementation for notes on bug fixes and customizations to addFrames
		int addFrames(const char* pList, const char* textureFileName);

		int GetNumFrames() { return _iNumFrames; }
		int GetFrameNumber( std::string sFrameName );

		void SetParentObject( CCObject* p ) { _pParentObject = p; }
		void SetNumLoops( int i ) { _iNumLoops = i; }
		void SetNumFramesPerSecond( int i );
		void SetOnAnimationDoneCallback( SEL_CallFunc pFunction ) { _pfnOnAnimationDoneCallback = pFunction; }
		void SetPlayReversedWhenAtEnd( bool b ) { _bPlayReversedWhenAtEnd = b; }

		void PlayAnimation();
		void PlayAnimationRanged( int iStartIndex, int iEndIndex );

		void PlayAnimationReverse();
		void PlayAnimationReverseRanged( int iStartIndex, int iEndIndex );

		void PauseAnimation() { pauseSchedulerAndActions(); }
		void ResumeAnimation() { resumeSchedulerAndActions(); }
		void StopAnimation();

		bool IsStopped();
		bool HasCompleted();

		CCSpriteFrame* displayFrame(void) { return CCSprite::displayFrame(); }

		static SoSpriteAnim* create();


	private:
		//-----------------------
		// private variables

		CCArray* _pAnimationFrames;

		//Indexing variable.
		int _iStartingIndex;
		int _iEndingIndex;
		int _iCurrentIndex;
		int _iIncrementValue;

		//Looping variable.
		int _iCurrentLoopCount;
		int _iNumLoops;

		//Callback function pointer.
		SEL_CallFunc _pfnOnAnimationDoneCallback;
		CCObject* _pParentObject;


		float _fSecondsPerFrame;
		float _fTimer;

		bool _bPlayReversedWhenAtEnd;
		bool _bIsForwardPlayback;
		bool _bIsPlistLoaded;
		bool _bIsAnimationRunning;

		int _iNumFrames;
		int _CurrentState;


		#ifdef _DEBUG
			static JpHashTable< int, 4096 > _DebugFrameNameCollisionChecker;
		#endif

		//-----------------------
		// private functions
		float calculateFrameRate( int number_frames ){ return 1.0f / number_frames; }

		virtual void update(float dt);

		void OnAnimationDone();


		// The original code had these public, but since I don't know what their best use
		// case is, and since the addFrames we use now seem good enough, I'm going to
		// hide these away for now.
		// mdomowicz 2013_02_22
		void addFrames(const char* pList); // Has Texture file name in pList.
		void addFrames(CCArray *frames);
		void addFrames(CCArray *frames, int displayTextureIndex);
		void addSpriteFramesWithFile(const char* plist, const char* textureFileName);
};
