/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "NeCsvIn.h"
#include "SoSpriteAnimSet.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoSpriteAnimSet
******************************************************************************/
SoSpriteAnimSet::SoSpriteAnimSet() :
_pParentNode( NULL ),
_pCurrentAnim( NULL ),
_iRenderZOrder( 0 ),
_vScale( 1.0f, 1.0f ),
_bIsFlippedInDirX( false ),
_bIsFlippedInDirY( false )
{
}



/******************************************************************************
 FUNCTION:  ~SoSpriteAnimSet
******************************************************************************/
SoSpriteAnimSet::~SoSpriteAnimSet()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoSpriteAnimSet::Init( CCNode* pParentNode )
{
	_pParentNode = pParentNode;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoSpriteAnimSet::Deinit()
{
	SoSpriteAnimTable64::Iterator Iter( &_SpriteAnimTable );
	while ( Iter.Next() )
	{
		SoSpriteAnim* pAnim = Iter.GetData();
		SoUtils::RemoveAndDestroyNode( _pParentNode, pAnim );
	}

	_SpriteAnimTable.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoSpriteAnimSet::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  Play
******************************************************************************/
void SoSpriteAnimSet::Play( const char* pacAnimTableKey )
{
	SoSpriteAnim* pAnimToPlay = _SpriteAnimTable.Find( JpHash( pacAnimTableKey ) );
	ASSERTF( pAnimToPlay, ( "\nCan't find requested animation." ) )
	
	if 
	( 
		pAnimToPlay && 
		(
			pAnimToPlay != _pCurrentAnim ||
			CurrentAnimIsStopped()
		)
	)
	{
		if ( _pCurrentAnim )
		{
			_pCurrentAnim->StopAnimation();
			_pCurrentAnim->setVisible( false );
		}

		SoUtils::SetRenderZOrder( _pParentNode, pAnimToPlay, _iRenderZOrder );
		pAnimToPlay->setPositionX( _vPosition.X );
		pAnimToPlay->setPositionY( _vPosition.Y );
		pAnimToPlay->setScaleX( _vScale.X );
		pAnimToPlay->setScaleY( _vScale.Y );
		pAnimToPlay->setFlipX( _bIsFlippedInDirX );
		pAnimToPlay->setFlipY( _bIsFlippedInDirY );

		pAnimToPlay->PlayAnimation();
		pAnimToPlay->setOpacity( _fOpacity * 255.0f );
		pAnimToPlay->setVisible( true );

		_pCurrentAnim = pAnimToPlay;
	}
}



/******************************************************************************
 FUNCTION:  PlayReverse
******************************************************************************/
void SoSpriteAnimSet::PlayReverse( const char* pacAnimTableKey )
{
	SoSpriteAnim* pAnimToPlay = _SpriteAnimTable.Find( JpHash( pacAnimTableKey ) );
	ASSERTF( pAnimToPlay, ( "\nCan't find requested animation." ) )
	
	if 
	( 
		pAnimToPlay && 
		(
			pAnimToPlay != _pCurrentAnim ||
			CurrentAnimIsStopped()
		)
	)
	{
		if ( _pCurrentAnim )
		{
			_pCurrentAnim->StopAnimation();
			_pCurrentAnim->setVisible( false );
		}

		SoUtils::SetRenderZOrder( _pParentNode, pAnimToPlay, _iRenderZOrder );
		pAnimToPlay->setPositionX( _vPosition.X );
		pAnimToPlay->setPositionY( _vPosition.Y );
		pAnimToPlay->setScaleX( _vScale.X );
		pAnimToPlay->setScaleY( _vScale.Y );
		pAnimToPlay->setFlipX( _bIsFlippedInDirX );
		pAnimToPlay->setFlipY( _bIsFlippedInDirY );

		pAnimToPlay->PlayAnimationReverse();
		pAnimToPlay->setOpacity( _fOpacity * 255.0f );
		pAnimToPlay->setVisible( true );

		_pCurrentAnim = pAnimToPlay;
	}
}



/******************************************************************************
 FUNCTION:  SetPosition
******************************************************************************/
void SoSpriteAnimSet::SetPosition( const JpVector2f& v )
{
	_vPosition = v;
	if ( _pCurrentAnim )
	{
		_pCurrentAnim->setPositionX( v.X );
		_pCurrentAnim->setPositionY( v.Y );
	}
}



/******************************************************************************
 FUNCTION:  SetScale
******************************************************************************/
void SoSpriteAnimSet::SetScale( float f ) 
{ 
	_vScale.Init( f, f ); 
	if ( _pCurrentAnim )
		_pCurrentAnim->setScale( f );
}



/******************************************************************************
 FUNCTION:  SetScaleX
******************************************************************************/
void SoSpriteAnimSet::SetScaleX( float f ) 
{ 
	_vScale.X = f;
	if ( _pCurrentAnim )
		_pCurrentAnim->setScaleX( f );
}



/******************************************************************************
 FUNCTION:  SetScaleY
******************************************************************************/
void SoSpriteAnimSet::SetScaleY( float f ) 
{ 
	_vScale.Y = f;
	if ( _pCurrentAnim )
		_pCurrentAnim->setScaleY( f );
}



/******************************************************************************
 FUNCTION:  SetOpacity
******************************************************************************/
void SoSpriteAnimSet::SetOpacity( float fPercent ) 
{ 
	_fOpacity = fPercent;

	if ( _pCurrentAnim )
		_pCurrentAnim->setOpacity( fPercent * 255.0f );
}



/******************************************************************************
 FUNCTION:  SetRenderZOrder
******************************************************************************/
void SoSpriteAnimSet::SetRenderZOrder( int iZWhereLargerMeansCloser ) 
{ 
	_iRenderZOrder = iZWhereLargerMeansCloser;

	if ( _pCurrentAnim )
		SoUtils::SetRenderZOrder( _pParentNode, _pCurrentAnim, _iRenderZOrder );
}



/******************************************************************************
 FUNCTION:  SetFlipInDirX
******************************************************************************/
void SoSpriteAnimSet::SetFlipInDirX( bool b ) 
{ 
	_bIsFlippedInDirX = b;

	if ( _pCurrentAnim )
		_pCurrentAnim->setFlipX( b );
}



/******************************************************************************
 FUNCTION:  SetFlipInDirY
******************************************************************************/
void SoSpriteAnimSet::SetFlipInDirY( bool b ) 
{ 
	_bIsFlippedInDirY = b;

	if ( _pCurrentAnim )
		_pCurrentAnim->setFlipY( b );
}



/******************************************************************************
 FUNCTION:  StopCurrentAnimation
******************************************************************************/
void SoSpriteAnimSet::StopCurrentAnimation()
{
	if ( _pCurrentAnim )
		return _pCurrentAnim->StopAnimation();
	
}



/******************************************************************************
 FUNCTION:  CurrentAnimIsStopped
 NOTES:     This is a weaker version of "HasCompleted", as it returns true
            as long as the anim is not running, and does not care if the
            animation got to the end first or not.
******************************************************************************/
bool SoSpriteAnimSet::CurrentAnimIsStopped()
{
	if ( _pCurrentAnim )
		return _pCurrentAnim->IsStopped();

	return true;
}



/******************************************************************************
 FUNCTION:  CurrentAnimHasCompleted
 NOTES:     "Has completed" means that the animation has both stopped and
            gotten to the end of the animation, including all required loops.
******************************************************************************/
bool SoSpriteAnimSet::CurrentAnimHasCompleted()
{
	if ( _pCurrentAnim )
		return _pCurrentAnim->HasCompleted();

	return true;
}



/******************************************************************************
 FUNCTION:  LoadAnimFilesByCsv
******************************************************************************/
void SoSpriteAnimSet::LoadAnimFilesByCsv( const char* pacCsvPathAndFilename )
{
	ASSERTF( _pParentNode, ( "\nCan't load csv.  Make sure you called the init and set the parent node!" ) );

	if ( _pParentNode )
	{
		NeCsvIn CsvReader;
		CsvReader.ReadCsvFile( pacCsvPathAndFilename );
		CsvReader.MakeHashIndicesFromColValuesAtRow( 0 );

		for ( int i = 1; i < CsvReader.GetNumRows(); i ++ )
		{
			//-----------------------
			// read the data from the csv
			char* pacHashTableKey = CsvReader.GetStringByIntAndHash( i, 0x4287ff69 /* "sKey" */ );
			char* pacFilenameNoExt = CsvReader.GetStringByIntAndHash( i, 0xa2777f89 /* "sFilenameNoExtension" */ );
			int iFramesPerSecond = CsvReader.GetIntByIntAndHash( i, 0x3677c97d /* "iFramesPerSecond" */, -1 );
			bool bLoop = CsvReader.GetBoolByIntAndHash( i, 0xd9ee8d8b /* "bLoop" */, false );

			//-----------------------
			// create a new animation
			std::string sFilenameBase = pacFilenameNoExt;
			std::string sSpriteFilename = sFilenameBase + ".png";
			std::string sPlistFilename = sFilenameBase + ".plist";
			std::string sSpritePathAndFilename = SoUtils::AddRelativePathToAnimation( sSpriteFilename.c_str() );
			std::string sPlistPathAndFilename = SoUtils::AddRelativePathToAnimation( sPlistFilename.c_str() );
			SoSpriteAnim* pNewAnim = SoUtils::CreateSpriteAnim( _pParentNode, sSpritePathAndFilename.c_str(), sPlistPathAndFilename.c_str() );


			//-----------------------
			// set animation options, from the csv data
			if ( bLoop )
				pNewAnim->SetNumLoops( -1 );
			else
				pNewAnim->SetNumLoops( 1 );

			pNewAnim->SetNumFramesPerSecond( iFramesPerSecond );
			pNewAnim->setVisible( false );

			//-----------------------
			_SpriteAnimTable.Store( JpHash( pacHashTableKey ), pNewAnim );
		}
	}
}
