/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoSprite.h"
#include "SoSpritePlus.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoSpritePlus
******************************************************************************/
SoSpritePlus::SoSpritePlus() :
_pParentNode( NULL ),
_pSprite( NULL ),
_fTimer( 0.0f ),
_eType( TYPE_LAST ),
_eState( STATE_LAST ),
_iRenderZOrder( 0 ),
_fTransitionSpeed( 1.0f ),
_vAnchorPoint( 0.5f, 0.5f ),
_vScale( 1.0f, 1.0f )
{
}



/******************************************************************************
 FUNCTION:  ~SoSpritePlus
******************************************************************************/
SoSpritePlus::~SoSpritePlus()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoSpritePlus::Init
(
	CCNode* pParentNode, 
	const char* pacPathAndFileName,
	bool bDelaySpriteCreationUntilNeeded
)
{
	_pParentNode = pParentNode;
	_sSpritePathAndFileName = pacPathAndFileName;

	if ( !bDelaySpriteCreationUntilNeeded )
	{
		_pSprite = SoUtils::CreateSprite( _pParentNode, pacPathAndFileName );
	}

	_eState = STATE_INIT;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoSpritePlus::Deinit()
{
	if ( _pSprite )
	{
		SoUtils::RemoveAndDestroyNode( _pParentNode, _pSprite );
		_pSprite = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoSpritePlus::Update( float dt )
{
	if ( _pSprite )
	{
		switch ( _eType )
		{
			case TYPE_FADE_IN_OUT:
				UpdateFadeInOut( dt );
				break;
			case TYPE_POP_UP_VIDEO_STYLE:
				UpdatePopupVideoStyle( dt );
				break;
			default:
				ASSERTF( 0, ( "\nUnknown SpritePlus type." ) );
		}
	}
}



/******************************************************************************
 FUNCTION:  UpdateFadeInOut
******************************************************************************/
void SoSpritePlus::UpdateFadeInOut( float dt )
{
	_fTimer += dt;

	_pSprite->setAnchorPoint( ccp( _vAnchorPoint.X, _vAnchorPoint.Y ) );
	_pSprite->setScaleX( _vScale.X );
	_pSprite->setScaleY( _vScale.Y );

	switch ( _eState )
	{
		case STATE_INIT:
		{
			_pSprite->setOpacity( 0 );
			break;
		}
		case STATE_TRANSITION_IN:
		{
			_pSprite->setVisible( true );

			float fAlpha = _fTimer * _fTransitionSpeed;
			fAlpha = Clamp( fAlpha, 0.0, 1.0f );
			_pSprite->setOpacity( fAlpha * 255.0f );
			if ( fAlpha == 1.0f )
			{
				_fTimer = 0.0f;
				_eState = STATE_SHOW;
			}

			break;
		}
		case STATE_SHOW:
		{
			break;
		}
		case STATE_TRANSITION_OUT:
		{
			float fAlpha = ( 1.0f - ( _fTimer * _fTransitionSpeed ) );
			fAlpha = Clamp( fAlpha, 0.0, 1.0f );
			_pSprite->setOpacity( fAlpha * 255.0f );
			if ( fAlpha == 0.0f )
			{
				_fTimer = 0.0f;
				_eState = STATE_HIDE;
			}
			break;
		}
		case STATE_HIDE:
		{
			_pSprite->setVisible( false );
			break;
		}
		default:
			ASSERTF( 0, ( "\nUnknown transition state." ) );
	}
}



/******************************************************************************
 FUNCTION:  UpdatePopupVideoStyle
******************************************************************************/
void SoSpritePlus::UpdatePopupVideoStyle( float dt )
{
	_fTimer += dt;

	_pSprite->setAnchorPoint( ccp( _vAnchorPoint.X, _vAnchorPoint.Y ) );
	_pSprite->setScaleX( _vScale.X );
	_pSprite->setScaleY( _vScale.Y );

	switch ( _eState )
	{
		case STATE_INIT:
		{
			_pSprite->setOpacity( 0 );
			_pSprite->setScale( 0.0f );
			break;
		}
		case STATE_TRANSITION_IN:
		{
			_pSprite->setVisible( true );

			float fAlpha = _fTimer * _fTransitionSpeed * 12.0f;
			fAlpha = Clamp( fAlpha, 0.0, 1.0f );
			_pSprite->setOpacity( fAlpha * 255.0f );

			float fTargetScale = 1.0f;
			float fMarginTimer = Clamp( _fTimer * _fTransitionSpeed * 10.0f, 0.0f, JpUtils::_fPi );
			float fMarginSize = 0.04f * SoUtils::GetiPadRatioX() * SoUtils::GetSpriteZoom();
			float fMargin = sinf( fMarginTimer ) * fMarginSize;

			float fScaleX = _vScale.X + fMargin;
			_pSprite->setScaleX( fScaleX );
			
			float fScaleY = _vScale.Y + fMargin;
			_pSprite->setScaleY( fScaleY );

			if ( fMarginTimer == JpUtils::_fPi )
			{
				_fTimer = 0.0f;
				_eState = STATE_SHOW;
			}

			break;
		}
		case STATE_SHOW:
		{
			break;
		}
		case STATE_TRANSITION_OUT:
		{
			float fAlpha = ( 1.0f - ( _fTimer * _fTransitionSpeed * 2.0f ) );
			fAlpha = Clamp( fAlpha, 0.0, 1.0f );
			_pSprite->setOpacity( fAlpha * 255.0f );

			float fTargetScale = 1.0f;
			float fMarginTimer = Clamp( _fTimer * _fTransitionSpeed * 10.0f, 0.0f, JpUtils::_fPi );
			float fMarginSize = 0.04f * SoUtils::GetiPadRatioX() * SoUtils::GetSpriteZoom();
			float fMargin = sinf( fMarginTimer ) * fMarginSize;

			float fScaleX = _vScale.X + fMargin;
			_pSprite->setScaleX( fScaleX );
			
			float fScaleY = _vScale.Y + fMargin;
			_pSprite->setScaleY( fScaleY );


			if ( fAlpha == 0.0f )
			{
				_fTimer = 0.0f;
				_eState = STATE_HIDE;
			}
			break;
		}
		case STATE_HIDE:
		{
			_pSprite->setVisible( false );
			break;
		}
		default:
			ASSERTF( 0, ( "\nUnknown transition state." ) );
	}
}



/******************************************************************************
 FUNCTION:  TransitionIn
******************************************************************************/
void SoSpritePlus::TransitionIn()
{
	_fTimer = 0.0f;
	_eState = STATE_TRANSITION_IN;

	if ( !_pSprite )
	{
		_pSprite = SoUtils::CreateSprite( _pParentNode, _sSpritePathAndFileName.c_str() );
		SetRenderZOrder( _iRenderZOrder );
	}
}



/******************************************************************************
 FUNCTION:  TransitionOut
******************************************************************************/
void SoSpritePlus::TransitionOut()
{
	_fTimer = 0.0f;
	_eState = STATE_TRANSITION_OUT;
}



/******************************************************************************
 FUNCTION:  SetPosition
******************************************************************************/
void SoSpritePlus::SetPosition( const JpVector2f& v )
{
	if ( _pSprite )
		_pSprite->setPosition( ccp(v.X, v.Y ) );
}



/******************************************************************************
 FUNCTION:  SetRenderZOrder
******************************************************************************/
void SoSpritePlus::SetRenderZOrder( int iZWhereLargerMeansCloser )
{
	_iRenderZOrder = iZWhereLargerMeansCloser;
	if ( _pSprite )
		SoUtils::SetRenderZOrder( _pParentNode, _pSprite, iZWhereLargerMeansCloser );
}
