/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"


USING_NS_CC;


class SoSprite;


class SoSpritePlus
{
	public:

		//-----------------------
		// public enums
		enum TYPE
		{
			TYPE_FADE_IN_OUT,
			TYPE_POP_UP_VIDEO_STYLE,
			TYPE_LAST
		};

		enum STATE
		{
			STATE_INIT,
			STATE_TRANSITION_IN,
			STATE_SHOW,
			STATE_TRANSITION_OUT,
			STATE_HIDE,
			STATE_LAST
		};


		//-----------------------
		// public functions

		SoSpritePlus();

		virtual ~SoSpritePlus();

		void Init
		(
			CCNode* pParentNode, 
			const char* pacPathAndFileName,
			bool bDelaySpriteCreationUntilNeeded
		);

		void Deinit();

		void Update( float dt );

		SoSprite* GetSoSprite() { return _pSprite; }

		void SetType( TYPE e ) { _eType = e; }

		void SetPosition( const JpVector2f& v );
		void SetRenderZOrder( int iZWhereLargerMeansCloser );

		void TransitionIn();
		void TransitionOut();

		void SetTransitionSpeed( float f ) { _fTransitionSpeed = f; }
		bool IsTransitionInCompleted() { return ( _eState == STATE_SHOW ); }
		bool IsTransitionOutCompleted() { return ( _eState == STATE_HIDE ); }

		void SetAnchorPoint( float fX, float fY ) { _vAnchorPoint.Init( fX, fY ); }

		void SetScale( float f ) { _vScale.Init( f, f ); }
		void SetScaleX( float f ) { _vScale.X = f; }
		void SetScaleY( float f ) { _vScale.Y = f; }

	private:

		//-----------------------
		// private variables

		CCNode* _pParentNode;
		SoSprite* _pSprite;

		float _fTimer;

		TYPE _eType;
		STATE _eState;

		std::string _sSpritePathAndFileName;
		int _iRenderZOrder;

		float _fTransitionSpeed;

		JpVector2f _vAnchorPoint;
		JpVector2f _vScale;

		//-----------------------
		// private functions
		void UpdateFadeInOut( float dt );
		void UpdatePopupVideoStyle( float dt );
};




