/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoActorManager.h"
#include "SoActorVisible.h"
#include "SoSaveSystem.h"
#include "SoTextureUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;

#if DCG_IS_TARGET_IOS
	float SoTextureUtils::_fTextureScaler = 1.0f;
#else
	float SoTextureUtils::_fTextureScaler = 1.0f;
#endif

int   SoTextureUtils::_iTextureFilter = 1;



/******************************************************************************
 FUNCTION:  ScaleImage
			
			This function scales a .png, and stores it as a CCImage.
		    Note: It works best (or maybe just 'only works') with 'even-multiple' scales.
		    ( Ie 0.5, 0.25, 0.125, etc ( which is 1/2, 1/4, 1/8, etc )
******************************************************************************/
CCImage* SoTextureUtils::ScaleImage( const char *pszSpriteFrameName, float fScale )
{
	unsigned long ulSize = 0;
	unsigned char* pSource = CCFileUtils::sharedFileUtils()->getFileData( pszSpriteFrameName, "rb", &ulSize );

	CCImage* pSourceImg = new CCImage;
	pSourceImg->initWithImageData( (void*)pSource, ulSize, CCImage::kFmtPng );
	
	#if DCG_IS_TARGET_IOS
		return pSourceImg;
	#endif

	if ( fScale == 1.0f )
		return pSourceImg;

	int iNumChannels = 4;

	int iOldWidth = pSourceImg->getWidth();
	int iOldHeight = pSourceImg->getHeight();

	int iNewWidth  = (int)( floor( ( float )iOldWidth * fScale ) );
	int iNewHeight = (int)( floor( ( float )iOldHeight * fScale ) );
	
	int iNewLengthInPixels = iNewWidth * iNewHeight;
	int iNewLengthInBytes = iNewLengthInPixels * iNumChannels; // 32 bit RGBA Png's for now

	unsigned char* pNewData = (unsigned char*)malloc( iNewLengthInBytes );  
	unsigned char* pOldData = pSourceImg->getData();

	float fInvScale = 1.0f / fScale;

	for ( int y = 0; y < iNewHeight; y ++ )
	{
		for ( int x = 0; x < iNewWidth; x ++ )
		{
			int iNewDataIndex = ( y * iNewWidth ) + x;
			iNewDataIndex *= 4;

			float fOldDataIndex = floor( ( (float)y * fInvScale * (float)iNewWidth * fInvScale ) + ( (float)x * fInvScale ) );		
			fOldDataIndex *= 4.0f;

			int iOldDataIndex = (int)fOldDataIndex;

			pNewData[ iNewDataIndex ] = pOldData[ iOldDataIndex ];
			pNewData[ iNewDataIndex + 1 ] = pOldData[ iOldDataIndex + 1 ];
			pNewData[ iNewDataIndex + 2 ] = pOldData[ iOldDataIndex + 2 ];
			pNewData[ iNewDataIndex + 3 ] = pOldData[ iOldDataIndex + 3 ];
		}
	}

	CCImage* pNewImage = new CCImage;
	pNewImage->initWithImageData( pNewData, iNewLengthInBytes, CCImage::kFmtRawData, iNewWidth, iNewHeight, 8 );

	pSourceImg->release();
	free( pNewData );

	return pNewImage;
}



/******************************************************************************
 FUNCTION:  ScaleImageBicubic
			NOTE:  This function does not work!  It is a work in progress.
			It seems to cut off the bottom portion of some sprites and/or 
			spritesheets.
			This function scales an image using bicubic interpolation.	
			Based on code from these pages:
			http://stackoverflow.com/questions/15176972/bi-cubic-interpolation-algorithm-for-image-scaling
			http://code.google.com/a/eclipselabs.org/p/bicubic-interpolation-image-processing/source/browse/trunk/libimage.c
******************************************************************************/
CCImage* SoTextureUtils::ScaleImageBicubic( const char *pszSpriteFrameName, float fScale )
{
	unsigned long ulSize = 0;
	unsigned char* pSource = CCFileUtils::sharedFileUtils()->getFileData( pszSpriteFrameName, "rb", &ulSize );

	CCImage* pSourceImg = new CCImage;
	pSourceImg->initWithImageData( (void*)pSource, ulSize, CCImage::kFmtPng );
	
	#if DCG_IS_TARGET_IOS
		return pSourceImg;
	#endif

	if ( fScale == 1.0f )
		return pSourceImg;

	int iNumChannels = 4;

	int iOldWidth = pSourceImg->getWidth();
	int iOldHeight = pSourceImg->getHeight();

	int iNewWidth  = (int)( floor( ( float )iOldWidth * fScale ) );
	int iNewHeight = (int)( floor( ( float )iOldHeight * fScale ) );
	
	int iNewLengthInPixels = iNewWidth * iNewHeight;
	int iNewLengthInBytes = iNewLengthInPixels * iNumChannels; // 32 bit RGBA Png's for now

	unsigned char* pNewData = (unsigned char*)malloc( iNewLengthInBytes );  
	unsigned char* pOldData = pSourceImg->getData();

	const float tx = (float)iOldWidth / (float)iNewWidth;
	const float ty = (float)iOldHeight / (float)iNewHeight;

	const int iRowStride = iNewWidth * iNumChannels;

	unsigned char C[5] = { 0 };

	//Almost works
	for (int i = 0; i < iNewHeight; i++ )
	{
		for (int j = 0; j < iNewWidth; j++ )
		{
			const int x = int(tx * (float)j);
			const int y = int(ty * (float)i);
			const float dx = tx * (float)j - (float)x;
			const float dy = ty * (float)i - (float)y;

			for (int k = 0; k < iNumChannels; k++ )
			{
				for (int jj = 0; jj < 4; jj++ )
				{
					const int z = y - 1 + jj;
					unsigned char a0 = GetPixelComponent( pOldData, z, x, iOldWidth, iOldHeight, k );
					unsigned char d0 = GetPixelComponent( pOldData, z, x - 1, iOldWidth, iOldHeight, k ) - a0;
					unsigned char d2 = GetPixelComponent( pOldData, z, x + 1, iOldWidth, iOldHeight, k ) - a0;
					unsigned char d3 = GetPixelComponent( pOldData, z, x + 2, iOldWidth, iOldHeight, k ) - a0;
					unsigned char a1 = (unsigned char)( -1.0f / 3.0f * (float)d0 + (float)d2 - 1.0f / 6.0f * (float)d3 );
					unsigned char a2 = (unsigned char)(  1.0f / 2.0f * (float)d0 + 1.0f / 2.0f * (float)d2);
					unsigned char a3 = (unsigned char)( -1.0f / 6.0f * (float)d0 - 1.0f / 2.0f * (float)d2 + 1.0f / 6.0f * (float)d3 );
					C[jj] = a0 + a1 * dx + a2 * dx * dx + a3 * dx * dx * dx;

					d0 = C[0] - C[1];
					d2 = C[2] - C[1];
					d3 = C[3] - C[1];
					a0 = C[1];
					a1 = (unsigned char)( -1.0f / 3.0f * (float)d0 + (float)d2 -1.0f / 6.0f * (float)d3 );
					a2 = (unsigned char)(  1.0f / 2.0f * (float)d0 + 1.0f / 2.0f * (float)d2 );
					a3 = (unsigned char)( -1.0f / 6.0f * (float)d0 - 1.0f / 2.0f * (float)d2 + 1.0f / 6.0f * (float)d3 );
					pNewData[ i * iRowStride + j * iNumChannels + k ] = a0 + a1 * dy + a2 * dy * dy + a3 * dy * dy * dy;
				}
			}
		}
	}

	// The following is another way of doing it

/*
	unsigned char Ca,Cb,Cc;
	unsigned char d0,d2,d3,a0,a1,a2,a3;
	int i,j,k,ii,jj;
	int x,y;
	float dx,dy;
	const int iOldRowStride = iOldWidth * iNumChannels;


	for ( i=0; i < iNewHeight; i++ )
		for ( j=0; j < iNewWidth; ++j )
		{
			x = (int)(tx*j);
			y =(int)(ty*i);

			dx= tx*j-x;
			dy=ty*i -y;

			for(k=0;k<iNumChannels;k++)
			{
				for(jj=0;jj<=3;jj++)
				{
// 					if ( y > 0 )
					{
						int ia0 = (y-1+jj)*iOldRowStride + (x)  *iNumChannels + k;
						int id0 = (y-1+jj)*iOldRowStride + (x-1)*iNumChannels + k;
						int id2 = (y-1+jj)*iOldRowStride + (x+1)*iNumChannels + k;
						int id3 = (y-1+jj)*iOldRowStride + (x+2)*iNumChannels + k;
						int iNum = (y-1+jj)*iOldRowStride + (x)*iNumChannels + k;
						d0 = pOldData[ id0 ] - pOldData[ iNum ];
						d2 = pOldData[ id2 ] - pOldData[ iNum ];
						d3 = pOldData[ id3 ] - pOldData[ iNum ];
						a0 = pOldData[ ia0 ];
						a1 =  -1.0/3*d0 + d2 -1.0/6*d3;
						a2 = 1.0/2*d0 + 1.0/2*d2;
						a3 = -1.0/6*d0 - 1.0/2*d2 + 1.0/6*d3;
						C[jj] = a0 + a1*dx + a2*dx*dx + a3*dx*dx*dx;

						d0 = C[0]-C[1];
						d2 = C[2]-C[1];
						d3 = C[3]-C[1];
						a0=C[1];
						a1 =  -1.0/3*d0 + d2 -1.0/6*d3;
						a2 = 1.0/2*d0 + 1.0/2*d2;
						a3 = -1.0/6*d0 - 1.0/2*d2 + 1.0/6*d3;
						Cc = a0 + a1*dy + a2*dy*dy + a3*dy*dy*dy;
						// if((int)Cc>255) Cc=255;
						//                 if((int)Cc<0) Cc=0;
						pNewData[ i * iRowStride +j * iNumChannels + k ] = Cc;
					}
				}
			}

		}
*/

	CCImage* pNewImage = new CCImage;
	pNewImage->initWithImageData( pNewData, iNewLengthInBytes, CCImage::kFmtRawData, iNewWidth, iNewHeight, 8 );

	pSourceImg->release();
	free( pNewData );

	return pNewImage;
}



/******************************************************************************
 FUNCTION:  AddUiImageScaled
			
			This function is a custom version of Cocos' CCTextureCache::addUiImage() 
			that works with scaled textures.
******************************************************************************/
CCTexture2D* SoTextureUtils::AddUiImageScaled( const char *pszSpriteFrameName, float fScale )
{
	CCTexture2D * texture = NULL;
	// textureForKey() use full path,so the key should be full path
	std::string forKey;
	if ( pszSpriteFrameName )
	{
		forKey = CCFileUtils::sharedFileUtils()->fullPathFromRelativePath( pszSpriteFrameName );
	}

	// Don't have to lock here, because addImageAsync() will not 
	// invoke opengl function in loading thread.
	do 
	{
		// If key is nil, then create a new texture each time
		if( pszSpriteFrameName && (texture = 
			(CCTexture2D *)CCTextureCache::sharedTextureCache()->m_pTextures->objectForKey(forKey.c_str())))
		{
			break;
		}

		//CCImage *pImage = ScaleImageBicubic( pszSpriteFrameName, fScale );    
		CCImage *pImage = ScaleImage( pszSpriteFrameName, fScale );    

		texture = new CCTexture2D();
		texture->initWithImage( pImage, kCCResolutionUnknown );

		ccTexParams texParams;
		
		switch ( SoTextureUtils::GetTextureFilter() )
		{
			case 0:
				texParams.minFilter = GL_NEAREST;
				texParams.magFilter = GL_NEAREST;
				texParams.wrapS     = GL_CLAMP_TO_EDGE;
				texParams.wrapT     = GL_CLAMP_TO_EDGE;	
				break;	
			case 1:  // Default mode
			default: // Default mode
				texParams.minFilter = GL_LINEAR;
				texParams.magFilter = GL_LINEAR;
				texParams.wrapS     = GL_CLAMP_TO_EDGE;
				texParams.wrapT     = GL_CLAMP_TO_EDGE;	
				break;	
			case 2:
				texParams.minFilter = GL_LINEAR;
				texParams.magFilter = GL_LINEAR;
				texParams.wrapS     = GL_CLAMP_TO_EDGE;
				texParams.wrapT     = GL_CLAMP_TO_EDGE;	
				break;	
		};

		texture->setTexParameters( &texParams );


		pImage->release();

		if ( pszSpriteFrameName && texture )
		{
			CCTextureCache::sharedTextureCache()->m_pTextures->setObject( texture, forKey.c_str() );
			texture->autorelease();
		}
		else
		{
			CCLOG("cocos2d: Couldn't add UIImage in CCTextureCache");
		}

	} while (0);

#if CC_ENABLE_CACHE_TEXTURE_DATA
	#if DCG_IS_TARGET_WIN32 || DCG_IS_TARGET_IOS
		// This was causing a compiler error on android.  Not sure what it is, so 
		// taking it out for now.  mdomowicz 2013_10_27
		VolatileTexture::addCCImage(texture, pImage);
	#endif
#endif

	return texture;
}



/******************************************************************************
 FUNCTION:  ResetSharedTextures
******************************************************************************/
void SoTextureUtils::ResetSharedTextures()
{
	// Purge the texture/sprite caches
	CCTextureCache::sharedTextureCache()->purgeSharedTextureCache();
	CCSpriteFrameCache::sharedSpriteFrameCache()->purgeSharedSpriteFrameCache();

	// Remove sprites from actors so they reload on next update()
	SoActorList* pActorList = SoActorManager::GetInstance()->GetActorList();
	SoActorList::Iterator Iter( pActorList );

	while ( Iter.Next() )
	{
		SoActor* pActor = Iter.GetNodeData();

		SoActorVisible* pActorVisible = pActor->GetAsVisible();
		if ( pActorVisible )
		{
			pActorVisible->RemoveSprite();
		}
	}
}



/******************************************************************************
 FUNCTION:  GetPixelComponent
******************************************************************************/
unsigned char SoTextureUtils::GetPixelComponent( unsigned char* pData, int iX, int iY, int iSourceWidth, int iSourceHeight, int iChannel, int iNumChannels )
{
	if  ( 
			iX < iSourceWidth  && 
			iY < iSourceHeight &&
			iX >= 0 && 
			iY >= 0
		)
	{
		unsigned long int iVal = ( iX * iNumChannels * iSourceWidth ) + ( iNumChannels * iY ) + iChannel;
		return pData[ iVal ];
	}

	return 0;
}



