/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoTileset.h"
#include "SoGameBitHotel.h"
#include "SoGameBitPlayer.h"
#include "SoConsole.h"

#include "mmgrChooser.h"


USING_NS_CC;


/******************************************************************************
 FUNCTION:  SoTileset
******************************************************************************/
SoTileSet::SoTileSet() :
_iBoardWidth( 0 ),
_iBoardHeight( 0 ),
_iNumTiles( 0 ),
_papTileArray( NULL ),
_AllTiles( TILESET_MAX_TILES ),
_DrawPile( TILESET_MAX_TILES ),
_DiscardPile( TILESET_MAX_TILES )
{
}



/******************************************************************************
 FUNCTION:  ~SoTileset
******************************************************************************/
SoTileSet::~SoTileSet()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoTileSet::Init()
{
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoTileSet::Deinit()
{
	if ( _papTileArray )
	{
		for ( int i = 0; i < _iNumTiles; i ++ )
			delete _papTileArray[ i ];

		delete[] _papTileArray;
		_papTileArray = NULL;
	}

	_AllTiles.Clear();
	_DrawPile.Clear();
	_DiscardPile.Clear();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoTileSet::Update( float dt )
{
}



/******************************************************************************
 FUNCTION:  AddTile
******************************************************************************/
void SoTileSet::AddTile( SoGameBitTile* pTile )
{
	_AllTiles.AddLast( pTile );
}



/******************************************************************************
 FUNCTION:  CreateTiles
******************************************************************************/
void SoTileSet::CreateTiles( int iWidthInTiles, int iHeightInTiles )
{
	ASSERTF( !_papTileArray, ( "\nTile array is already created." ) );

	if ( !_papTileArray )
	{
		_iBoardWidth = iWidthInTiles;
		_iBoardHeight = iHeightInTiles;

		_iNumTiles = ( _iBoardWidth * _iBoardHeight );

		_papTileArray = new SoGameBitTile*[ _iNumTiles ];



		for ( int y = 0; y < _iBoardHeight; y ++ )
		{
			for ( int x = 0; x < _iBoardWidth; x ++ )
			{
				int iIndex = ( y * _iBoardWidth ) + x;

				SoGameBitTile* pNewTile = new SoGameBitTile;
				pNewTile->SetId( iIndex );
				pNewTile->SetTileX( x );
				pNewTile->SetTileY( y );

				AddTile( pNewTile );

				_papTileArray[ iIndex ] = pNewTile;
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  DrawNextTile
******************************************************************************/
SoGameBitTile* SoTileSet::DrawNextTile()
{
	SoGameBitTile* pTile = _DrawPile.GetLast();

	if ( pTile )
	{
		_DrawPile.RemoveLast();
		_DiscardPile.AddLast( pTile );
	}

	return pTile;
}



/******************************************************************************
 FUNCTION:  DiscardTile
******************************************************************************/
void SoTileSet::DiscardTile( SoGameBitTile* pTile )
{
	// It's weird, but as of this writing, when a tile is drawn,
	// it is immediately added to the discard pile, but is also immediately 
	// assigned to a player.  So a tile is really only considered "discarded"
	// when it is BOTH in the discard pile, AND it is not assigned
	// to any player.  Eventually I should clean this up.
	// mdomowicz 2013_12_10
	pTile->SetOwnerPlayerNumber( PLAYER_UNDEFINED_PLAYER_NUMBER );
}



/******************************************************************************
 FUNCTION:  NewGameShuffle
******************************************************************************/
void SoTileSet::NewGameShuffle( hash hRngSeed )
{
	// Clear the draw and discard piles
	_DrawPile.Clear();
	_DiscardPile.Clear();


	// Copy every tile into the Draw pile
	SoGameBitTileArray::Iterator Iter( &_AllTiles );
	while ( Iter.Next() )
	{
		SoGameBitTile* pTile = Iter.GetData();
		_DrawPile.AddLast( pTile );
	}


	// Shuffle the draw pile
	JpRandom LocalRng( hRngSeed );
	ShuffleTileArray( &_DrawPile, &LocalRng );
}



/******************************************************************************
 FUNCTION:  ShuffleTileArray
******************************************************************************/
void SoTileSet::ShuffleTileArray( SoGameBitTileArray* pArray, JpRandom* pRng )
{
	//-----------------------
	// add every tile to a temp list, and make sure each
	// new list node is given a random sort key
	SoGameBitTileList TempList;

	SoGameBitTileArray::Iterator Iter( pArray );
	while ( Iter.Next() )
	{
		SoGameBitTile* pTile = Iter.GetData();
		TempList.AddLast( pTile );
	
		SoGameBitTileList::Node* pLastNode = TempList.GetLast();
		pLastNode->SetSortKey( pRng->GetInt() );
	}


	//-----------------------
	// sort the temp list - this "shuffles" the list
	TempList.QuickSort();


	//-----------------------
	// rebuild the given list, with the values from
	// the temp list.
	pArray->Clear();
	SoGameBitTileList::Iterator Iter2( &TempList );
	while ( Iter2.Next() )
	{
		SoGameBitTile* pTile = Iter2.GetNodeData();
		pArray->AddLast( pTile );
	}
}



/******************************************************************************
 FUNCTION:  GetTileById
******************************************************************************/
SoGameBitTile* SoTileSet::GetTileById( int iId )
{
	if ( iId >= 0 && iId < _iNumTiles )
		return _papTileArray[ iId ];

	return NULL;
}



/******************************************************************************
 FUNCTION:  GetTileByXy
******************************************************************************/
SoGameBitTile* SoTileSet::GetTileByXy( int iX, int iY )
{
	if ( iX >= 0 && iX < _iBoardWidth )
	{
		int iIndex = ( iY * _iBoardWidth ) + iX;
		return GetTileById( iIndex );
	}

	return NULL;
}



/******************************************************************************
 FUNCTION:  FindAdjacentTileWithMatchingHotelId
******************************************************************************/
int SoTileSet::FindAdjacentTileWithMatchingHotelId( int iTileX, int iTileY, int iHotelId )
{
	SoGameBitTile* pUp =    GetTileByXy( iTileX + 0, iTileY - 1 );
	SoGameBitTile* pDown =  GetTileByXy( iTileX + 0, iTileY + 1 );
	SoGameBitTile* pLeft =  GetTileByXy( iTileX - 1, iTileY + 0 );
	SoGameBitTile* pRight = GetTileByXy( iTileX + 1, iTileY + 0 );

	if ( pUp && pUp->GetHotelId() == iHotelId )
		return pUp->GetId();

	if ( pDown && pDown->GetHotelId() == iHotelId )
		return pDown->GetId();

	if ( pLeft && pLeft->GetHotelId() == iHotelId )
		return pLeft->GetId();

	if ( pRight && pRight->GetHotelId() == iHotelId )
		return pRight->GetId();

	return TILE_UNDEFINED_ID;
}



/******************************************************************************
 FUNCTION:  FindAdjacentHotelToExpand
******************************************************************************/
int SoTileSet::FindAdjacentHotelToExpand( int iTileX, int iTileY )
{
	SoGameBitTile* pUp =    GetTileByXy( iTileX + 0, iTileY - 1 );
	SoGameBitTile* pDown =  GetTileByXy( iTileX + 0, iTileY + 1 );
	SoGameBitTile* pLeft =  GetTileByXy( iTileX - 1, iTileY + 0 );
	SoGameBitTile* pRight = GetTileByXy( iTileX + 1, iTileY + 0 );

	if ( pUp && pUp->GetHotelId() != HOTEL_UNDEFINED_ID )
		return pUp->GetHotelId();

	if ( pDown && pDown->GetHotelId() != HOTEL_UNDEFINED_ID )
		return pDown->GetHotelId();

	if ( pLeft && pLeft->GetHotelId() != HOTEL_UNDEFINED_ID )
		return pLeft->GetHotelId();

	if ( pRight && pRight->GetHotelId() != HOTEL_UNDEFINED_ID )
		return pRight->GetHotelId();

	return HOTEL_UNDEFINED_ID;
}



/******************************************************************************
 FUNCTION:  GetHotelSizeAtTile
******************************************************************************/
int SoTileSet::GetHotelSizeAtTile( int iTileX, int iTileY )
{
	SoGameBitTile* pTile = GetTileByXy( iTileX, iTileY );

	if ( pTile && pTile->GetHotelId() != HOTEL_UNDEFINED_ID )
		return FlagContiguousTilesAtTile( NULL, iTileX, iTileY, pTile->GetHotelId() );

	return 0;
}



/******************************************************************************
 FUNCTION:  CreateNewHotelChainAtTile
******************************************************************************/
void SoTileSet::CreateNewHotelChainAtTile( int iTileX, int iTileY, int iHotelId )
{
	SoGameBitTileArray CollectedResultsList( TILESET_MAX_TILES );

	FlagContiguousTilesAtTile( &CollectedResultsList, iTileX, iTileY, HOTEL_UNDEFINED_ID );


	SoGameBitTileArray::Iterator Iter( &CollectedResultsList );
	while ( Iter.Next() )
	{
		SoGameBitTile* pTile = Iter.GetData();
		pTile->SetHotelId( iHotelId );
	}
}



/******************************************************************************
 FUNCTION:  ExpandHotelChainAtTile
******************************************************************************/
void SoTileSet::ExpandHotelChainAtTile( int iTileX, int iTileY, int iHotelId )
{
	// Although this just calls another function, I'm using two functions for clarity.
	// Also, I suspect the functionality will diverge soon.  mdomowicz 2013_12_03
	CreateNewHotelChainAtTile( iTileX, iTileY, iHotelId );
}



/******************************************************************************
 FUNCTION:  FlagContiguousTilesAtTile
******************************************************************************/
int SoTileSet::FlagContiguousTilesAtTile( SoGameBitTileArray* pCollectedResultsList, int iTileX, int iTileY, int iHotelId )
{
	//-----------------------
	// define local functions
	struct Local 
	{
		static void Count( SoGameBitTile* pTile, SoGameBitTileArray* pTilesToProcessList )
		{
			pTile->SetIsCounted( true );
			pTilesToProcessList->AddLast( pTile );
		}

		static bool IsCountable( SoGameBitTile* pTile, int iHotelId )
		{
			return 
			( 
				pTile && 
				!pTile->IsCounted() && 
				pTile->IsPlaced() &&
				pTile->GetHotelId() == iHotelId 
			);
		}

		static int CountContiguousTiles( SoGameBitTileArray* pCollectedResultsList, SoTileSet* pSet, int iX, int iZ, int iHotelId )
		{	
			SoGameBitTileArray TilesToProcessList( TILESET_MAX_TILES );

			SoGameBitTile* pTile = pSet->GetTileByXy( iX, iZ );
			
			int iNumContiguousTiles = 0;

			if ( pTile->GetHotelId() == iHotelId )
			{
				Count( pTile, &TilesToProcessList );

				while ( TilesToProcessList.GetNumEntries() > 0 )
				{
					SoGameBitTile* pLastTile = TilesToProcessList.GetLast();
					int iLastX = pLastTile->GetTileX();
					int iLastZ = pLastTile->GetTileY();

					SoGameBitTile* pTile1 = pSet->GetTileByXy( iLastX + 0, iLastZ - 1 );
					SoGameBitTile* pTile2 = pSet->GetTileByXy( iLastX - 1, iLastZ + 0 );
					SoGameBitTile* pTile3 = pSet->GetTileByXy( iLastX + 1, iLastZ + 0 );
					SoGameBitTile* pTile4 = pSet->GetTileByXy( iLastX + 0, iLastZ + 1 );


					if ( IsCountable( pTile1, iHotelId ) )
						Count( pTile1, &TilesToProcessList );
					else if ( IsCountable( pTile2, iHotelId ) )
						Count( pTile2, &TilesToProcessList );
					else if ( IsCountable( pTile3, iHotelId ) )
						Count( pTile3, &TilesToProcessList );
					else if ( IsCountable( pTile4, iHotelId ) )
						Count( pTile4, &TilesToProcessList );
					else
					{
						TilesToProcessList.RemoveLast();
						iNumContiguousTiles ++;

						if ( pCollectedResultsList )
							pCollectedResultsList->AddLast( pLastTile );
					}
				}
			}

			return iNumContiguousTiles;
		}
	};


	for ( int i = 0; i < _iNumTiles; i ++ )
		_papTileArray[ i ]->SetIsCounted( false );

	if ( pCollectedResultsList )
		pCollectedResultsList->Clear();

	return Local::CountContiguousTiles( pCollectedResultsList, this, iTileX, iTileY, iHotelId );
}



/******************************************************************************
 FUNCTION:  DoesTileMergeHotelsWhenPlaced
******************************************************************************/
bool SoTileSet::DoesTileMergeHotelsWhenPlaced( int iTileX, int iTileY )
{
	SoGameBitTile* pUp = GetTileByXy( iTileX, iTileY - 1 );
	SoGameBitTile* pDown = GetTileByXy( iTileX, iTileY + 1 );
	SoGameBitTile* pLeft = GetTileByXy( iTileX - 1, iTileY );
	SoGameBitTile* pRight = GetTileByXy( iTileX + 1, iTileY );



	//-----------------------
	// The logic is a bit strange here.  This is what it's doing:
	//
	// Check the tile in each direction, up, down, left and right,
	// and remember the hotel id of the first adjacent tile we find
	// (that has a valid hotel id).
	//
	// Once we have one hotel, then we try to find a second, different
	// hotel.
	//
	// As soon as we find such a second, different, hotel, then we 
	// know the given tile will merge two hotels.
	//
	// mdomowicz 2013_12_10


	int iFirstAdjacentHotelId = HOTEL_UNDEFINED_ID;

	if ( pUp )
	{
		if ( pUp->IsPlaced() && pUp->GetHotelId() != HOTEL_UNDEFINED_ID )
			iFirstAdjacentHotelId = pUp->GetHotelId();
	}

	if ( pDown )
	{
		if ( pDown->IsPlaced() && pDown->GetHotelId() != HOTEL_UNDEFINED_ID )
		{
			if ( iFirstAdjacentHotelId == HOTEL_UNDEFINED_ID )
				iFirstAdjacentHotelId = pDown->GetHotelId();
			else if ( iFirstAdjacentHotelId != pDown->GetHotelId() )
				return true;
		}
	}

	if ( pLeft )
	{
		if ( pLeft->IsPlaced() && pLeft->GetHotelId() != HOTEL_UNDEFINED_ID )
		{
			if ( iFirstAdjacentHotelId == HOTEL_UNDEFINED_ID )
				iFirstAdjacentHotelId = pLeft->GetHotelId();
			else if ( iFirstAdjacentHotelId != pLeft->GetHotelId() )
				return true;
		}
	}


	// If the first adjacent hotel id is not set by now, then it doesn't matter
	// what the HotelId of the last tile is - it will never cause a merger.
	if ( iFirstAdjacentHotelId == HOTEL_UNDEFINED_ID ) 
		return false;


	if ( pRight )
	{
		if ( pRight->IsPlaced() && pRight->GetHotelId() != HOTEL_UNDEFINED_ID )
		{
			if ( iFirstAdjacentHotelId != pRight->GetHotelId() )
				return true;
		}
	}

	return false;
}



/******************************************************************************
 FUNCTION:  DoesTileCreateNewHotelWhenPlaced
******************************************************************************/
bool SoTileSet::DoesTileCreateNewHotelWhenPlaced( int iTileX, int iTileY )
{
	//-----------------------
	// A new tile only creates a new hotel, if it doesn't first merge hotels.
	// If it merges hotels, then any would-be-created hotels (adjacent non-hotel
	// tiles) are considered part of the final remaining hotel after the merge.
	if ( !DoesTileMergeHotelsWhenPlaced( iTileX, iTileY ) )
	{
		SoGameBitTile* pUp = GetTileByXy( iTileX, iTileY - 1 );
		SoGameBitTile* pDown = GetTileByXy( iTileX, iTileY + 1 );
		SoGameBitTile* pLeft = GetTileByXy( iTileX - 1, iTileY );
		SoGameBitTile* pRight = GetTileByXy( iTileX + 1, iTileY );

		//-----------------------
		// If any of the neighbor tiles are already part of existing
		// hotels, then this tile placement cannot create a new hotel.
		// It would only extend the existing hotel.
		if ( pUp && pUp->IsPlaced() && pUp->GetHotelId() != HOTEL_UNDEFINED_ID )
			return false;

		if ( pDown && pDown->IsPlaced() && pDown->GetHotelId() != HOTEL_UNDEFINED_ID )
			return false;

		if ( pLeft && pLeft->IsPlaced() && pLeft->GetHotelId() != HOTEL_UNDEFINED_ID )
			return false;

		if ( pRight && pRight->IsPlaced() && pRight->GetHotelId() != HOTEL_UNDEFINED_ID )
			return false;


		//-----------------------
		// If we get here, then we know the tile placement does
		// not cause a merger, nor does it extend any existing hotels.
		// So, if there is a neighbor tile, it must form a new hotel
		// with the given tile.
		if ( pUp && pUp->IsPlaced() && pUp->GetHotelId() == HOTEL_UNDEFINED_ID )
			return true;

		if ( pDown && pDown->IsPlaced() && pDown->GetHotelId() == HOTEL_UNDEFINED_ID )
			return true;

		if ( pLeft && pLeft->IsPlaced() && pLeft->GetHotelId() == HOTEL_UNDEFINED_ID )
			return true;

		if ( pRight && pRight->IsPlaced() && pRight->GetHotelId() == HOTEL_UNDEFINED_ID )
			return true;
	}

	return false;
}



/******************************************************************************
 FUNCTION:  MakeCopy
******************************************************************************/
void SoTileSet::MakeCopy( SoTileSet* pSource )
{
	_iBoardWidth = pSource->_iBoardWidth;
	_iBoardHeight = pSource->_iBoardHeight;
	_iNumTiles = pSource->_iNumTiles;


	//-----------------------
	// create new tiles, and copy the source 
	// tiles into the new tiles
	CreateTiles( _iBoardWidth, _iBoardHeight );

	for ( int i = 0; i < _iNumTiles; i ++ )
	{
		_papTileArray[ i ]->MakeCopy( pSource->_papTileArray[ i ] );
	}


	//-----------------------
	// Reconstruct the _DrawPile
	SoGameBitTileArray::Iterator DrawPileIter( &pSource->_DrawPile );
	while ( DrawPileIter.Next() )
	{
		SoGameBitTile* pOldTile = DrawPileIter.GetData();
		SoGameBitTile* pNewTile = GetTileById( pOldTile->GetId() );
		_DrawPile.AddLast( pNewTile );
	}


	//-----------------------
	// Reconstruct the _DiscardPile.
	SoGameBitTileArray::Iterator DiscardPileIter( &pSource->_DiscardPile );
	while ( DiscardPileIter.Next() )
	{
		SoGameBitTile* pOldTile = DiscardPileIter.GetData();
		SoGameBitTile* pNewTile = GetTileById( pOldTile->GetId() );
		_DiscardPile.AddLast( pNewTile );
	}
}


/******************************************************************************
 FUNCTION:  PrintToConsole
******************************************************************************/
void SoTileSet::PrintToConsole( const std::string& s )
{
	SoConsole* pConsole = (SoConsole*)NeConsole::GetInstance();
	pConsole->AddOutputString( s );
	pConsole->SetCurrentTextColor( JpUtils::_White );
	CCLOG( s.c_str() );
}



/******************************************************************************
 FUNCTION:  CPrintF
******************************************************************************/
void SoTileSet::CPrintF( const char* pFormat, ... )
{
	if ( strlen( pFormat ) < 2048 )
	{
		char buffer[2048];
		va_list args;
		va_start (args, pFormat);
		vsprintf (buffer,pFormat, args);
		va_end (args);

		PrintToConsole( buffer );
	}
}






