/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"
#include "cocos2d.h"

#include "SoGameBitTile.h"

#define TILESET_MAX_TILES 256

USING_NS_CC;


class SoTileSet
{
	public:

		//-----------------------
		// public functions

		SoTileSet();

		virtual ~SoTileSet();

		void Init();

		void Deinit();

		void Update( float dt );

		virtual void NewGameShuffle( hash hRngSeed );

		void CreateTiles( int iWidthInTiles, int iHeightInTiles );

		int GetNumTiles() { return _iNumTiles; }

		SoGameBitTile* DrawNextTile();
		void DiscardTile( SoGameBitTile* pTile );

		SoGameBitTile* GetTileById( int iId );
		SoGameBitTile* GetTileByXy( int iX, int iY );

		void SetBoardWidth( int i ) { _iBoardWidth = i; }
		void SetBoardHeight( int i ) { _iBoardHeight = i; }

		SoGameBitTileArray* GetAllTiles() { return &_AllTiles; }
		SoGameBitTileArray* GetDrawPile() { return &_DrawPile; }
		SoGameBitTileArray* GetDiscardPile() { return &_DiscardPile; }

		int FindAdjacentTileWithMatchingHotelId( int iTileX, int iTileY, int iHotelId );
		int FindAdjacentHotelToExpand( int iTileX, int iTileY );

		int GetHotelSizeAtTile( int iTileX, int iTileY );
		void CreateNewHotelChainAtTile( int iTileX, int iTileY, int iHotelId );
		void ExpandHotelChainAtTile( int iTileX, int iTileY, int iHotelId );


		bool DoesTileMergeHotelsWhenPlaced( int iTileX, int iTileY );
		bool DoesTileCreateNewHotelWhenPlaced( int iTileX, int iTileY );

		void MakeCopy( SoTileSet* pSource );

	protected:

		//-----------------------
		// protected variables
		int _iBoardWidth;
		int _iBoardHeight;
		int _iNumTiles;

		SoGameBitTile** _papTileArray;

		SoGameBitTileArray _AllTiles;
		SoGameBitTileArray _DrawPile;
		SoGameBitTileArray _DiscardPile;


		//-----------------------
		// protected functions
		void ShuffleTileArray( SoGameBitTileArray* pArray, JpRandom* pRng );

		void PrintToConsole( const std::string& s );
		void CPrintF( const char* pFormat, ... );

		void AddTile( SoGameBitTile* pTile );

		int FlagContiguousTilesAtTile( SoGameBitTileArray* pCollectedResultsList, int iTileX, int iTileY, int iHotelId );
};




