/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "NeCsvIn.h"

#include "SoDrawDebug.h"
#include "SoSprite.h"
#include "SoSoundManager.h"
#include "SoUiButton.h"
#include "SoUtils.h"

#include "mmgrChooser.h"






/******************************************************************************
 FUNCTION:  SoUiButton
******************************************************************************/
SoUiButton::SoUiButton() :
_fAlpha( 1.0f ),
_fWidth( 0.0f ),
_fHeight( 0.0f ),
_fHalfWidth( 0.0f ),
_fHalfHeight( 0.0f ),
_fScale( 1.0f ),
_eState( STATE_UNPRESSED ),
_eStateToSwitchTo( STATE_UNPRESSED ),
_fAnimTimer( 0.0f ),
_eCurrentAnimState( ANIM_STATE_HIDDEN ),
_bIsActive( true ),
_pUnpressedSprite( NULL ),
_pPressedSprite( NULL ),
_pDisabledSprite( NULL ),
_pDrawDebug( NULL ),
_fTransitionInDelay( 0.0f )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoUiButton::Init
(
	CCNode* pParentNode,
	const std::string& sButtonName,
	const std::string& sUnpressedTextureName,
	const std::string& sPressedTextureName,
	const std::string& sDisabledTextureName 
)
{
	_pParentNode = pParentNode;
	pParentNode->addChild( this );

	setPosition( 0.0f, 0.0f );

	SetButtonName( sButtonName );
	SetPressedTextureName( sPressedTextureName );
	SetUnpressedTextureName( sUnpressedTextureName );

	_pUnpressedSprite = NULL;
	_pPressedSprite = NULL;

	_fWidth = 0.0f;
	_fHeight = 0.0f;
	_fHalfWidth = 0.0f;
	_fHalfHeight = 0.0f;


	if ( sUnpressedTextureName.length() > 0 )
	{
		_pUnpressedSprite = SoSprite::CreateScaledToDevice( sUnpressedTextureName.c_str() );
		_pUnpressedSprite->retain();
		addChild( _pUnpressedSprite );
		_pUnpressedSprite->setVisible( true );

		_fWidth = _pUnpressedSprite->getContentSize().width * SoUtils::GetiPadRatioX();
		_fHeight = _pUnpressedSprite->getContentSize().height * SoUtils::GetiPadRatioY();
		_fHalfWidth = _fWidth / 2.0f;
		_fHalfHeight = _fHeight / 2.0f;
	}

	if ( sPressedTextureName.length() > 0 )
	{
		_pPressedSprite = SoSprite::CreateScaledToDevice( sPressedTextureName.c_str() );
		_pPressedSprite->retain();
		addChild( _pPressedSprite );
		_pPressedSprite->setVisible( false );
	}

	if ( sDisabledTextureName.length() > 0 )
	{
		_pDisabledSprite = SoSprite::CreateScaledToDevice( sDisabledTextureName.c_str() );
		_pDisabledSprite->retain();
		addChild( _pDisabledSprite );
		_pDisabledSprite->setVisible( false );
	}

	_pDrawDebug = SoDrawDebug::create();
	_pDrawDebug->retain();
	_pDrawDebug->Setup( this );
	_pDrawDebug->setPosition( 0.0f, 0.0f );
	_pDrawDebug->SetIsActive( false );
	_pDrawDebug->AddBox( 0.0f, 0.0f, _fWidth, _fHeight );
}



/******************************************************************************
 FUNCTION:  InitInvisibleButton
******************************************************************************/
void SoUiButton::InitInvisibleButton
(
	CCNode* pParentNode,
	const std::string& sButtonName,
	float fWidth,
	float fHeight
)
{
	_pParentNode = pParentNode;
	pParentNode->addChild( this );

	setPosition( 0.0f, 0.0f );

	SetButtonName( sButtonName );

	_pUnpressedSprite = NULL;
	_pPressedSprite = NULL;
	_pDisabledSprite = NULL;

	_fWidth = fWidth;
	_fHeight = fHeight;
	_fHalfWidth = _fWidth / 2.0f;
	_fHalfHeight = _fHeight / 2.0f;


	_pDrawDebug = SoDrawDebug::create();
	_pDrawDebug->retain();
	_pDrawDebug->Setup( this );
	_pDrawDebug->setPosition( 0.0f, 0.0f );
	_pDrawDebug->SetIsActive( false );
	_pDrawDebug->AddBox( 0.0f, 0.0f, _fWidth, _fHeight );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoUiButton::Deinit() 
{
	if ( _pDrawDebug )
	{
		_pDrawDebug->Deinit();
		_pDrawDebug->release();
		_pDrawDebug = NULL;
	}

	if ( _pUnpressedSprite )
	{
		removeChild( _pUnpressedSprite, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pUnpressedSprite->release();
		_pUnpressedSprite = NULL;
	}

	if ( _pPressedSprite )
	{
		removeChild( _pPressedSprite, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pPressedSprite->release();
		_pPressedSprite = NULL;
	}

	if ( _pDisabledSprite )
	{
		removeChild( _pDisabledSprite, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pDisabledSprite->release();
		_pDisabledSprite = NULL;
	}

	if ( _pParentNode )
	{
		_pParentNode->removeChild( this, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pParentNode = NULL;
	}
};



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoUiButton::Update( float dt )
{
	switch ( _eState )
	{
		case STATE_UNPRESSED:
		{
			if ( _pUnpressedSprite )
				_pUnpressedSprite->setVisible( true );

			if ( _pPressedSprite )
				_pPressedSprite->setVisible( false );

			if ( _pDisabledSprite )
				_pDisabledSprite->setVisible( false );

			if ( _eStateToSwitchTo != _eState )
			{
				_eState = _eStateToSwitchTo;
			}

			break;
		}

		case STATE_PRESSED:
		{
			if ( _pUnpressedSprite )
				_pUnpressedSprite->setVisible( false );

			if ( _pPressedSprite )
				_pPressedSprite->setVisible( true );

			if ( _pDisabledSprite )
				_pDisabledSprite->setVisible( false );

			if ( _eStateToSwitchTo != _eState )
			{
				_eState = _eStateToSwitchTo;
			}
			break;
		}

		case STATE_DISABLED:
		{
			if ( _pUnpressedSprite )
				_pUnpressedSprite->setVisible( false );

			if ( _pPressedSprite )
				_pPressedSprite->setVisible( false );

			if ( _pDisabledSprite )
				_pDisabledSprite->setVisible( true );
		
			break;
		}

		case STATE_LAST:
		default:
			ASSERTF( 0, ( "\nUnknown button state." ) );
	}


	_fAnimTimer += dt;
	switch ( _eCurrentAnimState )
	{
		case ANIM_STATE_HIDDEN:
		{
			_fAnimTimer = 0.0f;
			Hide();
			break;
		}
		case ANIM_STATE_SHOWING:
		{
			_fAnimTimer = 1.0f;
			UpdateAnimateIn();
			break;
		}
		case ANIM_STATE_ANIMATE_IN:
		{
			if ( _fAnimTimer > _fTransitionInDelay )
			{
				_fAnimTimer = 0.0f;
				_eCurrentAnimState = ANIM_STATE_ANIMATE_IN_UPDATE;
			}

			break;
		}
		case ANIM_STATE_ANIMATE_IN_UPDATE:
		{
			bool bIsDone = UpdateAnimateIn();
			if ( bIsDone )
				_eCurrentAnimState = ANIM_STATE_SHOWING;

			break;
		}
		case ANIM_STATE_ANIMATE_OUT:
		{
			_eCurrentAnimState = ANIM_STATE_HIDDEN;
			break;
		}

		case ANIM_STATE_LAST:
		default:
			break;
	}


	if ( _pDrawDebug )
	{
		_pDrawDebug->SetIsActive( false );
		//_pDrawDebug->SetIsActive( IsActive() );
		_pDrawDebug->Update( dt );
	}
}



/******************************************************************************
 FUNCTION:  UpdateAnimateIn
******************************************************************************/
bool SoUiButton::UpdateAnimateIn()
{
	float fSpeed = 5.0f;
	float fZoom = Clamp( _fAnimTimer * fSpeed, 0.0f, 1.0f );
	fZoom *= fZoom;

	if ( _pUnpressedSprite && _pPressedSprite )
	{
		float fHeight = _pUnpressedSprite->getContentSize().height * SoUtils::GetiPadRatioY();
		float fScaleY = fZoom * ( fHeight / _pUnpressedSprite->getContentSize().height );

		_pUnpressedSprite->setScaleY( fScaleY );
		_pPressedSprite->setScaleY( fScaleY );

		if ( _pDisabledSprite )
			_pDisabledSprite->setScaleY( fScaleY );
	}

	return ( fZoom == 1.0f );
}



/******************************************************************************
 FUNCTION:  IsUsable
******************************************************************************/
bool SoUiButton::IsUsable()
{
	return
	(
		IsActive() &&
		_eState != STATE_DISABLED &&
		_eCurrentAnimState != ANIM_STATE_HIDDEN &&
		_eCurrentAnimState != ANIM_STATE_ANIMATE_IN &&
		_eCurrentAnimState != ANIM_STATE_ANIMATE_OUT &&
		_eCurrentAnimState != ANIM_STATE_LAST
	);
}



/******************************************************************************
 FUNCTION:  Hide
******************************************************************************/
void SoUiButton::Hide()
{ 
	_eCurrentAnimState = ANIM_STATE_HIDDEN; 

	if ( _pUnpressedSprite )
		_pUnpressedSprite->setScaleY( 0.0f );

	if ( _pPressedSprite )
		_pPressedSprite->setScaleY( 0.0f );

	if ( _pDisabledSprite )
		_pDisabledSprite->setScaleY( 0.0f );
}



/******************************************************************************
 FUNCTION:  Enable
******************************************************************************/
void SoUiButton::Enable( bool b)
{ 
	if ( b )
		_eState = STATE_UNPRESSED;
	else
		_eState = STATE_DISABLED;
}



/******************************************************************************
 FUNCTION:  TransitionIn
******************************************************************************/
void SoUiButton::TransitionIn() 
{
	_eCurrentAnimState = ANIM_STATE_ANIMATE_IN;
	_fAnimTimer = 0.0f;
}



/******************************************************************************
 FUNCTION:  TransitionOut
******************************************************************************/
void SoUiButton::TransitionOut() 
{
	_eCurrentAnimState = ANIM_STATE_ANIMATE_OUT;
	_fAnimTimer = 0.0f;
}



/******************************************************************************
 FUNCTION:  TouchIsInside
******************************************************************************/
bool SoUiButton::TouchIsInside( CCTouch* pTouch, float fExtraMarginInPixels )
{
	CCPoint ToutchPos = pTouch->locationInView();
	ToutchPos = CCDirector::sharedDirector()->convertToGL(ToutchPos);

	// convertToWorldSpace ADDS IN the local position automatically.
	// So, to get the world position of the local position, you just pass in 0,0
	CCPoint LocalPos( 0.0f, 0.0f );
	CCPoint WorldPos = convertToWorldSpace( LocalPos );
	float fX = WorldPos.x;
	float fY = WorldPos.y;

	float fHalfWidth = ( _fHalfWidth + fExtraMarginInPixels );
	float fHalfHeight = ( _fHalfHeight + fExtraMarginInPixels );

	bool bIsTouchInside = 
	( 
		ToutchPos.x > fX - fHalfWidth && 
		ToutchPos.x < fX + fHalfWidth && 
		ToutchPos.y > fY - fHalfHeight && 
		ToutchPos.y < fY + fHalfHeight
	);

	return bIsTouchInside;
}



/******************************************************************************
 FUNCTION:  draw
******************************************************************************/
void SoUiButton::draw(void)
{
	
}



/******************************************************************************
 FUNCTION:  create
******************************************************************************/
SoUiButton* SoUiButton::create()
{
	SoUiButton* pRet = new SoUiButton();
	if ( pRet && pRet->init() )
	{
		pRet->autorelease();
		return pRet;
	}
	else
	{
		delete pRet;
		pRet = NULL;
		return NULL;
	}
}












/******************************************************************************
 FUNCTION:  SoUiButtonManager
******************************************************************************/
SoUiButtonManager::SoUiButtonManager() :
_bListenerListLocked( false ),
_pParentNode( NULL ),
_fExtraMarginInPixelsForButtonUpTest( 0.0f )
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoUiButtonManager::Init( CCNode* pParentNode )
{
	_pParentNode = pParentNode;
	pParentNode->addChild( this );

	setTouchEnabled( true );

	SetExtraMarginInPixelsForButtonUpTest( 10.0f * SoUtils::GetiPadRatioX() );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoUiButtonManager::Deinit()
{
	if ( _pParentNode )
	{
		_pParentNode->removeChild( this, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pParentNode = NULL;
	}

	RemoveAllListeners();


	SoUiButtonList::Iterator Iter( &_UiButtonList );
	while ( Iter.Next() )
	{
		SoUiButton* pButton = Iter.GetNodeData();
		pButton->Deinit();
		pButton->release();
	}

	_UiButtonList.RemoveAll();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoUiButtonManager::Update( float dt )
{
	SoUiButtonList::Iterator Iter( &_UiButtonList );
	while ( Iter.Next() )
	{
		SoUiButton* pButton = Iter.GetNodeData();
		pButton->Update( dt );
	}
}



/******************************************************************************
 FUNCTION:  CreateButton
******************************************************************************/
SoUiButton* SoUiButtonManager::CreateButton
(
	const std::string& sButtonName, 
	const std::string& sImageName,
	const std::string& sPressedImageName
)
{
	SoUiButton* pNewButton = SoUiButton::create();
	pNewButton->retain();
	pNewButton->Init( this, sButtonName, sImageName, sPressedImageName, "" );
	AddUiButton( pNewButton );
	return pNewButton;
}



/******************************************************************************
 FUNCTION:  CreateButtonWithDisabledState
******************************************************************************/
SoUiButton* SoUiButtonManager::CreateButtonWithDisabledState
(
	const std::string& sButtonName, 
	const std::string& sImageName,
	const std::string& sPressedImageName,
	const std::string& sDisabledImageName 
)
{
	SoUiButton* pNewButton = SoUiButton::create();
	pNewButton->retain();
	pNewButton->Init( this, sButtonName, sImageName, sPressedImageName, sDisabledImageName );
	AddUiButton( pNewButton );
	return pNewButton;
}



/******************************************************************************
 FUNCTION:  CreateInvisibleButton
******************************************************************************/
SoUiButton* SoUiButtonManager::CreateInvisibleButton
(
	const std::string& sButtonName, 
	float fWidth,
	float fHeight
)
{
	SoUiButton* pNewButton = SoUiButton::create();
	pNewButton->retain();
	pNewButton->InitInvisibleButton( this, sButtonName, fWidth, fHeight );
	AddUiButton( pNewButton );
	return pNewButton;
}



/******************************************************************************
 FUNCTION:  AddUiButton
******************************************************************************/
void SoUiButtonManager::AddUiButton( SoUiButton* p )
{
	if ( _UiButtonList.Find( p ) == NULL )
		_UiButtonList.AddLast( p );
}



/******************************************************************************
 FUNCTION:  BroadcastUiButtonDown
******************************************************************************/
void SoUiButtonManager::BroadcastOnUiButtonDown( SoUiButton* pButton )
{
	//-----------------------
	// The event handler we are calling can do anything it likes...even attempt to
	// add or remove to the listener list.  That would be bad since we are in 
	// the middle of walking through the list.  So we lock the list to prevent tampering.

	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();
		pListener->OnUiButtonDown( pButton );
	}

	_bListenerListLocked = false;
}



/******************************************************************************
 FUNCTION:  BroadcastUiButtonUp
******************************************************************************/
void SoUiButtonManager::BroadcastOnUiButtonUp( SoUiButton* pButton )
{
	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();
		pListener->OnUiButtonUp( pButton );
	}

	_bListenerListLocked = false;
}



/******************************************************************************
 FUNCTION:  BroadcastOnUiButtonUpTouchOutside
******************************************************************************/
void SoUiButtonManager::BroadcastOnUiButtonUpTouchOutside( SoUiButton* pButton )
{
	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();
		pListener->OnUiButtonUpTouchOutside( pButton );
	}

	_bListenerListLocked = false;
}



/******************************************************************************
 FUNCTION:  BroadcastUiButtonRub
******************************************************************************/
void SoUiButtonManager::BroadcastOnUiButtonRub( SoUiButton* pButton )
{
	_bListenerListLocked = true;

	JpList< Listener >::Iterator Iter( &_ListenerList );
	while ( Iter.Next() )
	{
		Listener* pListener = Iter.GetNodeData();
		pListener->OnUiButtonRub( pButton );
	}

	_bListenerListLocked = false;
}



/******************************************************************************
 FUNCTION:  ccTouchesBegan
******************************************************************************/
void SoUiButtonManager::ccTouchesBegan(CCSet *pTouches, CCEvent *pEvent)
{
	//-----------------------
	// For every UNPRESSED button...
	SoUiButtonList::Iterator Iter( &_UiButtonList );
	while ( Iter.Next() )
	{
		SoUiButton* pButton = Iter.GetNodeData();
		if ( pButton->IsUnpressed() )
		{
			//-----------------------
			// ...for every touch...
			CCSetIterator iter = pTouches->begin();   // for now just use the first touch!
			//for ( ; iter != pTouches->end(); iter ++ )
			{
				//-----------------------
				// ...if that touch is inside...
				CCTouch* pTouch = (CCTouch*)(*iter);
				float fMargin = 0.0f;
				if ( pButton->IsUsable() && pButton->TouchIsInside( pTouch, fMargin ) )
				{
					pButton->SetPressed();
					BroadcastOnUiButtonDown( pButton );
				}
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  ccTouchesMoved
******************************************************************************/
void SoUiButtonManager::ccTouchesMoved(CCSet *pTouches, CCEvent *pEvent)
{
	//-----------------------
	// For every button...
	SoUiButtonList::Iterator Iter( &_UiButtonList );
	while ( Iter.Next() )
	{
		SoUiButton* pButton = Iter.GetNodeData();

		//-----------------------
		// ...for every touch...
		CCSetIterator iter = pTouches->begin();   // for now just use the first touch!
		//for ( ; iter != pTouches->end(); iter ++ )
		{
			//-----------------------
			// ...if that touch is inside...
			CCTouch* pTouch = (CCTouch*)(*iter);
			float fMargin = 0.0f;
			if ( pButton->IsUsable() && pButton->TouchIsInside( pTouch, fMargin ) )
			{
				BroadcastOnUiButtonRub( pButton );
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  ccTouchesEnded
******************************************************************************/
void SoUiButtonManager::ccTouchesEnded(CCSet *pTouches, CCEvent *pEvent)
{
	//-----------------------
	// For every PRESSED button...
	SoUiButtonList::Iterator Iter( &_UiButtonList );
	while ( Iter.Next() )
	{
		SoUiButton* pButton = Iter.GetNodeData();
		if ( pButton->IsPressed() )
		{
			pButton->SetUnpressed();

			//-----------------------
			// ...for every touch...
			CCSetIterator iter = pTouches->begin();  // // for now just use the first touch!
			//for ( ; iter != pTouches->end(); iter ++ )
			{
				//-----------------------
				// ...if that touch is inside...
				CCTouch* pTouch = (CCTouch*)(*iter);
				if ( pButton->IsUsable() )
				{
					if ( pButton->TouchIsInside( pTouch, _fExtraMarginInPixelsForButtonUpTest )  )
						BroadcastOnUiButtonUp( pButton );
					else
						BroadcastOnUiButtonUpTouchOutside( pButton );
				}
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  ccTouchesCancelled
******************************************************************************/
void SoUiButtonManager::ccTouchesCancelled(CCSet *pTouches, CCEvent *pEvent)
{
	ccTouchesEnded(pTouches, pEvent);
}



/******************************************************************************
 FUNCTION:  LoadButtonsFromCsv
******************************************************************************/
void SoUiButtonManager::LoadButtonsFromCsv( std::string sPathAndFilename )
{
	#if 0
		NeCsvIn Csv;
		Csv.ReadCsvFile( sPathAndFilename.c_str() );
		Csv.MakeHashIndicesFromColValuesAtRow( 0 );
		Csv.MakeHashIndicesFromRowValuesAtCol( 0 );

		int iNumRows = Csv.GetNumRows();

		for ( int i = 1; i < iNumRows; i ++ )
		{
			char* pacName        = Csv.GetStringByIntAndHash( i, 0xa3f15249 /* "sName" */ );
			char* pacFile        = Csv.GetStringByIntAndHash( i, 0x2ce99952 /* "sImageFilename" */ );
			char* pacPressedFile = Csv.GetStringByIntAndHash( i, 0xfbf2ce8d /* "sPressedFilename" */ );
			float fScale  = Clamp( Csv.GetFloatByIntAndHash ( i, 0x46ee2ead /* "fScale" */, 1.0f ), 0.0f, FLT_MAX );
			float fAlpha  = Clamp( Csv.GetFloatByIntAndHash ( i, 0x4492e72f /* "fAlpha" */, 1.0f ), 0.0f, 1.0f );
			float fX      = Clamp( Csv.GetFloatByIntAndHash ( i, 0x4d22c733 /* "fX" */, 0.5f ), 0.0f, 1.0f );
			float fY      = Clamp( Csv.GetFloatByIntAndHash ( i, 0x4c22c5a0 /* "fY" */, 0.5f ), 0.0f, 1.0f );

			SoUiButton* pButton = CreateButton( pacName, pacFile, pacPressedFile );
			pButton->SetPosition( fX, fY );
			pButton->SetScale( fScale );
			pButton->SetAlpha( fAlpha );
		}
	#endif
}


