/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#include <string>

#include "cocos2d.h"

#include "JpList.h"


USING_NS_CC;


#define SoUiButtonList JpList< SoUiButton >


class SoDrawDebug;
class SoSprite;


class SoUiButton : public CCNode
{
	public:

		//-----------------------
		// public enums
		enum STATE
		{
			STATE_UNPRESSED,
			STATE_PRESSED,
			STATE_DISABLED,
			STATE_LAST
		};

		enum ANIM_STATE
		{
			ANIM_STATE_HIDDEN,
			ANIM_STATE_SHOWING,
			ANIM_STATE_ANIMATE_IN,
			ANIM_STATE_ANIMATE_IN_UPDATE,
			ANIM_STATE_ANIMATE_OUT,
			ANIM_STATE_LAST
		};


		//-----------------------
		// public functions

		virtual ~SoUiButton() { Deinit(); }

		void Init
		( 
			CCNode* pParentNode,
			const std::string& sButtonName,
			const std::string& sUnpressedTextureName,
			const std::string& sPressedTextureName,
			const std::string& sDisabledTextureName 
		);

		void InitInvisibleButton
		( 
			CCNode* pParentNode,
			const std::string& sButtonName,
			float fWidth,
			float fHeight
		);


		void Deinit();
		void Update( float dt );

		bool IsActive() { return _bIsActive; }
		void SetIsActive( bool b ) { _bIsActive = b; }

		bool IsUsable();

		void Hide();
		void Enable( bool b ); 
		void TransitionIn(); 
		void TransitionOut(); 

		void SetTransitionInDelay( float fSeconds ) { _fTransitionInDelay = fSeconds; }

		bool TouchIsInside( CCTouch* pTouch, float fExtraMarginInPixels );

		const std::string& GetButtonName() { return _sButtonName; }
		void SetButtonName( const std::string& sName ) { _sButtonName = sName; }

		std::string& GetUnpressedTextureName() { return _sUnpressedTextureName; }
		void SetUnpressedTextureName( const std::string& sName ) { _sUnpressedTextureName = sName; }

		std::string& GetPressedTextureName() { return _sPressedTextureName; }
		void SetPressedTextureName( const std::string& sName ) { _sPressedTextureName = sName; }

		STATE GetState() { return _eState; }
		void SetState( STATE e ) { _eState = e; }

		bool IsPressed() { return _eState == STATE_PRESSED; }
		void SetPressed() { _eStateToSwitchTo = STATE_PRESSED; }

		bool IsUnpressed() { return _eState == STATE_UNPRESSED; }
		void SetUnpressed() { _eStateToSwitchTo = STATE_UNPRESSED; }

		float GetAlpha() { return _fAlpha; }
		void SetAlpha( float f ) { _fAlpha = f; }

		float GetWidth() { return _fWidth; }
		void SetWidth( float f ) { _fWidth = f; }

		float GetHeight() { return _fHeight; }
		void SetHeight( float f ) { _fHeight = f; }
		
		float GetScale() { return _fScale; }
		void SetScale( float f ) { _fScale = f; }

		static SoUiButton* create();


		// derived from CCNode
		virtual bool init() { return true; }
		virtual void draw(void);
	
	
	private:

		//-----------------------
		// private variables
		std::string _sButtonName;
		std::string _sUnpressedTextureName;
		std::string _sPressedTextureName;
		std::string _sDisabledTextureName;

		SoSprite* _pUnpressedSprite;
		SoSprite* _pPressedSprite;
		SoSprite* _pDisabledSprite;

		CCNode* _pParentNode;

		float _fAlpha;

		float _fWidth;
		float _fHeight;
		float _fHalfWidth;
		float _fHalfHeight;

		float _fScale;

		STATE _eState;
		STATE _eStateToSwitchTo;

		float _fAnimTimer;
		ANIM_STATE _eCurrentAnimState;

		float _fTransitionInDelay;

		bool _bIsActive;

		SoDrawDebug* _pDrawDebug;


		//-----------------------
		// private functions
		SoUiButton();
		bool UpdateAnimateIn();
};





// Note, we derive from CCLayer so we can receive Touch input callbacks
class SoUiButtonManager : public CCLayer  
{
	public:

		//-----------------------
		// public enum and classes
		class Listener
		{
			public:
				Listener() {}
				virtual ~Listener() {}
				virtual void OnUiButtonDown( SoUiButton* pButton ) {}
				virtual void OnUiButtonUp( SoUiButton* pButton ) {}
				virtual void OnUiButtonUpTouchOutside( SoUiButton* pButton ) {}
				virtual void OnUiButtonRub( SoUiButton* pButton ) {}
		};



		//-----------------------
		// public functions

		SoUiButtonManager();
		virtual ~SoUiButtonManager() {};

		void Init( CCNode* pParentNode );
		void Deinit();
		void Update( float dt );

		void LoadButtonsFromCsv( std::string sPathAndFilename );

		SoUiButton* CreateButton
		( 
			const std::string& sButtonName, 
			const std::string& sImageName,
			const std::string& sPressedImageName 
		);

		SoUiButton* CreateButtonWithDisabledState
		( 
			const std::string& sButtonName, 
			const std::string& sImageName,
			const std::string& sPressedImageName, 
			const std::string& sDisabledImageName 
		);

		SoUiButton* CreateInvisibleButton
		( 
			const std::string& sButtonName, 
			float fWidth,
			float fHeight
		);

		void AddUiButton( SoUiButton* p );

		void RemoveUiButton( SoUiButton* p ) { _UiButtonList.Remove( p ); }


		void AddListener( Listener* pListener )
		{
			ASSERTF( _bListenerListLocked == false, ("Can't add while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.AddUniquely( pListener, 0 );
		}

		void RemoveListener( Listener* pListener )
		{
			ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.Remove( pListener );
		}
		
		void RemoveAllListeners()
		{
			ASSERTF( _bListenerListLocked == false, ("Can't remove while list is locked.") )
			if ( _bListenerListLocked == false )
				_ListenerList.RemoveAll();
		}


		// For a button to be pressed, the touch and untouch both have to be inside the button.
		// In practice, lots of intended button presses start with touches inside, but untouches just
		// outside of the button, causing the press to fail.  This doesn't feel right to the user.
		// This function is for making the untouch in/out test more forgiving.
		void SetExtraMarginInPixelsForButtonUpTest( float f ) { _fExtraMarginInPixelsForButtonUpTest = f; }


		virtual void ccTouchesBegan(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);
		virtual void ccTouchesMoved(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);
		virtual void ccTouchesEnded(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);
		virtual void ccTouchesCancelled(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent);

	private:


		//-----------------------
		// private variables
		JpList< Listener > _ListenerList;
		bool _bListenerListLocked;


		SoUiButtonList _UiButtonList;
		CCNode* _pParentNode;

		float _fExtraMarginInPixelsForButtonUpTest;


		//-----------------------
		// private functions

		void BroadcastOnUiButtonDown( SoUiButton* pButton );
		void BroadcastOnUiButtonUp( SoUiButton* pButton );
		void BroadcastOnUiButtonUpTouchOutside( SoUiButton* pButton );
		void BroadcastOnUiButtonRub( SoUiButton* pButton );
};





template< class TYPE >
class SoUiButtonListener : public SoUiButtonManager::Listener
{
	public:
		//-----------------------
		// public functions

		SoUiButtonListener() : _pOwner( NULL ) {}
		~SoUiButtonListener() {}
		void SetOwner( TYPE* p ) { _pOwner = p; }

		virtual void OnUiButtonDown( SoUiButton* pButton ) { _pOwner->OnUiButtonDown( pButton ); }
		virtual void OnUiButtonUp( SoUiButton* pButton ) { _pOwner->OnUiButtonUp( pButton ); }
		virtual void OnUiButtonUpTouchOutside( SoUiButton* pButton ) { _pOwner->OnUiButtonUpTouchOutside( pButton ); }
		virtual void OnUiButtonRub( SoUiButton* pButton ) { _pOwner->OnUiButtonRub( pButton ); }

	private:

		//-----------------------
		// private variables.
		TYPE* _pOwner;


		//-----------------------
		// private functions.
		SoUiButtonListener( SoUiButtonListener& Copy )
		{
			ASSERTF( 0, ( "\nCannot copy instances of this class" ) );
		}
};
