/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#if DCG_IS_TARGET_IOS
	#include <sys/utsname.h>
	#include <mach/mach.h>
	#include <mach/mach_time.h>
#endif

#if DCG_IS_TARGET_ANDROID
	#include <android/log.h>
#endif

#if DCG_IS_TARGET_WIN32
	#include <windows.h>
	#include <ShellAPI.h>
	#include "direct.h"
#endif



#include <string.h>


#include "JpDebug.h"
#include "JpFileUtils.h"
#include "JpNoise.h"

#include "SoCamera.h"
#include "SoFsmStateMain.h"
#include "SoSaveSystem.h"
#include "SoSprite.h"
#include "SoSpriteAnim.h"
#include "SoUtils.h"


// Including this before SoUtils.h above, creates a compile error in sqlite.h.  
// No idea why...moving it here for now.
// mdomowicz 2011_09_16
#include "JpRandomStrong.h"

#include "mmgrChooser.h"

JpVector2f SoUtils::_v2fOffWorld( 999999.0f, 999999.0f );
JpVector2f SoUtils::_v2fZero( 0.0f, 0.0f );

char SoUtils::_acTempResults[ 256 ] = "";
SoUtils::DEVICE_TYPE SoUtils::_eDeviceType = SoUtils::DEVICE_TYPE_LAST;

float SoUtils::_fSpriteZoom = 1.0f;

float SoUtils::_fSpriteBaseWidth = 32.0f;
float SoUtils::_fSpriteBaseHeight = 32.0f;

bool  SoUtils::_bFontsAreLoaded = false;



/******************************************************************************
 FUNCTION:  ccc4FFromccc4B
******************************************************************************/
ccColor4B SoUtils::ccc4BFromccc4F(ccColor4F f)
{
	ccColor4B c4 = { (char)(f.r*255.0f), (char)(f.g*255.0f), (char)(f.b*255.0f), (char)(f.a*255.0f) };
	return c4;
}



/******************************************************************************
 FUNCTION:  ccc3BFromFloats
******************************************************************************/
ccColor3B SoUtils::ccc3BFromFloats( float fR, float fG, float fB )
{
	ccColor3B c3 = { (char)(fR*255.0f), (char)(fG*255.0f), (char)(fB*255.0f) };
	return c3;
}



/******************************************************************************
 FUNCTION:  ccc4BFromFloats
******************************************************************************/
ccColor4B SoUtils::ccc4BFromFloats( float fR, float fG, float fB, float fA )
{
	ccColor4B c4 = { (char)(fR*255.0f), (char)(fG*255.0f), (char)(fB*255.0f), (char)(fA*255.0f) };
	return c4;
}



/******************************************************************************
 FUNCTION:  ccc4FFromccc4B
******************************************************************************/
ccColor4B SoUtils::ccc4BFrom3B( ccColor3B b )
{
	ccColor4B c4 = { b.r, b.g, b.b, 255 };
	return c4;
}



/******************************************************************************
 FUNCTION:  ToCcp
******************************************************************************/
CCPoint SoUtils::ToCcp( JpVector2f vVector )
{
	CCPoint ccpVector( vVector.X, vVector.Y );

	return ccpVector;
}



/******************************************************************************
 FUNCTION:  ToJpVector2f
******************************************************************************/
JpVector2f SoUtils::ToJpVector2f( CCPoint vCCPoint )
{
	JpVector2f vVector( vCCPoint.x, vCCPoint.y );

	return vVector;
}



/******************************************************************************
FUNCTION:  AddRelativePathToSaveData
******************************************************************************/
const char* SoUtils::AddRelativePathToSaveData( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		sprintf( _acTempResults, "%sSave/%s", CCFileUtils::sharedFileUtils()->getWriteablePath().c_str(), pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Save\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Saves/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToTexture
******************************************************************************/
const char* SoUtils::AddRelativePathToTexture( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Textures\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Textures/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToSfx
******************************************************************************/
const char* SoUtils::AddRelativePathToSfx( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Sounds\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Sounds/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToCsv
******************************************************************************/
const char* SoUtils::AddRelativePathToCsv( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Csv\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Csv/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToParticles
******************************************************************************/
const char* SoUtils::AddRelativePathToFootprints( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Footprints\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Footprints/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToParticles
******************************************************************************/
const char* SoUtils::AddRelativePathToParticles( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Particles\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Particles/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToIni
******************************************************************************/
const char* SoUtils::AddRelativePathToIni( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Ini\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Ini/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToFont
******************************************************************************/
const char* SoUtils::AddRelativePathToFont( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Fonts\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Fonts/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToAnimation
******************************************************************************/
const char* SoUtils::AddRelativePathToAnimation( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Animations\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Animations/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToScreenshots
******************************************************************************/
const char* SoUtils::AddRelativePathToScreenshots( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Screenshots\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Screenshots/%s", pacFilename );
		return _acTempResults;
	#endif
}


/******************************************************************************
 FUNCTION:  AddRelativePathToPrefab
******************************************************************************/
const char* SoUtils::AddRelativePathToPrefab( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Prefab\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Prefab/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  AddRelativePathToScript
******************************************************************************/
const char* SoUtils::AddRelativePathToScript( const char* pacFilename )
{
	#if DCG_IS_TARGET_IOS
		return pacFilename;
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Scripts\\%s", pacFilename );
		return _acTempResults;
	#elif DCG_IS_TARGET_ANDROID
		sprintf( _acTempResults, "Scripts/%s", pacFilename );
		return _acTempResults;
	#endif
}



/******************************************************************************
 FUNCTION:  CreateFullPathWithRelativePathAndFilename
******************************************************************************/
void SoUtils::CreateFullPathWithRelativePathAndFilename( char* pacResult, const char* pacPathAndFileName )
{
	#if DCG_IS_TARGET_WIN32
		std::string sFullPath = CCFileUtils::sharedFileUtils()->fullPathFromRelativePath( pacPathAndFileName );
		sprintf( pacResult, "%s", sFullPath.c_str() );
	#elif DCG_IS_TARGET_IOS
		std::string sFullPath = CCFileUtils::sharedFileUtils()->fullPathFromRelativePath( pacPathAndFileName );
		sprintf( pacResult, "%s", sFullPath.c_str() );
	#elif DCG_IS_TARGET_ANDROID
		CCFileUtils* pFileUtils = CCFileUtils::sharedFileUtils();
		sprintf( pacResult, "%s%s", pFileUtils->getWriteablePath().c_str(), pacPathAndFileName );
	#endif
}



/******************************************************************************
 FUNCTION:  CreateSaveDirectory
******************************************************************************/
void SoUtils::CreateSaveDirectory()
{
	#if DCG_IS_TARGET_WIN32
		JpFileUtils::CreateDirectory( SoUtils::AddRelativePathToSaveData( "" ) );
	#elif DCG_IS_TARGET_IOS
		JpFileUtils::CreateDirectory( SoUtils::AddRelativePathToSaveData( "" ) );
	#elif DCG_IS_TARGET_ANDROID
		JpFileUtils::CreateDirectory( SoUtils::AddRelativePathToSaveData( "" ) );
	#else
		ASSERTF( 0, ( "\No directory created for save games.  Not supported by this platform." ) );
	#endif
}



/******************************************************************************
 FUNCTION:  GetiPadRatio
 NOTES:     This is for the situation where you can only use one scaler
            and have to pick between X or Y scaling.  Using this function
            in these cases will make the choice consistent.
******************************************************************************/
float SoUtils::GetiPadRatio()
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	return SizeInPixels.width / 1024.0f;
}



/******************************************************************************
 FUNCTION:  GetiPadRatioX
******************************************************************************/
float SoUtils::GetiPadRatioX()
{
	// Use this if you want to stretch the horizontal axis, even if it means
	// changing the aspect ratio.
	//CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	//return SizeInPixels.width / 1024.0f;

	// Use this if you want to stretch the horizontal axis proportionally to
	// the stretch of the vertical axis, thus preserving the aspect ratio.
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	return SizeInPixels.height / 768.0f;
}



/******************************************************************************
 FUNCTION:  GetiPadRatioY
******************************************************************************/
float SoUtils::GetiPadRatioY()
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	return SizeInPixels.height / 768.0f;
}



/******************************************************************************
 FUNCTION:  GetTrueiPadRatioX
 NOTE:      You almost always want to use the vanilla GetiPadRatioX
            which, depending on how it's setup, will either ignore or
            honor the aspect ratio.  However, sometimes you want to 
            ignore the aspect ratio no matter what, and so this function
            exists.  mdomowicz 2013_10_15
******************************************************************************/
float SoUtils::GetTrueiPadRatioX()
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	return SizeInPixels.width / 1024.0f;
}



/******************************************************************************
 FUNCTION:  RelativeToPixelX
******************************************************************************/
float SoUtils::RelativeToPixelX( float fRelativeX )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	return ( fRelativeX * SizeInPixels.width );
}



/******************************************************************************
 FUNCTION:  RelativeToPixelY
******************************************************************************/
float SoUtils::RelativeToPixelY( float fRelativeY )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	return ( fRelativeY * SizeInPixels.height );
}



/******************************************************************************
 FUNCTION:  RelativeToPixelXy
******************************************************************************/
CCPoint SoUtils::RelativeToPixelXy( float fRelativeX, float fRelativeY )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	float fX = ( fRelativeX * SizeInPixels.width );
	float fY = ( fRelativeY * SizeInPixels.height );

	return CCPoint( fX, fY );
}



/******************************************************************************
 FUNCTION:  iPadPixelToPixelX
******************************************************************************/
float SoUtils::iPadPixelToPixelX( float fiPadPixelX )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	float fRatioX = SizeInPixels.width / 1024.0f;
	return fiPadPixelX * fRatioX;
}



/******************************************************************************
 FUNCTION:  iPadPixelToPixelY
******************************************************************************/
float SoUtils::iPadPixelToPixelY( float fiPadPixelY )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	float fRatioY = SizeInPixels.height / 768.0f;
	return fiPadPixelY * fRatioY;
}



/******************************************************************************
 FUNCTION:  iPadPixelToPixelXy
******************************************************************************/
CCPoint SoUtils::iPadPixelToPixelXy( float fiPadPixelX, float fiPadPixelY )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();

	float fRatioX = SizeInPixels.width / 1024.0f;
	float fRatioY = SizeInPixels.height / 768.0f;

	float fScaledX = fiPadPixelX * fRatioX;
	float fScaledY = fiPadPixelY * fRatioY;

	return ccp( fScaledX, fScaledY );
}



/******************************************************************************
 FUNCTION:  GetCenteringX
******************************************************************************/
float SoUtils::GetCenteringX( float fWidthOfObjectToBeCentered )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	return ( SizeInPixels.width / 2 ) - ( fWidthOfObjectToBeCentered / 2 );
}



/******************************************************************************
 FUNCTION:  GetCenteringY
******************************************************************************/
float SoUtils::GetCenteringY( float fHeightOfObjectToBeCentered )
{
	CCSize SizeInPixels = CCDirector::sharedDirector()->getWinSize();
	return ( SizeInPixels.height / 2 ) - ( fHeightOfObjectToBeCentered / 2 );
}



/******************************************************************************
 FUNCTION:  TransformWorldToScreenSpace
******************************************************************************/
JpVector2f SoUtils::TransformWorldToScreenSpace( const JpVector2f& vWorldSpacePos )
{
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->GetCurrentCamera();
	
	if ( pCameraManager && pCamera )
		return pCamera->TransformWorldToScreenSpace( vWorldSpacePos );

	return _v2fZero;
}



/******************************************************************************
 FUNCTION:  TransformScreenToWorldSpace
******************************************************************************/
JpVector2f SoUtils::TransformScreenToWorldSpace( const JpVector2f& vScreenSpacePos )
{
	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->GetCurrentCamera();
	if ( pCameraManager && pCamera )
		return pCamera->TransformScreenToWorldSpace( vScreenSpacePos );

	return _v2fZero;
}



/******************************************************************************
 FUNCTION:  CreateBmFontLabel
******************************************************************************/
SoLabelBmFont* SoUtils::CreateBmFontLabel
( 
	CCNode* pParent, 
	const std::string& sText, 
	const std::string& sFontName, 
	cocos2d::CCTextAlignment Alignment 
)
{
	CCSize size = CCDirector::sharedDirector()->getWinSize();

	SoLabelBmFont* pLabel = SoLabelBmFont::create
	(
		sText.c_str(), 
		SoUtils::AddRelativePathToFont( sFontName.c_str() ), 
		size.width, 
		Alignment
	);

	CCPoint AnchorPoint = CCPoint( 0, 0 );
	if ( Alignment == kCCTextAlignmentCenter )
		AnchorPoint = CCPoint( 0.5f, 0.5f );

	pLabel->setColor( JpUtils::_White );
	pLabel->setOpacity( 255.0f );
	pLabel->retain();
	pLabel->setAnchorPoint( AnchorPoint );
	pLabel->setScaleX( SoUtils::GetiPadRatioX() );
	pLabel->setScaleY( SoUtils::GetiPadRatioY() );
	pLabel->setVisible( false );

	pParent->addChild( pLabel );

	return pLabel;
}



/******************************************************************************
 FUNCTION:  CreateSpriteAnim
******************************************************************************/
SoSpriteAnim* SoUtils::CreateSpriteAnim
( 
	CCNode* pParent, 
	const char* pacPathAndFilenameSprite, 
	const char* pacPathAndFilenamePlist 
)
{
	SoSpriteAnim* pAdvanceSprite;
	pAdvanceSprite = SoSpriteAnim::create();
	pAdvanceSprite->retain();
	pAdvanceSprite->init();    // Must call init for latest version cocos2d-x
	pParent->addChild( pAdvanceSprite );

	pAdvanceSprite->addFrames
	(
		pacPathAndFilenamePlist,
		pacPathAndFilenameSprite
	);

	return pAdvanceSprite;
}



/******************************************************************************
 FUNCTION:  CreateSprite
******************************************************************************/
SoSprite* SoUtils::CreateSprite( CCNode* pParent, const char* pacPathAndFilename )
{
	SoSprite* pSprite = SoSprite::CreateScaledToDevice( pacPathAndFilename );
	pSprite->retain();
	pSprite->setPosition( ccp( 0.0f, 0.0f ) );
	pParent->addChild( pSprite );
	pSprite->setVisible( false );
	return pSprite;
}



/******************************************************************************
 FUNCTION:  RemoveAndDestroyNode
******************************************************************************/
void SoUtils::RemoveAndDestroyNode( CCNode* pParent, CCNode* pNodeToDestroy )
{
	ASSERTF( pParent, ( "\npParent is NULL!" ) );
	if ( pParent && pNodeToDestroy )
	{
		pParent->removeChild( pNodeToDestroy, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		pNodeToDestroy->release();
	}
}



/******************************************************************************
 FUNCTION:  Perlin2dWrapper
 PURPOSE:   The JpNoise2D function can be a bit finiky to work with - getting
            deseriable results from discrete integer X and Y coordinates can be 
            tricky.
            This function helps make sure a given X and Y pair are converted to
            input suitable for JpNoise so that JpNoise will output useful
            values.  Thus, the user can do something like this:

                for ( int i = 0; i < 100; i ++ )
                    for ( int j = 0; j < 100; j ++ )
                        float f = Perlin2dWrapper( i, j, 123, 0.0416f );

 INPUT:     iRandomSeed
                This defines the perlin heightfield starting point.
                Different values here will result in different landscapes.  
                Keep this value constant as you iterate over x and y.
            fDelta
                This defines how fine our "sample" rate is.  A higher
                value will result in more hills and less gradient.  A smaller
                value will result in fewer hills and more gradient from peak
                to valley.

 NOTES:     for fDelta, I've had good results with the following values;

                fDelta = 1.0f / 60.0f * 2.5;   // 0.0416666
                fDelta = 1.0f / 60.0f * 5.5;   // 0.0916666

            mdomowicz 2009_02_27
******************************************************************************/
float SoUtils::Perlin2dWrapper
( 
	int iX, 
	int iY, 
	unsigned int iRandomSeed, 
	float fDelta 
)
{
	JpRandom LocalRandom;
	LocalRandom.Seed( iRandomSeed );
	
	float fStartX = LocalRandom.GetFloat() * 2000.0f;
	float fStartY = LocalRandom.GetFloat() * 2000.0f;

	float fX = fStartX + fDelta * (float)iX;
	float fY = fStartY + fDelta * (float)iY;

	float fHeight = JpNoise2D( fX, fY );
	return fHeight;
}



/******************************************************************************
 FUNCTION:  DetermineAndCacheDeviceType
******************************************************************************/
void SoUtils::DetermineAndCacheDeviceType()
{
	const char* pacDeviceDescription = GetAppleDeviceDescription();
	hash hDevice = JpHash( pacDeviceDescription );

	switch ( hDevice )
	{
		case 0x60670884 /* "win32" */:
			_eDeviceType = DEVICE_TYPE_WINDOWS_PC; break;
		case 0x99750df3 /* "i386" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPOD_SIMULATOR; break;
		case 0x97c52a31 /* "iPod1,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPOD_TOUCH_1; break;
		case 0x26042d48 /* "iPod2,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPOD_TOUCH_2; break;
		case 0x291de1e3 /* "iPod3,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPOD_TOUCH_3; break;
		case 0x58e9343a /* "iPod4,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPOD_TOUCH_4; break;
		case 0x38e3e944 /* "iPhone1,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPHONE_1; break;
		case 0x3be3edfd /* "iPhone1,2" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPHONE_3G; break;
		case 0x29b69a0d /* "iPhone2,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPHONE_3GS; break;
		case 0x9727deca /* "iPhone3,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPHONE_4; break;
		case 0xe7fb26b3 /* "iPhone4,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPHONE_4S; break;
		case 0xce44e237 /* "iPad1,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPAD_1; break;
		case 0x0920d1e2 /* "iPad2,1" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPAD_2; break;
		case 0x9a9d0c9f /* "iPad3,3" */:
			_eDeviceType = DEVICE_TYPE_IOS_IPAD_3; break;
		default:
			ASSERTF( 0, ( "\nUnable to determine device" ) );
	}
}



/******************************************************************************
 FUNCTION:  GetAppleDeviceDescription
 NOTES:     Got the gist of this from here:
http://stackoverflow.com/questions/1108859/detect-the-specific-iphone-ipod-touch-model
******************************************************************************/
const char* SoUtils::GetAppleDeviceDescription()
{
	#if DCG_IS_TARGET_IOS
		struct utsname systemInfo;
		uname( &systemInfo );
		sprintf( _acTempResults, "%s", systemInfo.machine );	
	#elif DCG_IS_TARGET_WIN32
		sprintf( _acTempResults, "Win32" );
	#else
		sprintf( _acTempResults, "error, unknown hardware" );
	#endif

	return _acTempResults;
}



/******************************************************************************
 FUNCTION:  GenerateStrongRngSeed
 NOTES:     This solves the same problem described in the notes for 
            GenerateDerivedRngSeed(), but instead of requiring a user-provided
            string, this function uses a second Rng algorithm to create
            a derived seed from the provided seed.
******************************************************************************/
hash SoUtils::GenerateStrongRngSeed( JpRandom* pRngWeakSource )
{
	hash hSeed = pRngWeakSource->GetInt();
	JpRandomStrong LocalRngStrong;
	LocalRngStrong.Seed( hSeed );

	return LocalRngStrong.GetInt();
}



/******************************************************************************
 FUNCTION:  GenerateDerivedRngSeed
 NOTES:

 A problem with Rngs is that their output value is used as input to get the 
 next value.  Because of this, "Rng1" cannot be used to generate a seed for
 a "Rng2", because Rng2 would end up generating the exact same numbers as 
 Rng1.  So for example, if Rng1 created this sequence:

    11 42 35 84 25 96 70 80 19 40 ...

 and the first value (11) was used to seed Rng2, Rng2 would create this
 sequence:

    42 35 84 25 96 70 80 19 40 ...

 Derived Rngs fix this and are created by taking the value of an Rng1, and 
 mixing it with another, outside, value.  This creates a value that's
 safe to use as the seed for an Rng2, without worry of value aliasing.

******************************************************************************/
hash SoUtils::GenerateDerivedRngSeed( hash hMasterSeed, const char* pacNewSeedString )
{
	ASSERTF( hMasterSeed, ( "\nProvided rng seed is 0." ) );
	ASSERTF( pacNewSeedString, ( "\nProvided string is null" ) );

	hash hNewSeed = JpHash( pacNewSeedString );
	hash hDerivedSeed = ( hNewSeed ^ hMasterSeed );

	return hDerivedSeed;
}




/******************************************************************************
 FUNCTION:  AlterGuidForCollisionPrevention
 NOTES:     This function is used to alter guids so that it is more or less
            guaranteed that guids are unique, not just in one level of the
            dungeon, but across levels.  Without this, there could be guid
            collisions whenever the user switched levels with items in his
            inventory.  

            This works by using the "num level switches" count as the first
            16 bits of the 32 bit guid.  Thus when the user switches levels
            any guids generated in the new level (either at level start,
            or during level play) are guaranteed to not conflict with any
            guids generated in the past.

            Here are examples of problems that this functions solves:

            1. Hero picks up Item1 in LevelA, goes to LevelB.
            2. Hero picks up Item1 in LevelA, goes to LevelB, drops Item1,
               returns to LevelA picks up Item2, goes to LevelB.

            Note that the original idea was to just use the LevelNode Id as
            the prefix, which would allow the prefix to be 8 bits.  However
            problem 2 above would still exist.

            Using the level switch count as the Id, solves 1 and 2.  But
            it means that Id range must be pretty big.  I used 16 bits
            which allows for 65535 level switches.  It is highly unlikely
            that the player will do that many level switches, so it should
            be enough.  With that, 16 bits remain to define the rest of the
            guid.  That means that for any given stage, only 65535 guids
            are available.  Worse, as soon as 32767 (half the number space)
            guids are used, the chance of generating a guid that conflicts
            is 50% (and when conflicts occur, the guid generator must try 
            again).  But, by my informal estimates, most levels (at 64x48)
            would never have need for more then 6000 unique guids.  So, in
            summary, I think 16 bits for prefix Id, and 16 bits for guid is
            good enough for the game's needs.
            
            If it turns out that more number space is needed, we should 
            consider switching to 64 bit guids.

			mdomowicz 2011_11_10
******************************************************************************/
guid SoUtils::AlterGuidForCollisionPrevention( guid g )
{
/*
	SoGameDungeon* pDungeon = SoGameDungeon::GetInstance();

	//-----------------------
	// determine the level id.
	int iLevelId = pDungeon->GetNumLevelSwitchesPerformed();	

	//-----------------------
	// clear the top 8 bits
	int iClearingMask = B32(00000000,00000000,11111111,11111111);
	g = g & iClearingMask;

	//-----------------------
	// Set the top 8 bits to the level id
	iLevelId = iLevelId << 16;
	g = g | iLevelId;

	return g;
*/
	return 0;
}



/******************************************************************************
 FUNCTION:  SetRenderZOrder
******************************************************************************/
void SoUtils::SetRenderZOrder
( 
	CCNode* pParentNode, 
	CCNode* pChildToOrder, 
	int iZWhereLargerMeansCloser 
)
{
	pParentNode->reorderChild( pChildToOrder, iZWhereLargerMeansCloser );
}



/******************************************************************************
 FUNCTION:  GetTickCount
******************************************************************************/
#if DCG_IS_TARGET_IOS
	uint64_t SoUtils::GetTickCount()
	{
		static mach_timebase_info_data_t sTimebaseInfo;
		uint64_t machTime = mach_absolute_time();

		// Convert to nanoseconds - if this is the first time we've run, get the timebase.
		if (sTimebaseInfo.denom == 0 )
		{
			(void) mach_timebase_info(&sTimebaseInfo);
		}

		// Convert the mach time to milliseconds
		uint64_t millis = ((machTime / 1000000) * sTimebaseInfo.numer) / sTimebaseInfo.denom;
		return millis;
	}
#else
	unsigned int SoUtils::GetTickCount()
	{
		ASSERTF( 0, ( "\nThis function is not yet implemented." ) );
		return 0;
	}
#endif


	
/******************************************************************************
 FUNCTION:  TakeScreenshot
 NOTE:      http://www.cocos2d-x.org/boards/6/topics/3485#sthash.UI6Jt5Mk.dpuf}
 ******************************************************************************/
void SoUtils::TakeScreenshot()
{
	#if DCG_IS_TARGET_WIN32
		//-----------------------
		// get the time info
		time_t rawtime;
		time( &rawtime );

		tm* timeinfo;
		timeinfo = localtime( &rawtime );

		char acTimeString[ 32 ] = "";
		strftime( acTimeString, 128, "%Y_%m_%d__%H.%M.%S.png", timeinfo );


		//-----------------------
		// Render the scene to a new texture
		CCSize ccsSize = CCDirector::sharedDirector()->getWinSize();
		CCRenderTexture* pTexture = CCRenderTexture::renderTextureWithWidthAndHeight(
			ccsSize.width, 
			ccsSize.height ); 
		pTexture->setPosition( ccp(ccsSize.width/2, ccsSize.height/2) ); 
		pTexture->begin(); 
		  CCDirector::sharedDirector()->getRunningScene()->visit();
		pTexture->end(); 


		//-----------------------
		// Assemble the file name with the time info
		std::string sPsName = "Achoir-";
		sPsName.append( acTimeString );

		bool bSuccess = false;

		CCImage *pImage = pTexture->newCCImage();


		//-----------------------
		// Write the file
		if (pImage)
		{
			//-----------------------
			// This makes sure the "save" folder exists
			_mkdir( SoUtils::AddRelativePathToScreenshots( "" ) );

			const char* cpFullPath = SoUtils::AddRelativePathToScreenshots( sPsName.c_str() );
			bSuccess = pImage->saveToFile( cpFullPath, true );
		}

		// Delete the image
		CC_SAFE_DELETE(pImage);
		pTexture->release();


		//-----------------------
		// Let us know
		/*
		Disabling this for now, as the "Screenshot!" text ends up appearing
		in the screen shots themselves, which we usually don't want.
		mdomowicz 2013_10_05

		ccColor3B Color = { 255, 255, 255 };
		SoFsmMainLayer* pSharedLayer = SoFsmStateMain::GetSharedMainLayer();
		if ( pSharedLayer )
		{
			SoHudLog* pLog = pSharedLayer->GetSharedHudLog();

			if ( pLog )
			{
				if ( bSuccess )
					pLog->AddMessageF( Color, "Screenshot!" );
				else
					pLog->AddMessageF( Color, "Screenshot FAILED!" );
			}
		}
		*/
	#endif
}



/******************************************************************************
 FUNCTION:  LaunchBrowserAndOpenUrl
 NOTES:     This code was taking from here:
            http://www.cocos2d-x.org/boards/6/topics/11290
******************************************************************************/
void SoUtils::LaunchBrowserAndOpenUrl( char* pacUrl )
{
	#if DCG_IS_TARGET_WIN32
		ShellExecuteA(NULL, "open", pacUrl, NULL, NULL, SW_SHOWNORMAL);
	#elif DCG_IS_TARGET_IOS
		CCApplication::sharedApplication().openURL(pacUrl);
	#elif DCG_IS_TARGET_ANDROID
		CCApplication::sharedApplication().openURL
		(
			"com/domowiczcg/achoir/ApplicationDemo", // the path to ApplicationDemo.java, starting from proj.android/src
			pacUrl
		);
	#endif
}



/******************************************************************************
 FUNCTION:  AndroidDebugPrint
 NOTES:     I was having trouble getting CCLOG to show up in the Eclipse output
			so I added this, which works.  mdomowicz 2014_08_02
******************************************************************************/
void SoUtils::AndroidDebugPrint( const char* pacText, ... )
{
	#if DCG_IS_TARGET_ANDROID
		va_list argsVA;
		const int iMaxSize = 4096;
		char buffer[ iMaxSize ];
		memset( (void*)buffer, 0, iMaxSize );

		va_start( argsVA, pacText );
		vsnprintf( buffer, iMaxSize, pacText, argsVA );
		va_end( argsVA );

		__android_log_write( ANDROID_LOG_INFO, "DCG Project", buffer );
	#endif
}