/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/



#pragma once

#include "stdafx.h"

#include "JpHashTable.h"
#include "JpList.h"


// Don't tweak these flags.  These are not options.  Instead they exist to make
// code easier to read, so that instead of stuff like:
//		removeChild( this, true )  // what does true do here?
// we can do:
//		removeChild( this, !DCG_CLEANUP_ALL_RUNNING_ACTIONS );

#define DCG_CLEANUP_ALL_RUNNING_ACTIONS true




#define BASE_RENDER_ORDER_PARTICLES                  800
#define BASE_RENDER_ORDER_MONEY                    12000
#define BASE_RENDER_ORDER_COORDINATE               13000
#define BASE_RENDER_ORDER_BOARD_SPOT               14000
#define OFFSET_RENDER_ORDER_TILE                       1
#define OFFSET_RENDER_ORDER_HOTEL                      2
#define OFFSET_RENDER_ORDER_PARKINGSPOT                3
#define BASE_RENDER_ORDER_TILE                     15000
#define BASE_RENDER_ORDER_HOTEL                    16000
#define BASE_RENDER_ORDER_PARKING_SPOT             53000


// These are absolute, and not relative to anything.
#define BASE_RENDER_ORDER_SKY_PLANE                0
#define BASE_RENDER_ORDER_HANGDOWN                 170000
#define BASE_RENDER_ORDER_MIST_ABOVE               180000
#define BASE_RENDER_ORDER_LIGHT_OVERLAY            190000
#define BASE_RENDER_ORDER_LIFEBAR                  200000
#define BASE_RENDER_ORDER_HUD                      220000
#define BASE_RENDER_ORDER_HUD_MAP                  220000
#define BASE_RENDER_ORDER_HUD_NOTICE               220000
#define BASE_RENDER_ORDER_DIALOG                   220000
#define BASE_RENDER_ORDER_SHELL                    220000
#define BASE_RENDER_ORDER_KICKER                   223000
#define BASE_RENDER_ORDER_GUI                      226000
#define BASE_RENDER_ORDER_GUI_BUTTON               227000
#define BASE_RENDER_ORDER_GUI_TOGGLE               228000
#define BASE_RENDER_ORDER_GUI_TEXT_ENTRY           229000
#define BASE_RENDER_ORDER_LOG                      230000
#define BASE_RENDER_ORDER_WATERMARK_BUILD_NUMBER   240000
#define BASE_RENDER_ORDER_FADER                    300000
#define BASE_RENDER_ORDER_WATERMARK_PRE_RELEASE    400000



#define GAMERULES_MAX_PLAYERS  6
#define GAMERULES_NUM_HOTELS 7
#define GAMERULES_NUM_STOCKS_PER_HOTEL 25
#define GAMERULES_NUM_TILES_PER_HAND 6
#define GAMERULES_STARTING_MONEY 60


class SoLabelBmFont;
class SoSprite;
class SoSpriteAnim;

class SoUtils
{
	public:

		//-----------------------
		// public enums
		enum DEVICE_TYPE
		{
			DEVICE_TYPE_WINDOWS_PC,
			DEVICE_TYPE_IOS_IPOD_SIMULATOR,
			DEVICE_TYPE_IOS_IPOD_TOUCH_1,
			DEVICE_TYPE_IOS_IPOD_TOUCH_2,
			DEVICE_TYPE_IOS_IPOD_TOUCH_3,
			DEVICE_TYPE_IOS_IPOD_TOUCH_4,
			DEVICE_TYPE_IOS_IPHONE_1,
			DEVICE_TYPE_IOS_IPHONE_3G,
			DEVICE_TYPE_IOS_IPHONE_3GS,
			DEVICE_TYPE_IOS_IPHONE_4,
			DEVICE_TYPE_IOS_IPHONE_4S,
			DEVICE_TYPE_IOS_IPAD_1,
			DEVICE_TYPE_IOS_IPAD_2,
			DEVICE_TYPE_IOS_IPAD_3,
			DEVICE_TYPE_LAST
		};


		//-----------------------
		// public functions
		SoUtils() {};
		virtual ~SoUtils() {};


		//-----------------------
		// Relative file locations
		static const char* AddRelativePathToSaveData( const char* pacFilename );
		static const char* AddRelativePathToTexture( const char* pacFilename );
		static const char* AddRelativePathToSfx( const char* pacFilename );
		static const char* AddRelativePathToCsv( const char* pacFilename );
		static const char* AddRelativePathToParticles( const char* pacFilename );
		static const char* AddRelativePathToFootprints( const char* pacFilename );
		static const char* AddRelativePathToIni( const char* pacFilename );
		static const char* AddRelativePathToFont( const char* pacFilename );
		static const char* AddRelativePathToAnimation( const char* pacFilename );
		static const char* AddRelativePathToScreenshots( const char* pacFilename );
		static const char* AddRelativePathToPrefab( const char* pacFilename );
		static const char* AddRelativePathToScript( const char* pacFilename );
		static void CreateFullPathWithRelativePathAndFilename( char* pacResult, const char* pacPathAndFileName );
		static void CreateSaveDirectory();


		//-----------------------
		// multi-resolution handling
		static float GetiPadRatio();
		static float GetiPadRatioX();
		static float GetiPadRatioY();

		static float GetTrueiPadRatioX();

		static float RelativeToPixelX( float fRelativeX );
		static float RelativeToPixelY( float fRelativeY );
		static CCPoint RelativeToPixelXy( float fRelativeX, float fRelativeY );

		static float iPadPixelToPixelX( float fiPadPixelX );
		static float iPadPixelToPixelY( float fiPadPixelY );
		static CCPoint iPadPixelToPixelXy( float fiPadPixelX, float fiPadPixelY );


		static float GetCenteringX( float fWidthOfObjectToBeCentered );
		static float GetCenteringY( float fHeightOfObjectToBeCentered );


		static SoLabelBmFont* CreateBmFontLabel
		( 
			CCNode* pParent, 
			const std::string& sText, 
			const std::string& sFontName, 
			cocos2d::CCTextAlignment Alignment 
		);

		static SoSpriteAnim* CreateSpriteAnim
		( 
			CCNode* pParent, 
			const char* pacPathAndFilenameSprite, 
			const char* pacPathAndFilenamePlist 
		);


		//-----------------------
		// transforms
		static JpVector2f TransformWorldToScreenSpace( const JpVector2f& vWorldSpacePos );
		static JpVector2f TransformScreenToWorldSpace( const JpVector2f& vScreenSpacePos );


		//-----------------------
		// sprite creation/destruction
		static SoSprite* CreateSprite( CCNode* pParent, const char* pacPathAndFilename );
		static void RemoveAndDestroyNode( CCNode* pParent, CCNode* pNodeToDestroy );


		//-----------------------
		// Sprite sizes
		static float GetSpriteZoom() { return _fSpriteZoom; }
		static void SetSpriteZoom( float f ) { _fSpriteZoom = f; }

		static float GetBaseSpriteWidth() { return _fSpriteBaseWidth; }
		static float GetBaseSpriteHeight() { return _fSpriteBaseHeight; }
		static float GetBaseSpriteWidthAllRes() { return ( _fSpriteBaseWidth * GetiPadRatioX() ); }
		static float GetBaseSpriteHeightAllRes() { return ( _fSpriteBaseHeight * GetiPadRatioY() ); }

		static float GetZoomedSpriteWidth() { return ( _fSpriteBaseWidth * _fSpriteZoom ); }
		static float GetZoomedSpriteHeight() { return ( _fSpriteBaseHeight * _fSpriteZoom ); }
		static float GetZoomedSpriteWidthAllRes() { return ( _fSpriteBaseWidth * _fSpriteZoom * GetiPadRatioX() ); }
		static float GetZoomedSpriteHeightAllRes() { return ( _fSpriteBaseHeight * _fSpriteZoom * GetiPadRatioY() ); }


		//-----------------------
		// Random numbers
		static hash GenerateStrongRngSeed( JpRandom* pRngWeakSource );
		static hash GenerateDerivedRngSeed( hash hMasterSeed, const char* pacNewSeedString );



		//-----------------------
		// device detection
		static DEVICE_TYPE GetDeviceType() { return _eDeviceType; }
		static void DetermineAndCacheDeviceType();
		static const char* GetAppleDeviceDescription();


		//-----------------------
		// miscellaneous 
		static float Perlin2dWrapper
		( 
			int iX, 
			int iY, 
			unsigned int iRandomSeed, 
			float fDelta 
		);

		static inline void StringToUpper( char* cpString )
		{    
			while ( *cpString )    
			{        
				*cpString = toupper(*cpString);
				cpString ++;
			}
		}


		#if DCG_IS_TARGET_IOS
			static uint64_t GetTickCount();
		#else
			static unsigned int GetTickCount();
		#endif

		static guid AlterGuidForCollisionPrevention( guid g );

		static void SetRenderZOrder( CCNode* pParentNode, CCNode* pChildToOrder, int iZWhereLargerMeansCloser );

		// A few ccColor helper functions that seem to
		// be missing from ccTypes.h
		static ccColor4B ccc4BFromccc4F(ccColor4F f);
		static ccColor3B ccc3BFromFloats( float fR, float fG, float fB );
		static ccColor4B ccc4BFromFloats( float fR, float fG, float fB, float fA );
		static ccColor4B ccc4BFrom3B( ccColor3B b );

		// JpVector2f / CCPoint conversion
		static CCPoint ToCcp( JpVector2f vVector );
		static JpVector2f ToJpVector2f( CCPoint vCCPoint );

		static void TakeScreenshot();

		static void LaunchBrowserAndOpenUrl( char* pacUrl );


		static void AndroidDebugPrint( const char* pacText, ... );

		//-----------------------
		// global values 
		static JpVector2f _v2fOffWorld;
		static JpVector2f _v2fZero;


	private:

		//-----------------------
		// private variables
		static DEVICE_TYPE _eDeviceType;
		static char _acTempResults[ 256 ];

		static float _fSpriteZoom;
		static float _fSpriteBaseWidth;
		static float _fSpriteBaseHeight;

		static bool _bFontsAreLoaded;
};





// A few CCPoint helper operators that seem to
// be missing from Cocos.

// GLOBAL VECTOR OVERLOADED OPERATORS
CCPoint operator * ( CCPoint vLeftHandArg, float fRightHandArg );
CCPoint operator * ( float fLeftHandArg, CCPoint vRightHandArg );
CCPoint& operator *= ( CCPoint& vLeftHandArg, float fRightHandArg );
CCPoint operator + ( CCPoint vLeftHandArg, CCPoint vRightHandArg );
CCPoint& operator += ( CCPoint& vLeftHandArg, CCPoint vRightHandArg );
CCPoint operator - ( CCPoint vLeftHandArg, CCPoint vRightHandArg );
CCPoint& operator -= ( CCPoint& vLeftHandArg, CCPoint vRightHandArg );
// bool CCPoint == ( CCPoint vLeftHandArg, CCPoint vRightHandArg );
// bool CCPoint != ( CCPoint vLeftHandArg, CCPoint vRightHandArg );
// const CCPoint operator-( CCPoint vArg );



/******************************************************************************
 FUNCTION:  *
 PURPOSE:   used to multiply a vector by a scaler value
 NOTES:     There are two versions of the overloaded * so that the operation
            can be both float * JpVector2 and JpVector2 * float
******************************************************************************/
inline CCPoint operator * ( CCPoint vLeftHandArg, float fRightHandArg )
{
	vLeftHandArg.x *= fRightHandArg;
	vLeftHandArg.y *= fRightHandArg;

	return vLeftHandArg;
}

inline CCPoint operator * ( float fLeftHandArg, CCPoint vRightHandArg )
{
	vRightHandArg.x *= fLeftHandArg;
	vRightHandArg.y *= fLeftHandArg;

	return vRightHandArg;
}




/******************************************************************************
 FUNCTION:  *=
 PURPOSE:   used to multiply a vector by a scaler value
******************************************************************************/
inline CCPoint& operator *= ( CCPoint& vLeftHandArg, float fRightHandArg )
{
	vLeftHandArg.x *= fRightHandArg;
	vLeftHandArg.y *= fRightHandArg;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  +
 PURPOSE:   used to add to vectors together
******************************************************************************/
inline CCPoint operator + ( CCPoint vLeftHandArg, CCPoint vRightHandArg )
{
	vLeftHandArg.x += vRightHandArg.x;
	vLeftHandArg.y += vRightHandArg.y;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  +=
 PURPOSE:   used to add to vectors together
******************************************************************************/
inline CCPoint& operator += ( CCPoint& vLeftHandArg, CCPoint vRightHandArg )
{
	vLeftHandArg.x += vRightHandArg.x;
	vLeftHandArg.y += vRightHandArg.y;

	return vLeftHandArg;
}




/******************************************************************************
 FUNCTION:  -
 PURPOSE:   used to subtract two vectors
 ARGS:      vRightHandArg gets subtracted from vLeftHandArg
******************************************************************************/
inline CCPoint operator - ( CCPoint vLeftHandArg, CCPoint vRightHandArg )
{
	vLeftHandArg.x -= vRightHandArg.x;
	vLeftHandArg.y -= vRightHandArg.y;

	return vLeftHandArg;
}





/******************************************************************************
 FUNCTION:  -=
 PURPOSE:   used to subtract two vectors
 ARGS:      vRightHandArg gets subtracted from vLeftHandArg
******************************************************************************/
inline CCPoint& operator -= ( CCPoint& vLeftHandArg, CCPoint vRightHandArg )
{
	vLeftHandArg.x -= vRightHandArg.x;
	vLeftHandArg.y -= vRightHandArg.y;

	return vLeftHandArg;
}




// /******************************************************************************
//  FUNCTION:  ==
//  PURPOSE:   used to compare two vectors for equality
// ******************************************************************************/
// inline bool operator == ( CCPoint vLeftHandArg, CCPoint vRightHandArg )
// {
// 	return
// 	(
// 		vLeftHandArg.x == vRightHandArg.x &&
// 		vLeftHandArg.y == vRightHandArg.y
// 	);
// }
// 
// 
// 
// 
// /******************************************************************************
//  FUNCTION:  !=
//  PURPOSE:   used to compare two vectors for inequality
// ******************************************************************************/
// inline bool operator != ( CCPoint vLeftHandArg, CCPoint vRightHandArg )
// {
// 	return
// 	(
// 		vLeftHandArg.x != vRightHandArg.x ||
// 		vLeftHandArg.y != vRightHandArg.y
// 	);
// }
// 
// 
// 
// 
// 
// /******************************************************************************
//  FUNCTION:  -
//  PURPOSE:   unary negation
// ******************************************************************************/
// inline const CCPoint operator-( CCPoint vArg )
// {
// 	return vArg * -1.0;
// }
// 



