/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "JpString.h"

#include "SoActor.h"
#include "SoFontDefs.h"
#include "SoWatermark.h"
#include "SoLabelBmFont.h"
#include "SoSprite.h"
#include "SoUtils.h"

#include "mmgrChooser.h"


USING_NS_CC;



/******************************************************************************
 FUNCTION:  SoWatermark
******************************************************************************/
SoWatermark::SoWatermark() :
_pParentNode( NULL ),
_vPosition( 0.0f, 0.0f ),
_pLabel( NULL )
{
}



/******************************************************************************
 FUNCTION:  ~SoWatermark
******************************************************************************/
SoWatermark::~SoWatermark()
{
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoWatermark::Init( CCNode* pParentNode )
{
	_pParentNode = pParentNode;

	float fX = 512.0f * SoUtils::GetTrueiPadRatioX();
	float fY = 0.0f * SoUtils::GetiPadRatioY();
	_vPosition.Init( fX, fY );

	_pLabel = CreateBmFontLabel( "", SoFontDefs::GetWatermark(), kCCTextAlignmentCenter );
	_pLabel->setOpacity( 255 );
	_pLabel->setScale( 0.75f * SoUtils::GetiPadRatio() );
	_pLabel->setAnchorPoint( ccp( 0.5, 0.0f ) );

	_pLabel->SetPosition( _vPosition );
	_pLabel->setVisible( true );

	SetTextColor( JpUtils::_White );

	SoUtils::SetRenderZOrder( _pParentNode, _pLabel, BASE_RENDER_ORDER_WATERMARK_PRE_RELEASE );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoWatermark::Deinit()
{
	if ( _pLabel )
	{
		_pParentNode->removeChild( _pLabel, DCG_CLEANUP_ALL_RUNNING_ACTIONS );
		_pLabel->release();
		_pLabel = NULL;
	}
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoWatermark::Update( float dt )
{
	_pLabel->SetPosition( _vPosition );
}



/******************************************************************************
 FUNCTION:  SetLabelText
******************************************************************************/
void SoWatermark::SetLabelText( const char* pacText )
{
	if ( _pLabel )
		_pLabel->setString( pacText );
	
}



/******************************************************************************
 FUNCTION:  SetTextColor
******************************************************************************/
void SoWatermark::SetTextColor( const ccColor3B& Color ) 
{ 
	_TextColor = Color; 
	_pLabel->setColor( Color );
}



/******************************************************************************
 FUNCTION:  CreateBmFontLabel
******************************************************************************/
SoLabelBmFont* SoWatermark::CreateBmFontLabel( const std::string& sText, const std::string& sFontName, cocos2d::CCTextAlignment Alignment )
{
	CCSize size = CCDirector::sharedDirector()->getWinSize();

	SoLabelBmFont* pLabel = SoLabelBmFont::create
	(
		sText.c_str(), 
		SoUtils::AddRelativePathToFont( sFontName.c_str() ), 
		size.width, 
		Alignment
	);

	CCPoint AnchorPoint = CCPoint( 0, 0 );
	if ( Alignment == kCCTextAlignmentCenter )
		AnchorPoint = CCPoint( 0.5f, 0.5f );

	pLabel->setColor( _TextColor );
	pLabel->setOpacity( 255.0f );
	pLabel->retain();
	pLabel->setAnchorPoint( AnchorPoint );
	pLabel->setScaleX( SoUtils::GetiPadRatioX() );
	pLabel->setScaleY( SoUtils::GetiPadRatioY() );

	_pParentNode->addChild(pLabel, BASE_RENDER_ORDER_GUI + 1 );

	return pLabel;
}









/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoWatermarkPreRelease::Init( CCNode* pParentNode )
{
	SoWatermark::Init( pParentNode );

	char acText[ 256 ] = "";

	// Finalizing the game - disabling this for now.  mdomowicz 2014_03_31
	//sprintf( acText, "Pre-release PreRelease.  Do not distribute.  Rev ??? Built on %s, at %s", __DATE__, __TIME__ );

	SetLabelText( acText );
	SetTextColor( JpUtils::_White );

	SoUtils::SetRenderZOrder( _pParentNode, _pLabel, BASE_RENDER_ORDER_WATERMARK_PRE_RELEASE );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoWatermarkPreRelease::Deinit()
{
	SoWatermark::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoWatermarkPreRelease::Update( float dt )
{
	SoWatermark::Update( dt );
}









/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoWatermarkBuildNumber::Init( CCNode* pParentNode )
{
	SoWatermark::Init( pParentNode );

	char acText[ 256 ] = "";

	sprintf( acText, "Build: 510");

	SetLabelText( acText );
	SetTextColor( JpUtils::_White );

	_pLabel->setOpacity( 128 );
	_pLabel->setAnchorPoint( ccp( 1.0, 0.0f ) );

	float fX = 1015.0f * SoUtils::GetTrueiPadRatioX();
	float fY = 0.0f * SoUtils::GetiPadRatioY();
	_vPosition.Init( fX, fY );
	_pLabel->SetPosition( _vPosition );



	SoUtils::SetRenderZOrder( _pParentNode, _pLabel, BASE_RENDER_ORDER_WATERMARK_BUILD_NUMBER );
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoWatermarkBuildNumber::Deinit()
{
	SoWatermark::Deinit();
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoWatermarkBuildNumber::Update( float dt )
{
	SoWatermark::Update( dt );
}
