/*-----------------------------------------------------------------------------
 |  Copyright (c) 2011 Domowicz Creative Group, LLC    All Rights Reserved
 +----------------------------------------------------------------------------*/


#include "stdafx.h"

#include "SoCamera.h"
#include "SoActorVisibleBoardSpot.h"
#include "SoActorVisibleTile.h"
#include "SoWorkerParkingSpot.h"

#include "mmgrChooser.h"


USING_NS_CC;




/******************************************************************************
 FUNCTION:  SoWorkerParkingSpot
******************************************************************************/
SoWorkerParkingSpot::SoWorkerParkingSpot() :
_pSprite( NULL ),
_pParentNode( NULL ),
_pGameBitTile( NULL ),
_fTimer( 0.0f ),
_iTouchState( 0 ),
_bIsClickedByUser( false ),
_bIsActivated( false ),
_eType( TYPE_LAST )
{
}



/******************************************************************************
 FUNCTION:  ~SoWorkerParkingSpot
******************************************************************************/
SoWorkerParkingSpot::~SoWorkerParkingSpot()
{
	Deinit();
}



/******************************************************************************
 FUNCTION:  Init
******************************************************************************/
void SoWorkerParkingSpot::Init( CCNode* pParentNode, TYPE eType )
{
	_pParentNode = pParentNode;
	_eType = eType;

	if ( _eType == TYPE_USABLE )
		_pSprite = SoUtils::CreateSprite( pParentNode, SoUtils::AddRelativePathToTexture( "ParkingSpotIndicator.png" ) );
	else
		_pSprite = SoUtils::CreateSprite( pParentNode, SoUtils::AddRelativePathToTexture( "ParkingSpotIndicatorUnusable.png" ) );

	_pSprite->setVisible( false );
	_pSprite->setOpacity( 0 );
	SoUtils::SetRenderZOrder( _pParentNode, _pSprite, BASE_RENDER_ORDER_PARKING_SPOT );

	_bIsClickedByUser = false;
}



/******************************************************************************
 FUNCTION:  Deinit
******************************************************************************/
void SoWorkerParkingSpot::Deinit()
{
	SoUtils::RemoveAndDestroyNode( _pParentNode, _pSprite );
}



/******************************************************************************
 FUNCTION:  Update
******************************************************************************/
void SoWorkerParkingSpot::Update( float dt )
{
	if ( _pSprite )
	{
		_pSprite->setVisible( _bIsActivated );

		_pSprite->setScaleX( SoUtils::GetSpriteZoom() * SoUtils::GetiPadRatioX() );
		_pSprite->setScaleY( SoUtils::GetSpriteZoom() * SoUtils::GetiPadRatioY() );	


		_fTimer += dt;
		float fPercent = ( 1.0 + sinf( _fTimer * 3.5f ) ) / 2.0f;

		float fAlpha = Interpolate( 0.5f, 1.0f, fPercent );
		_pSprite->setOpacity( fAlpha * 255.0f );
		


		JpVector2f vWorldPos = GetPosition();
		JpVector2f vScreenSpacePos = SoCameraManager::GetInstance()->GetCurrentCamera()->TransformWorldToScreenSpace( vWorldPos );
		_pSprite->SetPosition( vScreenSpacePos );
	}
}



/******************************************************************************
 FUNCTION:  SetGameBitTile
******************************************************************************/
void SoWorkerParkingSpot::SetGameBitTile( SoGameBitTile* p ) 
{ 
	_pGameBitTile = p;

	// As soon as the GameBitTile is set, we can
	// update the render order to match the corresponding
	// VisibleBoardSpot.
	if ( _pSprite )
	{
		SoActorVisibleBoardSpot* pVisibleBoardSpot = _pGameBitTile->GetActorVisibleTile()->GetVisibleBoardSpot();
		SoUtils::SetRenderZOrder( _pParentNode, _pSprite, pVisibleBoardSpot->GetRenderZOrder() + OFFSET_RENDER_ORDER_PARKINGSPOT );
	}
}



/******************************************************************************
 FUNCTION:  GetPosition
******************************************************************************/
JpVector2f SoWorkerParkingSpot::GetPosition()
{
	JpVector2f vPos = SoUtils::_v2fOffWorld;

	if ( _pGameBitTile )
	{
		SoActorVisibleBoardSpot* pVisibleBoardSpot = _pGameBitTile->GetActorVisibleTile()->GetVisibleBoardSpot();
		vPos = pVisibleBoardSpot->GetWorldPosition();
	}

	return vPos;
}



/******************************************************************************
 FUNCTION:  OnInput
******************************************************************************/
bool SoWorkerParkingSpot::OnInput( SoInputDispatcher::InputPacket* pInput )
{
	//-----------------------
	// early out!
	if ( !_bIsActivated )
		return false;


	SoCameraManager* pCameraManager = SoCameraManager::GetInstance();
	SoCamera* pCamera = pCameraManager->FindCamera( "Gameplay" );

	const int STATE_UP = 0;
	const int STATE_DOWN = 10;

	switch ( _iTouchState )
	{
		case STATE_UP:
		{
			if ( pInput->_pSoCcTouchesDownList )
			{
				//-----------------------
				// get the position of the touch, in screen space
				JpVector2f vScreenPos( 0.0f, 0.0f );
				SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pInput->_pSoCcTouchesDownList;
				if ( pNonConstTouchList->GetNumEntries() > 0 )
				{
					CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
					vScreenPos = SoInputTouch::TouchPosToScreenPos( pTouch0->locationInView() );
				}

				//-----------------------
				// convert to world space.
				JpVector2f vWorldPos = pCamera->TransformScreenToWorldSpace( vScreenPos );


				//-----------------------
				if ( IsPositionInside( vWorldPos ) )
					_iTouchState = STATE_DOWN;
			}

			break;
		}

		case STATE_DOWN:
		{
			//-----------------------
			// If the user moves the touch position before lifting
			// we cancel the press.
			// update: disabled this for Achoir. Since achoir doesn't let the user move the
			// camera, we have no reason to cancel the press on touch-movement, and in fact
			// its better if we don't.  mdomowicz 2014_02_17
			// update 2: just leaving a note to myself - if ever I do want to reenable canceling
			// due to touch movement, it's best to add some buffer distance logic, so that
			// canceling doesn't happen on the slightest move.  I added logic to Manhattan to
			// deal with this exact problem, so refer to that project for a working implementation.
			// mdomowicz 2014_08_03
			//if ( pInput->_pSoCcTouchesMoveList )
			//{
			//	_iTouchState = STATE_UP;
			//}
			//else if ( pInput->_pSoCcTouchesUpList )
			if ( pInput->_pSoCcTouchesUpList )
			{
				//-----------------------
				// get the position of the touch, in screen space
				JpVector2f vScreenPos( 0.0f, 0.0f );
				SoCcTouchList* pNonConstTouchList = (SoCcTouchList*)pInput->_pSoCcTouchesUpList;
				if ( pNonConstTouchList->GetNumEntries() > 0 )
				{
					CCTouch* pTouch0 = pNonConstTouchList->GetAtIndex( 0 )->GetData();
					vScreenPos = SoInputTouch::TouchPosToScreenPos( pTouch0->locationInView() );
				}

				//-----------------------
				// convert to world space.
				JpVector2f vWorldPos = pCamera->TransformScreenToWorldSpace( vScreenPos );


				//-----------------------
				if ( IsPositionInside( vWorldPos ) )
				{
					_bIsClickedByUser = true;
				}

				_iTouchState = STATE_UP;
			}
		}
	}


	return ( _iTouchState == STATE_DOWN );
}



/******************************************************************************
 FUNCTION:  IsPositionInside
******************************************************************************/
bool SoWorkerParkingSpot::IsPositionInside( const JpVector2f& vWorldPos )
{
	return IsPositionInsideSprite( vWorldPos, _pSprite );
}



/******************************************************************************
 FUNCTION:  IsPositionInsideSprite
******************************************************************************/
bool SoWorkerParkingSpot::IsPositionInsideSprite( const JpVector2f& vWorldPos, SoSprite* pSprite )
{
	if ( pSprite )
	{
		float fHalfWidth = ( pSprite->getContentSize().width / 2 );
		float fHalfHeight = ( pSprite->getContentSize().height / 2 );

		JpVector2f vCenter = GetPosition();

		return 
		( 
			vWorldPos.X > ( vCenter.X - fHalfWidth ) &&
			vWorldPos.X < ( vCenter.X + fHalfWidth ) &&
			vWorldPos.Y > ( vCenter.Y - fHalfHeight ) &&
			vWorldPos.Y < ( vCenter.Y + fHalfHeight )
		);
	}

	return false;
}
