<?php

require_once( "AchoirSqlUtils.php" );	

/*
 This loads in the credential strings from a file that's outside of
 the webroot.  This is so we do not have to have the sql login info in
 this file which is accessible to outsiders.
 
 If you don't have AchoirSqlCredentials.php, and must make your own, it 
 should look like this:

	<?php

	$sAchoirHostName = "mysql.yourhostname.com";
	$sAchoirSqlLogin = "YourSqlLoginGoesHere";
	$sAchoirSqlPass = "YourSqlPasswordGoesHere";
	$sAchoirSqlDbName = "TheNameOfYourDatabaseGoesHere";

 Using your own information of course.  mdomowicz 2013_10_18
*/
require_once( "../AchoirSqlCredentials.php" );


$GetOperation=$_GET['Operation'];
$GetServerId=$_GET['ServerId'];
$GetClientId=$_GET['ClientId'];

$PostOperation=$_POST['Operation'];
$PostServerId=$_POST['ServerId'];
$PostClientId=$_POST['ClientId'];
$PostData=$_POST['Data'];



/******************************************************************************
 FUNCTION:  SqlConnectToAchoirDb
******************************************************************************/
function SqlConnectToAchoirDb()
{
	//-----------------------
	// import the globals!
	global $sAchoirHostName;
	global $sAchoirSqlLogin;
	global $sAchoirSqlPass;
	global $sAchoirSqlDbName;


	$db = SqlConnect
	( 
		$sAchoirHostName, // Host name
		$sAchoirSqlLogin, // user login
		$sAchoirSqlPass,  // password
		$sAchoirSqlDbName // database name
	);

	return $db;
}



/******************************************************************************
 FUNCTION:  CreateMasterTableIfNeeded
******************************************************************************/
function CreateMasterTableIfNeeded( $db )
{
	if ( !SqlDoesTableExist( $db, "MasterTable" ) )
	{
		//echo "Mater table not found - creating it";
		$sQuery = "CREATE TABLE MasterTable ( P_Id int NOT NULL AUTO_INCREMENT, ServerId varchar(16), sServerStatus varchar(16), CreationTimestamp int NOT NULL, PRIMARY KEY (P_Id) )";
		SqlQuery( $db, $sQuery );
	}
}



/******************************************************************************
 FUNCTION:  IsServerIdInMasterTable
******************************************************************************/
function IsServerIdInMasterTable( $db, $ServerId )
{
	$sQuery = "SELECT P_Id FROM MasterTable WHERE ServerId = '$ServerId'";
	$oResult = SqlQuery( $db, $sQuery );
	$iNumResults = mysqli_num_rows( $oResult );
	return ( $iNumResults > 0 );
}



/******************************************************************************
 FUNCTION:  CreateMessageTablesIfNeeded
******************************************************************************/
function CreateMessageTablesIfNeeded( $db, $ServerId )
{
	CreateMasterTableIfNeeded( $db );

	//-----------------------
	// The MasterTable tracks already existing Message Tables.  If 
	// an entry for the given ServerId is not found in the MasterTable
	// then we know we must create new Message Tables.
	if ( IsServerIdInMasterTable( $db, $ServerId ) == false )
	{
		// Add the $ServerId to the MasterTable
		$iTimestamp = time();
		$sQuery = "INSERT INTO MasterTable ( ServerId, sServerStatus, CreationTimestamp ) VALUES ( '$ServerId', 'lobby', $iTimestamp )";
		$oResult = SqlQuery( $db, $sQuery );


		// Then create the two message tables.  The ServerId is used to
		// uniquely name both the server and client tables.
		$sClientMessagesTableName = GetClientMessagesTableName( $ServerId );
		$sQuery = "CREATE TABLE $sClientMessagesTableName ( P_Id int NOT NULL AUTO_INCREMENT, ClientId varchar(16), sDataString varchar(255), PRIMARY KEY (P_Id) )";
		SqlQuery( $db, $sQuery );
		
		$sServerMessagesTableName = GetServerMessagesTableName( $ServerId );
		$sQuery = "CREATE TABLE $sServerMessagesTableName ( P_Id int NOT NULL AUTO_INCREMENT, sDataString varchar(255), PRIMARY KEY (P_Id) )";
		SqlQuery( $db, $sQuery );
	}
}



/******************************************************************************
 FUNCTION:  CreateKeepAliveTableIfNeeded
******************************************************************************/
function CreateKeepAliveTableIfNeeded( $db, $ServerId )
{
	$sKeepAliveTableName = GetKeepAliveTableName( $ServerId );

	if ( !SqlDoesTableExist( $db, $sKeepAliveTableName ) )
	{
		$sQuery = "CREATE TABLE $sKeepAliveTableName ( P_Id int NOT NULL AUTO_INCREMENT, ServerOrClientId varchar(16), bIsServer tinyint(1), iTimestamp int NOT NULL, PRIMARY KEY (P_Id) )";
		$oResult = SqlQuery( $db, $sQuery );
	}
}



/******************************************************************************
 FUNCTION:  UpdateKeepAliveTable
******************************************************************************/
function UpdateKeepAliveTable( $db, $ServerId, $ServerOrClientIdToUpdate )
{
	CreateKeepAliveTableIfNeeded( $db, $ServerId );

	// See if the ServerOrClientId exists in the KeepAlive table
	$sKeepAliveTableName = GetKeepAliveTableName( $ServerId );
	$sQuery = "SELECT P_Id FROM $sKeepAliveTableName WHERE ServerOrClientId = '$ServerOrClientIdToUpdate'";
	$oResult = SqlQuery( $db, $sQuery );
	$iNumResults = mysqli_num_rows( $oResult );
	$bEntryFound = ( $iNumResults > 0 );

	// If it doesn't, add it.
	if ( $bEntryFound == false )
	{
		$bIsServer = 0;
		if ( $ServerId == $ServerOrClientIdToUpdate )
			$bIsServer = 1;

		$sQuery = "INSERT INTO $sKeepAliveTableName ( ServerOrClientId, bIsServer ) VALUES ( '$ServerOrClientIdToUpdate', $bIsServer )";
		$oResult = SqlQuery( $db, $sQuery );
	}


	// Update the timestamp for the given ServerOrClientId
	$iTimestamp = time();
	$sQuery = "UPDATE $sKeepAliveTableName SET iTimestamp=$iTimestamp WHERE ServerOrClientId='$ServerOrClientIdToUpdate'";
	$oResult = SqlQuery( $db, $sQuery );
}


/******************************************************************************
 FUNCTION:  GetServerMessagesTableName
******************************************************************************/
function GetServerMessagesTableName( $ServerId )
{
	// The prefix "A_" guarantees that the table name is valid (SQL table names shouldn't start with numbers)
	return "A_" . $ServerId . "MessagesForServer";
}



/******************************************************************************
 FUNCTION:  GetClientMessagesTableName
 NOTE:      Yes, you must pass in the ServerID !!!!  not the ClientId
******************************************************************************/
function GetClientMessagesTableName( $ServerId )
{
	// The prefix "A_" guarantees that the table name is valid (SQL table names shouldn't start with numbers)
	return "A_" . $ServerId . "MessagesForClient";
}



/******************************************************************************
 FUNCTION:  GetClientMessagesTableName
******************************************************************************/
function GetKeepAliveTableName( $ServerId )
{
	// The prefix "A_" guarantees that the table name is valid (SQL table names shouldn't start with numbers)
	return "A_" . $ServerId . "KeepAlive";
}



/******************************************************************************
 FUNCTION:  DropAllMessageTablesThatAreTooOld
******************************************************************************/
function DropAllMessageTablesThatAreTooOld( $db )
{
	if ( SqlDoesTableExist( $db, "MasterTable" ) )
	{
		$sQuery = "SELECT * FROM MasterTable";
		$oResult = SqlQuery( $db, $sQuery );

		while ( $Row = mysqli_fetch_array( $oResult, MYSQLI_ASSOC ) ) 
		{
			$iCurrentTime = time();
			$iTimestamp = $Row[ "CreationTimestamp" ];
			$iSecondsIn24Hours = 86400;

			if ( ( $iCurrentTime - $iTimestamp ) > $iSecondsIn24Hours )
			{
				//-----------------------
				// get the table names
				$MyServerId = $Row[ "ServerId" ];
				$sServerMessagesTableName = GetServerMessagesTableName( $MyServerId );
				$sClientMessagesTableName = GetClientMessagesTableName( $MyServerId );
				$sKeepAliveTableName = GetKeepAliveTableName( $MyServerId );

				//-----------------------
				// Delete all the associated tables
				$sQuery2 = "DROP TABLE IF EXISTS $sServerMessagesTableName";
				$sQuery3 = "DROP TABLE IF EXISTS $sClientMessagesTableName";
				$sQuery4 = "DROP TABLE IF EXISTS $sKeepAliveTableName";

				$oResult2 = SqlQuery( $db, $sQuery2 );
				$oResult3 = SqlQuery( $db, $sQuery3 );
				$oResult4 = SqlQuery( $db, $sQuery4 );

				//-----------------------
				// Remove the ServerId's row from the master table.
				$sQuery5 = "DELETE FROM MasterTable WHERE ServerId = '$MyServerId'";
				$oResult5 = SqlQuery( $db, $sQuery5 );
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  Main
******************************************************************************/
function Main()
{
	//-----------------------
	// import the globals!
	global $GetOperation;
	global $GetServerId;
	global $GetClientId;
	global $PostOperation;
	global $PostServerId;
	global $PostClientId;
	global $PostData;

	$bEchoDebugMessages = false;

	$db = SqlConnectToAchoirDb();


	//-----------------------
	// Store messages being sent to the Server
	if ( $PostOperation == 'SendMessageToServer' )
	{
		CreateMessageTablesIfNeeded( $db, $PostServerId );
		$sTableName = GetServerMessagesTableName( $PostServerId );
		$sQuery = "INSERT INTO $sTableName ( sDataString ) VALUES ( '$PostData' )";

		if ( $bEchoDebugMessages )
			echo "--SENT to Server: $PostData";

		$oResult = SqlQuery( $db, $sQuery );
	}

	//-----------------------
	// Store messages being sent to a Client
	else if ( $PostOperation == 'SendMessageToClient' )
	{
		CreateMessageTablesIfNeeded( $db, $PostServerId );
		$sTableName = GetClientMessagesTableName( $PostServerId );
		$sQuery = "INSERT INTO $sTableName ( ClientId, sDataString ) VALUES ( '$PostClientId', '$PostData' )";

		if ( $bEchoDebugMessages )
			echo "--SENT to Client: $PostData";

		$oResult = SqlQuery( $db, $sQuery );
	}

	//-----------------------
	// Retrieve the next message for the Server
	else if ( $GetOperation == 'GetNextMessageForServer' )
	{
		UpdateKeepAliveTable( $db, $GetServerId, $GetServerId );

		CreateMessageTablesIfNeeded( $db, $GetServerId );
		$sTableName = GetServerMessagesTableName( $GetServerId );

		// Note: We need the second P_Id so that we can do $Row[ "P_Id" ], later.
		$sQuery = "SELECT MIN(P_Id), P_Id, sDataString FROM $sTableName";
		$oResult = SqlQuery( $db, $sQuery );
		$iNumResults = mysqli_num_rows( $oResult );

		if ( $iNumResults > 0 )
		{
			$Row = mysqli_fetch_array( $oResult, MYSQLI_ASSOC );

			if ( isset( $Row[ "sDataString" ] ) )
			{
				$sOutputData = $Row[ "sDataString" ];

				echo "NextMessageForServer:" . $sOutputData;

				$iPrimaryKey = $Row[ "P_Id" ];
				$sQuery2 = "DELETE FROM $sTableName WHERE P_Id = $iPrimaryKey";
				$oResult2 = SqlQuery( $db, $sQuery2 );
			}
		}
	}

	//-----------------------
	// Retrieve the next message for a Client
	else if ( $GetOperation == 'GetNextMessageForClient' )
	{
		UpdateKeepAliveTable( $db, $GetServerId, $GetClientId );

		CreateMessageTablesIfNeeded( $db, $GetServerId );
		$sTableName = GetClientMessagesTableName( $GetServerId );

		// Note: We need the second P_Id so that we can do $Row[ "P_Id" ], later.
		$sQuery = "SELECT MIN(P_Id), P_Id, sDataString FROM $sTableName WHERE ClientId = '$GetClientId'";
		$oResult = SqlQuery( $db, $sQuery );
		$iNumResults = mysqli_num_rows( $oResult );

		if ( $iNumResults > 0 )
		{
			$Row = mysqli_fetch_array( $oResult, MYSQLI_ASSOC );

			if ( isset( $Row[ "sDataString" ] ) )
			{
				$sOutputData = $Row[ "sDataString" ];

				echo "NextMessageForClient:" . $sOutputData;

				$iPrimaryKey = $Row[ "P_Id" ];
				$sQuery2 = "DELETE FROM $sTableName WHERE P_Id = $iPrimaryKey";
				$oResult2 = SqlQuery( $db, $sQuery2 );
			}
		}
	}

	//-----------------------
	// Used by the client-side application to test if a server name is in use.
	else if ( $GetOperation == 'GetIsServerNameTaken' )
	{
		CreateMasterTableIfNeeded( $db );

		if ( IsServerIdInMasterTable( $db, $GetServerId ) )
		{
			echo "IsServerNameTaken=true";
		}
		else
		{
			echo "IsServerNameTaken=false";
		}
	}

	//-----------------------
	// Used by the client-side application to get the server status
	else if ( $GetOperation == 'GetServerStatus' )
	{
		CreateMasterTableIfNeeded( $db );

		$sQuery = "SELECT * FROM MasterTable WHERE ServerId = '$GetServerId'";
		$oResult = SqlQuery( $db, $sQuery );
		$iNumResults = mysqli_num_rows( $oResult );

		if ( $iNumResults > 0 )
		{
			$Row = mysqli_fetch_array( $oResult, MYSQLI_ASSOC );

			if ( isset( $Row[ "sServerStatus" ] ) )
			{
				$sServerStatus = $Row[ "sServerStatus" ];
				echo "ServerStatus=" . $sServerStatus;			
			}
		}
		else
		{
			echo "ServerStatus=noserver";
		}
	}

	//-----------------------
	// Used by the client-side application to change the server status
	else if ( $PostOperation == 'PostNewServerStatus' )
	{
		CreateMasterTableIfNeeded( $db );

		if ( IsServerIdInMasterTable( $db, $PostServerId ) )
		{
			$sQuery = "UPDATE MasterTable SET sServerStatus='$PostData' WHERE ServerId='$PostServerId'";
			$oResult = SqlQuery( $db, $sQuery );
		}
	}

	//-----------------------
	// Get Keep alive statuses
	else if ( $GetOperation == 'GetKeepAliveStatus' )
	{
		$sKeepAliveTableName = GetKeepAliveTableName( $GetServerId );

		if ( SqlDoesTableExist( $db, $sKeepAliveTableName ) )
		{
			$sQuery = "SELECT iTimestamp FROM $sKeepAliveTableName";
			$oResult = SqlQuery( $db, $sQuery );
			$bAllConnected = true;
			while ( $Row = mysqli_fetch_array( $oResult, MYSQLI_ASSOC ) ) 
			{
				$iCurrentTime = time();
				$iTimestamp = $Row[ "iTimestamp" ];
				if ( ( $iCurrentTime - $iTimestamp ) > 10 )
				{
					$bAllConnected = false;
					break;
				}
			}


			if ( $bAllConnected )
				echo "KeepAliveStatus:AllConnected";
			else
				echo "KeepAliveStatus:PlayerHasDropped";
		}
	}

	//-----------------------
	// Get the On Connect message
	else if ( $GetOperation == 'GetWebServersOnConnectMessage' )
	{
		// Do some house cleaning - we do this here because
		// this code block is only run once per game, which is the most
		// we'd ever want.
		DropAllMessageTablesThatAreTooOld( $db );

		echo "WebServersOnConnectMessage:Welcome to the Achoir Online server";
	}
	else
	{
		echo "*---- ERROR -- unknown mode ---*";
	}


	SqlClose( $db );
}


//-----------------------
// call entry point function
Main();

