<?php

require_once( "AchoirSqlUtils.php" );	

/*
 This loads in the credential strings from a file that's outside of
 the webroot.  This is so we do not have to have the sql login info in
 this file which is accessible to outsiders.
 
 If you don't have AchoirSqlCredentials.php, and must make your own, it 
 should look like this:

	<?php

	$sAchoirHostName = "mysql.yourhostname.com";
	$sAchoirSqlLogin = "YourSqlLoginGoesHere";
	$sAchoirSqlPass = "YourSqlPasswordGoesHere";
	$sAchoirSqlDbName = "TheNameOfYourDatabaseGoesHere";

 Using your own information of course.  mdomowicz 2013_10_18
*/
require_once( "../AchoirSqlCredentials.php" );


$GetOperation=$_GET['Operation'];
$GetServerId=$_GET['ServerId'];
$GetClientId=$_GET['ClientId'];

$PostOperation=$_POST['Operation'];
$PostServerId=$_POST['ServerId'];
$PostClientId=$_POST['ClientId'];
$PostData=$_POST['Data'];



/******************************************************************************
 FUNCTION:  SqlConnectToAchoirDb
******************************************************************************/
function SqlConnectToAchoirDb()
{
	//-----------------------
	// import the globals!
	global $sAchoirHostName;
	global $sAchoirSqlLogin;
	global $sAchoirSqlPass;
	global $sAchoirSqlDbName;


	$db = SqlConnect
	( 
		$sAchoirHostName, // Host name
		$sAchoirSqlLogin, // user login
		$sAchoirSqlPass,  // password
		$sAchoirSqlDbName // database name
	);

	return $db;
}



/******************************************************************************
 FUNCTION:  CreateMasterChatTableIfNeeded
******************************************************************************/
function CreateMasterChatTableIfNeeded( $db )
{
	if ( !SqlDoesTableExist( $db, "MasterChatTable" ) )
	{
		//echo "Mater table not found - creating it";
		$sQuery = "CREATE TABLE MasterChatTable ( P_Id int NOT NULL AUTO_INCREMENT, ServerId varchar(16), CreationTimestamp int NOT NULL, PRIMARY KEY (P_Id) )";
		SqlQuery( $db, $sQuery );
	}
}



/******************************************************************************
 FUNCTION:  CreateChatTablesIfNeeded
******************************************************************************/
function CreateChatTablesIfNeeded( $db, $ServerId )
{
	CreateMasterChatTableIfNeeded( $db );

	//-----------------------
	// See if the MasterChatTable has an entry for the given $ServerId
	//$sQuery = "SELECT P_Id FROM MasterChatTable WHERE ServerId = $ServerId";
	$sQuery = "SELECT P_Id FROM MasterChatTable WHERE ServerId = '$ServerId'";
	$oResult = SqlQuery( $db, $sQuery );
	$iNumResults = mysqli_num_rows( $oResult );
	$bEntryFound = ( $iNumResults > 0 );


	//-----------------------
	// The MasterChatTable tracks already existing Chat Tables.  If 
	// an entry was not found above, then we must create a new Chat Table.
	if ( $bEntryFound == false )
	{
		// First do some house cleaning - we do this here because
		// this code block is only run once per game, which is the most
		// we'd ever want.
		DropAllChatTablesThatAreTooOld( $db );

		// Add the $ServerId to the MasterChatTable
		$iTimestamp = time();
		$sQuery = "INSERT INTO MasterChatTable ( ServerId, CreationTimestamp ) VALUES ( '$ServerId', $iTimestamp )";
		$oResult = SqlQuery( $db, $sQuery );

		// Then create the client list table.  
		$sClientListTableName = GetClientListTableName( $ServerId );
		$sQuery = "CREATE TABLE $sClientListTableName ( P_Id int NOT NULL AUTO_INCREMENT, ClientId varchar(16), PRIMARY KEY (P_Id) )";
		SqlQuery( $db, $sQuery );

		// Then create the chat table.
		$sChatTableName = GetChatTableName( $ServerId );
		$sQuery = "CREATE TABLE $sChatTableName ( P_Id int NOT NULL AUTO_INCREMENT, ClientId varchar(16), sDataString varchar(255), PRIMARY KEY (P_Id) )";
		SqlQuery( $db, $sQuery );
	}
}



/******************************************************************************
 FUNCTION:  GetChatTableName
******************************************************************************/
function GetChatTableName( $ServerId )
{
	// The prefix "A_" guarantees that the table name is valid (SQL table names shouldn't start with numbers)
	return "A_" . $ServerId . "ChatTable";
}



/******************************************************************************
 FUNCTION:  GetClientListTableName
******************************************************************************/
function GetClientListTableName( $ServerId )
{
	// The prefix "A_" guarantees that the table name is valid (SQL table names shouldn't start with numbers)
	return "A_" . $ServerId . "ClientListTable";
}



/******************************************************************************
 FUNCTION:  DropAllChatTablesThatAreTooOld
******************************************************************************/
function DropAllChatTablesThatAreTooOld( $db )
{
	if ( SqlDoesTableExist( $db, "MasterChatTable" ) )
	{
		$sQuery = "SELECT * FROM MasterChatTable";
		$oResult = SqlQuery( $db, $sQuery );

		while ( $Row = mysqli_fetch_array( $oResult, MYSQLI_ASSOC ) ) 
		{
			$iCurrentTime = time();
			$iTimestamp = $Row[ "CreationTimestamp" ];
			$iSecondsIn24Hours = 86400;

			if ( ( $iCurrentTime - $iTimestamp ) > $iSecondsIn24Hours )
			{
				//-----------------------
				// get the table names
				$MyServerId = $Row[ "ServerId" ];
				$sClientListTableName = GetClientListTableName( $MyServerId );
				$sChatTableName = GetChatTableName( $MyServerId );

				//-----------------------
				// Delete all the associated tables
				$sQuery2 = "DROP TABLE IF EXISTS $sClientListTableName";
				$sQuery3 = "DROP TABLE IF EXISTS $sChatTableName";

				$oResult2 = SqlQuery( $db, $sQuery2 );
				$oResult3 = SqlQuery( $db, $sQuery3 );

				//-----------------------
				// Remove the ServerId's row from the master table.
				$sQuery5 = "DELETE FROM MasterChatTable WHERE ServerId = '$MyServerId'";
				$oResult5 = SqlQuery( $db, $sQuery5 );
			}
		}
	}
}



/******************************************************************************
 FUNCTION:  AddClientIdToClientListTableIfNeeded
******************************************************************************/
function AddClientIdToClientListTableIfNeeded( $db, $ServerId, $ClientId )
{
	$sClientListTableName = GetClientListTableName( $ServerId );

	// See if the ClientId exists in the ClientListTable
	$sQuery = "SELECT P_Id FROM $sClientListTableName WHERE ClientId = '$ClientId'";
	$oResult = SqlQuery( $db, $sQuery );
	$iNumResults = mysqli_num_rows( $oResult );
	$bClientEntryFound = ( $iNumResults > 0 );

	// If it doesn't, add it.
	if ( $bClientEntryFound == false )
	{
		$sQuery = "INSERT INTO $sClientListTableName ( ClientId ) VALUES ( '$ClientId' )";
		$oResult = SqlQuery( $db, $sQuery );

		// Send the newly added client a server message
		// Update - I've implemented an "on connect" message in the main php module.  But it
		// only sends a single line.  Use this if you need to broadcast a multi line message.
		// $sChatTableName = GetChatTableName( $ServerId );
		// $sServerMessageToNewClient = "Your important message goes here!";
		// $sQuery = "INSERT INTO $sChatTableName ( ClientId, sDataString ) VALUES ( '$ClientId', '$sServerMessageToNewClient' )";
		// $oInsertResult = SqlQuery( $db, $sQuery );
	}
}


/******************************************************************************
 FUNCTION:  Main
******************************************************************************/
function Main()
{
	//-----------------------
	// import the globals!
	global $GetOperation;
	global $GetServerId;
	global $GetClientId;
	global $PostOperation;
	global $PostServerId;
	global $PostClientId;
	global $PostData;

	$bEchoDebugMessages = false;

	$db = SqlConnectToAchoirDb();

	DropAllChatTablesThatAreTooOld( $db );


	//-----------------------
	// Store chat messages
	if ( $PostOperation == 'SendChatText' )
	{
		CreateChatTablesIfNeeded( $db, $PostServerId );
		AddClientIdToClientListTableIfNeeded( $db, $PostServerId, $PostClientId );


		$sClientListTableName = GetClientListTableName( $PostServerId );
		$sChatTableName = GetChatTableName( $PostServerId );

		// Loop through the client list, creating a message for each client, 
		// and adding it to the chat table.
		$sQuery = "SELECT ClientId FROM $sClientListTableName";
		$oResult = SqlQuery( $db, $sQuery );

		while ( $Row = mysqli_fetch_array( $oResult, MYSQLI_ASSOC ) ) 
		{
			$StoredClientId = $Row[ "ClientId" ];
			$sQuery = "INSERT INTO $sChatTableName ( ClientId, sDataString ) VALUES ( '$StoredClientId', '$PostData' )";
			$oInsertResult = SqlQuery( $db, $sQuery );
		}

		if ( $bEchoDebugMessages )
			echo "--SENT to Client: $PostData";
	}


	//-----------------------
	// Retrieve the next chat message for a client
	else if ( $GetOperation == 'GetChatText' )
	{
		CreateChatTablesIfNeeded( $db, $GetServerId );
		AddClientIdToClientListTableIfNeeded( $db, $GetServerId, $GetClientId );

		$sChatTableName = GetChatTableName( $GetServerId );

		// Note: We need the second P_Id so that we can do $Row[ "P_Id" ], later.
		$sQuery = "SELECT MIN(P_Id), P_Id, sDataString FROM $sChatTableName WHERE ClientId = '$GetClientId'";
		$oResult = SqlQuery( $db, $sQuery );
		$iNumResults = mysqli_num_rows( $oResult );

		if ( $iNumResults > 0 )
		{
			$Row = mysqli_fetch_array( $oResult, MYSQLI_ASSOC );

			if ( isset( $Row[ "sDataString" ] ) )
			{
				$sOutputData = $Row[ "sDataString" ];

				echo "ChatTextForClient:" . $sOutputData;

				$iPrimaryKey = $Row[ "P_Id" ];
				$sQuery2 = "DELETE FROM $sChatTableName WHERE P_Id = $iPrimaryKey";
				$oResult2 = SqlQuery( $db, $sQuery2 );
			}
		}
	}
	else
	{
		echo "*---- ERROR -- unknown mode ---*";
	}


	SqlClose( $db );
}


//-----------------------
// call entry point function
Main();

