/****************************************************************************
 Copyright (c) 2010 cocos2d-x.org
 
 http://www.cocos2d-x.org
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ****************************************************************************/

#import "CCApplication.h"

#import <UIKit/UIKit.h>

#import "CCGeometry.h"
#import "CCDirectorCaller.h"

NS_CC_BEGIN

CCApplication* CCApplication::sm_pSharedApplication = 0;

CCApplication::CCApplication()
{
    CC_ASSERT(! sm_pSharedApplication);
    sm_pSharedApplication = this;
}

CCApplication::~CCApplication()
{
    CC_ASSERT(this == sm_pSharedApplication);
    sm_pSharedApplication = 0;
}

int CCApplication::run()
{
    if (applicationDidFinishLaunching()) 
    {
        [[CCDirectorCaller sharedDirectorCaller] startMainLoop];
    }
    return 0;
}

void CCApplication::setAnimationInterval(double interval)
{
    [[CCDirectorCaller sharedDirectorCaller] setAnimationInterval: interval ];
}

CCApplication::Orientation CCApplication::setOrientation(Orientation eOritation)
{
    UIApplication * app = [UIApplication sharedApplication];
    UIInterfaceOrientation newOrientation;
    switch (eOritation)
    {
    case kOrientationPortrait:
        newOrientation = UIInterfaceOrientationPortrait;
        break;
    case kOrientationPortraitUpsideDown:
        newOrientation = UIInterfaceOrientationPortraitUpsideDown;
        break;
    case kOrientationLandscapeLeft:
        newOrientation = UIInterfaceOrientationLandscapeRight;
        break;
    case kOrientationLandscapeRight:
        newOrientation = UIInterfaceOrientationLandscapeLeft;
        break;
    default:
        newOrientation = UIInterfaceOrientationPortrait;
        break;
    }
    if (newOrientation != [app statusBarOrientation])
    {
        [app setStatusBarOrientation: newOrientation];
    }
    return eOritation;
}

//void CCApplication::statusBarFrame(cocos2d::CCRect * rect)
//{
//    rect->origin.x = [[UIApplication sharedApplication] statusBarFrame].origin.x;
//    rect->origin.y = [[UIApplication sharedApplication] statusBarFrame].origin.y;
//    rect->size.width = [[UIApplication sharedApplication] statusBarFrame].size.width;
//    rect->size.height = [[UIApplication sharedApplication] statusBarFrame].size.height;
//}

/////////////////////////////////////////////////////////////////////////////////////////////////
// static member function
//////////////////////////////////////////////////////////////////////////////////////////////////

CCApplication& CCApplication::sharedApplication()
{
    CC_ASSERT(sm_pSharedApplication);
    return *sm_pSharedApplication;
}

ccLanguageType CCApplication::getCurrentLanguage()
{
    // get the current language and country config
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSArray *languages = [defaults objectForKey:@"AppleLanguages"];
    NSString *currentLanguage = [languages objectAtIndex:0];

    // get the current language code.(such as English is "en", Chinese is "zh" and so on)
    NSDictionary* temp = [NSLocale componentsFromLocaleIdentifier:currentLanguage];
    NSString * languageCode = [temp objectForKey:NSLocaleLanguageCode];

    ccLanguageType ret = kLanguageEnglish;
    if ([languageCode isEqualToString:@"zh"])
    {
        ret = kLanguageChinese;
    }
    else if ([languageCode isEqualToString:@"en"])
    {
        ret = kLanguageEnglish;
    }
    else if ([languageCode isEqualToString:@"fr"]){
        ret = kLanguageFrench;
    }
    else if ([languageCode isEqualToString:@"it"]){
        ret = kLanguageItalian;
    }
    else if ([languageCode isEqualToString:@"de"]){
        ret = kLanguageGerman;
    }
    else if ([languageCode isEqualToString:@"es"]){
        ret = kLanguageSpanish;
    }
    else if ([languageCode isEqualToString:@"ru"]){
        ret = kLanguageRussian;
    }

    return ret;
}

bool CCApplication::isIpad()
{
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    {
        return true;
    }
    
    return false;
}


// This was added by mdomowicz on 2012_11_16
// The code is taken from: http://www.cocos2d-x.org/boards/6/topics/11290
void CCApplication::openURL(const char* pszUrl)
{
	NSString *msg = [NSString stringWithCString:pszUrl encoding:NSASCIIStringEncoding];
	NSURL *nsUrl = [NSURL URLWithString:msg];
	[[UIApplication sharedApplication] openURL:nsUrl];
}


// This was added by mdomowicz on 2013_03_23
//
// Note that this function takes only the new directory name, and assumes it should
// be placed in the Caches directory.
// If you want to use the Documents directory, either make a new function that uses
// NSDocumentDirectory instead of NSCachesDirectory, or use createDirectoryWithFullPath
// below and specify the Documents directory path manually.
//
// The code is taken from: http://cocos2d-x.org/boards/6/topics/4967
bool CCApplication::createDirectoryInCachesPath(const char *pDirectoryName)
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *newPath = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithUTF8String:pDirectoryName]];

    if (![[NSFileManager defaultManager] fileExistsAtPath:newPath])
    {
        NSError* error;

        if( [[NSFileManager defaultManager] createDirectoryAtPath:newPath withIntermediateDirectories:NO attributes:nil error:&error])
            return true;
        else
            return false;
    }

    return false;
}


// This was added by mdomowicz on 2013_04_08
//
// This function requires the user to specify the full path to the new directory.
// On iOS, the new directory must be created in a writable directory.
// In Cocos2dx, you can get the writable Documents directory like so:
//
//    CCFileUtils::sharedFileUtils()->getWriteablePath().c_str()
//
// Then you can append your new directory name to this string, and pass the whole path to this function.
//
// Based on code taken from: http://cocos2d-x.org/boards/6/topics/4967
bool CCApplication::createDirectoryWithFullPath(const char* pFullPath)
{
    NSString *newPath = [NSString stringWithUTF8String:pFullPath];
	
    if (![[NSFileManager defaultManager] fileExistsAtPath:newPath])
    {
        NSError* error = nil;

        if( [[NSFileManager defaultManager] createDirectoryAtPath:newPath withIntermediateDirectories:NO attributes:nil error:&error])
            return true;
        else
            return false;
    }

    return false;
}


NS_CC_END
